"use strict";
var __decorate = (this && this.__decorate) || function (decorators, target, key, desc) {
    var c = arguments.length, r = c < 3 ? target : desc === null ? desc = Object.getOwnPropertyDescriptor(target, key) : desc, d;
    if (typeof Reflect === "object" && typeof Reflect.decorate === "function") r = Reflect.decorate(decorators, target, key, desc);
    else for (var i = decorators.length - 1; i >= 0; i--) if (d = decorators[i]) r = (c < 3 ? d(r) : c > 3 ? d(target, key, r) : d(target, key)) || r;
    return c > 3 && r && Object.defineProperty(target, key, r), r;
};
exports.__esModule = true;
exports.GenericQueryAddComponent = void 0;
var core_1 = require("@angular/core");
var router_animation_1 = require("../../../../shared/animations/router.animation");
var combo_data_model_1 = require("../../../../models/combo-data.model");
var generic_query_validator_1 = require("../../../../validators/generic-query.validator");
var generic_query_component_1 = require("../../../../shared/generic-query-v2/generic-query.component");
var fast_equals_1 = require("fast-equals");
var GenericQueryAddComponent = /** @class */ (function () {
    function GenericQueryAddComponent(activatedRoute, languageService, loadingService, modalService, notificationsService, genericQueryService, router) {
        this.activatedRoute = activatedRoute;
        this.languageService = languageService;
        this.loadingService = loadingService;
        this.modalService = modalService;
        this.notificationsService = notificationsService;
        this.genericQueryService = genericQueryService;
        this.router = router;
        // Parámetros generales.
        this.userLanguage = this.languageService.GetCurrentLanguage();
        this.translateCategory = 'genericQuery';
        this.initialGenericQuery = {
            vcc: '',
            name: '',
            description: '',
            query: '',
            status: '',
            rawData: '',
            version: 2
        };
        this.renderNewGenericQuery = true;
        // Parámetros de entrada.
        this.genericQuery = {
            vcc: '',
            name: '',
            description: '',
            query: '',
            status: '',
            rawData: '',
            version: 2
        };
        // Idioma del generic-query.
        this.languageOptions = [
            new combo_data_model_1.ComboData('es', this.languageService.GetLanguageName('es')),
            new combo_data_model_1.ComboData('fr', this.languageService.GetLanguageName('fr')),
            new combo_data_model_1.ComboData('en', this.languageService.GetLanguageName('en')),
            new combo_data_model_1.ComboData('pt', this.languageService.GetLanguageName('pt'))
        ];
        this.selectedLanguage = this.languageOptions[0];
        this.GenericQueryModied = this.GenericQueryModied.bind(this);
    }
    Object.defineProperty(GenericQueryAddComponent.prototype, "routerTransition", {
        // Asocio binding para tener animaciones en el componente host (el router-outlet).
        get: function () { return ''; },
        enumerable: false,
        configurable: true
    });
    GenericQueryAddComponent.prototype.ngOnInit = function () {
        var version = this.activatedRoute.snapshot.queryParams.version;
        if (version && version == '1') {
            this.renderNewGenericQuery = false;
        }
        this.initialGenericQuery = JSON.parse(JSON.stringify(this.CollectGenericQueryData()));
    };
    GenericQueryAddComponent.prototype.ngOnDestroy = function () {
    };
    GenericQueryAddComponent.prototype.onGetGenericQuery = function ($event) {
        this.genericQuery = $event;
    };
    GenericQueryAddComponent.prototype.SaveGenericQuery = function () {
        var _this = this;
        if (!this.genericQueryComponent.ValidateQuery()) {
            return false;
        }
        // Armo objeto para pasarle al servicio.
        var genericQuery = this.CollectGenericQueryData();
        // Valido los campos antes de guardar.
        var validationErrors = generic_query_validator_1.GenericQueryValidator.Validate(this.genericQuery, true);
        // Verifico si hubo algún error de validación.
        if (validationErrors.length > 0) {
            // Muestro notificación con cada uno de los errores obtenidos.
            validationErrors.forEach(function (error) {
                _this.notificationsService.error(_this.languageService.GetMessage(_this.translateCategory, 'ERROR'), _this.languageService.GetMessage(_this.translateCategory, error));
            });
            return false;
        }
        // Habilito el spinner de loading
        this.loadingService.Open();
        this.genericQueryService.CreateGenericQuery(genericQuery).subscribe(function (result) {
            if (result) {
                _this.notificationsService.success(_this.languageService.GetMessage(_this.translateCategory, 'SUCCESS'), _this.languageService.GetMessage(_this.translateCategory, 'SUCCESS_GENERIC_QUERY_CREATE'));
                // Redirección hacia la página de listado de consultas genéricas.
                _this.router.navigate(['../'], { relativeTo: _this.activatedRoute });
            }
            // Deshabilito el spinner de loading.
            _this.loadingService.Close();
        }, function (err) {
            _this.notificationsService.error(_this.languageService.GetMessage(_this.translateCategory, 'ERROR'), err.replace('GENERIC_QUERY_ALREADY_EXISTS', _this.languageService.GetMessage(_this.translateCategory, 'GENERIC_QUERY_ALREADY_EXISTS')));
            // Deshabilito el spinner de loading.
            _this.loadingService.Close();
        });
        // Devuelvo false
        return false;
    };
    GenericQueryAddComponent.prototype.CollectGenericQueryData = function () {
        // Armo objeto donde voy a juntar todo lo que hay en la página.
        var newGenericQuery = this.genericQuery || {};
        // Asocio los valores de combos que se almacenan en memoria.
        return newGenericQuery;
    };
    GenericQueryAddComponent.prototype.GenericQueryModied = function () {
        // Armo objeto para pasarle al servicio.
        var genericQueryToSave = this.CollectGenericQueryData();
        console.log("genericQueryToSave", JSON.stringify(genericQueryToSave));
        console.log("this.initialGenericQuery", JSON.stringify(this.initialGenericQuery));
        // Comparo los objetos antes y después.
        return !(fast_equals_1.deepEqual(genericQueryToSave, this.initialGenericQuery));
    };
    __decorate([
        core_1.ViewChild(generic_query_component_1.GenericQueryComponent)
    ], GenericQueryAddComponent.prototype, "genericQueryComponent");
    __decorate([
        core_1.HostBinding('@routerTransition')
    ], GenericQueryAddComponent.prototype, "routerTransition");
    GenericQueryAddComponent = __decorate([
        core_1.Component({
            moduleId: module.id,
            selector: 'generic-query-add-cmp',
            templateUrl: 'generic-query-add.html',
            styleUrls: ['generic-query-add.css'],
            encapsulation: core_1.ViewEncapsulation.Emulated,
            animations: [
                router_animation_1.BlurAnimation('routerTransition')
            ]
        })
    ], GenericQueryAddComponent);
    return GenericQueryAddComponent;
}());
exports.GenericQueryAddComponent = GenericQueryAddComponent;
