"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.GetTeamPerformanceFromEvaluationPlan = exports.GetTeamPerformanceFilters = exports.TeamPerformanceMSSql = void 0;
var log4js = require("log4js");
var sql = require('mssql');
var logger = log4js.getLogger('Sql');
var sqlConn;
function TeamPerformanceMSSql(app) {
    sqlConn = app.get('sqlConn');
}
exports.TeamPerformanceMSSql = TeamPerformanceMSSql;
function GetTeamPerformanceFilters(vcc, batchType, evaluationFormatId, templateId) {
    return new sql.Request(sqlConn)
        .input('VCC', sql.VarChar(50), vcc)
        .input('BatchType', sql.VarChar(50), batchType)
        .input('EvaluationFormatId', sql.VarChar(50), evaluationFormatId)
        .input('TemplateId', sql.VarChar(50), templateId)
        .execute('GetTeamPerformanceFilters')
        .then(function (recordsets) {
        var format_template_records = recordsets && recordsets.length > 0 ? recordsets[0] : null;
        var evaluationplan_study_records = recordsets && recordsets.length > 0 ? recordsets[1] : null;
        var agent_records = recordsets && recordsets.length > 0 ? recordsets[2] : null;
        var supervisor_records = recordsets && recordsets.length > 0 ? recordsets[3] : null;
        return Promise.resolve({
            formats_templates: format_template_records,
            evaluationplans_studies: evaluationplan_study_records,
            agents: agent_records,
            supervisors: supervisor_records
        });
    })
        .catch(function (err) {
        var errorMsg = (err && err.procName ? 'Error al ejecutar \'' + err.procName + '\': ' : 'Error al ejecutar consulta: ') + (err ? typeof err === 'string' ? err : err.message || err.description || '' : '');
        logger.error(errorMsg);
        return Promise.reject(err);
    });
}
exports.GetTeamPerformanceFilters = GetTeamPerformanceFilters;
function GetTeamPerformanceFromEvaluationPlan(vcc, evaluationFormatId, evaluationPlans, agents, abilities, startDate, endDate, drilldownTarget, drilldownId, supervisors, resultBySupervisor) {
    return new sql.Request(sqlConn)
        .input('VCC', sql.VarChar(50), vcc)
        .input('EvaluationFormatId', sql.VarChar(50), evaluationFormatId)
        .input('EvaluationPlanIds', sql.VarChar(sql.MAX), evaluationPlans)
        .input('AgentIds', sql.VarChar(sql.MAX), agents)
        .input('AbilityIds', sql.VarChar(sql.MAX), abilities)
        .input('StartDate', sql.VarChar(50), startDate)
        .input('EndDate', sql.VarChar(50), endDate)
        .input('DrilldownTarget', sql.VarChar(50), drilldownTarget)
        .input('DrilldownId', sql.VarChar(50), drilldownId)
        .input('SupervisorIds', sql.VarChar(sql.MAX), supervisors)
        .input('ResultBySupervisor', sql.Bit(), resultBySupervisor)
        .execute('GetTeamPerformanceFromEvaluationPlan')
        .then(function (recordsets) {
        var columnsDB = recordsets && recordsets.length > 0 ? recordsets[0] : null;
        var dataDB = recordsets && recordsets.length > 0 ? recordsets[1] : null;
        return Promise.resolve({
            columnsDB: columnsDB,
            dataDB: dataDB
        });
    })
        .catch(function (err) {
        var errorMsg = (err && err.procName ? 'Error al ejecutar \'' + err.procName + '\': ' : 'Error al ejecutar consulta: ') + (err ? typeof err === 'string' ? err : err.message || err.description || '' : '');
        logger.error(errorMsg);
        return Promise.reject(err);
    });
}
exports.GetTeamPerformanceFromEvaluationPlan = GetTeamPerformanceFromEvaluationPlan;
