"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.getInteractionsRefactor = exports.isCampaignPresent = exports.getInteractionById = void 0;
var Knex = require("knex");
var knex_config_1 = require("../knex.config");
var interaction_model_1 = require("../../../models/interaction.model");
var log4js = require("log4js");
var moment = require("moment");
var sql = require('mssql');
var logger = log4js.getLogger('ServerScripts');
var callDB = Knex(knex_config_1.connections.call);
var interactionsTable = 'InteractionDetail ID';
var callInteractionDetail = 'CallInteractionDetail CID';
var idColumn = 'ID.Id';
var vccColumn = 'ID.VirtualCC';
var contactAddressColumn = 'ID.ContactId';
var numberColumn = 'CASE WHEN ID.Direction = \'E\' THEN CID.CallerId ELSE CID.CalledId END AS Number ';
var dispositionCodeColumn = 'ID.ManagementResult';
var durationTimeColumn = 'ID.AttentionTime';
var EndDateColumn = 'ID.EndDate';
var StartDateColumn = 'ID.StartDate';
var agentColumn = 'ID.FirstAgent';
var originColumn = 'ID.Direction';
var campaignColumn = 'ID.Campaign';
var getInteractionById = function (vcc, id, loadMultimedia) {
    if (loadMultimedia === void 0) { loadMultimedia = false; }
    logger.info('[call.ts::getInteractionById] Searching vcc|interaction: ' + vcc + '|' + id);
    return new Promise(function (resolve, reject) {
        callDB.select([idColumn, campaignColumn, agentColumn,
            StartDateColumn, EndDateColumn, durationTimeColumn,
            dispositionCodeColumn, contactAddressColumn, callDB.raw(numberColumn), originColumn
        ])
            .join(callDB.raw('CallInteractionDetail CID (NOLOCK) '), function () {
            this.on(function () {
                this.on('CID.Id', '=', 'ID.Id');
            });
        })
            .from(callDB.raw(interactionsTable + ' (NOLOCK) ')).where({
            'ID.VirtualCC': vcc,
            'ID.Id': id
        }).then(function (result) {
            if (result && result[0]) {
                var r = result[0];
                if (typeof r !== "undefined") {
                    var interaction = new interaction_model_1.Interaction(r['Id'], r['Campaign'], r['FirstAgent'], r['StartDate'], r['EndDate'], r['AttentionTime'], 'CALL', r['ManagementResult'], r['ContactId'], parseOrigin(r['Direction']));
                    interaction.number = r['Number'];
                    interaction.descriptionAgent = r['FirstAgent'];
                    interaction.descriptionCampaign = r['Campaign'];
                    if (loadMultimedia) {
                        getInteractionMultimediaPartsById(vcc, interaction)
                            .then(function (i) {
                            resolve(i);
                        }, function (err) {
                            reject(err);
                        });
                    }
                    else {
                        resolve(interaction);
                    }
                }
                else
                    resolve(null);
            }
            else
                resolve(null);
        }, function (error) {
            reject(error);
        });
    });
};
exports.getInteractionById = getInteractionById;
var getInteractionMultimediaPartsById = function (vcc, interaction) {
    return new Promise(function (resolve, reject) {
        return callDB.select([
            'InteractionMultimediaParts.Id',
            'InteractionMultimediaParts.Section',
            'InteractionMultimediaParts.Part',
            'InteractionActorDetail.Actor',
            'InteractionActorDetail.ActorName',
            'InteractionMultimediaParts.Format',
            'InteractionActorDetail.FirstAgent',
            'InteractionMultimediaParts.ContentType'
        ])
            .from(callDB.raw('InteractionMultimediaParts (NOLOCK) '))
            .join('InteractionActorDetail', function () {
            this.on(function () {
                this.on('InteractionActorDetail.Id', '=', 'InteractionMultimediaParts.Id')
                    .on('InteractionActorDetail.Section', '=', 'InteractionMultimediaParts.Section');
            });
        })
            .where({
            'InteractionMultimediaParts.Id': interaction.id
        })
            .orderBy('InteractionMultimediaParts.TmStmp', 'ASC')
            .then(function (result) {
            logger.info('[call::getInteractionMultimediaPartsById] Multimedia parts by id: ' + interaction.id + '|' + result.length);
            if (result) {
                logger.info('[call::getInteractionMultimediaPartsById] Agent\'s parts : ' + interaction.id + '|' + result.length);
                interaction.interactionPartGroups = [];
                interaction.agentName = result[0].ActorName;
                result.forEach(function (r) {
                    if (["mp3", "wav", "ogg"].includes(r.Format)) {
                        if (r['ContentType'] === 'screenrecording')
                            interaction.hasVideo = true;
                        var group = interaction.interactionPartGroups.find(function (e) { return e.agentId === r['Actor']; });
                        if (!group) {
                            group = new interaction_model_1.MultimediaPartGroup(r['Actor']);
                            interaction.interactionPartGroups.push(group);
                        }
                        var p = new interaction_model_1.MultimediaPart(r['Section'], r['Part'], '', r['Actor'], r['Format']);
                        group.parts.push(p);
                    }
                });
            }
            resolve(interaction);
        }, function (error) {
            reject(error);
        });
    });
};
var isCampaignPresent = function (vcc, campaignId) {
    return new Promise(function (resolve, reject) {
        callDB.distinct('Campaign').from(callDB.raw(interactionsTable + ' (NOLOCK) ')).where({
            'ID.VirtualCC': vcc,
            'ID.Campaign': campaignId
        }).then(function (result) {
            if (result && result.length > 0) {
                resolve(true);
            }
            else
                resolve(false);
        }, function (error) {
            reject(error);
        });
    });
};
exports.isCampaignPresent = isCampaignPresent;
function parseOrigin(origin) {
    if (origin === 'S')
        return 'OUT';
    else if (origin === 'E')
        return 'IN';
    return '';
}
var getInteractionsRefactor = function (vcc, limit, params) {
    logger.info('[HistoricalData::getInteractionsRefactor] Making interactions search query to HistoricalData.');
    return new Promise(function (resolve, reject) {
        var builder = callDB
            .from(callDB.raw(interactionsTable + ' (NOLOCK)'))
            .leftJoin(callDB.raw('InteractionMultimediaParts IMP (NOLOCK)'), 'IMP.Id', 'ID.id')
            .leftJoin(callDB.raw('InteractionActorDetail IAD (NOLOCK)'), 'IAD.Id', 'ID.id')
            .where({
            'ID.VirtualCC': vcc,
            'ID.IsTaked': true
        })
            .whereBetween(EndDateColumn, [params.dateIni, params.dateFin]);
        if (params.campaigns && params.campaigns.length)
            builder = builder.whereIn(campaignColumn, params.campaigns);
        if (params.agents && params.agents.length)
            builder = builder.whereIn('IAD.Actor', params.agents);
        if (params.origins && params.origins.length)
            builder = builder.whereIn(originColumn, params.origins);
        if (params.minDuration)
            builder = builder.whereRaw('?? >= ?', [durationTimeColumn, params.minDuration]);
        if (params.maxDuration)
            builder = builder.whereRaw('?? <= ?', [durationTimeColumn, params.maxDuration]);
        if (params.results && params.results.length)
            builder = builder.whereIn(dispositionCodeColumn, params.results);
        if (params.multimediaFormats && params.multimediaFormats.length) {
            builder = builder.whereIn('IMP.Format', params.multimediaFormats);
        }
        var horaIni = moment(params.hourIni, 'HH:mm:ss');
        var horaFin = moment(params.hourFin, 'HH:mm:ss');
        if (horaIni.isSameOrBefore(horaFin)) {
            builder = builder.whereRaw('CONVERT(VARCHAR(8), ??, 108) BETWEEN ? AND ?', [EndDateColumn, params.hourIni, params.hourFin]);
        }
        else if (horaIni.isAfter(horaFin)) {
            builder = builder.whereRaw('CONVERT(VARCHAR(8), ??, 108) NOT BETWEEN ? AND ?', [EndDateColumn, params.hourFin, params.hourIni]);
        }
        var builderResult = builder.clone();
        var builderTotal = builder.clone();
        var selectResultRaw = '\'CALL\' AS interactionType, '
            + idColumn + ' AS id, '
            + campaignColumn + ' AS idCampaign, '
            + campaignColumn + ' AS descriptionCampaign, '
            + agentColumn + ' AS idAgent, '
            + agentColumn + ' AS descriptionAgent, '
            + StartDateColumn + ' AS dateIni, '
            + EndDateColumn + ' AS dateFin, '
            + durationTimeColumn + ' AS interactionTime, '
            + dispositionCodeColumn + ' AS dispositionCode, '
            + contactAddressColumn + ' AS address, '
            + 'CASE ' + originColumn + ' WHEN \'S\' THEN \'OUT\' WHEN \'E\' THEN \'IN\' ELSE \'\' END AS origin ';
        if (params.withSectionPart === true) {
            builderTotal.countDistinct(idColumn + ' as totalRecords')
                .sum(durationTimeColumn + ' as totalTime')
                .select(callDB.raw('SUM(CEILING(' + durationTimeColumn + ' * 1.00 /60) * 60) as roundedTime'));
            selectResultRaw += ', '
                + '[IMP].[Format] AS format, '
                + '[IMP].[Section] AS interactionSection, '
                + '[IMP].[Part] AS interactionPart ';
        }
        else {
            builderTotal.countDistinct(idColumn + ' as totalRecords')
                .sum(durationTimeColumn + ' as totalTime')
                .select(callDB.raw('SUM(CEILING(' + durationTimeColumn + ' * 1.00 /60) * 60) as roundedTime'));
        }
        var result = new interaction_model_1.InteractionView;
        builderTotal
            .then(function (res) {
            result.totalRecords = res[0].totalRecords;
            result.totalTime = res[0].totalTime === null ? 0 : res[0].totalTime;
            result.roundedTime = res[0].roundedTime === null ? 0 : res[0].roundedTime;
            builderResult.clearSelect();
            builderResult.distinct(callDB.raw(selectResultRaw));
            if (limit != 0)
                builderResult.limit(limit);
            builderResult.orderBy(StartDateColumn);
            return new Promise(function (resolve, reject) {
                return resolve(builderResult);
            });
        })
            .then(function (res) {
            result.interactions = res;
            return resolve(result);
        })
            .catch(function (err) {
            logger.info('[HistoricalData::getInteractionsRefactor] Catch error: ' + err);
            reject(err);
        });
    });
};
exports.getInteractionsRefactor = getInteractionsRefactor;
