"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.GetAllegroBillingInteractionsForEvaluationPlan = exports.GetAllegroBillingInteractions = exports.GetAllegroInteractionVideos = exports.GetAllegroManagementResults = exports.GetAllegroCampaignMembers = exports.GetAllegroCampaigns = exports.GetAllegroAgents = exports.GetAllegroSupervisors = exports.AllegroMSSql = void 0;
var log4js = require("log4js");
var sql = require('mssql');
var arrays_shared_1 = require("../../shared/arrays.shared");
var logger = log4js.getLogger('Sql');
var sqlConn;
function AllegroMSSql(app) {
    sqlConn = app.get('sqlConn');
}
exports.AllegroMSSql = AllegroMSSql;
function GetAllegroSupervisors(ip, vcc, type) {
    return new sql.Request(sqlConn)
        .input('Ip', sql.VarChar(50), ip)
        .input('VCC', sql.VarChar(50), vcc)
        .input('Type', sql.Bit, type)
        .execute('GetAllegroSupervisors')
        .then(function (recordsets) {
        var recordset = (recordsets && recordsets.length > 0 ? recordsets[0] : null);
        var supervisors = [];
        var uniqueUserIds = (0, arrays_shared_1.GetUniqueFromObjectArray)(recordset, 'userId');
        uniqueUserIds.map(function (uui) {
            var supervisor = {};
            supervisor.userId = uui.userId;
            supervisor.userName = uui.userName;
            supervisor.campaigns = [];
            var currentSupervisor = recordset.filter(function (r) { return r.userId === uui.userId; });
            var currentSupervisorCampaigns = (0, arrays_shared_1.GetUniqueFromObjectArray)(currentSupervisor, 'campaign');
            currentSupervisorCampaigns.map(function (csc) {
                var campaign = {};
                campaign.campaignId = csc.campaignId;
                campaign.workGroups = [];
                currentSupervisor.filter(function (cs) { return cs.campaignId === campaign.campaign; }).map(function (cs) {
                    if (cs.workGroupId)
                        campaign.workGroups.push(cs.workGroupId);
                });
                supervisor.campaigns.push(campaign);
            });
            supervisors.push(supervisor);
        });
        return Promise.resolve(supervisors);
    })
        .catch(function (err) {
        var errorMsg = (err && err.procName ? 'Error al ejecutar \'' + err.procName + '\': ' : 'Error al ejecutar consulta: ') + (err ? typeof err === 'string' ? err : err.message || err.description || '' : '');
        logger.error(errorMsg);
        return Promise.reject(err);
    });
}
exports.GetAllegroSupervisors = GetAllegroSupervisors;
function GetAllegroAgents(ip, vcc, type, campaign) {
    return new sql.Request(sqlConn)
        .input('Ip', sql.VarChar(50), ip)
        .input('VCC', sql.VarChar(50), vcc)
        .input('Type', sql.Int, type)
        .input('Campaigns', sql.VarChar(sql.MAX), campaign)
        .execute('GetAllegroAgents')
        .then(function (recordsets) {
        var recordset = (recordsets && recordsets.length > 0 ? recordsets[0] : null);
        return Promise.resolve(recordset);
    })
        .catch(function (err) {
        var errorMsg = (err && err.procName ? 'Error al ejecutar \'' + err.procName + '\': ' : 'Error al ejecutar consulta: ') + (err ? typeof err === 'string' ? err : err.message || err.description || '' : '');
        logger.error(errorMsg);
        return Promise.reject(err);
    });
}
exports.GetAllegroAgents = GetAllegroAgents;
function GetAllegroCampaigns(ip, vcc, type) {
    return new sql.Request(sqlConn)
        .input('Ip', sql.VarChar(50), ip)
        .input('VCC', sql.VarChar(50), vcc)
        .input('Type', sql.Int, type)
        .execute('GetAllegroCampaigns')
        .then(function (recordsets) {
        var recordset = (recordsets && recordsets.length > 0 ? recordsets[0] : null);
        return Promise.resolve(recordset);
    }).catch(function (err) {
        var errorMsg = (err && err.procName ? 'Error al ejecutar \'' + err.procName + '\': ' : 'Error al ejecutar consulta: ') + (err ? typeof err === 'string' ? err : err.message || err.description || '' : '');
        logger.error(errorMsg);
        return Promise.reject(err);
    });
}
exports.GetAllegroCampaigns = GetAllegroCampaigns;
function GetAllegroCampaignMembers(ip, vcc, campaigns, type) {
    return new sql.Request(sqlConn)
        .input('Ip', sql.VarChar(50), ip)
        .input('VCC', sql.VarChar(50), vcc)
        .input('Campaigns', sql.VarChar(sql.MAX), campaigns.toString())
        .input('Type', sql.Int, type)
        .execute('GetAllegroCampaignMembers')
        .then(function (recordsets) {
        var recordset = (recordsets && recordsets.length > 0 ? recordsets[0] : null);
        var campaignsMembers = [];
        campaigns.map(function (c) {
            var campaignMembers = {};
            campaignMembers.id = c;
            var agents = [];
            var supervisors = [];
            var agentWg = [];
            var supervisorWg = [];
            recordset.filter(function (r) { return r.campaignId === c; }).map(function (r) {
                if (r.isAgent) {
                    agents.push(r.userId);
                    var agent = {};
                    agent.id = r.userId;
                    agent.workGroupId = r.agentWorkgroup ? r.agentWorkgroup : null;
                    agentWg.push(agent);
                }
                if (r.isSupervisor) {
                    supervisors.push(r.userId);
                    var alreadyAdded = false;
                    var supervisor = {};
                    if (supervisorWg.find(function (swg) { return swg.id === r.userId; })) {
                        alreadyAdded = true;
                        supervisor = supervisorWg.find(function (swg) { return swg.id === r.userId; });
                    }
                    supervisor.id = r.userId;
                    supervisor.workGroups = supervisor.workGroups ? supervisor.workGroups : [];
                    if (r.supervisorWorkgroup)
                        supervisor.workGroups.push(r.supervisorWorkgroup);
                    if (!alreadyAdded)
                        supervisorWg.push(supervisor);
                }
            });
            campaignMembers.supervisorAgents = [];
            supervisorWg.map(function (swg) {
                var supervisorAgents = {};
                supervisorAgents.id = swg.id;
                supervisorAgents.agents = [];
                if (swg.workGroups.length) {
                    swg.workGroups.map(function (wg) {
                        agentWg.filter(function (awg) { return awg.workGroupId === wg; }).map(function (awg) { return supervisorAgents.agents.push(awg.id); });
                    });
                }
                else {
                    agentWg.map(function (awg) { return supervisorAgents.agents.push(awg.id); });
                }
                campaignMembers.supervisorAgents.push(supervisorAgents);
            });
            campaignMembers.agents = (0, arrays_shared_1.GetUniqueFromArray)(agents);
            campaignMembers.supervisors = (0, arrays_shared_1.GetUniqueFromArray)(supervisors);
            campaignsMembers.push(campaignMembers);
        });
        return Promise.resolve(campaignsMembers);
    })
        .catch(function (err) {
        var errorMsg = (err && err.procName ? 'Error al ejecutar \'' + err.procName + '\': ' : 'Error al ejecutar consulta: ') + (err ? typeof err === 'string' ? err : err.message || err.description || '' : '');
        logger.error(errorMsg);
        return Promise.reject(err);
    });
}
exports.GetAllegroCampaignMembers = GetAllegroCampaignMembers;
function GetAllegroManagementResults(ip, vcc, type) {
    return new sql.Request(sqlConn)
        .input('Ip', sql.VarChar(50), ip)
        .input('VCC', sql.VarChar(50), vcc)
        .input('Type', sql.Int, type)
        .execute('GetAllegroManagementResults')
        .then(function (recordsets) {
        var recordset = (recordsets && recordsets.length > 0 ? recordsets[0] : null);
        return Promise.resolve(recordset);
    }).catch(function (err) {
        var errorMsg = (err && err.procName ? 'Error al ejecutar \'' + err.procName + '\': ' : 'Error al ejecutar consulta: ') + (err ? typeof err === 'string' ? err : err.message || err.description || '' : '');
        logger.error(errorMsg);
        return Promise.reject(err);
    });
}
exports.GetAllegroManagementResults = GetAllegroManagementResults;
function GetAllegroInteractionVideos(ip, vcc, type, interactionIds) {
    if (interactionIds === void 0) { interactionIds = ''; }
    return new sql.Request(sqlConn)
        .input('Ip', sql.VarChar(50), ip)
        .input('VCC', sql.VarChar(50), vcc)
        .input('Type', sql.Int, type)
        .input('InteractionIds', sql.VarChar(sql.MAX), interactionIds)
        .execute('GetAllegroInteractionVideos')
        .then(function (recordsets) {
        var recordset = (recordsets && recordsets.length > 0 ? recordsets[0] : null);
        return Promise.resolve(recordset);
    })
        .catch(function (err) {
        var errorMsg = (err && err.procName ? 'Error al ejecutar \'' + err.procName + '\': ' : 'Error al ejecutar consulta: ') + (err ? typeof err === 'string' ? err : err.message || err.description || '' : '');
        logger.error(errorMsg);
        return Promise.reject(err);
    });
}
exports.GetAllegroInteractionVideos = GetAllegroInteractionVideos;
function GetAllegroBillingInteractions(vcc, ip, type, secondsPerRequest, startDate, endDate, campaigns, agents, origins, startHour, endHour, minDuration, maxDuration, results, formats) {
    return new sql.Request(sqlConn)
        .input('VCC', sql.VarChar(50), vcc)
        .input('Ip', sql.VarChar(50), ip)
        .input('Type', sql.Int, type)
        .input('SecondsPerRequest', sql.Decimal(18, 5), secondsPerRequest)
        .input('StartDate', sql.DateTime, startDate)
        .input('EndDate', sql.DateTime, endDate)
        .input('Campaigns', sql.NVarChar(sql.MAX), campaigns)
        .input('Agents', sql.NVarChar(sql.MAX), agents)
        .input('Origins', sql.NVarChar(sql.MAX), origins)
        .input('StartHour', sql.VarChar(50), startHour)
        .input('EndHour', sql.VarChar(50), endHour)
        .input('MinDuration', sql.Int, minDuration)
        .input('MaxDuration', sql.Int, maxDuration)
        .input('Results', sql.NVarChar(sql.MAX), results)
        .input('Formats', sql.NVarChar(50), formats)
        .execute('GetAllegroBillingInteractions')
        .then(function (recordsets) {
        var recordset = (recordsets && recordsets.length > 0 ? recordsets[0] : null);
        var allegroBillingInteractions = (recordset && recordset.length > 0 ? recordset[0] : null);
        allegroBillingInteractions.interactionType = 'CALL';
        logger.info("[AllegroMSSql::GetAllegroBillingInteractions] Total Records: " + allegroBillingInteractions.quantity);
        return Promise.resolve(allegroBillingInteractions);
    })
        .catch(function (err) {
        var errorMsg = (err && err.procName ? 'Error al ejecutar \'' + err.procName + '\': ' : 'Error al ejecutar consulta: ') + (err ? typeof err === 'string' ? err : err.message || err.description || '' : '');
        logger.error(errorMsg);
        return Promise.reject(err);
    });
}
exports.GetAllegroBillingInteractions = GetAllegroBillingInteractions;
function GetAllegroBillingInteractionsForEvaluationPlan(vcc, processId, secondsPerRequest) {
    return new sql.Request(sqlConn)
        .input('VCC', sql.VarChar(50), vcc)
        .input('ProcessId', sql.VarChar(50), processId)
        .input('SecondsPerRequest', sql.Decimal(18, 5), secondsPerRequest)
        .execute('GetAllegroBillingInteractionsForEvaluationPlan')
        .then(function (recordsets) {
        var recordset = (recordsets && recordsets.length > 0 ? recordsets[0] : null);
        var result = (recordset && recordset.length > 0 ? recordset[0] : null);
        logger.info("[AllegroMSSql::GetAllegroBillingInteractionsForEvaluationPlan] Total Records :" + result.length);
        return Promise.resolve(result);
    })
        .catch(function (err) {
        var errorMsg = (err && err.procName ? 'Error al ejecutar \'' + err.procName + '\': ' : 'Error al ejecutar consulta: ') + (err ? typeof err === 'string' ? err : err.message || err.description || '' : '');
        logger.error(errorMsg);
        return Promise.reject(err);
    });
}
exports.GetAllegroBillingInteractionsForEvaluationPlan = GetAllegroBillingInteractionsForEvaluationPlan;
