"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.BulkAuditorEvaluationPlan = exports.AuditorEvaluationPlanMSSql = void 0;
var moment = require("moment");
var log4js = require("log4js");
var sql = require('mssql');
var logger = log4js.getLogger('Sql');
var sqlConn;
function AuditorEvaluationPlanMSSql(app) {
    sqlConn = app.get('sqlConn');
}
exports.AuditorEvaluationPlanMSSql = AuditorEvaluationPlanMSSql;
function BulkAuditorEvaluationPlan(vcc, evaluationPlan) {
    var table = SetAuditorEvaluationPlanTemporaryTable(vcc, evaluationPlan);
    return BulkAuditorEvaluationPlanTransaction(vcc, table);
}
exports.BulkAuditorEvaluationPlan = BulkAuditorEvaluationPlan;
function SetAuditorEvaluationPlanTemporaryTable(vcc, evaluationPlan) {
    var table = new sql.Table('#AuditorEvaluationPlanTemp');
    table.create = true;
    table.columns.add('VCC', sql.VarChar(50), { nullable: true });
    table.columns.add('EvaluationPlanId', sql.VarChar(50), { nullable: true });
    table.columns.add('CampaignId', sql.VarChar(100), { nullable: true });
    table.columns.add('AgentId', sql.VarChar(50), { nullable: true });
    table.columns.add('AuditorId', sql.VarChar(50), { nullable: true });
    table.columns.add('InteractionId', sql.VarChar(250), { nullable: true });
    table.columns.add('InteractionType', sql.VarChar(50), { nullable: true });
    table.columns.add('InteractionStartDate', sql.DateTime, { nullable: true });
    table.columns.add('InteractionEndDate', sql.DateTime, { nullable: true });
    table.columns.add('InteractionDuration', sql.Int, { nullable: true });
    table.columns.add('InteractionManagementResult', sql.VarChar(sql.MAX), { nullable: true });
    table.columns.add('InteractionOrigin', sql.VarChar(50), { nullable: true });
    table.columns.add('InteractionFormat', sql.VarChar(50), { nullable: true });
    table.columns.add('InteractionContactId', sql.VarChar(100), { nullable: true });
    table.columns.add('InteractionContactAddress', sql.VarChar(300), { nullable: true });
    table.columns.add('InteractionContactName', sql.VarChar(300), { nullable: true });
    logger.info("[AuditorEvaluationPlanMSSql::SetAuditorEvaluationPlanTemporaryTable] Unique interactions for plan: ".concat(evaluationPlan.uniqueInteractions.length));
    var tmpInteractionsAll = [];
    tmpInteractionsAll = JSON.parse(JSON.stringify(evaluationPlan.uniqueInteractions));
    evaluationPlan.auditors.forEach(function (a) {
        logger.info("[AuditorEvaluationPlanMSSql::SetAuditorEvaluationPlanTemporaryTable] Interactions for auditor ".concat(a.idAuditor, ": ").concat(a.numberInteractions));
        var auditorInteractions = [];
        if (evaluationPlan.uniqueInteractions.length === a.numberInteractions)
            auditorInteractions = evaluationPlan.uniqueInteractions;
        else {
            for (var i = 0; i < a.numberInteractions; i++) {
                var randomIndex = Math.floor(Math.random() * tmpInteractionsAll.length);
                if (tmpInteractionsAll[randomIndex])
                    auditorInteractions.push(tmpInteractionsAll[randomIndex]);
                tmpInteractionsAll.splice(randomIndex, 1);
            }
        }
        auditorInteractions.forEach(function (i) {
            var interactionType = i.interactionType ? i.interactionType : i.Channel;
            table.rows.add(vcc, evaluationPlan.id, interactionType === 'CALL' ? i.Campaign || i.idCampaign : i.campaign || i.idCampaign || '', interactionType === 'CALL' ? i.Agent || i.idAgent || '' : i.agent || i.idAgent || '', a.idAuditor, interactionType === 'CALL' ? i.InteractionId || i.id : i.id, interactionType, interactionType === 'CALL' ? moment(i.dateIni).toDate() : moment(i.dateFin).toDate() || null, interactionType === 'CALL' ? moment(i.dateFin).toDate() : moment(i.dateFin).toDate() || null, interactionType === 'CALL' ? i.DurationTime || i.interactionTime : i.interactionTime || null, interactionType === 'CALL' ? i.DispositionCode || i.dispositionCode : i.dispositionCode || null, interactionType === 'CALL' ? (i.Type && (i.Type === 'E' || i.origin === 'IN') ? 'IN' : 'OUT')
                :
                    i.origin || null, interactionType === 'CALL' ? i.format : null, interactionType === 'CALL' ? i.contactName : i.contactName, interactionType === 'CALL' ? i.contactAddress : i.contactAddress, interactionType === 'CALL' ? i.contactName : i.contactName, i.Gender || '');
        });
    });
    return table;
}
function BulkAuditorEvaluationPlanTransaction(vcc, temporaryTable) {
    var transaction = new sql.Transaction(sqlConn);
    var request;
    return transaction.begin()
        .then(function (_) {
        logger.info('[AuditorEvaluationPlanMSSql::BulkAuditorEvaluationPlanTransaction] Creating transaction');
        return new sql.Request(transaction);
    })
        .then(function (result) {
        request = result;
        return request.bulk(temporaryTable)
            .then(function (rowCount) {
            logger.info('[AuditorEvaluationPlanMSSql::BulkAuditorEvaluationPlanTransaction] Inserted ' + rowCount + ' records to temporary Table');
            return true;
        })
            .catch(function (err) {
            logger.error('[AuditorEvaluationPlanMSSql::BulkAuditorEvaluationPlanTransaction] Error on bulk: ' + err.message);
            throw err;
        });
    })
        .then(function (_) {
        return request
            .execute('BulkAuditorEvaluationPlan')
            .then(function (recordsets) {
            logger.info('[AuditorEvaluationPlanMSSql::BulkAuditorEvaluationPlanTransaction] Succed Execution');
            var results = (recordsets && recordsets.length > 0 ? recordsets[0] : null);
            var promiseArray = [];
            promiseArray.push(Promise.resolve(results && results.length > 0 ? results[0] : null));
            promiseArray.push(transaction.commit());
            return Promise.all(promiseArray);
        })
            .catch(function (err) {
            logger.error('[AuditorEvaluationPlanMSSql::BulkAuditorEvaluationPlanTransaction] Error on: ' + err.message);
            return Promise.reject(err);
        });
    })
        .then(function (_) {
        return Promise.resolve(true);
    })
        .catch(function (err) {
        logger.error('[AuditorEvaluationPlanMSSql::BulkAuditorEvaluationPlanTransaction] Parsing error: ' + err.message);
        transaction.rollback();
        return Promise.reject(err);
    });
}
