"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.BulkTipificationInteractionTemp = exports.SaveEvaluationTipificationItem = exports.SaveEvaluationTipification = exports.EvaluationTipificationMSSql = void 0;
var log4js = require("log4js");
var sql = require('mssql');
var logger = log4js.getLogger('Sql');
var sqlConn;
function EvaluationTipificationMSSql(app) {
    sqlConn = app.get('sqlConn');
}
exports.EvaluationTipificationMSSql = EvaluationTipificationMSSql;
function SaveEvaluationTipification(vcc, abilityId, scoreId, id, isChecked, comment, evaluationPlanId, interactionId, auditorId, userId) {
    return new sql.Request(sqlConn)
        .input('VCC', sql.VarChar(50), vcc)
        .input('AbilityId', sql.VarChar(50), abilityId)
        .input('ScoreId', sql.VarChar(50), scoreId)
        .input('ID', sql.VarChar(50), id)
        .input('IsChecked', sql.Bit, isChecked)
        .input('Comment', sql.VarChar(500), comment)
        .input('EvaluationPlanId', sql.VarChar(50), evaluationPlanId)
        .input('InteractionId', sql.VarChar(250), interactionId)
        .input('AuditorId', sql.VarChar(50), auditorId)
        .input('UserId', sql.VarChar(50), userId)
        .execute('SaveEvaluationTipification')
        .then(function (recordsets) {
        var recordset = (recordsets === null || recordsets === void 0 ? void 0 : recordsets.length) ? recordsets[0] : null;
        var output = (recordset === null || recordset === void 0 ? void 0 : recordset.length) ? recordset[0] : null;
        return Promise.resolve((output === null || output === void 0 ? void 0 : output.result) === 1);
    })
        .catch(function (err) {
        var errorMsg = (err && err.procName ? 'Error al ejecutar \'' + err.procName + '\': ' : 'Error al ejecutar consulta: ') + (err ? typeof err === 'string' ? err : err.message || err.description || '' : '');
        logger.error(errorMsg);
        return Promise.reject(err);
    });
}
exports.SaveEvaluationTipification = SaveEvaluationTipification;
function SaveEvaluationTipificationItem(vcc, scoreId, tipificationId, id, isChecked, evaluationPlanId, interactionId, auditorId, userId) {
    return new sql.Request(sqlConn)
        .input('VCC', sql.VarChar(50), vcc)
        .input('ScoreId', sql.VarChar(50), scoreId)
        .input('TipificationId', sql.VarChar(50), tipificationId)
        .input('ID', sql.VarChar(50), id)
        .input('IsChecked', sql.Bit, isChecked)
        .input('EvaluationPlanId', sql.VarChar(50), evaluationPlanId)
        .input('InteractionId', sql.VarChar(250), interactionId)
        .input('AuditorId', sql.VarChar(50), auditorId)
        .input('UserId', sql.VarChar(50), userId)
        .execute('SaveEvaluationTipificationItem')
        .then(function (recordsets) {
        var recordset = (recordsets === null || recordsets === void 0 ? void 0 : recordsets.length) ? recordsets[0] : null;
        var output = (recordset === null || recordset === void 0 ? void 0 : recordset.length) ? recordset[0] : null;
        return Promise.resolve((output === null || output === void 0 ? void 0 : output.result) === 1);
    })
        .catch(function (err) {
        var errorMsg = (err && err.procName ? 'Error al ejecutar \'' + err.procName + '\': ' : 'Error al ejecutar consulta: ') + (err ? typeof err === 'string' ? err : err.message || err.description || '' : '');
        logger.error(errorMsg);
        return Promise.reject(err);
    });
}
exports.SaveEvaluationTipificationItem = SaveEvaluationTipificationItem;
function BulkTipificationInteractionTemp(interactions) {
    var table = SetTipificationInteractionsTemporaryTable(interactions);
    return BulkTipificationInteractionTransaction(table);
}
exports.BulkTipificationInteractionTemp = BulkTipificationInteractionTemp;
function SetTipificationInteractionsTemporaryTable(data) {
    var table = new sql.Table('#TipificationInteractionTemp');
    table.create = true;
    table.columns.add('VCC', sql.VarChar(50), { nullable: true });
    table.columns.add('AbilityId', sql.VarChar(50), { nullable: true });
    table.columns.add('ScoreId', sql.VarChar(50), { nullable: true });
    table.columns.add('ID', sql.VarChar(50), { nullable: true });
    table.columns.add('IsChecked', sql.Bit, { nullable: true });
    table.columns.add('EvaluationPlanId', sql.VarChar(50), { nullable: true });
    table.columns.add('InteractionId', sql.VarChar(250), { nullable: true });
    table.columns.add('AuditorId', sql.VarChar(50), { nullable: true });
    data.forEach(function (d) {
        if (d.succedTipifications) {
            d.succedTipifications.forEach(function (st) {
                table.rows.add(d.vcc, st.abilityId, st.scoreId, st.id, st.isChecked, st.evaluationPlanId, st.interactionId, st.auditorId);
            });
        }
    });
    return table;
}
function BulkTipificationInteractionTransaction(temporaryTable) {
    var transaction = new sql.Transaction(sqlConn);
    var request;
    return transaction.begin()
        .then(function (result) {
        logger.info('[EvaluationTipificationMSSql::BulkTipificationInteractionTransaction] Creating transaction');
        return new sql.Request(transaction);
    })
        .then(function (result) {
        request = result;
        return request.bulk(temporaryTable)
            .then(function (rowCount) {
            logger.info('[EvaluationTipificationMSSql::BulkTipificationInteractionTransaction] Inserted ' + rowCount + ' records to temporary Table');
            return true;
        })
            .catch(function (err) {
            logger.error('[EvaluationTipificationMSSql::BulkTipificationInteractionTransaction] Error on bulk: ' + err.message);
            throw err;
        });
    })
        .then(function (result) {
        return request
            .execute('BulkTipificationInteraction')
            .then(function (recordsets) {
            logger.info('[EvaluationTipificationMSSql::BulkTipificationInteractionTransaction] Succed Execution');
            var results = ((recordsets === null || recordsets === void 0 ? void 0 : recordsets.length) > 0 ? recordsets[0] : null);
            var promiseArray = [];
            promiseArray.push(Promise.resolve((results === null || results === void 0 ? void 0 : results.length) > 0 ? results[0] : null));
            promiseArray.push(transaction.commit());
            return Promise.all(promiseArray);
        })
            .catch(function (err) {
            logger.error('[EvaluationTipificationMSSql::BulkTipificationInteractionTransaction] Error on: ' + err.message);
            return Promise.reject(err);
        });
    })
        .then(function (result) {
        return Promise.resolve(true);
    })
        .catch(function (err) {
        logger.error('[EvaluationTipificationMSSql::BulkTipificationInteractionTransaction] Parsing error: ' + err.message);
        transaction.rollback();
        return Promise.reject(err);
    });
}
