"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.RedisInit = void 0;
var redis = require("redis");
var bluebird = require("bluebird");
var log4js = require("log4js");
var redis_config_1 = require("../config/redis.config");
var batch_schedule_redis_1 = require("./redis/batch-schedule.redis");
var domain_redis_1 = require("./redis/domain.redis");
var evaluation_redis_1 = require("./redis/evaluation.redis");
var evaluation_plan_ability_redis_1 = require("./redis/evaluation-plan-ability.redis");
var i6_integration_redis_1 = require("./redis/i6-integration.redis");
var vcc_redis_1 = require("./redis/vcc.redis");
var role_redis_1 = require("./redis/role.redis");
var session_redis_1 = require("./redis/session.redis");
var user_redis_1 = require("./redis/user.redis");
var interaction_redis_1 = require("./redis/interaction.redis");
var process_redis_1 = require("./redis/process.redis");
var cognitive_redis_1 = require("./redis/cognitive.redis");
var queue_redis_1 = require("./redis/queue.redis");
var platform_redis_1 = require("./redis/platform.redis");
var nlu_redis_1 = require("./redis/nlu.redis");
var study_template_theme_redis_1 = require("./redis/study-template-theme.redis");
var speechmatics_redis_1 = require("./redis/speechmatics.redis");
var tipification_redis_1 = require("./redis/tipification.redis");
var cleaner_redis_1 = require("./redis/cleaner.redis");
var logger = log4js.getLogger('Redis');
function RedisInit(app) {
    return new Promise(function (resolve, reject) {
        logger.info('Iniciando conexión a Redis');
        console.log('Iniciando conexión a Redis');
        var redisClient = redis.createClient(redis_config_1.REDIS_OPTIONS)
            .on('error', function (err) {
            var errMsg = (typeof err === 'string' ? err : err.message || err.description || 'SERVER_ERROR_UNKNOWN_MESSAGE');
            logger.error('No se pudo inicializar cliente Redis: ' + errMsg);
            console.error('No se pudo inicializar cliente Redis: ' + errMsg);
            reject(err);
        })
            .on('connect', function () {
            logger.info('Conexión a Redis establecida');
            console.log('Conexión a Redis establecida');
            resolve(true);
        });
        bluebird.promisifyAll(redisClient);
        app.set('redisClient', redisClient);
        (0, batch_schedule_redis_1.BatchScheduleRedis)(app);
        (0, domain_redis_1.DomainRedis)(app);
        (0, evaluation_redis_1.EvaluationRedis)(app);
        (0, evaluation_plan_ability_redis_1.EvaluationPlanAbilityRedis)(app);
        (0, i6_integration_redis_1.I6IntegrationRedis)(app);
        (0, vcc_redis_1.VccRedis)(app);
        (0, role_redis_1.RoleRedis)(app);
        (0, session_redis_1.SessionRedis)(app);
        (0, speechmatics_redis_1.SpeechmaticsRedis)(app);
        (0, study_template_theme_redis_1.StudyTemplateThemeRedis)(app);
        (0, tipification_redis_1.TipificationRedis)(app);
        (0, user_redis_1.UserRedis)(app);
        (0, interaction_redis_1.InteractionRedis)(app);
        (0, process_redis_1.ProcessRedis)(app);
        (0, cognitive_redis_1.CognitiveRedis)(app);
        (0, queue_redis_1.QueueRedis)(app);
        (0, platform_redis_1.PlatformRedis)(app);
        (0, nlu_redis_1.NluRedis)(app);
        (0, cleaner_redis_1.CleanerRedis)(app);
    });
}
exports.RedisInit = RedisInit;
