"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.DeletEvaluationStatus = exports.GetEvaluationsStatus = exports.GetEvaluationStatus = exports.SaveEvaluationStatus = exports.EvaluationRedis = void 0;
var moment = require("moment");
var log4js = require("log4js");
var redis_config_1 = require("../../config/redis.config");
var logger = log4js.getLogger('Redis');
var redisClient;
function EvaluationRedis(app) {
    redisClient = app.get('redisClient');
}
exports.EvaluationRedis = EvaluationRedis;
function SaveEvaluationStatus(vcc, evaluationToUpdate) {
    var storage = vcc + ':EvaluationStatus';
    var key = '';
    key += evaluationToUpdate.evaluationPlanId;
    key += ':';
    key += evaluationToUpdate.interactionId;
    key += ':';
    key += evaluationToUpdate.auditorId;
    evaluationToUpdate.unlockDate = moment().utc().add(30, 'minutes').toDate();
    if (!key)
        return Promise.resolve(false);
    return redisClient.hsetAsync(storage, key, JSON.stringify(evaluationToUpdate))
        .then(function (result) {
        return redisClient.expireAsync(storage, 60 * 30);
    })
        .then(function (result) {
        return Promise.resolve(true);
    })
        .catch(function (err) {
        logger.error('[EvaluationPlanAbilityRedis::SaveEvaluationPlanAbility] getting error:', err, 'key:', key);
        return Promise.resolve(err);
    });
}
exports.SaveEvaluationStatus = SaveEvaluationStatus;
function GetEvaluationStatus(vcc, evaluationToGet) {
    var storage = vcc + ':EvaluationStatus';
    var evaluations = [];
    var key = '';
    key += evaluationToGet.evaluationPlanId;
    key += ':';
    key += evaluationToGet.interactionId;
    key += ':';
    key += evaluationToGet.auditorId;
    if (!key)
        return Promise.resolve(null);
    return redisClient.hgetAsync(storage, key)
        .then(function (result) {
        var evaluation;
        try {
            evaluation = (typeof result === 'string') ? JSON.parse(result) : result;
        }
        catch (err) {
            return Promise.resolve(null);
        }
        return Promise.resolve(evaluation);
    })
        .catch(function (err) {
        logger.error('[EvaluationPlanAbilityRedis::GetEvaluationStatus] getting err value:', err, 'hash:', storage, 'key:', key);
        return Promise.resolve(null);
    });
}
exports.GetEvaluationStatus = GetEvaluationStatus;
function GetEvaluationsStatus(vcc) {
    var storage = vcc + ':EvaluationStatus';
    var evaluations = [];
    return redisClient.hgetallAsync(storage)
        .then(function (result) {
        var promiseArray = [];
        try {
            var evaluationsToSendResults_1 = (typeof result === 'string') ? JSON.parse(result) : result;
            if (result && evaluationsToSendResults_1) {
                Object.keys(evaluationsToSendResults_1).map(function (k) { return evaluations.push(JSON.parse(evaluationsToSendResults_1[k])); });
                Object.keys(evaluationsToSendResults_1).forEach(function (k) {
                    var evaluation = JSON.parse(evaluationsToSendResults_1[k]);
                    var unlockDate = moment(evaluation.unlockDate);
                    evaluation.unlockDate = moment(evaluation.unlockDate).toDate();
                    if (unlockDate.isSameOrBefore(moment().utc()))
                        promiseArray.push(DeletEvaluationStatus(vcc, evaluation));
                    else
                        evaluations.push(evaluation);
                });
            }
        }
        catch (err) {
            logger.error('[EvaluationPlanAbilityRedis::GetEvaluationsStatus] getting err value:', result);
            return Promise.resolve(null);
        }
        if (promiseArray === null || promiseArray === void 0 ? void 0 : promiseArray.length)
            return Promise.all(promiseArray);
        else
            return Promise.all([true]);
    })
        .then(function (_) {
        return Promise.resolve(evaluations);
    })
        .catch(function (err) {
        logger.error('[EvaluationPlanAbilityRedis::GetEvaluationsStatus] getting err value:', err);
        return Promise.resolve(err);
    });
}
exports.GetEvaluationsStatus = GetEvaluationsStatus;
function DeletEvaluationStatus(vcc, evaluationToUpdate) {
    var storage = vcc + ':EvaluationStatus';
    var key = '';
    key += evaluationToUpdate.evaluationPlanId;
    key += ':';
    key += evaluationToUpdate.interactionId;
    key += ':';
    key += evaluationToUpdate.auditorId;
    return redisClient.hdelAsync(storage, key)
        .then(function (result) {
        return redisClient.expireAsync(storage, redis_config_1.REDIS_DATA_PERSISTENCE);
    })
        .then(function (result) {
        return Promise.resolve(true);
    })
        .catch(function (err) {
        logger.error('[EvaluationRedis::DeletEvaluationStatus] getting error:', err, 'with key:', key, 'with storage:', storage);
        return Promise.resolve(false);
    });
}
exports.DeletEvaluationStatus = DeletEvaluationStatus;
