"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.DeleteRetry = exports.DeleteErrorProgress = exports.DeleteProgress = exports.DeleteTotalProgress = exports.GetRetry = exports.GetErrorProgress = exports.GetProgress = exports.GetTotalProgress = exports.UpdateRetry = exports.UpdateErrorProgress = exports.UpdateProgress = exports.SaveRetry = exports.SaveErrorProgress = exports.SaveProgress = exports.SaveTotalProgress = exports.GetLengthFromList = exports.Del = exports.DeleteInsertOnList = exports.InsertQueue = exports.DeleteQueue = exports.CreateQueueR = exports.CreateQueueL = exports.RPopLPush = exports.LRange = exports.RemoveQueueR = exports.RemoveQueueL = exports.QueueRedis = void 0;
var log4js = require("log4js");
var redis_config_1 = require("../../config/redis.config");
var promises_shared_1 = require("../../shared/promises.shared");
var logger = log4js.getLogger('Redis');
var redisClient;
function QueueRedis(app) {
    redisClient = app.get('redisClient');
}
exports.QueueRedis = QueueRedis;
function RemoveQueueL(queueName) {
    return redisClient.lpopAsync(queueName)
        .then(function (result) {
        return Promise.resolve(result);
    })
        .catch(function (err) {
        logger.error('[InteractionRedis::RemoveQueueL] getting err value:', err, 'with name:', queueName);
        return Promise.resolve(null);
    });
}
exports.RemoveQueueL = RemoveQueueL;
function RemoveQueueR(queueName) {
    return redisClient.rpopAsync(queueName)
        .then(function (result) {
        return Promise.resolve(result);
    })
        .catch(function (err) {
        logger.error('[InteractionRedis::RemoveQueueR] getting err value:', err, 'with name:', queueName);
        return Promise.resolve(null);
    });
}
exports.RemoveQueueR = RemoveQueueR;
function LRange(source, from, to) {
    return redisClient.lrangeAsync(source, from, to)
        .then(function (result) {
        return Promise.resolve(result);
    })
        .catch(function (err) {
        logger.error('[InteractionRedis::LRange] getting err value:', err, 'with source:', source);
        return Promise.resolve(null);
    });
}
exports.LRange = LRange;
function RPopLPush(source, destination) {
    return redisClient.rpoplpushAsync(source, destination)
        .then(function (result) {
        return Promise.resolve(result);
    })
        .catch(function (err) {
        logger.error('[InteractionRedis::RPopLPush] getting err value:', err, 'with source:', source, 'with destination:', destination);
        return Promise.resolve(null);
    });
}
exports.RPopLPush = RPopLPush;
function CreateQueueL(queueName, value) {
    return redisClient.lpushAsync(queueName, typeof value === 'object' ? JSON.stringify(value) : value)
        .then(function (result) {
        return Promise.resolve(result);
    })
        .catch(function (err) {
        logger.error('[InteractionRedis::CreateQueueL] getting err value:', err, 'with queueName:', queueName, 'with value:', value);
        return Promise.resolve(null);
    });
}
exports.CreateQueueL = CreateQueueL;
function CreateQueueR(queueName, value) {
    return redisClient.rpushAsync(queueName, typeof value === 'object' ? JSON.stringify(value) : value)
        .then(function (result) {
        return Promise.resolve(result);
    })
        .catch(function (err) {
        logger.error('[InteractionRedis::CreateQueueR] getting err value:', err, 'with queueName:', queueName, 'with value:', value);
        return Promise.resolve(null);
    });
}
exports.CreateQueueR = CreateQueueR;
function DeleteQueue(queueName, value) {
    return redisClient.lremAsync(queueName, 1, value)
        .then(function (result) {
        return Promise.resolve(result);
    })
        .catch(function (err) {
        logger.error('[InteractionRedis::DeleteQueue] getting err value:', err, 'with queueName:', queueName, 'with value:', value);
        return Promise.resolve(null);
    });
}
exports.DeleteQueue = DeleteQueue;
function InsertQueue(queueName, value, index) {
    return redisClient.llenAsync(queueName)
        .then(function (result) {
        var total = 0;
        try {
            total = (typeof result === 'string') ? JSON.parse(result) : result;
        }
        catch (err) {
            total = 0;
        }
        var position = total > index ? index : total;
        logger.warn('[InsertQueue] position', 'with queueName:', queueName, 'with value:', position);
        return redisClient.lindexAsync(queueName, position);
    }).then(function (result) {
        return redisClient.linsertAsync(queueName, 'BEFORE', (result || ''), value);
    }).then(function (result) {
        var position = 0;
        try {
            position = (typeof result === 'string') ? JSON.parse(result) : result;
        }
        catch (err) {
            position = -1;
        }
        if (position <= 0) {
            logger.warn('[InsertQueue] lpushAsync', 'with queueName:', queueName, 'with value:', value);
            return redisClient.lpushAsync(queueName, typeof value === 'object' ? JSON.stringify(value) : value);
        }
        return position;
    }).then(function (result) {
        return Promise.resolve(result);
    }).catch(function (err) {
        logger.error('[InsertQueue] getting err value:', err, 'with queueName:', queueName, 'with value:', value);
        return Promise.resolve(null);
    });
}
exports.InsertQueue = InsertQueue;
function DeleteInsertOnList(source, destination, value) {
    return (0, promises_shared_1.DummyPromise)()
        .then(function (_) {
        return new Promise(function (resolve, reject) {
            redisClient.multi()
                .lrem(source, 1, value)
                .rpush(destination, value)
                .exec(function (err, res) {
                if (err) {
                    logger.error('[QueueRedis::DeleteInsertOnList] getting error:', err, 'on multi value:', value);
                    return reject('SERVER_ERROR_QUEUE_EXCHANGE_FAILED');
                }
                else {
                    if (res[0] === 0) {
                        logger.warn("[QueueRedis::DeleteInsertOnList] getting warning: ".concat(value, " key not found on ").concat(source, " queue"));
                        if (res[0] === 1)
                            redisClient.lremAsync(destination, 1, value);
                        return reject('SERVER_WARNING_QUEUE_NOT_FOUND_ON_ORIGIN');
                    }
                    return resolve(true);
                }
            });
        });
    })
        .then(function (result) {
        logger.info('[QueueRedis::DeleteInsertOnList] getting result:', result, 'after multi value:', value);
        return Promise.resolve(result);
    })
        .catch(function (err) {
        logger.error('[QueueRedis::DeleteInsertOnList] getting err value:', err, 'listSource:', source, 'value:', value);
        return Promise.resolve(null);
    });
}
exports.DeleteInsertOnList = DeleteInsertOnList;
function Del(hash, executionCode) {
    if (!hash)
        return Promise.resolve(0);
    return redisClient.delAsync(hash)
        .then(function (result) {
        logger.info('[Del] executionCode:', executionCode, 'getting result:', result, 'after delAsync hash:', hash);
        return Promise.resolve(result);
    })
        .catch(function (err) {
        logger.error('[InteractionRedis::SaveReadyInteractionProcess] getting error:', err, 'with hash:', hash);
        return Promise.resolve(null);
    });
}
exports.Del = Del;
function GetLengthFromList(queueName) {
    return redisClient.llenAsync(queueName)
        .then(function (result) {
        return Promise.resolve(parseInt(result));
    })
        .catch(function (err) {
        var errorMsg = (typeof err === 'string' ? err : err.message || err.description);
        logger.error('[queue.redis::GetLengthFromList] getting error', errorMsg, 'list:', queueName);
        return Promise.resolve(0);
    });
}
exports.GetLengthFromList = GetLengthFromList;
function SaveTotalProgress(key, value) {
    var storage = 'TotalProgress';
    if (!key) {
        logger.error('[SaveTotalProgress] Received null Key');
        return Promise.resolve(false);
    }
    return redisClient.hsetAsync(storage, key, JSON.stringify(value))
        .then(function (result) {
        return redisClient.expireAsync(storage, redis_config_1.REDIS_DATA_PERSISTENCE);
    })
        .then(function (result) {
        logger.info('[SaveTotalProgress] TotalProgress created on Redis: ' + key);
        return Promise.resolve(true);
    })
        .catch(function (err) {
        logger.error('[SaveTotalProgress] getting error:', err);
        return Promise.resolve(false);
    });
}
exports.SaveTotalProgress = SaveTotalProgress;
function SaveProgress(key, value) {
    var storage = 'Progress';
    if (!key)
        return Promise.resolve(false);
    return redisClient.hsetAsync(storage, key, JSON.stringify(value))
        .then(function (result) {
        return redisClient.expireAsync(storage, redis_config_1.REDIS_DATA_PERSISTENCE);
    })
        .then(function (result) {
        return Promise.resolve(true);
    })
        .catch(function (err) {
        logger.error('[SaveProgress] getting error:', err);
        return Promise.resolve(false);
    });
}
exports.SaveProgress = SaveProgress;
function SaveErrorProgress(key, value) {
    var storage = 'ErrorProgress';
    if (!key)
        return Promise.resolve(false);
    return redisClient.hsetAsync(storage, key, JSON.stringify(value))
        .then(function (result) {
        return redisClient.expireAsync(storage, redis_config_1.REDIS_DATA_PERSISTENCE);
    })
        .then(function (result) {
        return Promise.resolve(true);
    })
        .catch(function (err) {
        logger.error('[SaveProgress] getting error:', err);
        return Promise.resolve(false);
    });
}
exports.SaveErrorProgress = SaveErrorProgress;
function SaveRetry(key, value) {
    var storage = 'Retry';
    if (!key)
        return Promise.resolve(false);
    return redisClient.hsetAsync(storage, key, JSON.stringify(value))
        .then(function (result) {
        return redisClient.expireAsync(storage, redis_config_1.REDIS_DATA_PERSISTENCE);
    })
        .then(function (result) {
        return Promise.resolve(true);
    })
        .catch(function (err) {
        logger.error('[SaveRetry] getting error:', err);
        return Promise.resolve(false);
    });
}
exports.SaveRetry = SaveRetry;
function UpdateProgress(key) {
    var storage = 'Progress';
    return redisClient.hincrbyAsync(storage, key, -1)
        .then(function (result) {
        var value;
        try {
            value = (typeof result === 'string') ? JSON.parse(result) : result;
        }
        catch (err) {
            return Promise.resolve(null);
        }
        return value;
    })
        .catch(function (err) {
        logger.error('[UpdateProgress] getting err value:', err, 'storage:', storage, 'key:', key);
        return Promise.resolve(null);
    });
}
exports.UpdateProgress = UpdateProgress;
function UpdateErrorProgress(key) {
    var storage = 'ErrorProgress';
    return redisClient.hincrbyAsync(storage, key, 1)
        .then(function (result) {
        var value;
        try {
            value = (typeof result === 'string') ? JSON.parse(result) : result;
        }
        catch (err) {
            return Promise.resolve(null);
        }
        return value;
    })
        .catch(function (err) {
        logger.error('[UpdateProgress] getting err value:', err, 'storage:', storage, 'key:', key);
        return Promise.resolve(null);
    });
}
exports.UpdateErrorProgress = UpdateErrorProgress;
function UpdateRetry(key) {
    var storage = 'Retry';
    return redisClient.hincrbyAsync(storage, key, 1)
        .then(function (result) {
        var value;
        try {
            value = (typeof result === 'string') ? JSON.parse(result) : result;
        }
        catch (err) {
            return Promise.resolve(null);
        }
        return value;
    })
        .catch(function (err) {
        logger.error('[UpdateProgress] getting err value:', err, 'storage:', storage, 'key:', key);
        return Promise.resolve(null);
    });
}
exports.UpdateRetry = UpdateRetry;
function GetTotalProgress() {
    var storage = 'TotalProgress';
    return redisClient.hgetallAsync(storage)
        .then(function (result) {
        var summary = [];
        try {
            var results = (typeof result === 'string') ? JSON.parse(result) : result;
            if (result && results) {
                for (var key in results) {
                    var keys = key.split(':');
                    var obj = { key: key, vcc: keys[0], processId: keys[1], type: keys[2] ? keys[2] : null, typeId: keys[3] ? keys[3] : null, value: results[key] };
                    summary.push(obj);
                }
            }
        }
        catch (err) {
            logger.error('[GetTotalProgress] getting err value:', result);
            return Promise.resolve(null);
        }
        return summary;
    })
        .catch(function (err) {
        logger.error('[GetTotalProgress] getting err value:', err);
        return Promise.resolve(null);
    });
}
exports.GetTotalProgress = GetTotalProgress;
function GetProgress() {
    var storage = 'Progress';
    return redisClient.hgetallAsync(storage)
        .then(function (result) {
        var summary = [];
        try {
            var results = (typeof result === 'string') ? JSON.parse(result) : result;
            if (result && results) {
                for (var key in results) {
                    var keys = key.split(':');
                    var obj = { key: key, vcc: keys[0], processId: keys[1], type: keys[2] ? keys[2] : null, typeId: keys[3] ? keys[3] : null, value: results[key] };
                    summary.push(obj);
                }
            }
        }
        catch (err) {
            logger.error('[GetProgress] getting err value:', result);
            return Promise.resolve(null);
        }
        return summary;
    })
        .catch(function (err) {
        logger.error('[GetProgress] getting err value:', err);
        return Promise.resolve(null);
    });
}
exports.GetProgress = GetProgress;
function GetErrorProgress() {
    var storage = 'ErrorProgress';
    return redisClient.hgetallAsync(storage)
        .then(function (result) {
        var summary = [];
        try {
            var results = (typeof result === 'string') ? JSON.parse(result) : result;
            if (result && results) {
                for (var key in results) {
                    var keys = key.split(':');
                    var obj = { key: key, vcc: keys[0], processId: keys[1], type: keys[2] ? keys[2] : null, typeId: keys[3] ? keys[3] : null, value: results[key] };
                    summary.push(obj);
                }
            }
        }
        catch (err) {
            logger.error('[GetErrorProgress] getting err value:', result);
            return Promise.resolve(null);
        }
        return summary;
    })
        .catch(function (err) {
        logger.error('[GetProgress] getting err value:', err);
        return Promise.resolve(null);
    });
}
exports.GetErrorProgress = GetErrorProgress;
function GetRetry(key) {
    var storage = 'Retry';
    return redisClient.hgetAsync(storage, key)
        .then(function (result) {
        var retry;
        try {
            var value = (typeof result === 'string') ? JSON.parse(result) : result;
            retry = { key: key, value: value };
        }
        catch (err) {
            return Promise.resolve(null);
        }
        return retry;
    })
        .catch(function (err) {
        logger.error('[GetRetry] getting err value:', err);
        return Promise.resolve(null);
    });
}
exports.GetRetry = GetRetry;
function DeleteTotalProgress(key) {
    var storage = 'TotalProgress';
    if (!key)
        return Promise.resolve(false);
    return redisClient.hdelAsync(storage, key)
        .then(function (result) {
        return Promise.resolve(result ? true : false);
    })
        .catch(function (err) {
        logger.error('[DeleteTotalProgress] getting error:', err, 'with key:', key);
        return Promise.resolve(null);
    });
}
exports.DeleteTotalProgress = DeleteTotalProgress;
function DeleteProgress(key) {
    var storage = 'Progress';
    if (!key)
        return Promise.resolve(false);
    return redisClient.hdelAsync(storage, key)
        .then(function (result) {
        return Promise.resolve(result ? true : false);
    })
        .catch(function (err) {
        logger.error('[DeleteProgress] getting error:', err, 'with key:', key);
        return Promise.resolve(null);
    });
}
exports.DeleteProgress = DeleteProgress;
function DeleteErrorProgress(key) {
    var storage = 'ErrorProgress';
    if (!key)
        return Promise.resolve(false);
    return redisClient.hdelAsync(storage, key)
        .then(function (result) {
        return Promise.resolve(result ? true : false);
    })
        .catch(function (err) {
        logger.error('[DeleteErrorProgress] getting error:', err, 'with key:', key);
        return Promise.resolve(null);
    });
}
exports.DeleteErrorProgress = DeleteErrorProgress;
function DeleteRetry(key) {
    var storage = 'Retry';
    if (!key)
        return Promise.resolve(false);
    return redisClient.hdelAsync(storage, key)
        .then(function (result) {
        return Promise.resolve(result ? true : false);
    })
        .catch(function (err) {
        logger.error('[DeleteRetry] getting error:', err, 'with key:', key);
        return Promise.resolve(null);
    });
}
exports.DeleteRetry = DeleteRetry;
