"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.GetCurrentJob = exports.SaveCurrentJob = exports.SpeechmaticsRedis = void 0;
var log4js = require("log4js");
var redis_config_1 = require("../../config/redis.config");
var logger = log4js.getLogger('Redis');
var redisClient;
function SpeechmaticsRedis(app) {
    redisClient = app.get('redisClient');
}
exports.SpeechmaticsRedis = SpeechmaticsRedis;
function SaveCurrentJob(executionCode, key, jobs) {
    if (!key)
        throw 'SERVER_ERROR_CANNOT_SAVE_WITH_NULL_ID';
    var storageId = "CurrentJobs:".concat(executionCode);
    return redisClient.hsetAsync(storageId, key, JSON.stringify(jobs))
        .then(function (_) {
        return redisClient.expireAsync(storageId, redis_config_1.REDIS_DATA_PERSISTENCE_ONE_HOUR);
    })
        .then(function (result) {
        return Promise.resolve(true);
    })
        .catch(function (err) {
        var errorMsg = '[SpeechmaticsRedis::SaveCurrentJob] Error on: ' + (err ? typeof err === 'string' ? err : err.message || err.description || '' : '');
        logger.error(errorMsg);
        return Promise.resolve(false);
    });
}
exports.SaveCurrentJob = SaveCurrentJob;
function GetCurrentJob(executionCode, key) {
    var storageId = "CurrentJobs:".concat(executionCode);
    return redisClient.hgetAsync(storageId, key)
        .then(function (result) {
        var content = null;
        try {
            content = (typeof result === 'string') ? JSON.parse(result) : result;
        }
        catch (err) {
            return Promise.resolve(null);
        }
        return Promise.resolve(content);
    })
        .catch(function (err) {
        logger.error('[SpeechmaticsRedis::GetCurrentJob] getting err value:', err, 'key:', key);
        return Promise.resolve(null);
    });
}
exports.GetCurrentJob = GetCurrentJob;
