"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.BillingAccountImplementation = void 0;
var log4js = require("log4js");
var promises_shared_1 = require("../shared/promises.shared");
var md5 = require("md5");
var platform_service_1 = require("../services/platform.service");
var icModule = require('intermediate-component-sa');
var logger = log4js.getLogger('ServerScripts');
var BillingAccountImplementation = (function () {
    function BillingAccountImplementation() {
    }
    BillingAccountImplementation.prototype.GetSystemInformation = function (guid) {
        var systemInformationData = {
            installationId: guid,
            macAddress: '',
            publicIp: ''
        };
        return (0, promises_shared_1.DummyPromise)()
            .then(function (result) {
            if (result === null)
                throw 'SERVER_ERROR_NOT_FOUND_NETWORK_INTERFACE_INFORMATION';
            if (result && result.length) {
                logger.info('[BillingAccountImplementation::GetSystemInformation] guid:', guid, 'getting result[0].mac:', result[0].mac);
                var myNetworkInterface = null;
                myNetworkInterface = result.find(function (r) { return r.operstate === 'up'; })[0].mac;
                systemInformationData.macAddress = myNetworkInterface.mac;
                systemInformationData.publicIp = myNetworkInterface.ip4;
            }
            logger.info('[BillingAccountImplementation::GetSystemInformation] guid:', guid, 'getting systemInformationData:', systemInformationData);
            return Promise.resolve(systemInformationData);
        }).catch(function (err) {
            logger.error('[BillingAccountImplementation::GetSystemInformation] guid:', guid, 'getting error:', err);
            return Promise.resolve(false);
        });
    };
    BillingAccountImplementation.prototype.GenerateMD5Content = function (data) {
        var result = '';
        if (data) {
            result += data.installationId;
            result += data.macAddress;
            logger.info('[BillingAccountImplementation::GenerateMD5Content] getting result:', result);
            result = md5(result);
        }
        return result;
    };
    BillingAccountImplementation.prototype.GetAccountBalance = function (account) {
        var _this = this;
        var systemInformationData = null;
        var installationId;
        return (0, promises_shared_1.DummyPromise)()
            .then(function (result) {
            return (0, platform_service_1.GetPlatformVersion)();
        }).then(function (result) {
            logger.info('[BillingAccountImplementation::GetPlatformAvailableLicenses] Getting result:', result, 'after GetVersion');
            if (result) {
                installationId = result.installationId;
                return _this.GetSystemInformation(result.installationId);
            }
            else {
                throw 'ERROR_NO_VERSION_DATA';
            }
        }).then(function (result) {
            systemInformationData = result;
            var plaformHash = '';
            if (systemInformationData) {
                plaformHash = _this.GenerateMD5Content(systemInformationData);
            }
            var body = {
                platformHash: plaformHash,
                installationId: systemInformationData.installationId,
                account: account
            };
            return icModule.InconcertRequest(installationId, 'IC_PARAM_URL_ACCOUNT_GET_BALANCE', body);
        }).then(function (result) {
            logger.info('[BillingAccountImplementation::GetPlatformAvailableLicenses] Getting result:', result, 'from external server');
            return Promise.resolve(result);
        }).catch(function (err) {
            logger.error('[BillingAccountImplementation::GetPlatformAvailableLicenses] Getting error:', err);
            return Promise.resolve(null);
        });
    };
    BillingAccountImplementation.prototype.GetAccountStatusByVCC = function (vcc) {
        var installationId;
        return (0, promises_shared_1.DummyPromise)()
            .then(function (result) {
            return (0, platform_service_1.GetPlatformVersion)();
        })
            .then(function (result) {
            logger.info('[BillingAccountImplementation::GetAccountStatusByVCC] Getting result:', result, 'after GetVersion');
            if (result) {
                installationId = result.installationId;
                var body = {
                    installationId: installationId,
                    vcc: vcc
                };
                return icModule.InconcertRequest(installationId, 'IC_PARAM_URL_ACCOUNT_GET_ACCOUNT_STATUS_BY_VCC', body);
            }
            else {
                throw 'ERROR_NO_VERSION_DATA';
            }
        })
            .then(function (result) {
            logger.info('[BillingAccountImplementation::GetAccountStatusByVCC] Getting result:', result, 'from external server');
            return Promise.resolve(result);
        })
            .catch(function (err) {
            logger.error('[BillingAccountImplementation::GetAccountStatusByVCC] Getting error:', err);
            return Promise.resolve(err);
        });
    };
    BillingAccountImplementation.prototype.GetVCCsByAccount = function (accountId, type) {
        var _this = this;
        var systemInformationData = null;
        var installationId;
        return (0, promises_shared_1.DummyPromise)()
            .then(function (result) {
            return (0, platform_service_1.GetPlatformVersion)();
        }).then(function (result) {
            logger.info('[BillingAccountImplementation::GetVCCsByAccount] Getting result:', result, 'after GetVersion');
            if (result) {
                installationId = result.installationId;
                return _this.GetSystemInformation(result.installationId);
            }
            else {
                throw 'ERROR_NO_VERSION_DATA';
            }
        }).then(function (result) {
            systemInformationData = result;
            var plaformHash = '';
            if (systemInformationData) {
                plaformHash = _this.GenerateMD5Content(systemInformationData);
            }
            var body = {
                platformHash: plaformHash,
                installationId: systemInformationData.installationId,
                account: accountId,
                type: type
            };
            return icModule.InconcertRequest(installationId, 'IC_PARAM_URL_ACCOUNT_GET_VCC', body);
        }).then(function (result) {
            logger.info('[BillingAccountImplementation::GetVCCsByAccount] Getting result:', result, 'from external server');
            return Promise.resolve(result);
        }).catch(function (err) {
            logger.error('[BillingAccountImplementation::GetVCCsByAccount] Getting error:', err);
            return Promise.resolve(null);
        });
    };
    BillingAccountImplementation.prototype.GetMovements = function (billingMovement) {
        var _this = this;
        logger.info('[BillingAccountImplementation::GetMovements] Getting billingMovement input:', billingMovement);
        var systemInformationData = null;
        var installationId;
        return (0, promises_shared_1.DummyPromise)()
            .then(function (result) {
            return (0, platform_service_1.GetPlatformVersion)();
        }).then(function (result) {
            logger.info('[BillingAccountImplementation::GetMovements] Getting result:', result, 'after GetVersion');
            if (result) {
                installationId = result.installationId;
                return _this.GetSystemInformation(result.installationId);
            }
            else {
                throw 'ERROR_NO_VERSION_DATA';
            }
        }).then(function (result) {
            systemInformationData = result;
            var plaformHash = '';
            if (systemInformationData) {
                plaformHash = _this.GenerateMD5Content(systemInformationData);
            }
            var body = {
                platformHash: plaformHash,
                installationId: systemInformationData.installationId,
                startDate: billingMovement.startDate,
                endDate: billingMovement.endDate,
                accountId: billingMovement.accountId,
                subAccountId: billingMovement.subAccountId,
                vcc: billingMovement.vcc,
                type: billingMovement.type
            };
            logger.info('[BillingAccountImplementation::GetMovements] Getting body:', body, 'before request api/account/get_movements');
            return icModule.InconcertRequest(installationId, 'IC_PARAM_URL_ACCOUNT_GET_MOVEMENTS', body);
        }).then(function (result) {
            logger.info('[BillingAccountImplementation::GetMovements] Getting result:', result, 'from external server');
            var movements = null;
            if (result && result.status) {
                movements = result.data;
            }
            else {
                throw 'ERROR_SERVER_GETTING_MOVEMENTS_DATA';
            }
            return Promise.resolve(movements);
        }).catch(function (err) {
            logger.error('[BillingAccountImplementation::GetMovements] Getting error:', err, 'billingMovement input:', billingMovement);
            return Promise.resolve(null);
        });
    };
    BillingAccountImplementation.prototype.GetMovementCurrencies = function (billingMovement) {
        var _this = this;
        logger.info('[BillingAccountImplementation::GetMovementCurrencies] Getting billingMovement input:', billingMovement);
        var systemInformationData = null;
        var installationId;
        return (0, promises_shared_1.DummyPromise)()
            .then(function (result) {
            return (0, platform_service_1.GetPlatformVersion)();
        })
            .then(function (result) {
            logger.info('[BillingAccountImplementation::GetMovementCurrencies] Getting result:', result, 'after GetVersion');
            if (result) {
                installationId = result.installationId;
                return _this.GetSystemInformation(result.installationId);
            }
            else {
                throw 'ERROR_NO_VERSION_DATA';
            }
        })
            .then(function (result) {
            systemInformationData = result;
            var plaformHash = '';
            if (systemInformationData) {
                plaformHash = _this.GenerateMD5Content(systemInformationData);
            }
            var body = {
                platformHash: plaformHash,
                installationId: systemInformationData.installationId,
                startDate: billingMovement.startDate,
                endDate: billingMovement.endDate,
                accountId: billingMovement.accountId,
                subAccountId: billingMovement.subAccountId,
                vcc: billingMovement.vcc,
                type: billingMovement.type
            };
            logger.info('[BillingAccountImplementation::GetMovementCurrencies] Getting body:', body, 'before request api/account/GetMovementCurrencies');
            return icModule.InconcertRequest(installationId, 'IC_PARAM_URL_ACCOUNT_GET_MOVEMENT_CURRENCIES', body);
        })
            .then(function (result) {
            logger.info('[BillingAccountImplementation::GetMovementCurrencies] Getting result:', result, 'from external server');
            var movements = null;
            if (result && result.status) {
                movements = result.data;
            }
            else {
                throw 'ERROR_SERVER_GETTING_MOVEMENT_CURRENCIES_DATA';
            }
            return Promise.resolve(movements);
        })
            .catch(function (err) {
            logger.error('[BillingAccountImplementation::GetMovementCurrencies] Getting error:', err, 'billingMovement input:', billingMovement);
            return Promise.resolve(null);
        });
    };
    BillingAccountImplementation.prototype.GetCosts = function (billingMovement) {
        var _this = this;
        logger.info('[BillingAccountImplementation::GetCosts] Getting billingMovement input:', billingMovement);
        var systemInformationData = null;
        var installationId;
        return (0, promises_shared_1.DummyPromise)()
            .then(function (result) {
            return (0, platform_service_1.GetPlatformVersion)();
        }).then(function (result) {
            logger.info('[BillingAccountImplementation::GetCosts] Getting result:', result, 'after GetVersion');
            if (result) {
                installationId = result.installationId;
                return _this.GetSystemInformation(result.installationId);
            }
            else {
                throw 'ERROR_NO_VERSION_DATA';
            }
        }).then(function (result) {
            systemInformationData = result;
            var plaformHash = '';
            if (systemInformationData) {
                plaformHash = _this.GenerateMD5Content(systemInformationData);
            }
            var body = {
                platformHash: plaformHash,
                installationId: systemInformationData.installationId,
                startDate: billingMovement.startDate,
                endDate: billingMovement.endDate,
                accountId: billingMovement.accountId,
                subAccountId: billingMovement.subAccountId,
                vcc: billingMovement.vcc,
                type: billingMovement.type
            };
            logger.info('[BillingAccountImplementation::GetCosts] Getting body:', body, 'before request api/account/get_movements');
            return icModule.InconcertRequest(installationId, 'IC_PARAM_URL_ACCOUNT_GET_COSTS', body);
        }).then(function (result) {
            logger.info('[BillingAccountImplementation::GetCosts] Getting result:', result, 'from external server');
            var costs = null;
            if (result && result.status) {
                costs = result.data;
            }
            else {
                throw 'ERROR_SERVER_GETTING_COSTS_DATA';
            }
            return Promise.resolve(costs);
        }).catch(function (err) {
            logger.error('[BillingAccountImplementation::GetCosts] Getting error:', err, 'billingMovement input:', billingMovement);
            return Promise.resolve(null);
        });
    };
    return BillingAccountImplementation;
}());
exports.BillingAccountImplementation = BillingAccountImplementation;
