"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.GetCognitiveService = exports.CognitiveAnalysisImplementation = void 0;
var log4js = require("log4js");
var mssql_service_1 = require("../db/mssql.service");
var redis_service_1 = require("../db/redis.service");
var moment = require("moment");
var config_1 = require("../config/config");
var promises_shared_1 = require("../shared/promises.shared");
var file_manager_1 = require("../shared/file-manager");
var billing_implementation_1 = require("./billing.implementation");
var luis_service_implementation_1 = require("./luis-service.implementation");
var rasa_service_implementation_1 = require("./rasa-service.implementation");
var billing_batch_model_1 = require("../models/billing-batch.model");
var audio_service_1 = require("../services/audio.service");
var rasa_api_service_implementation_1 = require("./rasa-api-service.implementation");
var logger = log4js.getLogger('Task');
var cognitiveService;
var CURRENT_SYSTEM_SERVICE = 'COGNITIVE_ANALYSIS';
var CURRENT_IMPLEMENTATION = 'CognitiveAnalysisImplementation';
var useRasaApi = config_1.USE_RASA_API;
var CognitiveAnalysisImplementation = (function () {
    function CognitiveAnalysisImplementation() {
        this.errorData = new billing_batch_model_1.BatchError();
    }
    CognitiveAnalysisImplementation.prototype.Execute = function (executionCode, rateRecursive, availableBatch, cancelledBatch) {
        var _this = this;
        var executeIndex = 1;
        var ratio = config_1.RATIO_COGNITIVE_ANALYSIS;
        var alowedRatio = 0;
        var notAlowedRatio = 0;
        var executeResult = true;
        return (0, promises_shared_1.SleepPromise)(2)
            .then(function (result) {
            return redis_service_1.Redis.LRange('CognitiveAnalysis', 0, ratio - 1);
        })
            .then(function (result) {
            var promiseArray = [];
            ratio = result && result.length <= ratio ? result.length : 0;
            logger.info('[CognitiveAnalysisImplementation::Execute] CognitiveAnalysis lrange result: ' + result);
            result.map(function (r) {
                var arrQueue = r.split(':');
                if (availableBatch.find(function (b) { return b.VCC === arrQueue[0] && b.BatchTypeId.toLowerCase() === arrQueue[7].toLowerCase(); })) {
                    alowedRatio++;
                }
                else {
                    notAlowedRatio++;
                }
            });
            while (ratio >= executeIndex) {
                promiseArray.push(_this.CognitiveAnalysisExecute(executionCode, availableBatch, cancelledBatch));
                executeIndex++;
            }
            return Promise.all(promiseArray);
        }).then(function (results) {
            if (results.filter(function (item) { return item === false; }).length > notAlowedRatio) {
                executeResult = false;
            }
            return redis_service_1.Redis.LRange('CognitiveAnalysis', 0, ratio - 1);
        }).then(function (result) {
            if (result && result.length && rateRecursive >= 1)
                return _this.Execute(executionCode, rateRecursive - 1, availableBatch, cancelledBatch);
            return executeResult;
        }).then(function (result) {
            var currentResult = executeResult && result ? true : false;
            return Promise.resolve(currentResult);
        }).catch(function (err) {
            var errorMessage = typeof err === 'string' ? err : (err.message || err.description);
            logger.error('[CognitiveAnalysisImplementation::Execute] getting error:', errorMessage);
            return Promise.resolve(false);
        });
    };
    CognitiveAnalysisImplementation.prototype.CognitiveAnalysisExecute = function (executionCode, availableBatch, cancelledBatch) {
        var _this = this;
        var vcc = '';
        var processId = '';
        var interactionType = '';
        var type = '';
        var typeId = '';
        var currentQueueId = '';
        var interactionCognitive;
        var queueProgressStorage = '';
        var filePath = '';
        return redis_service_1.Redis.RemoveQueueL('CognitiveAnalysis')
            .then(function (result) {
            var arrQueue = result.split(':');
            var promiseArray = [];
            if (availableBatch.find(function (b) { return b.VCC === arrQueue[0] && b.BatchTypeId.toLowerCase() === arrQueue[8].toLowerCase(); })) {
                promiseArray.push(Promise.resolve('ALLOWED'));
                promiseArray.push(redis_service_1.Redis.CreateQueueL('CognitiveAnalysisExecute', result));
                promiseArray.push(Promise.resolve(result));
            }
            else {
                promiseArray.push(Promise.resolve('NOT_ALLOWED'));
                promiseArray.push(redis_service_1.Redis.CreateQueueL('CognitiveAnalysis', result));
                promiseArray.push(mssql_service_1.MSSql.SaveBatchStatus(arrQueue[0], arrQueue[8], 'COGNITIVE_ANALYSIS', 'PAUSED'));
            }
            return Promise.all(promiseArray);
        })
            .then(function (result) {
            if (result && result[0] === 'NOT_ALLOWED')
                throw 'SERVER_ERROR_NOT_ALLOWED_QUEUE';
            var queueId = result[2];
            if (queueId && queueId.length > 0) {
                currentQueueId = queueId;
                logger.info('[CognitiveQueryExecute] executionCode:', executionCode, 'getting queueId:', currentQueueId, 'START');
                var item = currentQueueId.split(':');
                if (item && item.length) {
                    vcc = item[0];
                    processId = item[1];
                    interactionType = item[2];
                    type = item[7] ? item[7] : null;
                    typeId = item[8] ? item[8] : null;
                    if (type && typeId)
                        queueProgressStorage = vcc + ':' + processId + ':' + type + ':' + typeId;
                    if (type && !typeId)
                        queueProgressStorage = vcc + ':' + processId + ':' + type;
                    logger.info('[CognitiveAnalysisExecute] executionCode:', executionCode, 'getting queueId:', currentQueueId, 'storage :', queueProgressStorage);
                    return redis_service_1.Redis.GetCognitiveInteraction(currentQueueId);
                }
                else {
                    logger.error('[CognitiveAnalysisExecute] executionCode:', executionCode, 'getting queueId:', currentQueueId, 'QUEUE_FORMAT_ERROR');
                }
            }
            return null;
        }).then(function (result) {
            if (!result)
                logger.warn('[CognitiveAnalysisExecute] executionCode:', executionCode, 'getting queueId:', currentQueueId, 'getting error: NOT_FOUND_INTERACTION_COGNITIVE');
            if (result) {
                if (cancelledBatch.find(function (b) { return b.VCC === result.vcc && b.BatchTypeId.toLowerCase() === (result.studyId ? result.studyId : result.evaluationPlanId).toLowerCase(); })) {
                    result.status = -2;
                    result.statusDescription = 'REMOVED_BY_CANCELLED';
                    result.retry = config_1.RETRY_COGNITIVE_ANALYSIS;
                    result.errorFunction = 'CognitiveAnalysisExecute';
                    result.errorMessage = 'REMOVED_BY_CANCELLED';
                    result.errorDetail = 'REMOVED_BY_CANCELLED';
                }
            }
            interactionCognitive = result;
            return (0, file_manager_1.CreateFolder)(config_1.COGNITIVE_ANALYSIS_FOLDER + '/' + vcc);
        }).then(function (result) {
            if (!result)
                throw 'ERROR_CREATE_COGNITIVE_ANALYSIS_FOLDER';
            if (interactionCognitive && interactionCognitive.statusDescription === 'pendingPhrases') {
                logger.info('[CognitiveAnalysisImplementation::CognitiveAnalysisExecute] executionCode:', executionCode, 'getting queueId:', currentQueueId, 'pendingPhrases');
                return _this.GetPhrasesFromIdathaJson(interactionCognitive, currentQueueId, executionCode);
            }
            else if (interactionCognitive && interactionCognitive.statusDescription === 'pendingNLU') {
                logger.info('[CognitiveAnalysisImplementation::CognitiveAnalysisExecute] executionCode:', executionCode, 'getting queueId:', currentQueueId, 'pendingNLU');
                return _this.GetCognitive(interactionCognitive, currentQueueId, executionCode);
            }
            return Promise.resolve(null);
        }).then(function (resultInteraction) {
            if (interactionCognitive)
                return redis_service_1.Redis.SaveCognitiveInteraction(currentQueueId, interactionCognitive);
            return Promise.resolve(null);
        }).then(function (result) {
            var promiseArray = [];
            if (result && (interactionCognitive === null || interactionCognitive === void 0 ? void 0 : interactionCognitive.status) === 6 && (interactionCognitive === null || interactionCognitive === void 0 ? void 0 : interactionCognitive.statusDescription) === 'ready') {
                logger.info('[CognitiveAnalysisExecute] executionCode:', executionCode, 'getting queueId:', currentQueueId, 'READY');
                _this.CloseCognitiveModel(currentQueueId, true, 'SUCCED');
                promiseArray.push(redis_service_1.Redis.DeleteQueue('CognitiveAnalysisExecute', currentQueueId));
                if (type === 'tipification') {
                    interactionCognitive.statusDescription = 'pendingAnalysis';
                    promiseArray.push(redis_service_1.Redis.CreateQueueL('TipificationAnalysis', currentQueueId));
                    promiseArray.push(redis_service_1.Redis.SaveTipificationInteraction(currentQueueId, interactionCognitive));
                }
            }
            else if (result && (interactionCognitive === null || interactionCognitive === void 0 ? void 0 : interactionCognitive.status) === 6 && (interactionCognitive === null || interactionCognitive === void 0 ? void 0 : interactionCognitive.statusDescription) === 'pendingNLU' && interactionCognitive.retry < config_1.RETRY_COGNITIVE_ANALYSIS) {
                promiseArray.push(redis_service_1.Redis.CreateQueueL('CognitiveAnalysis', currentQueueId));
                promiseArray.push(redis_service_1.Redis.DeleteQueue('CognitiveAnalysisExecute', currentQueueId));
            }
            else if (result && interactionCognitive && interactionCognitive.status === -2 && interactionCognitive.statusDescription === 'REMOVED_BY_CANCELLED') {
                logger.error('[CognitiveAnalysisImplementation::CognitiveAnalysisExecute] executionCode:', executionCode, 'getting queueId:', currentQueueId, 'REMOVED_BY_CANCELLED');
                _this.CloseCognitiveModel(currentQueueId, false, 'REMOVED_BY_CANCELLED');
                promiseArray.push(redis_service_1.Redis.DeleteQueue('CognitiveAnalysisExecute', currentQueueId));
            }
            else if (result && interactionCognitive && interactionCognitive.status === -6 && interactionCognitive.statusDescription === 'ERROR_FILE_JSON') {
                _this.CloseCognitiveModel(currentQueueId, false, 'JSON_PROBLEM');
                promiseArray.push(redis_service_1.Redis.DeleteQueue('CognitiveAnalysisExecute', currentQueueId));
            }
            else if (interactionCognitive && interactionCognitive.retry >= config_1.RETRY_COGNITIVE_ANALYSIS) {
                logger.error('[CognitiveAnalysisImplementation::CognitiveAnalysisExecute] executionCode:', executionCode, 'getting queueId:', currentQueueId, 'REMOVED_BY_RETRIES');
                _this.CloseCognitiveModel(currentQueueId, false, 'REMOVED_BY_RETRIES');
                promiseArray.push(redis_service_1.Redis.DeleteQueue('CognitiveAnalysisExecute', currentQueueId));
            }
            else if (currentQueueId && !interactionCognitive) {
                logger.error('[CognitiveAnalysisImplementation::CognitiveAnalysisExecute] executionCode:', executionCode, 'getting queueId:', currentQueueId, 'INTERACTION_DO_NOT_EXIST');
                _this.CloseCognitiveModel(currentQueueId, false, 'INTERACTION_DO_NOT_EXIST');
                promiseArray.push(redis_service_1.Redis.DeleteQueue('CognitiveAnalysisExecute', currentQueueId));
            }
            else {
                logger.error('[CognitiveAnalysisImplementation::CognitiveAnalysisExecute] executionCode:', executionCode, 'getting queueId:', currentQueueId, 'QUEUE_WAS_NOT_REMOVED');
            }
            return Promise.all(promiseArray);
        }).then(function (result) {
            var promiseArray = [];
            if (interactionCognitive
                && interactionCognitive.status === 6
                && interactionCognitive.statusDescription === 'ready') {
                promiseArray.push(redis_service_1.Redis.UpdateProgress(queueProgressStorage));
                promiseArray.push(redis_service_1.Redis.DeleteCognitiveInteraction(currentQueueId));
                if (['study', 'template'].includes(type)) {
                    logger.debug('[CognitiveAnalysisImplementation::CognitiveAnalysisExecute] executionCode:', executionCode, 'getting queueId:', currentQueueId, 'Saving cognitive result for study:', interactionCognitive);
                    promiseArray.push(redis_service_1.Redis.SaveReadyCognitiveInteraction(currentQueueId, interactionCognitive, executionCode));
                }
                else if (['ability', 'tipification'].includes(type)) {
                    logger.debug('[CognitiveAnalysisImplementation::CognitiveAnalysisExecute] executionCode:', executionCode, 'getting queueId:', currentQueueId, 'Saving cognitive result for evaluationplan:', interactionCognitive);
                    promiseArray.push(redis_service_1.Redis.SaveReadyCognitiveAbility(currentQueueId, interactionCognitive, executionCode));
                }
                else {
                    logger.error('[CognitiveAnalysisImplementation::CognitiveAnalysisExecute] executionCode:', executionCode, 'getting queueId:', currentQueueId, 'TYPE_IS_NOT_FOUND', type);
                }
            }
            else if (interactionCognitive && interactionCognitive.retry >= config_1.RETRY_COGNITIVE_ANALYSIS) {
                promiseArray.push(redis_service_1.Redis.UpdateProgress(queueProgressStorage));
                promiseArray.push(redis_service_1.Redis.UpdateErrorProgress(queueProgressStorage));
                promiseArray.push(redis_service_1.Redis.DeleteCognitiveInteraction(currentQueueId));
                if (['study', 'template'].includes(type)) {
                    logger.error('[CognitiveAnalysisImplementation::CognitiveAnalysisExecute] executionCode:', executionCode, 'getting queueId:', currentQueueId, 'Saving cognitive result for study:', interactionCognitive);
                    promiseArray.push(redis_service_1.Redis.SaveReadyCognitiveInteraction(currentQueueId, interactionCognitive, executionCode));
                }
                else if (['ability', 'tipification'].includes(type)) {
                    logger.error('[CognitiveAnalysisImplementation::CognitiveAnalysisExecute] executionCode:', executionCode, 'getting queueId:', currentQueueId, 'Saving cognitive result for evaluationplan:', interactionCognitive);
                    promiseArray.push(redis_service_1.Redis.SaveReadyCognitiveAbility(currentQueueId, interactionCognitive, executionCode));
                }
                else {
                    logger.error('[CognitiveAnalysisImplementation::CognitiveAnalysisExecute] executionCode:', executionCode, 'getting queueId:', currentQueueId, 'TYPE_IS_NOT_FOUND', type);
                }
                promiseArray.push(mssql_service_1.MSSql.SaveBatchError(vcc, CURRENT_SYSTEM_SERVICE, typeId, ['study', 'template'].includes(type) ? typeId : null, CURRENT_IMPLEMENTATION, interactionCognitive.id, interactionCognitive.nluId, _this.errorData.errorFunction, _this.errorData.errorMessage, _this.errorData.errorDetail));
            }
            return Promise.all(promiseArray);
        }).then(function (result) {
            return Promise.resolve(true);
        }).catch(function (err) {
            var errorMessage = typeof err === 'string' ? err : (err.message || err.description);
            logger.error('[CognitiveAnalysisExecute] executionCode:', executionCode, 'getting queueId:', currentQueueId, 'getting error:', errorMessage);
            return Promise.resolve(false);
        });
    };
    CognitiveAnalysisImplementation.prototype.GetPhrasesFromIdathaJson = function (interaction, currentQueueId, executionCode) {
        var _this = this;
        var existsDocument = false;
        var totalPhrases = 0;
        return (0, promises_shared_1.DummyPromise)()
            .then(function (_) {
            return GetCognitiveService(interaction.vcc, null);
        })
            .then(function (result) {
            cognitiveService = result;
            return _this.GetIdathaJson(interaction, currentQueueId, executionCode);
        })
            .then(function (result) {
            if (!result)
                throw 'ERROR_NOT_FOUND_FILE_JSON';
            if (result === null || result === void 0 ? void 0 : result.existsDocument) {
                existsDocument = result.existsDocument;
                totalPhrases = result.phrases.length;
                delete (result.existsDocument);
                return Promise.resolve(null);
            }
            var transcription = _this.SortJson(result);
            var phrases = _this.SplitTranscriptionContentInPhrases(cognitiveService.GetCognitiveEngineCharsPerPhrase(), transcription);
            logger.info('[CognitiveAnalysisImplementation::GetPhrasesFromIdathaJson] executionCode:', executionCode, 'getting queueId:', currentQueueId, 'getting phrases:', phrases.length);
            return Promise.resolve(phrases);
        }).then(function (result) {
            if (existsDocument)
                return Promise.resolve(null);
            var phrases;
            if (result && result.length) {
                totalPhrases = result.length;
                var regex = /\.(mp3|wav|ogg|json)$/;
                var jsonFileName = interaction.fileName.replace(regex, '.json');
                var path = config_1.COGNITIVE_ANALYSIS_FOLDER;
                path += '/';
                path += interaction.vcc;
                path += '/';
                path += jsonFileName;
                phrases = { phrases: result };
                var stringPhrases = JSON.stringify(phrases);
                return (0, file_manager_1.WriteFileContent)(path, stringPhrases, 'utf-8');
            }
            return Promise.resolve(false);
        }).then(function (result) {
            if (result || existsDocument) {
                interaction['status'] = 6;
                interaction['totalPhrases'] = interaction['totalPhrases'] ? interaction['totalPhrases'] : totalPhrases;
                interaction['statusDescription'] = 'pendingNLU';
                interaction['retry'] = 0;
                interaction['uploadedDate'] = moment().format('x');
                interaction['errorMessage'] = null;
                _this.errorData.CleanErrorData();
            }
            else {
                logger.warn('[CognitiveAnalysisImplementation::GetPhrasesFromIdathaJson] executionCode:', executionCode, 'getting queueId:', currentQueueId, 'getting error:', 'NOT_FOUND_PHASES');
                interaction['status'] = -6;
                interaction['retry'] = interaction['retry'] ? interaction['retry'] + 1 : 1;
                interaction['uploadedDate'] = moment().format('x');
                interaction['errorFunction'] = 'GetPhrasesFromIdathaJson';
                interaction['errorMessage'] = 'NOT_FOUND_PHASES';
                _this.errorData.SetErrorData('GetPhrasesFromIdathaJson', interaction['errorMessage'], interaction['errorMessage']);
            }
            return Promise.resolve(interaction);
        }).catch(function (err) {
            var errorMessage = typeof err === 'string' ? err : (err.message || err.description);
            logger.error('[CognitiveAnalysisImplementation::GetPhrasesFromIdathaJson] executionCode:', executionCode, 'getting queueId:', currentQueueId, 'getting error:', errorMessage);
            interaction['status'] = -6;
            interaction['retry'] = interaction['retry'] ? interaction['retry'] + 1 : 1;
            interaction['uploadedDate'] = moment().format('x');
            interaction['errorMessage'] = errorMessage;
            _this.errorData.SetErrorData('GetPhrasesFromIdathaJson', errorMessage, err);
            return Promise.resolve(interaction);
        });
    };
    CognitiveAnalysisImplementation.prototype.GetIdathaJson = function (interaction, currentQueueId, executionCode) {
        var _this = this;
        var existsDocument = false;
        var regex = /\.(mp3|wav|ogg|json)$/;
        var jsonFileName = interaction.fileName.replace(regex, '.json');
        return (0, promises_shared_1.DummyPromise)()
            .then(function (result) {
            logger.info('[CognitiveAnalysisImplementation::GetIdathaJson] executionCode:', executionCode, 'getting queueId:', currentQueueId, 'searching on cognitive folder.');
            var path = config_1.COGNITIVE_ANALYSIS_FOLDER;
            path += '/';
            path += interaction.vcc;
            path += '/';
            path += jsonFileName;
            return (0, file_manager_1.ReadFileContent)(path, 'utf-8', true);
        })
            .then(function (result) {
            if (!result) {
                logger.info('[CognitiveAnalysisImplementation::GetIdathaJson] executionCode:', executionCode, 'getting queueId:', currentQueueId, 'searching on semantic folder.');
                var path = config_1.SEMANCTIC_ANALYSIS_FOLDER;
                path += '/';
                path += interaction.vcc;
                path += '/';
                path += jsonFileName;
                return (0, file_manager_1.ReadFileContent)(path, 'utf-8', true);
            }
            else
                existsDocument = true;
            return Promise.resolve(result);
        })
            .then(function (result) {
            if (!result) {
                logger.info('[CognitiveAnalysisImplementation::GetIdathaJson] executionCode:', executionCode, 'getting queueId:', currentQueueId, 'searching on idatha server.');
                var path = config_1.SEMANCTIC_ANALYSIS_FOLDER;
                path += '/';
                path += interaction.vcc;
                path += '/';
                path += jsonFileName;
                return (0, audio_service_1.GetIdathaDocument)(interaction.vcc, interaction, path)
                    .then(function (res) {
                    if (res) {
                        var path_1 = config_1.SEMANCTIC_ANALYSIS_FOLDER;
                        path_1 += '/';
                        path_1 += interaction.vcc;
                        path_1 += '/';
                        path_1 += jsonFileName;
                        logger.info('[CognitiveAnalysisImplementation::GetIdathaJson] executionCode:', executionCode, 'getting queueId:', currentQueueId, 'searching json:', path_1);
                        return (0, file_manager_1.ReadFileContent)(path_1, 'utf-8');
                    }
                    else {
                        logger.info('[CognitiveAnalysisImplementation::GetIdathaJson] executionCode:', executionCode, 'getting queueId:', currentQueueId, 'not found on idatha server.');
                        throw 'ERROR_FILE_JSON';
                    }
                });
            }
            return Promise.resolve(result);
        })
            .then(function (result) {
            var jsonContent = null;
            try {
                jsonContent = typeof result === 'string' ? JSON.parse(result) : result;
            }
            catch (ex) {
                jsonContent = null;
                _this.errorData.SetErrorData('GetIdathaJson', 'ERROR_FILE_JSON', ex);
                throw 'ERROR_FILE_JSON';
            }
            jsonContent.existsDocument = existsDocument;
            return Promise.resolve(jsonContent);
        })
            .catch(function (err) {
            var errorMessage = typeof err === 'string' ? err : (err.message || err.description);
            logger.error('[CognitiveAnalysisImplementation::GetIdathaJson] executionCode:', executionCode, 'getting queueId:', currentQueueId, 'getting error:', errorMessage);
            throw err;
        });
    };
    CognitiveAnalysisImplementation.prototype.GetCognitive = function (interaction, currentQueueId, executionCode) {
        var _this = this;
        var item = currentQueueId.split(':');
        var type = item[7] ? item[7] : null;
        var path = '';
        var isLast = false;
        var azure = {};
        var phrase = '';
        var index = 0;
        var currentNlu = {};
        var currentSubscriptionKey = {};
        var apiIdLabel = null;
        return (0, promises_shared_1.DummyPromise)()
            .then(function (_) {
            return GetCognitiveService(interaction.vcc, null);
        })
            .then(function (result) {
            cognitiveService = result;
            apiIdLabel = cognitiveService.GetCognitiveEngineName() === 'LUIS' ? 'LuisApi' : 'RasaApi';
            var regex = /\.(mp3|wav|ogg|json)$/;
            var jsonFileName = interaction.fileName.replace(regex, '.json');
            path = config_1.COGNITIVE_ANALYSIS_FOLDER;
            path += '/';
            path += interaction.vcc;
            path += '/';
            path += jsonFileName;
            return _this.GetNluResults(interaction, path, currentQueueId, executionCode);
        })
            .then(function (result) {
            if (!result || !result.status)
                throw 'ERROR_SAVE_DOCUMENT';
            isLast = result.isLast;
            if (isLast && type !== 'tipification')
                return _this.EvaluateCognitive(path, interaction, executionCode, currentQueueId);
            return Promise.resolve(null);
        })
            .then(function (result) {
            if (isLast) {
                interaction.succedItems = result ? result : [];
                if (!result)
                    logger.warn('[CognitiveAnalysisImplementation::GetCognitive] executionCode:', executionCode, 'getting queueId:', currentQueueId, 'getting error:', 'NOT_FOUND_SCORE');
            }
            return Promise.resolve(true);
        })
            .then(function (result) {
            if (result) {
                interaction['status'] = 6;
                interaction['statusDescription'] = isLast ? 'ready' : 'pendingNLU';
                interaction['totalReadPhrases'] = interaction['totalReadPhrases'] ? interaction['totalReadPhrases'] + 1 : 1;
                interaction['retry'] = 0;
                interaction['uploadedDate'] = moment().format('x');
                interaction['errorMessage'] = null;
                _this.errorData.CleanErrorData();
            }
            return Promise.resolve(interaction);
        })
            .catch(function (err) {
            var errorMessage = typeof err === 'string' ? err : (err.message || err.description);
            if (errorMessage === 'NOT_FOUND_PHRASE') {
                logger.warn('[CognitiveAnalysisImplementation::GetCognitive] executionCode:', executionCode, 'getting queueId:', currentQueueId, 'getting error:', errorMessage);
                interaction['status'] = 6;
                interaction['statusDescription'] = 'ready';
                interaction['totalReadPhrases'] = interaction['totalReadPhrases'] ? interaction['totalReadPhrases'] + 1 : 1;
                interaction['retry'] = 0;
            }
            else if (errorMessage === 'OUT_OF_VOLUME_QUOTA') {
                logger.error('[CognitiveAnalysisImplementation::GetCognitive] executionCode:', executionCode, 'getting queueId:', currentQueueId, 'getting error:', errorMessage);
                interaction['status'] = -6;
                interaction['retry'] = config_1.RETRY_COGNITIVE_ANALYSIS;
            }
            else {
                logger.error('[CognitiveAnalysisImplementation::GetCognitive] executionCode:', executionCode, 'getting queueId:', currentQueueId, 'getting error:', errorMessage);
                interaction['status'] = -6;
                interaction['retry'] = interaction['retry'] ? interaction['retry'] + 1 : 1;
            }
            interaction['uploadedDate'] = moment().format('x');
            interaction['errorMessage'] = errorMessage;
            _this.errorData.SetErrorData('GetCognitive', errorMessage, err);
            return Promise.resolve(interaction);
        });
    };
    CognitiveAnalysisImplementation.prototype.GetNluResults = function (interaction, path, currentQueueId, executionCode) {
        var _this = this;
        var index = 0;
        var isLast = false;
        var alreadyEvaluated = false;
        return (0, promises_shared_1.DummyPromise)()
            .then(function (_) {
            return (0, file_manager_1.ReadFileContent)(path, 'utf-8', true);
        })
            .then(function (result) {
            var jsonContent;
            try {
                jsonContent = typeof result === 'string' ? JSON.parse(result) : result;
            }
            catch (ex) {
                jsonContent = null;
                _this.errorData.SetErrorData('GetNluResultsFromCognitiveEngine', 'ERROR_FILE_JSON', ex);
                throw 'ERROR_FILE_JSON';
            }
            return Promise.resolve(jsonContent);
        })
            .then(function (result) {
            if (!result)
                throw 'ERROR_NOT_FOUND_FILE_JSON';
            if (result.phrases)
                alreadyEvaluated = !result.phrases.map(function (p) { return p[interaction.nluId] ? true : false; }).includes(false) ? true : false;
            index = interaction.totalReadPhrases ? interaction.totalReadPhrases : 0;
            isLast = interaction.totalPhrases - 1 <= index ? true : false;
            return _this.GetNluResultsFromCognitiveEngine(interaction, path, result, currentQueueId, executionCode);
        })
            .then(function (result) {
            return Promise.resolve({ status: result ? true : false, isLast: isLast });
        })
            .catch(function (err) {
            var errorMessage = typeof err === 'string' ? err : (err.message || err.description);
            logger.info("[CognitiveAnalysisImplementation::GetNluResults] executionCode: ".concat(executionCode, " getting queueId: ").concat(currentQueueId, " getting error: ").concat(errorMessage));
            return Promise.reject(err);
        });
    };
    CognitiveAnalysisImplementation.prototype.GetNluResultsFromCognitiveEngine = function (interaction, path, content, currentQueueId, executionCode) {
        var _this = this;
        var phrase = '';
        var index = 0;
        var currentNlu = {};
        var currentSubscriptionKey = {};
        var apiIdLabel = cognitiveService.GetCognitiveEngineName() === 'LUIS' ? 'LuisApi' : 'RasaApi';
        return (0, promises_shared_1.DummyPromise)()
            .then(function (_) {
            var _a;
            index = interaction.totalReadPhrases ? interaction.totalReadPhrases : 0;
            phrase = ((_a = content.phrases[index]) === null || _a === void 0 ? void 0 : _a.content) ? content.phrases[index].content : null;
            if (!phrase)
                throw 'NOT_FOUND_PHRASE';
            return Promise.resolve(true);
        })
            .then(function (result) {
            return _this.GetNlu(interaction.vcc, interaction.nluId, currentQueueId);
        })
            .then(function (result) {
            if (!result)
                throw 'NLU_MODEL_NOT_FOUND';
            currentNlu = result;
            if (apiIdLabel === 'LuisApi') {
                return _this.GetAzureSubscriptionKey(interaction.vcc, currentNlu.subscriptionKey);
            }
            else {
                return Promise.resolve(null);
            }
        })
            .then(function (result) {
            if (apiIdLabel === 'LuisApi') {
                currentSubscriptionKey = result;
                if (!currentSubscriptionKey)
                    throw 'SUSCRIPTION_KEY_NOT_FOUND';
                _this.UpdateCognitiveModel(currentQueueId, apiIdLabel, currentSubscriptionKey.subscriptionKey);
                _this.UpdateCognitiveModel(currentQueueId, 'IsDemo', currentSubscriptionKey.isDemo);
            }
            else {
                currentSubscriptionKey.subscriptionRegion = null;
                currentSubscriptionKey.subscriptionKey = null;
                _this.UpdateCognitiveModel(currentQueueId, apiIdLabel, config_1.RASA_IP + ':' + config_1.RASA_PORT);
                _this.UpdateCognitiveModel(currentQueueId, 'IsDemo', false);
            }
            return _this.ModelQuery(cognitiveService, currentSubscriptionKey.subscriptionRegion, currentSubscriptionKey.subscriptionKey, interaction.appId, phrase, interaction.nluId, executionCode, currentQueueId);
        })
            .then(function (result) {
            if (!result)
                throw 'SERVER_ERROR_COGNITIVE_REQUEST_RESULT_NOT_FOUND';
            if (result && result.error)
                throw result.error;
            return _this.SaveDocumentCognitive(index, path, result, interaction.nluId, executionCode, currentQueueId);
        })
            .catch(function (err) {
            var errorMessage = typeof err === 'string' ? err : (err.message || err.description);
            logger.error('[CognitiveAnalysisImplementation::GetNluResultsFromCognitiveEngine] executionCode:', executionCode, 'getting queueId:', currentQueueId, 'getting error:', errorMessage);
            return Promise.reject(err);
        });
    };
    CognitiveAnalysisImplementation.prototype.GetNlu = function (vcc, name, queue) {
        var _this = this;
        logger.info('[CognitiveAnalysisImplementation::GetNLU] getting vcc:', vcc, 'name:', name);
        var cacheUpdateNeeded = false;
        var arrQueue = queue.split(':');
        var nluRedisKey = vcc + ":" + arrQueue[7] + ":" + arrQueue[8] + ":" + name;
        return redis_service_1.Redis.GetNlu(nluRedisKey)
            .then(function (result) {
            if (result) {
                return Promise.resolve(result);
            }
            else {
                cacheUpdateNeeded = true;
                return mssql_service_1.MSSql.GetNLU(vcc, name);
            }
        })
            .then(function (result) {
            if (result) {
                if (cacheUpdateNeeded)
                    redis_service_1.Redis.SaveNLU(nluRedisKey, result);
                _this.errorData.CleanErrorData();
                return Promise.resolve(result);
            }
            else {
                throw 'ERROR_NLU_MODEL_NOT_FOUND';
            }
        })
            .catch(function (err) {
            var errorMsg = (typeof err === 'string' ? err : err.message || err.description || 'SERVER_ERROR_UNKNOWN_MESSAGE');
            logger.error('[CognitiveAnalysisImplementation::GetNLU] getting errorMsg:', errorMsg);
            _this.errorData.SetErrorData('GetNLU', errorMsg, err);
            return Promise.resolve(null);
        });
    };
    CognitiveAnalysisImplementation.prototype.GetAzureSubscriptionKey = function (vcc, azureSubscriptionKey) {
        var _this = this;
        logger.info('[CognitiveAnalysisImplementation::GetAzureSubscriptionKey] getting vcc: ', vcc, ' azureSubscriptionKey: ', azureSubscriptionKey);
        var cacheUpdateNeeded = false;
        var azureSubscriptionKeyRedisKey = vcc + ":" + azureSubscriptionKey;
        return redis_service_1.Redis.GetSubscriptionKey(azureSubscriptionKeyRedisKey)
            .then(function (result) {
            if (result) {
                return Promise.resolve(result);
            }
            else {
                cacheUpdateNeeded = true;
                return mssql_service_1.MSSql.GetAzureSubscriptionKeyByKey(vcc, azureSubscriptionKey);
            }
        })
            .then(function (result) {
            if (result) {
                if (cacheUpdateNeeded)
                    redis_service_1.Redis.SaveSubscriptionKey(azureSubscriptionKeyRedisKey, result);
                _this.errorData.CleanErrorData();
                return Promise.resolve(result);
            }
            else {
                throw 'ERROR_AZURE_SUBSCRIPTIONKEY_NOT_FOUND';
            }
        })
            .catch(function (err) {
            var errorMsg = (typeof err === 'string' ? err : err.message || err.description || 'SERVER_ERROR_UNKNOWN_MESSAGE');
            logger.error('[CognitiveAnalysisImplementation::GetAzureSubscriptionKey] getting errorMsg:', errorMsg);
            _this.errorData.SetErrorData('GetAzureSubscriptionKey', errorMsg, err);
            return Promise.resolve(null);
        });
    };
    CognitiveAnalysisImplementation.prototype.SortJson = function (jsonobj) {
        var outlength;
        var inlength;
        var i;
        var audioInit;
        var obj = {};
        obj.transcription = [];
        obj.summarySpeechData = {};
        obj.confidence = {};
        obj.init = '';
        var json = jsonobj;
        audioInit = json.words[0].speaker;
        obj.init = audioInit;
        obj.nlp = {};
        outlength = json.out_sentences.length;
        inlength = json.in_sentences.length;
        if (outlength >= inlength) {
            for (i in json.out_sentences) {
                if (i < inlength && inlength > 0) {
                    if (audioInit === 'out' && inlength > 0) {
                        obj.transcription.push({
                            out: json.out_sentences[i].text,
                            in: json.in_sentences[i] === 'undefined' ? '' : json.in_sentences[i].text
                        });
                    }
                    else if (audioInit === 'in' && inlength > 0) {
                        obj.transcription.push({
                            in: json.in_sentences[i].text,
                            out: json.out_sentences[i].text
                        });
                    }
                }
                else {
                    if (audioInit === 'out' && inlength >= 0) {
                        obj.transcription.push({
                            out: json.out_sentences[i] === undefined ? '' : json.out_sentences[i].text,
                            in: ''
                        });
                    }
                    else if (audioInit === 'in' && inlength >= 0) {
                        obj.transcription.push({
                            in: '',
                            out: json.out_sentences[i] === undefined ? '' : json.out_sentences[i].text
                        });
                    }
                }
            }
        }
        else if (outlength <= inlength) {
            for (i in json.in_sentences) {
                if (i < outlength && outlength > 0) {
                    if (audioInit === 'out' && outlength > 0) {
                        obj.transcription.push({
                            out: json.out_sentences[i].text,
                            in: json.in_sentences[i].text
                        });
                    }
                    else if (audioInit === 'in' && outlength > 0) {
                        obj.transcription.push({
                            in: json.in_sentences[i].text,
                            out: json.out_sentences[i].text
                        });
                    }
                }
                else {
                    if (audioInit === 'out' && inlength >= 0) {
                        obj.transcription.push({
                            out: '',
                            in: json.in_sentences[i] === undefined ? '' : json.in_sentences[i].text,
                        });
                    }
                    else if (audioInit === 'in' && inlength >= 0) {
                        obj.transcription.push({
                            in: json.in_sentences[i] === undefined ? '' : json.in_sentences[i].text,
                            out: ''
                        });
                    }
                }
            }
        }
        obj.nlp.in = json.pnl_scores_in ? json.pnl_scores_in : 0;
        obj.nlp.out = json.pnl_scores_out ? json.pnl_scores_out : 0;
        var dataTotalIn = {
            duration: 0,
            silence: 0
        }, dataTotalOut = {
            duration: 0,
            silence: 0
        }, data = json.words, len = data.length, word, inDuration = 0, inSilence = 0, outDuration = 0, outSilence = 0;
        for (i = 0; i < len; i++) {
            word = data[i];
            if (word.speaker === 'in') {
                inDuration += word.duration;
                inSilence += word.silence;
            }
            else {
                outDuration += word.duration;
                outSilence += word.silence;
            }
        }
        dataTotalIn.duration = inDuration.toFixed(2) * 1;
        dataTotalIn.silence = inSilence.toFixed(2) * 1;
        dataTotalOut.duration = outDuration.toFixed(2) * 1;
        dataTotalOut.silence = outSilence.toFixed(2) * 1;
        obj.summarySpeechData = {
            dataTotalIn: dataTotalIn,
            dataTotalOut: dataTotalOut
        };
        obj.duration = json.duration;
        obj.words = json.words;
        obj.topics = json.topics;
        obj.confidence.in_confidence = json.in_confidence ? json.in_confidence : 0;
        obj.confidence.out_confidence = json.out_confidence ? json.out_confidence : 0;
        return obj;
    };
    CognitiveAnalysisImplementation.prototype.SplitTranscriptionContentInPhrases = function (cognitiveEngineCharsPerPhrase, data) {
        var string_500 = [], phrases = '', len = data.transcription.length, item, phrasesResult = '', total = 0;
        string_500.push({ content: '', length: 0 });
        for (var i = 0; i < len; i++) {
            phrases = '';
            item = data.transcription[i];
            if (data.init === 'out') {
                phrases += item.out;
                phrases += ' ';
                phrases += item.in;
            }
            else {
                phrases += item.in;
                phrases += ' ';
                phrases += item.out;
            }
            total = phrasesResult.length + phrases.length;
            if (total > cognitiveEngineCharsPerPhrase) {
                if (phrases.length < cognitiveEngineCharsPerPhrase) {
                    string_500.push({ content: phrases, length: phrases.length });
                    phrasesResult = phrases;
                }
                else {
                    phrasesResult += phrases;
                    var phraseParts = phrasesResult.split('.');
                    phrasesResult = '';
                    phraseParts.map(function (pp) {
                        while (pp.length > 0) {
                            if ((phrasesResult.length + pp.length + 1) <= cognitiveEngineCharsPerPhrase) {
                                phrasesResult += pp + '.';
                                pp = '';
                            }
                            else {
                                var arrPp = pp.split(' ');
                                if ((phrasesResult.length + 1 + arrPp[0].length + 1) > cognitiveEngineCharsPerPhrase) {
                                    string_500.push({ content: phrasesResult, length: phrasesResult.length });
                                    phrasesResult = arrPp.shift();
                                }
                                else {
                                    phrasesResult += ' ' + arrPp.shift();
                                    if (pp.length === 0)
                                        phrasesResult += '.';
                                }
                                pp = arrPp.join(' ');
                            }
                            if (phrasesResult.length <= cognitiveEngineCharsPerPhrase) {
                                string_500[string_500.length - 1].content = phrasesResult;
                                string_500[string_500.length - 1].length = phrasesResult.length;
                                if (phrasesResult.length === cognitiveEngineCharsPerPhrase) {
                                    phrasesResult = '';
                                    string_500.push({ content: phrasesResult, length: phrasesResult.length });
                                }
                            }
                        }
                    });
                }
            }
            else {
                phrasesResult += phrases;
                string_500[string_500.length - 1].content += phrases;
                string_500[string_500.length - 1].length = phrasesResult.length;
            }
            phrases = '';
        }
        return string_500;
    };
    CognitiveAnalysisImplementation.prototype.SaveDocumentCognitive = function (index, path, data, nluId, executionCode, currentQueueId) {
        var _this = this;
        return (0, promises_shared_1.DummyPromise)()
            .then(function () {
            return (0, file_manager_1.ReadFileContent)(path, 'utf-8', true);
        }).then(function (resultFile) {
            var jsonContent;
            var result = data ? data : [];
            logger.debug('[CognitiveAnalysisImplementation::SaveDocumentCognitive] executionCode:', executionCode, 'getting queueId:', currentQueueId, 'getting path: ', path);
            logger.debug('[CognitiveAnalysisImplementation::SaveDocumentCognitive] executionCode:', executionCode, 'getting queueId:', currentQueueId, 'getting index: ', index);
            logger.debug('[CognitiveAnalysisImplementation::SaveDocumentCognitive] executionCode:', executionCode, 'getting queueId:', currentQueueId, 'getting nlu: ', nluId);
            logger.debug('[CognitiveAnalysisImplementation::SaveDocumentCognitive] executionCode:', executionCode, 'getting queueId:', currentQueueId, 'getting result: ', result);
            try {
                if (resultFile) {
                    jsonContent = JSON.parse(resultFile);
                    logger.debug('[CognitiveAnalysisImplementation::SaveDocumentCognitive] executionCode:', executionCode, 'getting queueId:', currentQueueId, 'getting jsonContent: ', jsonContent);
                    logger.debug('[CognitiveAnalysisImplementation::SaveDocumentCognitive] executionCode:', executionCode, 'getting queueId:', currentQueueId, 'getting phrases: ', jsonContent.phrases.length);
                    jsonContent['phrases'][index][nluId] = result;
                }
                else {
                    jsonContent = { phrases: [{}] };
                    logger.debug('[CognitiveAnalysisImplementation::SaveDocumentCognitive] executionCode:', executionCode, 'getting queueId:', currentQueueId, 'creating item on jsonContent');
                    jsonContent['phrases'][index][nluId] = result;
                }
            }
            catch (ex) {
                logger.error('[CognitiveAnalysisImplementation::SaveDocumentCognitive] executionCode:', executionCode, 'getting queueId:', currentQueueId, 'getting error:', ex);
                _this.errorData.SetErrorData('SaveDocumentCognitive', 'ERROR_FILE_JSON', ex);
                throw 'ERROR_FILE_JSON';
            }
            return (0, file_manager_1.WriteFileContent)(path, JSON.stringify(jsonContent), 'utf-8');
        }).then(function (result) {
            _this.errorData.CleanErrorData();
            return Promise.resolve(true);
        }).catch(function (error) {
            var errorMessage = typeof error === 'string' ? error : (error.message || error.description);
            logger.error('[CognitiveAnalysisImplementation::SaveDocumentCognitive] executionCode:', executionCode, 'getting queueId:', currentQueueId, 'getting error:', errorMessage);
            _this.errorData.SetErrorData('SaveDocumentCognitive', errorMessage, error);
            return Promise.resolve(false);
        });
    };
    CognitiveAnalysisImplementation.prototype.ModelQuery = function (cognitiveService, subscriptionRegion, subscriptionKey, appId, query, nluName, executionCode, currentQueueId) {
        var _this = this;
        return cognitiveService.ModelQuery(subscriptionRegion, subscriptionKey, appId, query, null, null, null)
            .then(function (response) {
            logger.info('[CognitiveAnalysisImplementation::ModelQuery] executionCode:', executionCode, 'getting queueId:', currentQueueId, ', Request finished OK');
            logger.info('[CognitiveAnalysisImplementation::ModelQuery] Response: ' + JSON.stringify(response));
            _this.UpdateCognitiveModel(currentQueueId, 'TotalValidRequest', 1);
            _this.errorData.CleanErrorData();
            return Promise.resolve({
                intents: response.intents,
                entities: response.entities ? response.entities : null
            });
        })
            .catch(function (err) {
            var errorMessage = typeof err === 'string' ? err : (err.message || err.description);
            logger.error('[CognitiveAnalysisImplementation::ModelQuery] Error 1: ' + JSON.stringify(err));
            _this.UpdateCognitiveModel(currentQueueId, 'TotalInvalidRequest', 1);
            var responseError = {};
            if (typeof errorMessage === 'string') {
                responseError['error'] = err;
                if (errorMessage.indexOf('Out of call volume quota') > -1)
                    responseError['error'] = 'OUT_OF_VOLUME_QUOTA';
            }
            else
                responseError['error'] = 'SERVER_ERROR_MODEL_QUERY_UNKNOWN_ERROR';
            _this.errorData.SetErrorData('ModelQuery', responseError && responseError['error'] ? responseError['error'] : errorMessage, err);
            return Promise.resolve(responseError);
        })
            .catch(function (err) {
            var errorMessage = typeof err === 'string' ? err : (err.message || err.description);
            logger.error('[CognitiveAnalysisImplementation::ModelQuery] Error 2: ' + JSON.stringify(err));
            _this.UpdateCognitiveModel(currentQueueId, 'TotalInvalidRequest', 1);
            _this.errorData.SetErrorData('ModelQuery', errorMessage, err);
            return Promise.resolve(null);
        });
    };
    CognitiveAnalysisImplementation.prototype.EvaluateCognitive = function (path, interaction, executionCode, currentQueueId) {
        var _this = this;
        var studyTemplateThemes = [];
        var evaluationPlanAbilities = [];
        return (0, promises_shared_1.DummyPromise)()
            .then(function (result) {
            if (['STUDY', 'TEMPLATE'].includes(interaction.batchTypeName))
                return redis_service_1.Redis.GetStudyTemplateThemes(interaction.vcc, interaction.studyId);
            else
                return redis_service_1.Redis.GetEvaluationPlanAbilities(interaction.vcc, interaction.evaluationPlanId);
        })
            .then(function (result) {
            if (['STUDY', 'TEMPLATE'].includes(interaction.batchTypeName))
                studyTemplateThemes = result;
            else
                evaluationPlanAbilities = result;
            return (0, file_manager_1.ReadFileContent)(path, 'utf-8', true);
        })
            .then(function (resultFile) {
            var jsonContent = null;
            try {
                if (resultFile) {
                    jsonContent = JSON.parse(resultFile);
                }
            }
            catch (ex) {
                logger.error('[CognitiveAnalysisImplementation::EvaluateCognitive] executionCode:', executionCode, 'getting queueId:', currentQueueId, 'getting error:', ex);
                _this.errorData.SetErrorData('EvaluateCognitive', 'ERROR_FILE_JSON', ex);
                throw 'ERROR_FILE_JSON';
            }
            return jsonContent;
        })
            .then(function (jsonContent) {
            if (!jsonContent)
                throw 'ERROR_FILE_JSON';
            var phrases = jsonContent['phrases'];
            var succedItems = [];
            if (phrases && phrases.length) {
                phrases.forEach(function (phrase) {
                    if (phrase[interaction.nluId] && phrase[interaction.nluId].intents && phrase[interaction.nluId].intents.length) {
                        if (['STUDY', 'TEMPLATE'].includes(interaction.batchTypeName)) {
                            if (!studyTemplateThemes.length)
                                logger.warn('[CognitiveAnalysisImplementation::EvaluateCognitive] executionCode:', executionCode, 'getting queueId:', currentQueueId, 'studyTemplateThemes is empty, need items to get succedItems');
                            var templateThemes = interaction.templateThemes.split(',');
                            var succedTemplateThemeItems_1 = [];
                            templateThemes.map(function (tt) {
                                var currentTemplateTheme = studyTemplateThemes.find(function (stt) { return stt.id === tt; });
                                if (currentTemplateTheme) {
                                    var succedIntents_1 = [];
                                    currentTemplateTheme.items.map(function (tti) {
                                        var intent = phrase[interaction.nluId].intents.find(function (i) { return i.intent === tti.id.split('|')[1]; });
                                        if (!intent)
                                            intent = phrase[interaction.nluId].intents.find(function (i) { return i.intent === tti.query.split('|')[1]; });
                                        if (intent) {
                                            if (intent.score >= tti.acceptancePercentage) {
                                                if (tti.entities && tti.entities.length > 0) {
                                                    var succeedEntities_1 = [];
                                                    tti.entities.map(function (e) {
                                                        var entities = phrase[interaction.nluId].entities.filter(function (en) { return en.type === e.entityName && en.score >= e.acceptancePercentage; });
                                                        if (entities && entities.length) {
                                                            succeedEntities_1.push(entities[0]);
                                                        }
                                                    });
                                                    if (succeedEntities_1.length > 0) {
                                                        intent.entities = succeedEntities_1;
                                                        succedIntents_1.push(intent);
                                                        succedTemplateThemeItems_1.push(tti.id);
                                                    }
                                                }
                                                else {
                                                    intent.entities = [];
                                                    succedIntents_1.push(intent);
                                                    succedTemplateThemeItems_1.push(tti.id);
                                                }
                                            }
                                        }
                                    });
                                    logger.info('[CognitiveAnalysisImplementation::EvaluateCognitive] executionCode:', executionCode, 'getting queueId:', currentQueueId, 'succedIntents result:', succedIntents_1);
                                    if (succedIntents_1.length > 0) {
                                        if (succedItems.find(function (stt) { return stt.id === tt; })) {
                                            var succeedTemplateTheme = succedItems.find(function (stt) { return stt.id === tt; });
                                            succeedTemplateTheme.intents.map(function (i) {
                                                var succeedIntent = succedIntents_1.find(function (si) { return si.intent === i.id.split('|')[1]; });
                                                if (succeedIntent.score > i.score) {
                                                    i = succeedIntent;
                                                }
                                            });
                                        }
                                        else {
                                            succedItems.push({
                                                templateThemeId: tt,
                                                templateThemeItems: succedTemplateThemeItems_1,
                                                intents: succedIntents_1
                                            });
                                        }
                                    }
                                }
                            });
                        }
                        else {
                            if (!evaluationPlanAbilities.length)
                                logger.warn('[CognitiveAnalysisImplementation::EvaluateCognitive] executionCode:', executionCode, 'getting queueId:', currentQueueId, 'evaluationPlanAbilities is empty, need items to get succedItems');
                            var abilities = interaction.abilities.split(',');
                            abilities.map(function (a) {
                                var currentAbility = evaluationPlanAbilities.find(function (epa) { return epa.idAbility === a; });
                                if (currentAbility) {
                                    var succedIntents_2 = [];
                                    currentAbility.queries.map(function (q) {
                                        var intent = phrase[interaction.nluId].intents.find(function (i) { return i.intent === q.queryContent.split('|')[1]; });
                                        if (!intent)
                                            intent = phrase[interaction.nluId].intents.find(function (i) { return i.intent === q.queryId; });
                                        if (intent) {
                                            if (intent.score >= q.acceptancePercentage) {
                                                if (q.entities && q.entities.length > 0) {
                                                    var succeedEntities_2 = [];
                                                    q.entities.map(function (e) {
                                                        var entities = phrase[interaction.nluId].entities.filter(function (en) { return en.type === e.entityName && en.score >= e.acceptancePercentage; });
                                                        if (entities && entities.length) {
                                                            succeedEntities_2.push(entities[0]);
                                                        }
                                                    });
                                                    if (succeedEntities_2.length > 0) {
                                                        intent.entities = succeedEntities_2;
                                                        succedIntents_2.push(intent);
                                                    }
                                                }
                                                else {
                                                    intent.entities = [];
                                                    succedIntents_2.push(intent);
                                                }
                                            }
                                        }
                                    });
                                    logger.info('[CognitiveAnalysisImplementation::EvaluateCognitive] executionCode:', executionCode, 'getting queueId:', currentQueueId, 'succedIntents result:', succedIntents_2);
                                    if (succedIntents_2.length > 0) {
                                        if (succedItems.find(function (sa) { return sa.abilityId === a; })) {
                                            var succeedAbility = succedItems.find(function (sa) { return sa.abilityId === a; });
                                            succeedAbility.intents.map(function (i) {
                                                var succeedIntent = succedIntents_2.find(function (si) { return si.intent === i.intent; });
                                                if (succeedIntent.score > i.score) {
                                                    i = succeedIntent;
                                                }
                                            });
                                        }
                                        else {
                                            succedItems.push({
                                                abilityId: a,
                                                intents: succedIntents_2
                                            });
                                        }
                                    }
                                    else {
                                    }
                                }
                            });
                        }
                    }
                });
            }
            logger.info('[CognitiveAnalysisImplementation::EvaluateCognitive] executionCode:', executionCode, 'getting queueId:', currentQueueId, 'succedItems result:', succedItems);
            _this.errorData.CleanErrorData();
            return Promise.resolve(succedItems);
        })
            .catch(function (error) {
            var errorMessage = typeof error === 'string' ? error : (error.message || error.description);
            logger.error('[CognitiveAnalysisImplementation::EvaluateCognitive] executionCode:', executionCode, 'getting queueId:', currentQueueId, 'getting error:', errorMessage);
            _this.errorData.SetErrorData('EvaluateCognitive', errorMessage, error);
            return Promise.resolve(null);
        });
    };
    CognitiveAnalysisImplementation.prototype.UpdateCognitiveModel = function (queueId, target, value) {
        logger.info('[CognitiveAnalysisImplementation::UpdateCognitiveModel] Target::Value ' + target + '::' + value);
        var billing = new billing_implementation_1.BillingImplementation();
        var myBatchName = '';
        var myQueueIdArray = queueId.split(':');
        switch (myQueueIdArray[7]) {
            case 'study':
                myBatchName = 'STUDY';
                break;
            case 'template':
                myBatchName = 'TEMPLATE';
                break;
            case 'ability':
                myBatchName = 'EVALUATION_PLAN';
                break;
            case 'tipification':
                myBatchName = 'EVALUATION_PLAN';
                break;
        }
        billing.AddUpdateCognitiveAnalysisDetailToMainFile(myQueueIdArray[0], myQueueIdArray[1], myQueueIdArray[6], target, value, myBatchName, myQueueIdArray[8]);
    };
    CognitiveAnalysisImplementation.prototype.CloseCognitiveModel = function (queueId, resultCode, resultDescription) {
        var billing = new billing_implementation_1.BillingImplementation();
        var myBatchName = '';
        var myQueueIdArray = queueId.split(':');
        switch (myQueueIdArray[7]) {
            case 'study':
                myBatchName = 'STUDY';
                break;
            case 'template':
                myBatchName = 'TEMPLATE';
                break;
            case 'ability':
                myBatchName = 'EVALUATION_PLAN';
                break;
            case 'tipification':
                myBatchName = 'EVALUATION_PLAN';
                break;
        }
        billing.AddCloseCognitiveAnalysisDetailToMainFile(myQueueIdArray[0], myQueueIdArray[1], myQueueIdArray[6], myBatchName, myQueueIdArray[8], resultCode ? 1 : 0, resultDescription);
    };
    return CognitiveAnalysisImplementation;
}());
exports.CognitiveAnalysisImplementation = CognitiveAnalysisImplementation;
function GetCognitiveService(vcc, cognitiveEngine) {
    var cognitiveService;
    vcc = vcc ? vcc : null;
    cognitiveEngine = cognitiveEngine ? cognitiveEngine : null;
    return (0, promises_shared_1.DummyPromise)()
        .then(function (result) {
        if (vcc || cognitiveEngine) {
            if (vcc)
                return redis_service_1.Redis.GetVCC(vcc, vcc);
            if (cognitiveEngine)
                return Promise.resolve(true);
            throw 'SERVER_ERROR_MISSING_PARAMETERS';
        }
        else
            throw 'SERVER_ERROR_MISSING_PARAMETERS';
    }).then(function (result) {
        if (vcc) {
            cognitiveEngine = result && result.cognitiveEngine ? result.cognitiveEngine : null;
        }
        switch (cognitiveEngine) {
            case 'LUIS':
                cognitiveService = new luis_service_implementation_1.LUISServiceImplementation();
                break;
            case 'RASA':
                if (useRasaApi)
                    cognitiveService = new rasa_api_service_implementation_1.RASAAPIServiceImplementation();
                else
                    cognitiveService = new rasa_service_implementation_1.RASAServiceImplementation();
                break;
            case 'OLD_RASA_ENGINE':
                if (useRasaApi)
                    cognitiveService = new rasa_service_implementation_1.RASAServiceImplementation();
                else
                    cognitiveService = new rasa_api_service_implementation_1.RASAAPIServiceImplementation();
                break;
            case 'NEW_RASA_ENGINE':
                if (useRasaApi)
                    cognitiveService = new rasa_api_service_implementation_1.RASAAPIServiceImplementation();
                else
                    cognitiveService = new rasa_service_implementation_1.RASAServiceImplementation();
                break;
            default:
                cognitiveService = null;
                logger.warn('[cognitive-analysis.impl::GetCognitiveService] Undefined engine: ' + cognitiveEngine);
                break;
        }
        return Promise.resolve(cognitiveService);
    }).catch(function (error) {
        var errorMessage = typeof error === 'string' ? error : 'ERROR';
        logger.error('[cognitive-analysis.impl::GetCognitiveService] Error: ' + error);
        return Promise.reject(errorMessage);
    });
}
exports.GetCognitiveService = GetCognitiveService;
