"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.CognitiveEngineMigrateImplementation = void 0;
var redis_service_1 = require("../db/redis.service");
var log4js = require("log4js");
var mssql_service_1 = require("../db/mssql.service");
var promises_shared_1 = require("../shared/promises.shared");
var nlu_1 = require("./cognitive-engine-migration/nlu");
var entity_1 = require("./cognitive-engine-migration/entity");
var enum_1 = require("./cognitive-engine-migration/enum");
var intent_1 = require("./cognitive-engine-migration/intent");
var utterance_1 = require("./cognitive-engine-migration/utterance");
var logger = log4js.getLogger('Task');
var CognitiveEngineMigrateImplementation = (function () {
    function CognitiveEngineMigrateImplementation() {
    }
    CognitiveEngineMigrateImplementation.prototype.Execute = function (executionCode) {
        var _this = this;
        logger.info('[CognitiveEngineMigrateImplementation::Execute] executionCode:', executionCode);
        return (0, promises_shared_1.SleepPromise)(5).then(function (_) {
            return redis_service_1.Redis.GetCognitiveEngineMigrationData();
        })
            .then(function (results) {
            if (typeof results != "object" || !results) {
                logger.debug("[CognitiveEngineMigrateImplementation::Execute] NOT_FOUND_DATA ", results);
                return [false];
            }
            logger.info('[CognitiveEngineMigrateImplementation::Execute] FOUND_DATA_TO_MIGRATE: ', results);
            var promises = [];
            for (var vcc in results) {
                var data = JSON.parse(results[vcc]);
                switch (data.state) {
                    case 0:
                        promises.push(_this.StartMigration(vcc, data));
                        break;
                    case 1:
                        var migrator = new nlu_1.NluMigrator();
                        promises.push(migrator.MigrateNlus(vcc, data, 0, enum_1.ITEMS_PER_PAGE));
                        break;
                    default: break;
                }
            }
            return Promise.all(promises);
        })
            .then(function (results) {
            logger.info('[CognitiveEngineMigrateImplementation::Execute] Before task promises results ', results);
            return redis_service_1.Redis.GetCognitiveEngineMigrationData();
        })
            .then(function (tasks) {
            if (typeof tasks != "object" || !tasks) {
                return [false];
            }
            var taskPromises = [];
            for (var vcc in tasks) {
                var data = JSON.parse(tasks[vcc]);
                taskPromises.push(_this.CheckStatus(vcc, data));
            }
            return Promise.all(taskPromises);
        })
            .then(function (results) {
            var hasSuccess = !results.includes(false);
            logger.info('[CognitiveEngineMigrateImplementation::Execute] Final result ', hasSuccess);
            return hasSuccess;
        })
            .catch(function (err) {
            logger.error('[CognitiveEngineMigrateImplementation::Execute] Error ocurred on migrate', err);
            return false;
        });
    };
    CognitiveEngineMigrateImplementation.prototype.CheckStatus = function (vcc, data) {
        var promises = [];
        var totalMigration = false;
        logger.info("[CognitiveEngineMigrateImplementation::CheckStatus] Checking status migration for ", vcc, data);
        var hasCreatedNlu = (data.totalCreatedNlu + data.totalErrorNlu) >= data.totalNlu;
        if (hasCreatedNlu) {
            promises.push(mssql_service_1.MSSql.UpdateVccStatus(vcc, 0));
        }
        if (data.totalNlu * 2 == data.totalDeletedNlu + data.totalCreatedNlu) {
            logger.info("[CognitiveEngineMigrateImplementation::CheckStatus] Deleting keys for nlu");
            promises.push(redis_service_1.Redis.DeleteCognitiveEngineMigrationKey("".concat(enum_1.ROOT_KEY, ":").concat(vcc, ":Nlus")));
            promises.push(redis_service_1.Redis.DeleteCognitiveEngineMigrationKey("".concat(enum_1.ROOT_KEY, ":").concat(vcc, ":Nlus:Keys")));
            if (data.newEngine === 'NEW_RASA_ENGINE')
                totalMigration = true;
        }
        if (data.totalIntent > 0 && hasCreatedNlu) {
            var im = new intent_1.IntentMigrator();
            promises.push(im.MigrateIntents(vcc, data, 0, enum_1.ITEMS_PER_PAGE));
            if (data.totalIntent == data.totalCreatedIntent) {
                logger.info("[CognitiveEngineMigrateImplementation::CheckStatus] Deleting keys for intent");
                promises.push(redis_service_1.Redis.DeleteCognitiveEngineMigrationKey("".concat(enum_1.ROOT_KEY, ":").concat(vcc, ":Intents")));
                promises.push(redis_service_1.Redis.DeleteCognitiveEngineMigrationKey("".concat(enum_1.ROOT_KEY, ":").concat(vcc, ":Intents:Keys")));
            }
            if (data.totalUtterance > 0 && (data.totalCreatedIntent + data.totalErrorIntent) == data.totalIntent) {
                var um = new utterance_1.UtteranceMigrator();
                promises.push(um.MigrateUtterances(vcc, data, 0, enum_1.ITEMS_PER_PAGE));
                if (data.totalUtterance == data.totalCreatedUtterance) {
                    logger.info("[CognitiveEngineMigrateImplementation::CheckStatus] Deleting keys for utterance");
                    promises.push(redis_service_1.Redis.DeleteCognitiveEngineMigrationKey("".concat(enum_1.ROOT_KEY, ":").concat(vcc, ":Utterances")));
                    promises.push(redis_service_1.Redis.DeleteCognitiveEngineMigrationKey("".concat(enum_1.ROOT_KEY, ":").concat(vcc, ":Utterances:Keys")));
                }
            }
        }
        if (data.totalEntity > 0 && hasCreatedNlu) {
            var em = new entity_1.EntityMigrator();
            promises.push(em.MigrateEntities(vcc, data, 0, enum_1.ITEMS_PER_PAGE));
            if (data.totalEntity == data.totalCreatedEntity) {
                logger.info("[CognitiveEngineMigrateImplementation::CheckStatus] Deleting keys for entity");
                promises.push(redis_service_1.Redis.DeleteCognitiveEngineMigrationKey("".concat(enum_1.ROOT_KEY, ":").concat(vcc, ":Entities")));
                promises.push(redis_service_1.Redis.DeleteCognitiveEngineMigrationKey("".concat(enum_1.ROOT_KEY, ":").concat(vcc, ":Entities:Keys")));
            }
        }
        var finished = false;
        if ((data.totalNlu * 2 + data.totalUtterance + data.totalEntity + data.totalIntent) ==
            (data.totalDeletedNlu + data.totalCreatedNlu +
                data.totalCreatedUtterance +
                data.totalCreatedIntent +
                data.totalCreatedEntity)) {
            finished = true;
            logger.info("[CognitiveEngineMigrateImplementation::CheckStatus] Deleting all keys for ", vcc);
            promises.push(this.DeleteMigrationTask(vcc));
            promises.push(redis_service_1.Redis.DeleteCognitiveEngineMigrationKey("".concat(enum_1.ROOT_KEY, ":").concat(vcc, ":AppIds")));
            promises.push(redis_service_1.Redis.DeleteCognitiveEngineMigrationKey("".concat(enum_1.ROOT_KEY, ":").concat(vcc, ":IntentIds")));
        }
        promises.push(redis_service_1.Redis.UpdateCognitiveEngineMigrationData(vcc, Object.assign(data, {
            counter: data.counter + 1
        })));
        if (finished || data.counter > 10) {
            if (totalMigration)
                promises.push(mssql_service_1.MSSql.SaveCognitiveEngineMigrateStatus(vcc, 'nlu', 'AVAILABLE_TM', 1, enum_1.AVAILABLE));
            promises.push(mssql_service_1.MSSql.SaveCognitiveEngineMigrateStatus(vcc, 'nlu', enum_1.AVAILABLE, 1, enum_1.AVAILABLE));
        }
        return Promise.all(promises).then(function (results) {
            return !results.includes(false);
        });
    };
    CognitiveEngineMigrateImplementation.prototype.DeleteMigrationTask = function (vcc) {
        logger.info('[CognitiveEngineMigrateImplementation::DeleteMigrationTask] Deleting task for ', vcc);
        return redis_service_1.Redis.DeleteCognitiveEngineMigrationTask(enum_1.CONFIG_KEY, vcc).catch(function (err) {
            logger.error('[CognitiveEngineMigrateImplementation::DeleteMigrationTask] Error on delete task for redis:', vcc, err);
            return Promise.resolve(false);
        });
    };
    CognitiveEngineMigrateImplementation.prototype.StartMigration = function (vcc, data) {
        var _this = this;
        var listMode = 1;
        var totalMigration = false;
        if (data.oldEngine === 'OLD_RASA_ENGINE') {
            listMode = 3;
            totalMigration = true;
        }
        var nlusToMigrate = '';
        return mssql_service_1.MSSql.GetNluIntentUtteranceToExport(vcc, listMode).then(function (result) {
            var promises = [];
            if (result.nlus.length > 0) {
                nlusToMigrate = result.nlus.map(function (nlu) { return nlu.appId; }).join('\',\'');
                promises.push(_this.SaveDataToRedis(vcc, result.nlus, 'Nlus', 'appId'));
            }
            if (result.intents.length > 0) {
                promises.push(_this.SaveDataToRedis(vcc, result.intents, 'Intents', 'key'));
                if (result.utterances.length > 0) {
                    promises.push(_this.SaveDataToRedis(vcc, result.utterances, 'Utterances', 'key'));
                }
            }
            if (result.entities.length > 0) {
                promises.push(_this.SaveDataToRedis(vcc, result.entities, 'Entities', 'key'));
            }
            data.totalNlu = result.nlus.length;
            data.totalIntent = result.intents.length;
            data.totalUtterance = result.utterances.length;
            data.totalEntity = result.entities.length;
            return Promise.all(promises);
        }).then(function (results) {
            var errors = results.filter(function (r) { return r.includes("ERROR"); });
            if (errors.length > 0) {
                logger.error('[CognitiveEngineMigrateImplementation::StartMigration] AFTER SAVE DATA TO REDIS:', errors);
                return Promise.all([
                    false,
                    mssql_service_1.MSSql.SaveCognitiveEngineMigrateStatus(vcc, '*', '*', 3, enum_1.CANNOT_SAVE_DATA_TO_REDIS)
                ]);
            }
            return Promise.all([
                true,
                redis_service_1.Redis.UpdateCognitiveEngineMigrationData(vcc, Object.assign(data, { state: 1 })),
                totalMigration ? mssql_service_1.MSSql.SaveCognitiveEngineMigrateStatus(vcc, 'nlu', 'TOTAL_MIGRATION', 2, enum_1.MIGRATING, nlusToMigrate) : mssql_service_1.MSSql.SaveCognitiveEngineMigrateStatus(vcc, '*', '*', 2, enum_1.MIGRATING)
            ]);
        }).then(function (status) {
            return status[0];
        }).catch(function (err) {
            logger.error('[CognitiveEngineMigrateImplementation::StartMigration] Error ocurred on start migration:', err);
            return mssql_service_1.MSSql.SaveCognitiveEngineMigrateStatus(vcc, '*', '*', 3, enum_1.WITH_ERROR);
        });
    };
    CognitiveEngineMigrateImplementation.prototype.SaveDataToRedis = function (vcc, rows, key, id) {
        return redis_service_1.Redis.SaveCognitiveEngineMigrationModel("".concat(enum_1.ROOT_KEY, ":").concat(vcc, ":").concat(key), id, rows).then(function (saved) {
            return saved ? "SUCCESS_SAVE_".concat(key, "_TO_REDIS") : "ERROR_ON_SAVE_".concat(key, "_TO_REDIS");
        });
    };
    return CognitiveEngineMigrateImplementation;
}());
exports.CognitiveEngineMigrateImplementation = CognitiveEngineMigrateImplementation;
