"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.RASAAPIServiceImplementation = void 0;
var log4js = require("log4js");
var config_1 = require("../config/config");
var promises_shared_1 = require("../shared/promises.shared");
var request_shared_1 = require("../shared/request.shared");
var utilities_1 = require("../shared/utilities");
var mssql_service_1 = require("../db/mssql.service");
var utterance_service_1 = require("../services/utterance.service");
var platform_service_1 = require("../services/platform.service");
var logger = log4js.getLogger('Task');
var RASAAPIServiceImplementation = (function () {
    function RASAAPIServiceImplementation() {
    }
    RASAAPIServiceImplementation.prototype.GetCognitiveEngineName = function () {
        return 'RASAAPI';
    };
    RASAAPIServiceImplementation.prototype.GetCognitiveEngineCharsPerPhrase = function () {
        return config_1.COGNITIVE_SERVICES_RASA_API_CHARS_PER_PHRASE;
    };
    RASAAPIServiceImplementation.prototype.LoadAzureAccounts = function (subscriptionRegion, subscriptionKey, data) {
        return null;
    };
    RASAAPIServiceImplementation.prototype.ModelQuery = function (subscriptionRegion, subscriptionKey, appId, query, processAudios, nluName, resolve, task) {
        var _this = this;
        var apiKey = config_1.RASA_API_KEY;
        var appIdRasaApi = appId;
        var uri = config_1.RASA_API_URL.replace('/api', '');
        var data = {
            "message": query
        };
        var postOptions = {
            uri: "https://".concat(uri, "/query/").concat(appIdRasaApi),
            body: data,
            headers: {
                'apiKey': apiKey
            },
            retryProtocol: false,
            timeout: 60000
        };
        logger.info('[RASAAPIServiceImplementation::ModelQuery] getting postOptions:', postOptions, 'appId:', appId, 'query:', query, 'nluName:', nluName);
        return (0, request_shared_1.SimplePostRequest)(postOptions)
            .then(function (response) {
            logger.info('[RASAAPIServiceImplementation::ModelQuery] getting postOptions:', postOptions, 'appId:', appId, 'query:', query, 'result:', response);
            return Promise.resolve(response);
        })
            .then(function (result) {
            logger.info('[RASAAPIServiceImplementation::ModelQuery] getting postOptions:', postOptions, 'appId:', appId, 'query:', query, 'result:', result);
            return Promise.resolve(_this.FormatModelQueryResponse(result));
        })
            .catch(function (err) {
            var errorMessage = typeof err === 'string' ? err : (err.message || err.description);
            logger.error('[RASAAPIServiceImplementation::ModelQuery] getting error:', errorMessage, 'appId:', appId, 'query:', query, 'nluName:', nluName);
            return Promise.reject(err);
        });
    };
    RASAAPIServiceImplementation.prototype.FormatModelQueryResponse = function (originalResponse) {
        var formatedData;
        var originalIntents = originalResponse.intent_ranking;
        var originalEntities = originalResponse.entities;
        var originalQuery = originalResponse.text;
        var formatedIntents = [];
        var formatedEntities = [];
        var formatedQuery;
        formatedIntents = originalIntents.map(function (oi) {
            return {
                intent: oi.name,
                score: oi.confidence
            };
        });
        formatedEntities = originalEntities;
        formatedQuery = originalQuery;
        formatedData = {
            intents: formatedIntents,
            entities: formatedEntities,
            query: formatedQuery
        };
        return formatedData;
    };
    RASAAPIServiceImplementation.prototype.DeleteAppIntent = function (subscriptionRegion, subscriptionKey, appId, intentId, vcc) {
        return (0, promises_shared_1.DummyPromise)()
            .then(function (result) {
            return (0, platform_service_1.GetPlatformVersion)();
        }).then(function (result) {
            logger.info('[RASAServiceImplementation::DeleteAppIntent] Getting result:', result, 'after GetVersion');
            return mssql_service_1.MSSql.DeleteIntent(vcc, appId, intentId);
        })
            .then(function (result) {
            return (0, utterance_service_1.GetUtterances)(vcc, appId, null, null, null, true);
        }).catch(function (error) {
            var errorMessage = typeof error === 'string' ? error : 'ERROR';
            logger.error('[RASAAPIServiceImplementation::DeleteAppIntent] Error: ', error);
            if (error.code && error.code === 'ETIMEDOUT')
                errorMessage = 'SERVER_ERROR_TIMEOUT_FROM_RASA_SERVER';
            if (error.code && error.code === 'ECONNREFUSED')
                errorMessage = 'SERVER_ERROR_CONNECTION_REFUSED_FROM_RASA_SERVER';
            return Promise.reject(errorMessage);
        });
    };
    RASAAPIServiceImplementation.prototype.DeleteAppUtterance = function (subscriptionRegion, subscriptionKey, appId, intentId, utteranceId, vcc) {
        return (0, promises_shared_1.DummyPromise)()
            .then(function (result) {
            return (0, platform_service_1.GetPlatformVersion)();
        }).then(function (result) {
            logger.info('[RASAServiceImplementation::AddIntent] Getting result:', result, 'after GetVersion');
            return mssql_service_1.MSSql.DeleteUtterance(vcc, appId, intentId, utteranceId);
        })
            .then(function (result) {
            return (0, utterance_service_1.GetUtterances)(vcc, appId, null, null, null, true);
        }).catch(function (error) {
            var errorMessage = typeof error === 'string' ? error : 'ERROR';
            logger.error('[RASAServiceImplementation::DeleteUtterance] Error: ', error);
            if (error.code && error.code === 'ETIMEDOUT')
                errorMessage = 'SERVER_ERROR_TIMEOUT_FROM_RASA_SERVER';
            if (error.code && error.code === 'ECONNREFUSED')
                errorMessage = 'SERVER_ERROR_CONNECTION_REFUSED_FROM_RASA_SERVER';
            return Promise.reject(errorMessage);
        });
    };
    RASAAPIServiceImplementation.prototype.TrainApp = function (subscriptionRegion, subscriptionKey, appId, vcc) {
        var _this = this;
        var appIdRasaApi = appId;
        var nluObjectVersion = {
            name: "debug_speechanalytics",
            active: true,
            nlu: {
                intents: []
            }
        };
        return (0, promises_shared_1.DummyPromise)()
            .then(function (result) {
            return _this.GenerateNluObjectVersion(appId, vcc);
        }).then(function (result) {
            nluObjectVersion.nlu.intents = result;
            if (!result || (result === null || result === void 0 ? void 0 : result.length) === 0)
                throw 'SERVER_ERROR_GENERATE_NLU_OBJECT_VERSION_RASAAPI_FAILED';
            var data = nluObjectVersion;
            var apiKey = config_1.RASA_API_KEY;
            var postOptions = {
                uri: "https://".concat(config_1.RASA_API_URL, "/application/").concat(appIdRasaApi, "/version"),
                body: data,
                headers: {
                    'apiKey': apiKey,
                },
                retryProtocol: false,
                timeout: 60000
            };
            logger.info('[RASAAPIServiceImplementation::TrainApp] getting postOptions:', postOptions, 'subscriptionRegion:', subscriptionRegion, 'subscriptionKey:', subscriptionKey, 'data:', data);
            return (0, request_shared_1.SimplePostRequest)(postOptions)
                .then(function (result) {
                logger.info('[RASAAPIServiceImplementation::TrainApp] getting response successfully response:', result);
                return _this.TrainWithVersion(result.id);
            });
        }).then(function (result) {
            return Promise.resolve(result);
        }).catch(function (error) {
            var errorMessage = typeof error === 'string' ? error : 'ERROR';
            logger.error('[RASAServiceImplementation::TrainApp] Error: ', error);
            if (error === 'SERVER_ERROR_MISSING_UTTERANCES_TO_TRAIN')
                return Promise.resolve({ status: 'Queued', statusId: 9 });
            if (error.code && error.code === 'ETIMEDOUT')
                errorMessage = 'SERVER_ERROR_TIMEOUT_FROM_RASA_SERVER';
            if (error.code && error.code === 'ECONNREFUSED')
                errorMessage = 'SERVER_ERROR_CONNECTION_REFUSED_FROM_RASA_SERVER';
            return Promise.reject(errorMessage);
        });
    };
    RASAAPIServiceImplementation.prototype.GetTrainApp = function (subscriptionRegion, subscriptionKey, appId, trainId) {
        var getOptions = {
            uri: "https://".concat(config_1.RASA_API_URL, "/train/").concat(trainId),
            headers: {
                'apiKey': config_1.RASA_API_KEY,
            },
            retryProtocol: false,
            timeout: 60000
        };
        logger.info('[RASAAPIServiceImplementation::GetTrainApp] getting getOptions:', getOptions, 'trainId:', trainId);
        return (0, request_shared_1.SimpleGetRequest)(getOptions)
            .then(function (response) {
            logger.info('[RASAAPIServiceImplementation::GetTrainApp] getting response:', getOptions, 'result:', response);
            return Promise.resolve(response);
        }).catch(function (err) {
            var errorMessage = typeof err === 'string' ? err : (err.message || err.description);
            logger.error('[RASAAPIServiceImplementation::GetTrainApp] getting error:', errorMessage, 'trainId:', trainId);
            return Promise.reject(err);
        });
    };
    RASAAPIServiceImplementation.prototype.PublishApp = function (subscriptionRegion, subscriptionKey, appId, isStaging, trainVersion) {
        var apiKey = config_1.RASA_API_KEY;
        var appIdRasaApi = appId;
        var data = {
            "active": true
        };
        var putOptions = {
            uri: "https://".concat(config_1.RASA_API_URL, "/application/").concat(appIdRasaApi, "/version/").concat(trainVersion),
            body: data,
            headers: {
                'apiKey': apiKey
            },
            retryProtocol: false,
            timeout: 60000
        };
        logger.info('[RASAAPIServiceImplementation::PublishApp] getting postOptions:', putOptions, 'apikey:', apiKey, 'appId:', appId);
        return (0, request_shared_1.SimplePutRequest)(putOptions)
            .then(function (response) {
            logger.info('[RASAAPIServiceImplementation::PublishApp] getting response successfully apikey:', apiKey, 'appId:', appId, 'response', response);
            return Promise.resolve(response);
        })
            .catch(function (err) {
            var errorMessage = typeof err === 'string' ? err : (err.message || err.description);
            logger.error('[RASAAPIServiceImplementation::PublishApp] getting error:', errorMessage, 'apikey:', apiKey, 'appId:', appId);
            return Promise.reject(err);
        });
    };
    RASAAPIServiceImplementation.prototype.GetManagerToken = function (info) {
        return null;
    };
    RASAAPIServiceImplementation.prototype.UnAssignSubscriptionKeyOfVCC = function (subscriptionRegion, subscriptionKey, appId, data) {
        return null;
    };
    RASAAPIServiceImplementation.prototype.AssignSubscriptionKeyOfVCC = function (subscriptionRegion, subscriptionKey, appId, data) {
        return null;
    };
    RASAAPIServiceImplementation.prototype.GetCultures = function (subscriptionRegion, subscriptionKey) {
        var cultures = [];
        cultures.push({ name: "Chinese", code: "zh" });
        cultures.push({ name: "Dutch", code: "nl" });
        cultures.push({ name: "English", code: "en" });
        cultures.push({ name: "French", code: "fr" });
        cultures.push({ name: "German", code: "de" });
        cultures.push({ name: "Italian", code: "it" });
        cultures.push({ name: "Portuguese", code: "pt" });
        cultures.push({ name: "Spanish", code: "es" });
        return Promise.resolve(cultures.filter(function (c) { return config_1.COGNITIVE_SERVICES_RASA_API_AVAILABLE_LANGUAGES.includes(c.code); }));
    };
    RASAAPIServiceImplementation.prototype.AddApp = function (subscriptionRegion, subscriptionKey, nlu) {
        var data = { name: nlu.name, language: nlu.culture };
        var apiKey = config_1.RASA_API_KEY;
        var postOptions = {
            uri: "https://".concat(config_1.RASA_API_URL, "/application"),
            body: data,
            headers: {
                'apiKey': apiKey,
            },
            retryProtocol: false,
            timeout: 60000
        };
        logger.info('[RASAAPIServiceImplementation::AddApp] getting postOptions:', postOptions, 'subscriptionRegion:', subscriptionRegion, 'subscriptionKey:', subscriptionKey, 'nlu:', nlu);
        return (0, request_shared_1.SimplePostRequest)(postOptions)
            .then(function (response) {
            logger.info('[RASAAPIServiceImplementation::AddApp] getting response successfully subscriptionRegion:', subscriptionRegion, 'subscriptionKey:', subscriptionKey, 'nlu:', nlu);
            return Promise.resolve(response);
        })
            .then(function (result) {
            logger.info('[RASAAPIServiceImplementation::AddApp] getting response successfully RasaApiAppId:', result.id);
            return Promise.resolve(result.id);
        })
            .catch(function (err) {
            var errorMessage = typeof err === 'string' ? err : (err.message || err.description);
            logger.error('[RASAAPIServiceImplementation::AddApp] getting error:', errorMessage, 'subscriptionRegion:', subscriptionRegion, 'subscriptionKey:', subscriptionKey, 'nlu:', nlu);
            return Promise.reject(err);
        });
    };
    RASAAPIServiceImplementation.prototype.DeleteApp = function (subscriptionRegion, subscriptionKey, nlu) {
        var rasaAppId = nlu.appId;
        var apiKey = config_1.RASA_API_KEY;
        var deleteOptions = {
            uri: "https://".concat(config_1.RASA_API_URL, "/application/").concat(rasaAppId),
            headers: {
                'apiKey': apiKey
            },
            timeout: 60000
        };
        logger.info('[RASAAPIServiceImplementation::DeleteApp] getting deleteOptions:', deleteOptions, 'appId:', nlu.appId);
        return (0, request_shared_1.SimpleDeleteRequest)(deleteOptions)
            .then(function (response) {
            logger.info('[RASAAPIServiceImplementation::DeleteApp] getting response successfully', 'appId:', nlu.appId);
            return response;
        })
            .catch(function (err) {
            var errorMessage = typeof err === 'string' ? err : (err.message || err.description);
            logger.error('[RASAAPIServiceImplementation::DeleteApp] getting error:', errorMessage, 'appId:', nlu.appId);
            return Promise.resolve(err);
        });
    };
    RASAAPIServiceImplementation.prototype.GetAppsList = function (subscriptionRegion, subscriptionKey) {
        logger.info('[RASAAPIServiceImplementation::GetAppsList] getting subscriptionRegion:', subscriptionRegion, 'subscriptionKey:', subscriptionKey);
        return this.GetResultsFromPaginatedRequest(subscriptionRegion, subscriptionKey, "https://".concat(config_1.RASA_API_URL, "/application/"), 0)
            .then(function (response) {
            logger.info('[RASAAPIServiceImplementation::GetAppsList] getting response successfully response:', response);
            return Promise.resolve(response);
        })
            .catch(function (err) {
            var errorMessage = typeof err === 'string' ? err : (err.message || err.description);
            logger.error('[RASAAPIServiceImplementation::GetAppsList] getting error:', errorMessage, 'subscriptionRegion:', subscriptionRegion, 'subscriptionKey:', subscriptionKey);
            return Promise.reject(err);
        });
    };
    RASAAPIServiceImplementation.prototype.TrainWithVersion = function (trainVersion) {
        logger.info('[RASAAPIServiceImplementation::TrainWithVersion] getting trainVersion:', trainVersion);
        var data = '';
        var apiKey = config_1.RASA_API_KEY;
        var postOptions = {
            uri: "https://".concat(config_1.RASA_API_URL, "/train/").concat(trainVersion),
            body: data,
            headers: {
                'apiKey': apiKey,
            },
            retryProtocol: false,
            timeout: 60000
        };
        logger.info('[RASAAPIServiceImplementation::TrainWithVersion] getting postOptions:', postOptions);
        return (0, request_shared_1.SimplePostRequest)(postOptions)
            .then(function (response) {
            logger.info('[RASAAPIServiceImplementation::TrainWithVersion] getting response successfully response:', response);
            return response;
        })
            .catch(function (err) {
            var errorMessage = typeof err === 'string' ? err : (err.message || err.description);
            logger.error('[RASAAPIServiceImplementation::TrainWithVersion] getting error:', errorMessage);
            return Promise.reject(err);
        });
    };
    RASAAPIServiceImplementation.prototype.GetAppById = function (appId, subscriptionRegion, subscriptionKey) {
        var appIdRasaApi = appId;
        var getOptions = {
            uri: "https://".concat(config_1.RASA_API_URL, "/application/").concat(appIdRasaApi),
            headers: {
                'apiKey': config_1.RASA_API_KEY,
            },
            retryProtocol: false,
            timeout: 60000,
            validateErrorResponse: true
        };
        logger.info('[RASAAPIServiceImplementation::GetAppById] getting getOptions:', getOptions, 'appId:', appIdRasaApi);
        return (0, request_shared_1.SimpleGetRequest)(getOptions)
            .then(function (response) {
            logger.info('[RASAAPIServiceImplementation::GetAppById] getting response:', getOptions, 'result:', response);
            return Promise.resolve(response);
        }).catch(function (err) {
            var error = typeof err;
            logger.debug("tipo de error", error);
            var errorMessage = typeof err === 'string' ? err : (err.message || err.description || err.error);
            logger.error('[RASAAPIServiceImplementation::GetAppById] getting error:', err);
            logger.error('[RASAAPIServiceImplementation::GetAppById] getting error2:', err.error);
            logger.error('[RASAAPIServiceImplementation::GetAppById] getting error3:', errorMessage);
            if (errorMessage.includes('Application not found'))
                return Promise.resolve(errorMessage);
            return Promise.reject(errorMessage);
        });
    };
    RASAAPIServiceImplementation.prototype.AddIntent = function (subscriptionRegion, subscriptionKey, appId, name) {
        return (0, promises_shared_1.DummyPromise)()
            .then(function (result) {
            return Promise.resolve((0, utilities_1.guid)());
        });
    };
    RASAAPIServiceImplementation.prototype.AddUtterance = function (subscriptionRegion, subscriptionKey, appId, utterance, vcc) {
        var intentName = utterance.intent;
        var text = utterance.name;
        var entityLabels = utterance.entityLabels ? utterance.entityLabels : [];
        var savedUtterance = null;
        var savedUtteranceEntities = [];
        return (0, promises_shared_1.DummyPromise)()
            .then(function (result) {
            return mssql_service_1.MSSql.GetUtterance(vcc, appId, intentName, text);
        })
            .then(function (result) {
            savedUtterance = result;
            if (savedUtterance) {
                utterance.id = savedUtterance.id;
            }
            else {
                utterance.id = (0, utilities_1.guid)();
            }
        })
            .then(function (result) {
            if (savedUtterance) {
                return mssql_service_1.MSSql.GetUtteranceEntities(vcc, utterance.id);
            }
            else {
                return Promise.resolve([]);
            }
        })
            .then(function (result) {
            if (result)
                savedUtteranceEntities = result;
            var promiseArray = [];
            entityLabels.map(function (el) {
                if (!savedUtteranceEntities.find(function (sue) { return sue.utteranceId === utterance.id && sue.entityName === el.entityName && sue.start === el.start && sue.end === el.end; })) {
                    promiseArray.push(mssql_service_1.MSSql.SaveUtteranceEntity(vcc, utterance.id, el.entityName, el.start, el.end, el.startTokenIndex, el.endTokenIndex));
                }
            });
            savedUtteranceEntities.map(function (sue) {
                if (!entityLabels.find(function (el) { return el.utteranceId === utterance.id && el.entityName === sue.entityName && el.start === sue.start && el.end === sue.end; })) {
                    promiseArray.push(mssql_service_1.MSSql.DeleteUtteranceEntity(vcc, utterance.id, sue.entityName, sue.start, sue.end, sue.startTokenIndex, sue.endTokenIndex));
                }
            });
            return Promise.all(promiseArray);
        })
            .then(function (result) {
            return Promise.resolve({
                UtteranceText: utterance.name,
                ExampleId: savedUtterance ? savedUtterance.id : (0, utilities_1.guid)()
            });
        })
            .catch(function (error) {
            var errorMessage = typeof error === 'string' ? error : 'ERROR';
            logger.error('[RASAAPIServiceImplementation::AddUtterance] Error: ', error);
            if (error.code && error.code === 'ETIMEDOUT')
                errorMessage = 'SERVER_ERROR_TIMEOUT_FROM_RASA_SERVER';
            if (error.code && error.code === 'ECONNREFUSED')
                errorMessage = 'SERVER_ERROR_CONNECTION_REFUSED_FROM_RASA_SERVER';
            return Promise.reject(errorMessage);
        });
    };
    RASAAPIServiceImplementation.prototype.GetAppVersion = function (subscriptionRegion, subscriptionKey, appId, vcc) {
        return (0, promises_shared_1.DummyPromise)()
            .then(function (result) {
            return mssql_service_1.MSSql.GetNLUByAppId(vcc, appId);
        }).then(function (result) {
            return Promise.resolve(result);
        }).catch(function (error) {
            var errorMessage = typeof error === 'string' ? error : 'ERROR';
            logger.error('[RASAAPIServiceImplementation::GetAppVersion] Error: ', error);
            return Promise.reject(errorMessage);
        });
    };
    RASAAPIServiceImplementation.prototype.GetIntentsList = function (subscriptionRegion, subscriptionKey, appId, vcc) {
        return (0, promises_shared_1.DummyPromise)()
            .then(function (result) {
            return mssql_service_1.MSSql.GetIntents(vcc, appId, null, null, null);
        })
            .then(function (result) {
            var res = result ? result : [];
            var arrRes = [];
            res.map(function (r) {
                arrRes.push({
                    id: r.intentId,
                    name: r.name
                });
            });
            return Promise.resolve(arrRes);
        })
            .catch(function (error) {
            var errorMessage = typeof error === 'string' ? error : 'ERROR';
            logger.error('[RASAServiceImplementation::GetIntentsList] Error: ', error);
            if (error.code && error.code === 'ETIMEDOUT')
                errorMessage = 'SERVER_ERROR_TIMEOUT_FROM_RASA_SERVER';
            if (error.code && error.code === 'ECONNREFUSED')
                errorMessage = 'SERVER_ERROR_CONNECTION_REFUSED_FROM_RASA_SERVER';
            return Promise.reject(errorMessage);
        });
    };
    RASAAPIServiceImplementation.prototype.GetIntentsListDemo = function (subscriptionRegion, subscriptionKey, appId, resolve, reject, vcc) {
        return (0, promises_shared_1.DummyPromise)()
            .then(function (result) {
            return mssql_service_1.MSSql.GetIntents(vcc, appId, null, null, null);
        })
            .then(function (result) {
            var res = result ? result : [];
            var arrRes = [];
            res.map(function (r) {
                arrRes.push({
                    id: r.intentId,
                    name: r.name
                });
            });
            return Promise.resolve(arrRes);
        })
            .catch(function (error) {
            var errorMessage = typeof error === 'string' ? error : 'ERROR';
            logger.error('[RASAAPIServiceImplementation::GetIntentsListDemo] Error: ', error);
            if (error.code && error.code === 'ETIMEDOUT')
                errorMessage = 'SERVER_ERROR_TIMEOUT_FROM_RASA_SERVER';
            if (error.code && error.code === 'ECONNREFUSED')
                errorMessage = 'SERVER_ERROR_CONNECTION_REFUSED_FROM_RASA_SERVER';
            return Promise.reject(errorMessage);
        });
    };
    RASAAPIServiceImplementation.prototype.GetUtterancesList = function (subscriptionRegion, subscriptionKey, appId, vcc) {
        var utterances = [];
        return (0, promises_shared_1.DummyPromise)()
            .then(function (result) {
            return (0, utterance_service_1.GetUtterances)(vcc, appId, null, null, null, true);
        })
            .then(function (result) {
            if (result)
                utterances = result;
            return Promise.resolve(utterances);
        })
            .catch(function (error) {
            var errorMessage = typeof error === 'string' ? error : 'ERROR';
            logger.error('[RASAServiceImplementation::GetUtterancesList] Error: ', error);
            if (error.code && error.code === 'ETIMEDOUT')
                errorMessage = 'SERVER_ERROR_TIMEOUT_FROM_RASA_SERVER';
            if (error.code && error.code === 'ECONNREFUSED')
                errorMessage = 'SERVER_ERROR_CONNECTION_REFUSED_FROM_RASA_SERVER';
            return Promise.reject(errorMessage);
        });
    };
    RASAAPIServiceImplementation.prototype.GetUtterancesFromMD = function (appId) {
        return (0, promises_shared_1.DummyPromise)()
            .then(function (result) {
            return Promise.resolve([]);
        });
    };
    RASAAPIServiceImplementation.prototype.AddSimpleEntity = function (subscriptionRegion, subscriptionKey, appId, entity) {
        return (0, promises_shared_1.DummyPromise)()
            .then(function (result) {
            return Promise.resolve((0, utilities_1.guid)());
        });
    };
    RASAAPIServiceImplementation.prototype.RenameEntity = function (subscriptionRegion, subscriptionKey, appId, entityId, entityName) {
        return null;
    };
    RASAAPIServiceImplementation.prototype.GetEntitiesList = function (subscriptionRegion, subscriptionKey, appId, vcc) {
        return (0, promises_shared_1.DummyPromise)()
            .then(function (result) {
            return mssql_service_1.MSSql.GetEntities(vcc, appId, '', '', '');
        });
    };
    RASAAPIServiceImplementation.prototype.GetEntity = function (subscriptionRegion, subscriptionKey, appId, entityId) {
        return null;
    };
    RASAAPIServiceImplementation.prototype.DeleteAppEntity = function (subscriptionRegion, subscriptionKey, appId, entityId) {
        return null;
    };
    RASAAPIServiceImplementation.prototype.GetAllIntentsFromRASAContent = function (content) {
        return null;
    };
    RASAAPIServiceImplementation.prototype.GetAllUtterancesFromRASAContent = function (content) {
        return null;
    };
    RASAAPIServiceImplementation.prototype.ExportModel = function (nlu, res) {
        var _this = this;
        var installationId = '';
        var modelFolder = null;
        var modelToExport = {};
        var intentsToExport = [];
        var entitiesToExport = [];
        var utteranceEntitiesToExport = [];
        return (0, promises_shared_1.DummyPromise)()
            .then(function (result) {
            return mssql_service_1.MSSql.GetNLUByAppId(nlu.vcc, nlu.appId);
        })
            .then(function (result) {
            if (!result)
                throw 'SERVER_ERROR_NLU_NOT_FOUND';
            modelToExport = {
                name: result.name,
                culture: result.culture,
                description: result.description,
                cognitiveEngine: _this.GetCognitiveEngineName()
            };
            var promiseArray = [];
            promiseArray.push(mssql_service_1.MSSql.GetIntents(nlu.vcc, nlu.appId, null, null, null));
            promiseArray.push(mssql_service_1.MSSql.GetUtterances(nlu.vcc, nlu.appId, null));
            promiseArray.push(mssql_service_1.MSSql.GetEntities(nlu.vcc, nlu.appId, '', '', null));
            return Promise.all(promiseArray);
        })
            .then(function (result) {
            if (!result[0])
                throw 'SERVER_ERROR_NLU_INTENTS_NOT_FOUND';
            if (!result[1])
                throw 'SERVER_ERROR_NLU_UTTERANCES_NOT_FOUND';
            if (!result[2])
                throw 'SERVER_ERROR_NLU_ENTITIES_NOT_FOUND';
            var promiseArray = [];
            result[0].map(function (i) {
                var intentUtterances = [];
                result[1].filter(function (r) { return r.intentId === i.intentId; }).map(function (u) {
                    intentUtterances.push({
                        id: u.id,
                        text: u.text
                    });
                    promiseArray.push(mssql_service_1.MSSql.GetUtteranceEntities(nlu.vcc, u.id));
                });
                intentsToExport.push({
                    id: i.intentId,
                    name: i.name,
                    color: i.color,
                    utterances: intentUtterances
                });
            });
            result[2].map(function (e) {
                entitiesToExport.push({
                    id: (0, utilities_1.guid)(),
                    name: e.name,
                    type: e.type
                });
            });
            modelToExport['intents'] = intentsToExport;
            modelToExport['entities'] = entitiesToExport;
            return Promise.all(promiseArray);
        })
            .then(function (result) {
            result.map(function (r) {
                r.map(function (ue) {
                    utteranceEntitiesToExport.push({
                        utteranceId: ue.utteranceId,
                        entityName: ue.entityName,
                        start: ue.start,
                        end: ue.end,
                        startTokenIndex: ue.startTokenIndex,
                        endTokenIndex: ue.endTokenIndex
                    });
                });
            });
            modelToExport['utteranceEntities'] = utteranceEntitiesToExport;
            return Promise.resolve(true);
        })
            .then(function (result) {
            modelToExport.intents.map(function (i) {
                i.utterances.map(function (u) {
                    var newId = (0, utilities_1.guid)();
                    var oldId = u.id;
                    u.id = newId;
                    modelToExport.utteranceEntities.filter(function (ue) { return ue.utteranceId === oldId; }).map(function (ue) { ue.utteranceId = newId; });
                });
            });
            return Promise.resolve(modelToExport);
        })
            .catch(function (error) {
            var errorMessage = typeof error === 'string' ? error : 'ERROR';
            logger.error('[RASAServiceImplementation::ExportModel] Error: ', error);
            if (error.code && error.code === 'ETIMEDOUT')
                errorMessage = 'SERVER_ERROR_TIMEOUT_FROM_RASA_SERVER';
            if (error.code && error.code === 'ECONNREFUSED')
                errorMessage = 'SERVER_ERROR_CONNECTION_REFUSED_FROM_RASA_SERVER';
            return Promise.reject(errorMessage);
        });
    };
    RASAAPIServiceImplementation.prototype.GetResultsFromPaginatedRequest = function (subscriptionRegion, subscriptionKey, url, page) {
        var _this = this;
        page = page ? page : 0;
        var allResults = [];
        var getOptions = {
            uri: "".concat(url, "?page=").concat(page),
            headers: {
                'apiKey': config_1.RASA_API_KEY
            },
            retryProtocol: false,
            timeout: 60000
        };
        return (0, request_shared_1.SimpleGetRequest)(getOptions)
            .then(function (result) {
            if (!result)
                throw 'SERVER_ERROR_GET_RASA_API_ITEMS_SERVICE_FAILED';
            var totalRecords = result.count;
            var pageSize = result.pageSize;
            var maxRecordsToShow = (page + 1) * pageSize;
            if (maxRecordsToShow <= totalRecords) {
                allResults = result.data;
                return _this.GetResultsFromPaginatedRequest(subscriptionRegion, subscriptionKey, url, page + 1);
            }
            else {
                return Promise.resolve(result.data);
            }
        })
            .then(function (result) {
            logger.info('[RASAAPIServiceImplementation::GetResultsFromPaginatedRequest] getting result:', result, 'page:', page, 'totalRecords', 'getoptions:', getOptions);
            allResults = allResults.concat(result);
            return Promise.resolve(allResults);
        })
            .catch(function (err) {
            var errorMessage = typeof err === 'string' ? err : (err.message || err.description);
            logger.error('[RASAAPIServiceImplementation::GetResultsFromPaginatedRequest] getting error:', errorMessage, 'subscriptionRegion:', subscriptionRegion, 'subscriptionKey:', subscriptionKey);
            return Promise.reject(err);
        });
    };
    RASAAPIServiceImplementation.prototype.RunApp = function (appId, task) {
        return null;
    };
    RASAAPIServiceImplementation.prototype.StopApp = function (appId, port, task) {
        return null;
    };
    RASAAPIServiceImplementation.prototype.GenerateNluObjectVersion = function (appId, vcc) {
        var _this = this;
        var installationId = '';
        var modelFolder = '';
        var newNluTextContent = '';
        return (0, promises_shared_1.DummyPromise)()
            .then(function (result) {
            return (0, platform_service_1.GetPlatformVersion)();
        })
            .then(function (result) {
            if (!result)
                throw 'SERVER_ERROR_CANNOT_CREATE_NLU_TEMPORARY_FOLDER';
            return _this.GetNluContentFromDatabase(appId, vcc);
        })
            .then(function (result) {
            if (!result)
                throw 'SERVER_ERROR_CANNOT_GENERATE_NLU_CONTENT';
            newNluTextContent = result;
            return result;
        })
            .catch(function (error) {
            var errorMessage = typeof error === 'string' ? error : 'ERROR';
            logger.error('[RASAServiceImplementation::GenerateNluMd] Error: ', error);
            if (error.code && error.code === 'ETIMEDOUT')
                errorMessage = 'SERVER_ERROR_TIMEOUT_FROM_RASA_SERVER';
            if (error.code && error.code === 'ECONNREFUSED')
                errorMessage = 'SERVER_ERROR_CONNECTION_REFUSED_FROM_RASA_SERVER';
            return Promise.reject(errorMessage);
        });
    };
    RASAAPIServiceImplementation.prototype.GetNluContentFromDatabase = function (appId, vcc) {
        var _this = this;
        var nluIntents = [];
        var nluUtteranceObjects = [];
        var nluObjectVersionContent = null;
        return (0, promises_shared_1.DummyPromise)()
            .then(function (result) {
            return mssql_service_1.MSSql.GetIntents(vcc, appId, null, null, null);
        })
            .then(function (result) {
            if (!result)
                throw 'SERVER_ERROR_MISSING_INTENTS_TO_TRAIN';
            nluIntents = result;
            return (0, utterance_service_1.GetUtterances)(vcc, appId, null, null, null, true);
        })
            .then(function (result) {
            if (!result)
                throw 'SERVER_ERROR_MISSING_UTTERANCES_TO_TRAIN';
            nluUtteranceObjects = result;
            nluObjectVersionContent = _this.GenerateNluObjetVersionContent(nluIntents, nluUtteranceObjects);
            return Promise.resolve(nluObjectVersionContent);
        })
            .catch(function (error) {
            var errorMessage = typeof error === 'string' ? error : 'ERROR';
            logger.error('[RASAServiceImplementation::GetContentFromDatabase] Error: ', error);
            if (error.code && error.code === 'ETIMEDOUT')
                errorMessage = 'SERVER_ERROR_TIMEOUT_FROM_RASA_SERVER';
            if (error.code && error.code === 'ECONNREFUSED')
                errorMessage = 'SERVER_ERROR_CONNECTION_REFUSED_FROM_RASA_SERVER';
            return Promise.reject(errorMessage);
        });
    };
    RASAAPIServiceImplementation.prototype.GenerateNluObjetVersionContent = function (intents, utterances) {
        var nluContent = '';
        var intentsObj = [];
        var intentDummy = {
            "action": "None",
            "examples": [
                {
                    "raw": "xxxxx",
                    "data": []
                }
            ]
        };
        intents.map(function (i) {
            var intentObj = {
                action: i.name,
                examples: []
            };
            var intentBlockText = "## intent:".concat(i.intentId, "\n");
            utterances.filter(function (u) { return u.intentId === i.intentId; }).map(function (u) {
                var labeledUtterance = null;
                var entities = [];
                var uteranceObj = {
                    id: u.id,
                    raw: u.text,
                    data: []
                };
                if (u.entityLabels && u.entityLabels.length) {
                    u.entityLabels.forEach(function (el) {
                        entities.push({
                            "value": "entity-" + el.entityName,
                            "entity": el.entityName,
                            "start": el.start,
                            "end": el.entityName
                        });
                    });
                    uteranceObj.data = entities;
                }
                intentObj.examples.push(uteranceObj);
                intentBlockText += "- ".concat(labeledUtterance, "\n");
            });
            intentBlockText += '\n';
            nluContent += intentBlockText;
            intentsObj.push(intentObj);
        });
        intentsObj.push(intentDummy);
        return intentsObj;
    };
    RASAAPIServiceImplementation.prototype.GenerateLabeledUtterance = function (utteranceText, entityLabels) {
        var labeledUtterance = utteranceText;
        var addedText = '';
        entityLabels.map(function (el) {
            var splittedText = labeledUtterance.split('');
            labeledUtterance = '';
            splittedText.map(function (st, i) {
                if (i === el.start + addedText.length) {
                    labeledUtterance += '[' + st;
                }
                else if (i === el.end + addedText.length) {
                    labeledUtterance += st + "](".concat(el.entityName, ")");
                }
                else {
                    labeledUtterance += st;
                }
            });
            addedText += "[](".concat(el.entityName, ")");
        });
        return labeledUtterance;
    };
    return RASAAPIServiceImplementation;
}());
exports.RASAAPIServiceImplementation = RASAAPIServiceImplementation;
