"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.SemanticAnalysisConvertAudioImplementation = void 0;
var config_1 = require("../config/config");
var mssql_service_1 = require("../db/mssql.service");
var redis_service_1 = require("../db/redis.service");
var promises_shared_1 = require("../shared/promises.shared");
var log4js = require("log4js");
var moment = require("moment");
var billing_batch_model_1 = require("../models/billing-batch.model");
var file_manager_1 = require("../shared/file-manager");
var child_process_1 = require("child_process");
var billing_implementation_1 = require("./billing.implementation");
var task_service_1 = require("../services/task.service");
var logger = log4js.getLogger('Task');
var CURRENT_SYSTEM_SERVICE = 'TRANSCRIPTION';
var CURRENT_IMPLEMENTATION = 'SemanticAnalysisConvertAudioImplementation';
var SemanticAnalysisConvertAudioImplementation = (function () {
    function SemanticAnalysisConvertAudioImplementation() {
        this.errorData = new billing_batch_model_1.BatchError();
        this.queueBase = 'SemanticAnalysisConvertAudio';
    }
    SemanticAnalysisConvertAudioImplementation.prototype.Execute = function (executionCode, rateRecursive) {
        var _this = this;
        logger.info("[SemanticAnalysisConvertAudioImplementation::Execute] executionCode: ".concat(executionCode, " executing wave #").concat(rateRecursive));
        var executeIndex = 1;
        var ratio = config_1.RATIO_SEMANTIC_ANALYSIS;
        var executeResult = true;
        var currentBlockPromise = null;
        return (0, promises_shared_1.SleepPromise)(config_1.SLEEP_SEMANTIC_ANALYSIS)
            .then(function (_) {
            currentBlockPromise = 'BLOCK_01_GET_FROM_REDIS';
            return redis_service_1.Redis.GetLengthFromList(_this.queueBase);
        })
            .then(function (result) {
            currentBlockPromise = 'BLOCK_02_GET_QUEUE_ITEMS';
            if (!result)
                throw 'SERVER_INFO_REDIS_KEYS_FOR_SEMANTIC_ANALYSIS_CONVERT_AUDIO_NOT_FOUND';
            return redis_service_1.Redis.LRange(_this.queueBase, 0, ratio - 1);
        })
            .then(function (result) {
            currentBlockPromise = 'BLOCK_03_SEMANTIC_ANALYSIS_CONVERT_AUDIO_IMPL';
            ratio = result && result.length <= ratio ? result.length : 0;
            var promiseArray = [];
            while (ratio >= executeIndex) {
                promiseArray.push(_this.SemanticAnalysisConvertAudioExecute(executionCode));
                executeIndex++;
            }
            if (!promiseArray.length)
                promiseArray.push(Promise.resolve(true));
            return Promise.all(promiseArray);
        })
            .then(function (results) {
            currentBlockPromise = 'BLOCK_04_GETTING_KEYS_FOR_RECURSIVITY';
            if (results.filter(function (item) { return item === false; }).length > 0) {
                executeResult = false;
            }
            return redis_service_1.Redis.LRange(_this.queueBase, 0, ratio - 1);
        })
            .then(function (result) {
            currentBlockPromise = 'BLOCK_05_RELAUNCH_SEMANTIC_ANALYSIS_UPLOAD_IMP';
            if ((result === null || result === void 0 ? void 0 : result.length) && rateRecursive >= 1)
                return _this.Execute(executionCode, rateRecursive - 1);
            return Promise.resolve(executeResult);
        })
            .then(function (result) {
            currentBlockPromise = 'BLOCK_06_RESOLVING_IMPL';
            if (rateRecursive === config_1.RATE_RECURSIVE_SEMANTIC_ANALYSIS)
                logger.info("[SemanticAnalysisConvertAudioImplementation::Execute] executionCode: ".concat(executionCode, " Finishing execution wave #").concat(rateRecursive));
            var currentResult = executeResult && result ? true : false;
            return Promise.resolve(currentResult);
        })
            .catch(function (error) {
            var errorMessage = typeof error === 'string' ? error : (error.message || error.description);
            if (errorMessage.startsWith('SERVER_INFO_'))
                logger.info("[SemanticAnalysisConvertAudioImplementation::Execute] executionCode: ".concat(executionCode, ", Block: ").concat(currentBlockPromise, ", getting info:"), errorMessage);
            else
                logger.error("[SemanticAnalysisConvertAudioImplementation::Execute] executionCode: ".concat(executionCode, ", Block: ").concat(currentBlockPromise, ", getting error:"), errorMessage);
            return Promise.resolve(false);
        });
    };
    SemanticAnalysisConvertAudioImplementation.prototype.SemanticAnalysisConvertAudioExecute = function (executionCode) {
        var _this = this;
        var storage = '';
        var currentQueueId = '';
        var processInteraction = null;
        var queueItems = null;
        return redis_service_1.Redis.RPopLPush(this.queueBase, "".concat(this.queueBase, "Execute"))
            .then(function (queueId) {
            if ((queueId === null || queueId === void 0 ? void 0 : queueId.length) > 0) {
                currentQueueId = queueId;
                logger.info('[SemanticAnalysisImplementation::SemanticAnalysisExecute] executionCode:', executionCode, 'getting queueId:', currentQueueId, 'START');
                queueItems = (0, task_service_1.GetItemFromQueueId)(currentQueueId);
                storage = "".concat(queueItems.vcc, ":").concat(queueItems.processId);
                logger.info('[SemanticAnalysisConvertAudioImplementation::SemanticAnalysisConvertAudioExecute] executionCode:', executionCode, 'getting queueId:', currentQueueId, 'storage :', storage);
                return redis_service_1.Redis.GetProcessInteraction(storage, currentQueueId);
            }
            return Promise.resolve(null);
        })
            .then(function (processInteractionResult) {
            if (processInteractionResult) {
                processInteraction = processInteractionResult;
                logger.info('[SemanticAnalysisConvertAudioImplementation::SemanticAnalysisConvertAudioExecute] executionCode:', executionCode, 'getting queueId:', currentQueueId, 'getting GetProcessInteraction: 1, status:', processInteraction.statusDescription);
                if (processInteraction.statusDescription === 'pendingConvertAudio')
                    return _this.ConvertAudio(processInteraction, currentQueueId, executionCode);
            }
            if (!processInteraction && currentQueueId) {
                logger.warn('[SemanticAnalysisConvertAudioImplementation::SemanticAnalysisConvertAudioExecute] executionCode:', executionCode, 'getting queueId:', currentQueueId, 'getting GetProcessInteraction: 0');
                if (!processInteraction) {
                    redis_service_1.Redis.DeleteQueue("".concat(_this.queueBase, "Execute"), currentQueueId);
                    throw 'SERVER_ERROR_INTERACTION_DATA_NOT_FOUND_ON_STORAGE';
                }
            }
            return Promise.resolve(null);
        })
            .then(function (result) {
            if (result)
                processInteraction = result;
            if (processInteraction)
                return redis_service_1.Redis.SaveProcessInteraction(storage, currentQueueId, processInteraction);
            return Promise.resolve(null);
        })
            .then(function (result) {
            var promiseArray = [];
            var billing = new billing_implementation_1.BillingImplementation();
            if (result
                &&
                    (processInteraction === null || processInteraction === void 0 ? void 0 : processInteraction.status) === 6
                &&
                    (processInteraction === null || processInteraction === void 0 ? void 0 : processInteraction.statusDescription) === 'ready') {
                logger.info('[SemanticAnalysisImplementation::SemanticAnalysisConvertAudioExecute] executionCode:', executionCode, 'getting queueId:', currentQueueId, 'QUEUE_FINISHED');
                billing.AddUpdateTranscriptionDetailToMainFile(processInteraction, 'ValidChar', processInteraction.batchTypeName, processInteraction.batchTypeId);
                billing.AddUpdateTranscriptionDetailToMainFile(processInteraction, 'ValidWords', processInteraction.batchTypeName, processInteraction.batchTypeId);
                billing.AddUpdateTranscriptionDetailToMainFile(processInteraction, 'TotalPhrases', processInteraction.batchTypeName, processInteraction.batchTypeId);
                billing.AddUpdateTranscriptionDetailToMainFile(processInteraction, 'CharsPerPhrase', processInteraction.batchTypeName, processInteraction.batchTypeId);
                billing.AddUpdateTranscriptionDetailToMainFile(processInteraction, 'IdathaApi', processInteraction.batchTypeName, processInteraction.batchTypeId);
                billing.AddCloseTranscriptionDetailToMainFile(processInteraction, processInteraction.batchTypeName, processInteraction.batchTypeId, 1, 'SUCCED');
                promiseArray.push(redis_service_1.Redis.DeleteQueue('SemanticAnalysisConvertAudioExecute', currentQueueId));
            }
            else if ((processInteraction === null || processInteraction === void 0 ? void 0 : processInteraction.retry) >= config_1.RETRY_SEMANTIC_ANALYSIS) {
                logger.error('[SemanticAnalysisConvertAudioImplementation::SemanticAnalysisConvertAudioExecute] executionCode:', executionCode, 'getting queueId:', currentQueueId, 'REMOVED_BY_RETRIES');
                billing.AddCloseTranscriptionDetailToMainFile(processInteraction, processInteraction.batchTypeName, processInteraction.batchTypeId, 0, 'REMOVED_BY_RETRIES');
                promiseArray.push(redis_service_1.Redis.DeleteQueue('SemanticAnalysisConvertAudioExecute', currentQueueId));
            }
            else if (currentQueueId && !processInteraction) {
                logger.error('[SemanticAnalysisConvertAudioImplementation::SemanticAnalysisConvertAudioExecute] executionCode:', executionCode, 'getting queueId:', currentQueueId, 'INTERACTION_DO_NOT_EXIST');
                var interactionData = {
                    vcc: queueItems.vcc,
                    processId: queueItems.processId,
                    id: queueItems.interactionId,
                    section: queueItems.interactionSection,
                    part: queueItems.interactionPart
                };
                billing.AddCloseTranscriptionDetailToMainFile(interactionData, processInteraction.batchTypeName, processInteraction.batchTypeId, 0, 'INTERACTION_DO_NOT_EXIST');
                promiseArray.push(redis_service_1.Redis.DeleteQueue('SemanticAnalysisConvertAudioExecute', currentQueueId));
            }
            else {
                logger.warn('[SemanticAnalysisConvertAudioImplementation::SemanticAnalysisConvertAudioExecute] executionCode:', executionCode, 'getting queueId:', currentQueueId, 'QUEUE_WAS_NOT_REMOVED');
            }
            if (!promiseArray.length)
                promiseArray.push(Promise.resolve(true));
            return Promise.all(promiseArray);
        })
            .then(function (result) {
            var promiseArray = [];
            if ((processInteraction === null || processInteraction === void 0 ? void 0 : processInteraction.status) === 6
                &&
                    (processInteraction === null || processInteraction === void 0 ? void 0 : processInteraction.statusDescription) === 'ready') {
                promiseArray.push(redis_service_1.Redis.UpdateProgress(storage));
                promiseArray.push(redis_service_1.Redis.DeleteProcessInteraction(storage, currentQueueId));
                promiseArray.push(redis_service_1.Redis.SaveReadyProcessInteraction(executionCode, currentQueueId, processInteraction));
            }
            else if ((processInteraction === null || processInteraction === void 0 ? void 0 : processInteraction.retry) >= config_1.RETRY_SEMANTIC_ANALYSIS) {
                processInteraction.isCancel = true;
                promiseArray.push(redis_service_1.Redis.UpdateProgress(storage));
                promiseArray.push(redis_service_1.Redis.UpdateErrorProgress(storage));
                promiseArray.push(redis_service_1.Redis.DeleteProcessInteraction(storage, currentQueueId));
                promiseArray.push(redis_service_1.Redis.SaveReadyProcessInteraction(executionCode, currentQueueId, processInteraction));
                promiseArray.push(mssql_service_1.MSSql.SaveBatchError(queueItems.vcc, CURRENT_SYSTEM_SERVICE, processInteraction.batchTypeId, processInteraction.batchTypeId, CURRENT_IMPLEMENTATION, processInteraction.id, null, processInteraction.errorFunction, processInteraction.errorMessage, processInteraction.errorDetail));
            }
            if (!promiseArray.length)
                promiseArray.push(Promise.resolve(true));
            return Promise.all(promiseArray);
        })
            .then(function (_) {
            return Promise.resolve(true);
        })
            .catch(function (err) {
            var errorMessage = typeof err === 'string' ? err : (err.message || err.description);
            logger.error('[SemanticAnalysisConvertAudioImplementation::SemanticAnalysisConvertAudioExecute] executionCode:', executionCode, 'getting queueId:', currentQueueId, 'Catching error: ' + errorMessage);
            if (processInteraction)
                logger.error('[SemanticAnalysisConvertAudioImplementation::SemanticAnalysisConvertAudioExecute] executionCode:', executionCode, 'getting queueId:', currentQueueId, 'getting error:', errorMessage, ' vcc', processInteraction.vcc, 'processId:', processInteraction.processId, 'id:', processInteraction.id, 'section:', processInteraction.section, 'part:', processInteraction.part);
            return Promise.resolve(false);
        });
    };
    SemanticAnalysisConvertAudioImplementation.prototype.ConvertAudio = function (processInteraction, currentQueueId, executionCode) {
        var _this = this;
        var newFormat = 'mp3';
        var newOldFileNamePath = '';
        var currentFolder = ['allegro', 'i6', 'i-ftp', 'i-sftp'].includes(processInteraction === null || processInteraction === void 0 ? void 0 : processInteraction.environmentType) ? config_1.PATH_FOLDER_TEMP_AUDIO : config_1.PATH_FOLDER_PROCESS_AUDIO;
        var oldFilePath = "".concat(currentFolder, "/").concat(processInteraction.vcc, "/").concat(processInteraction.fileName);
        var newFilePath = "".concat(currentFolder, "/").concat(processInteraction.vcc, "/").concat(processInteraction.fileName.replace('.wav', ".".concat(newFormat)).replace('.ogg', ".".concat(newFormat)));
        var forceConvertion = false;
        if (oldFilePath === newFilePath) {
            forceConvertion = true;
            newOldFileNamePath = oldFilePath.replace('.mp3', '_changed.mp3');
        }
        return (0, promises_shared_1.DummyPromise)()
            .then(function (_) {
            if (forceConvertion)
                return (0, file_manager_1.RenameFile)(oldFilePath, newOldFileNamePath);
            return Promise.resolve(true);
        })
            .then(function (_) {
            if (forceConvertion)
                oldFilePath = newOldFileNamePath;
            return Promise.resolve(true);
        })
            .then(function (_) {
            var channel = 1;
            if (processInteraction.channels !== 'mono')
                channel = 2;
            var cmd = "".concat(config_1.FFMPEG_FOLDER_PATH, "ffmpeg -y -i \"").concat(oldFilePath, "\" -vn -ar 22050 -ac ").concat(channel, " -b:a 16k \"").concat(newFilePath, "\"");
            logger.debug('[SemanticAnalysisConvertAudioImplementation::ConvertAudio] executionCode:', executionCode, 'getting queueId:', currentQueueId, 'getting cmd:', cmd);
            return new Promise(function (resolve, reject) {
                var _a, _b;
                try {
                    (0, child_process_1.exec)(cmd, function (err, stdout, stderr) {
                        var _a, _b;
                        if (err) {
                            if (((_a = err.message) === null || _a === void 0 ? void 0 : _a.includes('No such file or directory'))
                                ||
                                    ((_b = err.message) === null || _b === void 0 ? void 0 : _b.includes('Command failed'))) {
                                logger.error('[SemanticAnalysisConvertAudioImplementation::ConvertAudio] executionCode:', executionCode, 'getting queueId:', currentQueueId, 'the original file was not found.');
                                return resolve(true);
                            }
                            else {
                                return reject(err);
                            }
                        }
                        else {
                            processInteraction.fileName = processInteraction.fileName.replace('.wav', ".".concat(newFormat)).replace('.ogg', ".".concat(newFormat));
                            return resolve(stdout.media);
                        }
                    });
                }
                catch (e) {
                    if (((_a = e.message) === null || _a === void 0 ? void 0 : _a.includes('No such file or directory'))
                        ||
                            ((_b = e.message) === null || _b === void 0 ? void 0 : _b.includes('Command failed'))) {
                        logger.error('[SemanticAnalysisConvertAudioImplementation::ConvertAudio] executionCode:', executionCode, 'getting queueId:', currentQueueId, 'the original file was not found.');
                        processInteraction.fileName = null;
                        return resolve(true);
                    }
                    else {
                        return reject(e);
                    }
                }
            });
        })
            .then(function (_) {
            return (0, file_manager_1.DeleteFile)(oldFilePath, true);
        })
            .then(function (_) {
            processInteraction['status'] = 6;
            processInteraction['statusDescription'] = 'ready';
            processInteraction['format'] = newFormat;
            processInteraction['fileName'] = processInteraction.fileName;
            processInteraction['retry'] = 0;
            return Promise.resolve(processInteraction);
        })
            .catch(function (err) {
            var errorMessage = typeof err === 'string' ? err : (err.message || err.description);
            logger.error('[SemanticAnalysisConvertAudioImplementation::ConvertAudio] executionCode:', executionCode, 'getting queueId:', currentQueueId, 'getting error:', errorMessage);
            processInteraction['status'] = -6;
            processInteraction['retry'] = processInteraction['retry'] ? processInteraction['retry'] + 1 : 1;
            processInteraction['uploadedDate'] = moment().format('x');
            processInteraction['errorMessage'] = errorMessage;
            _this.errorData.SetErrorData('ConvertAudio', errorMessage, err);
            if (processInteraction['retry'] === config_1.RETRY_SEMANTIC_ANALYSIS) {
                processInteraction['status'] = 6;
                processInteraction['statusDescription'] = 'ready';
            }
            return Promise.resolve(processInteraction);
        });
    };
    return SemanticAnalysisConvertAudioImplementation;
}());
exports.SemanticAnalysisConvertAudioImplementation = SemanticAnalysisConvertAudioImplementation;
