"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.SupervisorImplementation = void 0;
var log4js = require("log4js");
var redis_service_1 = require("../db/redis.service");
var promises_shared_1 = require("../shared/promises.shared");
var mssql_service_1 = require("../db/mssql.service");
var config_1 = require("../config/config");
var billing_implementation_1 = require("./billing.implementation");
var billing_batch_implementation_1 = require("./billing-batch.implementation");
var idatha_service_1 = require("../services/idatha.service");
var platform_service_1 = require("../services/platform.service");
var logger = log4js.getLogger('Task');
var SupervisorImplementation = (function () {
    function SupervisorImplementation() {
    }
    SupervisorImplementation.prototype.Execute = function (executionCode) {
        var _this = this;
        var queueItems = ['Process', 'I6Update', 'Audios', 'DiarizationConvert', 'TranscriptionsUpload', 'TranscriptionsStandby', 'TranscriptionsDownload', 'DiarizationAnalysis', 'SentimentAnalysis', 'SemanticAnalysisUpload', 'SemanticAnalysisDownload', 'SemanticAnalysisConvertAudio', 'SemanticQuery', 'CognitiveAnalysis', 'TipificationAnalysis', 'BatchScheduleList'];
        var processTail = queueItems[0];
        var executeIndex = 0;
        var ratio = config_1.RATIO_SUPERVISOR;
        var executeResult = true;
        return (0, promises_shared_1.DummyPromise)()
            .then(function (_) {
            return (0, platform_service_1.GetPlatformVersion)();
        })
            .then(function (result) {
            var promiseArray = [];
            var billing = new billing_implementation_1.BillingImplementation();
            billing.SplitGeneralFileToSegmentedFiles();
            promiseArray.push(billing.SegmentedFilesUpload(result.installationId));
            if (!result.isRegistered)
                promiseArray.push(billing.SavePlatformVersion());
            return Promise.all(promiseArray);
        })
            .then(function (_) {
            return mssql_service_1.MSSql.GetFinishedBatch();
        })
            .then(function (result) {
            if (result && result.length > 0) {
                var promiseArray_1 = [];
                var billingBatch_1 = new billing_batch_implementation_1.BillingBatchImplementation();
                result.map(function (b) {
                    promiseArray_1.push(billingBatch_1.CloseBatchSummary(b));
                });
                return Promise.all(promiseArray_1);
            }
            else {
                return Promise.resolve([true]);
            }
        })
            .then(function (result) {
            ratio = queueItems.length;
            var promiseArray = [];
            while (ratio > executeIndex) {
                promiseArray.push(_this.RestoreQueue(queueItems[executeIndex], executionCode));
                executeIndex++;
            }
            return Promise.all(promiseArray);
        }).then(function (results) {
            if (results.filter(function (item) { return item === false; }).length > 0) {
                executeResult = false;
            }
            return Promise.resolve(executeResult);
        }).then(function (result) {
            return redis_service_1.Redis.LRange(processTail, 0, -1);
        })
            .then(function (result) {
            var vccs = result.map(function (x) {
                return x.split(':')[0];
            });
            var uniqueVCCs = _this.UniqueVCC(vccs);
            ratio = uniqueVCCs.length;
            executeIndex = 0;
            var promiseArray = [];
            while (ratio > executeIndex) {
                promiseArray.push(_this.IdathaLogin(uniqueVCCs[executeIndex], executionCode));
                promiseArray.push(_this.AzureSuscription(uniqueVCCs[executeIndex], executionCode));
                executeIndex++;
            }
            return Promise.all(promiseArray);
        }).then(function (results) {
            if (results.filter(function (item) { return item === false; }).length > 0) {
                executeResult = false;
            }
            return Promise.resolve(executeResult);
        }).catch(function (err) {
            var errorMessage = typeof err === 'string' ? err : (err.message || err.description);
            logger.error('[Execute] executionCode:', executionCode, 'getting error:', errorMessage);
            return Promise.resolve(false);
        });
    };
    SupervisorImplementation.prototype.RestoreQueue = function (queueId, executionCode) {
        var source = queueId;
        var executionTail = queueId;
        executionTail += 'Execute';
        var executeResult = true;
        return (0, promises_shared_1.DummyPromise)()
            .then(function (_) {
            return redis_service_1.Redis.LRange(executionTail, 0, -1);
        })
            .then(function (result) {
            if ((result === null || result === void 0 ? void 0 : result.length) === 0)
                throw 'SERVER_INFO_REDIS_KEYS_NOT_FOUND';
            logger.info('[SupervisorImplementation::RestoreQueue] executionCode:', executionCode, "getting result for ".concat(queueId, ": ").concat(result.length));
            var lenItems = result && result.length;
            var executeIndex = 1;
            var promiseArray = [];
            while (lenItems >= executeIndex) {
                promiseArray.push(redis_service_1.Redis.RPopLPush(executionTail, source));
                executeIndex++;
            }
            return Promise.all(promiseArray);
        })
            .then(function (results) {
            if (results.filter(function (item) { return item === false; }).length > 0) {
                executeResult = false;
            }
            return Promise.resolve(executeResult);
        })
            .catch(function (error) {
            var errorMessage = typeof error === 'string' ? error : (error.message || error.description);
            if (errorMessage.startsWith('SERVER_INFO_'))
                logger.info("[SupervisorImplementation::RestoreQueue] executionCode: ".concat(executionCode, ", Queue: ").concat(source, ", getting info:"), errorMessage);
            else
                logger.error("[SupervisorImplementation::RestoreQueue] executionCode: ".concat(executionCode, ", Queue: ").concat(source, ", getting error:"), errorMessage);
            return Promise.resolve(false);
        });
    };
    SupervisorImplementation.prototype.IdathaLogin = function (vcc, executionCode) {
        logger.info('[SupervisorImplementation::IdathaLogin] executionCode: ', executionCode, 'vcc: ', vcc);
        var userAcessToken = {};
        return (0, promises_shared_1.DummyPromise)()
            .then(function (result) {
            return (0, idatha_service_1.IdathaLogin)(vcc);
        }).then(function (result) {
            return Promise.resolve(result ? true : false);
        }).catch(function (err) {
            var errorMessage = typeof err === 'string' ? err : (err.message || err.description);
            logger.error('[SupervisorImplementation::IdathaLogin] executionCode:', executionCode, 'getting error:', errorMessage);
            return Promise.resolve(null);
        });
    };
    SupervisorImplementation.prototype.AzureSuscription = function (vcc, executionCode) {
        var isNew = false;
        return (0, promises_shared_1.DummyPromise)()
            .then(function (result) {
            return redis_service_1.Redis.GetSubscriptionKey(vcc);
        }).then(function (result) {
            if (!result) {
                isNew = true;
                return mssql_service_1.MSSql.GetSubscriptionKey(vcc);
            }
            return result;
        }).then(function (result) {
            if (result && isNew) {
                return redis_service_1.Redis.SaveSubscriptionKey(vcc, result);
            }
            else if (result) {
                return true;
            }
            return false;
        }).then(function (result) {
            return Promise.resolve(result ? true : false);
        }).catch(function (err) {
            var errorMessage = typeof err === 'string' ? err : (err.message || err.description);
            logger.error('[SupervisorImplementation::AzureSuscription] executionCode:', executionCode, 'getting error:', errorMessage);
            return Promise.resolve(null);
        });
    };
    SupervisorImplementation.prototype.UniqueVCC = function (vccs) {
        var result = [];
        if (vccs && vccs.length) {
            result = vccs.filter(function (vcc, pos) {
                return vccs.indexOf(vcc) === pos;
            });
        }
        return result;
    };
    return SupervisorImplementation;
}());
exports.SupervisorImplementation = SupervisorImplementation;
