"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.TranscriptionsUploadImplementation = void 0;
var jwt = require("jsonwebtoken");
var moment = require("moment");
var log4js = require("log4js");
var fs = require("fs");
var FormData = require('form-data');
var mssql_service_1 = require("../db/mssql.service");
var config_1 = require("../config/config");
var redis_service_1 = require("../db/redis.service");
var file_manager_1 = require("../shared/file-manager");
var promises_shared_1 = require("../shared/promises.shared");
var billing_implementation_1 = require("./billing.implementation");
var io_request_1 = require("../io/io.request");
var io_model_1 = require("../models/io.model");
var events_1 = require("../io/events");
var request_shared_1 = require("../shared/request.shared");
var server_config_1 = require("../config/server.config");
var jwt_config_1 = require("../config/jwt.config");
var audio_transfer_implementation_1 = require("./audio-transfer.implementation");
var http = require('http');
var https = require('https');
var logger = log4js.getLogger('Task');
var CURRENT_SYSTEM_SERVICE = 'TRANSCRIPTION';
var CURRENT_IMPLEMENTATION = 'TranscriptionsUploadImplementation';
var TranscriptionsUploadImplementation = (function () {
    function TranscriptionsUploadImplementation() {
        this.externalUserId = 'ext_user';
        this.externalUserVcc = 'system';
        this.availableSpeechmatics = config_1.SPEECHMATICS_SERVERS;
    }
    TranscriptionsUploadImplementation.prototype.Execute = function (executionCode, rateRecursive, availableBatch, cancelledBatch) {
        var _this = this;
        logger.info("[TranscriptionsUploadImplementation::Execute] executionCode: ".concat(executionCode, " executing wave #").concat(rateRecursive));
        var executeIndex = 1;
        var ratio = config_1.RATIO_TRANSCRIPTION;
        var alowedRatio = 0;
        var notAlowedRatio = 0;
        var executeResult = true;
        var currentBlockPromise;
        return (0, promises_shared_1.SleepPromise)(config_1.SLEEP_TRANSCRIPTION)
            .then(function (_) {
            currentBlockPromise = 'BLOCK_01_GET_FROM_REDIS';
            return redis_service_1.Redis.GetLengthFromList('TranscriptionsUpload');
        })
            .then(function (result) {
            currentBlockPromise = 'BLOCK_02_GET_SPEECHMATICS_FREE_STORAGE';
            if (!result)
                throw 'SERVER_INFO_REDIS_KEYS_FOR_TRANSCRIPTIONS_UPLOAD_NOT_FOUND';
            return _this.GetSpeechmaticsFreeStorage(executionCode);
        })
            .then(function (_) {
            currentBlockPromise = 'BLOCK_03_GET_QUEUE_ITEMS';
            if (!_this.availableSpeechmatics.length)
                throw 'SERVER_INFO_NO_SPEECHMATICS_WITH_FREE_STORAGE';
            return redis_service_1.Redis.LRange('TranscriptionsUpload', 0, ratio - 1);
        })
            .then(function (result) {
            var promiseArray = [];
            ratio = (result === null || result === void 0 ? void 0 : result.length) <= ratio ? result.length : 0;
            result.forEach(function (r) {
                var arrQueue = r.split(':');
                if (availableBatch.find(function (b) { return b.VCC === arrQueue[0] && b.BatchTypeId.toLowerCase() === arrQueue[1].toLowerCase(); })) {
                    alowedRatio++;
                }
                else {
                    notAlowedRatio++;
                }
            });
            while (ratio >= executeIndex) {
                promiseArray.push(_this.TranscribeAudioExecute(executionCode, availableBatch, cancelledBatch, executeIndex));
                executeIndex++;
            }
            if (!promiseArray.length)
                promiseArray.push(Promise.resolve(true));
            return Promise.all(promiseArray);
        })
            .then(function (results) {
            if (results.filter(function (item) { return item === false; }).length > notAlowedRatio)
                executeResult = false;
            return redis_service_1.Redis.LRange('TranscriptionsUpload', 0, ratio - 1);
        })
            .then(function (result) {
            if ((result === null || result === void 0 ? void 0 : result.length) && rateRecursive >= 1)
                return _this.Execute(executionCode, rateRecursive - 1, availableBatch, cancelledBatch);
            return Promise.resolve(executeResult);
        })
            .then(function (result) {
            if (rateRecursive === config_1.RATE_RECURSIVE_TRANSCRIPTION)
                logger.info("[TranscriptionsUploadImplementation::Execute] executionCode: ".concat(executionCode, " Finishing execution wave #").concat(rateRecursive));
            var currentResult = executeResult && result ? true : false;
            return Promise.resolve(currentResult);
        })
            .catch(function (error) {
            var errorMessage = typeof error === 'string' ? error : (error.message || error.description);
            if (errorMessage.startsWith('SERVER_INFO_'))
                logger.info('[TranscriptionsUploadImplementation::Execute] executionCode:', executionCode, ', Block:', currentBlockPromise, ', getting info:', errorMessage);
            else
                logger.error('[TranscriptionsUploadImplementation::Execute] executionCode:', executionCode, ', Block:', currentBlockPromise, ', getting error:', errorMessage);
            return Promise.resolve(false);
        });
    };
    TranscriptionsUploadImplementation.prototype.TranscribeAudioExecute = function (executionCode, availableBatch, cancelledBatch, index) {
        var _this = this;
        var storage = '';
        var currentQueueId = '';
        var processInteraction;
        var vcc = '';
        var processId = '';
        var interactionType = '';
        var interactionId = '';
        var interactionSection = 0;
        var interactionPart = 0;
        var currentBlockPromise = null;
        return redis_service_1.Redis.RemoveQueueL('TranscriptionsUpload')
            .then(function (queueId) {
            currentBlockPromise = 'BLOCK_01_GETTING_INTERACTION_DATA_FROM_REDIS_STORAGE';
            currentQueueId = queueId;
            var item = currentQueueId.split(':');
            if (item === null || item === void 0 ? void 0 : item.length) {
                vcc = item[0];
                processId = item[1];
                interactionType = item[2];
                interactionId = item[3];
                interactionSection = Number(item[4]);
                interactionPart = Number(item[5]);
                storage = vcc + ':' + processId;
                logger.info('[TranscriptionsUploadImplementation::TranscribeAudioExecute] executionCode:', executionCode, 'getting queueId:', currentQueueId, 'storage :', storage);
                return redis_service_1.Redis.GetProcessInteraction(storage, currentQueueId);
            }
            else {
                logger.error('[TranscriptionsUploadImplementation::TranscribeAudioExecute] executionCode:', executionCode, 'getting queueId:', currentQueueId, 'QUEUE_FORMAT_ERROR');
                return Promise.resolve(null);
            }
        })
            .then(function (result) {
            currentBlockPromise = 'BLOCK_02_SETTING_INTERACTION_DATA';
            if (result) {
                processInteraction = result;
                logger.info('[TranscriptionsUploadImplementation::TranscribeAudioExecute] executionCode:', executionCode, 'getting queueId:', currentQueueId, 'getting GetProcessInteraction: 1, status:', processInteraction.statusDescription);
                return Promise.resolve(true);
            }
            else {
                if (currentQueueId)
                    logger.warn('[TranscriptionsUploadImplementation::TranscribeAudioExecute] executionCode:', executionCode, 'getting queueId:', currentQueueId, 'getting GetProcessInteraction: 0');
                return Promise.resolve(null);
            }
        })
            .then(function (result) {
            currentBlockPromise = 'BLOCK_03_VALIDATE_ALLOWABLE';
            var arrQueue = currentQueueId.split(':');
            var promiseArray = [];
            if (result) {
                if (availableBatch.find(function (b) { return b.VCC === arrQueue[0] && b.BatchTypeId.toLowerCase() === processInteraction.batchTypeId.toLowerCase(); })) {
                    promiseArray.push(Promise.resolve('ALLOWED'));
                    promiseArray.push(redis_service_1.Redis.CreateQueueL('TranscriptionsUploadExecute', currentQueueId));
                    promiseArray.push(Promise.resolve(currentQueueId));
                }
                else {
                    promiseArray.push(Promise.resolve('NOT_ALLOWED'));
                    promiseArray.push(redis_service_1.Redis.CreateQueueR('TranscriptionsUpload', currentQueueId));
                    promiseArray.push(mssql_service_1.MSSql.SaveBatchStatus(arrQueue[0], arrQueue[1], 'TRANSCRIPTION', 'PAUSED'));
                    promiseArray.push((0, io_request_1.SendIOEvents)([
                        new io_model_1.IOEvent(io_model_1.VCC_EVENT, events_1.UPDATE_TRANSCRIPTION, null, arrQueue[0])
                    ]));
                }
            }
            if (!promiseArray.length)
                promiseArray.push(Promise.resolve(true));
            return Promise.all(promiseArray);
        })
            .then(function (result) {
            currentBlockPromise = 'BLOCK_04_EXECUTE_MAIN_FUNCTION';
            if (result && result[0] === 'NOT_ALLOWED')
                throw 'SERVER_ERROR_NOT_ALLOWED_QUEUE';
            if (processInteraction) {
                if (processInteraction.statusDescription === 'pendingUpload') {
                    if (processInteraction.interactionType === 'CALL') {
                        if (!processInteraction.speechmaticsApi
                            ||
                                processInteraction.status === -3)
                            processInteraction.speechmaticsApi = _this.GetSpeechmaticsApi(index - 1);
                        return _this.UploadAudio(processInteraction, currentQueueId, executionCode);
                    }
                    else {
                        processInteraction.status = 3;
                        processInteraction.statusDescription = config_1.USAGE_SENTIMENT_ANALYSIS ? 'pendingSentimentAnalysis' : 'pendingAnalysis';
                    }
                }
                else
                    logger.error("[TranscriptionsUploadImplementation::TranscribeAudioExecute] executionCode: ".concat(executionCode, " getting queueId: ").concat(currentQueueId, ", Received invalid status: ").concat(processInteraction.statusDescription));
            }
            return Promise.resolve(null);
        })
            .then(function (result) {
            currentBlockPromise = 'BLOCK_05_SAVING_UPDATED_DATA_ON_REDIS';
            if (processInteraction)
                return redis_service_1.Redis.SaveProcessInteraction(storage, currentQueueId, processInteraction);
            return Promise.resolve(null);
        })
            .then(function (result) {
            currentBlockPromise = 'BLOCK_06_SENDING_TO_NEXT_QUEUE';
            var promiseArray = [];
            var billing = new billing_implementation_1.BillingImplementation();
            if (result
                &&
                    (processInteraction === null || processInteraction === void 0 ? void 0 : processInteraction.status) === 3
                &&
                    ['pendingTranscription', 'pendingSentimentAnalysis', 'pendingAnalysis'].includes(processInteraction === null || processInteraction === void 0 ? void 0 : processInteraction.statusDescription)
                &&
                    (processInteraction === null || processInteraction === void 0 ? void 0 : processInteraction.retry) < config_1.RETRY_TRANSCRIPTION) {
                if (processInteraction.interactionType === 'CALL' && config_1.SPEECHMATICS_CALLBACK_USAGE)
                    promiseArray.push(redis_service_1.Redis.InsertQueue('TranscriptionsStandby', currentQueueId));
                else
                    promiseArray.push(redis_service_1.Redis.InsertQueue('TranscriptionsDownload', currentQueueId, config_1.RATE_RECURSIVE_TRANSCRIPTION * config_1.RATIO_TRANSCRIPTION * 2));
                promiseArray.push(redis_service_1.Redis.DeleteQueue('TranscriptionsUploadExecute', currentQueueId));
            }
            else if (result
                &&
                    (processInteraction === null || processInteraction === void 0 ? void 0 : processInteraction.status) === -3
                &&
                    (processInteraction === null || processInteraction === void 0 ? void 0 : processInteraction.statusDescription) === 'REMOVED_BY_CANCELLED') {
                logger.error('[TranscriptionsUploadImplementation::TranscribeAudioExecute] executionCode:', executionCode, 'getting queueId:', currentQueueId, 'REMOVED_BY_CANCELLED');
                billing.AddCloseTranscriptionDetailToMainFile(processInteraction, processInteraction.batchTypeName, processInteraction.batchTypeId, 0, 'REMOVED_BY_CANCELLED');
                promiseArray.push(redis_service_1.Redis.DeleteQueue('TranscriptionsUploadExecute', currentQueueId));
            }
            else if ((processInteraction === null || processInteraction === void 0 ? void 0 : processInteraction.status) === -3
                &&
                    (processInteraction === null || processInteraction === void 0 ? void 0 : processInteraction.errorMessage) === 'UPLOAD_ERROR'
                &&
                    (processInteraction === null || processInteraction === void 0 ? void 0 : processInteraction.retry) >= 500) {
                logger.error('[TranscriptionsUploadImplementation::TranscribeAudioExecute] executionCode:', executionCode, 'getting queueId:', currentQueueId, 'UPLOAD_ERROR');
                billing.AddCloseTranscriptionDetailToMainFile(processInteraction, processInteraction.batchTypeName, processInteraction.batchTypeId, 0, 'UPLOAD_ERROR');
                promiseArray.push(redis_service_1.Redis.DeleteQueue('TranscriptionsUploadExecute', currentQueueId));
                processInteraction.retry = config_1.RETRY_TRANSCRIPTION;
            }
            else if ((processInteraction === null || processInteraction === void 0 ? void 0 : processInteraction.retry) >= config_1.RETRY_TRANSCRIPTION) {
                logger.error('[TranscriptionsUploadImplementation::TranscribeAudioExecute] executionCode:', executionCode, 'getting queueId:', currentQueueId, 'REMOVED_BY_RETRIES');
                billing.AddCloseTranscriptionDetailToMainFile(processInteraction, processInteraction.batchTypeName, processInteraction.batchTypeId, 0, 'REMOVED_BY_RETRIES');
                promiseArray.push(redis_service_1.Redis.DeleteQueue('TranscriptionsUploadExecute', currentQueueId));
                processInteraction.retry = config_1.RETRY_TRANSCRIPTION;
            }
            else if (currentQueueId && !processInteraction) {
                logger.error('[TranscriptionsUploadImplementation::TranscribeAudioExecute] executionCode:', executionCode, 'getting queueId:', currentQueueId, 'INTERACTION_DO_NOT_EXIST');
                var interactionData = {
                    vcc: vcc,
                    processId: processId,
                    id: interactionId,
                    section: interactionSection,
                    part: interactionPart,
                    retry: config_1.RETRY_TRANSCRIPTION
                };
                var audioTransfer = new audio_transfer_implementation_1.AudioTransferImplementation();
                audioTransfer.CloseTranscriptionDetailWithMissingData(billing, interactionData, 'TranscriptionsUploadImplementation');
                promiseArray.push(redis_service_1.Redis.DeleteQueue('TranscriptionsUploadExecute', currentQueueId));
            }
            else {
                logger.error('[TranscriptionsUploadImplementation::TranscribeAudioExecute] executionCode:', executionCode, 'getting queueId:', currentQueueId, 'QUEUE_WAS_NOT_REMOVED');
            }
            if (!promiseArray.length)
                promiseArray.push(Promise.resolve(true));
            return Promise.all(promiseArray);
        })
            .then(function (result) {
            currentBlockPromise = 'BLOCK_07_UPDATING_PROGRESS';
            var promiseArray = [];
            if (processInteraction && processInteraction.retry >= config_1.RETRY_TRANSCRIPTION) {
                processInteraction.isCancel = true;
                promiseArray.push(redis_service_1.Redis.UpdateProgress(storage));
                promiseArray.push(redis_service_1.Redis.UpdateErrorProgress(storage));
                promiseArray.push(redis_service_1.Redis.DeleteProcessInteraction(storage, currentQueueId));
                promiseArray.push(redis_service_1.Redis.SaveReadyProcessInteraction(executionCode, currentQueueId, processInteraction));
                promiseArray.push(mssql_service_1.MSSql.SaveBatchError(vcc, CURRENT_SYSTEM_SERVICE, processInteraction.batchTypeId, processInteraction.batchTypeId, CURRENT_IMPLEMENTATION, processInteraction.id, null, processInteraction.errorFunction, processInteraction.errorMessage, processInteraction.errorDetail));
            }
            if (!promiseArray.length)
                promiseArray.push(Promise.resolve(true));
            return Promise.all(promiseArray);
        })
            .then(function (result) {
            currentBlockPromise = 'BLOCK_08_FINISHING';
            return Promise.resolve(true);
        })
            .catch(function (err) {
            var errorMessage = typeof err === 'string' ? err : (err.message || err.description);
            logger.error('[TranscriptionsUploadImplementation::TranscribeAudioExecute] executionCode:', executionCode, 'getting queueId:', currentQueueId, ', Block:', currentBlockPromise, ', getting error:', errorMessage);
            return Promise.resolve(false);
        });
    };
    TranscriptionsUploadImplementation.prototype.UploadAudio = function (interaction, currentQueueId, executionCode) {
        var _this = this;
        var fullPath = '';
        var dictionaryContent = [];
        return (0, promises_shared_1.DummyPromise)()
            .then(function (_) {
            return _this.GetDictionaryContent(interaction, currentQueueId, executionCode);
        })
            .then(function (result) {
            dictionaryContent = result;
            return new Promise(function (resolve, reject) {
                var baseFolder = (['ftp', 'sftp', 's3'].includes(interaction.environmentType)) ? config_1.PATH_FOLDER_PROCESS_AUDIO : config_1.PATH_FOLDER_TEMP_AUDIO;
                fullPath = "".concat(baseFolder, "/").concat(interaction.vcc, "/").concat(interaction.fileName);
                logger.info('[TranscriptionsUploadImplementation::UploadAudio] executionCode:', executionCode, 'getting queueId:', currentQueueId, 'audio:', fullPath);
                fs.stat(fullPath, function (err, stat) {
                    resolve({ fullPath: fullPath, stat: stat });
                });
            });
        })
            .then(function (result) {
            if (result && result.stat && result.stat.size > 0) {
                return (0, file_manager_1.BufferFile)(result.fullPath);
            }
            else {
                logger.error('[TranscriptionsUploadImplementation::UploadAudio] executionCode:', executionCode, 'getting queueId:', currentQueueId, 'Audio file not found on fullPath:', fullPath);
                throw 'SERVER_ERROR_AUDIO_FILE_NOT_FOUND';
            }
        })
            .then(function (stream) {
            if (stream) {
                logger.debug('[TranscriptionsUploadImplementation::UploadAudio] executionCode:', executionCode, 'getting queueId:', currentQueueId, 'Process language:', interaction.language, 'READY_TO_UPLOAD');
                var trancriptionConfig = {
                    type: 'transcription',
                    transcription_config: {
                        diarization: 'channel',
                        channel_diarization_labels: ['Agent', 'Caller'],
                        language: interaction.language,
                        additional_vocab: dictionaryContent
                    }
                };
                if (config_1.SPEECHMATICS_CALLBACK_USAGE) {
                    trancriptionConfig.notification_config = [{
                            url: "".concat(server_config_1.WEBHOOK_URL, "/speechmatics/webhook/"),
                            contents: ['transcript', 'data'],
                            auth_headers: [
                                "Authorization: Bearer ".concat(_this.GetSystemUserToken()),
                                "speech: ".concat(server_config_1.EXTERNAL_URL, "/api/public/task/update_transcription_queue/").concat(currentQueueId, "/")
                            ]
                        }];
                }
                var formData = new FormData();
                formData.append('data_file', stream, interaction.fileName);
                formData.append('config', JSON.stringify(trancriptionConfig));
                var data = {
                    uri: "http://".concat(interaction.speechmaticsApi, "/v2/jobs/"),
                    fileName: interaction.fileName,
                    validateErrorResponse: true,
                    timeout: 600000
                };
                return (0, request_shared_1.AttachedFilePostRequest)(data, formData);
            }
            return Promise.resolve(null);
        })
            .then(function (jobStatus) {
            var jobId = jobStatus && jobStatus.id ? jobStatus.id : null;
            if (jobId) {
                logger.info('[TranscriptionsUploadImplementation::UploadAudio] executionCode:', executionCode, 'getting queueId:', currentQueueId, 'UPLOADED');
                interaction['status'] = 3;
                interaction['statusDescription'] = 'pendingTranscription';
                interaction['retry'] = 0;
                interaction['jobId'] = jobId;
                interaction['uploadedDate'] = moment().format('x');
                interaction['errorFunction'] = null;
                interaction['errorMessage'] = null;
                interaction['errorDetail'] = null;
            }
            else {
                logger.error('[TranscriptionsUploadImplementation::UploadAudio] executionCode:', executionCode, 'getting queueId:', currentQueueId, 'JOBID_IS_NOT_FOUND');
                interaction['status'] = -3;
                interaction['retry'] = interaction['retry'] ? interaction['retry'] + 1 : 1;
                interaction['errorFunction'] = 'UploadAudio';
                interaction['errorMessage'] = 'JOBID_IS_NOT_FOUND';
                interaction['errorDetail'] = 'JOBID_IS_NOT_FOUND';
            }
            return Promise.resolve(interaction);
        })
            .catch(function (err) {
            var errorMessage = typeof err === 'string' ? err : (err.message || err.description);
            logger.error('[TranscriptionsUploadImplementation::UploadAudio] executionCode:', executionCode, 'getting queueId:', currentQueueId, 'getting error:', errorMessage);
            if (typeof errorMessage === 'string'
                &&
                    errorMessage.includes('data_file is too small')) {
                interaction['isCancel'] = true;
                interaction['retry'] = config_1.RETRY_TRANSCRIPTION;
            }
            else
                interaction['retry'] = interaction['retry'] ? interaction['retry'] + 1 : 1;
            interaction['status'] = -3;
            interaction['errorFunction'] = 'UploadAudio';
            interaction['errorMessage'] = 'UPLOAD_ERROR';
            interaction['errorDetail'] = err.stack ? err.stack : errorMessage;
            return Promise.resolve(interaction);
        });
    };
    TranscriptionsUploadImplementation.prototype.GetDictionaryContent = function (interaction, currentQueueId, executionCode) {
        var dictionaryWords = [];
        var dictionaryContent = [];
        return (0, promises_shared_1.DummyPromise)()
            .then(function (result) {
            return mssql_service_1.MSSql.GetDictionaryWords(interaction.vcc, null, null, interaction.dictionaryId);
        })
            .then(function (result) {
            if (!result)
                throw 'SERVER_ERROR_CANNOT_GET_DICTIONARY_WORDS';
            dictionaryWords = result;
            var dictionaryWordIds = [];
            dictionaryWords.forEach(function (dw) { return dictionaryWordIds.push(dw.id); });
            return mssql_service_1.MSSql.GetDictionaryWordSoundLikes(interaction.vcc, dictionaryWordIds.join('\',\''));
        })
            .then(function (result) {
            logger.info('[TranscriptionsUploadImplementation::GetDictionaryContent] executionCode:', executionCode, 'getting queueId:', currentQueueId, 'found words:', dictionaryWords.length);
            dictionaryWords.forEach(function (dw) {
                dw.soundLikes = [];
                dw.soundLikes = result.filter(function (r) { return r.dictionaryWordId === dw.id; });
            });
            dictionaryWords.forEach(function (dw) {
                if (dw.soundLikes && dw.soundLikes.length) {
                    var soundLikeWords_1 = [];
                    dw.soundLikes.forEach(function (sl) { return soundLikeWords_1.push(sl.soundLike); });
                    dictionaryContent.push({
                        content: dw.word,
                        sounds_like: soundLikeWords_1
                    });
                }
                else {
                    dictionaryContent.push(dw.word);
                }
            });
            return Promise.resolve(dictionaryContent);
        })
            .catch(function (err) {
            var errorMessage = typeof err === 'string' ? err : (err.message || err.description);
            logger.error('[TranscriptionsUploadImplementation::GetDictionaryContent] executionCode:', executionCode, 'getting queueId:', currentQueueId, 'getting error:', errorMessage);
            return Promise.resolve([]);
        });
    };
    TranscriptionsUploadImplementation.prototype.GetSpeechmaticsFreeStorage = function (executionCode) {
        var _this = this;
        return (0, promises_shared_1.DummyPromise)()
            .then(function (_) {
            var promiseArray = [];
            _this.availableSpeechmatics.forEach(function (sa) {
                var getOptions = {
                    uri: "http://".concat(sa.ip, ":8080/v1/management/freestorage"),
                    timeout: 30000
                };
                promiseArray.push((0, request_shared_1.SimpleGetRequest)(getOptions));
            });
            return Promise.all(promiseArray);
        })
            .then(function (result) {
            result.forEach(function (r, i) {
                var _a;
                if ((_a = r === null || r === void 0 ? void 0 : r.items) === null || _a === void 0 ? void 0 : _a.length) {
                    var minimunStorage = (2 * 1024);
                    var freeStorage = r.items[0].bytes / (1024 * 1024);
                    _this.availableSpeechmatics[i].available = freeStorage > minimunStorage;
                    logger.info("[TranscriptionsUploadImplementation::GetSpeechmaticsFreeStorage] executionCode: ".concat(executionCode, " free storage for ").concat(_this.availableSpeechmatics[i].ip, " => ").concat(freeStorage, "mb => ").concat(_this.availableSpeechmatics[i].available ? 'OK' : 'KO'));
                }
            });
            _this.availableSpeechmatics = _this.availableSpeechmatics.filter(function (as) { return as.available; });
            return Promise.resolve(true);
        })
            .catch(function (err) {
            var errorMessage = typeof err === 'string' ? err : (err.message || err.description);
            logger.error('[TranscriptionsUploadImplementation::GetSpeechmaticsFreeStorage] executionCode:', executionCode, 'getting error:', errorMessage);
            return Promise.resolve(false);
        });
    };
    TranscriptionsUploadImplementation.prototype.GetSpeechmaticsApi = function (index) {
        var apiIp = '';
        var apiIndex = index % this.availableSpeechmatics.length;
        var server = this.availableSpeechmatics[apiIndex];
        apiIp = "".concat(server.ip, ":").concat(server.port);
        return apiIp;
    };
    TranscriptionsUploadImplementation.prototype.GetSystemUserToken = function () {
        var token = jwt.sign({ userId: this.externalUserId, vcc: this.externalUserVcc }, jwt_config_1.JWT_SECRET, { expiresIn: jwt_config_1.JWT_EXPIRATION_TIME });
        return token;
    };
    return TranscriptionsUploadImplementation;
}());
exports.TranscriptionsUploadImplementation = TranscriptionsUploadImplementation;
