"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.GetAudioFileFromIntegrated = exports.GetAudioFileFromAwsS3 = exports.GetAudioFileFromSftp = exports.GetAudioFileFromFtp = exports.GetIdathaDocument = exports.AudioService = void 0;
var moment = require("moment");
var log4js = require("log4js");
var Client = require('ftp');
var SSH2Promise = require("ssh2-promise");
var mssql_service_1 = require("../db/mssql.service");
var file_manager_1 = require("../shared/file-manager");
var promises_shared_1 = require("../shared/promises.shared");
var config_1 = require("../config/config");
var config_2 = require("../config/config");
var integration_config_1 = require("../config/integration.config");
var utilities_1 = require("../shared/utilities");
var idatha_service_1 = require("./idatha.service");
var aws_s3_implementation_1 = require("../implementation/aws-s3.implementation");
var diarization_analysis_implementation_1 = require("../implementation/diarization-analysis.implementation");
var semantic_analysis_implementation_1 = require("../implementation/semantic-analysis.implementation");
var sentiment_analysis_implementation_1 = require("../implementation/sentiment-analysis.implementation");
var mssql_config_1 = require("../config/mssql.config");
var http_shared_1 = require("../shared/http.shared");
var request_shared_1 = require("../shared/request.shared");
var http_request_occ_1 = require("../shared/http-request-occ");
var fs = require("fs");
var child_process_1 = require("child_process");
var logger = log4js.getLogger('ServerScripts');
function AudioService(app) {
    app.get('/api/process/audio/get_audio/:processId/:id?', function (req, res, next) {
        logger.info('Se recibió ' + req.method + ' para \'' + req.path + '\'');
        var vcc = res.locals.vcc;
        var processId = req.params.processId;
        var id = req.params.id;
        (0, promises_shared_1.DummyPromise)()
            .then(function (result) {
            if (processId !== 'null') {
                var file_1 = config_2.PATH_FOLDER_PROCESS_AUDIO_PLAY + '\/' + processId + '\/' + id + '.mp3';
                return (0, file_manager_1.BufferFile)(file_1)
                    .then(function (result) {
                    return Promise.resolve(result);
                })
                    .catch(function (err) {
                    file_1 = config_2.PATH_FOLDER_PROCESS_AUDIO_PLAY + '\/' + processId + '\/' + id + '.wav';
                    return (0, file_manager_1.BufferFile)(file_1)
                        .then(function (result) {
                        return Promise.resolve(result);
                    }).catch(function (err) {
                        file_1 = config_2.PATH_FOLDER_PROCESS_AUDIO_PLAY + '\/' + processId + '\/' + id + '.ogg';
                        return (0, file_manager_1.BufferFile)(file_1)
                            .then(function (result) {
                            return Promise.resolve(result);
                        });
                    });
                });
            }
            else {
                var url = id;
                url = Buffer.from(url, 'base64').toString();
                return ExternalRequest(url);
            }
        }).then(function (result) {
            res.json({ result: result });
        }).catch(function (err) {
            var clientMsg = (typeof err === 'string' ? err : 'CANNOT_PROCESS_REQUEST');
            var errorMsg = (typeof err === 'string' ? err : err.message || err.description || 'SERVER_ERROR_UNKNOWN_MESSAGE');
            logger.error('Error al procesar \'' + req.path + '\': ' + errorMsg);
            if (res.headersSent)
                return;
            res.json({ status: false, description: clientMsg, error: errorMsg, detail: { type: 'ERROR', datetime: moment().utc().toDate(), pid: process.pid, source: __filename, stack: err.stack } });
        });
    });
    app.get('/api/process/audio/get_audio_navigator_interactions/:processId', function (req, res) {
        var vcc = res.locals.vcc;
        var processId = req.params.processId;
        (0, promises_shared_1.DummyPromise)().then(function (_) {
            return mssql_service_1.MSSql.GetAudioNavigatorInteractions(vcc, processId);
        })
            .then(function (interactions) {
            res.json({ status: true, data: interactions });
        })
            .catch(function (err) {
            var clientMsg = (typeof err === 'string' ? err : 'CANNOT_PROCESS_REQUEST');
            var errorMsg = (typeof err === 'string' ? err : err.message || err.description || 'SERVER_ERROR_UNKNOWN_MESSAGE');
            logger.error('Error al procesar \'' + req.path + '\': ' + errorMsg);
            if (res.headersSent)
                return;
            res.json({ status: false, description: clientMsg, error: errorMsg });
        });
    });
    app.post('/api/process/audio/get_audios_process', function (req, res, next) {
        logger.info('[AudioService::get_audios_process] Received ' + req.method + ' method on \'' + req.path + '\'');
        var vcc = res.locals.vcc;
        var newAudio = (req.body || {});
        (0, promises_shared_1.DummyPromise)()
            .then(function (result) {
            var times = getTimes(newAudio.startTime, newAudio.endTime);
            return mssql_service_1.MSSql.GetAudiosProcess(newAudio.pageSize, newAudio.pageNumber, vcc, newAudio.processId, newAudio.id, newAudio.startDate, newAudio.endDate, times.startTime.hour, times.startTime.minute, times.endTime.hour, times.endTime.minute, newAudio.campaign, newAudio.agent, newAudio.resultCode, newAudio.type, newAudio.idathaResults, newAudio.totalIdathaResults, newAudio.interactionType, (newAudio.interactionIds || []).toString(), newAudio.folder, newAudio.contactAddress, newAudio.sentimentIds);
        }).then(function (result) {
            if (res.headersSent)
                return;
            res.json({ status: true, description: 'SERVER_SUCCESS_MESSAGE', data: result });
        }).catch(function (err) {
            var clientMsg = (typeof err === 'string' ? err : 'CANNOT_PROCESS_REQUEST');
            var errorMsg = (typeof err === 'string' ? err : err.message || err.description || 'SERVER_ERROR_UNKNOWN_MESSAGE');
            logger.error('[AudioService::get_audios_process] Error on \'' + req.path + '\': ' + errorMsg);
            if (res.headersSent)
                return;
            res.json({ status: false, description: clientMsg, error: errorMsg, detail: { type: 'ERROR', datetime: moment().utc().toDate(), pid: process.pid, source: __filename, stack: err.stack } });
        });
    });
    app.post('/api/process/audio/get_audios_transcriptions', function (req, res, next) {
        logger.info('Se recibió ' + req.method + ' para \'' + req.path + '\'');
        var vcc = res.locals.vcc;
        var newAudio = (req.body || {});
        (0, promises_shared_1.DummyPromise)()
            .then(function (result) {
            var times = getTimes(newAudio.startTime, newAudio.endTime);
            var interactionIds = (newAudio.interactionIds || []).toString();
            return mssql_service_1.MSSql.GetAudiosTranscriptions(newAudio.pageSize, newAudio.pageNumber, vcc, newAudio.processId, newAudio.id, newAudio.startDate, newAudio.endDate, times.startTime.hour, times.startTime.minute, times.endTime.hour, times.endTime.minute, newAudio.campaign, newAudio.agent, newAudio.resultCode, interactionIds, newAudio.contactAddress, newAudio.sentimentIds);
        }).then(function (result) {
            if (res.headersSent)
                return;
            res.json({ status: true, description: 'SERVER_SUCCESS_MESSAGE', data: result });
        }).catch(function (err) {
            var clientMsg = (typeof err === 'string' ? err : 'CANNOT_PROCESS_REQUEST');
            var errorMsg = (typeof err === 'string' ? err : err.message || err.description || 'SERVER_ERROR_UNKNOWN_MESSAGE');
            logger.error('Error al procesar \'' + req.path + '\': ' + errorMsg);
            if (res.headersSent)
                return;
            res.json({ status: false, description: clientMsg, error: errorMsg, detail: { type: 'ERROR', datetime: moment().utc().toDate(), pid: process.pid, source: __filename, stack: err.stack } });
        });
    });
    app.get('/api/process/allegro/get_allegro_audios_statistic/:id', function (req, res, next) {
        logger.info('Se recibió ' + req.method + ' para \'' + req.path + '\'');
        var vcc = res.locals.vcc;
        var id = req.params.id;
        (0, promises_shared_1.DummyPromise)()
            .then(function (result) {
            return mssql_service_1.MSSql.GetAllegroAudiosStatistic(vcc, id);
        }).then(function (result) {
            if (res.headersSent)
                return;
            res.json({ status: true, description: 'SERVER_SUCCESS_MESSAGE', data: result });
        }).catch(function (err) {
            var clientMsg = (typeof err === 'string' ? err : 'CANNOT_PROCESS_REQUEST');
            var errorMsg = (typeof err === 'string' ? err : err.message || err.description || 'SERVER_ERROR_UNKNOWN_MESSAGE');
            logger.error('Error al procesar \'' + req.path + '\': ' + errorMsg);
            if (res.headersSent)
                return;
            res.json({ status: false, description: clientMsg, error: errorMsg, detail: { type: 'ERROR', datetime: moment().utc().toDate(), pid: process.pid, source: __filename, stack: err.stack } });
        });
    });
    app.get('/api/process/audio/get_audio_transcription/:process_id/:audio_id/:section/:part', function (req, res, next) {
        req.setTimeout(300000, function () { });
        logger.info('[AudioService::get_audio_transcription] Received ' + req.method + ' method on \'' + req.path + '\'');
        var vcc = res.locals.vcc;
        var processId = ['null', 'undefined'].includes(req.params.process_id) ? null : req.params.process_id;
        var interactionType = req.params.interactionType;
        var audioId = req.params.audio_id;
        var audioData = {};
        var section = Number(req.params.section) * 1;
        var part = Number(req.params.part) * 1;
        var environment = null;
        var _resultReadFileContent = null;
        var _transcriptionQualification = 0;
        var buffer = null;
        var resultAudio;
        var interactionIdData;
        var fileNameAudio = '';
        var fileNameAnalysisSemantic = '';
        var fileNameAnalysisSemanticView = '';
        var jsonFileName = '';
        var isRemote = false;
        var fileNameAudioSaveFtpSftp = null;
        (0, promises_shared_1.DummyPromise)()
            .then(function (result) {
            if (processId === null) {
                interactionIdData = (0, utilities_1.getInteractionIdFromURLBase64)(audioId);
                logger.info('[AudioService::get_audio_transcription] Getting interactionIdData:', interactionIdData, 'with req.params:', req.params, 'with vcc:', vcc, 'with resultAudio:', resultAudio);
                if (interactionIdData) {
                    audioId = interactionIdData.id ? interactionIdData.id : audioId;
                    section = section || section === 0 ? section : interactionIdData.section;
                    part = part || part === 0 ? part : interactionIdData.part;
                }
                else {
                    throw 'INVALID_AUDIO_DATA';
                }
            }
            return mssql_service_1.MSSql.GetAudio(vcc, processId, audioId, section, part);
        }).then(function (result) {
            resultAudio = result;
            if (resultAudio) {
                processId = processId ? processId : result.processId;
                audioData = {
                    audioId: audioId,
                    relativePath: resultAudio.relativePath,
                    fileName: resultAudio.fileName,
                    relativeFileName: resultAudio.relativeFileName
                };
                return mssql_service_1.MSSql.GetProcess(vcc, processId);
            }
            else {
                return mssql_service_1.MSSql.GetEvaluationPlanByInteractionId(vcc, audioId);
            }
        }).then(function (result) {
            if (!result)
                throw 'SERVER_ERROR_AUDIO_SYSTEM_SERVICE_NOT_FOUND';
            if (result.relativePath !== undefined && result.fileName !== undefined) {
                audioData = {
                    audioId: audioId,
                    relativePath: result.relativePath,
                    fileName: result.fileName,
                    relativeFileName: result.relativeFileName
                };
            }
            return mssql_service_1.MSSql.GetEnvironment(vcc, (result.environment || result.environmentId));
        }).then(function (result) {
            if (!result)
                throw 'ENVIRONMENT_NOT_FOUND';
            environment = result;
            var environmentType = null;
            environmentType = result.type;
            if (['ftp', 'sftp', 'i-ftp', 'i-sftp', 's3'].includes(result.type)) {
                fileNameAudioSaveFtpSftp = GetAudioFileName(vcc, (resultAudio === null || resultAudio === void 0 ? void 0 : resultAudio.fileName) ? resultAudio.fileName : null, audioId, section, part, null, 'LOCAL');
                audioData.fileName = fileNameAudioSaveFtpSftp;
                audioData.section = section;
                audioData.part = part;
            }
            return GetAudioFileFromIntegrated(vcc, environmentType, audioId, section, part, (resultAudio === null || resultAudio === void 0 ? void 0 : resultAudio.fileName) ? resultAudio.fileName : null, null, 'LOCAL');
        }).catch(function (err) {
            if (typeof err === 'string'
                &&
                    [
                        'INVALID_AUDIO_DATA',
                        'SERVER_ERROR_AUDIO_SYSTEM_SERVICE_NOT_FOUND',
                        'ENVIRONMENT_NOT_FOUND'
                    ].includes(err))
                throw err;
            return Promise.resolve(null);
        }).then(function (result) {
            if (result) {
                return Promise.resolve(result);
            }
            else {
                isRemote = true;
                if (['allegro', 'i6'].includes(environment.type))
                    return GetAudioFileFromIntegrated(vcc, environment.type, audioId, section, part, (resultAudio === null || resultAudio === void 0 ? void 0 : resultAudio.fileName) ? resultAudio.fileName : null, null, null);
                else if (['ftp', 'i-ftp'].includes(environment.type))
                    return GetAudioFileFromFtp(vcc, audioData, environment);
                else if (['sftp', 'i-sftp'].includes(environment.type))
                    return GetAudioFileFromSftp(vcc, audioData, environment);
                else if (['s3'])
                    return GetAudioFileFromAwsS3(vcc, audioData, environment);
            }
            return Promise.resolve(true);
        }).catch(function (err) {
            if (typeof err === 'string'
                &&
                    [
                        'INVALID_AUDIO_DATA',
                        'SERVER_ERROR_AUDIO_SYSTEM_SERVICE_NOT_FOUND',
                        'ENVIRONMENT_NOT_FOUND'
                    ].includes(err))
                throw err;
            return Promise.resolve(null);
        }).then(function (result) {
            if (isRemote
                &&
                    (result === null || result === void 0 ? void 0 : result.buffer)
                &&
                    Buffer.isBuffer(result === null || result === void 0 ? void 0 : result.buffer)) {
                return SaveAndConvertAudioFile(result, environment.type, vcc, audioData, fileNameAudioSaveFtpSftp);
            }
            else {
                return Promise.resolve(result);
            }
        })
            .then(function (result) {
            logger.info("[AudioService::get_audio_transcription] Receiving audio from ".concat(environment.type));
            if (typeof result !== 'boolean') {
                if ((result === null || result === void 0 ? void 0 : result.audioFile) && typeof result.audioFile === 'string') {
                    fileNameAudio = (result === null || result === void 0 ? void 0 : result.audioFile) ? result.audioFile.split('/').pop() : null;
                }
                else {
                    fileNameAudio = null;
                }
                buffer = (result === null || result === void 0 ? void 0 : result.buffer) ? result.buffer : null;
                if (resultAudio) {
                    if (resultAudio.fileName === null) {
                        var fileName = resultAudio.id;
                        if (resultAudio.section !== '' && resultAudio.part !== '') {
                            fileName += '_';
                            fileName += section;
                            fileName += '_';
                            fileName += part;
                        }
                        fileName += '.json';
                        resultAudio.fileName = fileName;
                    }
                    var regex = /\.(mp3|wav|ogg|json)$/;
                    jsonFileName = resultAudio.fileName.replace(regex, '.json');
                    fileNameAnalysisSemantic = config_1.SEMANCTIC_ANALYSIS_FOLDER;
                    fileNameAnalysisSemantic += '/';
                    fileNameAnalysisSemantic += vcc;
                    fileNameAnalysisSemantic += '/';
                    fileNameAnalysisSemantic += jsonFileName;
                    fileNameAnalysisSemanticView = config_1.SEMANCTIC_ANALYSIS_VIEW_FOLDER;
                    fileNameAnalysisSemanticView += '/';
                    fileNameAnalysisSemanticView += vcc;
                    fileNameAnalysisSemanticView += '/';
                    fileNameAnalysisSemanticView += jsonFileName;
                    logger.info('[AudioService::get_audio_transcription] Getting resultAudio:', resultAudio, 'before ExportAudioTranscription with req.params:', req.params, 'with vcc:', vcc, 'with resultAudio:', resultAudio);
                    logger.debug('[AudioService::get_audio_transcription] Searching raw transcription:', fileNameAnalysisSemantic);
                    logger.debug('[AudioService::get_audio_transcription] Searching view transcription:', fileNameAnalysisSemanticView);
                    if (resultAudio.init === null && resultAudio.isTranscribed === 1 && !resultAudio.isCancel) {
                        if (resultAudio.retranscription && (0, file_manager_1.ExistInFileSystem)(fileNameAnalysisSemanticView))
                            (0, file_manager_1.DeleteFile)(fileNameAnalysisSemanticView);
                        return ExportAudioTranscription(vcc, resultAudio, fileNameAnalysisSemantic, fileNameAnalysisSemanticView);
                    }
                }
            }
            return Promise.resolve(true);
        }).then(function (result) {
            logger.info('[AudioService::get_audio_transcription] Getting result:', result, 'with req.params:', req.params, 'with vcc:', vcc, 'with resultAudio:', resultAudio);
            if (fileNameAnalysisSemanticView && fileNameAnalysisSemanticView.length > 0) {
                return (0, file_manager_1.ReadFileContent)(fileNameAnalysisSemanticView, 'utf-8');
            }
            return Promise.resolve('');
        }).then(function (result) {
            if (result && result.length > 0) {
                _resultReadFileContent = JSON.parse(result);
                var diarizationAnalysis = new diarization_analysis_implementation_1.DiarizationAnalysisImplementation();
                var phrases = diarizationAnalysis.GeneratePhrases(_resultReadFileContent.words, 1000000);
                _resultReadFileContent.chat = phrases.allPhrases;
            }
            if (resultAudio && resultAudio.processId) {
                var transcriptionDTO = {
                    vcc: vcc,
                    processId: resultAudio.processId,
                    id: resultAudio.id,
                    section: resultAudio.section,
                    part: resultAudio.part
                };
                return mssql_service_1.MSSql.GetTranscription(transcriptionDTO);
            }
            return Promise.resolve(true);
        }).then(function (result) {
            result = (result && result.StatusCodeError) ? null : result;
            if (typeof result !== 'boolean') {
                _transcriptionQualification = result ? result.value : null;
            }
            if (['allegro', 'i6', 'i-ftp', 'i-sftp'].includes(environment.type))
                return GetVideoFromAllegro(vcc, audioId);
            else
                return Promise.resolve([]);
        }).then(function (result) {
            var promiseArray = [];
            promiseArray.push(Promise.resolve(result));
            var sentimentAnalysis = new sentiment_analysis_implementation_1.SentimentAnalysisImplementation();
            promiseArray.push(Promise.resolve(sentimentAnalysis.GetSentimentResults(jsonFileName, vcc, 'CALL')));
            return Promise.all(promiseArray);
        })
            .then(function (result) {
            if (_resultReadFileContent)
                _resultReadFileContent.sentimentResults = result[1];
            result = result[0];
            var data = {
                audioFile: fileNameAudio,
                channelsChanged: (resultAudio === null || resultAudio === void 0 ? void 0 : resultAudio.channelsChanged) ? resultAudio.channelsChanged : false,
                buffer: buffer,
                resultTranscription: _resultReadFileContent,
                transcriptionQualification: _transcriptionQualification,
                video: result ? result : []
            };
            if (res.headersSent)
                return;
            logger.debug('[AudioService::get_audio_transcription] Response data:', fileNameAnalysisSemanticView, data);
            res.json({ status: true, description: 'SERVER_SUCCESS_MESSAGE', data: data });
        }).catch(function (err) {
            var clientMsg = (typeof err === 'string' ? err : 'CANNOT_PROCESS_REQUEST');
            var errorMsg = (typeof err === 'string' ? err : err.message || err.description || 'SERVER_ERROR_UNKNOWN_MESSAGE');
            logger.error('[AudioService::get_audio_transcription] Getting errorMsg:', errorMsg, 'with req.params:', req.params, 'with vcc:', vcc, 'with resultAudio:', resultAudio);
            if (res.headersSent)
                return;
            res.json({ status: false, description: clientMsg, error: errorMsg, detail: { type: 'ERROR', datetime: moment().utc().toDate(), pid: process.pid, source: __filename, stack: err.stack } });
        });
    });
    app.get('/api/process/audio/get_process_audio_summary/:processId', function (req, res, next) {
        logger.info('Se recibió ' + req.method + ' para \'' + req.path + '\'');
        var vcc = res.locals.vcc;
        var processId = req.params.processId;
        (0, promises_shared_1.DummyPromise)()
            .then(function (result) {
            return mssql_service_1.MSSql.GetProcessAudioSummary(vcc, processId);
        }).then(function (result) {
            if (res.headersSent)
                return;
            res.json({ status: true, description: 'SERVER_SUCCESS_MESSAGE', data: result });
        }).catch(function (err) {
            var clientMsg = (typeof err === 'string' ? err : 'CANNOT_PROCESS_REQUEST');
            var errorMsg = (typeof err === 'string' ? err : err.message || err.description || 'SERVER_ERROR_UNKNOWN_MESSAGE');
            logger.error('Error al procesar \'' + req.path + '\': ' + errorMsg);
            if (res.headersSent)
                return;
            res.json({ status: false, description: clientMsg, error: errorMsg, detail: { type: 'ERROR', datetime: moment().utc().toDate(), pid: process.pid, source: __filename, stack: err.stack } });
        });
    });
    app.get('/api/process/audio/get_audio_directory_count/:process_id', function (req, res, next) {
        logger.info('Se recibió ' + req.method + ' para \'' + req.path + '\'');
        var vcc = res.locals.vcc;
        var processId = req.params.process_id;
        (0, promises_shared_1.DummyPromise)()
            .then(function (result) {
            return (0, file_manager_1.GetFolderFiles)(config_1.PATH_FOLDER_PROCESS_AUDIO + '/' + processId, ['csv'], ['wav', 'mp3', 'ogg'], true);
        }).then(function (resultGetFolderFiles) {
            if (res.headersSent)
                return;
            resultGetFolderFiles = JSON.parse(JSON.stringify(resultGetFolderFiles));
            resultGetFolderFiles = resultGetFolderFiles.filter(function (x) { return x.type !== 'directory'; });
            res.json({ status: true, description: 'SERVER_SUCCESS_MESSAGE', data: resultGetFolderFiles.length });
        }).catch(function (err) {
            var clientMsg = (typeof err === 'string' ? err : 'CANNOT_PROCESS_REQUEST');
            var errorMsg = (typeof err === 'string' ? err : err.message || err.description || 'SERVER_ERROR_UNKNOWN_MESSAGE');
            logger.error('Error al procesar \'' + req.path + '\': ' + errorMsg);
            if (res.headersSent)
                return;
            res.json({ status: false, description: clientMsg, error: errorMsg, detail: { type: 'ERROR', datetime: moment().utc().toDate(), pid: process.pid, source: __filename, stack: err.stack } });
        });
    });
    app.post('/api/process/audio/save_transcription', function (req, res, next) {
        logger.info('[AudioService::save_transcription] Received ' + req.method + ' method on \'' + req.path + '\'');
        var vcc = res.locals.vcc;
        var userId = res.locals.userId;
        var newTranscriptionDTO = (req.body || {});
        newTranscriptionDTO.vcc = vcc;
        newTranscriptionDTO.userId = userId;
        (0, promises_shared_1.DummyPromise)()
            .then(function (result) {
            return mssql_service_1.MSSql.SaveTranscription(newTranscriptionDTO);
        }).then(function (resultSaveTranscription) {
            if (res.headersSent)
                return;
            res.json({ status: true, description: 'SERVER_SUCCESS_MESSAGE', data: resultSaveTranscription });
        }).catch(function (err) {
            var clientMsg = (typeof err === 'string' ? err : 'CANNOT_PROCESS_REQUEST');
            var errorMsg = (typeof err === 'string' ? err : err.message || err.description || 'SERVER_ERROR_UNKNOWN_MESSAGE');
            logger.error('Error al procesar \'' + req.path + '\': ' + errorMsg);
            if (res.headersSent)
                return;
            res.json({ status: false, description: clientMsg, error: errorMsg, detail: { type: 'ERROR', datetime: moment().utc().toDate(), pid: process.pid, source: __filename, stack: err.stack } });
        });
    });
}
exports.AudioService = AudioService;
function SaveAudio(vcc, audio) {
    var promiseArray = [];
    promiseArray.push(mssql_service_1.MSSql.SaveAudio(vcc, audio.id, audio.section, audio.part, audio.processId, audio.number, audio.gender, audio.silenceTime, audio.durationTime, audio.category1, audio.category2, audio.category3, audio.description, audio.campaign, audio.agent, audio.type, audio.init));
    return Promise.all(promiseArray).then(function (results) {
        if (results.filter(function (result) { return (result !== true); }).length > 0) {
            return false;
        }
        else {
            return true;
        }
    });
}
function UpdateAudioInit(vcc, audio, fileNameAnalysisSemantic, fileNameAnalysisSemanticView) {
    var fileContent = '';
    return Promise.resolve()
        .then(function (result) {
        logger.info('[AudioService::UpdateAudioInit] UpdateAudioInit: ', audio);
        return (0, file_manager_1.ReadFileContent)(fileNameAnalysisSemantic, 'utf-8');
    }).catch(function (err) {
        return GetIdathaDocument(vcc, audio, fileNameAnalysisSemantic);
    })
        .then(function (result) {
        return (0, file_manager_1.ReadFileContent)(fileNameAnalysisSemantic, 'utf-8');
    })
        .then(function (resultReadFileContent) {
        fileContent = resultReadFileContent;
        logger.info('[AudioService::UpdateAudioInit] Received ReadFileContent\'s result: ' + JSON.stringify(resultReadFileContent));
        var result = JSON.parse(fileContent).words;
        audio.init = result.length > 0 ? result[0].speaker : null;
        return SaveAudio(vcc, audio);
    }).then(function (result) {
        var resultSortJson = SortJson(fileContent);
        fileContent = resultSortJson;
        return (0, file_manager_1.WriteFileContent)(fileNameAnalysisSemanticView, resultSortJson, 'utf-8');
    }).catch(function (err) {
        var errMsg = (typeof err === 'string' ? err : err.message || err.description || 'SERVER_ERROR_UNKNOWN_MESSAGE');
        logger.error('Error al ejecutar UpdateAudioInit: ' + errMsg);
        throw errMsg;
    });
}
function SortJson(jsonobj) {
    var outlength;
    var inlength;
    var i;
    var audioInit;
    var obj = {};
    obj.transcription = [];
    obj.summarySpeechData = {};
    obj.confidence = {};
    obj.init = '';
    var json = JSON.parse(jsonobj);
    audioInit = json.words[0].speaker;
    obj.init = audioInit;
    obj.nlp = {};
    outlength = json.out_sentences.length;
    inlength = json.in_sentences.length;
    if (outlength >= inlength) {
        for (i in json.out_sentences) {
            if (i < inlength && inlength > 0) {
                if (audioInit === 'out' && inlength > 0) {
                    obj.transcription.push({
                        out: json.out_sentences[i].text,
                        in: json.in_sentences[i] === 'undefined' ? '' : json.in_sentences[i].text
                    });
                }
                else if (audioInit === 'in' && inlength > 0) {
                    obj.transcription.push({
                        in: json.in_sentences[i].text,
                        out: json.out_sentences[i].text
                    });
                }
            }
            else {
                if (audioInit === 'out' && inlength >= 0) {
                    obj.transcription.push({
                        out: json.out_sentences[i] === undefined ? '' : json.out_sentences[i].text,
                        in: ''
                    });
                }
                else if (audioInit === 'in' && inlength >= 0) {
                    obj.transcription.push({
                        in: '',
                        out: json.out_sentences[i] === undefined ? '' : json.out_sentences[i].text
                    });
                }
            }
        }
    }
    else if (outlength <= inlength) {
        for (i in json.in_sentences) {
            if (i < outlength && outlength > 0) {
                if (audioInit === 'out' && outlength > 0) {
                    obj.transcription.push({
                        out: json.out_sentences[i].text,
                        in: json.in_sentences[i].text
                    });
                }
                else if (audioInit === 'in' && outlength > 0) {
                    obj.transcription.push({
                        in: json.in_sentences[i].text,
                        out: json.out_sentences[i].text
                    });
                }
            }
            else {
                if (audioInit === 'out' && inlength >= 0) {
                    obj.transcription.push({
                        out: '',
                        in: json.in_sentences[i] === undefined ? '' : json.in_sentences[i].text,
                    });
                }
                else if (audioInit === 'in' && inlength >= 0) {
                    obj.transcription.push({
                        in: json.in_sentences[i] === undefined ? '' : json.in_sentences[i].text,
                        out: ''
                    });
                }
            }
        }
    }
    obj.nlp.in = json.pnl_scores_in ? json.pnl_scores_in : 0;
    obj.nlp.out = json.pnl_scores_out ? json.pnl_scores_out : 0;
    var dataTotalIn = {
        duration: 0,
        silence: 0
    }, dataTotalOut = {
        duration: 0,
        silence: 0
    }, data = json.words, len = data.length, word, inDuration = 0, inSilence = 0, outDuration = 0, outSilence = 0;
    for (i = 0; i < len; i++) {
        word = data[i];
        if (word.speaker === 'in') {
            inDuration += word.duration;
            inSilence += word.silence;
        }
        else {
            outDuration += word.duration;
            outSilence += word.silence;
        }
    }
    dataTotalIn.duration = inDuration.toFixed(2) * 1;
    dataTotalIn.silence = inSilence.toFixed(2) * 1;
    dataTotalOut.duration = outDuration.toFixed(2) * 1;
    dataTotalOut.silence = outSilence.toFixed(2) * 1;
    obj.summarySpeechData = {
        dataTotalIn: dataTotalIn,
        dataTotalOut: dataTotalOut
    };
    obj.duration = json.duration;
    obj.words = json.words;
    obj.topics = json.topics;
    obj.confidence.in_confidence = json.in_confidence ? json.in_confidence : 0;
    obj.confidence.out_confidence = json.out_confidence ? json.out_confidence : 0;
    return JSON.stringify(obj);
}
function ExportAudioTranscription(vcc, audio, fileNameAnalysisSemantic, fileNameAnalysisSemanticView) {
    return Promise.resolve()
        .then(function (result) {
        return (0, file_manager_1.CreateFolder)(config_1.SEMANCTIC_ANALYSIS_FOLDER + '/' + vcc);
    }).then(function (result) {
        return (0, file_manager_1.CreateFolder)(config_1.SEMANCTIC_ANALYSIS_VIEW_FOLDER + '/' + vcc);
    }).then(function () {
        return (0, file_manager_1.ExistInFileSystem)(fileNameAnalysisSemanticView);
    }).then(function (resultExistInFileSystem) {
        if (!resultExistInFileSystem) {
            return UpdateAudioInit(vcc, audio, fileNameAnalysisSemantic, fileNameAnalysisSemanticView);
        }
        return Promise.resolve(true);
    }).catch(function (err) {
        var errMsg = (typeof err === 'string' ? err : err.message || err.description || 'SERVER_ERROR_UNKNOWN_MESSAGE');
        logger.error('[AudioService::ExportAudioTranscription] Getting error:', err, 'with vcc:', vcc, 'with audio:', audio, 'fileNameAnalysisSemantic:', fileNameAnalysisSemantic, 'fileNameAnalysisSemanticView:', fileNameAnalysisSemanticView);
        throw errMsg;
    });
}
function ExternalRequest(url, timeout) {
    if (timeout === void 0) { timeout = 30000; }
    var options = {
        uri: url,
        encoding: null,
        retryProtocol: true,
        timeout: timeout,
        responseType: 'stream'
    };
    logger.info('[audio.service::ExternalRequest] Option:', options);
    return (0, request_shared_1.SimpleGetRequest)(options)
        .then(function (result) {
        logger.info('[audio.service::ExternalRequest] OK');
        var buffer = null;
        var bufs = [];
        return new Promise(function (resolve, reject) {
            result.on('error', function (err) {
                logger.error('[audio.service::ExternalRequest] SERVER_ERROR_CREATE_READ_STREAM_FAILED');
                reject('SERVER_ERROR_CREATE_READ_STREAM_FAILED');
            });
            result.once('close', function () {
                try {
                    buffer = Buffer.concat(bufs);
                    logger.info('[audio.service::ExternalRequest] closi');
                    return resolve(buffer);
                }
                catch (e) {
                    logger.info('[audio.service::ExternalRequest] close', e);
                    return reject('SERVER_ERROR_CREATE_READ_STREAM_ON_CLOSE_FAILED');
                }
            });
            result.on('data', function (d) {
                try {
                    bufs.push(d);
                }
                catch (e) {
                    logger.info('[audio.service::ExternalRequest] data', e);
                    return reject('SERVER_ERROR_CREATE_READ_STREAM_ON_DATA_FAILED');
                }
            });
        });
    })
        .catch(function (err) {
        logger.info('[audio.service::ExternalRequest] SERVER_ERROR_UNKNOWN_MESSAGE');
        return Promise.reject(err);
    });
}
function getTimes(startTime, endTime) {
    var startHour, startMinute, endHour, endMinute;
    if (startTime && startTime.length > 0) {
        var arrayStartTime = startTime.split(':');
        if (arrayStartTime && arrayStartTime.length > 0) {
            startHour = arrayStartTime[0];
            startMinute = arrayStartTime[1];
        }
    }
    else {
        startHour = 0;
        startMinute = 0;
    }
    if (endTime && endTime.length > 0) {
        var arrayEndTime = endTime.split(':');
        if (arrayEndTime && arrayEndTime.length > 0) {
            endHour = arrayEndTime[0];
            endMinute = arrayEndTime[1];
        }
    }
    else {
        endHour = 23;
        endMinute = 59;
    }
    return {
        startTime: {
            hour: startHour,
            minute: startMinute,
        },
        endTime: {
            hour: endHour,
            minute: endMinute,
        }
    };
}
function GetIdathaDocument(vcc, interaction, fileName) {
    logger.info('[AudioService::GetIdathaDocument] Receiving interaction:', interaction);
    logger.info('[AudioService::GetIdathaDocument] Receiving filename:', fileName);
    var accessToken = {};
    return (0, promises_shared_1.DummyPromise)()
        .then(function (_) {
        return (0, idatha_service_1.IdathaLogin)(vcc);
    })
        .then(function (result) {
        if (!result)
            throw 'SERVER_ERROR_IDATHA_LOGIN_FAILED';
        accessToken = result;
        var regex = /\.(mp3|wav|ogg|json)$/;
        var fileName = interaction.fileName.replace(regex, '');
        var semanticAnalyisImpl = new semantic_analysis_implementation_1.SemanticAnalysisImplementation();
        return semanticAnalyisImpl.IdathaDocumentWords(accessToken.accessToken, interaction.id, interaction.section, interaction.part, null, null, fileName);
    })
        .then(function (data) {
        if (data)
            return (0, file_manager_1.WriteFileContent)(fileName, data, 'utf-8');
        else
            return Promise.resolve(false);
    });
}
exports.GetIdathaDocument = GetIdathaDocument;
function GetAudioFilePossiblePaths(audioId, audioRelativePath, audioRelativeFileName, environment) {
    var _a, _b;
    audioRelativeFileName || (audioRelativeFileName = null);
    var filePaths = [];
    var isWav = audioRelativeFileName ? audioRelativeFileName.toLowerCase().endsWith('.wav') : false;
    var isMp3 = audioRelativeFileName ? audioRelativeFileName.toLowerCase().endsWith('.mp3') : false;
    var isOgg = audioRelativeFileName ? audioRelativeFileName.toLowerCase().endsWith('.ogg') : false;
    var hasExtension = isWav || isMp3 || isOgg;
    if ((_a = environment.ftpDirectory) === null || _a === void 0 ? void 0 : _a.endsWith('/'))
        environment.ftpDirectory = environment.ftpDirectory.slice(0, -1);
    if (audioRelativePath === null || audioRelativePath === void 0 ? void 0 : audioRelativePath.endsWith('/'))
        audioRelativePath = audioRelativePath.slice(0, -1);
    if (!((_b = environment.ftpDirectory) === null || _b === void 0 ? void 0 : _b.startsWith('/')))
        environment.ftpDirectory = '/' + environment.ftpDirectory;
    if (audioRelativePath === null && audioRelativeFileName !== null) {
        logger.info('[AudioService::GetAudioFilePossiblePaths] Getting from block 01');
        if (hasExtension) {
            filePaths.push("".concat(environment.ftpDirectory, "/").concat(audioRelativeFileName));
        }
        else {
            filePaths.push("".concat(environment.ftpDirectory, "/").concat(audioRelativeFileName, ".wav"));
            filePaths.push("".concat(environment.ftpDirectory, "/").concat(audioRelativeFileName, ".mp3"));
            filePaths.push("".concat(environment.ftpDirectory, "/").concat(audioRelativeFileName, ".ogg"));
        }
    }
    else if (audioRelativePath !== null && audioRelativeFileName === null) {
        logger.info('[AudioService::GetAudioFilePossiblePaths] Getting from block 02');
        filePaths.push("".concat(audioRelativePath, "/").concat(audioId, ".wav"));
        filePaths.push("".concat(audioRelativePath, "/").concat(audioId, ".mp3"));
        filePaths.push("".concat(audioRelativePath, "/").concat(audioId, ".gg"));
        filePaths.push("".concat(environment.ftpDirectory, "/").concat(audioId, ".wav"));
        filePaths.push("".concat(environment.ftpDirectory, "/").concat(audioId, ".mp3"));
        filePaths.push("".concat(environment.ftpDirectory, "/").concat(audioId, ".ogg"));
        filePaths.push("".concat(environment.ftpDirectory, "/").concat(audioRelativePath, "/").concat(audioId, ".wav"));
        filePaths.push("".concat(environment.ftpDirectory, "/").concat(audioRelativePath, "/").concat(audioId, ".mp3"));
        filePaths.push("".concat(environment.ftpDirectory, "/").concat(audioRelativePath, "/").concat(audioId, ".ogg"));
    }
    else if (audioRelativePath !== null && audioRelativeFileName !== null) {
        logger.info('[AudioService::GetAudioFilePossiblePaths] Getting from block 03');
        if (hasExtension) {
            filePaths.push("".concat(audioRelativePath, "/").concat(audioRelativeFileName));
            filePaths.push("".concat(environment.ftpDirectory, "/").concat(audioRelativeFileName));
            filePaths.push("".concat(environment.ftpDirectory, "/").concat(audioRelativePath, "/").concat(audioRelativeFileName));
        }
        else {
            filePaths.push("".concat(audioRelativePath, "/").concat(audioRelativeFileName, ".mp3"));
            filePaths.push("".concat(audioRelativePath, "/").concat(audioRelativeFileName, ".wav"));
            filePaths.push("".concat(audioRelativePath, "/").concat(audioRelativeFileName, ".ogg"));
            filePaths.push("".concat(environment.ftpDirectory, "/").concat(audioRelativePath, "/").concat(audioRelativeFileName, ".mp3"));
            filePaths.push("".concat(environment.ftpDirectory, "/").concat(audioRelativePath, "/").concat(audioRelativeFileName, ".wav"));
            filePaths.push("".concat(environment.ftpDirectory, "/").concat(audioRelativePath, "/").concat(audioRelativeFileName, ".ogg"));
            filePaths.push("".concat(environment.ftpDirectory, "/").concat(audioRelativeFileName, ".mp3"));
            filePaths.push("".concat(environment.ftpDirectory, "/").concat(audioRelativeFileName, ".wav"));
            filePaths.push("".concat(environment.ftpDirectory, "/").concat(audioRelativeFileName, ".ogg"));
        }
    }
    else {
        logger.info('[AudioService::GetAudioFilePossiblePaths] Getting from block 04');
        filePaths.push("".concat(environment.ftpDirectory, "/").concat(audioId, ".wav"));
        filePaths.push("".concat(environment.ftpDirectory, "/").concat(audioId, ".mp3"));
        filePaths.push("".concat(environment.ftpDirectory, "/").concat(audioId, ".ogg"));
    }
    filePaths = filePaths.map(function (fp) { return fp.replace('//', '/'); });
    logger.debug('[AudioService::GetAudioFilePossiblePaths] Possible filepaths', filePaths);
    return filePaths;
}
function GetAudioFileFromFtp(vcc, audioData, environment, ftpClient) {
    var audioId = audioData.audioId;
    var audioRelativePath = audioData.relativePath;
    var audioRelativeFileName = audioData.relativeFileName;
    var filePaths = GetAudioFilePossiblePaths(audioId, audioRelativePath, audioRelativeFileName, environment);
    var environmentConfig = null;
    var ftp = null;
    if (!ftpClient || !ftpClient.connected) {
        environmentConfig = {
            host: environment.apiHost,
            port: environment.apiPort,
            user: environment.apiUser,
            password: environment.apiPassword,
            connTimeout: config_1.FTP_CONNECTION_TIMEOUT,
            pasvTimeout: config_1.FTP_CONNECTION_TIMEOUT
        };
        ftp = new Client();
        ftp.connect(environmentConfig);
    }
    var genericResult = {};
    return (0, promises_shared_1.DummyPromise)()
        .then(function (_) {
        return new Promise(function (resolve, reject) {
            ftp !== null && ftp !== void 0 ? ftp : (ftp = ftpClient);
            if (ftp) {
                if (ftpClient) {
                    SearchAudioFileOnFtp(ftp, filePaths, !ftpClient, genericResult, resolve, reject);
                }
                else {
                    ftp.on('greeting', function (message) {
                        logger.info('[AudioService::GetAudioFileFromFtp] Greeting event message:', message);
                    });
                    ftp.on('ready', function () {
                        SearchAudioFileOnFtp(ftp, filePaths, !ftpClient, genericResult, resolve, reject);
                    });
                    ftp.on('end', function () {
                        var eventMessage = 'FTP_CONNECTION_ENDED';
                        logger.warn('[AudioService::GetAudioFileFromFtp] getting eventMessage:', eventMessage);
                        return resolve(eventMessage);
                    });
                    ftp.on('close', function (hadError) {
                        var eventMessage = hadError ? 'FTP_CONNECTION_CLOSE_DUE_ERROR' : 'FTP_CONNECTION_CLOSE';
                        logger.warn('[AudioService::GetAudioFileFromFtp] hadError:', hadError);
                        return resolve(eventMessage);
                    });
                    ftp.on('error', function (err) {
                        logger.error('[AudioService::GetAudioFileFromFtp] NOT_FTP_CLIENT', err);
                        if (filePaths.length)
                            SearchAudioFileOnFtp(ftp, filePaths, !ftpClient, genericResult, resolve, reject);
                        else
                            return reject('NOT_FTP_CLIENT');
                    });
                }
            }
            else {
                logger.error('[AudioService::GetAudioFileFromFtp] SERVER_ERROR_FTP_NOT_CLIENT');
                return reject('SERVER_ERROR_NO_FTP_CLIENT');
            }
        });
    })
        .then(function (result) {
        logger.info("[AudioService::GetAudioFileFromFtp] Found file ".concat(result, " for interaction ").concat(audioId));
        return Promise.resolve({
            audioFile: result,
            buffer: genericResult.extraContent
        });
    })
        .catch(function (error) {
        if (environment.type === 'i-ftp' && !error.status && error.extraContent === 'SERVER_ERROR_AUDIO_FILE_GET_FTP') {
            var fileName = audioData.fileName ? audioData.fileName.split('/').pop() : null;
            return GetAudioFileFromIntegrated(vcc, environment.type, audioData.audioId, audioData.section, audioData.part, fileName, null, 'REMOTE', true);
        }
        return Promise.reject(error);
    });
}
exports.GetAudioFileFromFtp = GetAudioFileFromFtp;
function SearchAudioFileOnFtp(clientFtp, filePaths, closeOnFile, genericResult, resolve, reject) {
    var ftpFile = filePaths.shift();
    logger.debug('[AudioService::SearchAudioFileOnFtp] Searching:', ftpFile, closeOnFile);
    clientFtp.get(ftpFile, function (err, stream) {
        logger.debug('[AudioService::SearchAudioFileOnFtp] GET for:', ftpFile);
        if (!err) {
            logger.debug('[AudioService::SearchAudioFileOnFtp] No error condition');
            genericResult.status = true;
            var bufs_1 = [];
            stream.once('error', function (err) {
                logger.error('[AudioService::SearchAudioFileOnFtp] Once error:', err);
            });
            stream.once('end', function () {
                logger.info('[AudioService::SearchAudioFileOnFtp] Once end');
            });
            stream.once('close', function () {
                logger.info('[AudioService::SearchAudioFileOnFtp] Once close');
                if (closeOnFile)
                    clientFtp.end();
                genericResult.extraContent = Buffer.concat(bufs_1);
                resolve(ftpFile.split('/').pop());
            });
            stream.on('data', function (d) {
                bufs_1.push(d);
            });
            stream.on('end', function () {
                logger.info('[AudioService::SearchAudioFileOnFtp] On end');
            });
            stream.on('error', function (err) {
                logger.error('[AudioService::SearchAudioFileOnFtp] On error:', err);
            });
            stream.on('close', function () {
                logger.error('[AudioService::SearchAudioFileOnFtp] On close');
            });
        }
        else {
            logger.error('[AudioService::SearchAudioFileOnFtp] Audiofile not found on:', ftpFile);
            if (filePaths.length) {
                SearchAudioFileOnFtp(clientFtp, filePaths, closeOnFile, genericResult, resolve, reject);
            }
            else {
                if (closeOnFile)
                    clientFtp.end();
                genericResult.status = false;
                genericResult.extraContent = 'SERVER_ERROR_AUDIO_FILE_GET_FTP';
                reject(genericResult);
            }
        }
    });
}
function GetAudioFileFromSftp(vcc, audioData, environment, sftpClient) {
    var audioId = audioData.audioId;
    var audioRelativePath = audioData.relativePath;
    var audioFileName = audioData.fileName;
    var audioRelativeFileName = audioData.relativeFileName;
    var filePaths = GetAudioFilePossiblePaths(audioId, audioRelativePath, audioRelativeFileName, environment);
    var audioFile = null;
    var genericResult = {};
    var environmentConfig = null;
    var ssh2 = null;
    var sftp = null;
    if (!sftpClient) {
        environmentConfig = {
            host: environment.apiHost,
            port: environment.apiPort,
            username: environment.apiUser,
            password: environment.apiPassword,
            connTimeout: config_1.FTP_CONNECTION_TIMEOUT,
            pasvTimeout: config_1.FTP_CONNECTION_TIMEOUT
        };
        ssh2 = new SSH2Promise(environmentConfig);
    }
    return (0, promises_shared_1.DummyPromise)()
        .then(function (_) {
        if (sftpClient || ssh2) {
            sftp = sftpClient || ssh2.sftp();
            return SearchAudioFileOnSftp(sftp, filePaths, !sftpClient, genericResult);
        }
        else {
            logger.error('[AudioService::GetAudioFileFromSftp] SERVER_ERROR_NOT_SFTP_CLIENT');
            throw 'SERVER_ERROR_NOT_SFTP_CLIENT';
        }
    })
        .then(function (result) {
        if (genericResult.status === true) {
            logger.info('[AudioService::GetAudioFileFromSftp] Resolving general result');
            return Promise.resolve({
                audioFile: result,
                buffer: genericResult.extraContent
            });
        }
        else {
            logger.error('[AudioService::GetAudioFileFromSftp] SERVER_ERROR_GET_AUDIO_FROM_SFTP_FAILED');
            throw 'SERVER_ERROR_GET_AUDIO_FROM_SFTP_FAILED';
        }
    })
        .catch(function (err) {
        genericResult.status = false;
        genericResult.error = err;
        logger.error('[AudioService::GetAudioFileFromSftp] General error: ', err);
        if (environment.type === 'i-sftp' && !err.status && err.extraContent === 'SERVER_ERROR_AUDIO_FILE_GET_SFTP') {
            var fileName = audioData.fileName ? audioData.fileName.split('/').pop() : null;
            return GetAudioFileFromIntegrated(vcc, environment.type, null, null, null, audioData.fileName, null, 'REMOTE', true);
        }
        return Promise.reject({
            audioFile: audioFile,
            buffer: genericResult.error
        });
    });
}
exports.GetAudioFileFromSftp = GetAudioFileFromSftp;
function SearchAudioFileOnSftp(sftp, filePaths, closeOnFile, genericResult) {
    var sftpFile = filePaths.shift();
    logger.debug('[AudioService::SearchAudioFileOnSftp] Searching:', sftpFile);
    return sftp.stat(sftpFile)
        .then(function (_) {
        return sftp.createReadStream(sftpFile, { flags: 'r', mode: 438 })
            .then(function (stream) {
            if (stream) {
                genericResult.status = true;
                var bufs_2 = [];
                return new Promise(function (resolve, reject) {
                    stream.on('error', function (err) {
                        logger.error('[AudioService::SearchAudioFileOnSftp] SERVER_ERROR_CREATE_READ_STREAM_FAILED');
                        return reject('SERVER_ERROR_CREATE_READ_STREAM_FAILED');
                    });
                    stream.once('close', function () {
                        try {
                            if (closeOnFile)
                                sftp.close();
                            genericResult.extraContent = Buffer.concat(bufs_2);
                            return resolve(sftpFile.split('/').pop());
                        }
                        catch (e) {
                            return reject('SERVER_ERROR_CREATE_READ_STREAM_ON_CLOSE_FAILED');
                        }
                    });
                    stream.on('data', function (d) {
                        try {
                            bufs_2.push(d);
                        }
                        catch (e) {
                            return reject('SERVER_ERROR_CREATE_READ_STREAM_ON_DATA_FAILED');
                        }
                    });
                });
            }
            else {
                logger.error('[AudioService::SearchAudioFileOnSftp] SERVER_ERROR_GET_AUDIO_STREAM_FAILED');
                throw 'SERVER_ERROR_GET_AUDIO_STREAM_FAILED';
            }
        })
            .catch(function (err) {
            throw err;
        });
    })
        .catch(function (err) {
        logger.warn('[AudioService::SearchAudioFileOnSftp] Audiofile not found on:', sftpFile);
        if (filePaths.length) {
            return SearchAudioFileOnSftp(sftp, filePaths, closeOnFile, genericResult);
        }
        else {
            return Promise.resolve(null);
        }
    });
}
function GetAudioFileFromAwsS3(vcc, audioData, environment, awsS3Impl) {
    var audioId = audioData.audioId;
    var audioRelativePath = audioData.relativePath;
    var audioRelativeFileName = audioData.relativeFileName;
    var filePaths = GetAudioFilePossiblePaths(audioId, audioRelativePath, audioRelativeFileName, environment);
    var genericResult = {};
    var awsS3 = awsS3Impl ? awsS3Impl : new aws_s3_implementation_1.AwsS3Implementation(environment.apiHost, environment.apiUser, environment.apiPassword, environment.bucket);
    return (0, promises_shared_1.DummyPromise)()
        .then(function (_) {
        return SearchAudioFileFromAwsS3(awsS3, filePaths, genericResult);
    })
        .then(function (result) {
        logger.info("[AudioService::GetAudioFileFromAwsS3] Found file ".concat(result, " for interaction ").concat(audioId));
        return Promise.resolve({
            audioFile: result,
            buffer: genericResult.extraContent
        });
    })
        .catch(function (error) {
        if (environment.type === 'i-ftp' && !error.status && error.extraContent === 'SERVER_ERROR_AUDIO_FILE_GET_FTP') {
            var fileName = audioData.fileName ? audioData.fileName.split('/').pop() : null;
            return GetAudioFileFromIntegrated(vcc, environment.type, audioData.audioId, audioData.section, audioData.part, fileName, null, 'REMOTE', true);
        }
        return Promise.reject(error);
    });
}
exports.GetAudioFileFromAwsS3 = GetAudioFileFromAwsS3;
function SearchAudioFileFromAwsS3(awsS3impl, filePaths, genericResult) {
    var awsS3File = filePaths.shift();
    logger.debug('[AudioService::SearchAudioFileFromAwsS3] Searching:', awsS3File);
    return (0, promises_shared_1.DummyPromise)()
        .then(function (_) {
        return awsS3impl.DownloadFile(awsS3File);
    })
        .then(function (result) {
        if (result) {
            genericResult.status = true;
            var bufs_3 = [];
            return new Promise(function (resolve, reject) {
                result.on('error', function (err) {
                    logger.error('[AudioService::SearchAudioFileFromAwsS3] SERVER_ERROR_CREATE_READ_STREAM_FAILED');
                    return reject('SERVER_ERROR_CREATE_READ_STREAM_FAILED');
                });
                result.once('close', function () {
                    try {
                        genericResult.extraContent = Buffer.concat(bufs_3);
                        return resolve(awsS3File.split('/').pop());
                    }
                    catch (e) {
                        return reject('SERVER_ERROR_CREATE_READ_STREAM_ON_CLOSE_FAILED');
                    }
                });
                result.on('data', function (d) {
                    try {
                        bufs_3.push(d);
                    }
                    catch (e) {
                        return reject('SERVER_ERROR_CREATE_READ_STREAM_ON_DATA_FAILED');
                    }
                });
            });
        }
        else {
            logger.error('[AudioService::SearchAudioFileFromAwsS3] SERVER_ERROR_GET_AUDIO_STREAM_FAILED');
            throw 'SERVER_ERROR_GET_AUDIO_STREAM_FAILED';
        }
    })
        .catch(function (err) {
        logger.warn('[AudioService::SearchAudioFileOnSftp] Audiofile not found on:', awsS3File);
        if (filePaths.length)
            return SearchAudioFileFromAwsS3(awsS3impl, filePaths, genericResult);
        else
            return Promise.resolve(null);
    });
}
function GetAudioFileName(vcc, fileName, interactionId, section, part, ext, target) {
    if (!target)
        target = 'LOCAL';
    if (!ext)
        ext = '.wav';
    var fileNameAudio = null;
    if (fileName === null) {
        fileNameAudio = interactionId;
        fileNameAudio += '_';
        fileNameAudio += section;
        fileNameAudio += '_';
        fileNameAudio += part;
        fileNameAudio += ext;
    }
    else {
        fileNameAudio = fileName;
        ext = ".".concat(fileName.split('.').pop());
        var interactionSectionPart = fileNameAudio.replace(ext, '').split('_');
        part = parseInt(interactionSectionPart.pop());
        section = parseInt(interactionSectionPart.pop());
        interactionId = interactionSectionPart.join('_');
    }
    if (fileNameAudio !== null) {
        var filePath = '';
        if (target === 'LOCAL') {
            filePath = part !== null && !isNaN(part) && part > 0 ? config_1.PATH_FOLDER_TEMP_AUDIO : config_1.PATH_FOLDER_PROCESS_AUDIO;
            filePath += '/';
            filePath += vcc;
            filePath += '/';
        }
        fileNameAudio = filePath + fileNameAudio;
    }
    return fileNameAudio;
}
function GetAudioFileFromIntegrated(vcc, environmentType, interactionId, section, part, fileName, ext, target, notFoundOnFtp) {
    if (!ext)
        ext = '.wav';
    if (!target)
        target = 'LOCAL';
    if (!ext.includes('.'))
        ext = ".".concat(ext);
    var fileNameAudio = GetAudioFileName(vcc, fileName, interactionId, section, part, ext, target);
    return (0, promises_shared_1.DummyPromise)()
        .then(function (result) {
        switch (target) {
            case 'LOCAL': return (0, file_manager_1.BufferFile)(fileNameAudio);
            case 'REMOTE':
                if (['allegro', 'i6'].includes(environmentType))
                    return GetAudioFromIntegrated(vcc, environmentType, interactionId, section, part, fileNameAudio);
                else if (notFoundOnFtp && ['i-ftp', 'i-sftp'].includes(environmentType))
                    return GetAudioFromIntegrated(vcc, environmentType, interactionId, section, part, fileNameAudio);
                else
                    throw 'SERVER_ERROR_NOT_ALLOWED_TARGET';
            default: throw 'SERVER_ERROR_INVALID_TARGET';
        }
    }).catch(function (error) {
        logger.error('[AudioService::GetAudioFileFromIntegrated] Tried: ', fileNameAudio);
        if (ext === '.wav' && target === 'REMOTE' && environmentType === 'i6') {
            logger.error('[AudioService::GetAudioFileFromIntegrated] Audio not found on I6 api: ', fileNameAudio);
            throw 'SERVER_ERROR_AUDIO_FILE_NOT_FOUND';
        }
        if (ext === '.wav' && target === 'LOCAL' && (fileName === null || fileName.includes('.mp3')))
            ext = '.mp3';
        else if (ext === '.mp3' && target === 'LOCAL' && (fileName === null || fileName.includes('.ogg')))
            ext = '.ogg';
        else if (ext === '.ogg' && target === 'LOCAL' && (fileName === null || fileName.includes('.wav'))) {
            ext = '.wav';
            target = 'REMOTE';
        }
        else if (ext === '.wav' && target === 'REMOTE' && (fileName === null || fileName.includes('.mp3')))
            ext = '.mp3';
        else if (ext === '.mp3' && target === 'REMOTE' && (fileName === null || fileName.includes('.ogg')))
            ext = '.ogg';
        else if (ext === '.ogg' && target === 'REMOTE')
            throw 'SERVER_ERROR_AUDIO_FILE_NOT_FOUND';
        fileNameAudio = null;
        return GetAudioFileFromIntegrated(vcc, environmentType, interactionId, section, part, null, ext, target);
    }).then(function (result) {
        if (!result
            ||
                (!Buffer.isBuffer(result)
                    &&
                        target === 'REMOTE'
                    &&
                        environmentType === 'i6'
                    &&
                        !result.status))
            throw 'SERVER_ERROR_AUDIO_FILE_NOT_FOUND';
        var res = {};
        if (fileNameAudio) {
            logger.info('[AudioService::GetAudioFileFromIntegrated] Found on: ', fileNameAudio);
            res = {
                audioFile: fileNameAudio,
                buffer: result
            };
        }
        else {
            res = result;
        }
        return Promise.resolve(res);
    }).catch(function (error) {
        logger.error('[AudioService::GetAudioFileFromIntegrated] Found on: ', error);
        return Promise.reject(error);
    });
}
exports.GetAudioFileFromIntegrated = GetAudioFileFromIntegrated;
function GetAudioFromIntegrated(vcc, environmentType, interactionId, section, part, fileNameAudio) {
    var audioFilePath = null;
    if (['i-ftp', 'i-sftp'].includes(environmentType))
        environmentType = 'allegro';
    return (0, promises_shared_1.DummyPromise)()
        .then(function (_) {
        switch (environmentType) {
            case 'allegro':
                audioFilePath = "http://".concat(integration_config_1.IP_MW, ":").concat(integration_config_1.PORT_MW, "/inconcert/apps/mediaserver/action/download/repository/").concat(interactionId, "/").concat(section, "/").concat(part, "/");
                fileNameAudio = audioFilePath + fileNameAudio;
                return ExternalRequest(fileNameAudio);
            case 'i6':
                return (0, http_request_occ_1.ApiI6GetInteractionFile)(vcc, interactionId, section, part);
            default:
                throw 'SERVER_ERROR_UNKNOWN_ENVIRONMENT_TYPE';
        }
    })
        .then(function (result) {
        return Promise.resolve(result);
    })
        .catch(function (err) {
        logger.info('[audioService::GetAudioFromIntegrated] Catch: ', err);
        return Promise.reject(err);
    });
}
function GetVideoFromAllegro(vcc, interactionId) {
    var flagAllegro = integration_config_1.IP_BD === mssql_config_1.MSSQL_CONFIG.server ? 1 : 0;
    var myVCC = null;
    return (0, promises_shared_1.DummyPromise)()
        .then(function (result) {
        return (0, http_shared_1.GetVCC)(vcc);
    })
        .then(function (result) {
        if (!result)
            throw 'SERVER_ERROR_VCC_NOT_FOUND';
        myVCC = result;
        if (myVCC.type === 'OCC' || myVCC.type === 'I6') {
            return mssql_service_1.MSSql.GetAllegroInteractionVideos(integration_config_1.IP_BD, vcc, flagAllegro, interactionId);
        }
        else
            throw 'SERVER_ERROR_VCC_TYPE_NOT_SUPPORTED';
    })
        .then(function (result) {
        result.map(function (r) {
            r.path = 'http://';
            r.path += integration_config_1.DNS;
            r.path += '/inconcert/apps/mediaserver/action/download/repository/';
            r.path += r.id;
            r.path += '/';
            r.path += r.section;
            r.path += '/';
            r.path += r.part;
            r.path += '/';
            r.path += r.id;
            r.path += '_';
            r.path += r.section;
            r.path += '_';
            r.path += r.part;
            r.path += '.';
            r.path += r.format;
        });
        return Promise.resolve(result);
    })
        .catch(function (err) {
        logger.info('[audioService::GetVideoFromAllegro] Catch: ', err);
        return Promise.resolve([]);
    });
}
function SaveAndConvertAudioFile(result, environmentType, vcc, audioData, fileNameAudioSaveFtpSftp) {
    var pathFolder = '';
    var localFileName = '';
    var pathFolderVCC = '';
    if (['allegro', 'i6', 'i-ftp', 'i-sftp'].includes(environmentType)) {
        pathFolder = config_1.PATH_FOLDER_TEMP_AUDIO;
        localFileName += fileNameAudioSaveFtpSftp ? fileNameAudioSaveFtpSftp.split('/').pop() : result.audioFile.split('/').pop();
    }
    else {
        pathFolder = config_1.PATH_FOLDER_PROCESS_AUDIO;
        localFileName = "".concat(audioData.audioId, "_").concat(audioData.section, "_").concat(audioData.part, ".");
        localFileName += result.audioFile.split('.').pop();
    }
    pathFolderVCC = "".concat(pathFolder, "/").concat(vcc);
    var localFilePath = '';
    localFilePath = pathFolderVCC + '/' + localFileName;
    return (0, promises_shared_1.DummyPromise)()
        .then(function (_) {
        return (0, file_manager_1.CreateFolder)(pathFolderVCC);
    }).then(function (res) {
        return new Promise(function (resolve, reject) {
            fs.open(localFilePath, 'w', function (err, fd) {
                if (err) {
                    logger.info('[AudioService::SaveAndConvertAudioFile]', 'getting error creating local audio file:', err);
                    reject('SERVER_ERROR_CREATE_LOCAL_AUDIO_FILE_FAILED');
                }
                fs.write(fd, result.buffer, 0, result.buffer.length, null, function (err) {
                    if (err) {
                        logger.info('[AudioService::SaveAndConvertAudioFile]', 'getting error writing local audio file:', err);
                        reject('SERVER_ERROR_WRITE_LOCAL_AUDIO_FILE_FAILED');
                    }
                    fs.close(fd, function (err) {
                        if (err)
                            throw err;
                        if (localFileName.split('.').pop() === 'wav' || localFileName.split('.').pop() === 'ogg')
                            resolve(ConvertAudioToReduceSize(result, pathFolder, vcc, localFileName));
                        resolve(result);
                    });
                });
            });
        });
    });
}
function ConvertAudioToReduceSize(result, pathFolder, vcc, fileName) {
    var newFormat = 'mp3';
    var newOldFileNamePath = '';
    var newFileName = fileName.replace('.wav', ".".concat(newFormat)).replace('.ogg', ".".concat(newFormat));
    var oldFilePath = pathFolder + '/' + vcc + '/' + fileName;
    ;
    var newFilePath = pathFolder + '/' + vcc + '/' + newFileName;
    ;
    var forceConvertion = false;
    if (oldFilePath === newFilePath) {
        forceConvertion = true;
        newOldFileNamePath = oldFilePath.replace('.mp3', '_changed.mp3');
    }
    return (0, promises_shared_1.DummyPromise)()
        .then(function (_) {
        if (forceConvertion)
            return (0, file_manager_1.RenameFile)(oldFilePath, newOldFileNamePath);
        return true;
    })
        .then(function (_) {
        if (forceConvertion)
            oldFilePath = newOldFileNamePath;
        return true;
    })
        .then(function (_) {
        var cmd = "".concat(config_1.FFMPEG_FOLDER_PATH, "ffmpeg -y -i \"").concat(oldFilePath, "\" -vn -ar 22050 -ac 1 -b:a 16k \"").concat(newFilePath, "\"");
        logger.debug('[AudioService::ConvertAudioToReduceSize]', 'getting cmd:', cmd);
        return new Promise(function (resolve, reject) {
            try {
                (0, child_process_1.exec)(cmd, function (err, stdout, stderr) {
                    if (err) {
                        reject(err);
                    }
                    else {
                        resolve(result);
                    }
                });
            }
            catch (e) {
                reject(e);
            }
        });
    })
        .then(function (_) {
        return (0, file_manager_1.DeleteFile)(oldFilePath, true);
    })
        .then(function (_) {
        return GetAudioFileFromIntegrated(vcc, null, null, null, null, newFileName, newFormat, 'LOCAL');
    })
        .catch(function (err) {
        var errorMessage = typeof err === 'string' ? err : (err.message || err.description);
        logger.error('[AudioService::ConvertAudioToReduceSize]', 'getting error:', errorMessage);
        return Promise.resolve(true);
    });
}
