"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.BatchService = void 0;
var log4js = require("log4js");
var mssql_service_1 = require("../db/mssql.service");
var promises_shared_1 = require("../shared/promises.shared");
var billing_batch_implementation_1 = require("../implementation/billing-batch.implementation");
var billing_batch_model_1 = require("../models/billing-batch.model");
var logger = log4js.getLogger('ServerScripts');
function BatchService(app) {
    app.post('/api/batch/get_batch_status', function (req, res, next) {
        logger.info('[BatchService::get_batch_status] Received ' + req.method + ' method on \'' + req.path + '\'');
        var vcc = res.locals.vcc;
        var userId = res.locals.userId;
        var batchTypeName = req.body.batchTypeName;
        var batchTypeId = req.body.batchTypeId;
        var batchDetail = {};
        var genericResult = {};
        (0, promises_shared_1.DummyPromise)()
            .then(function (result) {
            if (!userId)
                throw 'SERVER_ERROR_MISSING_PARAMETER';
            return mssql_service_1.MSSql.GetApiUser(vcc, userId);
        })
            .then(function (resultUser) {
            if (!resultUser)
                throw 'SERVER_ERROR_API_USER_NOT_FOUND';
            return mssql_service_1.MSSql.GetBatchDetail(vcc, batchTypeName, batchTypeId);
        })
            .then(function (result) {
            if (!result)
                throw 'SERVER_ERROR_BATCH_DETAIL_NOT_FOUND';
            batchDetail = result;
            var billingBatch = new billing_batch_implementation_1.BillingBatchImplementation();
            return billingBatch.GetBatchBalanceDetail(batchDetail);
        })
            .then(function (result) {
            if (!result)
                throw 'SERVER_ERROR_BATCH_BALANCE_DETAIL_NOT_FOUND';
            if (res.headersSent)
                return;
            genericResult.status = true;
            genericResult.data = { batchDetail: batchDetail, batchBalanceDetail: result.data };
            genericResult.description = 'OK';
            res.json(genericResult);
        })
            .catch(function (err) {
            var clientMsg = (typeof err === 'string' ? err : 'CANNOT_PROCESS_REQUEST');
            var errorMsg = (typeof err === 'string' ? err : err.message || err.description || 'SERVER_ERROR_UNKNOWN_MESSAGE');
            logger.error('[BatchService::get_batch_status] Error al procesar \'' + req.path + '\': ' + errorMsg);
            genericResult.status = false;
            genericResult.description = clientMsg;
            genericResult.error = errorMsg;
            if (res.headersSent)
                return;
            res.json(genericResult);
        });
    });
    app.post('/api/batch/cancel_batch_system_service', function (req, res, next) {
        logger.info('[BatchService::cancel_batch_system_service] Received ' + req.method + ' method on \'' + req.path + '\'');
        var vcc = res.locals.vcc;
        var userId = res.locals.userId;
        var batchTypeName = req.body.batchTypeName;
        var batchTypeId = req.body.batchTypeId;
        var systemService = req.body.systemService;
        var genericResult = {};
        (0, promises_shared_1.DummyPromise)()
            .then(function (result) {
            if (!userId)
                throw 'SERVER_ERROR_MISSING_PARAMETER';
            return mssql_service_1.MSSql.GetApiUser(vcc, userId);
        })
            .then(function (resultUser) {
            if (!resultUser)
                throw 'SERVER_ERROR_API_USER_NOT_FOUND';
            var batchManage = new billing_batch_model_1.BatchManage;
            batchManage.vcc = vcc;
            batchManage.batchTypeName = batchTypeName;
            batchManage.batchTypeId = batchTypeId;
            batchManage.systemService = systemService;
            batchManage.user = userId;
            var billingBatch = new billing_batch_implementation_1.BillingBatchImplementation();
            return billingBatch.CancelBatchSummary(batchManage);
        })
            .then(function (result) {
            if (!result)
                throw 'SERVER_ERROR_BATCH_CANCEL_FAILED';
            var batchRunningStatus = null;
            switch (batchTypeName) {
                case 'TRANSCRIPTION':
                    batchRunningStatus = 'CANCELLING';
                    break;
                case 'STUDY':
                    batchRunningStatus = 'CANCELLING';
                    break;
                case 'EVALUATION_PLAN':
                    batchRunningStatus = 'CANCELLING';
                    break;
            }
            return mssql_service_1.MSSql.SaveBatchStatus(vcc, batchTypeId, systemService, batchRunningStatus);
        })
            .then(function (result) {
            if (!result)
                throw 'SERVER_ERROR_BATCH_STATUS_UPDATE_FAILED';
            if (res.headersSent)
                return;
            genericResult.status = true;
            genericResult.data = result.data;
            genericResult.description = 'OK';
            res.json(genericResult);
        })
            .catch(function (err) {
            var clientMsg = (typeof err === 'string' ? err : 'CANNOT_PROCESS_REQUEST');
            var errorMsg = (typeof err === 'string' ? err : err.message || err.description || 'SERVER_ERROR_UNKNOWN_MESSAGE');
            logger.error('[BatchService::cancel_batch_system_service] Error al procesar \'' + req.path + '\': ' + errorMsg);
            genericResult.status = false;
            genericResult.description = clientMsg;
            genericResult.error = errorMsg;
            if (res.headersSent)
                return;
            res.json(genericResult);
        });
    });
    app.post('/api/batch/resume_batch_system_service', function (req, res, next) {
        logger.info('[BatchService::resume_batch_system_service] Received ' + req.method + ' method on \'' + req.path + '\'');
        var vcc = res.locals.vcc;
        var userId = res.locals.userId;
        var batchTypeName = req.body.batchTypeName;
        var batchTypeId = req.body.batchTypeId;
        var systemService = req.body.systemService;
        var genericResult = {};
        (0, promises_shared_1.DummyPromise)()
            .then(function (result) {
            if (!userId)
                throw 'SERVER_ERROR_MISSING_PARAMETER';
            return mssql_service_1.MSSql.GetApiUser(vcc, userId);
        })
            .then(function (resultUser) {
            if (!resultUser)
                throw 'SERVER_ERROR_API_USER_NOT_FOUND';
            var batchManage = new billing_batch_model_1.BatchManage;
            batchManage.vcc = vcc;
            batchManage.batchTypeName = batchTypeName;
            batchManage.batchTypeId = batchTypeId;
            batchManage.systemService = systemService;
            batchManage.user = userId;
            var billingBatch = new billing_batch_implementation_1.BillingBatchImplementation();
            return billingBatch.ResumeBatchSummary(batchManage);
        })
            .then(function (result) {
            if (!result)
                throw 'SERVER_ERROR_BATCH_RESUME_FAILED';
            var batchRunningStatus = null;
            switch (batchTypeName) {
                case 'TRANSCRIPTION':
                    batchRunningStatus = 'INPROCESS_TRANSCRIPTION';
                    break;
                case 'STUDY':
                    batchRunningStatus = 'RUNNING';
                    break;
                case 'EVALUATION_PLAN':
                    batchRunningStatus = 'RUNNING';
                    break;
            }
            return mssql_service_1.MSSql.SaveBatchStatus(vcc, batchTypeId, systemService, batchRunningStatus);
        })
            .then(function (result) {
            if (!result)
                throw 'SERVER_ERROR_BATCH_STATUS_UPDATE_FAILED';
            if (res.headersSent)
                return;
            genericResult.status = true;
            genericResult.data = result.data;
            genericResult.description = 'OK';
            res.json(genericResult);
        })
            .catch(function (err) {
            var clientMsg = (typeof err === 'string' ? err : 'CANNOT_PROCESS_REQUEST');
            var errorMsg = (typeof err === 'string' ? err : err.message || err.description || 'SERVER_ERROR_UNKNOWN_MESSAGE');
            logger.error('[BatchService::resume_batch_system_service] Error al procesar \'' + req.path + '\': ' + errorMsg);
            genericResult.status = false;
            genericResult.description = clientMsg;
            genericResult.error = errorMsg;
            if (res.headersSent)
                return;
            res.json(genericResult);
        });
    });
    app.post('/api/batch/get_batch_errors', function (req, res, next) {
        logger.info('[BatchService::get_batch_errors] Received ' + req.method + ' method on \'' + req.path + '\'');
        var vcc = res.locals.vcc;
        var userId = res.locals.userId;
        var batchType = req.body.batchType;
        var batchTypeId = req.body.batchTypeId;
        var genericResult = {};
        (0, promises_shared_1.DummyPromise)()
            .then(function (result) {
            return mssql_service_1.MSSql.GetBatchErrors(vcc, batchType, batchTypeId);
        })
            .then(function (result) {
            if (!result)
                throw 'SERVER_ERROR_GET_BATCH_ERRORS_FAILED';
            if (res.headersSent)
                return;
            genericResult.status = true;
            genericResult.data = result;
            genericResult.description = 'OK';
            res.json(genericResult);
        })
            .catch(function (err) {
            var clientMsg = (typeof err === 'string' ? err : 'CANNOT_PROCESS_REQUEST');
            var errorMsg = (typeof err === 'string' ? err : err.message || err.description || 'Error al ejecutar solicitud');
            logger.error('[BatchService::get_batch_errors] Error on \'' + req.path + '\': ' + errorMsg);
            genericResult.status = false;
            genericResult.description = clientMsg;
            genericResult.error = errorMsg;
            if (res.headersSent)
                return;
            res.json(genericResult);
        });
    });
    app.post('/api/batch/check_batch_name', function (req, res, next) {
        var vcc = res.locals.vcc;
        var currentUserId = res.locals.userId;
        var batchType = req.body.type;
        var batchId = req.body.name;
        var genericResult = {};
        (0, promises_shared_1.DummyPromise)()
            .then(function (_) {
            switch (batchType) {
                case 'SCHEDULE': return mssql_service_1.MSSql.GetBatchScheduleById(vcc, batchId);
                case 'TRANSCRIPTION': return mssql_service_1.MSSql.GetProcessByName(vcc, batchId);
                default: throw 'SERVER_ERROR_UNKNOWN_BATCH_TYPE';
            }
        })
            .then(function (result) {
            if (res.headersSent)
                return;
            genericResult.status = true;
            genericResult.data = result ? true : false;
            genericResult.description = 'OK';
            res.json(genericResult);
        })
            .catch(function (err) {
            var clientMsg = (typeof err === 'string' ? err : 'CANNOT_PROCESS_REQUEST');
            var errorMsg = (typeof err === 'string' ? err : err.message || err.description || 'SERVER_ERROR_UNKNOWN_MESSAGE');
            logger.error('[BatchService::check_batch_name] Error al procesar \'' + req.path + '\': ' + errorMsg);
            genericResult.status = false;
            genericResult.description = clientMsg;
            genericResult.error = errorMsg;
            if (res.headersSent)
                return;
            res.json(genericResult);
        });
    });
}
exports.BatchService = BatchService;
