"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.SaveCategories = exports.SaveCategory = exports.GetCategory = exports.CategoryService = void 0;
var moment = require("moment");
var log4js = require("log4js");
var mssql_service_1 = require("../db/mssql.service");
var promises_shared_1 = require("../shared/promises.shared");
var logger = log4js.getLogger('ServerScripts');
function CategoryService(app) {
    app.get('/api/category/get_categories/:search_criteria/:search_expression?', function (req, res, next) {
        logger.info('Se recibió ' + req.method + ' para \'' + req.path + '\'');
        var vcc = res.locals.vcc;
        var search_criteria = req.params.search_criteria || '';
        var search_expression = req.params.search_expression || '';
        (0, promises_shared_1.DummyPromise)()
            .then(function (result) {
            return mssql_service_1.MSSql.GetCategories(vcc, search_criteria, search_expression);
        }).then(function (categories) {
            if (res.headersSent)
                return;
            res.json({ status: true, description: 'SERVER_SUCCESS_MESSAGE', data: categories });
        }).catch(function (err) {
            var clientMsg = (typeof err === 'string' ? err : 'CANNOT_PROCESS_REQUEST');
            var errorMsg = (typeof err === 'string' ? err : err.message || err.description || 'SERVER_ERROR_UNKNOWN_MESSAGE');
            logger.error('Error al procesar \'' + req.path + '\': ' + errorMsg);
            if (res.headersSent)
                return;
            res.json({ status: false, description: clientMsg, error: errorMsg });
        });
    });
    app.get('/api/category/get_category/:category_id', function (req, res, next) {
        logger.info('Se recibió ' + req.method + ' para \'' + req.path + '\'');
        var vcc = res.locals.vcc;
        var categoryId = req.params.category_id;
        (0, promises_shared_1.DummyPromise)()
            .then(function (result) {
            if (!categoryId)
                throw 'CATEGORY_SERVER_ERROR_MISSING_ID';
            return mssql_service_1.MSSql.GetCategory(vcc, categoryId);
        }).then(function (resultCategory) {
            if (res.headersSent)
                return;
            res.json({ status: true, description: 'SERVER_SUCCESS_MESSAGE', data: resultCategory });
        }).catch(function (err) {
            var clientMsg = (typeof err === 'string' ? err : 'CANNOT_PROCESS_REQUEST');
            var errorMsg = (typeof err === 'string' ? err : err.message || err.description || 'SERVER_ERROR_UNKNOWN_MESSAGE');
            logger.error('Error al procesar \'' + req.path + '\': ' + errorMsg);
            if (res.headersSent)
                return;
            res.json({ status: false, description: clientMsg, error: errorMsg });
        });
    });
    app.post('/api/category/create_category', function (req, res, next) {
        logger.info('Se recibió ' + req.method + ' para \'' + req.path + '\'');
        var vcc = res.locals.vcc;
        var userId = res.locals.userId;
        var nowUtc = moment().utc().toDate();
        var newCategory = (req.body || {});
        newCategory.createdDate = nowUtc;
        newCategory.createdByUserId = userId;
        newCategory.lastModifiedDate = nowUtc;
        newCategory.lastModifiedByUserId = userId;
        (0, promises_shared_1.DummyPromise)()
            .then(function (result) {
            return mssql_service_1.MSSql.GetCategoryByName(vcc, newCategory.name);
        }).then(function (resultCategory) {
            if (resultCategory)
                throw 'CATEGORY_SERVER_ERROR_EXISTS';
            return SaveCategory(vcc, newCategory);
        }).then(function (resultSaveCategory) {
            if (!resultSaveCategory)
                throw 'CATEGORY_SERVER_ERROR_ON_SAVE';
            return resultSaveCategory;
        }).then(function (resultSaveCategory) {
            if (res.headersSent)
                return;
            res.json({ status: true, description: (resultSaveCategory ? 'SERVER_SUCCESS_MESSAGE' : 'CATEGORY_SERVER_ERROR_ON_SAVE'), data: resultSaveCategory });
        }).catch(function (err) {
            var clientMsg = (typeof err === 'string' ? err : 'CANNOT_PROCESS_REQUEST');
            var errorMsg = (typeof err === 'string' ? err : err.message || err.description || 'SERVER_ERROR_UNKNOWN_MESSAGE');
            logger.error('Error al procesar \'' + req.path + '\': ' + errorMsg);
            if (res.headersSent)
                return;
            res.json({ status: false, description: clientMsg, error: errorMsg });
        });
    });
    app.post('/api/category/save_category', function (req, res, next) {
        logger.info('Se recibió ' + req.method + ' para \'' + req.path + '\'');
        var vcc = res.locals.vcc;
        var userId = res.locals.userId;
        var nowUtc = moment().utc().toDate();
        var newCategory = (req.body || {});
        newCategory.createdDate = null;
        newCategory.createdByUserId = null;
        newCategory.lastModifiedDate = nowUtc;
        newCategory.lastModifiedByUserId = userId;
        (0, promises_shared_1.DummyPromise)()
            .then(function (resultDeleteCategory) {
            if (!resultDeleteCategory)
                throw 'CATEGORY_SERVER_ERROR_ON_SAVE';
            return SaveCategory(vcc, newCategory);
        }).then(function (resultSaveCategory) {
            if (!resultSaveCategory)
                throw 'CATEGORY_SERVER_ERROR_ON_SAVE';
            return Promise.resolve(true);
        }).then(function (resultSaveCategory) {
            if (res.headersSent)
                return;
            res.json({ status: resultSaveCategory, description: (resultSaveCategory ? 'SERVER_SUCCESS_MESSAGE' : 'CATEGORY_SERVER_ERROR_ON_SAVE'), data: resultSaveCategory });
        }).catch(function (err) {
            var clientMsg = (typeof err === 'string' ? err : 'CANNOT_PROCESS_REQUEST');
            var errorMsg = (typeof err === 'string' ? err : err.message || err.description || 'SERVER_ERROR_UNKNOWN_MESSAGE');
            logger.error('Error al procesar \'' + req.path + '\': ' + errorMsg);
            if (res.headersSent)
                return;
            res.json({ status: false, description: clientMsg, error: errorMsg });
        });
    });
    app.post('/api/category/delete_category', function (req, res, next) {
        logger.info('Se recibió ' + req.method + ' para \'' + req.path + '\'');
        var vcc = res.locals.vcc;
        var category = req.body.category;
        var categoryId = category.name;
        logger.info("[CategoryService::delete_category] Deleting category ".concat(categoryId, " by the user ").concat(res.locals.userId));
        (0, promises_shared_1.DummyPromise)()
            .then(function (result) {
            if (!category)
                throw 'CATEGORY_SERVER_ERROR_MISSING_ID';
            return mssql_service_1.MSSql.DeleteCategory(vcc, categoryId);
        }).then(function (resultDeleteCategory) {
            if (!resultDeleteCategory)
                throw 'CATEGORY_SERVER_ERROR_ON_DELETE';
            return Promise.resolve(true);
        }).then(function (resultDeleteCategory) {
            if (res.headersSent)
                return;
            res.json({ status: resultDeleteCategory, description: (resultDeleteCategory ? 'SERVER_SUCCESS_MESSAGE' : 'CATEGORY_SERVER_ERROR_ON_DELETE'), data: resultDeleteCategory });
        }).catch(function (err) {
            var clientMsg = (typeof err === 'string' ? err : 'CANNOT_PROCESS_REQUEST');
            var errorMsg = (typeof err === 'string' ? err : err.message || err.description || 'SERVER_ERROR_UNKNOWN_MESSAGE');
            logger.error('Error al procesar \'' + req.path + '\': ' + errorMsg);
            if (res.headersSent)
                return;
            res.json({ status: false, description: clientMsg, error: errorMsg });
        });
    });
    app.post('/api/category/delete_categories', function (req, res, next) {
        logger.info('Se recibió ' + req.method + ' para \'' + req.path + '\'');
        var vcc = res.locals.vcc;
        var categories = req.body.categories || [];
        logger.info("[CategoryService::delete_categories] Deleting categories ".concat(categories.map(function (x) { return x.name; }).join(','), " by the user ").concat(res.locals.userId));
        (0, promises_shared_1.DummyPromise)()
            .then(function (result) {
            if (!categories)
                throw 'CATEGORY_SERVER_ERROR_MISSING_ID';
            var promiseArray = [];
            categories.forEach(function (category) {
                promiseArray.push(mssql_service_1.MSSql.DeleteCategory(vcc, category.name));
            });
            return Promise.all(promiseArray);
        }).then(function (resultDeleteCategories) {
            var result;
            if (resultDeleteCategories.filter(function (result) { return (result !== true); }).length > 0) {
                result = false;
                logger.error('No se pudo eliminar todos los elementos solicitados.');
            }
            else {
                result = true;
                logger.info('Categorías eliminadas correctamente');
            }
            return result;
        }).then(function (resultDeleteCategories) {
            if (res.headersSent)
                return;
            res.json({ status: resultDeleteCategories, description: (resultDeleteCategories ? 'SERVER_SUCCESS_MESSAGE' : 'CATEGORY_SERVER_ERROR_ON_DELETE'), data: resultDeleteCategories });
        }).catch(function (err) {
            var clientMsg = (typeof err === 'string' ? err : 'CANNOT_PROCESS_REQUEST');
            var errorMsg = (typeof err === 'string' ? err : err.message || err.description || 'SERVER_ERROR_UNKNOWN_MESSAGE');
            logger.error('Error al procesar \'' + req.path + '\': ' + errorMsg);
            if (res.headersSent)
                return;
            res.json({ status: false, description: clientMsg, error: errorMsg });
        });
    });
}
exports.CategoryService = CategoryService;
function GetCategory(vcc, name) {
    var cacheUpdateNeeded = false;
    return Promise.resolve()
        .then(function () {
        return mssql_service_1.MSSql.GetCategory(vcc, name);
    }).catch(function (err) {
        var errMsg = (typeof err === 'string' ? err : err.message || err.description || 'SERVER_ERROR_UNKNOWN_MESSAGE');
        logger.error('Error al ejecutar GetCategory: ' + errMsg);
    });
}
exports.GetCategory = GetCategory;
function SaveCategory(vcc, category) {
    var promiseArray = [];
    promiseArray.push(mssql_service_1.MSSql.SaveCategory(vcc, category.name, category.description, category.createdDate, category.createdByUserId, category.lastModifiedDate, category.lastModifiedByUserId));
    return Promise.all(promiseArray).then(function (results) {
        if (results.filter(function (result) { return (result !== true); }).length > 0) {
            return false;
        }
        else {
            return true;
        }
    });
}
exports.SaveCategory = SaveCategory;
function SaveCategories(vcc, category) {
    var promiseArray = [];
    if (category) {
        category.forEach(function (categories) {
            promiseArray.push(mssql_service_1.MSSql.SaveCategory(vcc, categories.name, categories.description, categories.createdDate, categories.createdByUserId, categories.lastModifiedDate, categories.lastModifiedByUserId));
        });
    }
    return Promise.all(promiseArray).then(function (results) {
        if (results.filter(function (result) { return (result !== true); }).length > 0) {
            return false;
        }
        else {
            return true;
        }
    });
}
exports.SaveCategories = SaveCategories;
