"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.GetClient = exports.ClientService = void 0;
var moment = require("moment");
var log4js = require("log4js");
var mssql_service_1 = require("../db/mssql.service");
var promises_shared_1 = require("../shared/promises.shared");
var logger = log4js.getLogger('ServerScripts');
function ClientService(app) {
    app.get('/api/client/get_clients/:search_criteria/:search_expression?', function (req, res, next) {
        logger.info('Se recibió ' + req.method + ' para \'' + req.path + '\'');
        var vcc = res.locals.vcc;
        var search_criteria = req.params.search_criteria || '';
        var search_expression = req.params.search_expression || '';
        (0, promises_shared_1.DummyPromise)()
            .then(function (result) {
            return mssql_service_1.MSSql.GetClients(vcc, search_criteria, search_expression);
        }).then(function (clients) {
            if (res.headersSent)
                return;
            res.json({ status: true, description: 'SERVER_SUCCESS_MESSAGE', data: clients });
        }).catch(function (err) {
            var clientMsg = (typeof err === 'string' ? err : 'CANNOT_PROCESS_REQUEST');
            var errorMsg = (typeof err === 'string' ? err : err.message || err.description || 'SERVER_ERROR_UNKNOWN_MESSAGE');
            logger.error('Error al procesar \'' + req.path + '\': ' + errorMsg);
            if (res.headersSent)
                return;
            res.json({ status: false, description: clientMsg, error: errorMsg });
        });
    });
    app.get('/api/client/get_client/:client_id', function (req, res, next) {
        logger.info('Se recibió ' + req.method + ' para \'' + req.path + '\'');
        var vcc = res.locals.vcc;
        var clientId = req.params.client_id;
        (0, promises_shared_1.DummyPromise)()
            .then(function (result) {
            if (!clientId)
                throw 'CLIENT_SERVER_ERROR_MISSING_ID';
            return mssql_service_1.MSSql.GetClient(vcc, clientId);
        }).then(function (resultClient) {
            if (res.headersSent)
                return;
            res.json({ status: true, description: 'SERVER_SUCCESS_MESSAGE', data: resultClient });
        }).catch(function (err) {
            var clientMsg = (typeof err === 'string' ? err : 'CANNOT_PROCESS_REQUEST');
            var errorMsg = (typeof err === 'string' ? err : err.message || err.description || 'SERVER_ERROR_UNKNOWN_MESSAGE');
            logger.error('Error al procesar \'' + req.path + '\': ' + errorMsg);
            if (res.headersSent)
                return;
            res.json({ status: false, description: clientMsg, error: errorMsg });
        });
    });
    app.post('/api/client/create_client', function (req, res, next) {
        logger.info('Se recibió ' + req.method + ' para \'' + req.path + '\'');
        var vcc = res.locals.vcc;
        var userId = res.locals.userId;
        var nowUtc = moment().utc().toDate();
        var newClient = (req.body || {});
        newClient.createdDate = nowUtc;
        newClient.createdByUserId = userId;
        newClient.lastModifiedDate = nowUtc;
        newClient.lastModifiedByUserId = userId;
        (0, promises_shared_1.DummyPromise)()
            .then(function (result) {
            return mssql_service_1.MSSql.GetClient(vcc, newClient.id);
        }).then(function (resultClient) {
            if (resultClient)
                throw 'CLIENT_SERVER_ERROR_EXISTS';
            return SaveClient(vcc, newClient);
        }).then(function (resultSaveClient) {
            if (!resultSaveClient)
                throw 'CLIENT_SERVER_ERROR_ON_SAVE';
            return Promise.resolve(true);
        }).then(function (resultSaveClient) {
            if (res.headersSent)
                return;
            res.json({ status: resultSaveClient, description: (resultSaveClient ? 'SERVER_SUCCESS_MESSAGE' : 'CLIENT_SERVER_ERROR_ON_SAVE'), data: resultSaveClient });
        }).catch(function (err) {
            var clientMsg = (typeof err === 'string' ? err : 'CANNOT_PROCESS_REQUEST');
            var errorMsg = (typeof err === 'string' ? err : err.message || err.description || 'SERVER_ERROR_UNKNOWN_MESSAGE');
            logger.error('Error al procesar \'' + req.path + '\': ' + errorMsg);
            if (res.headersSent)
                return;
            res.json({ status: false, description: clientMsg, error: errorMsg });
        });
    });
    app.post('/api/client/save_client', function (req, res, next) {
        logger.info('Se recibió ' + req.method + ' para \'' + req.path + '\'');
        var vcc = res.locals.vcc;
        var userId = res.locals.userId;
        var nowUtc = moment().utc().toDate();
        var newClient = (req.body || {});
        newClient.createdDate = null;
        newClient.createdByUserId = null;
        newClient.lastModifiedDate = nowUtc;
        newClient.lastModifiedByUserId = userId;
        (0, promises_shared_1.DummyPromise)()
            .then(function (resultDeleteClient) {
            if (!resultDeleteClient)
                throw 'CLIENT_SERVER_ERROR_ON_SAVE';
            return SaveClient(vcc, newClient);
        }).then(function (resultSaveClient) {
            if (!resultSaveClient)
                throw 'CLIENT_SERVER_ERROR_ON_SAVE';
            return Promise.resolve(true);
        }).then(function (resultSaveClient) {
            if (res.headersSent)
                return;
            res.json({ status: resultSaveClient, description: (resultSaveClient ? 'SERVER_SUCCESS_MESSAGE' : 'CLIENT_SERVER_ERROR_ON_SAVE'), data: resultSaveClient });
        }).catch(function (err) {
            var clientMsg = (typeof err === 'string' ? err : 'CANNOT_PROCESS_REQUEST');
            var errorMsg = (typeof err === 'string' ? err : err.message || err.description || 'SERVER_ERROR_UNKNOWN_MESSAGE');
            logger.error('Error al procesar \'' + req.path + '\': ' + errorMsg);
            if (res.headersSent)
                return;
            res.json({ status: false, description: clientMsg, error: errorMsg });
        });
    });
    app.post('/api/client/delete_client', function (req, res, next) {
        logger.info('Se recibió ' + req.method + ' para \'' + req.path + '\'');
        var vcc = res.locals.vcc;
        var client = req.body.client;
        var clientId = client.id;
        logger.info("[ClientService::delete_client] Deleting client ".concat(clientId, " by the user ").concat(res.locals.userId));
        (0, promises_shared_1.DummyPromise)()
            .then(function (result) {
            if (!client)
                throw 'CLIENT_SERVER_ERROR_MISSING_ID';
            return mssql_service_1.MSSql.DeleteClient(vcc, clientId);
        }).then(function (resultDeleteClient) {
            if (!resultDeleteClient)
                throw 'CLIENT_SERVER_ERROR_ON_DELETE';
            return Promise.resolve(true);
        }).then(function (resultDeleteClient) {
            if (res.headersSent)
                return;
            res.json({ status: resultDeleteClient, description: (resultDeleteClient ? 'SERVER_SUCCESS_MESSAGE' : 'CLIENT_SERVER_ERROR_ON_DELETE'), data: resultDeleteClient });
        }).catch(function (err) {
            var clientMsg = (typeof err === 'string' ? err : 'CANNOT_PROCESS_REQUEST');
            var errorMsg = (typeof err === 'string' ? err : err.message || err.description || 'SERVER_ERROR_UNKNOWN_MESSAGE');
            logger.error('Error al procesar \'' + req.path + '\': ' + errorMsg);
            if (res.headersSent)
                return;
            res.json({ status: false, description: clientMsg, error: errorMsg });
        });
    });
    app.post('/api/client/delete_clients', function (req, res, next) {
        logger.info('Se recibió ' + req.method + ' para \'' + req.path + '\'');
        var vcc = res.locals.vcc;
        var clients = req.body.clients || [];
        logger.info("[ClientService::delete_clients] Deleting clients ".concat(clients.map(function (x) { return x.id; }).join(','), " by the user ").concat(res.locals.userId));
        (0, promises_shared_1.DummyPromise)()
            .then(function (result) {
            if (!clients)
                throw 'CLIENT_SERVER_ERROR_MISSING_IDS';
            var promiseArray = [];
            clients.forEach(function (client) {
                promiseArray.push(mssql_service_1.MSSql.DeleteClient(vcc, client.id));
            });
            return Promise.all(promiseArray);
        }).then(function (resultDeleteClients) {
            var result;
            if (resultDeleteClients.filter(function (result) { return (result !== true); }).length > 0) {
                result = false;
                logger.error('No se pudo eliminar todos los elementos solicitados.');
            }
            else {
                result = true;
                logger.info('Clientes eliminados correctamente');
            }
            return result;
        }).then(function (resultDeleteClients) {
            if (res.headersSent)
                return;
            res.json({ status: resultDeleteClients, description: (resultDeleteClients ? 'SERVER_SUCCESS_MESSAGE' : 'CLIENT_SERVER_ERROR_ON_MULTIPLE_DELETE'), data: resultDeleteClients });
        }).catch(function (err) {
            var clientMsg = (typeof err === 'string' ? err : 'CANNOT_PROCESS_REQUEST');
            var errorMsg = (typeof err === 'string' ? err : err.message || err.description || 'SERVER_ERROR_UNKNOWN_MESSAGE');
            logger.error('Error al procesar \'' + req.path + '\': ' + errorMsg);
            if (res.headersSent)
                return;
            res.json({ status: false, description: clientMsg, error: errorMsg });
        });
    });
}
exports.ClientService = ClientService;
function GetClient(vcc, id) {
    var cacheUpdateNeeded = false;
    return Promise.resolve()
        .then(function () {
        return mssql_service_1.MSSql.GetClient(vcc, id);
    }).catch(function (err) {
        var errMsg = (typeof err === 'string' ? err : err.message || err.description || 'SERVER_ERROR_UNKNOWN_MESSAGE');
        logger.error('Error al ejecutar GetClient: ' + errMsg);
    });
}
exports.GetClient = GetClient;
function SaveClient(vcc, client) {
    var promiseArray = [];
    promiseArray.push(mssql_service_1.MSSql.SaveClient(vcc, client.id, client.description, client.country, client.contact, client.createdDate, client.createdByUserId, client.lastModifiedDate, client.lastModifiedByUserId));
    return Promise.all(promiseArray).then(function (results) {
        if (results.filter(function (result) { return (result !== true); }).length > 0) {
            return false;
        }
        else {
            return true;
        }
    });
}
