"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.EfectivityPlanService = void 0;
var log4js = require("log4js");
var mssql_service_1 = require("../db/mssql.service");
var promises_shared_1 = require("../shared/promises.shared");
var logger = log4js.getLogger('ServerScripts');
function EfectivityPlanService(app) {
    app.get('/api/efectivity_plan/get_efectivity_plans/:search_criteria/:search_expression?', function (req, res, next) {
        logger.info('[EfectivityPlanService::get_efectivityplans] Received ' + req.method + ' post method on \'' + req.path + '\'');
        var vcc = res.locals.vcc;
        var search_criteria = req.params.search_criteria || '';
        var search_expression = req.params.search_expression || '';
        var disabled = req.query.disabled === 'true' || false;
        (0, promises_shared_1.DummyPromise)()
            .then(function (_) {
            return mssql_service_1.MSSql.GetEfectivityPlans(vcc, search_criteria, search_expression, disabled);
        })
            .then(function (result) {
            if (res.headersSent)
                return;
            res.json({ status: true, description: 'SERVER_SUCCESS_MESSAGE', data: result });
        })
            .catch(function (err) {
            var clientMsg = (typeof err === 'string' ? err : 'CANNOT_PROCESS_REQUEST');
            var errorMsg = (typeof err === 'string' ? err : err.message || err.description || 'SERVER_ERROR_UNKNOWN_MESSAGE');
            logger.error('Error al procesar \'' + req.path + '\': ' + errorMsg);
            if (res.headersSent)
                return;
            res.json({ status: false, description: clientMsg, error: errorMsg });
        });
    });
    app.post('/api/efectivity_plan/save_evaluation_efectivity_sample', function (req, res, next) {
        logger.info('[EfectivityPlanService::save_evaluationplan] Received ' + req.method + ' method on \'' + req.path + '\'');
        var vcc = res.locals.vcc;
        var efectivityPlanId = req.body.efectivityPlanId;
        var evaluationPlanId = req.body.evaluationPlanId;
        var auditorId = req.body.auditorId;
        var sampleSize = req.body.sampleSize;
        (0, promises_shared_1.DummyPromise)()
            .then(function (_) {
            return mssql_service_1.MSSql.SaveEvaluationEfectivitySample(vcc, efectivityPlanId, evaluationPlanId, auditorId, sampleSize);
        })
            .then(function (resultSave) {
            if (res.headersSent)
                return;
            res.json({ status: resultSave, description: (resultSave ? 'SERVER_SUCCESS_MESSAGE' : 'ERROR_EFECTIVITY_ID_ALREADY_EXISTS'), data: resultSave });
        })
            .catch(function (err) {
            var clientMsg = (typeof err === 'string' ? err : 'CANNOT_PROCESS_REQUEST');
            var errorMsg = (typeof err === 'string' ? err : err.message || err.description || 'SERVER_ERROR_UNKNOWN_MESSAGE');
            logger.error('[EfectivityPlanService::save_evaluation-efectivity-sample] Error on \'' + req.path + '\': ' + errorMsg);
            if (res.headersSent)
                return;
            res.json({ status: false, description: clientMsg, error: errorMsg });
        });
    });
    app.get('/api/efectivity_plan/get_evaluation_result_efectivity_summary/:efectivityPlanId', function (req, res, next) {
        logger.info('[EfectivityPlanService::get_evaluation_result_efectivity_summary] Received ' + req.method + ' post method on \'' + req.path + '\'');
        var vcc = res.locals.vcc;
        var efectivityPlanId = req.params.efectivityPlanId;
        (0, promises_shared_1.DummyPromise)()
            .then(function (_) {
            return mssql_service_1.MSSql.GetEvaluationResultEfectivitySummary(vcc, efectivityPlanId);
        })
            .then(function (result) {
            if (res.headersSent)
                return;
            res.json({ status: true, description: 'SERVER_SUCCESS_MESSAGE', data: result });
        })
            .catch(function (err) {
            var clientMsg = (typeof err === 'string' ? err : 'CANNOT_PROCESS_REQUEST');
            var errorMsg = (typeof err === 'string' ? err : err.message || err.description || 'SERVER_ERROR_UNKNOWN_MESSAGE');
            logger.error('Error al procesar \'' + req.path + '\': ' + errorMsg);
            if (res.headersSent)
                return;
            res.json({ status: false, description: clientMsg, error: errorMsg });
        });
    });
}
exports.EfectivityPlanService = EfectivityPlanService;
