"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
var __generator = (this && this.__generator) || function (thisArg, body) {
    var _ = { label: 0, sent: function() { if (t[0] & 1) throw t[1]; return t[1]; }, trys: [], ops: [] }, f, y, t, g;
    return g = { next: verb(0), "throw": verb(1), "return": verb(2) }, typeof Symbol === "function" && (g[Symbol.iterator] = function() { return this; }), g;
    function verb(n) { return function (v) { return step([n, v]); }; }
    function step(op) {
        if (f) throw new TypeError("Generator is already executing.");
        while (g && (g = 0, op[0] && (_ = 0)), _) try {
            if (f = 1, y && (t = op[0] & 2 ? y["return"] : op[0] ? y["throw"] || ((t = y["return"]) && t.call(y), 0) : y.next) && !(t = t.call(y, op[1])).done) return t;
            if (y = 0, t) op = [op[0] & 2, t.value];
            switch (op[0]) {
                case 0: case 1: t = op; break;
                case 4: _.label++; return { value: op[1], done: false };
                case 5: _.label++; y = op[1]; op = [0]; continue;
                case 7: op = _.ops.pop(); _.trys.pop(); continue;
                default:
                    if (!(t = _.trys, t = t.length > 0 && t[t.length - 1]) && (op[0] === 6 || op[0] === 2)) { _ = 0; continue; }
                    if (op[0] === 3 && (!t || (op[1] > t[0] && op[1] < t[3]))) { _.label = op[1]; break; }
                    if (op[0] === 6 && _.label < t[1]) { _.label = t[1]; t = op; break; }
                    if (t && _.label < t[2]) { _.label = t[2]; _.ops.push(op); break; }
                    if (t[2]) _.ops.pop();
                    _.trys.pop(); continue;
            }
            op = body.call(thisArg, _);
        } catch (e) { op = [6, e]; y = 0; } finally { f = t = 0; }
        if (op[0] & 5) throw op[1]; return { value: op[0] ? op[1] : void 0, done: true };
    }
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.SaveEnvironments = exports.EnvironmentService = void 0;
var moment = require("moment");
var log4js = require("log4js");
var fs = require("fs");
var mssql_service_1 = require("../db/mssql.service");
var promises_shared_1 = require("../shared/promises.shared");
var SSH2Promise = require("ssh2-promise");
var Client = require('ftp');
var config_1 = require("../config/config");
var aws_s3_implementation_1 = require("../implementation/aws-s3.implementation");
var aws_model_1 = require("../models/aws.model");
var audio_service_1 = require("../services/audio.service");
var logger = log4js.getLogger('ServerScripts');
function EnvironmentService(app) {
    app.post('/api/environment/get_environments/:search_criteria/:search_expression?', function (req, res, next) {
        logger.info('[EnvironmentService::get_environments] Received ' + req.method + ' method on \'' + req.path + '\'');
        var vcc = res.locals.vcc;
        var search_criteria = req.params.search_criteria || '';
        var search_expression = req.params.search_expression || '';
        var search_integrated = req.body.includeIntegrated !== undefined ? req.body.includeIntegrated : true;
        (0, promises_shared_1.DummyPromise)()
            .then(function (result) {
            return mssql_service_1.MSSql.GetEnvironments(vcc, search_criteria, search_expression, search_integrated);
        }).then(function (resultEnvironments) {
            if (res.headersSent)
                return;
            res.json({ status: true, description: 'SERVER_SUCCESS_MESSAGE', data: resultEnvironments });
        }).catch(function (err) {
            var environmentMsg = (typeof err === 'string' ? err : 'CANNOT_PROCESS_REQUEST');
            var errorMsg = (typeof err === 'string' ? err : err.message || err.description || 'SERVER_ERROR_UNKNOWN_MESSAGE');
            logger.error('Error al procesar \'' + req.path + '\': ' + errorMsg);
            if (res.headersSent)
                return;
            res.json({ status: false, description: environmentMsg, error: errorMsg });
        });
    });
    app.get('/api/environment/get_environment/:environment_id', function (req, res, next) {
        logger.info('[EnvironmentService::get_environment] Received ' + req.method + ' on \'' + req.path + '\'');
        var vcc = res.locals.vcc;
        var environmentId = req.params.environment_id;
        var environment = null;
        (0, promises_shared_1.DummyPromise)()
            .then(function (result) {
            if (!environmentId)
                throw 'ENVIRONMENT_SERVER_ERROR_MISSING_ID';
            return mssql_service_1.MSSql.GetEnvironment(vcc, environmentId);
        }).
            then(function (env) {
            environment = env;
            return mssql_service_1.MSSql.GetEnvironmentsUsage(vcc, environmentId);
        }).then(function (usages) {
            if (!environment)
                throw 'ENVIRONMENT_SERVER_ERROR_NOT_FOUND';
            if (res.headersSent)
                return;
            environment.dbPassword = '******';
            environment.pbxPassword = '******';
            environment.inUse = (usages.plans.length > 0 || usages.process.length > 0);
            res.json({ status: true, description: 'SERVER_SUCCESS_MESSAGE', data: environment, usages: usages });
        }).catch(function (err) {
            var environmentMsg = (typeof err === 'string' ? err : 'CANNOT_PROCESS_REQUEST');
            var errorMsg = (typeof err === 'string' ? err : err.message || err.description || 'SERVER_ERROR_UNKNOWN_MESSAGE');
            logger.error('[EnvironmentService::get_environment] Error on \'' + req.path + '\': ' + errorMsg);
            if (res.headersSent)
                return;
            res.json({ status: false, description: environmentMsg, error: errorMsg });
        });
    });
    app.get('/api/environment/get_environment_type_by_name/:environment_id', function (req, res, next) {
        logger.info('Se recibio ' + req.method + ' para \'' + req.path + '\'');
        var vcc = res.locals.vcc;
        var environmentId = req.params.environment_id;
        (0, promises_shared_1.DummyPromise)()
            .then(function (result) {
            if (!environmentId)
                throw 'ENVIRONMENT_SERVER_ERROR_MISSING_ID';
            return mssql_service_1.MSSql.GetEnvironmentTypeByName(vcc, environmentId);
        }).then(function (resultEnvironmentType) {
            if (res.headersSent)
                return;
            res.json({ status: true, description: 'SERVER_SUCCESS_MESSAGE', data: resultEnvironmentType });
        }).catch(function (err) {
            var environmentMsg = (typeof err === 'string' ? err : 'CANNOT_PROCESS_REQUEST');
            var errorMsg = (typeof err === 'string' ? err : err.message || err.description || 'SERVER_ERROR_UNKNOWN_MESSAGE');
            logger.error('Error al procesar \'' + req.path + '\': ' + errorMsg);
            if (res.headersSent)
                return;
            res.json({ status: false, description: environmentMsg, error: errorMsg });
        });
    });
    app.get('/api/environment/get_environment_vcc_list/:search_criteria/:search_expression?', function (req, res, next) {
        logger.info('Se recibio ' + req.method + ' para \'' + req.path + '\'');
        var vcc = res.locals.vcc;
        var search_criteria = req.params.search_criteria || '';
        var search_expression = req.params.search_expression || '';
        (0, promises_shared_1.DummyPromise)()
            .then(function (result) {
            return mssql_service_1.MSSql.GetEnvironmentVccs(vcc, search_criteria, search_expression);
        }).then(function (resultEnvironmentVccs) {
            if (res.headersSent)
                return;
            res.json({ status: true, description: 'SERVER_SUCCESS_MESSAGE', data: resultEnvironmentVccs });
        }).catch(function (err) {
            var environmentMsg = (typeof err === 'string' ? err : 'CANNOT_PROCESS_REQUEST');
            var errorMsg = (typeof err === 'string' ? err : err.message || err.description || 'SERVER_ERROR_UNKNOWN_MESSAGE');
            logger.error('Error al procesar \'' + req.path + '\': ' + errorMsg);
            if (res.headersSent)
                return;
            res.json({ status: false, description: environmentMsg, error: errorMsg });
        });
    });
    app.post('/api/environment/create_environment', function (req, res, next) {
        logger.info('[EnvironmentService::create_environment] Received ' + req.method + ' method on \'' + req.path + '\'');
        var vcc = res.locals.vcc;
        var userId = res.locals.userId;
        var nowUtc = moment().utc().toDate();
        var newEnvironment = (req.body || {});
        newEnvironment.bucket = newEnvironment.bucket ? newEnvironment.bucket.trim() : null;
        newEnvironment.apiUser = newEnvironment.apiUser.trim();
        newEnvironment.apiPassword = newEnvironment.apiPassword.trim();
        newEnvironment.apiHost = newEnvironment.apiHost.trim();
        newEnvironment.apiPort = newEnvironment.apiPort;
        newEnvironment.createdDate = nowUtc;
        newEnvironment.createdByUserId = userId;
        newEnvironment.lastModifiedDate = nowUtc;
        newEnvironment.lastModifiedByUserId = userId;
        newEnvironment.dbDisabled = (newEnvironment.dbDisabled ? newEnvironment.dbDisabled : false);
        logger.info('[EnvironmentService::create_environment] Params: ', newEnvironment);
        (0, promises_shared_1.DummyPromise)()
            .then(function (result) {
            return mssql_service_1.MSSql.GetEnvironmentByName(vcc, newEnvironment.name);
        }).then(function (resultEnvironment) {
            if (resultEnvironment)
                throw 'ENVIRONMENT_SERVER_ERROR_EXISTS';
            return SaveEnvironment(vcc, newEnvironment);
        }).then(function (resultSaveEnvironment) {
            return Promise.resolve(resultSaveEnvironment);
        }).then(function (resultSaveEnvironment) {
            if (res.headersSent)
                return;
            res.json({ status: true, description: (resultSaveEnvironment ? 'SERVER_SUCCESS_MESSAGE' : 'ENVIRONMENT_SERVER_ERROR_ON_SAVE'), data: resultSaveEnvironment });
        }).catch(function (err) {
            var environmentMsg = (typeof err === 'string' ? err : 'CANNOT_PROCESS_REQUEST');
            var errorMsg = (typeof err === 'string' ? err : err.message || err.description || 'SERVER_ERROR_UNKNOWN_MESSAGE');
            logger.error('[EnvironmentService::create_environment] Error on \'' + req.path + '\': ' + errorMsg);
            if (res.headersSent)
                return;
            res.json({ status: false, description: environmentMsg, error: errorMsg, detail: { type: 'ERROR', datetime: moment().utc().toDate(), pid: process.pid, source: __filename, stack: err.stack } });
        });
    });
    app.post('/api/environment/save_environment', function (req, res, next) {
        logger.info('[EnvironmentService::save_environment] Received ' + req.method + ' method on \'' + req.path + '\'');
        var vcc = res.locals.vcc;
        var userId = res.locals.userId;
        var nowUtc = moment().utc().toDate();
        var environment = (req.body || {});
        environment.apiUser = environment.apiUser.trim();
        environment.apiPassword = environment.apiPassword.trim();
        environment.apiHost = environment.apiHost.trim();
        environment.apiPort = environment.apiPort;
        environment.createdDate = null;
        environment.createdByUserId = null;
        environment.lastModifiedDate = nowUtc;
        environment.lastModifiedByUserId = userId;
        (0, promises_shared_1.DummyPromise)()
            .then(function (resultDeleteEnvironment) {
            if (!resultDeleteEnvironment)
                throw 'SERVER_ERROR_UNKNOWN_MESSAGE';
            return SaveEnvironment(vcc, environment);
        }).then(function (resultSaveEnvironment) {
            if (!resultSaveEnvironment)
                return false;
            return Promise.resolve(true);
        }).then(function (resultSaveEnvironment) {
            if (res.headersSent)
                return;
            res.json({ status: resultSaveEnvironment, description: (resultSaveEnvironment ? 'SERVER_SUCCESS_MESSAGE' : 'ENVIRONMENT_SERVER_ERROR_ON_SAVE'), data: resultSaveEnvironment });
        }).catch(function (err) {
            var environmentMsg = (typeof err === 'string' ? err : 'CANNOT_PROCESS_REQUEST');
            var errorMsg = (typeof err === 'string' ? err : err.message || err.description || 'SERVER_ERROR_UNKNOWN_MESSAGE');
            logger.error('[EnvironmentService::save_environment] Error on \'' + req.path + '\': ' + errorMsg);
            if (res.headersSent)
                return;
            res.json({ status: false, description: environmentMsg, error: errorMsg });
        });
    });
    app.post('/api/environment/delete_environment', function (req, res, next) {
        logger.info('Se recibio ' + req.method + ' para \'' + req.path + '\'');
        var vcc = res.locals.vcc;
        var environment = req.body.environment;
        var environmentId = environment.name;
        logger.info("[EnvironmentService::delete_environment] Deleting environment ".concat(environmentId, " by the user ").concat(res.locals.userId));
        (0, promises_shared_1.DummyPromise)()
            .then(function (_) {
            if (!environment)
                throw 'ENVIRONMENT_SERVER_ERROR_MISSING_ID';
            return mssql_service_1.MSSql.GetEnvironmentsUsage(vcc, environmentId);
        })
            .then(function (usages) {
            if (usages.plans.length > 0 || usages.process.length > 0) {
                throw 'ENVIRONMENT_IS_USED';
            }
            return mssql_service_1.MSSql.DeleteEnvironment(vcc, environmentId);
        }).then(function (resultDeleteEnvironment) {
            if (!resultDeleteEnvironment)
                throw 'ENVIRONMENT_SERVER_ERROR_ON_DELETE';
            return Promise.resolve(true);
        }).then(function (resultDeleteEnvironment) {
            if (res.headersSent)
                return;
            if (resultDeleteEnvironment)
                logger.info('Ambiente \'' + environmentId + '\' eliminado correctamente');
            res.json({ status: resultDeleteEnvironment, description: (resultDeleteEnvironment ? 'SERVER_SUCCESS_MESSAGE' : 'ENVIRONMENT_SERVER_ERROR_ON_DELETE'), data: resultDeleteEnvironment });
        }).catch(function (err) {
            var environmentMsg = (typeof err === 'string' ? err : 'CANNOT_PROCESS_REQUEST');
            var errorMsg = (typeof err === 'string' ? err : err.message || err.description || 'SERVER_ERROR_UNKNOWN_MESSAGE');
            logger.error('Error al procesar \'' + req.path + '\': ' + errorMsg);
            if (res.headersSent)
                return;
            res.json({ status: false, description: environmentMsg, error: errorMsg });
        });
    });
    app.post('/api/environment/delete_environments', function (req, res, next) {
        logger.info('Se recibio ' + req.method + ' para \'' + req.path + '\'');
        var vcc = res.locals.vcc;
        var environments = req.body.environments || [];
        var environmentIds = environments.map(function (e) { return e.name; });
        logger.info("[EnvironmentService::delete_environments] Deleting environments ".concat(environmentIds.join(','), " by the user ").concat(res.locals.userId));
        (0, promises_shared_1.DummyPromise)()
            .then(function (_) {
            if (!environments)
                throw 'ENVIRONMENT_SERVER_ERROR_MISSING_IDS';
            return mssql_service_1.MSSql.GetEnvironmentsUsage(vcc, environmentIds.toString());
        })
            .then(function (usages) {
            if (usages.plans.length > 0 || usages.process.length > 0) {
                throw 'ENVIRONMENTS_IN_USE';
            }
            var promiseArray = [];
            environmentIds.forEach(function (environment) {
                promiseArray.push(mssql_service_1.MSSql.DeleteEnvironment(vcc, environment));
            });
            return Promise.all(promiseArray);
        }).then(function (resultDeleteEnvironments) {
            var result;
            if (resultDeleteEnvironments.filter(function (result) { return (result !== true); }).length > 0) {
                result = false;
                logger.error('No se pudo eliminar todos los elementos solicitados.');
            }
            else {
                result = true;
                logger.info('Ambientes eliminados correctamente');
            }
            return result;
        }).then(function (resultDeleteEnvironments) {
            if (res.headersSent)
                return;
            var description = (resultDeleteEnvironments ? 'SERVER_SUCCESS_MESSAGE' : 'ENVIRONMENT_SERVER_ERROR_ON_DELETE');
            res.json({
                status: resultDeleteEnvironments,
                description: description,
                data: resultDeleteEnvironments
            });
        }).catch(function (err) {
            var environmentMsg = (typeof err === 'string' ? err : 'CANNOT_PROCESS_REQUEST');
            var errorMsg = (typeof err === 'string' ? err : err.message || err.description || 'SERVER_ERROR_UNKNOWN_MESSAGE');
            logger.error('Error al procesar \'' + req.path + '\': ' + errorMsg);
            if (res.headersSent)
                return;
            res.json({ status: false, description: environmentMsg, error: errorMsg });
        });
    });
    app.post('/api/environment/test_environment', function (req, res) {
        var params = {
            host: req.body.apiHost,
            port: req.body.apiPort,
            password: req.body.apiPassword,
        };
        switch (req.body.type) {
            case 'sftp':
            case 'i-sftp':
                TestSFTP(Object.assign(params, {
                    username: req.body.apiUser,
                    readyTimeout: config_1.FTP_CONNECTION_TIMEOUT
                }), req.body.ftpDirectory).then(function (result) {
                    res.json(result);
                });
                break;
            case 'ftp':
            case 'i-ftp':
                TestFTP(Object.assign(params, {
                    user: req.body.apiUser,
                    connTimeout: config_1.FTP_CONNECTION_TIMEOUT,
                    pasvTimeout: config_1.FTP_CONNECTION_TIMEOUT,
                }), req.body.ftpDirectory).then(function (result) {
                    res.json(result);
                }).catch(function (err) {
                    logger.error('[EnvironmentService::test_environment] Error on \'' + req.path + '\': ' + err);
                    res.json({ status: false, code: err });
                });
                break;
            case 's3':
                TestS3({
                    region: req.body.apiHost,
                    accessKeyId: req.body.apiUser,
                    secretAccessKey: req.body.apiPassword,
                    bucket: req.body.bucket,
                    path: req.body.ftpDirectory
                })
                    .then(function (result) {
                    res.json(result);
                })
                    .catch(function (err) {
                    logger.error('[EnvironmentService::test_environment] Error on \'' + req.path + '\': ' + err);
                    res.json({ status: false, code: err });
                });
                break;
            default:
                break;
        }
    });
    function TestSFTP(config, dir) {
        var ssh2 = new SSH2Promise(config);
        var response = { status: false };
        return new Promise(function (resolve, reject) {
            ssh2.connect().then(function () {
                var sftp = ssh2.sftp();
                sftp.readdir(dir).then(function (result) {
                    response.status = result ? true : false;
                    response.code = 'SERVER_SUCCESS_SFTP_CONNECTION';
                    response.result = result;
                    sftp.close();
                    resolve(response);
                }).catch(function (err) {
                    logger.error('[EnvironmentService::test_environment] Error on TestSFTP ', err.toString());
                    response.code = 'SERVER_ERROR_SFTP_PATH_NOT_FOUND';
                    resolve(response);
                });
            }).catch(function (err) {
                logger.error('[EnvironmentService::test_environment] Error on TestSFTP ', err.toString());
                if (err.level == "client-authentication") {
                    response.code = 'SERVER_ERROR_BAD_USER_OR_PASSWORD';
                }
                else if (err.level == "client-socket") {
                    response.code = 'SERVER_ERROR_BAD_IP';
                }
                else {
                    response.code = 'SERVER_ERROR_SFTP_NOT_CLIENT';
                }
                resolve(response);
            });
        });
    }
    function TestFTPDir(client, dir) {
        return new Promise(function (resolve, reject) {
            client.list(dir, function (err, list) {
                if (err) {
                    logger.error('[EnvironmentService::test_environment] Error on TestFTPDir ', err.toString());
                    reject("SERVER_ERROR_FTP_PATH_NOT_FOUND");
                }
                else {
                    resolve({ status: true, list: list, code: 'SERVER_SUCCESS_FTP_CONNECTION' });
                }
            });
        });
    }
    function TestFTP(config, dir) {
        var _this = this;
        return new Promise(function (resolve, reject) { return __awaiter(_this, void 0, void 0, function () {
            var clientFtp, err_1;
            var _this = this;
            return __generator(this, function (_a) {
                switch (_a.label) {
                    case 0:
                        clientFtp = new Client();
                        _a.label = 1;
                    case 1:
                        _a.trys.push([1, 3, , 4]);
                        return [4, clientFtp.connect(config)];
                    case 2:
                        _a.sent();
                        if (clientFtp) {
                            clientFtp.on('ready', function () { return __awaiter(_this, void 0, void 0, function () {
                                var _a, error_1;
                                return __generator(this, function (_b) {
                                    switch (_b.label) {
                                        case 0:
                                            _b.trys.push([0, 2, , 3]);
                                            _a = resolve;
                                            return [4, TestFTPDir(clientFtp, dir)];
                                        case 1:
                                            _a.apply(void 0, [_b.sent()]);
                                            return [3, 3];
                                        case 2:
                                            error_1 = _b.sent();
                                            reject(error_1);
                                            return [3, 3];
                                        case 3: return [2];
                                    }
                                });
                            }); });
                            clientFtp.on('end', function () {
                                reject('SERVER_ERROR_FTP_CONNECTION_ENDED');
                            });
                            clientFtp.on('close', function (hadError) {
                                reject('SERVER_ERROR_FTP_CONNECTION_REFUSED');
                            });
                            clientFtp.on('error', function (err) {
                                logger.error('[EnvironmentService::test_environment] Error on TestFTP ' + err.toString());
                                reject('SERVER_ERROR_FTP_NOT_CLIENT');
                            });
                        }
                        else {
                            reject('SERVER_ERROR_FTP_NOT_CLIENT');
                        }
                        return [3, 4];
                    case 3:
                        err_1 = _a.sent();
                        reject(err_1);
                        return [3, 4];
                    case 4: return [2];
                }
            });
        }); });
    }
    function TestS3(config) {
        var awsS3 = new aws_s3_implementation_1.AwsS3Implementation(config.region, config.accessKeyId, config.secretAccessKey, config.bucket);
        return (0, promises_shared_1.DummyPromise)()
            .then(function (_) {
            return awsS3.TestBucket(config.bucket);
        })
            .then(function (result) {
            if (!result)
                throw 'SERVER_ERROR_S3_BUCKET_NOT_FOUND';
            return awsS3.TestPath(config.bucket, config.path);
        })
            .then(function (result) {
            if (!result)
                throw 'SERVER_ERROR_S3_PATH_NOT_FOUND';
            return Promise.resolve({ status: true, code: 'SERVER_SUCCESS_S3_CONNECTION' });
        })
            .catch(function (err) {
            return Promise.reject(err);
        });
    }
    app.get('/api/environment/allegro/get_vcc_list/:search_criteria/:search_expression?', function (req, res, next) {
        logger.info('Se recibio ' + req.method + ' para \'' + req.path + '\'');
        var vcc = res.locals.vcc;
        var search_criteria = req.params.search_criteria || '';
        var search_expression = req.params.search_expression || '';
        (0, promises_shared_1.DummyPromise)()
            .then(function (resultEnvironment) {
            return mssql_service_1.MSSql.GetAllegroVccs(vcc, search_criteria, search_expression);
        }).then(function (vccList) {
            if (res.headersSent)
                return;
            res.json({ status: true, description: 'SERVER_SUCCESS_MESSAGE', data: vccList });
        }).catch(function (err) {
            var environmentMsg = (typeof err === 'string' ? err : 'CANNOT_PROCESS_REQUEST');
            var errorMsg = (typeof err === 'string' ? err : err.message || err.description || 'SERVER_ERROR_UNKNOWN_MESSAGE');
            logger.error('Error al procesar \'' + req.path + '\': ' + errorMsg);
            if (res.headersSent)
                return;
            res.json({ status: false, description: environmentMsg, error: errorMsg });
        });
    });
    app.post('/api/environment/allegro/get_bulk_vcc/:environment_id', function (req, res, next) {
        logger.info('Se recibio ' + req.method + ' para \'' + req.path + '\'');
        var vcc = res.locals.vcc;
        var userId = res.locals.userId;
        var nowUtc = moment().utc().toDate();
        var environmentId = req.params.environment_id;
        (0, promises_shared_1.DummyPromise)()
            .then(function (result) {
            return GetEnvironment(vcc, environmentId);
        }).then(function (resultEnvironment) {
            return GetBulkVcc(vcc, resultEnvironment, userId);
        }).then(function (resultSaveBulkVcc) {
            if (!resultSaveBulkVcc)
                return false;
            return Promise.resolve(true);
        }).then(function (resultSaveBulkVcc) {
            if (res.headersSent)
                return;
            res.json({ status: resultSaveBulkVcc, description: (resultSaveBulkVcc ? 'SERVER_SUCCESS_MESSAGE' : 'SERVER_ERROR_UNKNOWN_MESSAGE'), data: resultSaveBulkVcc });
        }).catch(function (err) {
            var environmentMsg = (typeof err === 'string' ? err : 'CANNOT_PROCESS_REQUEST');
            var errorMsg = (typeof err === 'string' ? err : err.message || err.description || 'SERVER_ERROR_UNKNOWN_MESSAGE');
            logger.error('Error al procesar \'' + req.path + '\': ' + errorMsg);
            if (res.headersSent)
                return;
            res.json({ status: false, description: environmentMsg, error: errorMsg });
        });
    });
    app.get('/api/environment/get_amazon_regions', function (req, res, next) {
        var awsRegios = [
            new aws_model_1.AwsRegion('us-east-1', 'AMAZON_REGION_US_EAST_1'),
            new aws_model_1.AwsRegion('us-east-2', 'AMAZON_REGION_US_EAST_2')
        ];
        (0, promises_shared_1.DummyPromise)()
            .then(function (_) {
            if (res.headersSent)
                return;
            res.json({ status: true, description: 'SERVER_SUCCESS_MESSAGE', data: awsRegios });
        })
            .catch(function (err) {
            var environmentMsg = (typeof err === 'string' ? err : 'CANNOT_PROCESS_REQUEST');
            var errorMsg = (typeof err === 'string' ? err : err.message || err.description || 'SERVER_ERROR_UNKNOWN_MESSAGE');
            logger.error('[EnvironmentService::get_amazon_regions] Error on \'' + req.path + '\': ' + errorMsg);
            if (res.headersSent)
                return;
            res.json({ status: false, description: environmentMsg, error: errorMsg });
        });
    });
    app.post('/api/environment/get_environment_audio', function (req, res, next) {
        var _a, _b, _c;
        logger.info('[EnvironmentService::get_environment_audio] Received ' + req.method + ' method on \'' + req.path + '\'');
        var newEnvironment = (req.body || {});
        newEnvironment.relativePath = ((_a = newEnvironment.relativePath) === null || _a === void 0 ? void 0 : _a.trim()) || null;
        newEnvironment.idAudio = ((_b = newEnvironment.idAudio) === null || _b === void 0 ? void 0 : _b.trim()) || null;
        newEnvironment.nameEnvironment = ((_c = newEnvironment.nameEnvironment) === null || _c === void 0 ? void 0 : _c.trim()) || null;
        logger.info('[EnvironmentService::get_environment_audio] Params: ', newEnvironment);
        (0, promises_shared_1.DummyPromise)()
            .then(function (result) {
            if (newEnvironment.idAudio == null)
                throw 'UTTERANCE_SERVER_ERROR_MISSING_AUDIO_ID';
            if (newEnvironment.nameEnvironment == null)
                throw 'ENVIRONMENT_SERVER_ERROR_MISSING_ID';
            return mssql_service_1.MSSql.GetEnvironmentByName(null, newEnvironment.nameEnvironment)
                .catch(function (err) {
                throw err && err.extraContent ? err.extraContent : 'DOES_NOT_EXIST_IN_DATABASE';
            });
        })
            .then(function (resultEnvironment) {
            if (!resultEnvironment)
                throw 'DOES_NOT_EXIST_IN_DATABASE';
            if ((resultEnvironment === null || resultEnvironment === void 0 ? void 0 : resultEnvironment.type) === 'ftp') {
                return GetFTPAudioFile(newEnvironment, resultEnvironment)
                    .catch(function (err) {
                    throw err && err.extraContent ? err.extraContent : 'ERROR_AUDIO_FILE_GET_FTP';
                });
            }
            else if ((resultEnvironment === null || resultEnvironment === void 0 ? void 0 : resultEnvironment.type) === 'sftp') {
                return GetSFTPAudioFile(newEnvironment, resultEnvironment)
                    .catch(function (err) {
                    throw err && err.extraContent ? err.extraContent : 'ERROR_AUDIO_FILE_GET_SFTP';
                });
            }
            return Promise.resolve(null);
        })
            .then(function (result) {
            logger.debug("datraResultado", result);
            if (res.headersSent)
                return;
            var statusR = false;
            if (result && result.status === 1) {
                statusR = true;
            }
            res.json({ status: statusR, description: (result ? 'SERVER_SUCCESS_MESSAGE' : result.errorFunction), data: result });
        })
            .catch(function (err) {
            var environmentMsg = (typeof err === 'string' ? err : 'CANNOT_PROCESS_REQUEST');
            var errorMsg = (typeof err === 'string' ? err : err.message || err.description || 'SERVER_ERROR_UNKNOWN_MESSAGE');
            logger.error('[EnvironmentService::get_environment_audio] Error on \'' + req.path + '\': ' + errorMsg);
            if (res.headersSent)
                return;
            res.json({ status: false, description: environmentMsg, error: errorMsg, detail: { type: 'ERROR', datetime: moment().utc().toDate(), pid: process.pid, source: __filename, stack: err.stack } });
        });
    });
}
exports.EnvironmentService = EnvironmentService;
function GetEnvironment(vcc, id) {
    var cacheUpdateNeeded = false;
    return Promise.resolve()
        .then(function () {
        return mssql_service_1.MSSql.GetEnvironment(vcc, id);
    }).catch(function (err) {
        var errMsg = (typeof err === 'string' ? err : err.message || err.description || 'SERVER_ERROR_UNKNOWN_MESSAGE');
        logger.error('Error al ejecutar GetEnvironment: ' + errMsg);
    });
}
function GetEnvironmentTypeByName(vcc, id) {
    var cacheUpdateNeeded = false;
    return Promise.resolve()
        .then(function () {
        return mssql_service_1.MSSql.GetEnvironmentTypeByName(vcc, id);
    }).catch(function (err) {
        var errMsg = (typeof err === 'string' ? err : err.message || err.description || 'SERVER_ERROR_UNKNOWN_MESSAGE');
        logger.error('Error al ejecutar GetEnvironment: ' + errMsg);
    });
}
function SaveEnvironment(vcc, environment) {
    var promiseArray = [];
    promiseArray.push(mssql_service_1.MSSql.SaveEnvironment(vcc, environment.name, environment.type, environment.description, environment.bucket, environment.ftpDirectory, environment.apiHost, environment.apiPort, environment.apiUser, (environment.apiPassword === '******' ? null : environment.apiPassword), environment.dbDisabled, environment.dbServer, environment.dbPort, environment.dbUser, (environment.dbPassword === '******' ? null : environment.dbPassword), environment.pbxDisabled, environment.pbxServer, environment.pbxPort, environment.pbxUser, (environment.pbxPassword === '******' ? null : environment.pbxPassword), environment.createdDate, environment.createdByUserId, environment.lastModifiedDate, environment.lastModifiedByUserId));
    return Promise.all(promiseArray).then(function (results) {
        if (results.filter(function (result) { return (result !== true); }).length > 0) {
            return false;
        }
        else {
            return true;
        }
    });
}
function SaveEnvironments(vcc, environments) {
    var promiseArray = [];
    if (environments) {
        environments.forEach(function (environment) {
            promiseArray.push(mssql_service_1.MSSql.SaveEnvironment(vcc, environment.name, environment.type, environment.description, environment.ftpDirectory, environment.apiHost, environment.apiPort, environment.apiUser, (environment.apiPassword === '******' ? null : environment.apiPassword), environment.dbDisabled, environment.dbServer, environment.dbPort, environment.dbUser, (environment.dbPassword === '******' ? null : environment.dbPassword), environment.pbxDisabled, environment.pbxServer, environment.pbxPort, environment.pbxUser, (environment.pbxPassword === '******' ? null : environment.pbxPassword), environment.createdDate, environment.createdByUserId, environment.lastModifiedDate, environment.lastModifiedByUserId));
        });
    }
    return Promise.all(promiseArray).then(function (results) {
        if (results.filter(function (result) { return (result !== true); }).length > 0) {
            return false;
        }
        else {
            return true;
        }
    });
}
exports.SaveEnvironments = SaveEnvironments;
function GetBulkVcc(vcc, environment, userId) {
    return (0, promises_shared_1.DummyPromise)()
        .then(function (result) {
        return mssql_service_1.MSSql.GetAllegroIntegrationVccs(environment);
    }).then(function (resultVccs) {
        return mssql_service_1.MSSql.BulkAllegroVccTemp(vcc, environment, resultVccs, userId);
    }).catch(function (err) {
        var errMsg = (typeof err === 'string' ? err : err.message || err.description || 'SERVER_ERROR_UNKNOWN_MESSAGE');
        logger.error('Error al ejecutar BulkVcc: ' + errMsg);
    });
}
function GetFTPAudioFile(newEnvironment, resultEnvironment) {
    var localFilePath = '';
    var localFileName = '';
    var filePachRe = {};
    return (0, promises_shared_1.DummyPromise)()
        .then(function (resultVccs) {
        var audioData = {
            audioId: newEnvironment.idAudio,
            relativePath: newEnvironment.relativePath,
            fileName: null,
            relativeFileName: null,
            section: null,
            part: null
        };
        var environment = {};
        environment.apiHost = resultEnvironment.apiHost;
        environment.apiPort = resultEnvironment.apiPort;
        environment.apiUser = resultEnvironment.apiUser;
        environment.apiPassword = resultEnvironment.apiPassword;
        environment.ftpDirectory = resultEnvironment.ftpDirectory;
        environment.type = resultEnvironment.type;
        return (0, audio_service_1.GetAudioFileFromFtp)(resultEnvironment.vcc, audioData, environment, null)
            .catch(function (err) {
            throw err && err.extraContent ? err.extraContent : 'ERROR_AUDIO_FILE_GET_FTP';
        });
    })
        .then(function (result) {
        localFileName += '.' + result.audioFile.split('.').pop();
        if (resultEnvironment.type === 'ftp' || resultEnvironment.type === 'sftp')
            localFilePath = config_1.PATH_FOLDER_PROCESS_AUDIO + '/' + resultEnvironment.vcc + '/' + newEnvironment.idAudio + '' + localFileName;
        else
            localFilePath = config_1.PATH_FOLDER_TEMP_AUDIO + '/' + resultEnvironment.vcc + '/' + newEnvironment.idAudio + '' + localFileName;
        return new Promise(function (resolve, reject) {
            fs.open(localFilePath, 'w', function (err, fd) {
                if (err) {
                    logger.info('[EnvironmentService::GetFTPAudioFile]', 'getting error creating local audio file:', err);
                    reject('SERVER_ERROR_CREATE_LOCAL_AUDIO_FILE_FAILED');
                }
                fs.write(fd, result.buffer, 0, result.buffer.length, null, function (err) {
                    if (err) {
                        logger.info('[EnvironmentService::GetFTPAudioFile]', 'getting error writing local audio file:', err);
                        reject('SERVER_ERROR_WRITE_LOCAL_AUDIO_FILE_FAILED');
                    }
                    fs.close(fd, function () {
                        resolve('OK');
                    });
                });
            });
        });
    })
        .then(function (result) {
        if (result === 'OK') {
            filePachRe['status'] = 1;
            filePachRe['filePath'] = localFilePath;
            filePachRe['fileName'] = localFileName;
        }
        else {
            logger.error('[EnvironmentService::GetFTPAudioFile]', 'getting errorMessage:', result);
            filePachRe['status'] = -1;
            filePachRe['errorFunction'] = 'GetFTPAudioFile';
            filePachRe['errorMessage'] = result;
            filePachRe['errorDetail'] = result;
        }
        return Promise.resolve(filePachRe);
    })
        .catch(function (err) {
        var errorMessage = typeof err === 'string' ? err : (err.message || err.description);
        logger.error('Error al ejecutar GetFTPAudioFile: ' + errorMessage);
        filePachRe['status'] = -1;
        filePachRe['errorFunction'] = 'GetFTPAudioFile';
        filePachRe['errorMessage'] = errorMessage;
        filePachRe['errorDetail'] = JSON.stringify(err);
        return Promise.resolve(filePachRe);
    });
}
function GetSFTPAudioFile(newEnvironment, resultEnvironment) {
    var localFilePath = '';
    var localFileName = '';
    var filePachRe = {};
    return (0, promises_shared_1.DummyPromise)()
        .then(function (resultVccs) {
        var audioData = {
            audioId: newEnvironment.idAudio,
            relativePath: newEnvironment.relativePath,
            fileName: null,
            relativeFileName: null,
            section: null,
            part: null
        };
        var environment = {};
        environment.apiHost = resultEnvironment.apiHost;
        environment.apiPort = resultEnvironment.apiPort;
        environment.apiUser = resultEnvironment.apiUser;
        environment.apiPassword = resultEnvironment.apiPassword;
        environment.ftpDirectory = resultEnvironment.ftpDirectory;
        environment.type = resultEnvironment.type;
        return (0, audio_service_1.GetAudioFileFromSftp)(resultEnvironment.vcc, audioData, environment, null)
            .catch(function (err) {
            throw err && err.extraContent ? err.extraContent : 'ERROR_AUDIO_FILE_GET_SFTP';
        });
    })
        .then(function (result) {
        localFileName += '.' + result.audioFile.split('.').pop();
        if (resultEnvironment.type === 'ftp' || resultEnvironment.type === 'sftp')
            localFilePath = config_1.PATH_FOLDER_PROCESS_AUDIO + '/' + resultEnvironment.vcc + '/' + newEnvironment.idAudio + '' + localFileName;
        else
            localFilePath = config_1.PATH_FOLDER_TEMP_AUDIO + '/' + resultEnvironment.vcc + '/' + newEnvironment.idAudio + '' + localFileName;
        return new Promise(function (resolve, reject) {
            fs.open(localFilePath, 'w', function (err, fd) {
                if (err) {
                    logger.info('[EnvironmentService::GetSFTPAudioFile]', 'getting error creating local audio file:', err);
                    reject('SERVER_ERROR_CREATE_LOCAL_AUDIO_FILE_FAILED');
                }
                fs.write(fd, result.buffer, 0, result.buffer.length, null, function (err) {
                    if (err) {
                        logger.info('[EnvironmentService::GetSFTPAudioFile]', 'getting error writing local audio file:', err);
                        reject('SERVER_ERROR_WRITE_LOCAL_AUDIO_FILE_FAILED');
                    }
                    fs.close(fd, function () {
                        resolve('OK');
                    });
                });
            });
        });
    })
        .then(function (result) {
        if (result === 'OK') {
            filePachRe['status'] = 1;
            filePachRe['filePath'] = localFilePath;
            filePachRe['fileName'] = localFileName;
        }
        else {
            logger.error('[EnvironmentService::GetSFTPAudioFile]', 'getting errorMessage:', result);
            filePachRe['status'] = -1;
            filePachRe['errorFunction'] = 'GetSFTPAudioFile';
            filePachRe['errorMessage'] = result;
            filePachRe['errorDetail'] = result;
        }
        return Promise.resolve(filePachRe);
    })
        .catch(function (err) {
        var errorMessage = typeof err === 'string' ? err : (err.message || err.description);
        logger.error('Error al ejecutar GetSFTPAudioFile: ' + errorMessage);
        filePachRe['status'] = -1;
        filePachRe['errorFunction'] = 'GetSFTPAudioFile';
        filePachRe['errorMessage'] = errorMessage;
        filePachRe['errorDetail'] = JSON.stringify(err);
        return Promise.resolve(filePachRe);
    });
}
