"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.GroupService = void 0;
var moment = require("moment");
var log4js = require("log4js");
var mssql_service_1 = require("../db/mssql.service");
var promises_shared_1 = require("../shared/promises.shared");
var utilities_1 = require("../shared/utilities");
var logger = log4js.getLogger('ServerScripts');
function GroupService(app) {
    app.get('/api/group/get_groups/:search_criteria/:search_expression?', function (req, res, next) {
        logger.info('Se recibió ' + req.method + ' para \'' + req.path + '\'' + ' | ' + res.locals.instance);
        var instance = res.locals.instance;
        var search_criteria = req.params.search_criteria || '';
        var search_expression = req.params.search_expression || '';
        var disabled = req.query.disabled === 'true' || false;
        (0, promises_shared_1.DummyPromise)()
            .then(function (result) {
            return mssql_service_1.MSSql.GetGroups(instance, search_criteria, search_expression, disabled);
        }).then(function (result) {
            if (res.headersSent)
                return;
            res.json({ status: true, description: 'SERVER_SUCCESS_MESSAGE', data: result });
        }).catch(function (err) {
            var groupMsg = (typeof err === 'string' ? err : 'CANNOT_PROCESS_REQUEST');
            var errorMsg = (typeof err === 'string' ? err : err.message || err.description || 'SERVER_ERROR_UNKNOWN_MESSAGE');
            logger.error('Error al procesar \'' + req.path + '\': ' + errorMsg);
            if (res.headersSent)
                return;
            res.json({ status: false, description: groupMsg, error: errorMsg });
        });
    });
    app.post('/api/group/create_group', function (req, res, next) {
        logger.info('Se recibió ' + req.method + ' para \'' + req.path + '\'' + ' | ' + res.locals.instance);
        var instance = res.locals.instance;
        var userId = res.locals.userId;
        var nowUtc = moment().utc().toDate();
        var newGroup = (req.body || {});
        newGroup.createdDate = nowUtc;
        newGroup.createdByUserId = userId;
        newGroup.lastModifiedDate = nowUtc;
        newGroup.lastModifiedByUserId = userId;
        newGroup.description = newGroup.description.trim();
        (0, promises_shared_1.DummyPromise)()
            .then(function (result) {
            return mssql_service_1.MSSql.GetGroupDescription(instance, newGroup.description);
        }).then(function (result) {
            if (result)
                throw 'GROUP_SERVER_ERROR_EXISTS';
            newGroup.id = (0, utilities_1.guid)();
            return SaveGroup(instance, newGroup);
        }).then(function (result) {
            if (!result)
                throw 'GROUP_SERVER_ERROR_ON_SAVE';
            return Promise.resolve(newGroup.id);
        }).then(function (resultSave) {
            if (res.headersSent)
                return;
            res.json({ status: true, description: (resultSave ? 'SERVER_SUCCESS_MESSAGE' : 'GROUP_SERVER_ERROR_ON_SAVE'), data: resultSave });
        }).catch(function (err) {
            var clientMsg = (typeof err === 'string' ? err : 'CANNOT_PROCESS_REQUEST');
            var errorMsg = (typeof err === 'string' ? err : err.message || err.description || 'SERVER_ERROR_UNKNOWN_MESSAGE');
            logger.error('Error al procesar \'' + req.path + '\': ' + errorMsg);
            if (res.headersSent)
                return;
            res.json({ status: false, description: clientMsg, error: errorMsg });
        });
    });
    app.get('/api/group/get_group/:group_id', function (req, res, next) {
        logger.info('Se recibió ' + req.method + ' para \'' + req.path + '\'');
        var instance = res.locals.instance;
        var groupId = req.params.group_id;
        (0, promises_shared_1.DummyPromise)()
            .then(function (result) {
            if (!groupId)
                throw 'GROUP_SERVER_ERROR_MISSING_ID';
            return mssql_service_1.MSSql.GetGroup(instance, groupId);
        }).then(function (result) {
            if (res.headersSent)
                return;
            res.json({ status: true, description: 'SERVER_SUCCESS_MESSAGE', data: result });
        }).catch(function (err) {
            var clientMsg = (typeof err === 'string' ? err : 'CANNOT_PROCESS_REQUEST');
            var errorMsg = (typeof err === 'string' ? err : err.message || err.description || 'SERVER_ERROR_UNKNOWN_MESSAGE');
            logger.error('Error al procesar \'' + req.path + '\': ' + errorMsg);
            if (res.headersSent)
                return;
            res.json({ status: false, description: clientMsg, error: errorMsg });
        });
    });
    app.post('/api/group/save_group', function (req, res, next) {
        logger.info('Se recibió ' + req.method + ' para \'' + req.path + '\'' + ' | ' + res.locals.instance);
        var instance = res.locals.instance;
        var userId = res.locals.userId;
        var nowUtc = moment().utc().toDate();
        var newGroup = (req.body || {});
        newGroup.createdDate = null;
        newGroup.createdByUserId = null;
        newGroup.lastModifiedDate = nowUtc;
        newGroup.lastModifiedByUserId = userId;
        (0, promises_shared_1.DummyPromise)()
            .then(function (result) {
            return mssql_service_1.MSSql.GetGroupDescription(instance, newGroup.description);
        }).then(function (result) {
            var grupDesc = {};
            grupDesc = result || grupDesc;
            if (result && grupDesc.id !== newGroup.id)
                throw 'GROUP_SERVER_ERROR_EXISTS';
            return SaveGroup(instance, newGroup);
        }).then(function (resultSave) {
            if (!resultSave)
                throw 'GROUP_SERVER_ERROR_ON_SAVE';
            return Promise.resolve(true);
        }).then(function (resultSave) {
            if (res.headersSent)
                return;
            res.json({ status: resultSave, description: (resultSave ? 'SERVER_SUCCESS_MESSAGE' : 'GROUP_SERVER_ERROR_ON_SAVE'), data: resultSave });
        }).catch(function (err) {
            var clientMsg = (typeof err === 'string' ? err : 'CANNOT_PROCESS_REQUEST');
            var errorMsg = (typeof err === 'string' ? err : err.message || err.description || 'SERVER_ERROR_UNKNOWN_MESSAGE');
            logger.error('Error al procesar \'' + req.path + '\': ' + errorMsg);
            if (res.headersSent)
                return;
            res.json({ status: false, description: clientMsg, error: errorMsg });
        });
    });
    app.post('/api/group/delete_group', function (req, res, next) {
        logger.info('Se recibió ' + req.method + ' para \'' + req.path + '\'');
        var instance = res.locals.instance;
        var group = (req.body || {});
        var groupId = group.id;
        logger.info("[GroupService::delete_group] Deleting group ".concat(groupId, " by the user ").concat(res.locals.userId));
        (0, promises_shared_1.DummyPromise)()
            .then(function (result) {
            if (!group)
                throw 'GROUP_SERVER_ERROR_MISSING_ID';
            return mssql_service_1.MSSql.DeleteGroup(instance, groupId);
        }).then(function (resultDelete) {
            if (!resultDelete)
                throw 'GROUP_SERVER_ERROR_ON_DELETE';
            return Promise.resolve(true);
        }).then(function (resultDelete) {
            if (res.headersSent)
                return;
            res.json({ status: resultDelete, description: (resultDelete ? 'SERVER_SUCCESS_MESSAGE' : 'GROUP_SERVER_ERROR_ON_DELETE'), data: resultDelete });
        }).catch(function (err) {
            var clientMsg = (typeof err === 'string' ? err : 'CANNOT_PROCESS_REQUEST');
            var errorMsg = (typeof err === 'string' ? err : err.message || err.description || 'SERVER_ERROR_UNKNOWN_MESSAGE');
            logger.error('Error al procesar \'' + req.path + '\': ' + errorMsg);
            if (res.headersSent)
                return;
            res.json({ status: false, description: clientMsg, error: errorMsg });
        });
    });
    app.post('/api/group/delete_groups', function (req, res, next) {
        logger.info('[GroupService::delete_groups] Received ' + req.method + ' method on \'' + req.path + '\'');
        var vcc = res.locals.vcc;
        var groups = req.body.groups || [];
        var ids = groups.map(function (item) { return item.id; });
        logger.info("[GroupService::delete_groups] Deleting groups ".concat(ids.join(','), " by the user ").concat(res.locals.userId));
        var response;
        return (0, promises_shared_1.DummyPromise)()
            .then(function (_) {
            return mssql_service_1.MSSql.GetGroupsUsage(vcc, ids);
        })
            .then(function (resultNoDeleted) {
            var deleteData = ids;
            var noDelete = [];
            if (resultNoDeleted.length > 0) {
                deleteData = [];
                groups.forEach(function (item) {
                    if (!resultNoDeleted.find(function (noDeleted) { return noDeleted.idGroup === item.id; })) {
                        deleteData.push(item.id);
                    }
                    else {
                        noDelete.push(item.description);
                    }
                });
            }
            return Promise.resolve({ noDelete: noDelete, deleteData: deleteData });
        })
            .then(function (resultDelete) {
            response = resultDelete;
            if (resultDelete.deleteData.length)
                return mssql_service_1.MSSql.DeleteGroups(vcc, resultDelete.deleteData);
            return Promise.resolve(true);
        })
            .then(function (resultDelete) {
            if (res.headersSent)
                return;
            res.json({ status: resultDelete, description: 'SERVER_SUCCESS_MESSAGE', data: response });
        })
            .catch(function (err) {
            var clientMsg = (typeof err === 'string' ? err : 'CANNOT_PROCESS_REQUEST');
            var errorMsg = (typeof err === 'string' ? err : err.message || err.description || 'SERVER_ERROR_UNKNOWN_MESSAGE');
            logger.error('[GroupService::delete_groups] Error on \'' + req.path + '\': ' + errorMsg);
            if (res.headersSent)
                return;
            res.json({ status: false, description: clientMsg, error: errorMsg });
        });
    });
}
exports.GroupService = GroupService;
function SaveGroup(instance, group) {
    var promiseArray = [];
    promiseArray.push(mssql_service_1.MSSql.SaveGroup(instance, group.id, group.description, group.createdDate, group.createdByUserId, group.lastModifiedDate, group.lastModifiedByUserId, group.enabled));
    return Promise.all(promiseArray)
        .then(function (results) {
        if (results.filter(function (result) { return (result !== true); }).length > 0) {
            return false;
        }
        else {
            return true;
        }
    });
}
