"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.SaveIntent = exports.GetIntent = exports.IntentService = void 0;
var moment = require("moment");
var log4js = require("log4js");
var mssql_service_1 = require("../db/mssql.service");
var promises_shared_1 = require("../shared/promises.shared");
var utilities_1 = require("../shared/utilities");
var cognitive_analysis_implementation_1 = require("../implementation/cognitive-analysis.implementation");
var config_1 = require("../config/config");
var utterance_service_1 = require("../services/utterance.service");
var nlu_service_1 = require("../services/nlu.service");
var logger = log4js.getLogger('ServerScripts');
function IntentService(app) {
    var cognitiveService;
    app.get('/api/intent/get_intents/:app_id/:search_criteria/:search_expression?', function (req, res, next) {
        logger.info('[IntentService::get_intents] Received ' + req.method + ' method on \'' + req.path + '\'');
        logger.info('[IntentService::/api/intent/get_intents/] getting app_id:', req.params.app_id, 'search_criteria:', req.params.search_criteria, 'search_expression:', req.params.search_expression);
        var vcc = res.locals.vcc;
        var azure = res.locals.azure;
        logger.info('[IntentService::/api/intent/get_intents/] getting vcc:', vcc, 'azure:', azure);
        var search_criteria = req.params.search_criteria || '';
        var search_expression = req.params.search_expression || '';
        var app_id = req.params.app_id || '';
        var none;
        var arrayId = [];
        var stringId;
        var isSentimentVccSystem;
        (0, promises_shared_1.DummyPromise)()
            .then(function (result) {
            return mssql_service_1.MSSql.GetNLUByAppId(vcc, app_id);
        })
            .then(function (result) {
            vcc = result.vcc;
            if (result.isSystemApp)
                return (0, cognitive_analysis_implementation_1.GetCognitiveService)(null, 'RASA');
            return (0, cognitive_analysis_implementation_1.GetCognitiveService)(vcc, null);
        }).then(function (result) {
            cognitiveService = result;
            return cognitiveService.GetIntentsList(azure.subscriptionRegion, azure.subscriptionKey, app_id, vcc);
        })
            .then(function (result) {
            logger.info('[IntentService::/api/intent/get_intents/] getting result:', result, 'of cognitiveService.GetIntentsList');
            none = result.filter(function (x) { return x.name = 'None'; });
            arrayId = result.map(function (a) { return a.id; });
            stringId = arrayId.join('\',\'');
            logger.info('[IntentService::/api/intent/get_intents/] getting arrayId:', arrayId, 'stringId:', stringId);
            return mssql_service_1.MSSql.GetIntents(vcc, app_id, search_criteria, search_expression, stringId);
        })
            .then(function (intents) {
            logger.info('[IntentService::/api/intent/get_intents/] getting intents:', intents, 'of MSSql.GetIntents');
            if (res.headersSent)
                return;
            res.json({ status: true, description: 'SERVER_SUCCESS_MESSAGE', data: intents });
        })
            .catch(function (err) {
            var clientMsg = (typeof err === 'string' ? err : 'CANNOT_PROCESS_REQUEST');
            var errorMsg = (typeof err === 'string' ? err : err.message || err.description || 'ERROR_EXECUTING_REQUEST');
            logger.error('Error al procesar \'' + req.path + '\': ' + errorMsg);
            logger.error('[IntentService::/api/intent/get_intents/] getting error:', err, 'of MSSql.GetIntents');
            if (res.headersSent)
                return;
            res.json({ status: false, description: clientMsg, error: errorMsg });
        });
    });
    app.get('/api/nlu/get_nlus_intents/:search_criteria/:search_expression?', function (req, res, next) {
        logger.info('[IntentService::get_nlus_intents] Received ' + req.method + ' on \'' + req.path + '\'');
        logger.info('[IntentService::get_nlus_intents] getting search_criteria:', req.params.search_criteria, 'search_expression:', req.params.search_expression);
        var vcc = res.locals.vcc;
        var azure = res.locals.azure;
        logger.info('[IntentService::get_nlus_intents] getting vcc:', vcc, 'azure:', azure);
        var search_criteria = req.params.search_criteria || '';
        var search_expression = req.params.search_expression || '';
        var apps_list;
        var nlus = [];
        (0, promises_shared_1.DummyPromise)()
            .then(function (result) {
            return (0, cognitive_analysis_implementation_1.GetCognitiveService)(vcc, null);
        }).then(function (result) {
            if (!result)
                throw 'SERVER_ERROR_COGNITIVE_SERVICE_NOT_FOUND';
            cognitiveService = result;
            logger.info('[IntentService::get_nlus_intents] Calling GetAppsList from cognitive service', result);
            return cognitiveService.GetAppsList(azure && azure.subscriptionRegion ? azure.subscriptionRegion : null, azure && azure.subscriptionKey ? azure.subscriptionKey : null);
        }).then(function (result) {
            logger.info('[IntentService::get_nlus_intents] getting ' + result.length + ' result from cognitiveService.GetIntentsList');
            var arrayId = [], stringId = '';
            if (result && result.length) {
                apps_list = result;
                arrayId = result.map(function (a) { return a.id; });
                stringId = arrayId.join('\',\'');
            }
            var isSystemApp = false;
            if (cognitiveService.GetCognitiveEngineName() === 'RASA' || cognitiveService.GetCognitiveEngineName() === 'RASAAPI')
                return mssql_service_1.MSSql.GetNLUs(vcc, search_criteria, search_expression, stringId, isSystemApp, true, config_1.USE_RASA_API);
            return mssql_service_1.MSSql.GetNLUs(vcc, search_criteria, search_expression, stringId, isSystemApp);
        }).then(function (nlusResult) {
            nlus = nlusResult ? nlusResult.filter(function (nr) { return nr && nr.isPublished; }) : [];
            logger.info('[IntentService::get_nlus_intents] getting ' + nlus.length + ' nlus from MSSql.GetNLUs');
            var arrayAppId = nlus.map(function (a) { return a.appId; });
            logger.info('[IntentService::get_nlus_intents] getting arrayAppId: ' + arrayAppId.length);
            var promiseArray = [];
            var timeOut = 0;
            arrayAppId.forEach(function (app_id, index) {
                if (cognitiveService.GetCognitiveEngineName() === 'LUIS') {
                    timeOut = Math.floor(index / 5) * 1500;
                }
                else {
                    timeOut = 0;
                }
                promiseArray.push(new Promise(function (resolve, reject) {
                    setTimeout(function () {
                        return resolve(cognitiveService.GetIntentsListDemo(azure && azure.subscriptionRegion ? azure.subscriptionRegion : null, azure && azure.subscriptionKey ? azure.subscriptionKey : null, app_id, null, null, vcc));
                    }, timeOut);
                }));
            });
            return Promise.all(promiseArray);
        }).then(function (results) {
            logger.info('[IntentService::get_nlus_intents] getting results: ' + results.length + 'after Promise.all');
            var intentArray = [];
            results.forEach(function (object, index) {
                object.forEach(function (element) {
                    if (element.name !== 'None') {
                        element.appId = nlus[index].appId;
                        element.appName = nlus[index].name;
                        element.culture = nlus[index].culture;
                        intentArray.push(element);
                    }
                });
            });
            logger.info('[IntentService::get_nlus_intents] getting results:', results.length, 'after results.forEach');
            return Promise.resolve(intentArray);
        }).then(function (intentArray) {
            if (res.headersSent)
                return;
            res.json({ status: true, description: 'SERVER_SUCCESS_MESSAGE', data: intentArray });
        }).catch(function (err) {
            if (typeof err === 'string' && err === 'SERVER_ERROR_COGNITIVE_SERVICE_NOT_FOUND') {
                if (res.headersSent)
                    return;
                res.json({ status: true, description: 'SERVER_SUCCESS_MESSAGE', data: [] });
            }
            else {
                var clientMsg = (typeof err === 'string' ? err : 'ERROR_COGNITIVE_SERVICE_GET_NLUS');
                var errorMsg = (typeof err === 'string' ? err : err.message || err.description || 'SERVER_ERROR_UNKNOWN_MESSAGE');
                logger.error('Error al procesar \'' + req.path + '\': ' + errorMsg);
                logger.error('[NLUService::get_nlus_intents] getting errorMsg:', errorMsg);
                if (res.headersSent)
                    return;
                res.json({ status: false, description: clientMsg, error: clientMsg });
            }
        });
    });
    app.get('/api/intent/get_intent/:intent_id', function (req, res, next) {
        logger.info('Se recibió ' + req.method + ' para \'' + req.path + '\'');
        logger.info('[IntentService::/api/intent/get_intent/] getting intent_id:', req.params.intent_id);
        var vcc = res.locals.vcc;
        var azure = res.locals.azure;
        logger.info('[IntentService::/api/intent/get_intent/] getting vcc:', vcc, 'azure', azure);
        var intentId = req.params.intent_id;
        var intent;
        (0, promises_shared_1.DummyPromise)()
            .then(function (result) {
            if (!intentId)
                throw 'MISSING_INTENT_INPUT_PARAMETER';
            return mssql_service_1.MSSql.GetIntent(vcc, intentId);
        })
            .then(function (resultIntent) {
            logger.info('[IntentService::/api/intent/get_intent/] getting resultIntent:', resultIntent, 'after of MSSql.GetIntent');
            intent = resultIntent;
            return Promise.resolve(true);
        })
            .then(function (result) {
            return intent;
        })
            .then(function (intent) {
            if (res.headersSent)
                return;
            res.json({ status: true, description: 'SERVER_SUCCESS_MESSAGE', data: intent });
        })
            .catch(function (err) {
            var clientMsg = (typeof err === 'string' ? err : 'CANNOT_PROCESS_REQUEST');
            var errorMsg = (typeof err === 'string' ? err : err.message || err.description || 'ERROR_EXECUTING_REQUEST');
            logger.error('Error al procesar \'' + req.path + '\': ' + errorMsg);
            logger.error('[IntentService::/api/intent/get_intent/] getting error:', errorMsg);
            if (res.headersSent)
                return;
            res.json({ status: false, description: clientMsg, error: errorMsg });
        });
    });
    app.post('/api/intent/create_intent', function (req, res, next) {
        logger.info('[IntentService::create_intent] Received ' + req.method + ' method on \'' + req.path + '\'');
        var vcc = res.locals.vcc;
        var azure = res.locals.azure;
        var userId = res.locals.userId;
        var nowUtc = moment().utc().toDate();
        var isSystemApp;
        var newIntent = (req.body || {});
        newIntent.createdDate = nowUtc;
        newIntent.createdByUserId = userId;
        newIntent.lastModifiedDate = nowUtc;
        newIntent.lastModifiedByUserId = userId;
        logger.info('[IntentService::create_intent] getting newIntent:', newIntent);
        var cognitiveEngineName = '';
        var nlu = null;
        (0, promises_shared_1.DummyPromise)()
            .then(function (result) {
            return mssql_service_1.MSSql.GetNLUByAppId(vcc, newIntent.appId);
        })
            .then(function (result) {
            nlu = result;
            if (result.vcc === 'system') {
                nlu = result;
                nlu.vcc = vcc;
                nlu.appId = vcc + '::' + nlu.name;
            }
            isSystemApp = result.isSystemApp;
            if (isSystemApp)
                return (0, cognitive_analysis_implementation_1.GetCognitiveService)(null, 'RASA');
            return (0, cognitive_analysis_implementation_1.GetCognitiveService)(vcc, null);
        })
            .then(function (result) {
            cognitiveService = result;
            if (nlu.isSystemApp) {
                if (nlu.cognitiveEngine === 'RASA')
                    return (0, utterance_service_1.CreateSystemAppIfNotExists)(cognitiveService, nlu.vcc, nlu);
                return (0, utterance_service_1.CreateRasaApiSystemAppIfNotExists)(cognitiveService, nlu.vcc, nlu);
            }
            else {
                return Promise.resolve(true);
            }
        })
            .then(function (result) {
            if (typeof result === "string") {
                nlu.appId = result;
                newIntent.appId = result;
            }
            nlu.lastModifiedDate = nowUtc;
            nlu.lastModifiedByUserId = userId;
            return (0, nlu_service_1.SaveNLU)(nlu.vcc, nlu);
        })
            .then(function (result) {
            return mssql_service_1.MSSql.GetIntentByAppId(nlu.vcc, newIntent.appId, newIntent.name);
        })
            .then(function (resultIntent) {
            logger.info('[IntentService::create_intent] getting result:', resultIntent, 'of MSSql.GetIntentByAppId');
            if (resultIntent)
                throw 'INTENT_ALREADY_EXIST ' + resultIntent.name;
            return cognitiveService.AddIntent(azure.subscriptionRegion, azure.subscriptionKey, newIntent.appId, newIntent.name);
        })
            .then(function (result) {
            logger.info('[IntentService::create_intent] getting result:', result, 'after of cognitiveService.AddIntent');
            newIntent.intentId = result;
            if (!isSystemApp)
                newIntent.color = (0, utilities_1.getRandomHexColor)();
            return SaveIntent(nlu.vcc, newIntent);
        })
            .then(function (resultSaveIntent) {
            logger.info('[IntentService::create_intent] getting resultSaveIntent:', resultSaveIntent, 'after of SaveIntent');
            if (!resultSaveIntent)
                throw 'ERROR_ON_SAVE';
            return resultSaveIntent;
        })
            .then(function (resultSaveIntent) {
            if (res.headersSent)
                return;
            res.json({ status: true, description: (resultSaveIntent ? 'SERVER_SUCCESS_MESSAGE' : 'INTENT_SAVE_FAILED'), data: resultSaveIntent });
        })
            .catch(function (err) {
            var clientMsg = (typeof err === 'string' ? err : 'CANNOT_PROCESS_REQUEST');
            var errorMsg = (typeof err === 'string' ? err : err.message || err.description || 'ERROR_EXECUTING_REQUEST');
            logger.error('Error al procesar \'' + req.path + '\': ' + errorMsg);
            logger.error('[IntentService::create_intent] getting errorMsg:', errorMsg);
            if (res.headersSent)
                return;
            res.json({ status: false, description: clientMsg, error: errorMsg });
        });
    });
    app.post('/api/intent/save_intent', function (req, res, next) {
        logger.info('Se recibió ' + req.method + ' para \'' + req.path + '\'');
        var vcc = res.locals.vcc;
        var userId = res.locals.userId;
        var nowUtc = moment().utc().toDate();
        var newIntent = (req.body || {});
        newIntent.createdDate = null;
        newIntent.createdByUserId = null;
        newIntent.lastModifiedDate = nowUtc;
        newIntent.lastModifiedByUserId = userId;
        logger.info('[IntentService::/api/intent/save_intent] getting newIntent:', newIntent);
        (0, promises_shared_1.DummyPromise)()
            .then(function (result) {
            return SaveIntent(vcc, newIntent);
        })
            .then(function (resultSaveIntent) {
            logger.info('[IntentService::/api/intent/save_intent] getting resultSaveIntent:', resultSaveIntent, 'after of SaveIntent');
            if (!resultSaveIntent)
                throw 'ERROR_ON_SAVE';
            return Promise.resolve(true);
        })
            .then(function (resultSaveIntent) {
            if (res.headersSent)
                return;
            res.json({ status: resultSaveIntent, description: (resultSaveIntent ? 'SERVER_SUCCESS_MESSAGE' : 'INTENT_SAVE_FAILED'), data: resultSaveIntent });
        })
            .catch(function (err) {
            var clientMsg = (typeof err === 'string' ? err : 'CANNOT_PROCESS_REQUEST');
            var errorMsg = (typeof err === 'string' ? err : err.message || err.description || 'ERROR_EXECUTING_REQUEST');
            logger.error('Error al procesar \'' + req.path + '\': ' + errorMsg);
            logger.error('[IntentService::/api/intent/save_intent] getting errorMsg:', errorMsg);
            if (res.headersSent)
                return;
            res.json({ status: false, description: clientMsg, error: errorMsg });
        });
    });
    app.post('/api/intent/delete_intent', function (req, res, next) {
        logger.info('[IntentService::delete_intent] Received ' + req.method + ' method on \'' + req.path + '\'');
        var vcc = res.locals.vcc;
        var azure = res.locals.azure;
        var intent = req.body.intent;
        var appId = intent.appId;
        var intentId = intent.intentId;
        var appIdRasaApi = appId;
        logger.info("[IntentService::delete_intent] Deleting intent ".concat(intentId, " by the user ").concat(res.locals.userId));
        (0, promises_shared_1.DummyPromise)()
            .then(function (result) {
            return mssql_service_1.MSSql.GetNLUByAppId(vcc, appId);
        })
            .then(function (result) {
            vcc = result.vcc;
            if (result.isSystemApp)
                return (0, cognitive_analysis_implementation_1.GetCognitiveService)(null, 'RASA');
            return (0, cognitive_analysis_implementation_1.GetCognitiveService)(vcc, null);
        }).then(function (result) {
            cognitiveService = result;
            if (!intent)
                throw 'MISSING_INTENT_INPUT_PARAMETER';
            return cognitiveService.DeleteAppIntent(azure.subscriptionRegion, azure.subscriptionKey, intent.appId, intent.intentId, vcc);
        })
            .then(function (resultDeleteIntent) {
            logger.info('[IntentService::delete_intent] getting resultDeleteIntent:', resultDeleteIntent, 'after of cognitiveService.DeleteAppIntent');
            if (!intent)
                throw 'MISSING_INTENT_INPUT_PARAMETER';
            return mssql_service_1.MSSql.DeleteIntent(vcc, appId, intentId);
        })
            .then(function (resultDeleteIntent) {
            logger.info('[IntentService::delete_intent] getting resultDeleteIntent:', resultDeleteIntent, 'after of MSSql.DeleteIntent');
            if (!resultDeleteIntent)
                throw 'INTENT_SERVER_ERROR_ON_DELETE';
            return Promise.resolve(true);
        })
            .then(function (resultDeleteIntent) {
            if (res.headersSent)
                return;
            res.json({ status: resultDeleteIntent, description: (resultDeleteIntent ? 'SERVER_SUCCESS_MESSAGE' : 'INTENT_SERVER_ERROR_ON_DELETE'), data: resultDeleteIntent });
        })
            .catch(function (err) {
            var clientMsg = (typeof err === 'string' ? err : 'CANNOT_PROCESS_REQUEST');
            var errorMsg = (typeof err === 'string' ? err : err.message || err.description || 'ERROR_EXECUTING_REQUEST');
            logger.error('Error al procesar \'' + req.path + '\': ' + errorMsg);
            logger.error('[IntentService::delete_intent] getting errorMsg:', errorMsg);
            if (res.headersSent)
                return;
            res.json({ status: false, description: clientMsg, error: errorMsg });
        });
    });
    app.post('/api/intent/delete_intents', function (req, res, next) {
        logger.info('Se recibió ' + req.method + ' para \'' + req.path + '\'');
        var vcc = res.locals.vcc;
        var intents = req.body.intents || [];
        logger.info("[IntentService::delete_intents] Deleting intents ".concat(intents.map(function (x) { return x.name; }).join(','), " by the user ").concat(res.locals.userId));
        (0, promises_shared_1.DummyPromise)()
            .then(function (result) {
            if (!intents)
                throw 'MISSING_INTENTS_INPUT_PARAMETER';
            var promiseArray = [];
            intents.forEach(function (intent) {
                promiseArray.push(mssql_service_1.MSSql.DeleteIntent(vcc, intent.name));
            });
            return Promise.all(promiseArray);
        })
            .then(function (resultDeleteIntents) {
            logger.info('[IntentService::/api/intent/delete_intents] getting resultDeleteIntents:', resultDeleteIntents);
            var result;
            if (resultDeleteIntents.filter(function (result) { return (result !== true); }).length > 0) {
                result = false;
                logger.error('[IntentService::/api/intent/delete_intents] could not delete all requested items');
            }
            else {
                result = true;
                logger.info('[IntentService::/api/intent/delete_intents] intents removed successfully');
            }
            return result;
        })
            .then(function (resultDeleteIntents) {
            logger.error('[IntentService::/api/intent/delete_intents] getting resultDeleteIntents:', resultDeleteIntents);
            if (res.headersSent)
                return;
            res.json({ status: resultDeleteIntents, description: (resultDeleteIntents ? 'SERVER_SUCCESS_MESSAGE' : 'INTENTS_DELETE_FAILED'), data: resultDeleteIntents });
        })
            .catch(function (err) {
            var clientMsg = (typeof err === 'string' ? err : 'CANNOT_PROCESS_REQUEST');
            var errorMsg = (typeof err === 'string' ? err : err.message || err.description || 'ERROR_EXECUTING_REQUEST');
            logger.error('Error al procesar \'' + req.path + '\': ' + errorMsg);
            logger.error('[IntentService::/api/intent/delete_intents] getting errorMsg:', errorMsg);
            if (res.headersSent)
                return;
            res.json({ status: false, description: clientMsg, error: errorMsg });
        });
    });
}
exports.IntentService = IntentService;
function GetIntent(vcc, name) {
    logger.info('[IntentService::GetIntent] getting vcc:', vcc, 'name:', name);
    var cacheUpdateNeeded = false;
    return Promise.resolve()
        .then(function () {
        return mssql_service_1.MSSql.GetIntent(vcc, name);
    })
        .catch(function (err) {
        var errMsg = (typeof err === 'string' ? err : err.message || err.description || 'ERROR_EXECUTING_REQUEST');
        logger.error('[IntentService::GetIntent] getting errMsg:', errMsg);
    });
}
exports.GetIntent = GetIntent;
function SaveIntent(vcc, intent) {
    logger.info('[IntentService::SaveIntent] getting vcc:', vcc, 'intent:', intent);
    var promiseArray = [];
    promiseArray.push(mssql_service_1.MSSql.SaveIntent(vcc, intent.name, intent.intentId, intent.appId, intent.color, intent.createdDate, intent.createdByUserId, intent.lastModifiedDate, intent.lastModifiedByUserId));
    return Promise.all(promiseArray)
        .then(function (results) {
        if (results.filter(function (result) { return (result !== true); }).length > 0) {
            return false;
        }
        else {
            return true;
        }
    });
}
exports.SaveIntent = SaveIntent;
