"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.ProfileImageService = void 0;
var log4js = require("log4js");
var mssql_service_1 = require("../db/mssql.service");
var promises_shared_1 = require("../shared/promises.shared");
var logger = log4js.getLogger('ServerScripts');
function ProfileImageService(app) {
    app.get('/api/profile_image/:type/get_profile_image/:id', function (req, res, next) {
        var instance = res.locals.instance;
        var type = req.params.type;
        var id = req.params.id;
        (0, promises_shared_1.DummyPromise)()
            .then(function (result) {
            if (!type || !id)
                throw 'SERVER_ERROR_MISSING_PARAMETERS';
            return mssql_service_1.MSSql.GetProfileImage(instance, type, id);
        }).then(function (profileImage) {
            if (res.headersSent)
                return;
            res.json({ status: true, description: 'SERVER_SUCCESS_MESSAGE', data: profileImage });
        }).catch(function (err) {
            var errMsg = (typeof err === 'string' ? err : err.message || err.description || 'SERVER_ERROR_UNKNOWN_MESSAGE');
            logger.error('Error al procesar \'' + req.path + '\': ' + errMsg);
            if (res.headersSent)
                return;
            res.json({ status: false, description: 'CANNOT_PROCESS_REQUEST', error: errMsg });
        });
    });
}
exports.ProfileImageService = ProfileImageService;
