"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.JSONToCSV = exports.BeginStudy = exports.CreateStudy = exports.GetStudy = exports.StudyService = void 0;
var moment = require("moment");
var log4js = require("log4js");
var mssql_service_1 = require("../db/mssql.service");
var promises_shared_1 = require("../shared/promises.shared");
var redis_service_1 = require("../db/redis.service");
var billing_batch_implementation_1 = require("../implementation/billing-batch.implementation");
var billing_implementation_1 = require("../implementation/billing.implementation");
var billing_model_1 = require("../models/billing.model");
var platform_service_1 = require("./platform.service");
var logger = log4js.getLogger('ServerScripts');
function StudyService(app) {
    app.post('/api/study/begin_study', function (req, res, next) {
        logger.info('[StudyService::begin_study] Received ' + req.method + ' on \'' + req.path + '\'');
        var vcc = res.locals.vcc;
        var userId = res.locals.userId;
        var studyData = (req.body || {});
        (0, promises_shared_1.DummyPromise)()
            .then(function (_) {
            return BeginStudy(vcc, studyData, userId);
        })
            .then(function (result) {
            if (res.headersSent)
                return;
            res.json({ status: true, description: (result.status ? 'SERVER_SUCCESS_MESSAGE' : result.saveResult ? result.saveResult : 'STUDY_START_FAILED'), data: result.status });
        })
            .catch(function (err) {
            var clientMsg = (typeof err === 'string' ? err : 'CANNOT_PROCESS_REQUEST');
            var errorMsg = (typeof err === 'string' ? err : err.message || err.description || 'SERVER_ERROR_UNKNOWN_MESSAGE');
            logger.error('[StudyService::begin_study] with param:', studyData, 'an error has occured in:', req.path, 'with error meesage:', errorMsg);
            if (res.headersSent)
                return;
            res.json({ status: false, description: clientMsg, error: errorMsg, detail: { type: 'ERROR', datetime: moment().utc().toDate(), pid: process.pid, source: __filename, stack: err.stack } });
        });
    });
    app.post('/api/study/get_studies', function (req, res, next) {
        logger.info('Se recibio ' + req.method + ' para \'' + req.path + '\'');
        var vcc = res.locals.vcc;
        var study = (req.body || {});
        (0, promises_shared_1.DummyPromise)()
            .then(function (result) {
            return mssql_service_1.MSSql.GetStudies(vcc, study.searchCriteria, study.searchExpression, study.startDate, study.endDate, study.type);
        }).then(function (result) {
            if (res.headersSent)
                return;
            res.json({ status: true, description: 'SERVER_SUCCESS_MESSAGE', data: result });
        }).catch(function (err) {
            var clientMsg = (typeof err === 'string' ? err : 'CANNOT_PROCESS_REQUEST');
            var errorMsg = (typeof err === 'string' ? err : err.message || err.description || 'SERVER_ERROR_UNKNOWN_MESSAGE');
            logger.error('Error al procesar \'' + req.path + '\': ' + errorMsg);
            if (res.headersSent)
                return;
            res.json({ status: false, description: clientMsg, error: errorMsg, detail: { type: 'ERROR', datetime: moment().utc().toDate(), pid: process.pid, source: __filename, stack: err.stack } });
        });
    });
    app.get('/api/study/get_study_result_summary/:study_id/?', function (req, res, next) {
        logger.info('[StudyService::get_study_result_summary] Received ' + req.method + ' method on \'' + req.path + '\'');
        var vcc = res.locals.vcc;
        var studyId = req.params.study_id || '';
        (0, promises_shared_1.DummyPromise)()
            .then(function (result) {
            return mssql_service_1.MSSql.GetStudyResultSummary(vcc, studyId, null);
        }).then(function (result) {
            if (res.headersSent)
                return;
            res.json({ status: true, description: 'SERVER_SUCCESS_MESSAGE', data: result });
        }).catch(function (err) {
            var clientMsg = (typeof err === 'string' ? err : 'CANNOT_PROCESS_REQUEST');
            var errorMsg = (typeof err === 'string' ? err : err.message || err.description || 'SERVER_ERROR_UNKNOWN_MESSAGE');
            logger.error('Error al procesar \'' + req.path + '\': ' + errorMsg);
            if (res.headersSent)
                return;
            res.json({ status: false, description: clientMsg, error: errorMsg, detail: { type: 'ERROR', datetime: moment().utc().toDate(), pid: process.pid, source: __filename, stack: err.stack } });
        });
    });
    app.get('/api/study/get_study/:study_id', function (req, res, next) {
        logger.info('Se recibió ' + req.method + ' para \'' + req.path + '\'');
        var vcc = res.locals.vcc;
        var studyId = req.params.study_id;
        (0, promises_shared_1.DummyPromise)()
            .then(function (result) {
            if (!studyId)
                throw 'STUDY_SERVER_ERROR_MISSING_ID';
            return mssql_service_1.MSSql.GetStudy(vcc, studyId);
        }).then(function (resultStudy) {
            if (res.headersSent)
                return;
            res.json({ status: true, description: 'SERVER_SUCCESS_MESSAGE', data: resultStudy });
        }).catch(function (err) {
            var clientMsg = (typeof err === 'string' ? err : 'CANNOT_PROCESS_REQUEST');
            var errorMsg = (typeof err === 'string' ? err : err.message || err.description || 'SERVER_ERROR_UNKNOWN_MESSAGE');
            logger.error('Error al procesar \'' + req.path + '\': ' + errorMsg);
            if (res.headersSent)
                return;
            res.json({ status: false, description: clientMsg, error: errorMsg, detail: { type: 'ERROR', datetime: moment().utc().toDate(), pid: process.pid, source: __filename, stack: err.stack } });
        });
    });
    app.post('/api/study/create_study', function (req, res, next) {
        logger.info('[StudyService::create_study] Received ' + req.method + ' method on \'' + req.path + '\'');
        var vcc = res.locals.vcc;
        var vccType = res.locals.type;
        var userId = res.locals.userId;
        var newStudy = (req.body || {});
        newStudy.name = (newStudy.name || '').toLowerCase();
        (0, promises_shared_1.DummyPromise)()
            .then(function (_) {
            return CreateStudy(vcc, vccType, newStudy, userId);
        })
            .then(function (resultSaveStudy) {
            if (res.headersSent)
                return;
            res.json({ status: true, description: (resultSaveStudy ? 'SERVER_SUCCESS_MESSAGE' : 'STUDY_SERVER_ERROR_ON_SAVE'), data: resultSaveStudy });
        })
            .catch(function (err) {
            var clientMsg = (typeof err === 'string' ? err : 'CANNOT_PROCESS_REQUEST');
            var errorMsg = (typeof err === 'string' ? err : err.message || err.description || 'SERVER_ERROR_UNKNOWN_MESSAGE');
            logger.error('[StudyService::create_study] Error on \'' + req.path + '\': ' + errorMsg);
            if (res.headersSent)
                return;
            res.json({ status: false, description: clientMsg, error: errorMsg, detail: { type: 'ERROR', datetime: moment().utc().toDate(), pid: process.pid, source: __filename, stack: err.stack } });
        });
    });
    app.post('/api/study/save_study', function (req, res, next) {
        logger.info('[StudyService::save_study] Received ' + req.method + ' method on \'' + req.path + '\'');
        var vcc = res.locals.vcc;
        var userId = res.locals.userId;
        var nowUtc = moment().utc().toDate();
        var newStudy = (req.body || {});
        newStudy.name = (newStudy.name || '').toLowerCase();
        newStudy.createdDate = null;
        newStudy.createdByUserId = null;
        newStudy.lastModifiedDate = nowUtc;
        newStudy.lastModifiedByUserId = userId;
        (0, promises_shared_1.DummyPromise)()
            .then(function (result) {
            return SaveStudy(vcc, newStudy);
        }).then(function (resultSaveStudy) {
            if (!resultSaveStudy)
                throw 'STUDY_SERVER_ERROR_ON_SAVE';
            return Promise.resolve(true);
        }).then(function (resultSaveStudy) {
            if (res.headersSent)
                return;
            res.json({ status: resultSaveStudy, description: (resultSaveStudy ? 'SERVER_SUCCESS_MESSAGE' : 'STUDY_SERVER_ERROR_ON_SAVE'), data: resultSaveStudy });
        }).catch(function (err) {
            var clientMsg = (typeof err === 'string' ? err : 'CANNOT_PROCESS_REQUEST');
            var errorMsg = (typeof err === 'string' ? err : err.message || err.description || 'SERVER_ERROR_UNKNOWN_MESSAGE');
            logger.error('Error al procesar \'' + req.path + '\': ' + errorMsg);
            if (res.headersSent)
                return;
            res.json({ status: false, description: clientMsg, error: errorMsg, detail: { type: 'ERROR', datetime: moment().utc().toDate(), pid: process.pid, source: __filename, stack: err.stack } });
        });
    });
    app.post('/api/study/delete_study', function (req, res, next) {
        logger.info('Se recibió ' + req.method + ' para \'' + req.path + '\'');
        var vcc = res.locals.vcc;
        var study = req.body.study;
        var studyId = study.name;
        logger.info("[StudyService::delete_study] Deleting study ".concat(studyId, " by the user ").concat(res.locals.userId));
        (0, promises_shared_1.DummyPromise)()
            .then(function (result) {
            if (!study)
                throw 'STUDY_SERVER_ERROR_MISSING_ID';
            return mssql_service_1.MSSql.DeleteStudy(vcc, studyId);
        }).then(function (resultDeleteStudy) {
            if (!resultDeleteStudy)
                throw 'STUDY_SERVER_ERROR_ON_DELETE';
            return Promise.resolve(true);
        }).then(function (resultDeleteStudy) {
            if (res.headersSent)
                return;
            res.json({ status: resultDeleteStudy, description: (resultDeleteStudy ? 'SERVER_SUCCESS_MESSAGE' : 'STUDY_SERVER_ERROR_ON_DELETE'), data: resultDeleteStudy });
        }).catch(function (err) {
            var clientMsg = (typeof err === 'string' ? err : 'CANNOT_PROCESS_REQUEST');
            var errorMsg = (typeof err === 'string' ? err : err.message || err.description || 'SERVER_ERROR_UNKNOWN_MESSAGE');
            logger.error('Error al procesar \'' + req.path + '\': ' + errorMsg);
            if (res.headersSent)
                return;
            res.json({ status: false, description: clientMsg, error: errorMsg, detail: { type: 'ERROR', datetime: moment().utc().toDate(), pid: process.pid, source: __filename, stack: err.stack } });
        });
    });
    app.post('/api/study/delete_studies', function (req, res, next) {
        logger.info('Se recibió ' + req.method + ' para \'' + req.path + '\'');
        var vcc = res.locals.vcc;
        var studies = req.body.studies || [];
        logger.info("[StudyService::delete_studies] Deleting studies ".concat(studies.map(function (x) { return x.name; }).join(','), " by the user ").concat(res.locals.userId));
        (0, promises_shared_1.DummyPromise)()
            .then(function (result) {
            if (!studies)
                throw 'STUDY_SERVER_ERROR_MISSING_IDS';
            var promiseArray = [];
            studies.forEach(function (study) {
                promiseArray.push(mssql_service_1.MSSql.DeleteStudy(vcc, study.name));
            });
            return Promise.all(promiseArray);
        }).then(function (resultDeleteStudies) {
            var result;
            if (resultDeleteStudies.filter(function (result) { return (result !== true); }).length > 0) {
                result = false;
                logger.error('No se pudo eliminar todos los elementos solicitados.');
            }
            else {
                result = true;
                logger.info('Estudios eliminados correctamente');
            }
            return result;
        }).then(function (resultDeleteStudies) {
            if (res.headersSent)
                return;
            res.json({ status: resultDeleteStudies, description: (resultDeleteStudies ? 'SERVER_SUCCESS_MESSAGE' : 'STUDY_SERVER_ERROR_ON_DELETE'), data: resultDeleteStudies });
        }).catch(function (err) {
            var clientMsg = (typeof err === 'string' ? err : 'CANNOT_PROCESS_REQUEST');
            var errorMsg = (typeof err === 'string' ? err : err.message || err.description || 'SERVER_ERROR_UNKNOWN_MESSAGE');
            logger.error('Error al procesar \'' + req.path + '\': ' + errorMsg);
            if (res.headersSent)
                return;
            res.json({ status: false, description: clientMsg, error: errorMsg, detail: { type: 'ERROR', datetime: moment().utc().toDate(), pid: process.pid, source: __filename, stack: err.stack } });
        });
    });
    app.get('/public/study/get_study_report/:start_date/:end_date/:processes', function (req, res, next) {
        logger.info('Se recibio ' + req.method + ' para \'' + req.path + '\'');
        var startDate = req.params.start_date || '';
        var endDate = req.params.end_date || '';
        var processes = req.params.processes || '';
        (0, promises_shared_1.DummyPromise)()
            .then(function (result) {
            return mssql_service_1.MSSql.GetStudyReport(startDate, endDate, processes);
        })
            .then(function (result) {
            return Promise.resolve(JSONToCSV(result));
        }).then(function (result) {
            if (res.headersSent)
                return;
            var fileName = [startDate.replace('-', ''), endDate.replace('-', ''), processes.replace(',', '_')].join('_');
            res.setHeader('Content-disposition', 'attachment; filename=' + fileName + '.csv');
            res.set('Content-Type', 'text/csv');
            res.status(200).send(result);
        }).catch(function (err) {
            var clientMsg = (typeof err === 'string' ? err : 'CANNOT_PROCESS_REQUEST');
            var errorMsg = (typeof err === 'string' ? err : err.message || err.description || 'SERVER_ERROR_UNKNOWN_MESSAGE');
            logger.error('Error al procesar \'' + req.path + '\': ' + errorMsg);
            if (res.headersSent)
                return;
            res.json({ status: false, description: clientMsg, error: errorMsg, detail: { type: 'ERROR', datetime: moment().utc().toDate(), pid: process.pid, source: __filename, stack: err.stack } });
        });
    });
}
exports.StudyService = StudyService;
function GetStudy(vcc, studyId) {
    studyId = (studyId || '').toLowerCase();
    var cacheUpdateNeeded = false;
    return Promise.resolve()
        .then(function () {
        return mssql_service_1.MSSql.GetStudy(vcc, studyId);
    }).catch(function (err) {
        var errMsg = (typeof err === 'string' ? err : err.message || err.description || 'SERVER_ERROR_UNKNOWN_MESSAGE');
        logger.error('Error al ejecutar GetStudy: ' + errMsg);
    });
}
exports.GetStudy = GetStudy;
function CreateStudy(vcc, vccType, newStudy, userId) {
    var nowUtc = moment().utc().toDate();
    newStudy.createdDate = nowUtc;
    newStudy.createdByUserId = userId;
    newStudy.lastModifiedDate = nowUtc;
    newStudy.lastModifiedByUserId = userId;
    return (0, promises_shared_1.DummyPromise)()
        .then(function (_) {
        return mssql_service_1.MSSql.GetStudyByName(vcc, newStudy.name);
    })
        .then(function (resultStudy) {
        if (resultStudy)
            throw 'SERVER_ERROR_STUDY_ALREADY_EXISTS';
        return (0, platform_service_1.GetPlatformVersion)();
    })
        .then(function (result) {
        if (result === null)
            throw 'SERVER_ERROR_INSTALLATIONID_MISSING';
        var batch = {};
        batch.installationId = result.installationId;
        batch.vcc = vcc;
        batch.batchTypeName = newStudy.type === 'SPEECH' ? 'STUDY' : 'TEMPLATE';
        batch.batchTypeId = newStudy.name;
        var billing = new billing_batch_implementation_1.BillingBatchImplementation();
        return billing.FindBatchId(batch);
    })
        .then(function (result) {
        if (result && result.result && result.result.length > 0)
            throw 'SERVER_ERROR_STUDY_ID_INVALID';
        return SaveStudy(vcc, newStudy);
    })
        .then(function (resultSaveStudy) {
        if (!resultSaveStudy)
            throw 'SERVER_ERROR_SAVE_STUDY_FAILED';
        logger.info('[StudyService::CreateStudy] Finishing function');
        return Promise.resolve({ resultSaveStudy: resultSaveStudy });
    })
        .catch(function (err) {
        logger.error('[StudyService::CreateStudy] Function failed');
        return Promise.reject(err);
    });
}
exports.CreateStudy = CreateStudy;
function BeginStudy(vcc, studyData, userId) {
    var currentQueueId = '';
    var study = {};
    var channelSummaries = [];
    var totalSyncInteraction = 0;
    var totalAudios = 0;
    var totalAsyncInteraction = 0;
    var totalAsyncDetail = 0;
    var averageModelRequest = 0;
    var nowUtc = moment().utc().toDate();
    var batch = {};
    var billingVccRates = new billing_model_1.BillingVCCRates();
    var status = false;
    var saveResult = null;
    return (0, promises_shared_1.DummyPromise)()
        .then(function (_) {
        return mssql_service_1.MSSql.GetStudy(vcc, studyData.studyId);
    })
        .then(function (result) {
        if (!result)
            throw 'SERVER_ERROR_STUDY_NOT_FOUND';
        study = result;
        var billing = new billing_implementation_1.BillingImplementation();
        return billing.GetVCCRates(vcc, study.processId);
    })
        .then(function (result) {
        if (result === null)
            throw 'SERVER_ERROR_RATES_NOT_FOUND';
        billingVccRates.SetBillingVCCRates(result);
        var promiseArray = [];
        promiseArray.push((0, platform_service_1.GetPlatformVersion)());
        promiseArray.push(mssql_service_1.MSSql.GetTemplateThemeItems(vcc, study.templateId, null));
        promiseArray.push(mssql_service_1.MSSql.GetTemplateThemeModels(vcc, study.templateId, null));
        promiseArray.push(mssql_service_1.MSSql.GetProcessInteractionsHistory(vcc, study.processId, billingVccRates.cognitiveAnalysisAverageSecondsPerRequest));
        return Promise.all(promiseArray);
    })
        .then(function (result) {
        if (result[0] === null)
            throw 'INSTALLATIONID_MISSING';
        if (result[1] === null)
            throw 'QUERIES_INFO_MISSING';
        if (result[2] === null)
            throw 'MODELS_INFO_MISSING';
        if (result[3] === null)
            throw 'INTERACTIONS_INFO_MISSING';
        var modelIntents = result[2].map(function (mi) { return mi.id; });
        var models = modelIntents.map(function (i) { return i.split('|')[0]; });
        var nluModels = models.filter(function (item, i, ar) { return ar.indexOf(item) === i; });
        channelSummaries = result[3];
        channelSummaries.map(function (cs) {
            if (cs) {
                if (cs.interactionType === 'CALL') {
                    totalSyncInteraction = cs.totalRecords;
                    totalAudios = cs.audios;
                    averageModelRequest += cs.totalCallPhrases || cs.averageModelRequest || 0;
                }
                else {
                    totalAsyncInteraction += cs.totalInteractions;
                    totalAsyncDetail += cs.totalInteractionDetails;
                    averageModelRequest += cs.totalTextPhrases || cs.averageModelRequest || 0;
                    ;
                }
            }
        });
        batch.installationId = result[0].installationId;
        batch.vcc = vcc;
        batch.totalAsync = 0;
        batch.totalAsyncDetail = 0;
        batch.totalSync = 0;
        batch.totalAudio = 0;
        batch.durationTime = 0;
        batch.roundedTime = 0;
        batch.totalQueries = result[1].length;
        batch.totalModels = nluModels.length;
        batch.modelRequests = billingVccRates.GetCognitiveAnalysisTotalAveragePhrasesFromProcess();
        batch.batchTypeName = study.type === 'SPEECH' ? 'STUDY' : 'TEMPLATE';
        batch.batchTypeId = study.name;
        batch.templateId = study.templateId;
        batch.processId = study.processId;
        batch.userApi = userId;
        batch.startDate = nowUtc;
        var billing = new billing_batch_implementation_1.BillingBatchImplementation();
        return billing.SaveBatchSummary(batch);
    })
        .then(function (result) {
        if (typeof result !== 'boolean') {
            if (result) {
                if (result.status === true) {
                    status = true;
                    study.status = 0.1;
                    var lastModifiedDate = moment().utc().toDate();
                    return mssql_service_1.MSSql.SaveStudyStatus(vcc, studyData.processId, study.templateId, studyData.studyId, study.status, null, null, lastModifiedDate, userId);
                }
                else {
                    status = false;
                    saveResult = result.error ? result.error : 'SERVER_ERROR_CAN_NOT_SAVE_BATCH';
                }
            }
            else {
                return Promise.resolve(false);
            }
        }
        else {
            return Promise.resolve(false);
        }
    })
        .then(function (_) {
        if (status === true) {
            currentQueueId = vcc;
            currentQueueId += ':';
            currentQueueId += study.processId;
            currentQueueId += study.type === 'SPEECH' ? ':study:' : ':template:';
            currentQueueId += study.name;
            return redis_service_1.Redis.CreateQueueL('Process', currentQueueId);
        }
        return Promise.resolve(true);
    })
        .then(function (_) {
        return Promise.resolve({ status: status, saveResult: saveResult });
    })
        .catch(function (err) {
        logger.error('[StudyService::BeginStudy] Function failed');
        return Promise.reject(err);
    });
}
exports.BeginStudy = BeginStudy;
function SaveStudy(vcc, study) {
    var promiseArray = [];
    promiseArray.push(mssql_service_1.MSSql.SaveStudy(vcc, study.processId, study.templateId, study.name, study.description, study.status, study.type, study.batchScheduleId, study.createdDate, study.createdByUserId, study.lastModifiedDate, study.lastModifiedByUserId));
    return Promise.all(promiseArray).then(function (results) {
        if (results.filter(function (result) { return (result !== true); }).length > 0) {
            return Promise.resolve(false);
        }
        else {
            return Promise.resolve(true);
        }
    });
}
function JSONToCSV(data) {
    var items = data, replacer = function (key, value) { return value === null ? '' : value; }, header = Object.keys(items[0]), csv = items.map(function (row) { return header.map(function (fieldName) { return JSON.stringify(row[fieldName], replacer); }).join(','); });
    csv.unshift(header.join(','));
    csv = csv.join('\r\n');
    return csv;
}
exports.JSONToCSV = JSONToCSV;
