"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.GetItemFromQueueId = exports.TaskService = void 0;
var jwt = require("jsonwebtoken");
var log4js = require("log4js");
var redis_service_1 = require("../db/redis.service");
var promises_shared_1 = require("../shared/promises.shared");
var transcriptions_upload_implementation_1 = require("../implementation/transcriptions-upload.implementation");
var jwt_config_1 = require("../config/jwt.config");
var transcriptions_download_implementation_1 = require("../implementation/transcriptions-download.implementation");
var logger = log4js.getLogger('ServerScripts');
function TaskService(app) {
    app.post('/api/public/task/update_transcription_queue/:queue_id', function (req, res, next) {
        logger.info('[TaskService::update_transcription_queue] Received ' + req.method + ' on \'' + req.path + '\'');
        var queueId = req.params.queue_id;
        var callbackData = (req.body || {});
        var storage = null;
        var executionCode = Date.now().toString();
        var processInteraction = null;
        var _a = GetItemFromQueueId(queueId), vcc = _a.vcc, processId = _a.processId;
        logger.info("[TaskService::update_transcription_queue] Received speechmatics callback for ".concat(queueId, " queue"));
        logger.info("[TaskService::update_transcription_queue] Received:", callbackData);
        (0, promises_shared_1.DummyPromise)()
            .then(function (_) {
            var decodedToken = null;
            try {
                decodedToken = jwt.verify((req.headers['authorization'] || '').replace('Bearer ', ''), jwt_config_1.JWT_SECRET);
            }
            catch (e) {
                decodedToken = null;
                logger.warn("[TaskService::update_transcription_queue] Received invalid token: ".concat(req.headers['authorization']));
                throw 'SERVER_ERROR_CANNOT_DECODE_ACCESS_TOKEN';
            }
            var transcriptionsUpload = new transcriptions_upload_implementation_1.TranscriptionsUploadImplementation();
            if (decodedToken.userId === transcriptionsUpload.externalUserId
                &&
                    decodedToken.vcc === transcriptionsUpload.externalUserVcc)
                return Promise.resolve(true);
            else
                throw 'SERVER_ERROR_ACCESS_TOKEN_VALIDATION_FAILED';
        })
            .then(function (_) {
            return redis_service_1.Redis.DeleteInsertOnList('TranscriptionsStandby', 'TranscriptionsStandbyTask', queueId);
        })
            .then(function (result) {
            if (!result)
                throw 'SERVER_ERROR_QUEUE_EXCHANGE_FAILED';
            storage = "".concat(vcc, ":").concat(processId);
            return redis_service_1.Redis.GetProcessInteraction(storage, queueId);
        })
            .then(function (result) {
            if (!result) {
                logger.warn("[TaskService::update_transcription_queue] ProcessInteraction for \"".concat(queueId, "\" was not found on storage \"").concat(vcc, ":").concat(processId, "\""));
                throw 'SERVER_ERROR_INTERACTION_DATA_WAS_NOT_FOUND';
            }
            processInteraction = result;
            logger.info("[TaskService::update_transcription_queue] Saving content from callback for ".concat(queueId, " queue"));
            var transcriptionsDownload = new transcriptions_download_implementation_1.TranscriptionsDownloadImplementation();
            return transcriptionsDownload.DownloadTranscription(processInteraction, callbackData, queueId, executionCode);
        })
            .then(function (result) {
            return redis_service_1.Redis.SaveProcessInteraction(storage, queueId, result);
        })
            .then(function (_) {
            logger.info("[TaskService::update_transcription_queue] Inserting the ".concat(queueId, " queue to download"));
            return redis_service_1.Redis.DeleteInsertOnList('TranscriptionsStandbyTask', 'TranscriptionsDownload', queueId);
        })
            .then(function (result) {
            if (res.headersSent)
                return;
            res.json({ status: true, description: 'SERVER_SUCCESS_MESSAGE', data: result });
        })
            .catch(function (err) {
            var clientMsg = (typeof err === 'string' ? err : 'CANNOT_PROCESS_REQUEST');
            var errorMsg = (typeof err === 'string' ? err : err.message || err.description || 'SERVER_ERROR_UNKNOWN_MESSAGE');
            logger.error('[TaskService::update_transcription_queue] Error on \'' + req.path + '\': ' + errorMsg);
            if (res.headersSent)
                return;
            redis_service_1.Redis.lremAsync('TranscriptionsStandbyTask', 1, queueId);
            res.json({ status: false, description: clientMsg, error: errorMsg });
        });
    });
}
exports.TaskService = TaskService;
function GetItemFromQueueId(queueId) {
    var vcc = '';
    var processId = '';
    var interactionType = '';
    var interactionId = '';
    var interactionSection = 0;
    var interactionPart = 0;
    var item = queueId.split(':');
    if ((item === null || item === void 0 ? void 0 : item.length) === 6) {
        vcc = item[0];
        processId = item[1];
        interactionType = item[2];
        interactionId = item[3];
        interactionSection = Number(item[4]);
        interactionPart = Number(item[5]);
    }
    else {
        logger.error('[TaskService::GetItemFromQueueId] QueueId has not conrrect format');
        throw 'SERVER_ERROR_QUEUEID_INVALID_FORMAT';
    }
    return { vcc: vcc, processId: processId, interactionType: interactionType, interactionId: interactionId, interactionSection: interactionSection, interactionPart: interactionPart };
}
exports.GetItemFromQueueId = GetItemFromQueueId;
