"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.WorkGroupSupervisorService = void 0;
var moment = require("moment");
var log4js = require("log4js");
var mssql_service_1 = require("../db/mssql.service");
var promises_shared_1 = require("../shared/promises.shared");
var logger = log4js.getLogger('ServerScripts');
function WorkGroupSupervisorService(app) {
    app.post('/api/workgroupssupervisor/get_workgroupssupervisor', function (req, res, next) {
        logger.info('Se recibio ' + req.method + ' para \'' + req.path + '\'');
        var vcc = res.locals.vcc;
        var workGroupSupervisor = (req.body || {});
        (0, promises_shared_1.DummyPromise)()
            .then(function (result) {
            return mssql_service_1.MSSql.GetWorkGroupsSupervisor(vcc, workGroupSupervisor.searchCriteria, workGroupSupervisor.searchExpression);
        }).then(function (result) {
            if (res.headersSent)
                return;
            res.json({ status: true, description: 'SERVER_SUCCESS_MESSAGE', data: result });
        }).catch(function (err) {
            var clientMsg = (typeof err === 'string' ? err : 'CANNOT_PROCESS_REQUEST');
            var errorMsg = (typeof err === 'string' ? err : err.message || err.description || 'SERVER_ERROR_UNKNOWN_MESSAGE');
            logger.error('Error al procesar \'' + req.path + '\': ' + errorMsg);
            if (res.headersSent)
                return;
            res.json({ status: false, description: clientMsg, error: errorMsg, detail: { type: 'ERROR', datetime: moment().utc().toDate(), pid: process.pid, source: __filename, stack: err.stack } });
        });
    });
    app.post('/api/workgroupsupervisor/create_workgroupsupervisor', function (req, res, next) {
        logger.info('[WorkGroupSupervisor::create_workgroupsupervisor] Received ' + req.method + ' method on \'' + req.path + '\'');
        var vcc = res.locals.vcc;
        var userId = res.locals.userId;
        var nowUtc = moment().utc().toDate();
        var workGroupSupervisorCollection = (req.body.workGroupSupervisorCollection || {});
        var flagEdit = req.body.flagEdit;
        (0, promises_shared_1.DummyPromise)()
            .then(function (result) {
            if (flagEdit)
                return mssql_service_1.MSSql.DeleteWorkGroupSupervisor(vcc, workGroupSupervisorCollection[0].description, workGroupSupervisorCollection[0].supervisorId);
            else
                return false;
        })
            .then(function (result) {
            var promiseArray = [];
            workGroupSupervisorCollection.forEach(function (wgsc) {
                promiseArray.push(mssql_service_1.MSSql.SaveWorkGroupSupervisor(vcc, userId, nowUtc, wgsc));
            });
            return Promise.all(promiseArray);
        })
            .then(function (results) {
            var result = (results && results.length > 0) ? !results.includes(false) : true;
            return result;
        }).then(function (resultSaveWorkGroupSupervisor) {
            if (res.headersSent)
                return;
            res.json({ status: resultSaveWorkGroupSupervisor, description: (resultSaveWorkGroupSupervisor ? 'SERVER_SUCCESS_MESSAGE' : 'WORKGROUPSUPERVISOR_SERVER_ERROR_ON_MULTIPLE_SAVE'), data: resultSaveWorkGroupSupervisor });
        }).catch(function (err) {
            var clientMsg = (typeof err === 'string' ? err : 'CANNOT_PROCESS_REQUEST');
            var errorMsg = (typeof err === 'string' ? err : err.message || err.description || 'SERVER_ERROR_UNKNOWN_MESSAGE');
            logger.error('[WorkGroupSupervisor::create_workgroupsupervisor] Error on \'' + req.path + '\': ' + errorMsg);
            if (res.headersSent)
                return;
            res.json({ status: false, description: clientMsg, error: errorMsg });
        });
    });
    app.get('/api/workgroupsupervisor/get_workgroupsupervisor/:supervisor_id', function (req, res, next) {
        logger.info('[WorkGroupSupervisorService::get_workgroupsupervisor] Received ' + req.method + ' on \'' + req.path + '\'');
        var vcc = res.locals.vcc;
        logger.info('[WorkGroupSupervisor::get_workgroupsupervisor] Getting vcc:', vcc);
        var supervisorId = req.params.supervisor_id;
        logger.info('[WorkGroupSupervisor::get_workgroupsupervisor] Getting supervisorId:', supervisorId);
        (0, promises_shared_1.DummyPromise)()
            .then(function (result) {
            if (!supervisorId)
                throw 'MISSING_SUPERVISOR_INPUT_PARAMTER';
            return mssql_service_1.MSSql.GetWorkGroupSupervisor(vcc, supervisorId);
        })
            .then(function (resultWorkGroupSupervisor) {
            logger.info('[ProcessService::get_workgroupsupervisor] Getting resultProcess after MSSql.GetWorkGroupSupervisor');
            if (res.headersSent)
                return;
            res.json({ status: true, description: 'SERVER_SUCCESS_MESSAGE', data: resultWorkGroupSupervisor });
        })
            .catch(function (err) {
            var processMsg = (typeof err === 'string' ? err : 'CANNOT_PROCESS_REQUEST');
            var errorMsg = (typeof err === 'string' ? err : err.message || err.description || 'SERVER_ERROR_UNKNOWN_MESSAGE');
            logger.error('Error al procesar \'' + req.path + '\': ' + errorMsg);
            logger.error('[ProcessService::get_workgroupsupervisor] Getting errorMsg:', errorMsg);
            if (res.headersSent)
                return;
            res.json({ status: false, description: processMsg, error: errorMsg });
        });
    });
    app.post('/api/workgroupsupervisor/get_workgroups_supervisor_ep', function (req, res, next) {
        logger.info('Se recibio ' + req.method + ' para \'' + req.path + '\'');
        var vcc = res.locals.vcc;
        var campaignIds = (req.body || {});
        var campaignIdsString = campaignIds ? campaignIds.join(',') : '';
        (0, promises_shared_1.DummyPromise)()
            .then(function (result) {
            return mssql_service_1.MSSql.GetWorkGroupsFromSupervisorEP(vcc, campaignIds);
        }).then(function (result) {
            if (res.headersSent)
                return;
            res.json({ status: true, description: 'SERVER_SUCCESS_MESSAGE', data: result });
        }).catch(function (err) {
            var clientMsg = (typeof err === 'string' ? err : 'CANNOT_PROCESS_REQUEST');
            var errorMsg = (typeof err === 'string' ? err : err.message || err.description || 'SERVER_ERROR_UNKNOWN_MESSAGE');
            logger.error('Error al procesar \'' + req.path + '\': ' + errorMsg);
            if (res.headersSent)
                return;
            res.json({ status: false, description: clientMsg, error: errorMsg, detail: { type: 'ERROR', datetime: moment().utc().toDate(), pid: process.pid, source: __filename, stack: err.stack } });
        });
    });
    app.post('/api/workgroupsupervisor/delete_workgroupsupervisor', function (req, res, next) {
        logger.info("[WorkGroupSupervisorService::delete_workgroupsupervisor] Received ".concat(req.method, " method on '").concat(req.path, "'"));
        var vcc = res.locals.vcc;
        var description = (req.body.description || {});
        var supervisorId = (req.body.supervisorId || {});
        (0, promises_shared_1.DummyPromise)()
            .then(function (_) {
            if (!supervisorId)
                throw 'SERVER_ERROR_MISSING_SUPERVISOR_ID';
            return mssql_service_1.MSSql.DeleteWorkGroupSupervisor(vcc, description, supervisorId);
        })
            .then(function (result) {
            if (!result)
                throw 'SERVER_ERROR_WORKGROUSUPERVISOR_DELETE_FAILED';
            if (res.headersSent)
                return;
            res.json({ status: result, description: 'SERVER_SUCCESS_MESSAGE', data: result });
        })
            .catch(function (err) {
            var clientMsg = (typeof err === 'string' ? err : 'CANNOT_PROCESS_REQUEST');
            var errorMsg = (typeof err === 'string' ? err : err.message || err.description || 'SERVER_ERROR_UNKNOWN_MESSAGE');
            logger.error("[WorkGroupSupervisorService::delete_workgroupsupervisor] Error on '".concat(req.path, "': ").concat(errorMsg));
            if (res.headersSent)
                return;
            res.json({ status: false, description: clientMsg, error: errorMsg });
        });
    });
}
exports.WorkGroupSupervisorService = WorkGroupSupervisorService;
