"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.ApiI6GetInteractions = exports.ApiI6GetCampaignMembers = exports.ApiI6GetAsyncInteractionContent = exports.ApiI6GetInteractionFile = exports.ApiI6GetInteractionParts = exports.ApiI6GetInteractionData = exports.ApiI6ConfiguredChannels = exports.ApiI6GetCampaignDispositions = exports.ApiI6SearchCampaigns = exports.ApiI6SearchUsers = exports.SaveUserByAPIi6 = exports.ApiI6GetUser = exports.SearchUsers = exports.GetRequestQuote = exports.Login = void 0;
var log4js = require("log4js");
var moment = require("moment");
var integration_config_1 = require("../config/integration.config");
var redis_service_1 = require("../db/redis.service");
var request_shared_1 = require("./request.shared");
var arrays_shared_1 = require("./arrays.shared");
var promises_shared_1 = require("./promises.shared");
var loggerAllegro = log4js.getLogger('ServerScripts');
function Login(vcc, user, password) {
    var i6IntegrationData = null;
    var needUpdateOnRedis = false;
    if (!user)
        user = "".concat(integration_config_1.USER_API_I6, "@").concat(vcc);
    if (!password)
        password = integration_config_1.PASS_API_I6;
    return (0, promises_shared_1.DummyPromise)()
        .then(function (_) {
        return redis_service_1.Redis.GetI6Integration(vcc);
    })
        .then(function (result) {
        if (result === null || result === void 0 ? void 0 : result.accessToken)
            return Promise.resolve(result);
        needUpdateOnRedis = true;
        loggerAllegro.info('[http-request-occ::Login] Login por API i6', JSON.stringify({ user: user, password: password }));
        var data = { user: user, password: password };
        var postOptions = {
            uri: "http://".concat(integration_config_1.DNS, "/inconcert/api/login/"),
            body: data,
            headers: {
                'Content-Type': 'application/json'
            },
            retryProtocol: true
        };
        return (0, request_shared_1.SimplePostRequest)(postOptions);
    })
        .then(function (response) {
        if (needUpdateOnRedis) {
            var status_1 = (response === null || response === void 0 ? void 0 : response.status) ? true : false;
            var message = response ? response : '';
            i6IntegrationData = { status: status_1, accessToken: message.token };
            return redis_service_1.Redis.SaveI6Integration(vcc, i6IntegrationData);
        }
        else {
            i6IntegrationData = response;
            return Promise.resolve(true);
        }
    })
        .then(function (_) {
        return Promise.resolve(i6IntegrationData);
    })
        .catch(function (err) {
        loggerAllegro.error('[http-request-occ::Login] Error ' + err.message);
        return Promise.resolve({ status: false, body: err.message });
    });
}
exports.Login = Login;
function GetRequestQuote(vcc) {
    var redisData = null;
    var i6IRequestQuote = null;
    var needUpdateOnRedis = false;
    return (0, promises_shared_1.DummyPromise)()
        .then(function (_) {
        return Login(vcc);
    })
        .then(function (result) {
        if (!result.accessToken)
            throw 'API_I6_LOGIN_FAILED';
        if (result === null || result === void 0 ? void 0 : result.requestQuote)
            return Promise.resolve(result);
        redisData = result;
        needUpdateOnRedis = true;
        loggerAllegro.info('[http-request-occ::GetRequestQuote] Getting request quote');
        var options = {
            uri: "http://".concat(integration_config_1.DNS, "/inconcert/api/vcc/get/"),
            method: 'GET',
            headers: {
                'Content-Type': 'application/json',
                'Authorization': result.accessToken
            },
            retryProtocol: true
        };
        return (0, request_shared_1.SimpleGetRequest)(options);
    })
        .then(function (response) {
        var _a, _b, _c, _d, _e, _f;
        if (needUpdateOnRedis) {
            if (!((_c = (_b = (_a = response === null || response === void 0 ? void 0 : response.vcc) === null || _a === void 0 ? void 0 : _a.ApiConfiguration) === null || _b === void 0 ? void 0 : _b.urlsConfig) === null || _c === void 0 ? void 0 : _c.resources))
                throw 'API_I6_GET_REQUEST_QUOTE_FAILED';
            i6IRequestQuote = (_f = (_e = (_d = response === null || response === void 0 ? void 0 : response.vcc) === null || _d === void 0 ? void 0 : _d.ApiConfiguration) === null || _e === void 0 ? void 0 : _e.urlsConfig) === null || _f === void 0 ? void 0 : _f.resources;
            redisData.requestQuote = i6IRequestQuote;
            return redis_service_1.Redis.SaveI6Integration(vcc, redisData);
        }
        else {
            i6IRequestQuote = response.requestQuote;
            return Promise.resolve(true);
        }
    })
        .then(function (result) {
        loggerAllegro.debug("allegroi6quote", i6IRequestQuote);
        return Promise.resolve({ status: true, body: { i6IRequestQuote: i6IRequestQuote } });
    })
        .catch(function (err) {
        var errorMessage = typeof err === 'string' ? err : err.message;
        loggerAllegro.error('[http-request-occ::GetRequestQuote] Error ' + errorMessage);
        return Promise.resolve({ status: false, body: errorMessage });
    });
}
exports.GetRequestQuote = GetRequestQuote;
function SearchUsers(accessToken, request) {
    loggerAllegro.info('[http-request-occ::SearchUsers] Request (inConcert): Listado de usuarios por API i6');
    var postOptions = {
        uri: "http://".concat(integration_config_1.DNS, "/inconcert/api/users/search/"),
        body: request,
        headers: {
            'Content-Type': 'application/json',
            'Authorization': accessToken
        },
        retryProtocol: true
    };
    return (0, request_shared_1.SimplePostRequest)(postOptions)
        .then(function (response) {
        return Promise.resolve(response);
    })
        .catch(function (err) {
        loggerAllegro.error('[http-request-occ::SearchUsers] : Error ' + err.message);
        return Promise.resolve({ status: false, body: err.message });
    });
}
exports.SearchUsers = SearchUsers;
function ApiI6GetUser(vcc, userId) {
    loggerAllegro.info('[http-request-occ::ApiI6GetUser] Getting campaigns');
    return Login(vcc)
        .then(function (result) {
        if (!result.accessToken)
            throw 'API_I6_LOGIN_FAILED';
        var getOptions = {
            uri: "http://".concat(integration_config_1.DNS, "/inconcert/api/users/get/").concat(userId),
            method: 'GET',
            headers: {
                'Content-Type': 'application/json',
                'Authorization': result.accessToken
            },
            retryProtocol: true
        };
        return (0, request_shared_1.SimpleGetRequest)(getOptions);
    })
        .then(function (response) {
        return Promise.resolve({ status: true, body: response });
    })
        .catch(function (err) {
        var errorMessage = typeof err === 'string' ? err : err.message;
        loggerAllegro.error('[http-request-occ::ApiI6GetUser] Error ' + errorMessage);
        return Promise.resolve({ status: false, body: errorMessage });
    });
}
exports.ApiI6GetUser = ApiI6GetUser;
function SaveUserByAPIi6(vcc, user) {
    loggerAllegro.info('[http-request-occ::SaveUserByAPIi6] Request (inConcert): Creación de usuario por API i6');
    return Login(vcc)
        .then(function (result) {
        if (!result.accessToken)
            throw 'API_I6_LOGIN_FAILED';
        var postOptions = {
            uri: "http://".concat(integration_config_1.DNS, "/inconcert/api/users/save/"),
            body: user,
            headers: {
                'Content-Type': 'application/json',
                'Authorization': result.accessToken
            },
            retryProtocol: true
        };
        return (0, request_shared_1.SimplePostRequest)(postOptions);
    })
        .then(function (response) {
        return Promise.resolve(response);
    })
        .catch(function (err) {
        loggerAllegro.error('[http-request-occ::SaveUserByAPIi6] Error ' + err.message);
        return Promise.resolve({ status: false, body: err.message });
    });
}
exports.SaveUserByAPIi6 = SaveUserByAPIi6;
function ApiI6SearchUsers(vcc, campaignIds, applicationId) {
    if (applicationId === void 0) { applicationId = 'interaction'; }
    loggerAllegro.info('[http-request-occ::ApiI6SearchUsers] Getting users');
    return Login(vcc)
        .then(function (result) {
        if (!result.accessToken)
            throw 'API_I6_LOGIN_FAILED';
        var postOptions = {
            uri: "http://".concat(integration_config_1.DNS, "/inconcert/api/users/search/"),
            body: {
                query: { Application: applicationId }
            },
            headers: {
                'Content-Type': 'application/json',
                'Authorization': result.accessToken
            },
            retryProtocol: true
        };
        return (0, request_shared_1.SimplePostRequest)(postOptions);
    })
        .then(function (response) {
        if (response && response.status === false)
            throw 'SERVER_ERROR_GET_API_I6_AGENTS_FAILED';
        var agents = [];
        response['rows'].map(function (r) {
            agents.push({
                vcc: vcc,
                campaignId: null,
                campaigns: [],
                userId: r.Id,
                userName: (r.Name ? r.Name.trim() + ' ' : '') + (r.FirstSurname ? r.FirstSurname.trim() : '')
            });
        });
        return Promise.resolve({ status: true, body: agents });
    })
        .catch(function (err) {
        var errorMessage = typeof err === 'string' ? err : err.message;
        loggerAllegro.error('[http-request-occ::ApiI6SearchUsers] Error ' + errorMessage);
        return Promise.resolve({ status: false, body: errorMessage });
    });
}
exports.ApiI6SearchUsers = ApiI6SearchUsers;
function ApiI6SearchCampaigns(vcc) {
    loggerAllegro.info('[http-request-occ::ApiI6SearchCampaigns] Getting campaigns');
    return Login(vcc)
        .then(function (result) {
        if (!result.accessToken)
            throw 'API_I6_LOGIN_FAILED';
        var postOptions = {
            uri: "http://".concat(integration_config_1.DNS, "/inconcert/api/campaigns/search/"),
            body: {
                columns: ["Id", "Name", "Description", "dispositionCodes"]
            },
            headers: {
                'Content-Type': 'application/json',
                'Authorization': result.accessToken
            },
            retryProtocol: true
        };
        return (0, request_shared_1.SimplePostRequest)(postOptions);
    })
        .then(function (response) {
        if (response && response.status === false)
            throw 'SERVER_ERROR_GET_API_I6_CAMPAIGNS_FAILED';
        var result;
        result = response['rows'].map(function (r) {
            var campaign = r.Id.split('@');
            var dispositionCodes = [];
            if (r.dispositionCodes) {
                r.dispositionCodes.map(function (dc) {
                    if (dc.childs) {
                        dc.childs.map(function (c1) {
                            if (c1.childs) {
                                c1.childs.map(function (c2) {
                                    if (c2.childs) {
                                        c2.childs.map(function (c3) {
                                            dispositionCodes.push({ parent: dc.name, child: c1.name, child2: c2.name, child3: c3.name });
                                        });
                                    }
                                    else {
                                        dispositionCodes.push({ parent: dc.name, child: c1.name, child2: c2.name, child3: null });
                                    }
                                });
                            }
                            else {
                                dispositionCodes.push({ parent: dc.name, child: c1.name, child2: null, child3: null });
                            }
                        });
                    }
                    else {
                        dispositionCodes.push({ parent: dc.name, child: null, child2: null, child3: null });
                    }
                });
            }
            return { id: campaign[0], description: r.Description, dispositionCodes: dispositionCodes };
        });
        return Promise.resolve({ status: true, body: result });
    })
        .catch(function (err) {
        var errorMessage = typeof err === 'string' ? err : err.message;
        loggerAllegro.error('[http-request-occ::ApiI6SearchCampaigns] Error ' + errorMessage);
        return Promise.resolve({ status: false, body: errorMessage });
    });
}
exports.ApiI6SearchCampaigns = ApiI6SearchCampaigns;
function ApiI6GetCampaignDispositions(vcc) {
    loggerAllegro.info('[http-request-occ::ApiI6GetCampaignDispositions] Getting disposition codes');
    return Login(vcc)
        .then(function (result) {
        if (!result.accessToken)
            throw 'API_I6_LOGIN_FAILED';
        var postOptions = {
            uri: 'http' + '://' + integration_config_1.DNS + '/inconcert/api/campaigns/get_dispositions/' + null,
            method: 'GET',
            body: {
                columns: ["Id", "Name", "Description", "dispositionCodes"]
            },
            headers: {
                'Content-Type': 'application/json',
                'Authorization': result.accessToken
            },
            retryProtocol: true
        };
        return (0, request_shared_1.SimpleGetRequest)(postOptions);
    })
        .then(function (response) {
        return Promise.resolve({ status: true, body: response });
    })
        .catch(function (err) {
        var errorMessage = typeof err === 'string' ? err : err.message;
        loggerAllegro.error('[http-request-occ::ApiI6GetCampaignDispositions] Error ' + errorMessage);
        return Promise.resolve({ status: false, body: errorMessage });
    });
}
exports.ApiI6GetCampaignDispositions = ApiI6GetCampaignDispositions;
function ApiI6ConfiguredChannels(vcc, campaign) {
    loggerAllegro.info('[http-request-occ::ApiI6ConfiguredChannels] Getting campaigns:', campaign.toString());
    return Login(vcc)
        .then(function (result) {
        if (!result.accessToken)
            throw 'API_I6_LOGIN_FAILED';
        var postOptions = {
            uri: "http://".concat(integration_config_1.DNS, "/inconcert/api/campaigns/get_configured_channels/"),
            body: {
                campaign: campaign && campaign.length && campaign.length === 1 ? campaign[0] : null
            },
            headers: {
                'Content-Type': 'application/json',
                'Authorization': result.accessToken
            },
            retryProtocol: true
        };
        return (0, request_shared_1.SimplePostRequest)(postOptions);
    })
        .then(function (response) {
        if ((!response) || (response && response.status === false))
            throw 'SERVER_ERROR_GET_API_I6_INTERACTION_TYPES_FAILED';
        var keys = Object.keys(response.channels);
        keys = keys.filter(function (k) { return k !== 'WEBCONTACT'; });
        var result = {};
        if (campaign && (campaign.length === 0 || campaign.length && campaign.length === 1)) {
            result = keys.map(function (k) {
                return {
                    instance: vcc,
                    id: k,
                    description: k
                };
            });
        }
        else {
            var channelCampaigns_1 = [];
            var channels_1 = [];
            keys.map(function (k) {
                var campaigns = Object.keys(response.channels[k]);
                campaigns = campaigns.map(function (c) { return c = c.split('@')[0]; });
                channelCampaigns_1.push({
                    channel: k,
                    campaigns: campaigns
                });
            });
            channelCampaigns_1.map(function (cc) {
                cc.campaigns.map(function (c) {
                    if (campaign.includes(c))
                        channels_1.push(cc.channel);
                });
            });
            channels_1 = (0, arrays_shared_1.GetUniqueFromArray)(channels_1);
            result = channels_1.map(function (c) {
                return {
                    instance: vcc,
                    id: c,
                    description: c
                };
            });
        }
        result = (0, arrays_shared_1.SortObjectArrayByValue)(result, 'id', 'A');
        return Promise.resolve({ status: true, body: result });
    })
        .catch(function (err) {
        var errorMessage = typeof err === 'string' ? err : err.message;
        loggerAllegro.error('[http-request-occ::ApiI6ConfiguredChannels] Error ' + errorMessage);
        return Promise.resolve({ status: false, body: errorMessage });
    });
}
exports.ApiI6ConfiguredChannels = ApiI6ConfiguredChannels;
function ApiI6GetInteractionData(vcc, interactionId) {
    loggerAllegro.info('[http-request-occ::ApiI6GetInteractionData] Getting interaction parts');
    return Login(vcc)
        .then(function (result) {
        if (!result.accessToken)
            throw 'API_I6_LOGIN_FAILED';
        var getOptions = {
            uri: "http://".concat(integration_config_1.DNS, "/inconcert/api/interactions/get/").concat(interactionId),
            headers: {
                'Content-Type': 'application/json',
                'Authorization': result.accessToken
            },
            retryProtocol: true
        };
        return (0, request_shared_1.SimpleGetRequest)(getOptions);
    })
        .then(function (response) {
        if ((!response) || (response && response.status === false))
            throw 'SERVER_ERROR_GET_API_I6_INTERACTION_DATA_FAILED';
        return Promise.resolve({ status: true, body: response.interaction });
    })
        .catch(function (err) {
        var errorMessage = typeof err === 'string' ? err : err.message;
        loggerAllegro.error('[http-request-occ::ApiI6GetInteractionData] Error ' + errorMessage);
        return Promise.resolve({ status: false, body: errorMessage });
    });
}
exports.ApiI6GetInteractionData = ApiI6GetInteractionData;
function ApiI6GetInteractionParts(vcc, interactionId) {
    loggerAllegro.info('[http-request-occ::ApiI6GetInteractionParts] Getting interaction parts');
    return Login(vcc)
        .then(function (result) {
        if (!result.accessToken)
            throw 'API_I6_LOGIN_FAILED';
        var getOptions = {
            uri: "http://".concat(integration_config_1.DNS, "/inconcert/api/interactions/media/list/").concat(interactionId),
            headers: {
                'Content-Type': 'application/json',
                'Authorization': result.accessToken
            },
            retryProtocol: true
        };
        return (0, request_shared_1.SimpleGetRequest)(getOptions);
    })
        .then(function (response) {
        if ((!response) || (response && response.status === false))
            throw 'SERVER_ERROR_GET_API_I6_INTERACTION_PARTS_FAILED';
        return Promise.resolve({ status: true, body: response.media });
    })
        .catch(function (err) {
        var errorMessage = typeof err === 'string' ? err : err.message;
        loggerAllegro.error('[http-request-occ::ApiI6GetInteractionParts] Error ' + errorMessage);
        return Promise.resolve({ status: false, body: errorMessage });
    });
}
exports.ApiI6GetInteractionParts = ApiI6GetInteractionParts;
function ApiI6GetInteractionFile(vcc, interactionId, section, part) {
    loggerAllegro.info('[http-request-occ::ApiI6GetInteractionFile] Getting audio file');
    return Login(vcc)
        .then(function (result) {
        if (!result.accessToken)
            throw 'API_I6_LOGIN_FAILED';
        var getOptions = {
            uri: "http://".concat(integration_config_1.DNS, "/inconcert/api/interactions/media/get/?interaction=").concat(interactionId, "&id=").concat(interactionId, "&section=").concat(section, "&part=").concat(part),
            headers: {
                'Authorization': result.accessToken
            },
            responseType: 'stream',
            retryProtocol: true
        };
        return (0, request_shared_1.SimpleGetRequest)(getOptions);
    })
        .then(function (result) {
        if (result && result.body && result.body.includes('404'))
            throw 'SERVER_ERROR_AUDIO_FILE_NOT_FOUND';
        loggerAllegro.info('[http-request-occ::ApiI6GetInteractionFile] OK');
        var buffer = null;
        var bufs = [];
        return new Promise(function (resolve, reject) {
            result.on('error', function (err) {
                loggerAllegro.error('[http-request-occ::ApiI6GetInteractionFile] SERVER_ERROR_CREATE_READ_STREAM_FAILED');
                reject('SERVER_ERROR_CREATE_READ_STREAM_FAILED');
            });
            result.once('close', function () {
                try {
                    buffer = Buffer.concat(bufs);
                    return resolve(buffer);
                }
                catch (e) {
                    return reject('SERVER_ERROR_CREATE_READ_STREAM_ON_CLOSE_FAILED');
                }
            });
            result.on('data', function (d) {
                try {
                    bufs.push(d);
                }
                catch (e) {
                    return reject('SERVER_ERROR_CREATE_READ_STREAM_ON_DATA_FAILED');
                }
            });
        });
    })
        .catch(function (err) {
        var errorMessage = typeof err === 'string' ? err : (err.message || err.description);
        loggerAllegro.error('[http-request-occ::ApiI6GetInteractionFile] Error ' + errorMessage);
        return Promise.resolve({ status: false, body: errorMessage });
    });
}
exports.ApiI6GetInteractionFile = ApiI6GetInteractionFile;
function ApiI6GetAsyncInteractionContent(vcc, channel, contentId) {
    loggerAllegro.info('[http-request-occ::ApiI6GetAsyncInteractionContent] Getting text content');
    var section = 0;
    switch (channel) {
        case 'MAIL':
            section = -1;
            break;
    }
    return Login(vcc)
        .then(function (result) {
        if (!result.accessToken)
            throw 'API_I6_LOGIN_FAILED';
        var getOptions = {
            uri: "http://".concat(integration_config_1.DNS, "/inconcert/api/repository/get/?id=").concat(contentId, "&section=").concat(section),
            headers: {
                'Content-Type': 'application/json',
                'Authorization': result.accessToken
            },
            retryProtocol: true
        };
        return (0, request_shared_1.SimpleGetRequest)(getOptions);
    })
        .then(function (result) {
        return Promise.resolve({ status: true, body: result });
    })
        .catch(function (err) {
        var errorMessage = typeof err === 'string' ? err : err.message;
        loggerAllegro.error('[http-request-occ::ApiI6GetAsyncInteractionContent] Error ' + errorMessage);
        return Promise.resolve({ status: false, body: errorMessage });
    });
}
exports.ApiI6GetAsyncInteractionContent = ApiI6GetAsyncInteractionContent;
function ApiI6GetCampaignMembers(vcc, campaigns) {
    loggerAllegro.info('[http-request-occ::ApiI6GetCampaignMembers] Getting campaign members');
    return Login(vcc)
        .then(function (result) {
        if (!result.accessToken)
            throw 'API_I6_LOGIN_FAILED';
        var postOptions = {
            uri: "http://".concat(integration_config_1.DNS, "/inconcert/api/campaigns/get_campaign_members/"),
            body: {
                campaigns: campaigns
            },
            headers: {
                'Content-Type': 'application/json',
                'Authorization': result.accessToken
            },
            retryProtocol: true
        };
        return (0, request_shared_1.SimplePostRequest)(postOptions);
    })
        .then(function (response) {
        if (response && response.status === false)
            throw 'SERVER_ERROR_GET_API_I6_CAMPAIGN_MEMBERS_FAILED';
        var campaigns = [];
        response.rows.map(function (r) {
            var campaign = {};
            campaign.id = r.Id ? r.Id.split('@')[0] : null;
            if (campaign.id) {
                var agents_1 = [];
                var supervisors_1 = [];
                var agentWg_1 = [];
                var supervisorWg_1 = [];
                r.members.map(function (m) {
                    if (m.apps_permisson) {
                        if (m.apps_permisson.interaction) {
                            agents_1.push(m.Id.split('@')[0]);
                            var agent = {};
                            agent.id = m.Id.split('@')[0];
                            agent.workGroupId = m.WorkgroupId ? m.WorkgroupId : null;
                            agentWg_1.push(agent);
                        }
                        if (m.apps_permisson.supervisor) {
                            supervisors_1.push(m.Id.split('@')[0]);
                            var supervisor_1 = {};
                            supervisor_1.id = m.Id.split('@')[0];
                            supervisor_1.workGroups = [];
                            if (m.supervisor_workgroups)
                                Object.keys(m.supervisor_workgroups).map(function (w) {
                                    if (m.supervisor_workgroups[w])
                                        supervisor_1.workGroups.push(w);
                                });
                            supervisorWg_1.push(supervisor_1);
                        }
                    }
                });
                campaign.supervisorAgents = [];
                supervisorWg_1.map(function (swg) {
                    var supervisorAgents = {};
                    supervisorAgents.id = swg.id;
                    supervisorAgents.agents = [];
                    if (swg.workGroups.length) {
                        swg.workGroups.map(function (wg) {
                            agentWg_1.filter(function (awg) { return awg.workGroupId === wg; }).map(function (awg) { return supervisorAgents.agents.push(awg.id); });
                        });
                    }
                    else {
                        agentWg_1.map(function (awg) { return supervisorAgents.agents.push(awg.id); });
                    }
                    campaign.supervisorAgents.push(supervisorAgents);
                });
                campaign.agents = (0, arrays_shared_1.GetUniqueFromArray)(agents_1);
                campaign.supervisors = (0, arrays_shared_1.GetUniqueFromArray)(supervisors_1);
                campaigns.push(campaign);
            }
        });
        return Promise.resolve({ status: true, body: campaigns });
    })
        .catch(function (err) {
        var errorMessage = typeof err === 'string' ? err : err.message;
        loggerAllegro.error('[http-request-occ::ApiI6SearchUsers] Error ' + errorMessage);
        return Promise.resolve({ status: false, body: errorMessage });
    });
}
exports.ApiI6GetCampaignMembers = ApiI6GetCampaignMembers;
function ApiI6GetInteractions(vcc, channel, filters, defaultRowsNumber, page) {
    var _a, _b, _c, _d, _e;
    var promiseArray = [];
    if (filters.interactionIds.length) {
        filters.interactionIds.forEach(function (item) {
            promiseArray.push(ApiI6GetInteractionData(vcc, item));
        });
    }
    if (!page)
        page = 1;
    if (!defaultRowsNumber)
        defaultRowsNumber = 500;
    var from = (page - 1) * defaultRowsNumber;
    loggerAllegro.info('[http-request-occ::ApiI6GetInteractions] Getting interactions');
    var occFilters = {
        type: channel,
        state: 'FINISHED',
        date_last_event_from: moment(filters.dateIni).utc().format('YYYY-MM-DD HH:mm:ss'),
        date_last_event_to: moment(filters.dateFin).utc().format('YYYY-MM-DD HH:mm:ss'),
    };
    if ((_a = filters === null || filters === void 0 ? void 0 : filters.campaigns) === null || _a === void 0 ? void 0 : _a.length)
        occFilters.campaign = filters.campaigns;
    if ((_b = filters === null || filters === void 0 ? void 0 : filters.agents) === null || _b === void 0 ? void 0 : _b.length)
        occFilters.agent = filters.agents;
    if ((_c = filters === null || filters === void 0 ? void 0 : filters.origins) === null || _c === void 0 ? void 0 : _c.length)
        occFilters.initiative = filters.origins[0] === 'E' ? 'INBOUND' : 'OUTBOUND';
    if ((_d = filters === null || filters === void 0 ? void 0 : filters.contactAddress) === null || _d === void 0 ? void 0 : _d.length)
        occFilters.contact_id = filters.contactAddress;
    if ((_e = filters === null || filters === void 0 ? void 0 : filters.results) === null || _e === void 0 ? void 0 : _e.length)
        occFilters.disposition_code = filters.results;
    if ((filters === null || filters === void 0 ? void 0 : filters.minDuration) || (filters === null || filters === void 0 ? void 0 : filters.maxDuration)) {
        occFilters.min_duration = filters.minDuration;
        occFilters.max_duration = filters.maxDuration;
    }
    var data = filters.interactionIds.length ? Promise.all(promiseArray) : Login(vcc);
    return data
        .then(function (result) {
        if (filters.interactionIds.length) {
            return Promise.resolve(result);
        }
        if (!result.accessToken)
            throw 'API_I6_LOGIN_FAILED';
        var postOptions = {
            uri: "http://".concat(integration_config_1.DNS, "/inconcert/api/interactions/load/"),
            body: {
                query: occFilters,
                last: from,
                rows: defaultRowsNumber
            },
            headers: {
                'Content-Type': 'application/json',
                'Authorization': result.accessToken
            },
            retryProtocol: true
        };
        return (0, request_shared_1.SimplePostRequest)(postOptions);
    })
        .then(function (response) {
        var interactions = [];
        if (filters.interactionIds.length) {
            response.forEach(function (item) {
                if (item.status) {
                    var body = item.body;
                    if (body.type === channel) {
                        if (!occFilters.campaign) {
                            interactions.push({
                                agent: body.__last_step.agent,
                                campaign: body.campaign,
                                id: body.id,
                                disposition_code: body.dispositionCode,
                                start_timestamp: moment.utc(new Date(body.startTimestamp * 1000)).format('YYYY-MM-DD HH:mm:ss'),
                                state_timestamp: body.timestamp,
                                total_characters: body.interactionData.totals.totalCharacters || 0,
                                total_records_time_sec: body.interactionData.totals.totalRecordsTimeSec || 0,
                                type: body.type,
                                last_event_timestamp: body.timestamp,
                                state: body.state
                            });
                        }
                        else if (occFilters.campaign && occFilters.campaign.includes(body.campaign.split('@')[0])) {
                            interactions.push({
                                agent: body.__last_step.agent,
                                campaign: body.campaign,
                                id: body.id,
                                disposition_code: body.dispositionCode,
                                start_timestamp: moment.utc(new Date(body.startTimestamp * 1000)).format('YYYY-MM-DD HH:mm:ss'),
                                state_timestamp: body.timestamp,
                                total_characters: body.interactionData.totals.totalCharacters || 0,
                                total_records_time_sec: body.interactionData.totals.totalRecordsTimeSec || 0,
                                type: body.type,
                                last_event_timestamp: body.timestamp,
                                state: body.state
                            });
                        }
                    }
                }
            });
        }
        else {
            if ((response === null || response === void 0 ? void 0 : response.status) === false)
                throw 'SERVER_ERROR_GET_API_I6_LOAD_INTERACTIONS_FAILED';
            if (response.content.total_interactions > 0) {
                var interactionIds = Object.keys(response.content.interactions);
                interactions = interactionIds.map(function (ii) { return response.content.interactions[ii]; });
                interactions = interactions.filter(function (i) {
                    if (i.type === 'CALL') {
                        return i.total_records_time_sec ? true : false;
                    }
                    else
                        return true;
                });
                var interactionEndDate_1 = null;
                var hourFormat_1 = 'hh:mm:ss';
                var referenceMinDate_1 = moment('00:00:00', hourFormat_1);
                var mintHour = filters.hourIni.split(':');
                referenceMinDate_1.set('hour', Number(mintHour[0]));
                referenceMinDate_1.set('minute', Number(mintHour[1]));
                referenceMinDate_1.set('second', Number(mintHour[2]));
                var referenceMaxDate_1 = moment('00:00:00', hourFormat_1);
                var maxHour = filters.hourFin.split(':');
                referenceMaxDate_1.set('hour', Number(maxHour[0]));
                referenceMaxDate_1.set('minute', Number(maxHour[1]));
                referenceMaxDate_1.set('second', Number(maxHour[2]));
                interactions = interactions.filter(function (i) {
                    interactionEndDate_1 = moment.utc(i.last_event_timestamp);
                    var currentHour = moment.utc("".concat(interactionEndDate_1.hour(), ":").concat(interactionEndDate_1.minutes(), ":").concat(interactionEndDate_1.seconds()), hourFormat_1);
                    if (referenceMaxDate_1.isSameOrAfter(referenceMinDate_1))
                        return currentHour.isBetween(referenceMinDate_1, referenceMaxDate_1);
                    else
                        return !currentHour.isBetween(referenceMaxDate_1, referenceMinDate_1);
                });
            }
        }
        return Promise.resolve({
            status: true,
            body: {
                interactions: interactions,
                total: filters.interactionIds.length ? interactions.length : response.content.total_interactions
            }
        });
    })
        .catch(function (err) {
        var errorMessage = typeof err === 'string' ? err : err.message;
        loggerAllegro.error('[http-request-occ::ApiI6GetInteractions] Error ' + errorMessage);
        return Promise.resolve({ status: false, body: errorMessage });
    });
}
exports.ApiI6GetInteractions = ApiI6GetInteractions;
