"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.InitTask = void 0;
var express = require("express");
var log4js = require("log4js");
var mssql_service_1 = require("../db/mssql.service");
var mssql_config_1 = require("../config/mssql.config");
var mssql_init_1 = require("../db/mssql.init");
var promises_shared_1 = require("../shared/promises.shared");
var redis_init_1 = require("../db/redis.init");
var process_implementation_1 = require("../implementation/process.implementation");
var audio_transfer_implementation_1 = require("../implementation/audio-transfer.implementation");
var logger_config_1 = require("../config/logger.config");
var transcriptions_download_implementation_1 = require("../implementation/transcriptions-download.implementation");
var transcriptions_upload_implementation_1 = require("../implementation/transcriptions-upload.implementation");
var cognitive_analysis_implementation_1 = require("../implementation/cognitive-analysis.implementation");
var redis_service_1 = require("../db/redis.service");
var moment = require("moment");
var supervisor_implementation_1 = require("../implementation/supervisor.implementation");
var cleaner_implementation_1 = require("../implementation/cleaner.implementation");
var file_manager_1 = require("../shared/file-manager");
var semantic_query_implementation_1 = require("../implementation/semantic-query.implementation");
var config_1 = require("../config/config");
var billing_batch_status_implementation_1 = require("../implementation/billing-batch-status.implementation");
var diarization_convert_implementation_1 = require("../implementation/diarization-convert.implementation");
var diarization_analysis_implementation_1 = require("../implementation/diarization-analysis.implementation");
var train_service_1 = require("../services/train.service");
var utterance_service_1 = require("../services/utterance.service");
var cognitive_engine_migrate_implementation_1 = require("../implementation/cognitive-engine-migrate.implementation");
var nlu_service_1 = require("../services/nlu.service");
var arrays_shared_1 = require("../shared/arrays.shared");
var i6_update_implementation_1 = require("../implementation/i6-update.implementation");
var batch_schedule_implementation_1 = require("../implementation/batch-schedule.implementation");
var tipification_analysis_implementation_1 = require("../implementation/tipification-analysis.implementation");
var sentiment_analysis_implementation_1 = require("../implementation/sentiment-analysis.implementation");
var evaluationPlans_implementations_1 = require("../implementation/evaluationPlans.implementations");
var cognitive_engine_migration_model_1 = require("../models/cognitive-engine-migration.model");
var transcriptions_standby_implementation_1 = require("../implementation/transcriptions-standby.implementation");
var semantic_analysis_upload_implementation_1 = require("../implementation/semantic-analysis-upload.implementation");
var semantic_analysis_download_implementation_1 = require("../implementation/semantic-analysis-download.implementation");
var semantic_analysis_convert_audio_implementation_1 = require("../implementation/semantic-analysis-convert-audio.implementation");
log4js.configure(logger_config_1.LOGGER_CONFIG);
var logger = log4js.getLogger('Task');
function InitTask() {
    mssql_config_1.MSSQL_CONFIG.connectionTimeout = 300 * 1000;
    mssql_config_1.MSSQL_CONFIG.requestTimeout = 300 * 1000;
    var sqlConn;
    var redisClient;
    var executionCode = Date.now().toString();
    var app = express();
    logger.info('[schedule.task.ts::InitTask] begin Task executionCode:', executionCode);
    var useRasaApi = config_1.USE_RASA_API;
    return (0, promises_shared_1.DummyPromise)()
        .then(function (_) {
        return (0, redis_init_1.RedisInit)(app);
    })
        .then(function (_) {
        redisClient = app.get('redisClient');
        return (0, mssql_init_1.MsSqlInit)(app);
    })
        .then(function (_) {
        sqlConn = app.get('sqlConn');
        var supervisor = new supervisor_implementation_1.SupervisorImplementation();
        return supervisor.Execute(executionCode);
    })
        .then(function (_) {
        if (!useRasaApi)
            return InitCognitiveModel();
        return InitRasaApiCognitiveModel();
    })
        .then(function (_) {
        if (!useRasaApi)
            return StopAllRunningRasaModelFromTest();
        return true;
    })
        .then(function (_) {
        return MigrateRasaCognitiveEngine();
    })
        .then(function (_) {
        return TrainSystemCognitiveModel();
    })
        .then(function (_) {
        return InitBatchSchedule(executionCode);
    })
        .then(function (_) {
        return InitEvaluationPlansExpired(executionCode);
    })
        .then(function (result) {
        if (!result)
            logger.warn('[schedule.task::InitTask] InitCognitiveModel function failed');
        var billingBatchStatus = new billing_batch_status_implementation_1.BillingBatchStatusImplementation();
        return billingBatchStatus.Execute(executionCode);
    })
        .then(function (result) {
        var _a, _b, _c, _d, _e, _f, _g, _h;
        if (!(result && result.data && result.data['allowedBatches'])
            &&
                !(result && result.data && result.data['cancelledBatches']))
            logger.debug('[schedule.task.ts::InitTask] Result from billingBatchStatus: SERVER_ERROR_BATCHES_NOT_FOUND');
        var promiseArray = [];
        var process = new process_implementation_1.ProcessImplementation();
        var i6Update = new i6_update_implementation_1.I6UpdateImplementation();
        var audioTransfer = new audio_transfer_implementation_1.AudioTransferImplementation();
        var diarizarionConvert = new diarization_convert_implementation_1.DiarizationConvertImplementation();
        var transcriptionsUpload = new transcriptions_upload_implementation_1.TranscriptionsUploadImplementation();
        var transcriptionsStandby = new transcriptions_standby_implementation_1.TranscriptionsStandbyImplementation();
        var transcriptionsDownload = new transcriptions_download_implementation_1.TranscriptionsDownloadImplementation();
        var diarizarionAnalysis = new diarization_analysis_implementation_1.DiarizationAnalysisImplementation();
        var semanticAnalysisUpload = new semantic_analysis_upload_implementation_1.SemanticAnalysisUploadImplementation();
        var semanticAnalysisDownload = new semantic_analysis_download_implementation_1.SemanticAnalysisDownloadImplementation();
        var semanticAnalysisConvertAudio = new semantic_analysis_convert_audio_implementation_1.SemanticAnalysisConvertAudioImplementation();
        var semanticQuery = new semantic_query_implementation_1.SemanticQueryImplementation();
        var cognitiveAnalysis = new cognitive_analysis_implementation_1.CognitiveAnalysisImplementation();
        var cognitiveEngineMigationImplementation = new cognitive_engine_migrate_implementation_1.CognitiveEngineMigrateImplementation();
        var batchScheduleImplementation = new batch_schedule_implementation_1.BatchScheduleImplementation();
        var tipificationAnalysisImplementation = new tipification_analysis_implementation_1.TipificationAnalysisImplementation();
        var sentimentAnalysis = new sentiment_analysis_implementation_1.SentimentAnalysisImplementation();
        promiseArray.push(batchScheduleImplementation.Execute(executionCode, config_1.RATE_RECURSIVE_BATCH_SCHEDULE));
        promiseArray.push(process.Execute(executionCode, config_1.RATE_RECURSIVE_PROCESS));
        promiseArray.push(i6Update.Execute(executionCode, config_1.RATE_RECURSIVE_I6_UPDATE));
        promiseArray.push(audioTransfer.Execute(executionCode, config_1.RATE_RECURSIVE_AUDIO_TRANSFER));
        promiseArray.push(diarizarionConvert.Execute(executionCode, config_1.RATE_RECURSIVE_DIARIZATION_CONVERT));
        promiseArray.push(transcriptionsUpload.Execute(executionCode, config_1.RATE_RECURSIVE_TRANSCRIPTION, ((_a = result.data) === null || _a === void 0 ? void 0 : _a.allowedBatches.transcription) ? result.data.allowedBatches.transcription : [], ((_b = result.data) === null || _b === void 0 ? void 0 : _b.cancelledBatches.transcription) ? result.data.cancelledBatches.transcription : []));
        promiseArray.push(transcriptionsStandby.Execute(executionCode, config_1.RATE_RECURSIVE_TRANSCRIPTION));
        promiseArray.push(transcriptionsDownload.Execute(executionCode, config_1.RATE_RECURSIVE_TRANSCRIPTION));
        promiseArray.push(diarizarionAnalysis.Execute(executionCode, config_1.RATE_RECURSIVE_DIARIZATION_ANALYSIS));
        promiseArray.push(sentimentAnalysis.Execute(executionCode, config_1.RATE_RECURSIVE_SENTIMENT_ANALYSIS));
        promiseArray.push(semanticAnalysisUpload.Execute(executionCode, config_1.RATE_RECURSIVE_SEMANTIC_ANALYSIS_UPLOAD));
        promiseArray.push(semanticAnalysisDownload.Execute(executionCode, config_1.RATE_RECURSIVE_SEMANTIC_ANALYSIS_DOWNLOAD));
        promiseArray.push(semanticAnalysisConvertAudio.Execute(executionCode, config_1.RATE_RECURSIVE_SEMANTIC_ANALYSIS));
        promiseArray.push(semanticQuery.Execute(executionCode, config_1.RATE_RECURSIVE_SEMANTIC_QUERY, ((_c = result.data) === null || _c === void 0 ? void 0 : _c.allowedBatches.semanticAnalysis) ? result.data.allowedBatches.semanticAnalysis : [], ((_d = result.data) === null || _d === void 0 ? void 0 : _d.cancelledBatches.semanticAnalysis) ? result.data.cancelledBatches.semanticAnalysis : []));
        promiseArray.push(cognitiveAnalysis.Execute(executionCode, config_1.RATE_RECURSIVE_COGNITIVE_ANALYSIS, ((_e = result.data) === null || _e === void 0 ? void 0 : _e.allowedBatches.cognitiveAnalysis) ? result.data.allowedBatches.cognitiveAnalysis : [], ((_f = result.data) === null || _f === void 0 ? void 0 : _f.cancelledBatches.cognitiveAnalysis) ? result.data.cancelledBatches.cognitiveAnalysis : []));
        promiseArray.push(tipificationAnalysisImplementation.Execute(executionCode, config_1.RATE_RECURSIVE_TIPIFICATION_ANALYSIS, ((_g = result.data) === null || _g === void 0 ? void 0 : _g.allowedBatches.cognitiveAnalysis) ? result.data.allowedBatches.cognitiveAnalysis : [], ((_h = result.data) === null || _h === void 0 ? void 0 : _h.cancelledBatches.cognitiveAnalysis) ? result.data.cancelledBatches.cognitiveAnalysis : []));
        promiseArray.push(cognitiveEngineMigationImplementation.Execute(executionCode));
        return Promise.all(promiseArray);
    })
        .then(function (results) {
        logger.info('[schedule.task::InitTask] executionCode:', executionCode, 'Receiving result from implementations');
        if (results.filter(function (result) { return (result !== true); }).length > 0) {
            logger.error('[schedule.task::InitTask] executionCode:', executionCode, 'getting error results');
            return Promise.resolve(false);
        }
        else {
            return Promise.resolve(true);
        }
    })
        .then(function (_) {
        var promiseArray = [];
        promiseArray.push((0, file_manager_1.CleanupTempFile)(config_1.COGNITIVE_ANALYSIS_FOLDER, config_1.PERSISTENCE));
        promiseArray.push((0, file_manager_1.CleanupTempFile)(config_1.SEMANCTIC_ANALYSIS_FOLDER, config_1.PERSISTENCE));
        promiseArray.push((0, file_manager_1.CleanupTempFile)(config_1.QUERY_FOLDER, config_1.PERSISTENCE));
        return Promise.all(promiseArray);
    })
        .then(function (_) {
        var cleaner = new cleaner_implementation_1.CleanerImplementation();
        return cleaner.Execute(executionCode);
    })
        .then(function (_) {
        logger.info('[schedule.task::InitTask] executionCode:', executionCode, 'Task has been finished');
        if (sqlConn)
            sqlConn.close();
        if (redisClient)
            redisClient.quit();
        setTimeout(function () {
            process.exit(0);
        }, 200);
        return Promise.resolve(true);
    })
        .catch(function (err) {
        logger.error('[schedule.task::InitTask] executionCode:', executionCode, 'Task has been finished', executionCode, 'getting error:', err);
        sqlConn.close();
        if (redisClient)
            redisClient.quit();
        setTimeout(function () {
            process.exit(0);
        }, 200);
        return Promise.resolve(false);
    });
}
exports.InitTask = InitTask;
function MigrateRasaCognitiveEngine() {
    var cognitiveService;
    var vccToMigrate;
    var vccNluUsage;
    return (0, promises_shared_1.DummyPromise)().then(function (result) {
        return mssql_service_1.MSSql.GetNluIntentUtteranceToExport('ALL', 2);
    })
        .then(function (result) {
        if (!result)
            throw 'SERVER_ERROR_NLUS_NOT_FOUND';
        vccToMigrate = result.quantity;
        vccNluUsage = result.usage.InUse;
        logger.debug("[CognitiveEngineMigrateImplementation::Execute] vcctoMigrate ", vccToMigrate);
        return redis_service_1.Redis.GetCognitiveEngineMigrationData();
    })
        .then(function (results) {
        var promiseArray = [];
        if (typeof results != "object" || !results) {
            logger.debug("[CognitiveEngineMigrateImplementation::Execute] NOT_FOUND_DATA ", results);
            if (vccToMigrate.totalNlu && vccToMigrate.totalNlu > 0 && !vccNluUsage) {
                logger.info('[schedule.task::MigrateRasaToRasaApi] Migrating:', vccToMigrate.vcc);
                promiseArray.push(redis_service_1.Redis.AddTaskCognitiveEngineMigration(vccToMigrate.vcc, new cognitive_engine_migration_model_1.CognitiveEngineMigrationConfig('OLD_RASA_ENGINE', 'NEW_RASA_ENGINE', null, null, vccToMigrate.totalNlu, vccToMigrate.totalIntent, vccToMigrate.totalUtterance, vccToMigrate.totalEntity)));
            }
        }
        return Promise.all(promiseArray);
    })
        .then(function (result) {
        return Promise.resolve(true);
    })
        .catch(function (error) {
        var errorMessage = typeof error === 'string' ? error : 'SERVER_ERROR_RUN_CURRENT_BATCHES_NLUS_FAILED';
        logger.error('[schedule.task::TrainSystemCognitiveModel] Error: ' + JSON.stringify(error));
        return Promise.resolve(false);
    });
}
function TrainSystemCognitiveModel() {
    var cognitiveService;
    var needUpdateNLUToUtterancesBulk = false;
    var needUpdateNLUToTrainDueToRasaUpdate = false;
    return (0, promises_shared_1.DummyPromise)().then(function (result) {
        return (0, cognitive_analysis_implementation_1.GetCognitiveService)(null, 'RASA');
    })
        .then(function (result) {
        cognitiveService = result;
        return mssql_service_1.MSSql.GetNLUs('ALL', null, null, null, true);
    })
        .then(function (result) {
        if (!result)
            throw 'SERVER_ERROR_SYSTEM_APPS_NOT_FOUND';
        var promiseArray = [];
        result = (0, arrays_shared_1.SortObjectArrayByShuffle)(result);
        var timeOut = 0;
        result.forEach(function (r, i) {
            if (r.vcc !== 'system' || (r.vcc === 'system' && r.name.includes('sentiment-'))) {
                if (r.needTrain
                    &&
                        r.lastModifiedDate
                    &&
                        (moment().utc() > moment(r.lastModifiedDate).utc().add(5, 'minutes'))
                    &&
                        !config_1.USE_RASA_API
                    &&
                        r.cognitiveEngine === 'RASA') {
                    logger.info('[schedule.task::TrainSystemCognitiveModel] Training system app:', r.appId);
                    timeOut = i * 1000;
                    promiseArray.push(new Promise(function (resolve, reject) {
                        setTimeout(function () {
                            return resolve((0, train_service_1.TrainApp)(cognitiveService, null, r));
                        }, timeOut);
                    }));
                }
            }
        });
        result.filter(function (r) { return r.lastModifiedByUserId === 'dev-1'
            &&
                r.isSystemApp === true
            &&
                r.lastTrainedDateTime; })
            .map(function (n, i) {
            if (i === 0) {
                promiseArray.push((0, utterance_service_1.GetSaveUtterances)(n));
            }
        });
        return Promise.all(promiseArray);
    })
        .then(function (result) {
        return (0, nlu_service_1.GetNLUToUtterancesBulk)(false);
    })
        .then(function (result) {
        var promiseArray = [];
        if (result && result.length) {
            needUpdateNLUToUtterancesBulk = true;
            result = (0, arrays_shared_1.SortObjectArrayByShuffle)(result);
            result.map(function (n, i) {
                if (n.lastModifiedByUserId === 'dev-1' && i === 0) {
                    promiseArray.push((0, utterance_service_1.GetSaveUtterances)(n));
                }
            });
        }
        return Promise.all(promiseArray);
    })
        .then(function (result) {
        return (0, nlu_service_1.GetNLUToTrainDueToRasaUpdate)(false);
    })
        .then(function (result) {
        var promiseArray = [];
        if (result && result.length) {
            needUpdateNLUToTrainDueToRasaUpdate = true;
            result = (0, arrays_shared_1.SortObjectArrayByShuffle)(result);
            result.map(function (n, i) {
                if (i === 0
                    &&
                        n.needTrain
                    &&
                        n.lastModifiedByUserId === 'dev-2') {
                    promiseArray.push((0, train_service_1.TrainApp)(cognitiveService, null, n));
                }
            });
        }
        return Promise.all(promiseArray);
    })
        .then(function (result) {
        var promiseArray = [];
        if (needUpdateNLUToUtterancesBulk)
            promiseArray.push((0, nlu_service_1.GetNLUToUtterancesBulk)(true));
        if (needUpdateNLUToTrainDueToRasaUpdate)
            promiseArray.push((0, nlu_service_1.GetNLUToTrainDueToRasaUpdate)(true));
        return Promise.all(promiseArray);
    })
        .then(function (result) {
        return Promise.resolve(true);
    })
        .catch(function (error) {
        var errorMessage = typeof error === 'string' ? error : 'SERVER_ERROR_RUN_CURRENT_BATCHES_NLUS_FAILED';
        logger.error('[schedule.task::TrainSystemCognitiveModel] Error: ' + JSON.stringify(error));
        return Promise.resolve(false);
    });
}
function InitCognitiveModel() {
    var cognitiveService;
    return (0, promises_shared_1.DummyPromise)().then(function (result) {
        return (0, cognitive_analysis_implementation_1.GetCognitiveService)(null, 'RASA');
    }).then(function (result) {
        cognitiveService = result;
        var promiseArray = [];
        promiseArray.push(redis_service_1.Redis.GetCurrentBatchesRasaModels());
        promiseArray.push(redis_service_1.Redis.GetRunningRasaModels());
        return Promise.all(promiseArray);
    }).then(function (result) {
        logger.debug('[schedule.task::InitCognitiveModel] uniqueCurrentBatchesNlus: ', result[0]);
        logger.debug('[schedule.task::InitCognitiveModel] allRunningRasaModels: ', result[1]);
        var allCurrentBatchesNlus = [];
        if (result[0]) {
            var attributesArray = Object.getOwnPropertyNames(result[0]);
            attributesArray.map(function (aa) {
                allCurrentBatchesNlus = allCurrentBatchesNlus.concat(JSON.parse(result[0][aa].toLowerCase()));
            });
        }
        var uniqueCurrentBatchesNlus = allCurrentBatchesNlus.filter(function (item, i, ar) { return ar.indexOf(item) === i; });
        logger.debug('[schedule.task::InitCognitiveModel] uniqueCurrentBatchesNlus: ', uniqueCurrentBatchesNlus);
        var allRunningRasaModels = [];
        if (result[1]) {
            var attributesArray = Object.getOwnPropertyNames(result[1]);
            attributesArray.map(function (aa) {
                var arr = aa.split('::');
                allRunningRasaModels = allRunningRasaModels.concat(arr[1] + '::' + arr[2]);
            });
        }
        var uniqueRunningRasaModels = allRunningRasaModels.filter(function (item, i, ar) { return ar.indexOf(item) === i; });
        logger.debug('[schedule.task::InitCognitiveModel] uniqueRunningRasaModels: ', uniqueRunningRasaModels);
        var promiseArray = [];
        var index = 0;
        uniqueCurrentBatchesNlus.map(function (ucbn) {
            if (!uniqueRunningRasaModels.includes(ucbn)) {
                var timeOut_1 = index * 1000;
                promiseArray.push(new Promise(function (resolve, reject) {
                    setTimeout(function () {
                        return resolve(cognitiveService.PublishApp(null, null, ucbn, true, ''));
                    }, timeOut_1);
                }));
                index++;
            }
        });
        uniqueRunningRasaModels.map(function (urrm) {
            if (!uniqueCurrentBatchesNlus.includes(urrm)) {
                var timeOut_2 = index * 1000;
                promiseArray.push(new Promise(function (resolve, reject) {
                    setTimeout(function () {
                        return resolve(cognitiveService.PublishApp(null, null, urrm, false, ''));
                    }, timeOut_2);
                }));
                index++;
            }
        });
        return Promise.all(promiseArray);
    }).then(function (result) {
        logger.info('[schedule.task::InitCognitiveModel] Result: ' + result);
        return Promise.resolve(true);
    }).catch(function (error) {
        var errorMessage = typeof error === 'string' ? error : 'SERVER_ERROR_RUN_CURRENT_BATCHES_NLUS_FAILED';
        logger.error('[schedule.task::InitCognitiveModel] Error: ' + JSON.stringify(error));
        return Promise.resolve(false);
    });
}
function InitRasaApiCognitiveModel() {
    var cognitiveService;
    return (0, promises_shared_1.DummyPromise)().then(function (result) {
        return (0, cognitive_analysis_implementation_1.GetCognitiveService)(null, 'RASA');
    })
        .then(function (result) {
        cognitiveService = result;
        var promiseArray = [];
        promiseArray.push(redis_service_1.Redis.GetCurrentBatchesRasaModels());
        promiseArray.push(redis_service_1.Redis.GetRunningRasaModels());
        promiseArray.push(mssql_service_1.MSSql.GetNLUs('ALL', null, null, null, null));
        return Promise.all(promiseArray);
    }).then(function (result) {
        logger.debug('[schedule.task::InitRasaApiCognitiveModel] uniqueCurrentBatchesNlus: ', result[0]);
        logger.debug('[schedule.task::InitRasaApiCognitiveModel] allRunningRasaModels: ', result[1]);
        logger.debug('[schedule.task::InitRasaApiCognitiveModel] NeedTrainRasaModels: ', result[2]);
        var allNluNeedTrain = result[2].filter(function (nlu) { return nlu.needTrain; });
        var allNluNeedPublish = result[2].filter(function (nlu) { return nlu.needPublish; });
        logger.debug('[schedule.task::InitRasaApiCognitiveModel] allNluNeedTrain: ', allNluNeedTrain);
        logger.debug('[schedule.task::InitRasaApiCognitiveModel] allNluNeedPublish: ', allNluNeedPublish);
        var allCurrentBatchesNlus = [];
        if (result[0]) {
            var attributesArray = Object.getOwnPropertyNames(result[0]);
            attributesArray.map(function (aa) {
                allCurrentBatchesNlus = allCurrentBatchesNlus.concat(JSON.parse(result[0][aa].toLowerCase()));
            });
        }
        var uniqueCurrentBatchesNlus = allCurrentBatchesNlus.filter(function (item, i, ar) { return ar.indexOf(item) === i; });
        logger.debug('[schedule.task::InitRasaApiCognitiveModel] uniqueCurrentBatchesNlus: ', uniqueCurrentBatchesNlus);
        var allRunningRasaModels = [];
        if (result[1]) {
            var attributesArray = Object.getOwnPropertyNames(result[1]);
            attributesArray.map(function (aa) {
                var arr = aa.split('::');
                allRunningRasaModels = allRunningRasaModels.concat(arr[1] + '::' + arr[2]);
            });
        }
        var uniqueRunningRasaModels = allRunningRasaModels.filter(function (item, i, ar) { return ar.indexOf(item) === i; });
        logger.debug('[schedule.task::InitRasaApiCognitiveModel] uniqueRunningRasaModels: ', uniqueRunningRasaModels);
        var promiseArray = [];
        var index = 0;
        uniqueCurrentBatchesNlus.map(function (ucbn) {
            if (!uniqueRunningRasaModels.includes(ucbn)) {
                var nluNeedTrain_1 = allNluNeedTrain.find(function (nnt) { return nnt.vcc + '::' + nnt.name === ucbn; });
                var timeOut_3 = index * 1000;
                if (nluNeedTrain_1 && nluNeedTrain_1.cognitiveEngine === 'RASAAPI' && config_1.USE_RASA_API) {
                    nluNeedTrain_1.trainMode = 'A';
                    promiseArray.push(new Promise(function (resolve, reject) {
                        setTimeout(function () {
                            return resolve((0, train_service_1.TrainApp)(cognitiveService, null, nluNeedTrain_1));
                        }, timeOut_3);
                    }));
                    index++;
                }
            }
        });
        uniqueRunningRasaModels.map(function (urrm) {
            if (!uniqueCurrentBatchesNlus.includes(urrm)) {
                var nluNeedTrain_2 = allNluNeedTrain.find(function (nnt) { return nnt.vcc + '::' + nnt.name === urrm; });
                var timeOut_4 = index * 1000;
                if (nluNeedTrain_2 && nluNeedTrain_2.cognitiveEngine === 'RASAAPI' && config_1.USE_RASA_API) {
                    nluNeedTrain_2.trainMode = 'A';
                    promiseArray.push(new Promise(function (resolve, reject) {
                        setTimeout(function () {
                            return resolve((0, train_service_1.TrainApp)(cognitiveService, null, nluNeedTrain_2));
                        }, timeOut_4);
                    }));
                    index++;
                }
            }
        });
        uniqueCurrentBatchesNlus.map(function (ucbn) {
            if (!uniqueRunningRasaModels.includes(ucbn)) {
                var nluNeedPublish_1 = allNluNeedPublish.find(function (nnt) { return nnt.vcc + '::' + nnt.name === ucbn; });
                var timeOut_5 = index * 1000;
                if (nluNeedPublish_1 && nluNeedPublish_1.cognitiveEngine === 'RASAAPI' && config_1.USE_RASA_API) {
                    promiseArray.push(new Promise(function (resolve, reject) {
                        setTimeout(function () {
                            return resolve((0, train_service_1.PublishRasaApiApp)(cognitiveService, nluNeedPublish_1));
                        }, timeOut_5);
                    }));
                    index++;
                }
            }
        });
        uniqueRunningRasaModels.map(function (urrm) {
            if (!uniqueCurrentBatchesNlus.includes(urrm)) {
                var nluNeedPublish_2 = allNluNeedPublish.find(function (nnt) { return nnt.vcc + '::' + nnt.name === urrm; });
                var timeOut_6 = index * 1000;
                if (nluNeedPublish_2 && nluNeedPublish_2.cognitiveEngine === 'RASAAPI' && config_1.USE_RASA_API) {
                    promiseArray.push(new Promise(function (resolve, reject) {
                        setTimeout(function () {
                            return resolve((0, train_service_1.PublishRasaApiApp)(cognitiveService, nluNeedPublish_2));
                        }, timeOut_6);
                    }));
                    index++;
                }
            }
        });
        return Promise.all(promiseArray);
    }).then(function (result) {
        logger.info('[schedule.task::InitRasaApiCognitiveModel] Result: ' + result);
        return Promise.resolve(true);
    }).catch(function (error) {
        var errorMessage = typeof error === 'string' ? error : 'SERVER_ERROR_RUN_CURRENT_BATCHES_NLUS_FAILED';
        logger.error('[schedule.task::InitRasaApiCognitiveModel] Error: ' + JSON.stringify(error));
        return Promise.resolve(false);
    });
}
function StopAllRunningRasaModelFromTest() {
    var cognitiveService;
    return (0, promises_shared_1.DummyPromise)().then(function (result) {
        return (0, cognitive_analysis_implementation_1.GetCognitiveService)(null, 'RASA');
    }).then(function (result) {
        cognitiveService = result;
        return redis_service_1.Redis.GetAllRunningRasaModelFromTest();
    })
        .then(function (result) {
        var promiseArray = [];
        var index = 0;
        if (result !== null) {
            var _loop_1 = function (key) {
                var value = JSON.parse(result[key]);
                var timeOut = index * 1000;
                if (moment.utc(value.created).add(45, 'minutes').isBefore(moment().utc())) {
                    promiseArray.push(new Promise(function (resolve, reject) {
                        setTimeout(function () {
                            return resolve(cognitiveService.StopApp(value.id, value.port, 'TEST'));
                        }, timeOut);
                    }));
                }
                index++;
            };
            for (var key in result) {
                _loop_1(key);
            }
        }
        return Promise.all(promiseArray);
    }).then(function (result) {
        return Promise.resolve(true);
    })
        .catch(function (error) {
        var errorMessage = typeof error === 'string' ? error : 'SERVER_ERROR_STOP_EXPIRED_MODELS_FROM_TEST';
        logger.error('[schedule.task::StopAllRunningModelsFromTest] Error: ' + JSON.stringify(error));
        return Promise.resolve(false);
    });
}
function InitBatchSchedule(executionCode, force) {
    var batchScheduleImplementation = new batch_schedule_implementation_1.BatchScheduleImplementation();
    return (0, promises_shared_1.DummyPromise)()
        .then(function (_) {
        var promiseArray = [];
        var startRange01 = moment();
        startRange01.set('hour', 0);
        startRange01.set('minute', 0);
        startRange01.set('second', 0);
        var endRange01 = moment();
        endRange01.set('hour', 0);
        endRange01.set('minute', 4);
        endRange01.set('second', 59);
        var startRange02 = moment();
        startRange02.set('hour', 12);
        startRange02.set('minute', 0);
        startRange02.set('second', 0);
        var endRange02 = moment();
        endRange02.set('hour', 12);
        endRange02.set('minute', 4);
        endRange02.set('second', 59);
        var firstRange = moment().isBetween(startRange01, endRange01);
        var secondRange = moment().isBetween(startRange02, endRange02);
        if (firstRange || secondRange || force) {
            logger.info('[schedule.task::InitBatchSchedule] Registering scheduleExecutions on redis at:', moment().format('YYYY-MM-DD HH:mm:ss'));
            promiseArray.push(batchScheduleImplementation.GetNewScheduledBatchExecutions(executionCode));
            if (firstRange && [1, 4].includes(moment().isoWeekday()) || force) {
                logger.info('[schedule.task::InitBatchSchedule] Generating new scheduleExecutions on db at:', moment().format('YYYY-MM-DD HH:mm:ss'));
                promiseArray.push(batchScheduleImplementation.GenerateNewScheduleBatchExecution(executionCode));
            }
        }
        return Promise.all(promiseArray);
    })
        .then(function (_) {
        return Promise.resolve(true);
    })
        .catch(function (error) {
        var errorMessage = typeof error === 'string' ? error : 'SERVER_ERROR_RUN_CURRENT_BATCHES_NLUS_FAILED';
        logger.error('[schedule.task::InitBatchSchedule] Error: ' + JSON.stringify(error));
        return Promise.resolve(false);
    });
}
function InitEvaluationPlansExpired(executionCode) {
    return (0, promises_shared_1.DummyPromise)()
        .then(function (_) {
        var evaluationPlansImplementation = new evaluationPlans_implementations_1.EvaluationPlansImplementation();
        var startRange01 = moment();
        startRange01.set('hour', 0);
        startRange01.set('minute', 0);
        startRange01.set('second', 0);
        var endRange01 = moment();
        endRange01.set('hour', 0);
        endRange01.set('minute', 4);
        endRange01.set('second', 59);
        logger.info('[schedule.task::InitEvaluationPlansExpired] currentDate:', moment().format('YYYY-MM-DD HH:mm:ss'));
        logger.info('[schedule.task::InitEvaluationPlansExpired] startRange01:', startRange01.format('YYYY-MM-DD HH:mm:ss'));
        logger.info('[schedule.task::InitEvaluationPlansExpired] endRange01:', endRange01.format('YYYY-MM-DD HH:mm:ss'));
        var firstRange = moment().isBetween(startRange01, endRange01);
        if (firstRange) {
            logger.info('[schedule.task::InitEvaluationPlansExpired] Update EvaluationPlans to Expired on BD at:', moment().format('YYYY-MM-DD HH:mm:ss'));
            return evaluationPlansImplementation.Execute(new Date(startRange01.toDate()));
        }
        return Promise.resolve(true);
    })
        .then(function (_) {
        return Promise.resolve(true);
    })
        .catch(function (error) {
        logger.error('[schedule.task::InitEvaluationPlansExpired] Error: ' + JSON.stringify(error));
        return Promise.resolve(false);
    });
}
