"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.BatchScheduleImplementation = void 0;
var config_1 = require("../config/config");
var mssql_service_1 = require("../db/mssql.service");
var redis_service_1 = require("../db/redis.service");
var schedule_model_1 = require("../models/schedule.model");
var interaction_model_1 = require("../models/interaction.model");
var interaction_service_1 = require("../services/interaction.service");
var linguistic_service_1 = require("../services/linguistic.service");
var process_service_1 = require("../services/process.service");
var promises_shared_1 = require("../shared/promises.shared");
var utilities_1 = require("../shared/utilities");
var log4js = require("log4js");
var moment = require("moment");
var billing_implementation_1 = require("./billing.implementation");
var batch_schedule_service_1 = require("../services/batch-schedule.service");
var study_service_1 = require("../services/study.service");
var arrays_shared_1 = require("../shared/arrays.shared");
var evaluation_plan_service_1 = require("../services/evaluation-plan.service");
var logger = log4js.getLogger('Task');
var CURRENT_SYSTEM_SERVICE = 'SCHEDULE';
var CURRENT_IMPLEMENTATION = 'BacthScheduleImplementation';
var BatchScheduleImplementation = (function () {
    function BatchScheduleImplementation() {
    }
    BatchScheduleImplementation.prototype.Execute = function (executionCode, rateRecursive) {
        var _this = this;
        var executeIndex = 1;
        var ratio = config_1.RATIO_BATCH_SCHEDULE;
        var executeResult = true;
        return (0, promises_shared_1.SleepPromise)(config_1.SLEEP_BATCH_SCHEDULE)
            .then(function (_) {
            return redis_service_1.Redis.LRange('BatchScheduleList', 0, ratio - 1);
        })
            .then(function (result) {
            var promiseArray = [];
            ratio = result && result.length <= ratio ? result.length : 0;
            while (ratio >= executeIndex) {
                promiseArray.push(_this.BacthScheduleExecute(executionCode));
                executeIndex++;
            }
            return Promise.all(promiseArray);
        })
            .then(function (results) {
            if (results.filter(function (item) { return item === false; }).length > 0) {
                executeResult = false;
            }
            return redis_service_1.Redis.LRange('BatchScheduleList', 0, ratio - 1);
        })
            .then(function (result) {
            if (result && result.length && rateRecursive >= 1)
                return _this.Execute(executionCode, rateRecursive - 1);
            return Promise.resolve(executeResult);
        })
            .then(function (result) {
            var currentResult = executeResult && result ? true : false;
            return Promise.resolve(currentResult);
        })
            .catch(function (error) {
            var errorMessage = typeof error === 'string' ? error : (error.message || error.description);
            logger.error('[BatchScheduleImplementation::Execute] executionCode:', executionCode, 'getting error:', errorMessage);
            return Promise.resolve(false);
        });
    };
    BatchScheduleImplementation.prototype.BacthScheduleExecute = function (executionCode) {
        var _this = this;
        var currentQueueId = null;
        var currentBatchScheduleData = null;
        var vcc = null;
        var batchScheduleId = null;
        var batchtype = null;
        var batchScheduleExecutionId = null;
        return redis_service_1.Redis.RPopLPush('BatchScheduleList', 'BatchScheduleListExecute')
            .then(function (queueId) {
            if (!queueId)
                throw 'SERVER_ERROR_NO_QUEUE_ID_FOUND_ON_BATCH_SCHEDULE_LIST';
            logger.debug('[BatchScheduleImplementation::BacthScheduleExecute] executionCode:', executionCode, 'launching queueId:', queueId);
            currentQueueId = queueId;
            var queueData = currentQueueId.split(':');
            vcc = queueData[0];
            batchScheduleId = queueData[1];
            batchtype = queueData[2];
            batchScheduleExecutionId = queueData[3];
            return redis_service_1.Redis.GetBatchSchedule(currentQueueId);
        })
            .then(function (result) {
            var _a, _b, _c;
            currentBatchScheduleData = result;
            if (currentBatchScheduleData.execution.status === 'PAUSED_BY_INSUFFICIENT_REQUESTS_BALANCE')
                currentBatchScheduleData.execution.status = 'BATCH_CREATED';
            if (((_a = currentBatchScheduleData === null || currentBatchScheduleData === void 0 ? void 0 : currentBatchScheduleData.execution) === null || _a === void 0 ? void 0 : _a.status)
                &&
                    ['REGISTERED_ON_REDIS', 'READY_TO_EXECUTE'].includes(currentBatchScheduleData.execution.status))
                return _this.CreateNewScheduledBatchExecutions(currentQueueId, executionCode, currentBatchScheduleData);
            else if (((_b = currentBatchScheduleData === null || currentBatchScheduleData === void 0 ? void 0 : currentBatchScheduleData.execution) === null || _b === void 0 ? void 0 : _b.status) === 'BATCH_CREATED'
                &&
                    (!currentBatchScheduleData.batchData.process
                        &&
                            ((_c = currentBatchScheduleData.batchData.process) === null || _c === void 0 ? void 0 : _c.name)))
                return _this.StartScheduleBatchExecutions(currentQueueId, executionCode, currentBatchScheduleData);
            return Promise.resolve(true);
        })
            .then(function (result) {
            if (currentBatchScheduleData)
                return redis_service_1.Redis.SaveBatchSchedule(currentQueueId, currentBatchScheduleData);
            return Promise.resolve(true);
        })
            .then(function (_) {
            var _a, _b, _c, _d;
            var promiseArray = [];
            if (((_a = currentBatchScheduleData === null || currentBatchScheduleData === void 0 ? void 0 : currentBatchScheduleData.execution) === null || _a === void 0 ? void 0 : _a.status) === 'BATCH_STARTED') {
                logger.info('[BatchScheduleImplementation::BacthScheduleExecute] executionCode:', executionCode, 'getting queueId:', currentQueueId, 'BATCH_STARTED');
                promiseArray.push(redis_service_1.Redis.DeleteQueue('BatchScheduleListExecute', currentQueueId));
                promiseArray.push(redis_service_1.Redis.DeleteBatchSchedule(currentQueueId));
            }
            else if (['CANCELLED_BY_FAILED_PROCESS', 'CANCELLED_BY_NO_INTERACTIONS_FOUND'].includes((_b = currentBatchScheduleData === null || currentBatchScheduleData === void 0 ? void 0 : currentBatchScheduleData.execution) === null || _b === void 0 ? void 0 : _b.status)) {
                logger.info('[BatchScheduleImplementation::BacthScheduleExecute] executionCode:', executionCode, 'getting queueId:', currentQueueId, currentBatchScheduleData.execution.status);
                promiseArray.push(redis_service_1.Redis.DeleteQueue('BatchScheduleListExecute', currentQueueId));
                promiseArray.push(redis_service_1.Redis.DeleteBatchSchedule(currentQueueId));
            }
            else if (((_c = currentBatchScheduleData === null || currentBatchScheduleData === void 0 ? void 0 : currentBatchScheduleData.execution) === null || _c === void 0 ? void 0 : _c.retry) >= config_1.RETRY_BATCH_SCHEDULE) {
                logger.warn('[BatchScheduleImplementation::BacthScheduleExecute] executionCode:', executionCode, 'getting queueId:', currentQueueId, 'REMOVED_BY_RETRIES', currentBatchScheduleData.execution.errorMessage);
                promiseArray.push(redis_service_1.Redis.DeleteQueue('BatchScheduleListExecute', currentQueueId));
                promiseArray.push(redis_service_1.Redis.DeleteBatchSchedule(currentQueueId));
            }
            else if ((_d = currentBatchScheduleData === null || currentBatchScheduleData === void 0 ? void 0 : currentBatchScheduleData.execution) === null || _d === void 0 ? void 0 : _d.errorMessage) {
                logger.warn('[BatchScheduleImplementation::BacthScheduleExecute] executionCode:', executionCode, 'getting queueId:', currentQueueId, 'ERROR', currentBatchScheduleData.execution.errorMessage);
                promiseArray.push(mssql_service_1.MSSql.SaveBatchError(vcc, CURRENT_SYSTEM_SERVICE, batchScheduleId, batchtype, CURRENT_IMPLEMENTATION, batchScheduleExecutionId, null, currentBatchScheduleData.execution.errorFunction, currentBatchScheduleData.execution.errorMessage, currentBatchScheduleData.execution.errorDetail));
            }
            else if (currentQueueId && !currentBatchScheduleData) {
                logger.warn('[BatchScheduleImplementation::BacthScheduleExecute] executionCode:', executionCode, 'getting queueId:', currentQueueId, 'DATA_DOES_NOT_EXIST');
                promiseArray.push(redis_service_1.Redis.DeleteQueue('BatchScheduleListExecute', currentQueueId));
            }
            else {
                logger.info('[BatchScheduleImplementation::BacthScheduleExecute] executionCode:', executionCode, 'getting queueId:', currentQueueId, 'QUEUE_WAS_NOT_REMOVED');
            }
            return Promise.all(promiseArray);
        })
            .then(function (_) {
            var _a;
            var promiseArray = [];
            if (((_a = currentBatchScheduleData === null || currentBatchScheduleData === void 0 ? void 0 : currentBatchScheduleData.execution) === null || _a === void 0 ? void 0 : _a.retry) >= config_1.RETRY_BATCH_SCHEDULE) {
                promiseArray.push(redis_service_1.Redis.DeleteBatchSchedule(currentQueueId));
                promiseArray.push(mssql_service_1.MSSql.SaveBatchError(vcc, CURRENT_SYSTEM_SERVICE, batchScheduleId, batchtype, CURRENT_IMPLEMENTATION, batchScheduleExecutionId, null, currentBatchScheduleData.errorFunction, currentBatchScheduleData.errorMessage, currentBatchScheduleData.errorDetail));
            }
            return Promise.all(promiseArray);
        })
            .then(function (_) {
            return Promise.resolve(true);
        })
            .catch(function (err) {
            var errorMessage = typeof err === 'string' ? err : (err.message || err.description);
            logger.error('[BatchScheduleImplementation::BacthScheduleExecute] executionCode:', executionCode, 'getting queueId:', currentQueueId, 'getting errorMessage:', errorMessage);
            return Promise.resolve(false);
        });
    };
    BatchScheduleImplementation.prototype.GenerateNewScheduleBatchExecution = function (executionCode, batchscheduleId) {
        var batchSchedules = [];
        return (0, promises_shared_1.DummyPromise)()
            .then(function (_) {
            return mssql_service_1.MSSql.GetBatchSchedulesByCriteria(null, 'STATUS', 'CREATED,RUNNING', true);
        })
            .then(function (result) {
            if (!result)
                throw 'SERVER_ERROR_SEARCH_BATCH_SCHEDULES_FAILED';
            if (result && !result.length)
                throw 'SERVER_WARNING_BATCH_SCHEDULES_NOT_FOUND';
            batchSchedules = result.filter(function (r) { return ['TRANSCRIPTION', 'EVALUATION_PLAN'].includes(r.batchType); });
            if (batchscheduleId)
                batchSchedules = batchSchedules.filter(function (bs) { return bs.id === batchscheduleId; });
            return mssql_service_1.MSSql.GetBatchScheduleExecutionsByCriteria(null, 'LAST_CREATED_EXECUTIONS', batchSchedules.map(function (bs) { return bs.id; }).toString());
        })
            .then(function (result) {
            var executionList = [];
            batchSchedules.forEach(function (bs) {
                var lastBatchScheduleExecution = result.find(function (r) { return r.batchScheduleId === bs.id; });
                bs.lastExecutionDate = lastBatchScheduleExecution ? lastBatchScheduleExecution.executionDate : bs.firstExecutionDate;
                var newBatchScheduleExecutions = (0, batch_schedule_service_1.GenerateNewBatcScheduleExecutions)(bs, 7, lastBatchScheduleExecution ? lastBatchScheduleExecution.order : 0);
                executionList = executionList.concat(newBatchScheduleExecutions);
            });
            var promiseArray = [];
            executionList.forEach(function (el) { return promiseArray.push(mssql_service_1.MSSql.SaveBatchScheduleExecution(el.vcc, el, el.createdByUserId)); });
            return Promise.all(promiseArray);
        })
            .then(function (_) {
            return Promise.resolve(true);
        })
            .catch(function (err) {
            var errorMessage = typeof err === 'string' ? err : (err.message || err.description);
            logger.error('[BacthScheduleImplementation::GenerateNewScheduleBatchExecution] executionCode:', executionCode, 'getting errorMessage:', errorMessage);
            return Promise.resolve(false);
        });
    };
    BatchScheduleImplementation.prototype.RemoveScheduleBatchExecutions = function (vcc, scheduleData, executionCode) {
        return (0, promises_shared_1.DummyPromise)()
            .then(function (_) {
            return mssql_service_1.MSSql.GetBatchScheduleExecutionsByCriteria(vcc, 'NO_EXECUTED_BY_SCHEDULE_ID', scheduleData.id);
        })
            .then(function (result) {
            var promiseArray = [];
            result.map(function (r) {
                promiseArray.push(mssql_service_1.MSSql.DeleteBatchScheduleExecution(vcc, r.id));
                if (r.status === 'REGISTERED_ON_REDIS') {
                    var queueId = "".concat(vcc, ":").concat(scheduleData.id, ":").concat(scheduleData.batchType, ":").concat(r.id);
                    promiseArray.push(redis_service_1.Redis.DeleteQueue('BatchScheduleListExecute', queueId));
                    promiseArray.push(redis_service_1.Redis.DeleteBatchSchedule(queueId));
                }
            });
            return Promise.all(promiseArray);
        })
            .then(function (_) {
            return Promise.resolve(true);
        })
            .catch(function (err) {
            var errorMessage = typeof err === 'string' ? err : (err.message || err.description);
            logger.error('[BacthScheduleImplementation::RemoveScheduleBatchExecutions] executionCode:', executionCode, 'getting errorMessage:', errorMessage);
            return Promise.resolve(false);
        });
    };
    BatchScheduleImplementation.prototype.GetNewScheduledBatchExecutions = function (executionCode, batchscheduleId) {
        var _this = this;
        var batchSchedules = [];
        return (0, promises_shared_1.DummyPromise)()
            .then(function (_) {
            return mssql_service_1.MSSql.GetBatchSchedulesByCriteria(null, 'STATUS', 'CREATED,RUNNING', true);
        })
            .then(function (result) {
            if (!result)
                throw 'SERVER_ERROR_SEARCH_BATCH_SCHEDULES_FAILED';
            if (result && !result.length)
                throw 'SERVER_WARNING_BATCH_SCHEDULES_NOT_FOUND';
            batchSchedules = result;
            if (batchscheduleId)
                batchSchedules = batchSchedules.filter(function (bs) { return bs.id === batchscheduleId; });
            return mssql_service_1.MSSql.GetBatchScheduleExecutionsByCriteria(null, 'NOT_CREATED_BY_BS_ID', batchSchedules.map(function (bs) { return bs.id; }).toString());
        })
            .then(function (result) {
            batchSchedules.forEach(function (bs) {
                bs.executionList = result.filter(function (r) { return r.batchScheduleId === bs.id; });
            });
            return Promise.resolve(true);
        })
            .then(function (_) {
            var promiseArray = [];
            batchSchedules.forEach(function (bs) {
                var pendingBatchSchedules = bs.executionList.filter(function (el) { return el.status === 'PENDING'; });
                pendingBatchSchedules = pendingBatchSchedules.filter(function (pbs) { return moment(pbs.executionDate).isBefore(moment().utc().add(1, 'days').utc()); });
                if (pendingBatchSchedules.length) {
                    promiseArray.push(_this.SaveNewScheduledExecutions(executionCode, bs, pendingBatchSchedules));
                }
                else {
                    logger.warn('[BacthScheduleImplementation::GetNewScheduledBatchExecutions] executionCode:', executionCode, 'no execution to register on redis for ', bs.id);
                }
            });
            return Promise.all(promiseArray);
        })
            .then(function (_) {
            return Promise.resolve(true);
        })
            .catch(function (err) {
            var errorMessage = typeof err === 'string' ? err : (err.message || err.description);
            logger.error('[BacthScheduleImplementation::GetNewScheduledBatchExecutions] executionCode:', executionCode, 'getting errorMessage:', errorMessage);
            return Promise.resolve(false);
        });
    };
    BatchScheduleImplementation.prototype.SaveNewScheduledExecutions = function (executionCode, bs, pendingBatchSchedules) {
        var _this = this;
        return (0, promises_shared_1.DummyPromise)()
            .then(function (_) {
            var promiseArray = [];
            pendingBatchSchedules.map(function (pbs) {
                pbs.status = 'REGISTERED_ON_REDIS';
                promiseArray.push(_this.SaveNewScheduledExecution(executionCode, bs, pbs));
            });
            return Promise.all(promiseArray);
        })
            .then(function (result) {
            logger.info('[BacthScheduleImplementation::SaveNewScheduledExecutions] executionCode:', executionCode, 'updating db for:', bs.id);
            return mssql_service_1.MSSql.SaveBatchSchedule(bs.vcc, bs, 'system');
        })
            .then(function (_) {
            return Promise.resolve(true);
        })
            .catch(function (err) {
            var errorMessage = typeof err === 'string' ? err : (err.message || err.description);
            logger.error('[BacthScheduleImplementation::SaveNewScheduledExecutions] executionCode:', executionCode, 'getting errorMessage:', errorMessage);
            return Promise.resolve(false);
        });
    };
    BatchScheduleImplementation.prototype.SaveNewScheduledExecution = function (executionCode, bs, bse) {
        var queueId = "".concat(bs.vcc, ":").concat(bs.id, ":").concat(bs.batchType, ":").concat(bse.id);
        return (0, promises_shared_1.DummyPromise)()
            .then(function (_) {
            return redis_service_1.Redis.GetBatchSchedule(queueId);
        })
            .then(function (result) {
            if (result || (result && result.status === 'REGISTERED_ON_REDIS')) {
                logger.warn('[BacthScheduleImplementation::SaveNewScheduledExecution] executionCode:', executionCode, 'the scheduleExecution is already registered:', bs.id, '=>', bse.id);
                throw 'SERVER_WARNING_SCHEDULE_EXECUTION_IS_ALREADY_REGISTERED';
            }
            var startDate = null;
            var endDate = null;
            switch (bs.frequency) {
                case 'daily':
                    startDate = moment(bse.executionDate).utc().add(-1, 'days').utc();
                    endDate = moment(bse.executionDate).utc().add(-1, 'milliseconds').utc();
                    break;
                case 'weekly':
                    startDate = moment(bse.executionDate).utc().add(-1, 'weeks').utc();
                    endDate = moment(bse.executionDate).utc().add(-1, 'milliseconds').utc();
                    break;
                case 'monthly':
                    startDate = moment(bse.executionDate).utc().add(-1, 'month').set('date', bs.monthDay).utc();
                    endDate = moment(bse.executionDate).utc().add(-1, 'milliseconds').utc();
                    break;
                case 'custom_per_days':
                    startDate = moment(bse.executionDate).utc().add(-1 * bs.daysInterval, 'days').utc();
                    endDate = moment(bse.executionDate).utc().add(-1, 'milliseconds').utc();
                    break;
                case 'custom_per_hours':
                    startDate = moment(bse.executionDate).utc().add(-1 * bs.hoursInterval, 'hours').utc();
                    endDate = moment(bse.executionDate).utc().add(-1, 'milliseconds').utc();
                    break;
            }
            switch (bs.batchType) {
                case 'EVALUATION_PLAN':
                    bs.batchData.evaluationPlan.startDate = startDate.toDate();
                    bs.batchData.evaluationPlan.endDate = endDate.toDate();
                    bs.batchData.evaluationPlan.dateFin = moment(bse.executionDate).utc().add(bs.daysToExpire, 'days').utc().toDate();
                    break;
                case 'TEMPLATE': break;
                case 'TRANSCRIPTION':
                    bs.batchData.startDate = bs.batchData.startDate.toDate();
                    bs.batchData.endDate = bs.batchData.endDate.toDate();
                    break;
            }
            var promiseArray = [];
            promiseArray.push(redis_service_1.Redis.CreateQueueL('BatchScheduleList', queueId));
            promiseArray.push(redis_service_1.Redis.SaveBatchSchedule(queueId, {
                execution: bse,
                batchData: bs.batchData,
                processFiltersId: bs.processFiltersId,
                retry: 0
            }));
            return Promise.all(promiseArray);
        })
            .then(function (_) {
            return mssql_service_1.MSSql.SaveBatchScheduleExecution(bse.vcc, bse, 'system');
        })
            .then(function (_) {
            return Promise.resolve(true);
        })
            .catch(function (err) {
            var errorMessage = typeof err === 'string' ? err : (err.message || err.description);
            logger.error('[BacthScheduleImplementation::SaveNewScheduledExecution] executionCode:', executionCode, 'getting errorMessage:', errorMessage);
            return Promise.resolve(false);
        });
    };
    BatchScheduleImplementation.prototype.CreateNewScheduledBatchExecutions = function (queueId, executionCode, scheduleBatchData) {
        var _this = this;
        var queueData = queueId.split(':');
        var vcc = queueData[0];
        var batchScheduleId = queueData[1];
        var batchtype = queueData[2];
        var batchScheduleExecutionId = queueData[3];
        return (0, promises_shared_1.DummyPromise)()
            .then(function (result) {
            var _a;
            if (scheduleBatchData.execution && scheduleBatchData.execution.status && scheduleBatchData.execution.status === 'REGISTERED_ON_REDIS') {
                if (moment(scheduleBatchData.execution.executionDate).add(scheduleBatchData.batchData.executionDelay ? scheduleBatchData.batchData.executionDelay : 0, 'hours').isBefore(moment().utc())) {
                    scheduleBatchData.execution.status = 'READY_TO_EXECUTE';
                }
            }
            if (((_a = scheduleBatchData.execution) === null || _a === void 0 ? void 0 : _a.status) === 'READY_TO_EXECUTE') {
                switch (batchtype) {
                    case 'TRANSCRIPTION': return _this.CreateProcessSchedule(scheduleBatchData, queueId, executionCode);
                    case 'TEMPLATE': return _this.CreateTemplateSchedule(scheduleBatchData, queueId, executionCode);
                    case 'EVALUATION_PLAN': return _this.CreateEvaluationPlanSchedule(scheduleBatchData, queueId, executionCode);
                    default: throw 'SERVER_ERROR_BATCH_SCHEDULE_WITH_UNKNOWN_BATCH_TYPE';
                }
            }
            else
                throw 'SERVER_ERROR_BATCH_SCHEDULE_WITH_NOT_ALLOWED_STATUS';
        })
            .then(function (result) {
            var scheduleExecution = new schedule_model_1.ScheduleExecution();
            scheduleExecution.vcc = vcc;
            scheduleExecution.id = batchScheduleExecutionId;
            scheduleExecution.batchScheduleId = batchScheduleId;
            scheduleExecution.batchId = scheduleBatchData.execution.batchId;
            scheduleExecution.status = scheduleBatchData.execution.status;
            return mssql_service_1.MSSql.SaveBatchScheduleExecution(vcc, scheduleExecution, 'system');
        })
            .then(function (_) {
            return Promise.resolve(true);
        })
            .catch(function (err) {
            var errorMessage = typeof err === 'string' ? err : (err.message || err.description);
            logger.error('[BacthScheduleImplementation::CreateNewScheduledBatchExecutions] executionCode:', executionCode, 'getting queueId:', queueId, 'getting errorMessage:', errorMessage);
            return Promise.resolve(false);
        });
    };
    BatchScheduleImplementation.prototype.CreateProcessSchedule = function (scheduleBatchData, queueId, executionCode) {
        var queueData = queueId.split(':');
        var vcc = queueData[0];
        var batchScheduleId = queueData[1];
        var batchtype = queueData[2];
        var batchScheduleExecutionId = queueData[3];
        var interactionSearch = new interaction_model_1.InteractionSearch();
        var process = {};
        process.start = false;
        process.clientId = scheduleBatchData.batchData.clientId;
        process.description = scheduleBatchData.batchData.description;
        process.environment = scheduleBatchData.batchData.environment;
        process.environmentType = scheduleBatchData.batchData.environmentType;
        process.language = scheduleBatchData.batchData.language;
        process.percentage = scheduleBatchData.batchData.samplePercentage;
        process.type = scheduleBatchData.batchData.type;
        process.batchScheduleId = batchScheduleId;
        return (0, promises_shared_1.DummyPromise)()
            .then(function (_) {
            return mssql_service_1.MSSql.GetProcessFilters(vcc, scheduleBatchData.processFiltersId);
        })
            .then(function (result) {
            interactionSearch = (0, interaction_service_1.GetInteractionSearchFromProcessFilter)(result);
            interactionSearch.searchType = 'TO_SEARCH';
            interactionSearch.dateIni = moment(scheduleBatchData.batchData.startDate).toDate();
            interactionSearch.dateFin = moment(scheduleBatchData.batchData.endDate).toDate();
            return Promise.resolve(true);
        })
            .then(function (_) {
            var promiseArray = [];
            promiseArray = (0, interaction_service_1.GetChannelNewInteractionSummaryPromises)(vcc, scheduleBatchData.batchData.vccType, true, 'SUMMARIZED', interactionSearch, null);
            return Promise.all(promiseArray);
        })
            .then(function (result) {
            var totalInteractions = 0;
            result.forEach(function (r) { return totalInteractions += r.totalInteractions; });
            if (totalInteractions) {
                logger.info('[BacthScheduleImplementation::CreateProcessScheduleBatch] executionCode:', executionCode, 'creating process for', queueId, 'interactions:', totalInteractions);
                interactionSearch.searchType = 'TO_OPERATE';
                process.name = batchScheduleId;
                if (process.name.length > 40)
                    process.name = process.name.substr(0, 40);
                process.name += '-' + ('000' + scheduleBatchData.execution.order).slice(-3);
                process.name += '-' + (0, utilities_1.guid)().slice(-5);
                scheduleBatchData.execution.batchId = process.name;
                return (0, process_service_1.CreateProcess)(vcc, scheduleBatchData.batchData.vccType, process, interactionSearch, scheduleBatchData.execution.createdByUserId);
            }
            else {
                logger.info('[BacthScheduleImplementation::CreateProcessScheduleBatch] executionCode:', executionCode, 'no interactions found for', queueId);
                scheduleBatchData.execution.status = 'CANCELLED_BY_NO_INTERACTIONS_FOUND';
                throw 'SERVER_ERROR_NO_INTERACTIONS_FOUND_FOR_EXECUTION';
            }
        })
            .then(function (result) {
            if (result && result.status) {
                scheduleBatchData.execution.status = 'BATCH_CREATED';
                scheduleBatchData.retry = 0;
                scheduleBatchData.errorMessage = null;
                scheduleBatchData.errorDetail = null;
            }
            else {
                scheduleBatchData.retry = scheduleBatchData.retry ? scheduleBatchData.retry + 1 : 0;
                scheduleBatchData.errorFunction = 'CreateProcessScheduleBatch';
                scheduleBatchData.errorMessage = result ? result.saveResult : 'CREATE_BATCH_SCHEDULE_FAILED';
                scheduleBatchData.errorDetail = result ? result.saveResult : 'CREATE_BATCH_SCHEDULE_FAILED';
            }
            return Promise.resolve(true);
        })
            .catch(function (err) {
            var errorMessage = typeof err === 'string' ? err : (err.message || err.description);
            logger.error('[BacthScheduleImplementation::CreateProcessScheduleBatch] executionCode:', executionCode, 'getting errorMessage:', errorMessage);
            scheduleBatchData.retry = scheduleBatchData.retry ? scheduleBatchData.retry + 1 : 0;
            scheduleBatchData.errorFunction = 'CreateProcessScheduleBatch';
            scheduleBatchData.errorMessage = errorMessage;
            scheduleBatchData.errorDetail = JSON.stringify(err);
            if (scheduleBatchData.execution.status === 'CANCELLED_BY_NO_INTERACTIONS_FOUND')
                scheduleBatchData.retry = config_1.RETRY_BATCH_SCHEDULE;
            return Promise.resolve(false);
        });
    };
    BatchScheduleImplementation.prototype.CreateTemplateSchedule = function (scheduleBatchData, queueId, executionCode) {
        var queueData = queueId.split(':');
        var vcc = queueData[0];
        var batchScheduleId = queueData[1];
        var batchtype = queueData[2];
        var batchScheduleExecutionId = queueData[3];
        var process = {};
        var study = {};
        study.description = 'Scheduled template';
        study.processId = scheduleBatchData.execution.processId;
        study.status = 0.01;
        study.templateId = scheduleBatchData.processFiltersId;
        study.batchScheduleId = batchScheduleId;
        study.type = 'TEMPLATE';
        return (0, promises_shared_1.DummyPromise)()
            .then(function (_) {
            return mssql_service_1.MSSql.GetProcessTask(vcc, study.processId);
        })
            .then(function (result) {
            process = result;
            if (process && process.status !== 'FAILED') {
                logger.info('[BacthScheduleImplementation::CreateTemplateSchedule] executionCode:', executionCode, 'creating process for', queueId, 'process:', process.name);
                study.name = batchScheduleId;
                if (study.name.length > 40)
                    study.name = study.name.substr(0, 40);
                study.name += '-' + (0, utilities_1.guid)().slice(-9);
                scheduleBatchData.execution.batchId = study.name;
                return (0, study_service_1.CreateStudy)(vcc, scheduleBatchData.batchData.vccType, study, scheduleBatchData.execution.createdByUserId);
            }
            else {
                logger.info('[BacthScheduleImplementation::CreateTemplateSchedule] executionCode:', executionCode, 'failed process for', queueId);
                scheduleBatchData.execution.status = 'CANCELLED_BY_FAILED_PROCESS';
                throw 'SERVER_ERROR_FAILED_PROCESS_FOR_EXECUTION';
            }
        })
            .then(function (result) {
            if (result && result.resultSaveStudy) {
                scheduleBatchData.execution.status = 'BATCH_CREATED';
                scheduleBatchData.retry = 0;
                scheduleBatchData.errorMessage = null;
                scheduleBatchData.errorDetail = null;
            }
            else {
                scheduleBatchData.retry = scheduleBatchData.retry ? scheduleBatchData.retry + 1 : 0;
                scheduleBatchData.errorFunction = 'CreateTemplateSchedule';
                scheduleBatchData.errorMessage = result ? result.saveResult : 'CREATE_BATCH_SCHEDULE_FAILED';
                scheduleBatchData.errorDetail = result ? result.saveResult : 'CREATE_BATCH_SCHEDULE_FAILED';
            }
            return Promise.resolve(true);
        })
            .catch(function (err) {
            var errorMessage = typeof err === 'string' ? err : (err.message || err.description);
            logger.error('[BacthScheduleImplementation::CreateTemplateSchedule] executionCode:', executionCode, 'getting errorMessage:', errorMessage);
            scheduleBatchData.retry = scheduleBatchData.retry ? scheduleBatchData.retry + 1 : 0;
            scheduleBatchData.errorFunction = 'CreateTemplateSchedule';
            scheduleBatchData.errorMessage = errorMessage;
            scheduleBatchData.errorDetail = JSON.stringify(err);
            if (scheduleBatchData.execution.status === 'CANCELLED_BY_FAILED_PROCESS')
                scheduleBatchData.retry = config_1.RETRY_BATCH_SCHEDULE;
            return Promise.resolve(false);
        });
    };
    BatchScheduleImplementation.prototype.CreateEvaluationPlanSchedule = function (scheduleBatchData, queueId, executionCode) {
        var queueData = queueId.split(':');
        var vcc = queueData[0];
        var batchScheduleId = queueData[1];
        var batchtype = queueData[2];
        var batchScheduleExecutionId = queueData[3];
        var interactionSearch = new interaction_model_1.InteractionSearch();
        var evaluationPlan = {};
        evaluationPlan.status = 1;
        evaluationPlan.description = scheduleBatchData.batchData.evaluationPlan.description;
        evaluationPlan.environmentId = scheduleBatchData.batchData.evaluationPlan.environmentId;
        evaluationPlan.transcriptionLanguage = scheduleBatchData.batchData.evaluationPlan.transcriptionLanguage;
        evaluationPlan.modality = scheduleBatchData.batchData.evaluationPlan.modality;
        evaluationPlan.dateFin = scheduleBatchData.batchData.evaluationPlan.dateFin;
        evaluationPlan.idEvaluationFormat = scheduleBatchData.batchData.evaluationPlan.idEvaluationFormat;
        evaluationPlan.templateId = scheduleBatchData.batchData.evaluationPlan.templateId;
        evaluationPlan.enabled = scheduleBatchData.batchData.evaluationPlan.enabled;
        evaluationPlan.evaluation_mode = scheduleBatchData.batchData.evaluationPlan.evaluation_mode;
        evaluationPlan.filterByContentPercentage = scheduleBatchData.batchData.evaluationPlan.filterByContentPercentage;
        evaluationPlan.filterByContentQueryName = scheduleBatchData.batchData.evaluationPlan.filterByContentQueryName;
        evaluationPlan.filterByContentQuery = scheduleBatchData.batchData.evaluationPlan.filterByContentQuery;
        evaluationPlan.evaluatePercentage = scheduleBatchData.batchData.evaluationPlan.evaluatePercentage;
        evaluationPlan.auditors = scheduleBatchData.batchData.evaluationPlan.auditors.map(function (a) {
            var aep = {};
            aep.idAuditor = a.auditorId;
            aep.percentage = a.percentage;
            return aep;
        });
        evaluationPlan.evaluationStarted = true;
        evaluationPlan.batchScheduleId = batchScheduleId;
        var process = {};
        if (scheduleBatchData.batchData.process) {
            process.start = false;
            process.clientId = scheduleBatchData.batchData.process.clientId;
            process.description = scheduleBatchData.batchData.process.description;
            process.environment = scheduleBatchData.batchData.process.environment;
            process.environmentType = scheduleBatchData.batchData.process.environmentType;
            process.language = scheduleBatchData.batchData.process.language;
            process.percentage = scheduleBatchData.batchData.process.samplePercentage;
            process.type = scheduleBatchData.batchData.process.type;
            process.batchScheduleId = batchScheduleId;
            evaluationPlan.processId = process.name;
        }
        return (0, promises_shared_1.DummyPromise)()
            .then(function (_) {
            return mssql_service_1.MSSql.GetProcessFilters(vcc, scheduleBatchData.processFiltersId);
        })
            .then(function (result) {
            interactionSearch = (0, interaction_service_1.GetInteractionSearchFromProcessFilter)(result);
            interactionSearch.searchType = 'TO_SEARCH';
            interactionSearch.storageId = (0, utilities_1.guid)();
            interactionSearch.dateIni = moment(scheduleBatchData.batchData.evaluationPlan.startDate).toDate();
            interactionSearch.dateFin = moment(scheduleBatchData.batchData.evaluationPlan.endDate).toDate();
            logger.info('[BacthScheduleImplementation::CreateEvaluationPlanSchedule] executionCode:', executionCode, 'filter for schedule', queueId, interactionSearch);
            return Promise.resolve(true);
        })
            .then(function (_) {
            var promiseArray = [];
            promiseArray = (0, interaction_service_1.GetChannelNewInteractionSummaryPromises)(vcc, scheduleBatchData.batchData.evaluationPlan.vccType, true, 'SUMMARIZED', interactionSearch, null);
            return Promise.all(promiseArray);
        })
            .then(function (result) {
            var totalInteractions = 0;
            result.forEach(function (r) { return totalInteractions += r ? r.totalInteractions : 0; });
            if (totalInteractions) {
                logger.info('[BacthScheduleImplementation::CreateEvaluationPlanSchedule] executionCode:', executionCode, 'creating process for', queueId, 'interactions:', totalInteractions);
                interactionSearch.searchType = 'TO_OPERATE';
                if (scheduleBatchData.batchData.process) {
                    process.name = (0, utilities_1.guid)();
                    evaluationPlan.processId = process.name;
                }
                evaluationPlan.id = batchScheduleId.toLowerCase();
                if (evaluationPlan.id.length > 40)
                    evaluationPlan.id = evaluationPlan.id.substr(0, 40);
                evaluationPlan.id += '-' + ('000' + scheduleBatchData.execution.order).slice(-3);
                evaluationPlan.id += '-' + (0, utilities_1.guid)().slice(-5);
                evaluationPlan.description = evaluationPlan.id;
                scheduleBatchData.execution.batchId = evaluationPlan.id;
                (0, evaluation_plan_service_1.SetAuditorInteractionsNumbersFromPercentage)(evaluationPlan.auditors, evaluationPlan.modality, totalInteractions);
                return (0, evaluation_plan_service_1.CreateEvaluationPlan)(vcc, scheduleBatchData.batchData.evaluationPlan.vccType, evaluationPlan, process, (process === null || process === void 0 ? void 0 : process.type) ? true : false, interactionSearch, scheduleBatchData.execution.createdByUserId);
            }
            else {
                logger.info('[BacthScheduleImplementation::CreateEvaluationPlanSchedule] executionCode:', executionCode, 'no interactions found for', queueId);
                scheduleBatchData.execution.status = 'CANCELLED_BY_NO_INTERACTIONS_FOUND';
                throw 'SERVER_ERROR_NO_INTERACTIONS_FOUND_FOR_EXECUTION';
            }
        })
            .then(function (result) {
            return Promise.resolve(result);
        })
            .then(function (result) {
            if (result) {
                if (scheduleBatchData.batchData.process)
                    scheduleBatchData.batchData.process.name = process.name;
                scheduleBatchData.batchData.evaluationPlan.id = evaluationPlan.id;
                scheduleBatchData.batchData.evaluationPlan.description = evaluationPlan.description;
                scheduleBatchData.batchData.evaluationPlan.processId = evaluationPlan.processId;
                scheduleBatchData.execution.status = 'BATCH_CREATED';
                scheduleBatchData.retry = 0;
                scheduleBatchData.errorMessage = null;
                scheduleBatchData.errorDetail = null;
            }
            else {
                scheduleBatchData.retry = scheduleBatchData.retry ? scheduleBatchData.retry + 1 : 0;
                scheduleBatchData.errorFunction = 'CreateEvaluationPlanSchedule';
                scheduleBatchData.errorMessage = 'CREATE_BATCH_SCHEDULE_FAILED';
                scheduleBatchData.errorDetail = 'CREATE_BATCH_SCHEDULE_FAILED';
            }
            return Promise.resolve(true);
        })
            .catch(function (err) {
            var errorMessage = typeof err === 'string' ? err : (err.message || err.description);
            logger.error('[BacthScheduleImplementation::CreateEvaluationPlanSchedule] executionCode:', executionCode, 'getting errorMessage:', errorMessage);
            scheduleBatchData.retry = scheduleBatchData.retry ? scheduleBatchData.retry + 1 : 0;
            scheduleBatchData.errorFunction = 'CreateEvaluationPlanSchedule';
            scheduleBatchData.errorMessage = errorMessage;
            scheduleBatchData.errorDetail = JSON.stringify(err);
            if (scheduleBatchData.execution.status === 'CANCELLED_BY_NO_INTERACTIONS_FOUND')
                scheduleBatchData.retry = config_1.RETRY_BATCH_SCHEDULE;
            return Promise.resolve(false);
        });
    };
    BatchScheduleImplementation.prototype.StartScheduleBatchExecutions = function (queueId, executionCode, scheduleBatchData) {
        var _this = this;
        var queueData = queueId.split(':');
        var vcc = queueData[0];
        var batchScheduleId = queueData[1];
        var batchtype = queueData[2];
        var batchScheduleExecutionId = queueData[3];
        return (0, promises_shared_1.DummyPromise)()
            .then(function (_) {
            var promiseArray = [];
            var billing = new billing_implementation_1.BillingImplementation();
            promiseArray.push(billing.GetVCCRates(vcc, scheduleBatchData.execution.processId));
            promiseArray.push(billing.GetVCCBalance(vcc));
            return Promise.all(promiseArray);
        })
            .then(function (result) {
            var billingData = {
                rates: result[0],
                balance: result[1]
            };
            switch (batchtype) {
                case 'TRANSCRIPTION': return _this.StartScheduledProcess(scheduleBatchData, billingData, queueId, executionCode);
                case 'TEMPLATE': return _this.StartScheduledTemplate(scheduleBatchData, billingData, queueId, executionCode);
                case 'EVALUATION_PLAN': return _this.StartScheduledEvaluationPlan(scheduleBatchData, billingData, queueId, executionCode);
                default: throw 'SERVER_ERROR_BATCH_SCHEDULE_WITH_UNKNOWN_BATCH_TYPE';
            }
        })
            .then(function (_) {
            var scheduleExecution = new schedule_model_1.ScheduleExecution();
            scheduleExecution.vcc = vcc;
            scheduleExecution.id = batchScheduleExecutionId;
            scheduleExecution.batchScheduleId = batchScheduleId;
            scheduleExecution.batchId = scheduleBatchData.execution.batchId;
            scheduleExecution.status = scheduleBatchData.execution.status;
            return mssql_service_1.MSSql.SaveBatchScheduleExecution(vcc, scheduleExecution, 'system');
        })
            .then(function (_) {
            return Promise.resolve(true);
        })
            .catch(function (err) {
            var errorMessage = typeof err === 'string' ? err : (err.message || err.description);
            logger.error('[BacthScheduleImplementation::StartScheduleBatchExecutions] executionCode:', executionCode, 'getting errorMessage:', errorMessage);
            scheduleBatchData.retry = scheduleBatchData.retry ? scheduleBatchData.retry + 1 : 0;
            scheduleBatchData.errorFunction = 'StartScheduleBatchExecutions';
            scheduleBatchData.errorMessage = errorMessage;
            scheduleBatchData.errorDetail = JSON.stringify(err);
            if (scheduleBatchData.execution.status === 'CANCELLED_BY_NO_INTERACTIONS_FOUND')
                scheduleBatchData.retry = config_1.RETRY_BATCH_SCHEDULE;
            return Promise.resolve(false);
        });
    };
    BatchScheduleImplementation.prototype.StartScheduledProcess = function (scheduleBatchData, billingData, queueId, executionCode) {
        var queueData = queueId.split(':');
        var vcc = queueData[0];
        var batchScheduleId = queueData[1];
        var batchtype = queueData[2];
        var batchScheduleExecutionId = queueData[3];
        var started = true;
        return (0, promises_shared_1.DummyPromise)()
            .then(function (_) {
            return (0, process_service_1.GetProcessChannelInteractions)(vcc, scheduleBatchData.execution.batchId);
        })
            .then(function (result) {
            if (!result)
                throw 'SERVER_ERROR_CAN_NOT_GET_PROCESS_INTERACTIONS';
            var callResult = result.find(function (r) { return r.interactionType === 'CALL'; });
            if (callResult) {
                var transcriptionBalance = billingData.balance.find(function (b) { return b.systemService === 'TRANSCRIPTION'; });
                var ratedTime = billingData.rates.transcriptionRatePerSecond * callResult.notTranscribedDurationTime;
                var transcriptionAvailableBalance = transcriptionBalance.availableBalance - (transcriptionBalance.exceeded ? transcriptionBalance.difference : 0);
                if (ratedTime <= transcriptionAvailableBalance) {
                    return (0, linguistic_service_1.StartProcess)(vcc, scheduleBatchData.execution.batchId, scheduleBatchData.execution.createdByUserId);
                }
                else {
                    started = false;
                    scheduleBatchData.execution.status === 'PAUSED_BY_INSUFFICIENT_TRANSCRIPTION_BALANCE';
                }
            }
            else {
                return (0, linguistic_service_1.StartProcess)(vcc, scheduleBatchData.execution.batchId, scheduleBatchData.execution.createdByUserId);
            }
            return Promise.resolve(true);
        })
            .then(function (result) {
            if (started) {
                if (result)
                    scheduleBatchData.execution.status = 'BATCH_STARTED';
                else
                    scheduleBatchData.execution.status = 'BATCH_COULD_NOT_BE_STARTED';
            }
            scheduleBatchData.retry = 0;
            scheduleBatchData.errorFunction = null;
            scheduleBatchData.errorMessage = null;
            scheduleBatchData.errorDetail = null;
            return Promise.resolve(true);
        })
            .catch(function (err) {
            var errorMessage = typeof err === 'string' ? err : (err.message || err.description);
            logger.error('[BacthScheduleImplementation::StartScheduleProcess] executionCode:', executionCode, 'getting errorMessage:', errorMessage);
            scheduleBatchData.retry = scheduleBatchData.retry ? scheduleBatchData.retry + 1 : 0;
            scheduleBatchData.errorFunction = 'StartScheduleProcess';
            scheduleBatchData.errorMessage = errorMessage;
            scheduleBatchData.errorDetail = JSON.stringify(err);
            if (scheduleBatchData.execution.status === 'CANCELLED_BY_NO_INTERACTIONS_FOUND')
                scheduleBatchData.retry = config_1.RETRY_BATCH_SCHEDULE;
            return Promise.resolve(false);
        });
    };
    BatchScheduleImplementation.prototype.StartScheduledTemplate = function (scheduleBatchData, billingData, queueId, executionCode) {
        var queueData = queueId.split(':');
        var vcc = queueData[0];
        var batchScheduleId = queueData[1];
        var batchtype = queueData[2];
        var batchScheduleExecutionId = queueData[3];
        var started = true;
        var studyId = scheduleBatchData.execution.batchId;
        var templateId = scheduleBatchData.processFiltersId;
        var processId = scheduleBatchData.execution.processId;
        return (0, promises_shared_1.DummyPromise)()
            .then(function (_) {
            var promiseArray = [];
            promiseArray.push(mssql_service_1.MSSql.GetTemplateThemeItems(vcc, templateId, null));
            promiseArray.push(mssql_service_1.MSSql.GetTemplateThemeModels(vcc, templateId, null));
            promiseArray.push(mssql_service_1.MSSql.GetProcessInteractionsHistory(vcc, processId, billingData.rates.cognitiveAnalysisAverageSecondsPerRequest));
            return Promise.all(promiseArray);
        })
            .then(function (result) {
            var semanticAnalysisBalance = billingData.balance.find(function (b) { return b.systemService === 'SEMANTIC_ANALYSIS'; });
            var cognitiveAnalysisBalance = billingData.balance.find(function (b) { return b.systemService === 'COGNITIVE_ANALYSIS'; });
            var totalQueries = result[0].length;
            var totalModels = 0;
            var modelIntents = result[1].map(function (mi) { return mi.id; });
            var models = modelIntents.map(function (i) { return i.split('|')[0]; });
            var nluModels = (0, arrays_shared_1.GetUniqueFromArray)(models);
            totalModels = nluModels.length;
            var historicalInteractionsData = result[2];
            var semanticAnalysisAvailableBalance = semanticAnalysisBalance.availableBalance - (semanticAnalysisBalance.exceeded ? semanticAnalysisBalance.difference : 0);
            var cognitiveAnalysisAvailableBalance = cognitiveAnalysisBalance.availableBalance - (cognitiveAnalysisBalance.exceeded ? cognitiveAnalysisBalance.difference : 0);
            var ratedQueries = billingData.rates.semanticAnalysisRatePerTransaction * totalQueries;
            var ratedModels = 0;
            ratedModels = Math.ceil(totalModels
                *
                    billingData.rates.cognitiveAnalysisRatePerTransaction
                * ((billingData.rates.processCallPhrases * (1 + billingData.rates.cognitiveAnalysisBillingSecureRate))
                    +
                        (billingData.rates.processTextPhrases * (1 + billingData.rates.cognitiveAnalysisTextBillingSecureRate))));
            if (ratedQueries <= semanticAnalysisAvailableBalance
                &&
                    ratedModels <= cognitiveAnalysisAvailableBalance) {
                var studyData = {
                    studyId: studyId,
                    processId: processId
                };
                return (0, study_service_1.BeginStudy)(vcc, studyData, scheduleBatchData.execution.createdByUserId);
            }
            else {
                started = false;
                scheduleBatchData.execution.status = 'PAUSED_BY_INSUFFICIENT_REQUESTS_BALANCE';
                logger.warn('[BacthScheduleImplementation::StartScheduledTemplate] executionCode:', executionCode, 'RatedQueries:', ratedQueries, 'AvailableBalance:', semanticAnalysisAvailableBalance);
                logger.warn('[BacthScheduleImplementation::StartScheduledTemplate] executionCode:', executionCode, 'RatedModels:', ratedModels, 'AvailableBalance:', cognitiveAnalysisAvailableBalance);
            }
            return Promise.resolve(true);
        })
            .then(function (result) {
            if (started) {
                if (result === null || result === void 0 ? void 0 : result.status)
                    scheduleBatchData.execution.status = 'BATCH_STARTED';
                else
                    scheduleBatchData.execution.status = 'BATCH_COULD_NOT_BE_STARTED';
            }
            scheduleBatchData.retry = 0;
            scheduleBatchData.errorFunction = null;
            scheduleBatchData.errorMessage = null;
            scheduleBatchData.errorDetail = null;
            return Promise.resolve(true);
        })
            .catch(function (err) {
            var errorMessage = typeof err === 'string' ? err : (err.message || err.description);
            logger.error('[BacthScheduleImplementation::StartScheduledTemplate] executionCode:', executionCode, 'getting errorMessage:', errorMessage);
            scheduleBatchData.retry = scheduleBatchData.retry ? scheduleBatchData.retry + 1 : 0;
            scheduleBatchData.errorFunction = 'StartScheduledTemplate';
            scheduleBatchData.errorMessage = errorMessage;
            scheduleBatchData.errorDetail = JSON.stringify(err);
            return Promise.resolve(false);
        });
    };
    BatchScheduleImplementation.prototype.StartScheduledEvaluationPlan = function (scheduleBatchData, billingData, queueId, executionCode) {
        var queueData = queueId.split(':');
        var vcc = queueData[0];
        var batchScheduleId = queueData[1];
        var batchtype = queueData[2];
        var batchScheduleExecutionId = queueData[3];
        var started = true;
        return (0, promises_shared_1.DummyPromise)()
            .then(function (_) {
            var promiseArray = [];
            promiseArray.push((0, process_service_1.GetProcessChannelInteractions)(vcc, scheduleBatchData.batchData.evaluationPlan.processId));
            promiseArray.push(mssql_service_1.MSSql.GetTemplateThemeItems(vcc, scheduleBatchData.batchData.evaluationPlan.templateId, null));
            promiseArray.push(mssql_service_1.MSSql.GetTemplateThemeModels(vcc, scheduleBatchData.batchData.evaluationPlan.templateId, null));
            promiseArray.push(mssql_service_1.MSSql.GetProcessInteractionsHistory(vcc, scheduleBatchData.batchData.evaluationPlan.processId, billingData.rates.cognitiveAnalysisAverageSecondsPerRequest));
            return Promise.all(promiseArray);
        })
            .then(function (result) {
            var transcriptionBalance = billingData.balance.find(function (b) { return b.systemService === 'TRANSCRIPTION'; });
            var semanticAnalysisBalance = billingData.balance.find(function (b) { return b.systemService === 'SEMANTIC_ANALYSIS'; });
            var cognitiveAnalysisBalance = billingData.balance.find(function (b) { return b.systemService === 'COGNITIVE_ANALYSIS'; });
            var callResult = result[0].find(function (r) { return r.interactionType === 'CALL'; });
            var totalQueries = result[1].length;
            var totalModels = 0;
            var ratedTime = billingData.rates.transcriptionRatePerSecond * callResult.notTranscribedDurationTime;
            var modelIntents = result[2].map(function (mi) { return mi.id; });
            var models = modelIntents.map(function (i) { return i.split('|')[0]; });
            var nluModels = (0, arrays_shared_1.GetUniqueFromArray)(models);
            totalModels = nluModels.length;
            var historicalInteractionsData = result[3];
            var transcriptionAvailableBalance = transcriptionBalance.availableBalance - (transcriptionBalance.exceeded ? transcriptionBalance.difference : 0);
            var semanticAnalysisAvailableBalance = semanticAnalysisBalance.availableBalance - (semanticAnalysisBalance.exceeded ? semanticAnalysisBalance.difference : 0);
            var cognitiveAnalysisAvailableBalance = cognitiveAnalysisBalance.availableBalance - (cognitiveAnalysisBalance.exceeded ? cognitiveAnalysisBalance.difference : 0);
            var ratedQueries = billingData.rates.semanticAnalysisRatePerTransaction * totalQueries;
            var ratedModels = 0;
            ratedModels = Math.ceil(totalModels
                *
                    billingData.rates.cognitiveAnalysisRatePerTransaction
                * ((billingData.rates.processCallPhrases * (1 + billingData.rates.cognitiveAnalysisBillingSecureRate))
                    +
                        (billingData.rates.processTextPhrases * (1 + billingData.rates.cognitiveAnalysisTextBillingSecureRate))));
            if (ratedTime <= transcriptionAvailableBalance
                &&
                    ratedQueries <= semanticAnalysisAvailableBalance
                &&
                    ratedModels <= cognitiveAnalysisAvailableBalance) {
                return (0, evaluation_plan_service_1.StartEvaluationPlan)(vcc, scheduleBatchData.batchData.evaluationPlan);
            }
            else {
                started = false;
                scheduleBatchData.execution.status = 'PAUSED_BY_INSUFFICIENT_BALANCE';
                logger.warn('[BacthScheduleImplementation::StartScheduledEvaluationPlan] executionCode:', executionCode, 'RatedTime:', ratedTime, 'AvailableBalance:', transcriptionAvailableBalance);
                logger.warn('[BacthScheduleImplementation::StartScheduledEvaluationPlan] executionCode:', executionCode, 'RatedQueries:', ratedQueries, 'AvailableBalance:', semanticAnalysisAvailableBalance);
                logger.warn('[BacthScheduleImplementation::StartScheduledEvaluationPlans] executionCode:', executionCode, 'RatedModels:', ratedModels, 'AvailableBalance:', cognitiveAnalysisAvailableBalance);
            }
            return Promise.resolve(true);
        })
            .then(function (result) {
            if (started) {
                if (result)
                    scheduleBatchData.execution.status = 'BATCH_STARTED';
                else
                    scheduleBatchData.execution.status = 'BATCH_COULD_NOT_BE_STARTED';
            }
            scheduleBatchData.retry = 0;
            scheduleBatchData.errorFunction = null;
            scheduleBatchData.errorMessage = null;
            scheduleBatchData.errorDetail = null;
            return Promise.resolve(true);
        })
            .catch(function (err) {
            var errorMessage = typeof err === 'string' ? err : (err.message || err.description);
            logger.error('[BacthScheduleImplementation::StartScheduledEvaluationPlan] executionCode:', executionCode, 'getting errorMessage:', errorMessage);
            scheduleBatchData.retry = scheduleBatchData.retry ? scheduleBatchData.retry + 1 : 0;
            scheduleBatchData.errorFunction = 'StartScheduledEvaluationPlan';
            scheduleBatchData.errorMessage = errorMessage;
            scheduleBatchData.errorDetail = JSON.stringify(err);
            return Promise.resolve(false);
        });
    };
    BatchScheduleImplementation.prototype.UpdateScheduleBatch = function (queueId, executionCode, scheduleBatchData) {
        return (0, promises_shared_1.DummyPromise)()
            .then(function (_) {
            return Promise.resolve(true);
        })
            .catch(function (err) {
            var errorMessage = typeof err === 'string' ? err : (err.message || err.description);
            logger.error('[BacthScheduleImplementation::UpdateScheduleBatch] executionCode:', executionCode, 'getting errorMessage:', errorMessage);
            scheduleBatchData.retry = scheduleBatchData.retry ? scheduleBatchData.retry + 1 : 0;
            scheduleBatchData.errorFunction = 'UpdateScheduleBatch';
            scheduleBatchData.errorMessage = errorMessage;
            scheduleBatchData.errorDetail = JSON.stringify(err);
            if (scheduleBatchData.execution.status === 'CANCELLED_BY_NO_INTERACTIONS_FOUND')
                scheduleBatchData.retry = config_1.RETRY_BATCH_SCHEDULE;
            return Promise.resolve(false);
        });
    };
    return BatchScheduleImplementation;
}());
exports.BatchScheduleImplementation = BatchScheduleImplementation;
