"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.IntentMigrator = void 0;
var redis_service_1 = require("../../db/redis.service");
var log4js = require("log4js");
var mssql_service_1 = require("../../db/mssql.service");
var promises_shared_1 = require("../../shared/promises.shared");
var helper_1 = require("./helper");
var enum_1 = require("./enum");
var arrays_shared_1 = require("../../shared/arrays.shared");
var logger = log4js.getLogger('Task');
var IntentMigrator = (function () {
    function IntentMigrator() {
    }
    IntentMigrator.prototype.MigrateIntents = function (vcc, data, start, end) {
        var _this = this;
        logger.debug("[CognitiveEngineMigrateImplementation::MigrateIntents] Migrating intents for ", vcc, " into ", data.newEngine, " from ", start, " to ", end);
        return (0, promises_shared_1.DummyPromise)().then(function (_) {
            return redis_service_1.Redis.LRange("".concat(enum_1.ROOT_KEY, ":").concat(vcc, ":Intents:Keys"), start, end - 1);
        }).then(function (intentKeys) {
            logger.debug("[CognitiveEngineMigrateImplementation::MigrateIntents] Found intent keys:", intentKeys);
            if (!intentKeys || intentKeys.length == 0) {
                return false;
            }
            return redis_service_1.Redis.GetCognitiveEngineMigrationModel("".concat(enum_1.ROOT_KEY, ":").concat(vcc, ":Intents"), (0, arrays_shared_1.GetUniqueFromArray)(intentKeys));
        }).then(function (intents) {
            if (intents) {
                var intentList = [];
                for (var _i = 0, intents_1 = intents; _i < intents_1.length; _i++) {
                    var i = intents_1[_i];
                    var intent = JSON.parse(i);
                    if (intent) {
                        intentList.push(intent);
                    }
                }
                if (intentList.length > 0) {
                    return _this.ProcessIntents(vcc, data, intentList);
                }
            }
            return false;
        }).then(function (_) {
            if (end < data.totalIntent) {
                return _this.MigrateIntents(vcc, data, start + enum_1.ITEMS_PER_PAGE, end + enum_1.ITEMS_PER_PAGE);
            }
            return true;
        });
    };
    IntentMigrator.prototype.ProcessIntents = function (vcc, data, intents) {
        var _this = this;
        return (0, promises_shared_1.DummyPromise)().then(function (_) {
            var promises = [];
            for (var _i = 0, intents_2 = intents; _i < intents_2.length; _i++) {
                var intent = intents_2[_i];
                switch (intent.status) {
                    case 0:
                        promises.push(_this.AddIntent(vcc, data, intent));
                        break;
                    default: break;
                }
            }
            return Promise.all(promises);
        }).then(function (results) {
            logger.info('[CognitiveEngineMigrateImplementation::ProcessIntents] Results for process intents:', results);
            if (results.length > 0) {
                var totalCreated = results.filter(function (r) { return r == true; }).length;
                var totalError = results.filter(function (r) { return !r; }).length;
                return redis_service_1.Redis.UpdateCognitiveEngineMigrationData(vcc, Object.assign(data, {
                    totalCreatedIntent: totalCreated + data.totalCreatedIntent,
                    totalErrorIntent: totalError + data.totalErrorIntent
                }));
            }
            return false;
        });
    };
    IntentMigrator.prototype.AddIntent = function (vcc, data, intent) {
        logger.info('[CognitiveEngineMigrateImplementation::AddIntent]  Adding intent ', intent.name, ' in ', data.newEngine);
        var AppId = null;
        return (0, promises_shared_1.DummyPromise)().then(function (_) {
            return redis_service_1.Redis.GetCognitiveEngineMigrationNewId("".concat(enum_1.ROOT_KEY, ":").concat(vcc, ":AppIds"), intent.appId);
        }).
            then(function (appId) {
            if (appId) {
                AppId = appId;
                return (0, helper_1.GetServiceInstance)(data.newEngine);
            }
            logger.debug('[CognitiveEngineMigrateImplementation::AddIntent] Not found AppId for intent ', intent.name);
            return null;
        })
            .then(function (service) {
            if (service && AppId) {
                return service.AddIntent(data.subscriptionRegion, data.subscriptionKey, AppId, intent.name);
            }
            logger.error('[CognitiveEngineMigrateImplementation::AddIntent] Error on get service and appId for ', intent.name);
            return null;
        }).then(function (newIntentId) {
            if (newIntentId) {
                logger.info('[CognitiveEngineMigrateImplementation::AddIntent] Success new IntentId:', newIntentId, ' for intent:', intent.name);
                var promises = [];
                promises.push(Promise.resolve(newIntentId));
                promises.push(mssql_service_1.MSSql.UpdateIntentId(vcc, AppId, intent.intentId, newIntentId));
                promises.push(redis_service_1.Redis.SaveCognitiveEngineMigrationNewId("".concat(enum_1.ROOT_KEY, ":").concat(vcc, ":IntentIds"), intent.key, newIntentId));
                return Promise.all(promises);
            }
            logger.error("[CognitiveEngineMigrateImplementation::AddIntent] Error on add intent into ".concat(data.newEngine, ":"), newIntentId);
            return [null];
        })
            .then(function (results) {
            logger.info('[CognitiveEngineMigrateImplementation::AddIntent] Results for Update new intent Id:', results);
            var newIntentId = results[0];
            if (newIntentId) {
                var promises = [];
                promises.push(mssql_service_1.MSSql.SaveCognitiveEngineMigrateStatus(vcc, 'intent', newIntentId, 1, enum_1.AVAILABLE));
                promises.push(redis_service_1.Redis.DeleteCognitiveEngineMigrationModel("".concat(enum_1.ROOT_KEY, ":").concat(vcc, ":Intents"), intent.key));
                return Promise.all(promises);
            }
            return [false];
        })
            .then(function (results) {
            return !results.includes(false);
        })
            .catch(function (err) {
            logger.error("[CognitiveEngineMigrateImplementation::AddIntent] Unknown error on add intent into ".concat(data.newEngine), err);
            if (typeof err === 'string' && err == "SERVER_ERROR_INTENT_ALREADY_EXIST") {
                var promises = [];
                promises.push(redis_service_1.Redis.UpdateCognitiveEngineMigrationModel("".concat(enum_1.ROOT_KEY, ":").concat(vcc, ":Intents"), intent.key, Object.assign(intent, {
                    status: 3
                })));
                promises.push(mssql_service_1.MSSql.SaveCognitiveEngineMigrateStatus(vcc, 'intent', intent.intentId, 3, 'ERROR_INTENT_EXISTS_IN_RASA'));
                return Promise.all(promises).then(function () {
                    return false;
                });
            }
            return false;
        });
    };
    return IntentMigrator;
}());
exports.IntentMigrator = IntentMigrator;
