"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.SemanticAnalysisDownloadImplementation = void 0;
var config_1 = require("../config/config");
var mssql_service_1 = require("../db/mssql.service");
var redis_service_1 = require("../db/redis.service");
var promises_shared_1 = require("../shared/promises.shared");
var log4js = require("log4js");
var moment = require("moment");
var billing_batch_model_1 = require("../models/billing-batch.model");
var billing_implementation_1 = require("./billing.implementation");
var file_manager_1 = require("../shared/file-manager");
var idatha_service_1 = require("../services/idatha.service");
var cognitive_analysis_implementation_1 = require("./cognitive-analysis.implementation");
var idatha_model_1 = require("../models/idatha.model");
var request_shared_1 = require("../shared/request.shared");
var logger = log4js.getLogger('Task');
var CURRENT_SYSTEM_SERVICE = 'TRANSCRIPTION';
var CURRENT_IMPLEMENTATION = 'SemanticAnalysisDownloadImplementation';
var SemanticAnalysisDownloadImplementation = (function () {
    function SemanticAnalysisDownloadImplementation() {
        this.errorData = new billing_batch_model_1.BatchError();
        this.queueBase = 'SemanticAnalysisDownload';
    }
    SemanticAnalysisDownloadImplementation.prototype.Execute = function (executionCode, rateRecursive) {
        var _this = this;
        logger.info("[SemanticAnalysisDownloadImplementation::Execute] executionCode: ".concat(executionCode, " executing wave #").concat(rateRecursive));
        var executeIndex = 1;
        var ratio = config_1.RATIO_SEMANTIC_ANALYSIS_DOWNLOAD;
        var executeResult = true;
        var currentBlockPromise = null;
        return (0, promises_shared_1.SleepPromise)(config_1.SLEEP_SEMANTIC_ANALYSIS)
            .then(function (_) {
            currentBlockPromise = 'BLOCK_01_GET_FROM_REDIS';
            return redis_service_1.Redis.GetLengthFromList(_this.queueBase);
        })
            .then(function (result) {
            currentBlockPromise = 'BLOCK_02_GET_QUEUE_ITEMS';
            if (!result)
                throw 'SERVER_INFO_REDIS_KEYS_FOR_SEMANTIC_ANALYSIS_DOWNLOAD_NOT_FOUND';
            return redis_service_1.Redis.LRange(_this.queueBase, 0, ratio - 1);
        })
            .then(function (result) {
            currentBlockPromise = 'BLOCK_03_SEMANTIC_ANALYSIS_DOWNLOAD_IMPL';
            ratio = result && result.length <= ratio ? result.length : 0;
            var promiseArray = [];
            while (ratio >= executeIndex) {
                promiseArray.push(_this.SemanticAnalysisDownloadExecute(executionCode));
                executeIndex++;
            }
            if (!promiseArray.length)
                promiseArray.push(Promise.resolve(true));
            return Promise.all(promiseArray);
        })
            .then(function (results) {
            currentBlockPromise = 'BLOCK_04_GETTING_KEYS_FOR_RECURSIVITY';
            if (results.filter(function (item) { return item === false; }).length > 0) {
                executeResult = false;
            }
            return redis_service_1.Redis.LRange(_this.queueBase, 0, ratio - 1);
        })
            .then(function (result) {
            currentBlockPromise = 'BLOCK_05_RELAUNCH_SEMANTIC_ANALYSIS_UPLOAD_IMP';
            if ((result === null || result === void 0 ? void 0 : result.length) && rateRecursive >= 1)
                return _this.Execute(executionCode, rateRecursive - 1);
            return Promise.resolve(executeResult);
        })
            .then(function (result) {
            currentBlockPromise = 'BLOCK_06_RESOLVING_IMPL';
            if (rateRecursive === config_1.RATE_RECURSIVE_SEMANTIC_ANALYSIS_DOWNLOAD)
                logger.info("[SemanticAnalysisDownloadImplementation::Execute] executionCode: ".concat(executionCode, " Finishing execution wave #").concat(rateRecursive));
            var currentResult = executeResult && result ? true : false;
            return Promise.resolve(currentResult);
        })
            .catch(function (error) {
            var errorMessage = typeof error === 'string' ? error : (error.message || error.description);
            if (errorMessage.startsWith('SERVER_INFO_'))
                logger.info("[SemanticAnalysisDownloadImplementation::Execute] executionCode: ".concat(executionCode, ", Block: ").concat(currentBlockPromise, ", getting info:"), errorMessage);
            else
                logger.error("[SemanticAnalysisDownloadImplementation::Execute] executionCode: ".concat(executionCode, ", Block: ").concat(currentBlockPromise, ", getting error:"), errorMessage);
            return Promise.resolve(false);
        });
    };
    SemanticAnalysisDownloadImplementation.prototype.SemanticAnalysisDownloadExecute = function (executionCode) {
        var _this = this;
        var storage = '';
        var currentQueueId = '';
        var processInteraction = null;
        var vcc = '';
        var processId = '';
        var interactionType = '';
        var interactionId = '';
        var interactionSection = 0;
        var interactionPart = 0;
        return redis_service_1.Redis.RPopLPush(this.queueBase, "".concat(this.queueBase, "Execute"))
            .then(function (queueId) {
            if ((queueId === null || queueId === void 0 ? void 0 : queueId.length) > 0) {
                currentQueueId = queueId;
                logger.info('[SemanticAnalysisDownloadImplementation::SemanticAnalysisDownloadExecute] executionCode:', executionCode, 'getting queueId:', currentQueueId, 'START');
                var item = queueId.split(':');
                if (item === null || item === void 0 ? void 0 : item.length) {
                    vcc = item[0];
                    processId = item[1];
                    interactionType = item[2];
                    interactionId = item[3];
                    interactionSection = Number(item[4]);
                    interactionPart = Number(item[5]);
                    storage = vcc + ':' + processId;
                    logger.info('[SemanticAnalysisDownloadImplementation::SemanticAnalysisDownloadExecute] executionCode:', executionCode, 'getting queueId:', currentQueueId, 'storage :', storage);
                    return redis_service_1.Redis.GetProcessInteraction(storage, currentQueueId);
                }
                else {
                    logger.error('[SemanticAnalysisDownloadImplementation::SemanticAnalysisDownloadExecute] executionCode:', executionCode, 'getting queueId:', currentQueueId, 'QUEUE_FORMAT_ERROR');
                }
            }
            return Promise.resolve(null);
        })
            .then(function (processInteractionResult) {
            if (processInteractionResult) {
                processInteraction = processInteractionResult;
                logger.info('[SemanticAnalysisDownloadImplementation::SemanticAnalysisDownloadExecute] executionCode:', executionCode, 'getting queueId:', currentQueueId, 'getting GetProcessInteraction: 1, status:', processInteraction.statusDescription);
                if (processInteraction.statusDescription === 'pendingDownloadJSON') {
                    var validTime = Number(moment().format('x'));
                    var nextExecutionTime = Number(processInteraction.uploadedDate);
                    nextExecutionTime = nextExecutionTime + config_1.NEXT_EXECUTION_TIME_SEMANTIC_ANALYSIS;
                    if (validTime > nextExecutionTime) {
                        return _this.DownloadJSON(processInteraction, currentQueueId, executionCode);
                    }
                    else {
                        logger.info('[SemanticAnalysisDownloadImplementation::SemanticAnalysisDownloadExecute] executionCode:', executionCode, 'getting queueId:', currentQueueId, 'NEXT_EXECUTION', nextExecutionTime, 'CURRENT_TIME', validTime);
                    }
                }
            }
            if (!processInteraction && currentQueueId) {
                logger.warn('[SemanticAnalysisDownloadImplementation::SemanticAnalysisDownloadExecute] executionCode:', executionCode, 'getting queueId:', currentQueueId, 'getting GetProcessInteraction: 0');
                if (!processInteraction) {
                    redis_service_1.Redis.DeleteQueue("".concat(_this.queueBase, "Execute"), currentQueueId);
                    throw 'SERVER_ERROR_INTERACTION_DATA_NOT_FOUND_ON_STORAGE';
                }
            }
            return Promise.resolve(null);
        })
            .then(function (result) {
            if (result)
                processInteraction = result;
            if (processInteraction)
                return redis_service_1.Redis.SaveProcessInteraction(storage, currentQueueId, processInteraction);
            return Promise.resolve(null);
        })
            .then(function (result) {
            var promiseArray = [];
            var billing = new billing_implementation_1.BillingImplementation();
            if (result
                &&
                    (processInteraction === null || processInteraction === void 0 ? void 0 : processInteraction.status) === 6
                &&
                    (processInteraction === null || processInteraction === void 0 ? void 0 : processInteraction.statusDescription) === 'ready') {
                logger.info('[SemanticAnalysisDownloadImplementation::SemanticAnalysisDownloadExecute] executionCode:', executionCode, 'getting queueId:', currentQueueId, 'QUEUE_FINISHED');
                billing.AddUpdateTranscriptionDetailToMainFile(processInteraction, 'ValidChar', processInteraction.batchTypeName, processInteraction.batchTypeId);
                billing.AddUpdateTranscriptionDetailToMainFile(processInteraction, 'ValidWords', processInteraction.batchTypeName, processInteraction.batchTypeId);
                billing.AddUpdateTranscriptionDetailToMainFile(processInteraction, 'TotalPhrases', processInteraction.batchTypeName, processInteraction.batchTypeId);
                billing.AddUpdateTranscriptionDetailToMainFile(processInteraction, 'CharsPerPhrase', processInteraction.batchTypeName, processInteraction.batchTypeId);
                billing.AddUpdateTranscriptionDetailToMainFile(processInteraction, 'IdathaApi', processInteraction.batchTypeName, processInteraction.batchTypeId);
                billing.AddCloseTranscriptionDetailToMainFile(processInteraction, processInteraction.batchTypeName, processInteraction.batchTypeId, 1, 'SUCCED');
                promiseArray.push(redis_service_1.Redis.DeleteQueue("".concat(_this.queueBase, "Execute"), currentQueueId));
            }
            else if (result
                &&
                    (processInteraction === null || processInteraction === void 0 ? void 0 : processInteraction.status) === 6
                &&
                    (processInteraction === null || processInteraction === void 0 ? void 0 : processInteraction.statusDescription) === 'pendingConvertAudio'
                &&
                    (processInteraction === null || processInteraction === void 0 ? void 0 : processInteraction.retry) < config_1.RETRY_SEMANTIC_ANALYSIS) {
                promiseArray.push(redis_service_1.Redis.InsertQueue('SemanticAnalysisConvertAudio', currentQueueId, config_1.RATE_RECURSIVE_SEMANTIC_ANALYSIS_DOWNLOAD * config_1.RATIO_SEMANTIC_ANALYSIS_DOWNLOAD * 2));
                promiseArray.push(redis_service_1.Redis.DeleteQueue("".concat(_this.queueBase, "Execute"), currentQueueId));
            }
            else if (currentQueueId && !processInteraction) {
                logger.error('[SemanticAnalysisDownloadImplementation::SemanticAnalysisDownloadExecute] executionCode:', executionCode, 'getting queueId:', currentQueueId, 'INTERACTION_DO_NOT_EXIST');
                var interactionData = {
                    vcc: vcc,
                    processId: processId,
                    id: interactionId,
                    section: interactionSection,
                    part: interactionPart
                };
                billing.AddCloseTranscriptionDetailToMainFile(interactionData, processInteraction.batchTypeName, processInteraction.batchTypeId, 0, 'INTERACTION_DO_NOT_EXIST');
                promiseArray.push(redis_service_1.Redis.DeleteQueue("".concat(_this.queueBase, "Execute"), currentQueueId));
            }
            else {
                logger.warn('[SemanticAnalysisDownloadImplementation::SemanticAnalysisDownloadExecute] executionCode:', executionCode, 'getting queueId:', currentQueueId, 'QUEUE_WAS_NOT_REMOVED');
            }
            if (!promiseArray.length)
                promiseArray.push(Promise.resolve(true));
            return Promise.all(promiseArray);
        })
            .then(function (_) {
            var promiseArray = [];
            if ((processInteraction === null || processInteraction === void 0 ? void 0 : processInteraction.status) === 6
                &&
                    (processInteraction === null || processInteraction === void 0 ? void 0 : processInteraction.statusDescription) === 'ready') {
                promiseArray.push(redis_service_1.Redis.UpdateProgress(storage));
                promiseArray.push(redis_service_1.Redis.DeleteProcessInteraction(storage, currentQueueId));
                promiseArray.push(redis_service_1.Redis.SaveReadyProcessInteraction(executionCode, currentQueueId, processInteraction));
            }
            else if ((processInteraction === null || processInteraction === void 0 ? void 0 : processInteraction.retry) >= config_1.RETRY_SEMANTIC_ANALYSIS) {
                processInteraction.isCancel = true;
                promiseArray.push(redis_service_1.Redis.UpdateProgress(storage));
                promiseArray.push(redis_service_1.Redis.UpdateErrorProgress(storage));
                promiseArray.push(redis_service_1.Redis.DeleteProcessInteraction(storage, currentQueueId));
                promiseArray.push(redis_service_1.Redis.SaveReadyProcessInteraction(executionCode, currentQueueId, processInteraction));
                promiseArray.push(mssql_service_1.MSSql.SaveBatchError(vcc, CURRENT_SYSTEM_SERVICE, processInteraction.batchTypeId, processInteraction.batchTypeId, CURRENT_IMPLEMENTATION, processInteraction.id, null, processInteraction.errorFunction, processInteraction.errorMessage, processInteraction.errorDetail));
            }
            if (!promiseArray.length)
                promiseArray.push(Promise.resolve(true));
            return Promise.all(promiseArray);
        })
            .then(function (_) {
            return Promise.resolve(true);
        })
            .catch(function (err) {
            var errorMessage = typeof err === 'string' ? err : (err.message || err.description);
            logger.error('[SemanticAnalysisDownloadImplementation::SemanticAnalysisDownloadExecute] executionCode:', executionCode, 'getting queueId:', currentQueueId, 'Catching error: ' + errorMessage);
            if (processInteraction)
                logger.error('[SemanticAnalysisDownloadImplementation::SemanticAnalysisDownloadExecute] executionCode:', executionCode, 'getting queueId:', currentQueueId, 'getting error:', errorMessage, ' vcc', processInteraction.vcc, 'processId:', processInteraction.processId, 'id:', processInteraction.id, 'section:', processInteraction.section, 'part:', processInteraction.part);
            return Promise.resolve(false);
        });
    };
    SemanticAnalysisDownloadImplementation.prototype.DownloadJSON = function (processInteraction, currentQueueId, executionCode) {
        var _this = this;
        var path = '';
        var data;
        var silenceInTime;
        var silenceOutTime;
        var cognitiveService;
        return (0, promises_shared_1.DummyPromise)()
            .then(function (_) {
            return (0, cognitive_analysis_implementation_1.GetCognitiveService)(processInteraction.vcc, null);
        })
            .then(function (result) {
            cognitiveService = result;
            var promiseArray = [];
            promiseArray.push((0, file_manager_1.CreateFolder)(config_1.SEMANCTIC_ANALYSIS_FOLDER + '/' + processInteraction.vcc));
            promiseArray.push((0, file_manager_1.CreateFolder)(config_1.SEMANCTIC_ANALYSIS_VIEW_FOLDER + '/' + processInteraction.vcc));
            promiseArray.push((0, file_manager_1.CreateFolder)(config_1.COGNITIVE_ANALYSIS_FOLDER + '/' + processInteraction.vcc));
            return Promise.all(promiseArray);
        })
            .then(function (_) {
            var regex = /\.(mp3|wav|ogg|json)$/;
            var jsonFileName = processInteraction.fileName.replace(regex, '.json');
            path = "".concat(config_1.SEMANCTIC_ANALYSIS_FOLDER, "/").concat(processInteraction.vcc, "/").concat(jsonFileName);
            return (0, idatha_service_1.IdathaLogin)(processInteraction.vcc);
        })
            .then(function (userAccessToken) {
            if (!userAccessToken)
                throw 'NOT_FOUND_ACCESS_TOKEN';
            var regex = /\.(mp3|wav|ogg|json)$/;
            var fileName = processInteraction.fileName.replace(regex, '');
            return _this.GetJson(userAccessToken.accessToken, currentQueueId, executionCode, fileName);
        })
            .then(function (result) {
            var _a;
            data = result;
            logger.info('[SemanticAnalysisDownloadImplementation::DownloadJSON] executionCode:', executionCode, 'getting queueId:', currentQueueId, 'getting path:', path);
            if (result) {
                var jsonData = JSON.parse(data);
                var validChar_1 = 0;
                var validWords = 0;
                if ((_a = jsonData === null || jsonData === void 0 ? void 0 : jsonData.words) === null || _a === void 0 ? void 0 : _a.length) {
                    validWords = jsonData.words.length;
                    jsonData.words.forEach(function (r) {
                        validChar_1 += r.name ? r.name.length + 1 : 0;
                    });
                }
                processInteraction['validChar'] = validChar_1;
                processInteraction['validWords'] = validWords;
                processInteraction['idathaApi'] = "".concat(config_1.IDATHA_IP, ":").concat(config_1.IDATHA_HTTP_PORT);
                return (0, file_manager_1.WriteFileContent)(path, data, 'utf-8');
            }
            return Promise.resolve(null);
        })
            .then(function (result) {
            if (result) {
                var cognitiveAnalysis = new cognitive_analysis_implementation_1.CognitiveAnalysisImplementation();
                var transcription = cognitiveAnalysis.SortJson(JSON.parse(data));
                var phrases = cognitiveAnalysis.SplitTranscriptionContentInPhrases(cognitiveService ? cognitiveService.GetCognitiveEngineCharsPerPhrase() : 100, transcription);
                silenceInTime = transcription.summarySpeechData.dataTotalIn.silence;
                silenceOutTime = transcription.summarySpeechData.dataTotalOut.silence;
                logger.info('[SemanticAnalysisDownloadImplementation::DownloadJSON] executionCode:', executionCode, 'getting queueId:', currentQueueId, 'getting phrases:', phrases.length);
                return Promise.resolve(phrases);
            }
            return Promise.resolve(null);
        })
            .then(function (result) {
            var phrases;
            if (result === null || result === void 0 ? void 0 : result.length) {
                processInteraction['charsPerPhrase'] = cognitiveService ? cognitiveService.GetCognitiveEngineCharsPerPhrase() : 0;
                processInteraction['totalPhrases'] = result.length;
                var regex = /\.(mp3|wav|ogg|json)$/;
                var jsonFileName = processInteraction.fileName.replace(regex, '.json');
                var path_1 = "".concat(config_1.COGNITIVE_ANALYSIS_FOLDER, "/").concat(processInteraction.vcc, "/").concat(jsonFileName);
                phrases = { phrases: result };
                var stringPhrases = JSON.stringify(phrases);
                return (0, file_manager_1.WriteFileContent)(path_1, stringPhrases, 'utf-8');
            }
            return Promise.resolve(null);
        })
            .then(function (result) {
            if (result) {
                logger.info('[SemanticAnalysisDownloadImplementation::DownloadJSON] executionCode:', executionCode, 'getting queueId:', currentQueueId, 'getting succes: ready');
                processInteraction['status'] = 6;
                processInteraction['statusDescription'] = processInteraction.interactionType !== 'CALL' ? 'ready' : 'pendingConvertAudio';
                processInteraction['isTranscribed'] = 1;
                processInteraction['retry'] = 0;
                processInteraction['uploadedDate'] = moment().format('x');
                processInteraction['errorFunction'] = null;
                processInteraction['errorMessage'] = null;
                processInteraction['silenceInTime'] = Number(silenceInTime);
                processInteraction['silenceOutTime'] = Number(silenceOutTime);
                processInteraction['silenceTime'] = Number(silenceInTime) + Number(silenceOutTime);
                _this.errorData.CleanErrorData();
            }
            else {
                logger.error('[SemanticAnalysisDownloadImplementation::DownloadJSON] executionCode:', executionCode, 'getting queueId:', currentQueueId, 'getting error: JSON_IDATHA_IS_NOT_FOUND transcription content:', data, 'path:', path);
                processInteraction['status'] = -6;
                processInteraction['retry'] = processInteraction['retry'] ? processInteraction['retry'] + 1 : 1;
                processInteraction['uploadedDate'] = moment().format('x');
                processInteraction['errorFunction'] = 'DownloadJSON';
                processInteraction['errorMessage'] = 'JSON_IDATHA_IS_NOT_FOUND';
                _this.errorData.SetErrorData('DownloadJSON', processInteraction['errorMessage'], processInteraction['errorMessage']);
            }
            return Promise.resolve(processInteraction);
        })
            .catch(function (err) {
            var errorMessage = typeof err === 'string' ? err : (err.message || err.description);
            logger.error('[SemanticAnalysisDownloadImplementation::DownloadJSON] executionCode:', executionCode, 'getting queueId:', currentQueueId, 'getting error:', errorMessage);
            processInteraction['status'] = -6;
            processInteraction['retry'] = processInteraction['retry'] ? processInteraction['retry'] + 1 : 1;
            processInteraction['uploadedDate'] = moment().format('x');
            processInteraction['errorFunction'] = 'DownloadJSON';
            processInteraction['errorMessage'] = errorMessage;
            _this.errorData.SetErrorData('DownloadJSON', errorMessage, err);
            return Promise.resolve(processInteraction);
        });
    };
    SemanticAnalysisDownloadImplementation.prototype.GetJson = function (accessToken, currentQueueId, executionCode, fileName) {
        var _this = this;
        var idatha = new idatha_model_1.IdathaConnection();
        var url = "http://".concat(idatha.ip, ":").concat(idatha.httpPort, "/api/query/document/").concat(fileName);
        var options = {
            uri: url,
            retryProtocol: false,
            timeout: 180000,
            headers: {
                'Authorization': 'Bearer ' + accessToken
            }
        };
        var data = '';
        return (0, promises_shared_1.DummyPromise)()
            .then(function (_) {
            return (0, request_shared_1.SimpleGetRequest)(options);
        })
            .then(function (response) {
            try {
                var regex = /\bNaN\b/g;
                data = JSON.stringify(response);
                data = data.replace(regex, 'null');
                _this.errorData.CleanErrorData();
                return Promise.resolve(data);
            }
            catch (err) {
                logger.error('[SemanticAnalysisDownloadImplementation::IdathaDocumentWords] executionCode:', executionCode, 'getting queueId:', currentQueueId, err, 'options:', options);
                _this.errorData.SetErrorData('IdathaDocumentWords', 'ERROR', err);
                return Promise.resolve(null);
            }
        })
            .catch(function (err) {
            var errorMessage = typeof err === 'string' ? err : (err.message || err.description);
            logger.error('[SemanticAnalysisDownloadImplementation::IdathaDocumentWords] executionCode:', executionCode, 'getting queueId:', currentQueueId, 'getting error:', errorMessage, 'options:', options);
            _this.errorData.SetErrorData('IdathaDocumentWords', errorMessage, err);
            return Promise.resolve(null);
        });
    };
    return SemanticAnalysisDownloadImplementation;
}());
exports.SemanticAnalysisDownloadImplementation = SemanticAnalysisDownloadImplementation;
