"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.SemanticAnalysisUploadImplementation = void 0;
var config_1 = require("../config/config");
var mssql_service_1 = require("../db/mssql.service");
var redis_service_1 = require("../db/redis.service");
var promises_shared_1 = require("../shared/promises.shared");
var log4js = require("log4js");
var moment = require("moment");
var billing_batch_model_1 = require("../models/billing-batch.model");
var idatha_service_1 = require("../services/idatha.service");
var idatha_model_1 = require("../models/idatha.model");
var file_manager_1 = require("../shared/file-manager");
var SSH2Promise = require("ssh2-promise");
var arrays_shared_1 = require("../shared/arrays.shared");
var billing_implementation_1 = require("./billing.implementation");
var logger = log4js.getLogger('Task');
var CURRENT_SYSTEM_SERVICE = 'TRANSCRIPTION';
var CURRENT_IMPLEMENTATION = 'SemanticAnalysisUploadImplementation';
var SemanticAnalysisUploadImplementation = (function () {
    function SemanticAnalysisUploadImplementation() {
        this.idathaConnection = new idatha_model_1.IdathaConnection();
        this.sshConnection = null;
        this.sftpConnection = null;
        this.defaultFolders = [];
        this.errorData = new billing_batch_model_1.BatchError();
        this.queueBase = 'SemanticAnalysisUpload';
    }
    SemanticAnalysisUploadImplementation.prototype.Execute = function (executionCode, rateRecursive) {
        var _this = this;
        logger.info("[SemanticAnalysisUploadImplementation::Execute] executionCode: ".concat(executionCode, " executing wave #").concat(rateRecursive));
        var executeIndex = 1;
        var ratio = config_1.RATIO_SEMANTIC_ANALYSIS_UPLOAD;
        var executeResult = true;
        var currentBlockPromise = null;
        return (0, promises_shared_1.SleepPromise)(config_1.SLEEP_SEMANTIC_ANALYSIS)
            .then(function (_) {
            currentBlockPromise = 'BLOCK_01_GET_FROM_REDIS';
            return redis_service_1.Redis.GetLengthFromList(_this.queueBase);
        })
            .then(function (result) {
            currentBlockPromise = 'BLOCK_02_GET_QUEUE_ITEMS';
            if (!result)
                throw 'SERVER_INFO_REDIS_KEYS_FOR_SEMANTIC_ANALYSIS_UPLOAD_NOT_FOUND';
            return redis_service_1.Redis.LRange(_this.queueBase, 0, ratio - 1);
        })
            .then(function (result) {
            currentBlockPromise = 'BLOCK_03_VERIFY_EXTERNAL_CONNECTION';
            ratio = result && result.length <= ratio ? result.length : 0;
            var vccs = (0, arrays_shared_1.GetUniqueFromArray)(result.map(function (r) {
                var items = r.split(':');
                return items[0];
            }));
            return _this.VerifyConnection(vccs);
        })
            .then(function (_) {
            currentBlockPromise = 'BLOCK_04_SEMANTIC_ANALYSIS_UPLOAD_IMPL';
            var promiseArray = [];
            while (ratio >= executeIndex) {
                promiseArray.push(_this.SemanticAnalysisUploadExecute(executionCode));
                executeIndex++;
            }
            if (!promiseArray.length)
                promiseArray.push(Promise.resolve(true));
            return Promise.all(promiseArray);
        })
            .then(function (results) {
            currentBlockPromise = 'BLOCK_05_GETTING_KEYS_FOR_RECURSIVITY';
            if (results.filter(function (item) { return item === false; }).length > 0) {
                executeResult = false;
            }
            return redis_service_1.Redis.LRange(_this.queueBase, 0, ratio - 1);
        })
            .then(function (result) {
            currentBlockPromise = 'BLOCK_06_RELAUNCH_SEMANTIC_ANALYSIS_UPLOAD_IMP';
            if ((result === null || result === void 0 ? void 0 : result.length) && rateRecursive >= 1)
                return _this.Execute(executionCode, rateRecursive - 1);
            return Promise.resolve(executeResult);
        })
            .then(function (result) {
            currentBlockPromise = 'BLOCK_07_RESOLVING_IMPL';
            if (rateRecursive === config_1.RATE_RECURSIVE_SEMANTIC_ANALYSIS_UPLOAD)
                logger.info("[SemanticAnalysisUploadImplementation::Execute] executionCode: ".concat(executionCode, " Finishing execution wave #").concat(rateRecursive));
            if (_this.sshConnection)
                _this.sshConnection.close();
            var currentResult = executeResult && result ? true : false;
            return Promise.resolve(currentResult);
        })
            .catch(function (error) {
            var errorMessage = typeof error === 'string' ? error : (error.message || error.description);
            if (errorMessage.startsWith('SERVER_INFO_'))
                logger.info('[SemanticAnalysisUploadImplementation::Execute] executionCode:', executionCode, ', Block:', currentBlockPromise, ', getting info:', errorMessage);
            else
                logger.error('[SemanticAnalysisUploadImplementation::Execute] executionCode:', executionCode, ', Block:', currentBlockPromise, ', getting error:', errorMessage);
            if (_this.sshConnection)
                _this.sshConnection.close();
            return Promise.resolve(false);
        });
    };
    SemanticAnalysisUploadImplementation.prototype.SemanticAnalysisUploadExecute = function (executionCode) {
        var _this = this;
        var storage = '';
        var currentQueueId = '';
        var processInteraction = null;
        var vcc = '';
        var processId = '';
        var interactionType = '';
        var interactionId = '';
        var interactionSection = 0;
        var interactionPart = 0;
        return redis_service_1.Redis.RPopLPush(this.queueBase, "".concat(this.queueBase, "Execute"))
            .then(function (queueId) {
            if ((queueId === null || queueId === void 0 ? void 0 : queueId.length) > 0) {
                currentQueueId = queueId;
                logger.info('[SemanticAnalysisUploadImplementation::SemanticAnalysisUploadExecute] executionCode:', executionCode, 'getting queueId:', currentQueueId, 'START');
                var item = queueId.split(':');
                if (item === null || item === void 0 ? void 0 : item.length) {
                    vcc = item[0];
                    processId = item[1];
                    interactionType = item[2];
                    interactionId = item[3];
                    interactionSection = Number(item[4]);
                    interactionPart = Number(item[5]);
                    storage = vcc + ':' + processId;
                    logger.info('[SemanticAnalysisUploadImplementation::SemanticAnalysisUploadExecute] executionCode:', executionCode, 'getting queueId:', currentQueueId, 'storage :', storage);
                    return redis_service_1.Redis.GetProcessInteraction(storage, currentQueueId);
                }
                else {
                    logger.error('[SemanticAnalysisUploadImplementation::SemanticAnalysisUploadExecute] executionCode:', executionCode, 'getting queueId:', currentQueueId, 'QUEUE_FORMAT_ERROR');
                }
            }
            return Promise.resolve(null);
        })
            .then(function (processInteractionResult) {
            if (processInteractionResult) {
                processInteraction = processInteractionResult;
                logger.info('[SemanticAnalysisUploadImplementation::SemanticAnalysisUploadExecute] executionCode:', executionCode, 'getting queueId:', currentQueueId, 'getting GetProcessInteraction: 1, status:', processInteraction.statusDescription);
                if (processInteraction.statusDescription === 'pendingAnalysis') {
                    return _this.SendInteraction(processInteraction, currentQueueId, executionCode);
                }
            }
            if (!processInteraction && currentQueueId) {
                logger.warn('[SemanticAnalysisUploadImplementation::SemanticAnalysisUploadExecute] executionCode:', executionCode, 'getting queueId:', currentQueueId, 'getting GetProcessInteraction: 0');
                if (!processInteraction) {
                    redis_service_1.Redis.DeleteQueue("".concat(_this.queueBase, "Execute"), currentQueueId);
                    throw 'SERVER_ERROR_INTERACTION_DATA_NOT_FOUND_ON_STORAGE';
                }
            }
            return Promise.resolve(null);
        })
            .then(function (result) {
            if (result)
                processInteraction = result;
            if (processInteraction)
                return redis_service_1.Redis.SaveProcessInteraction(storage, currentQueueId, processInteraction);
            return Promise.resolve(null);
        })
            .then(function (result) {
            var promiseArray = [];
            var billing = new billing_implementation_1.BillingImplementation();
            if (result
                &&
                    (processInteraction === null || processInteraction === void 0 ? void 0 : processInteraction.status) === 6
                &&
                    (processInteraction === null || processInteraction === void 0 ? void 0 : processInteraction.statusDescription) === 'pendingDownloadJSON'
                &&
                    (processInteraction === null || processInteraction === void 0 ? void 0 : processInteraction.retry) < config_1.RETRY_SEMANTIC_ANALYSIS) {
                promiseArray.push(redis_service_1.Redis.InsertQueue('SemanticAnalysisDownload', currentQueueId, config_1.RATE_RECURSIVE_SEMANTIC_ANALYSIS * config_1.RATIO_SEMANTIC_ANALYSIS * 2));
                promiseArray.push(redis_service_1.Redis.DeleteQueue("".concat(_this.queueBase, "Execute"), currentQueueId));
            }
            else if (result
                &&
                    (processInteraction === null || processInteraction === void 0 ? void 0 : processInteraction.status) === -6
                &&
                    ((processInteraction === null || processInteraction === void 0 ? void 0 : processInteraction.statusDescription) === 'pendingRemove'
                        ||
                            (processInteraction === null || processInteraction === void 0 ? void 0 : processInteraction.retry) >= config_1.RETRY_SEMANTIC_ANALYSIS)) {
                var resultMessage = (processInteraction === null || processInteraction === void 0 ? void 0 : processInteraction.retry) >= config_1.RETRY_SEMANTIC_ANALYSIS ? 'REMOVED_BY_RETRIES' : 'REMOVED_BY_INVALID_CONTENT';
                logger.error('[SemanticAnalysisUploadImplementation::SemanticAnalysisUploadExecute] executionCode:', executionCode, 'getting queueId:', currentQueueId, resultMessage);
                billing.AddCloseTranscriptionDetailToMainFile(processInteraction, processInteraction.batchTypeName, processInteraction.batchTypeId, 0, resultMessage);
                promiseArray.push(redis_service_1.Redis.DeleteQueue("".concat(_this.queueBase, "Execute"), currentQueueId));
            }
            else if (currentQueueId && !processInteraction) {
                logger.error('[SemanticAnalysisUploadImplementation::SemanticAnalysisUploadExecute] executionCode:', executionCode, 'getting queueId:', currentQueueId, 'INTERACTION_DO_NOT_EXIST');
                var interactionData = {
                    vcc: vcc,
                    processId: processId,
                    id: interactionId,
                    section: interactionSection,
                    part: interactionPart
                };
                billing.AddCloseTranscriptionDetailToMainFile(interactionData, processInteraction.batchTypeName, processInteraction.batchTypeId, 0, 'INTERACTION_DO_NOT_EXIST');
                promiseArray.push(redis_service_1.Redis.DeleteQueue("".concat(_this.queueBase, "Execute"), currentQueueId));
            }
            else {
                logger.warn('[SemanticAnalysisUploadImplementation::SemanticAnalysisUploadExecute] executionCode:', executionCode, 'getting queueId:', currentQueueId, 'QUEUE_WAS_NOT_REMOVED');
            }
            if (!promiseArray.length)
                promiseArray.push(Promise.resolve(true));
            return Promise.all(promiseArray);
        })
            .then(function (_) {
            var promiseArray = [];
            if (processInteraction
                &&
                    (processInteraction.retry >= config_1.RETRY_SEMANTIC_ANALYSIS
                        ||
                            [
                                'JSON_TRANSCRIPTION_CONTENT_IS_EMPTY',
                                'JSON_TRANSCRIPTION_CONTENT_HAS_NOT_WORD_TYPE_DATA'
                            ].includes(processInteraction.errorMessage))) {
                processInteraction.isCancel = true;
                promiseArray.push(redis_service_1.Redis.UpdateProgress(storage));
                promiseArray.push(redis_service_1.Redis.UpdateErrorProgress(storage));
                promiseArray.push(redis_service_1.Redis.DeleteProcessInteraction(storage, currentQueueId));
                promiseArray.push(redis_service_1.Redis.SaveReadyProcessInteraction(executionCode, currentQueueId, processInteraction));
                promiseArray.push(mssql_service_1.MSSql.SaveBatchError(vcc, CURRENT_SYSTEM_SERVICE, processInteraction.batchTypeId, processInteraction.batchTypeId, CURRENT_IMPLEMENTATION, processInteraction.id, null, processInteraction.errorFunction, processInteraction.errorMessage, processInteraction.errorDetail));
            }
            if (!promiseArray.length)
                promiseArray.push(Promise.resolve(true));
            return Promise.all(promiseArray);
        })
            .then(function (_) {
            return Promise.resolve(true);
        })
            .catch(function (err) {
            var errorMessage = typeof err === 'string' ? err : (err.message || err.description);
            logger.error('[SemanticAnalysisUploadImplementation::SemanticAnalysisUploadExecute] executionCode:', executionCode, 'getting queueId:', currentQueueId, 'Catching error: ' + errorMessage);
            if (processInteraction)
                logger.error('[SemanticAnalysisUploadImplementation::SemanticAnalysisUploadExecute] executionCode:', executionCode, 'getting queueId:', currentQueueId, 'getting error:', errorMessage, ' vcc', processInteraction.vcc, 'processId:', processInteraction.processId, 'id:', processInteraction.id, 'section:', processInteraction.section, 'part:', processInteraction.part);
            return Promise.resolve(false);
        });
    };
    SemanticAnalysisUploadImplementation.prototype.SendInteraction = function (interaction, currentQueueId, executionCode) {
        var _this = this;
        var path = config_1.TRANSCRIPTION_FOLDER;
        var jsonContent = null;
        return (0, promises_shared_1.DummyPromise)()
            .then(function (_) {
            return _this.UpdateJsonMetadata(interaction, currentQueueId, executionCode);
        })
            .then(function (result) {
            var _a;
            if (!result)
                throw 'SERVER_ERROR_METADATA_UPDATE_FAILED';
            jsonContent = result;
            if ((_a = jsonContent === null || jsonContent === void 0 ? void 0 : jsonContent.results) === null || _a === void 0 ? void 0 : _a.length) {
                var wordResults = jsonContent.results.filter(function (r) { return r.type === 'word'; }).length;
                if (wordResults)
                    return _this.SendJsonFile(interaction, currentQueueId, executionCode);
                else
                    return Promise.resolve('JSON_TRANSCRIPTION_CONTENT_HAS_NOT_WORD_TYPE_DATA');
            }
            else {
                return Promise.resolve('JSON_TRANSCRIPTION_CONTENT_IS_EMPTY');
            }
        })
            .then(function (result) {
            if (typeof result === 'boolean' && result === true) {
                interaction['status'] = 6;
                interaction['statusDescription'] = 'pendingDownloadJSON';
                interaction['retry'] = 0;
                interaction['uploadedDate'] = moment().format('x');
                interaction['errorMessage'] = null;
                _this.errorData.CleanErrorData();
            }
            else {
                logger.error('[SemanticAnalysisUploadImplementation::UploadJSON] executionCode:', executionCode, 'getting queueId:', currentQueueId, 'JSON_TRANSCRIPTION_WAS_NOT_TRANSFERRED');
                interaction['status'] = -6;
                interaction['retry'] = interaction['retry'] ? interaction['retry'] + 1 : 1;
                if (typeof result === 'string'
                    && [
                        'JSON_TRANSCRIPTION_CONTENT_IS_EMPTY',
                        'JSON_TRANSCRIPTION_CONTENT_HAS_NOT_WORD_TYPE_DATA'
                    ].includes(result)) {
                    logger.error('[SemanticAnalysisUploadImplementation::UploadJSON] executionCode:', executionCode, 'getting queueId:', currentQueueId, result);
                    interaction['statusDescription'] = 'pendingRemove';
                    interaction['errorMessage'] = result;
                }
                else {
                    interaction['errorMessage'] = 'JSON_TRANSCRIPTION_WAS_NOT_TRANSFERRED';
                }
                _this.errorData.SetErrorData('UploadJSON', interaction['errorMessage'], interaction['errorMessage']);
            }
            logger.info('[SemanticAnalysisUploadImplementation::UploadJSON] executionCode:', executionCode, 'getting queueId:', currentQueueId, "Resolving for ".concat(path));
            return Promise.resolve(interaction);
        })
            .catch(function (err) {
            var errorMessage = typeof err === 'string' ? err : (err.message || err.description);
            logger.error('[SemanticAnalysisUploadImplementation::UploadJSON] executionCode:', executionCode, 'getting queueId:', currentQueueId, errorMessage);
            interaction['status'] = -6;
            interaction['retry'] = interaction['retry'] ? interaction['retry'] + 1 : 1;
            interaction['errorMessage'] = errorMessage;
            _this.errorData.SetErrorData('UploadJSON', errorMessage, err);
            return Promise.resolve(interaction);
        });
    };
    SemanticAnalysisUploadImplementation.prototype.UpdateJsonMetadata = function (interaction, currentQueueId, executionCode) {
        var _this = this;
        var path = config_1.TRANSCRIPTION_FOLDER;
        var jsonContent = null;
        return (0, promises_shared_1.DummyPromise)()
            .then(function (_) {
            return (0, idatha_service_1.IdathaLogin)(interaction.vcc);
        })
            .then(function (result) {
            if (!result || !result.accessToken)
                throw 'SERVER_ERROR_CANNOT_GET_USER_ACCESS_TOKEN';
            var regex = /\.(mp3|wav|ogg)$/;
            var jsonFileName = interaction.fileName;
            if (!jsonFileName) {
                jsonFileName = "".concat(interaction.id, "_").concat(interaction.section, "_").concat(interaction.part, ".").concat(interaction.format || 'wav');
                interaction.fileName = jsonFileName;
            }
            jsonFileName = interaction.fileName.replace(regex, '.json');
            path += "/".concat(interaction.vcc, "/").concat(jsonFileName);
            logger.info('[SemanticAnalysisUploadImplementation::UpdateJsonMetadata] executionCode:', executionCode, 'getting queueId:', currentQueueId, "Reading file ".concat(path));
            return (0, file_manager_1.ReadFileContent)(path, 'utf-8');
        })
            .then(function (result) {
            try {
                jsonContent = typeof result === 'string' ? JSON.parse(result) : result;
                jsonContent.filterdata = {
                    start_date: new Date(interaction.startDate),
                    end_date: new Date(interaction.endDate),
                    channel: interaction.interactionType,
                    campaign: interaction.campaign,
                    disposition: interaction.dispositionCode,
                    agent: interaction.agent,
                    origin: interaction.origin
                };
            }
            catch (ex) {
                jsonContent = '';
                logger.error('[SemanticAnalysisUploadImplementation::UpdateJsonMetadata] executionCode:', executionCode, 'getting queueId:', currentQueueId, "error on parsing json:", ex);
                throw 'ERROR_FILE_JSON';
            }
            logger.info('[SemanticAnalysisUploadImplementation::UpdateJsonMetadata] executionCode:', executionCode, 'getting queueId:', currentQueueId, "Writing file ".concat(path));
            return (0, file_manager_1.WriteFileContent)(path, JSON.stringify(jsonContent), 'utf-8', true);
        })
            .then(function (_) {
            return Promise.resolve(jsonContent);
        })
            .catch(function (err) {
            var errorMessage = typeof err === 'string' ? err : (err.message || err.description);
            logger.error('[SemanticAnalysisUploadImplementation::UpdateJsonMetadata] executionCode:', executionCode, 'getting queueId:', currentQueueId, errorMessage);
            interaction['status'] = -6;
            interaction['retry'] = interaction['retry'] ? interaction['retry'] + 1 : 1;
            interaction['errorMessage'] = errorMessage;
            _this.errorData.SetErrorData('UpdateJsonMetadata', errorMessage, err);
            return Promise.resolve(null);
        });
    };
    SemanticAnalysisUploadImplementation.prototype.SendJsonFile = function (processInteraction, currentQueueId, executionCode) {
        var _this = this;
        var regex = /\.(mp3|wav|ogg)$/;
        var src = "".concat(config_1.TRANSCRIPTION_FOLDER, "/").concat(processInteraction.vcc.toLowerCase(), "/").concat(processInteraction.fileName.replace(regex, '.json'));
        var target = "/home/".concat(this.idathaConnection.user, "/in_concert_speech_analytics/input/").concat(processInteraction.vcc.toLowerCase(), "/default/");
        var destination = "".concat(target).concat(processInteraction.fileName.replace(regex, '.json'));
        return (0, promises_shared_1.DummyPromise)()
            .then(function (_) {
            if (_this.defaultFolders.includes(processInteraction.vcc))
                return _this.UploadJson(src, destination);
            return Promise.resolve(false);
        })
            .then(function (result) {
            if (result)
                _this.errorData.CleanErrorData();
            return Promise.resolve(result);
        })
            .catch(function (err) {
            var errorMessage = typeof err === 'string' ? err : (err.message || err.description);
            logger.error('[SemanticAnalysisImplementation::IdathaSendJSONFiles] executionCode:', executionCode, 'getting queueId:', currentQueueId, 'getting error:', errorMessage);
            _this.errorData.SetErrorData('IdathaSendJSONFiles', errorMessage, err);
            return Promise.resolve(false);
        });
    };
    SemanticAnalysisUploadImplementation.prototype.UploadJson = function (src, destination) {
        var _this = this;
        var _a;
        (_a = this.sftpConnection) !== null && _a !== void 0 ? _a : (this.sftpConnection = this.sshConnection.sftp());
        logger.info("[SemanticAnalysisUploadImplementation::UploadJson] Uploading from: \"".concat(src, "\""));
        return (0, promises_shared_1.DummyPromise)()
            .then(function (_) { return _this.sftpConnection.fastPut(src, destination); })
            .then(function (_) {
            logger.info("[SemanticAnalysisUploadImplementation::UploadJson] Uploaded corretly: \"".concat(src, "\""));
            return Promise.resolve(true);
        })
            .catch(function (err) {
            logger.error('[SemanticAnalysisUploadImplementation::UploadJson]', err);
            return Promise.reject(err);
        });
    };
    SemanticAnalysisUploadImplementation.prototype.VerifyConnection = function (vccs) {
        var _this = this;
        return (0, promises_shared_1.DummyPromise)()
            .then(function (_) {
            return _this.ConnectToSsh();
        })
            .then(function (result) {
            if (!result)
                throw 'SERVER_ERROR_CANNOT_CONNECT_TO_IDATHA_SSH';
            var promiseArray = [];
            vccs.forEach(function (vcc) {
                promiseArray.push(_this.CreateDefaultDirIfNotExists(vcc));
            });
            return Promise.all(promiseArray);
        })
            .then(function (results) {
            if (results.includes(false))
                logger.error('[SemanticAnalysisUploadImplementation::VerifyConnection] Error creating a default folder');
            return Promise.resolve(true);
        })
            .catch(function (error) {
            logger.error('[SemanticAnalysisUploadImplementation::VerifyConnection] Getting error:', error);
            return Promise.resolve(false);
        });
    };
    SemanticAnalysisUploadImplementation.prototype.ConnectToSsh = function () {
        var _this = this;
        var sshConfig = {
            host: this.idathaConnection.ip,
            port: this.idathaConnection.sshPort,
            username: this.idathaConnection.user,
            password: this.idathaConnection.password,
            connTimeout: 30000,
            pasvTimeout: 30000
        };
        return (0, promises_shared_1.DummyPromise)()
            .then(function (_) {
            if (!_this.sshConnection) {
                var ssh2 = new SSH2Promise(sshConfig);
                _this.sshConnection = ssh2;
            }
            return Promise.resolve(true);
        })
            .catch(function (error) {
            logger.error('[SemanticAnalysisUploadImplementation::ConnectToSsh] Getting error:', error);
            return Promise.resolve(false);
        });
    };
    SemanticAnalysisUploadImplementation.prototype.CreateDefaultDirIfNotExists = function (vcc) {
        var _this = this;
        var path = "/home/".concat(this.idathaConnection.user, "/in_concert_speech_analytics/input/").concat(vcc.toLowerCase(), "/default/");
        return (0, promises_shared_1.DummyPromise)()
            .then(function (_) {
            if (!_this.defaultFolders.includes(vcc))
                return _this.sshConnection.exec("mkdir -p ".concat(path));
            return Promise.resolve(true);
        })
            .then(function (_) {
            _this.defaultFolders.push(vcc);
            return Promise.resolve(true);
        })
            .catch(function (err) {
            logger.error('[SemanticAnalysisUploadImplementation::CreateDefaultDirIfNotExists]', err);
            return Promise.resolve(false);
        });
    };
    return SemanticAnalysisUploadImplementation;
}());
exports.SemanticAnalysisUploadImplementation = SemanticAnalysisUploadImplementation;
