"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.GetQueryIdathaProcessV2 = exports.SemanticQueryImplementation = void 0;
var log4js = require("log4js");
var idatha_model_1 = require("../models/idatha.model");
var moment = require("moment");
var config_1 = require("../config/config");
var mssql_service_1 = require("../db/mssql.service");
var redis_service_1 = require("../db/redis.service");
var promises_shared_1 = require("../shared/promises.shared");
var config_2 = require("../config/config");
var file_manager_1 = require("../shared/file-manager");
var billing_implementation_1 = require("../implementation/billing.implementation");
var idatha_service_1 = require("../services/idatha.service");
var request_shared_1 = require("../shared/request.shared");
var billing_batch_model_1 = require("../models/billing-batch.model");
var logger = log4js.getLogger('Task');
var CURRENT_SYSTEM_SERVICE = 'SEMANTIC_ANALYSIS';
var CURRENT_IMPLEMENTATION = 'SemanticQueryImplementation';
var SemanticQueryImplementation = (function () {
    function SemanticQueryImplementation() {
        this.errorData = new billing_batch_model_1.BatchError();
    }
    SemanticQueryImplementation.prototype.Execute = function (executionCode, rateRecursive, availableBatch, cancelledBatch) {
        var _this = this;
        var executeIndex = 1;
        var ratio = config_1.RATIO_SEMANTIC_QUERY;
        var alowedRatio = 0;
        var notAlowedRatio = 0;
        var executeResult = true;
        return (0, promises_shared_1.DummyPromise)()
            .then(function (result) {
            return redis_service_1.Redis.LRange('SemanticQuery', 0, ratio - 1);
        })
            .then(function (result) {
            var promiseArray = [];
            ratio = result && result.length <= ratio ? result.length : 0;
            result.map(function (r) {
                var arrQueue = r.split(':');
                if (availableBatch.find(function (b) { return b.VCC === arrQueue[0] && b.BatchTypeId.toLowerCase() === arrQueue[4].toLowerCase(); })) {
                    alowedRatio++;
                }
                else {
                    notAlowedRatio++;
                }
            });
            while (ratio >= executeIndex) {
                promiseArray.push(_this.SemanticQueryExecute(executionCode, availableBatch, cancelledBatch));
                executeIndex++;
            }
            return Promise.all(promiseArray);
        }).then(function (results) {
            if (results.filter(function (item) { return item === false; }).length > notAlowedRatio) {
                executeResult = false;
            }
            return redis_service_1.Redis.LRange('SemanticQuery', 0, ratio - 1);
        }).then(function (result) {
            if (result && result.length && rateRecursive >= 1)
                return _this.Execute(executionCode, rateRecursive - 1, availableBatch, cancelledBatch);
            return executeResult;
        }).then(function (result) {
            var currentResult = executeResult && result ? true : false;
            return Promise.resolve(currentResult);
        }).catch(function (err) {
            var errorMessage = typeof err === 'string' ? err : (err.message || err.description);
            logger.error('[Execute] getting error:', errorMessage);
            return Promise.resolve(false);
        });
    };
    SemanticQueryImplementation.prototype.SemanticQueryExecute = function (executionCode, availableBatch, cancelledBatch) {
        var _this = this;
        var currentQueueId = '';
        var query = {};
        var process = {};
        var queryId = '';
        var genericQueryId = '';
        var type = '';
        var typeId = '';
        var templateId = '';
        var currentUserAccessToken = {};
        var isError = false;
        var isComplete = false;
        var isCancel = false;
        var queueProgressStorage = '';
        var retry;
        var queryFilePath = '';
        return redis_service_1.Redis.RemoveQueueL('SemanticQuery')
            .then(function (result) {
            logger.info('[SemanticQueryImplementation::SemanticQueryExecute] executionCode:', executionCode, 'getting queueId:', currentQueueId, 'Creating queues');
            logger.info('[SemanticQueryImplementation::SemanticQueryExecute] executionCode:', executionCode, 'getting queueId:', currentQueueId, 'Current queue: ', result);
            var arrQueue = result.split(':');
            var promiseArray = [];
            if (availableBatch.find(function (b) { return b.VCC === arrQueue[0] && b.BatchTypeId.toLowerCase() === arrQueue[4].toLowerCase(); })) {
                promiseArray.push(Promise.resolve('ALLOWED'));
                promiseArray.push(redis_service_1.Redis.CreateQueueL('SemanticQueryExecute', result));
                promiseArray.push(Promise.resolve(result));
            }
            else {
                logger.warn('[SemanticQueryImplementation::SemanticQueryExecute] executionCode:', executionCode, 'getting queueId:', currentQueueId, 'SEMANTIC_QUERY_ITEM_NOT_ALLOWED');
                logger.info('[SemanticQueryImplementation::SemanticQueryExecute] executionCode:', executionCode, 'getting queueId:', currentQueueId, 'Current item was not found on allowed Batches: ', availableBatch);
                promiseArray.push(Promise.resolve('NOT_ALLOWED'));
                promiseArray.push(redis_service_1.Redis.CreateQueueR('SemanticQuery', result));
                promiseArray.push(mssql_service_1.MSSql.SaveBatchStatus(arrQueue[0], arrQueue[4], 'SEMANTIC_ANALYSIS', 'PAUSED'));
            }
            return Promise.all(promiseArray);
        })
            .then(function (result) {
            if (result && result[0] === 'NOT_ALLOWED')
                throw 'SERVER_ERROR_NOT_ALLOWED_QUEUE';
            var queueId = result[2];
            if (queueId && queueId.length > 0) {
                currentQueueId = queueId;
                logger.info('[SemanticQueryImplementation::SemanticQueryExecute] executionCode:', executionCode, 'getting queueId:', currentQueueId, 'START');
                var item_1 = queueId.split(':');
                if (item_1 && item_1.length) {
                    if (cancelledBatch.find(function (b) { return b.VCC === item_1[1] && b.BatchTypeId.toLowerCase() === item_1[4].toLowerCase(); })) {
                        logger.info('[SemanticQueryImplementation::SemanticQueryExecute] executionCode:', executionCode, 'getting queueId:', currentQueueId, ' REMOVING CANCELLED BATCH');
                        throw 'REMOVED_BY_CANCELLED';
                    }
                    genericQueryId = item_1[2];
                    type = item_1[3] ? item_1[3] : null;
                    typeId = item_1[4] ? item_1[4] : null;
                    if (type && typeId)
                        queueProgressStorage = item_1[0] + ':' + item_1[1] + ':' + item_1[3] + ':' + item_1[4];
                    if (type && !typeId)
                        queueProgressStorage = item_1[0] + ':' + item_1[1] + ':' + item_1[3];
                    logger.info('[SemanticQueryImplementation::SemanticQueryExecute] executionCode:', executionCode, 'getting queueId:', currentQueueId, 'storage :', queueProgressStorage);
                    return mssql_service_1.MSSql.GetProcessTask(item_1[0], item_1[1]);
                }
                else {
                    logger.error('[SemanticQueryImplementation::SemanticQueryExecute] executionCode:', executionCode, 'getting queueId:', currentQueueId, 'QUEUE_FORMAT_ERROR');
                }
            }
            return Promise.resolve(null);
        }).then(function (result) {
            process = result;
            if (!process)
                throw 'PROCESS_NOT_FOUND';
            return redis_service_1.Redis.GetRetry(currentQueueId);
        }).then(function (result) {
            retry = result && result.value ? result.value : 0;
            logger.info('[SemanticQueryImplementation::SemanticQueryExecute] executionCode:', executionCode, 'getting queueId:', currentQueueId, 'Current retry :', retry);
            return mssql_service_1.MSSql.GetGenericQuery(process.vcc, genericQueryId);
        }).then(function (result) {
            query = result;
            if (!query || (query && !query.query))
                throw 'QUERY_IS_NOT_FOUND';
            var key = process.vcc + ':' + process.vcc;
            return (0, idatha_service_1.IdathaLogin)(process.vcc);
        }).then(function (userAccessToken) {
            if (!userAccessToken)
                throw 'NOT_FOUND_ACCESS_TOKEN';
            currentUserAccessToken = userAccessToken;
            return _this.GetIdathaQueryId(currentUserAccessToken.accessToken, process, executionCode, currentQueueId);
        }).then(function (resultQueryId) {
            queryId = resultQueryId;
            if (!queryId)
                throw 'NOT_FOUND_QUERY_ID';
            if (['study', 'template'].includes(type) && typeId)
                return mssql_service_1.MSSql.GetStudy(process.vcc, typeId);
            return Promise.resolve(null);
        }).then(function (result) {
            if (result && ['study', 'template'].includes(type))
                templateId = result.templateId;
            queryFilePath = config_2.QUERY_FOLDER + '/' + currentQueueId.replace(/:/g, '=') + '.json';
            return (0, file_manager_1.DeleteFile)(queryFilePath, true);
        }).then(function (result) {
            return _this.GetIdathaQuery(currentUserAccessToken.accessToken, process.vcc, queryId, query.name, query.query, 0, 25, executionCode, currentQueueId, query.version, process);
        }).then(function (result) {
            if (result === null)
                throw 'IDATHA_REQUEST_FAILED';
            if (typeof result === 'boolean' && result === false)
                throw 'NOT_FOUND_DATA';
            return _this.GetDocumentQuery(executionCode, currentQueueId);
        }).then(function (results) {
            if (['study', 'template'].includes(type) && typeId) {
                logger.info('[SemanticQueryImplementation::SemanticQueryExecute] executionCode:', executionCode, 'getting queueId:', currentQueueId, 'BULK_STUDY');
                return mssql_service_1.MSSql.BulkInteractionQueryStudyTemp(process.vcc, process.name, typeId, templateId, genericQueryId, results[query.name]);
            }
            else if (type === 'evaluationPlan' && typeId) {
                logger.info('[SemanticQueryImplementation::SemanticQueryExecute] executionCode:', executionCode, 'getting queueId:', currentQueueId, 'BULK_EVALUATION_PLAN_CONTENT_SEARCH');
                return mssql_service_1.MSSql.BulkEvaluationPlanQueryTemp(process.vcc, process.name, typeId, genericQueryId, results[query.name]);
            }
            else if (type === 'ability' && typeId) {
                logger.info('[SemanticQueryImplementation::SemanticQueryExecute] executionCode:', executionCode, 'getting queueId:', currentQueueId, 'BULK_ABILITY');
                return mssql_service_1.MSSql.BulkEvaluationPlanAbilityTemp(process.vcc, process.name, typeId, genericQueryId, results[query.name]);
            }
            else if (process && type === 'search') {
                logger.info('[SemanticQueryImplementation::SemanticQueryExecute] executionCode:', executionCode, 'getting queueId:', currentQueueId, 'BULK_AUDIO');
                return mssql_service_1.MSSql.BulkInteractionQueryTemp(process.vcc, process.name, genericQueryId, results[query.name]);
            }
            else if (results && results[query.name]) {
                logger.info('[SemanticQueryImplementation::SemanticQueryExecute] executionCode:', executionCode, 'getting queueId:', currentQueueId, 'BULK_NONE');
                return Promise.resolve(true);
            }
            return Promise.resolve(null);
        }).then(function (result) {
            isComplete = result ? true : false;
            if (!isComplete) {
                retry++;
                return redis_service_1.Redis.UpdateRetry(currentQueueId);
            }
            return Promise.resolve(isComplete);
        }).catch(function (err) {
            var errorMessage = typeof err === 'string' ? err : (err.message || err.description);
            logger.error('[SemanticQueryImplementation::SemanticQueryExecute] executionCode:', executionCode, 'getting queueId:', currentQueueId, 'getting error:', errorMessage);
            if (!_this.errorData.errorFunction)
                _this.errorData.errorFunction = 'SemanticQueryExecute';
            if (!_this.errorData.errorMessage)
                _this.errorData.errorMessage = errorMessage ? errorMessage : 'ERROR';
            if (!_this.errorData.errorDetail)
                _this.errorData.errorDetail = err;
            if (errorMessage === 'PROCESS_NOT_FOUND'
                || errorMessage === 'QUERY_IS_NOT_FOUND'
                || errorMessage === 'NOT_FOUND_ACCESS_TOKEN'
                || errorMessage === 'NOT_FOUND_QUERY_ID'
                || errorMessage === 'PROCESS_FILTER_NOT_FOUND'
                || errorMessage === 'REMOVED_BY_CANCELLED') {
                isError = true;
            }
            else {
                isComplete = false;
                retry++;
                return redis_service_1.Redis.UpdateRetry(currentQueueId);
            }
            return Promise.resolve(false);
        }).then(function (result) {
            if (isComplete) {
                logger.info('[SemanticQueryImplementation::SemanticQueryExecute] executionCode:', executionCode, 'getting queueId:', currentQueueId, 'QUEUE_FINISHED');
                _this.CloseSemanticQuery(currentQueueId, true, 'SUCCED');
                return redis_service_1.Redis.DeleteQueue('SemanticQueryExecute', currentQueueId);
            }
            else if (isError) {
                logger.error('[SemanticQueryImplementation::SemanticQueryExecute] executionCode:', executionCode, 'getting queueId:', currentQueueId, 'REMOVED_BY_ERRORS_QUEUE_FINISHED');
                _this.CloseSemanticQuery(currentQueueId, false, 'REMOVED_BY_ERRORS_QUEUE_FINISHED');
                return redis_service_1.Redis.DeleteQueue('SemanticQueryExecute', currentQueueId);
            }
            else if (config_1.RETRY_SEMANTIC_QUERY <= retry) {
                logger.error('[SemanticQueryImplementation::SemanticQueryExecute] executionCode:', executionCode, 'getting queueId:', currentQueueId, 'REMOVED_BY_RETRY');
                _this.CloseSemanticQuery(currentQueueId, false, 'REMOVED_BY_RETRY');
                return redis_service_1.Redis.DeleteQueue('SemanticQueryExecute', currentQueueId);
            }
            else {
                logger.error('[SemanticQueryImplementation::SemanticQueryExecute] executionCode:', executionCode, 'getting queueId:', currentQueueId, 'QUEUE_WAS_NOT_REMOVED');
            }
            return Promise.resolve(null);
        }).then(function (result) {
            var promiseArray = [];
            if (isComplete) {
                promiseArray.push(redis_service_1.Redis.UpdateProgress(queueProgressStorage));
                promiseArray.push(redis_service_1.Redis.DeleteRetry(currentQueueId));
            }
            else if (isError
                ||
                    config_1.RETRY_SEMANTIC_QUERY <= retry) {
                promiseArray.push(redis_service_1.Redis.UpdateProgress(queueProgressStorage));
                promiseArray.push(redis_service_1.Redis.UpdateErrorProgress(queueProgressStorage));
                promiseArray.push(redis_service_1.Redis.DeleteRetry(currentQueueId));
                promiseArray.push(mssql_service_1.MSSql.SaveBatchError(process.vcc, CURRENT_SYSTEM_SERVICE, typeId, ['study', 'template'].includes(type) ? typeId : null, CURRENT_IMPLEMENTATION, null, query.name, _this.errorData.errorFunction, _this.errorData.errorMessage, _this.errorData.errorDetail));
            }
            return Promise.all(promiseArray);
        }).catch(function (err) {
            var errorMessage = typeof err === 'string' ? err : (err.message || err.description);
            logger.error('[SemanticQueryImplementation::SemanticQueryExecute] executionCode:', executionCode, 'getting queueId:', currentQueueId, 'getting error:', errorMessage);
            return Promise.resolve(false);
        });
    };
    SemanticQueryImplementation.prototype.GetIdathaQueryId = function (accessToken, process, executionCode, currentQueueId) {
        var _this = this;
        var queryId = '';
        return (0, promises_shared_1.DummyPromise)()
            .then(function (result) {
            var currentDate = new Date();
            var queryDataIdDate = new Date(process.queryDataIdDate);
            var diffDays = Math.round(Math.abs((currentDate.getTime() - queryDataIdDate.getTime()) / (24 * 60 * 60 * 1000)));
            if (!process.queryDataId || diffDays >= config_1.IDATHA_QUERY_ID_EXPIRATION) {
                if (!process.processFilters)
                    throw 'PROCESS_FILTER_NOT_FOUND';
                var queryProcess = GetQueryIdathaProcessV2(process.processFilters);
                logger.debug('[SemanticQueryImplementation::GetIdathaQueryId] Result from GetQueryIdathaProcessV2:', queryProcess);
                var options = _this.GetIdathaRequestOptions(process.vcc, accessToken, queryId, queryProcess, 0, 1, 2, process.processFilters);
                logger.info('[SemanticQueryImplementation::GetIdathaQueryId] executionCode:', executionCode, 'getting queueId:', currentQueueId, 'option request:', options);
                return (0, request_shared_1.SimplePostRequest)(options.body);
            }
            queryId = process.queryDataId;
            return Promise.resolve(null);
        }).then(function (result) {
            if (result) {
                queryId = result.query_id;
                return mssql_service_1.MSSql.SaveProcess(process.vcc, process.name, process.description, process.clientId, process.status, process.type, process.startDate, process.endDate, process.minDurationTime, process.maxDurationTime, process.language, process.dictionaryId, process.environment, process.queryId, queryId, moment().utc().toDate(), process.batchScheduleId, process.createdDate, process.createdByUserId, process.lastModifiedDate, process.lastModifiedByUserId);
            }
            return Promise.resolve(null);
        }).then(function (result) {
            _this.errorData.CleanErrorData();
            return Promise.resolve(queryId);
        }).catch(function (err) {
            var errorMessage = typeof err === 'string' ? err : (err.message || err.description);
            logger.error('[SemanticQueryImplementation::GetIdathaQueryId] executionCode:', executionCode, 'getting queueId:', currentQueueId, 'getting error:', errorMessage);
            _this.errorData.SetErrorData('GetIdathaQueryId', errorMessage, err);
            return Promise.resolve(null);
        });
    };
    SemanticQueryImplementation.prototype.GetIdathaQueryType = function (queryString) {
        var queryObject = null;
        var queryType = 'REGULAR';
        try {
            queryObject = JSON.parse(queryString);
            if (typeof queryObject === 'object') {
                if (queryObject.filter.includes('silences:'))
                    queryType = 'SILENCES';
                if (queryObject.filter.includes('fillers'))
                    queryType = 'FILLERS';
            }
        }
        catch (e) {
            logger.info('[SemanticQueryImplementation::GetIdathaQueryType] Could not parse query:', queryString, ', is not silences query');
        }
        return {
            queryType: queryType,
            queryString: queryString,
            queryObject: queryObject
        };
    };
    SemanticQueryImplementation.prototype.GetIdathaRequestOptions = function (vcc, accessToken, queryId, queryString, from, size, version, processFilters) {
        var _a, _b, _c, _d;
        if (version === void 0) { version = 1; }
        var idatha = new idatha_model_1.IdathaConnection();
        var uri = null;
        var path = null;
        var headers = { 'Authorization': null, 'Content-Type': 'application/json' };
        var body = {};
        var bodyParsed = null;
        var idathaQueryType = this.GetIdathaQueryType(queryString);
        var queryType = idathaQueryType.queryType;
        var queryObject = idathaQueryType.queryObject;
        switch (queryType) {
            case 'SILENCES':
                headers['Authorization'] = 'Basic ZWxhc3RpYzpFbGFzdGlj';
                path = "".concat(vcc, "_entries/_search");
                idatha.httpPort = '9200';
                var must = [];
                var silencesRange = { boost: 1.0 };
                var speaker = null;
                var startDate = ((_a = processFilters === null || processFilters === void 0 ? void 0 : processFilters.find(function (pf) { return pf.key === 'startDate'; })) === null || _a === void 0 ? void 0 : _a.value) || '2021-06-01';
                var endDate = ((_b = processFilters === null || processFilters === void 0 ? void 0 : processFilters.find(function (pf) { return pf.key === 'endDate'; })) === null || _b === void 0 ? void 0 : _b.value) || '2050-12-31';
                startDate = startDate.slice(0, 10);
                endDate = endDate.slice(0, 10);
                if ((_c = queryObject === null || queryObject === void 0 ? void 0 : queryObject.minType) === null || _c === void 0 ? void 0 : _c.length) {
                    silencesRange[queryObject.minType.toLowerCase()] = queryObject.minValue;
                    silencesRange['include_lower'] = queryObject.minType.includes('e') ? true : false;
                }
                if ((_d = queryObject === null || queryObject === void 0 ? void 0 : queryObject.maxType) === null || _d === void 0 ? void 0 : _d.length) {
                    silencesRange[queryObject.maxType.toLowerCase()] = queryObject.maxValue;
                    silencesRange['include_upper'] = queryObject.maxType.includes('e') ? true : false;
                }
                must.push({ range: { "words.silence": silencesRange } });
                if (queryObject.scope && queryObject.scope !== 'TEXT_ALL') {
                    if (queryObject.scope === 'in_sentences.')
                        speaker = 'in';
                    if (queryObject.scope === 'out_sentences.')
                        speaker = 'out';
                    if (speaker)
                        must.push({ match: { "words.speaker": speaker } });
                }
                bodyParsed = {
                    from: from,
                    size: size,
                    query: {
                        bool: {
                            filter: [
                                {
                                    bool: {
                                        must: [
                                            {
                                                nested: {
                                                    query: {
                                                        bool: {
                                                            must: must
                                                        }
                                                    },
                                                    path: "words",
                                                    ignore_unmapped: false,
                                                    score_mode: "none",
                                                    boost: 1.0,
                                                    inner_hits: {
                                                        ignore_unmapped: false,
                                                        from: 0,
                                                        size: 100,
                                                        version: false,
                                                        explain: false,
                                                        track_scores: false,
                                                        _source: {
                                                            includes: ["words.silence", "words.name", "words.speaker", "words.time"],
                                                            excludes: []
                                                        }
                                                    }
                                                }
                                            },
                                            {
                                                nested: {
                                                    query: {
                                                        bool: {
                                                            must: [
                                                                {
                                                                    range: {
                                                                        "filter_data.end_date": {
                                                                            gte: "".concat(startDate, "T00:00:00"),
                                                                            lte: "".concat(endDate, "T23:59:59"),
                                                                            include_lower: true,
                                                                            include_upper: true,
                                                                            boost: 1.0
                                                                        }
                                                                    }
                                                                }
                                                            ]
                                                        }
                                                    },
                                                    path: "filter_data",
                                                    ignore_unmapped: false,
                                                    score_mode: "none",
                                                    boost: 1.0,
                                                    inner_hits: {
                                                        ignore_unmapped: false,
                                                        from: 0,
                                                        size: 100,
                                                        version: false,
                                                        explain: false,
                                                        track_scores: false,
                                                        _source: {
                                                            includes: ["words.silence", "words.name", "words.speaker", "words.time"],
                                                            excludes: []
                                                        }
                                                    }
                                                }
                                            }
                                        ],
                                        adjust_pure_negative: true,
                                        boost: 1.0
                                    }
                                }
                            ],
                            adjust_pure_negative: true,
                            boost: 1.0
                        }
                    }
                };
                break;
            case 'FILLERS':
                headers['Authorization'] = 'Basic ZWxhc3RpYzpFbGFzdGlj';
                path = "".concat(vcc, "_entries/_search");
                idatha.httpPort = '9200';
                var searchText = queryObject.value;
                var fromText = from;
                var sizeText = size;
                var scope = queryObject.scope;
                var stringQuery = '';
                switch (scope) {
                    case '':
                        var stringQuery_1 = '{"from":' + fromText + ',"size":' + sizeText + ',"query": {"bool": {"filter": [{"bool": {"should": [{"nested": {"query": {"bool": {"must": [{"match_phrase": {"in_sentences.text":';
                        var stringQuery_2 = '"' + searchText + '"}}]}},"path": "in_sentences","ignore_unmapped": false,"score_mode": "none","boost": 1.0}},{"nested": {"query": {"bool": {"must": [{"match_phrase": {"out_sentences.text":';
                        var stringQuery_3 = '"' + searchText + '"}}]}},"path": "out_sentences","ignore_unmapped": false,"score_mode": "none","boost": 1.0}}],"adjust_pure_negative": true,"boost": 1.0}}],"adjust_pure_negative": true,"boost": 1.0}},"script_fields": {"ocurrencies_in": {"script": {"lang": "painless","source": "int count = 0; if(params._source.in_sentences != null) {for(int i=0; i<params._source.in_sentences.length; i++){String tmp = params._source.in_sentences[i].text;';
                        var stringQuery_4 = 'Matcher m = /(?i)\\\\b' + searchText + '\\\\b/.matcher(tmp); while (m.find()) { count++;}}} return count;"}},"ocurrencies_out": {"script": {"lang": "painless","source": "int count = 0; if(params._source.out_sentences != null) {for(int i=0; i<params._source.out_sentences.length; i++){String tmp = params._source.out_sentences[i].text;';
                        var stringQuery_5 = 'Matcher m = /(?i)\\\\b' + searchText + '\\\\b/.matcher(tmp); while (m.find()) { count++;}}} return count;"}},';
                        var stringQuery_6 = '"phrases_ocurrencies_in": {"script": {"lang": "painless","source": "def arr = []; if(params._source.in_sentences != null) {for(int i=0; i<params._source.in_sentences.length; i++){String tmp = params._source.in_sentences[i].text;';
                        var stringQuery_7 = 'Matcher m = /\\\\b' + searchText + '\\\\b/i.matcher(tmp); if(m.find()) { arr.add(params._source.in_sentences[i].text)}}} return arr;"}},';
                        var stringQuery_8 = '"phrases_ocurrencies_out": {"script": {"lang": "painless","source": "def arr = []; if(params._source.out_sentences != null) {for(int i=0; i<params._source.out_sentences.length; i++){String tmp = params._source.out_sentences[i].text;';
                        var stringQuery_9 = 'Matcher m = /\\\\b' + searchText + '\\\\b/i.matcher(tmp); if(m.find()) { arr.add(params._source.out_sentences[i].text)}}} return arr;"}},';
                        var stringQuery_10 = '"filter_data":{"script": {"source": "params._source.filter_data"}},"words":{"script":{"source": "params._source.words"}}}}';
                        stringQuery = stringQuery_1 + stringQuery_2 + stringQuery_3 + stringQuery_4 + stringQuery_5 + stringQuery_6 + stringQuery_7 + stringQuery_8 + stringQuery_9 + stringQuery_10;
                        break;
                    case 'in_sentences.':
                        var stringQueryIn_1 = '{"from": ' + fromText + ',"size":' + sizeText + ',"query": {"bool": {"filter": [{"bool": { "must": [ { "nested": { "query": { "bool": { "must": [ {"match_phrase":';
                        var stringQueryIn_2 = '{"in_sentences.text": "' + searchText + '"}}]}}, "path": "in_sentences", "ignore_unmapped": false, "score_mode": "none", "boost": 1.0}}], "adjust_pure_negative": true, "boost": 1.0}}],"adjust_pure_negative": true,"boost": 1.0}},"script_fields": { "ocurrencies_in": {"script": {"lang": "painless","source": "int count = 0;if(params._source.in_sentences != null) {for(int i=0; i<params._source.in_sentences.length; i++){String tmp = params._source.in_sentences[i].text;';
                        var stringQueryIn_3 = 'Matcher m = /(?i)\\\\b' + searchText + '\\\\b/.matcher(tmp); while (m.find()) { count++;}}} return count;"}},';
                        var stringQueryIn_4 = '"phrases_ocurrencies_in": {"script": {"lang": "painless","source": "def arr = []; if(params._source.in_sentences != null) {for(int i=0; i<params._source.in_sentences.length; i++){String tmp = params._source.in_sentences[i].text;';
                        var stringQueryIn_5 = 'Matcher m = /\\\\b' + searchText + '\\\\b/i.matcher(tmp); if(m.find()) { arr.add(params._source.in_sentences[i].text)}}} return arr;"}},';
                        var stringQueryIn_6 = '"filter_data":{ "script": {"source": "params._source.filter_data"}},"words":{"script":{"source": "params._source.words"}}}}';
                        stringQuery = stringQueryIn_1 + stringQueryIn_2 + stringQueryIn_3 + stringQueryIn_4 + stringQueryIn_5 + stringQueryIn_6;
                        break;
                    case 'out_sentences.':
                        var stringQueryOut_1 = '{"from": ' + fromText + ',"size":' + sizeText + ',"query": {"bool": {"filter": [{"bool": {"must": [{"nested": {"query": {"bool": {"must": [ {"match_phrase":';
                        var stringQueryOut_2 = '{"out_sentences.text": "' + searchText + '"}}]}}, "path": "out_sentences", "ignore_unmapped": false, "score_mode": "none", "boost": 1.0}}], "adjust_pure_negative": true, "boost": 1.0}}],"adjust_pure_negative": true,"boost": 1.0}},"script_fields": { "ocurrencies_out": {"script": {"lang": "painless","source": "int count = 0;if(params._source.out_sentences != null) {for(int i=0; i<params._source.out_sentences.length; i++){String tmp = params._source.out_sentences[i].text;';
                        var stringQueryOut_3 = 'Matcher m = /(?i)\\\\b' + searchText + '\\\\b/.matcher(tmp); while (m.find()) { count++;}}} return count;"}},';
                        var stringQueryOut_4 = '"phrases_ocurrencies_out": {"script": {"lang": "painless","source": "def arr = []; if(params._source.out_sentences != null) {for(int i=0; i<params._source.out_sentences.length; i++){String tmp = params._source.out_sentences[i].text;';
                        var stringQueryOut_5 = 'Matcher m = /\\\\b' + searchText + '\\\\b/i.matcher(tmp); if(m.find()) { arr.add(params._source.out_sentences[i].text)}}} return arr;"}},';
                        var stringQueryOut_6 = '"filter_data":{ "script": {"source": "params._source.filter_data"}},"words":{"script":{"source": "params._source.words"}}}}';
                        stringQuery = stringQueryOut_1 + stringQueryOut_2 + stringQueryOut_3 + stringQueryOut_4 + stringQueryOut_5 + stringQueryOut_6;
                        break;
                }
                bodyParsed = JSON.parse(stringQuery);
                break;
            default:
                headers['Authorization'] = "Bearer ".concat(accessToken);
                path = 'api/query';
                if (version === 2) {
                    var dataObject = {
                        version: version,
                        from: from,
                        size: size,
                        query: queryString
                    };
                    if (queryId)
                        dataObject.query_id = queryId;
                    var data = JSON.stringify(dataObject);
                    bodyParsed = JSON.parse(data.replace(/\\\\\\/g, "\\"));
                }
                else {
                    var stringQuery_11 = '{"query_id": "' + queryId + '", "query": "' + queryString.replace(/\"/g, "\\\\\\\\\\\\\"") + '", "from": ' + from + ', "size": ' + size + ',"version":' + (version ? version : 1) + '}';
                    bodyParsed = JSON.parse(stringQuery_11);
                }
                break;
        }
        uri = "http://".concat(idatha.ip, ":").concat(idatha.httpPort, "/");
        uri += path;
        body = {
            uri: uri,
            timeout: 60000,
            body: bodyParsed,
            headers: headers,
            retryProtocol: false
        };
        return {
            queryType: queryType,
            body: body
        };
    };
    SemanticQueryImplementation.prototype.GetIdathaRequestResponse = function (queryType, queryString, rawResponse) {
        var response = {};
        switch (queryType) {
            case 'SILENCES':
                var queryObject_1 = JSON.parse(queryString);
                response.total_results = rawResponse.hits.total.value;
                response.results = [];
                rawResponse.hits.hits.map(function (h) {
                    h._source.matches = h.inner_hits.words.hits.total.value;
                    h._source.matchesDetail = [];
                    h.inner_hits.words.hits.hits.map(function (hh) {
                        if (hh._source.words)
                            h._source.matchesDetail.push(hh._source.words);
                        else
                            h._source.matchesDetail.push(hh._source);
                    });
                    if (h._source.matches >= queryObject_1.minOcurrencies)
                        response.results.push(h._source);
                });
                break;
            case 'FILLERS':
                var queryFillersObject_1 = JSON.parse(queryString);
                response.total_results = rawResponse.hits.total.value;
                response.results = [];
                rawResponse.hits.hits.map(function (h) {
                    var _a, _b, _c, _d;
                    var totalOcurrencies = 0;
                    var _source = {};
                    var scope = queryFillersObject_1.scope;
                    switch (scope) {
                        case '':
                            totalOcurrencies = h.fields.ocurrencies_out[0] + h.fields.ocurrencies_in[0];
                            break;
                        case 'in_sentences.':
                            totalOcurrencies = h.fields.ocurrencies_in[0];
                            break;
                        case 'out_sentences.':
                            totalOcurrencies = h.fields.ocurrencies_out[0];
                            break;
                    }
                    if (totalOcurrencies >= queryFillersObject_1.minOcurrencies) {
                        h._source = _source;
                        switch (scope) {
                            case '':
                                logger.debug("entraqui con ", totalOcurrencies);
                                logger.debug("entraqui con2 ", h.fields.phrases_ocurrencies_in);
                                h._source.matches = totalOcurrencies;
                                h._source.matchesDetail = [];
                                (_a = h.fields.phrases_ocurrencies_in) === null || _a === void 0 ? void 0 : _a.map(function (hh) {
                                    if (hh)
                                        h._source.matchesDetail.push(hh);
                                });
                                (_b = h.fields.phrases_ocurrencies_out) === null || _b === void 0 ? void 0 : _b.map(function (hh) {
                                    if (hh)
                                        h._source.matchesDetail.push(hh);
                                });
                                break;
                            case 'in_sentences.':
                                h._source.matches = totalOcurrencies;
                                h._source.matchesDetail = [];
                                (_c = h.fields.phrases_ocurrencies_in) === null || _c === void 0 ? void 0 : _c.map(function (hh) {
                                    if (hh)
                                        h._source.matchesDetail.push(hh);
                                });
                                break;
                            case 'out_sentences.':
                                h._source.matches = totalOcurrencies;
                                h._source.matchesDetail = [];
                                (_d = h.fields.phrases_ocurrencies_out) === null || _d === void 0 ? void 0 : _d.map(function (hh) {
                                    if (hh)
                                        h._source.matchesDetail.push(hh);
                                });
                                break;
                        }
                        h._source.id = h._id;
                        h._source.words = h.fields.words;
                        h._source.filter_data = h.fields.filter_data[0];
                        delete (h.fields);
                        response.results.push(h._source);
                    }
                });
                break;
            default:
                response = rawResponse;
                break;
        }
        return response;
    };
    SemanticQueryImplementation.prototype.GetIdathaQuery = function (accessToken, vcc, queryId, queryName, query, from, size, executionCode, currentQueueId, version, process) {
        var _this = this;
        if (version === void 0) { version = 1; }
        var isNext = true;
        var queryType = null;
        var queryResultTotal = 0;
        var queryResultPages = 0;
        return (0, promises_shared_1.DummyPromise)()
            .then(function (result) {
            var options = _this.GetIdathaRequestOptions(vcc, accessToken, queryId, query, from, size, version, process.processFilters);
            queryType = options.queryType;
            logger.info('[SemanticQueryImplementation::GetIdathaRegularQuery] executionCode:', executionCode, 'getting queueId:', currentQueueId, 'option request:', options);
            return (0, request_shared_1.SimplePostRequest)(options.body);
        })
            .then(function (result) {
            var response = _this.GetIdathaRequestResponse(queryType, query, result);
            logger.info('[SemanticQueryImplementation::GetIdathaRegularQuery] Query: "' + query + '", Items per page: ' + size);
            logger.info('[SemanticQueryImplementation::GetIdathaRegularQuery] Query: "' + query + '", Total results: ' + response.total_results);
            logger.info('[SemanticQueryImplementation::GetIdathaRegularQuery] Query: "' + query + '", Results for this request: ' + response.results.length);
            isNext = from + size <= (response.total_results * 1) ? true : false;
            queryResultTotal = response.total_results;
            queryResultPages = response.total_results > 0 ? Math.ceil(response.total_results / size) : 1;
            response.results = _this.ReduceRequestResult(response, query);
            return _this.SaveDocumentQuery(response.results, queryName, executionCode, currentQueueId);
        }).then(function (result) {
            if (!result)
                throw 'ERROR_GET_IDATHA_INTERACTIONS';
            if (isNext)
                return _this.GetIdathaQuery(accessToken, vcc, queryId, queryName, query, (from + size), size, executionCode, currentQueueId, version, process);
            return true;
        }).then(function (result) {
            _this.SaveSemanticQueryResult(currentQueueId, queryResultTotal, queryResultPages, size);
            _this.errorData.CleanErrorData();
            return Promise.resolve(result);
        }).catch(function (err) {
            var errorMessage = typeof err === 'string' ? err : (err.message || err.description);
            logger.debug('[SemanticQueryImplementation::GetIdathaRegularQuery] executionCode:', executionCode, 'getting queueId:', currentQueueId, 'getting rawError:', err);
            logger.error('[SemanticQueryImplementation::GetIdathaRegularQuery] executionCode:', executionCode, 'getting queueId:', currentQueueId, 'getting error:', errorMessage);
            _this.errorData.SetErrorData('GetIdathaRegularQuery', errorMessage, err);
            if (err && err.statusCode && err.statusCode === 500) {
                return Promise.resolve(null);
            }
            else {
                return Promise.resolve(false);
            }
        });
    };
    SemanticQueryImplementation.prototype.ReduceRequestResult = function (response, query) {
        var _a;
        var pattern = /\in_sentences|\out_sentences/g;
        if (query.match(pattern)) {
            var orPattern1 = /((?!out_sentences|in_sentences)\w+\s(OR)\s.*(in_sentences|out_sentences)[^\s]+)/g;
            var orPattern2 = /((out_sentences|in_sentences)[^\s]+\s(OR)\s((?!out_sentences|in_sentences)\w+[^\s]))/g;
            if (query.match(orPattern1) || query.match(orPattern2)) {
            }
            else {
                response.results = response.results.filter(function (r) { return r && r.filter_data && r.filter_data.channel && r.filter_data.channel !== 'MAIL'; });
            }
        }
        if ((_a = response === null || response === void 0 ? void 0 : response.results) === null || _a === void 0 ? void 0 : _a.length) {
            response.results.forEach(function (r, i) {
                if (r) {
                    r.char = r.words.map(function (w) { return w.name.length; }).reduce(function (acc, curr) { return acc + curr + 1; }, 0);
                    var inWords = r.words.filter(function (w) { return w.speaker === 'in'; });
                    var outWords = r.words.filter(function (w) { return w.speaker === 'out'; });
                    r.lenWordsIn = inWords.length || 0;
                    r.lenWordsOut = outWords.length || 0;
                    r.callDurationIn = 0;
                    r.callSilenceIn = 0;
                    r.callConfidenceIn = 0;
                    r.callDurationOut = 0;
                    r.callSilenceOut = 0;
                    r.callConfidenceOut = 0;
                    r.otherDurationIn = 0;
                    r.otherSilenceIn = 0;
                    r.otherConfidenceIn = 0;
                    r.otherDurationOut = 0;
                    r.otherSilenceOut = 0;
                    r.otherConfidenceOut = 0;
                    r.totalConfidenceIn = 0;
                    r.totalConfidenceOut = 0;
                    if (r.filter_data.channel === 'CALL') {
                        r.callDurationIn = inWords.map(function (iw) { return iw.duration; }).reduce(function (acc, curr) { return acc + curr; }, 0);
                        r.callSilenceIn = inWords.filter(function (iw) { return iw.silence > 0; }).map(function (iw) { return iw.silence; }).reduce(function (acc, curr) { return acc + curr; }, 0);
                        r.callConfidenceIn = inWords.map(function (iw) { return iw.confidence; }).reduce(function (acc, curr) { return acc + curr; }, 0);
                        r.callDurationOut = outWords.map(function (ow) { return ow.duration; }).reduce(function (acc, curr) { return acc + curr; }, 0);
                        r.callSilenceOut = outWords.filter(function (ow) { return ow.silence > 0; }).map(function (ow) { return ow.silence; }).reduce(function (acc, curr) { return acc + curr; }, 0);
                        r.callConfidenceOut = outWords.map(function (ow) { return ow.confidence; }).reduce(function (acc, curr) { return acc + curr; }, 0);
                    }
                    else {
                        r.otherDurationIn = inWords.map(function (iw) { return iw.duration; }).reduce(function (acc, curr) { return acc + curr; }, 0);
                        r.otherSilenceIn = inWords.filter(function (iw) { return iw.silence > 0; }).map(function (iw) { return iw.silence; }).reduce(function (acc, curr) { return acc + curr; }, 0);
                        r.otherConfidenceIn = inWords.map(function (iw) { return iw.confidence; }).reduce(function (acc, curr) { return acc + curr; }, 0);
                        r.otherDurationOut = outWords.map(function (ow) { return ow.duration; }).reduce(function (acc, curr) { return acc + curr; }, 0);
                        r.otherSilenceOut = outWords.filter(function (ow) { return ow.silence > 0; }).map(function (ow) { return ow.silence; }).reduce(function (acc, curr) { return acc + curr; }, 0);
                        r.otherConfidenceOut = outWords.map(function (ow) { return ow.confidence; }).reduce(function (acc, curr) { return acc + curr; }, 0);
                    }
                    if (r.lenWordsIn)
                        r.totalConfidenceIn = ((r.callConfidenceIn + r.otherConfidenceIn) / r.lenWordsIn);
                    if (r.lenWordsOut)
                        r.totalConfidenceOut = ((r.callConfidenceOut + r.otherConfidenceOut) / r.lenWordsOut);
                    if (r.words)
                        delete (r.words);
                    if (r.in_sentences)
                        delete (r.in_sentences);
                    if (r.out_sentences)
                        delete (r.out_sentences);
                }
            });
        }
        return response.results;
    };
    SemanticQueryImplementation.prototype.SaveDocumentQuery = function (data, id, executionCode, currentQueueId) {
        var _this = this;
        var fileName = config_2.QUERY_FOLDER + '/' + currentQueueId.replace(/:/g, '=') + '.json';
        return (0, promises_shared_1.DummyPromise)()
            .then(function () {
            return (0, file_manager_1.CreateFolder)(config_2.QUERY_FOLDER);
        }).then(function () {
            return (0, file_manager_1.ReadFileContent)(fileName, 'utf-8', true);
        }).then(function (resultFile) {
            var jsonContent;
            var results = data ? data : [];
            try {
                if (resultFile) {
                    jsonContent = JSON.parse(resultFile);
                    jsonContent[id] = jsonContent[id] ? jsonContent[id].concat(results) : [];
                }
                else {
                    jsonContent = {};
                    jsonContent[id] = results;
                }
            }
            catch (ex) {
                logger.error('[SemanticQueryImplementation::SaveDocumentQuery] executionCode:', executionCode, 'getting queueId:', currentQueueId, 'getting error:', ex);
                throw 'ERROR_FILE_JSON';
            }
            return (0, file_manager_1.WriteFileContent)(fileName, JSON.stringify(jsonContent), 'utf-8', true);
        }).then(function (result) {
            _this.errorData.CleanErrorData();
            return Promise.resolve(true);
        }).catch(function (error) {
            var errorMessage = typeof error === 'string' ? error : (error.message || error.description);
            logger.error('[SemanticQueryImplementation::SaveDocumentQuery] executionCode:', executionCode, 'getting queueId:', currentQueueId, 'getting error:', errorMessage);
            _this.errorData.SetErrorData('SaveDocumentQuery', errorMessage, error);
            return Promise.resolve(false);
        });
    };
    SemanticQueryImplementation.prototype.GetDocumentQuery = function (executionCode, currentQueueId) {
        var _this = this;
        var fileName = config_2.QUERY_FOLDER + '/' + currentQueueId.replace(/:/g, '=') + '.json';
        return (0, promises_shared_1.DummyPromise)()
            .then(function () {
            return (0, file_manager_1.ReadFileContent)(fileName, 'utf-8', true);
        }).then(function (resultFile) {
            var jsonContent;
            try {
                jsonContent = JSON.parse(resultFile);
            }
            catch (ex) {
                jsonContent = null;
                throw 'ERROR_FILE_JSON';
            }
            _this.errorData.CleanErrorData();
            return Promise.resolve(jsonContent);
        }).catch(function (error) {
            var errorMessage = typeof error === 'string' ? error : (error.message || error.description);
            logger.error('[SemanticQueryImplementation::GetDocumentQuery] executionCode:', executionCode, 'getting queueId:', currentQueueId, 'getting error:', errorMessage);
            _this.errorData.SetErrorData('GetDocumentQuery', errorMessage, error);
            return Promise.resolve(null);
        });
    };
    SemanticQueryImplementation.prototype.SaveSemanticQueryResult = function (queueId, totalResults, totalPages, totalItemsPerPage) {
        var billing = new billing_implementation_1.BillingImplementation();
        var myBatchName = '';
        var myQueueIdArray = queueId.split(':');
        switch (myQueueIdArray[3]) {
            case 'study':
                myBatchName = 'STUDY';
                break;
            case 'template':
                myBatchName = 'TEMPLATE';
                break;
            case 'evaluationPlan':
                myBatchName = 'EVALUATION_PLAN';
                break;
            case 'ability':
                myBatchName = 'EVALUATION_PLAN';
                break;
            case 'search':
                myBatchName = '';
                break;
        }
        billing.AddUpdateSemanticAnalysisDetailToMainFile(myQueueIdArray[0], myQueueIdArray[1], myQueueIdArray[2], myBatchName, myQueueIdArray[4], 'TotalResults', totalResults);
        billing.AddUpdateSemanticAnalysisDetailToMainFile(myQueueIdArray[0], myQueueIdArray[1], myQueueIdArray[2], myBatchName, myQueueIdArray[4], 'TotalPages', totalPages);
        billing.AddUpdateSemanticAnalysisDetailToMainFile(myQueueIdArray[0], myQueueIdArray[1], myQueueIdArray[2], myBatchName, myQueueIdArray[4], 'TotalItemsPerPage', totalItemsPerPage);
    };
    SemanticQueryImplementation.prototype.CloseSemanticQuery = function (queueId, resultCode, resultDescription) {
        var billing = new billing_implementation_1.BillingImplementation();
        var myBatchName = '';
        var myQueueIdArray = queueId.split(':');
        switch (myQueueIdArray[3]) {
            case 'study':
                myBatchName = 'STUDY';
                break;
            case 'template':
                myBatchName = 'TEMPLATE';
                break;
            case 'evaluationPlan':
                myBatchName = 'EVALUATION_PLAN';
                break;
            case 'ability':
                myBatchName = 'EVALUATION_PLAN';
                break;
            case 'search':
                myBatchName = '';
                break;
        }
        billing.AddCloseSemanticAnalysisDetailToMainFile(myQueueIdArray[0], myQueueIdArray[1], myQueueIdArray[2], myBatchName, myQueueIdArray[4], resultCode ? 1 : 0, resultDescription);
    };
    return SemanticQueryImplementation;
}());
exports.SemanticQueryImplementation = SemanticQueryImplementation;
function GetQueryIdathaProcessV2(processFilters) {
    var queries = [];
    var campaigns = [];
    var agents = [];
    var channels = [];
    var startDate = '';
    var endDate = '';
    for (var _i = 0, processFilters_1 = processFilters; _i < processFilters_1.length; _i++) {
        var p = processFilters_1[_i];
        if (p.value) {
            if (p.key == 'agents') {
                agents.push("filter_data:(agent:\"".concat(p.value, "\")"));
            }
            if (p.key == 'endDate') {
                endDate = p.value;
            }
            if (p.key == 'startDate') {
                startDate = p.value;
            }
            if (p.key == 'types') {
                channels.push("filter_data:(channel:\"".concat(p.value, "\")"));
            }
        }
    }
    if (startDate != '' && endDate != '') {
        startDate = startDate.slice(0, 10);
        endDate = endDate.slice(0, 10);
        queries.push("filter_data:(end_date:[".concat(startDate, " TO ").concat(endDate, "])"));
    }
    if (agents.length > 0) {
        queries.push("(".concat(agents.join(' OR '), ")"));
    }
    if (campaigns.length > 0) {
        queries.push("(".concat(campaigns.join(' OR '), ")"));
    }
    if (channels.length > 0) {
        queries.push("(".concat(channels.join(' OR '), ")"));
    }
    return queries.join(' AND ');
}
exports.GetQueryIdathaProcessV2 = GetQueryIdathaProcessV2;
