"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.TipificationAnalysisImplementation = void 0;
var log4js = require("log4js");
var mssql_service_1 = require("../db/mssql.service");
var redis_service_1 = require("../db/redis.service");
var moment = require("moment");
var config_1 = require("../config/config");
var promises_shared_1 = require("../shared/promises.shared");
var file_manager_1 = require("../shared/file-manager");
var cognitive_analysis_implementation_1 = require("./cognitive-analysis.implementation");
var billing_batch_model_1 = require("../models/billing-batch.model");
var evaluation_result_model_1 = require("../models/evaluation-result.model");
var logger = log4js.getLogger('Task');
var CURRENT_SYSTEM_SERVICE = 'TIPIFICATION_ANALYSIS';
var CURRENT_IMPLEMENTATION = 'TipificationAnalysisImplementation';
var TipificationAnalysisImplementation = (function () {
    function TipificationAnalysisImplementation() {
        this.errorData = new billing_batch_model_1.BatchError();
        this.cognitiveAnalysis = new cognitive_analysis_implementation_1.CognitiveAnalysisImplementation();
    }
    TipificationAnalysisImplementation.prototype.Execute = function (executionCode, rateRecursive, availableBatch, cancelledBatch) {
        var _this = this;
        var executeIndex = 1;
        var ratio = config_1.RATIO_TIPIFICATION_ANALYSIS;
        var alowedRatio = 0;
        var notAlowedRatio = 0;
        var executeResult = true;
        return (0, promises_shared_1.SleepPromise)(2)
            .then(function (_) {
            return redis_service_1.Redis.LRange('TipificationAnalysis', 0, ratio - 1);
        })
            .then(function (result) {
            if (!result)
                result = [];
            var promiseArray = [];
            ratio = result && result.length <= ratio ? result.length : 0;
            logger.info('[TipificationAnalysisImplementation::Execute] TipificationAnalysis lrange result: ' + result);
            result.forEach(function (r) {
                var arrQueue = r.split(':');
                if (availableBatch.find(function (b) { return b.VCC === arrQueue[0] && b.BatchTypeId.toLowerCase() === arrQueue[7].toLowerCase(); })) {
                    alowedRatio++;
                }
                else {
                    notAlowedRatio++;
                }
            });
            while (ratio >= executeIndex) {
                promiseArray.push(_this.TipificationAnalysisExecute(executionCode, availableBatch, cancelledBatch));
                executeIndex++;
            }
            return Promise.all(promiseArray);
        })
            .then(function (results) {
            if (results.filter(function (item) { return item === false; }).length > notAlowedRatio)
                executeResult = false;
            return redis_service_1.Redis.LRange('TipificationAnalysis', 0, ratio - 1);
        })
            .then(function (result) {
            if (result && result.length && rateRecursive >= 1)
                return _this.Execute(executionCode, rateRecursive - 1, availableBatch, cancelledBatch);
            return executeResult;
        })
            .then(function (result) {
            var currentResult = executeResult && result ? true : false;
            return Promise.resolve(currentResult);
        })
            .catch(function (err) {
            var errorMessage = typeof err === 'string' ? err : (err.message || err.description);
            logger.error('[TipificationAnalysisImplementation::Execute] getting error:', errorMessage);
            return Promise.resolve(false);
        });
    };
    TipificationAnalysisImplementation.prototype.TipificationAnalysisExecute = function (executionCode, availableBatch, cancelledBatch) {
        var _this = this;
        var vcc = '';
        var processId = '';
        var interactionType = '';
        var type = '';
        var typeId = '';
        var currentQueueId = '';
        var interactionCognitive;
        var queueProgressStorage = '';
        var filePath = '';
        return redis_service_1.Redis.RemoveQueueL('TipificationAnalysis')
            .then(function (result) {
            var arrQueue = result.split(':');
            var promiseArray = [];
            if (availableBatch.find(function (b) { return b.VCC === arrQueue[0] && b.BatchTypeId.toLowerCase() === arrQueue[8].toLowerCase(); })) {
                promiseArray.push(Promise.resolve('ALLOWED'));
                promiseArray.push(redis_service_1.Redis.CreateQueueL('TipificationAnalysisExecute', result));
                promiseArray.push(Promise.resolve(result));
            }
            else {
                promiseArray.push(Promise.resolve('NOT_ALLOWED'));
                promiseArray.push(redis_service_1.Redis.CreateQueueL('TipificationAnalysis', result));
                promiseArray.push(mssql_service_1.MSSql.SaveBatchStatus(arrQueue[0], arrQueue[8], 'COGNITIVE_ANALYSIS', 'PAUSED'));
            }
            return Promise.all(promiseArray);
        })
            .then(function (result) {
            if (result && result[0] === 'NOT_ALLOWED')
                throw 'SERVER_ERROR_NOT_ALLOWED_QUEUE';
            var queueId = result[2];
            if ((queueId === null || queueId === void 0 ? void 0 : queueId.length) > 0) {
                currentQueueId = queueId;
                logger.info('[TipificationAnalysisImplementation::TipificationQueryExecute] executionCode:', executionCode, 'getting queueId:', currentQueueId, 'START');
                var item = queueId.split(':');
                if (item === null || item === void 0 ? void 0 : item.length) {
                    vcc = item[0];
                    processId = item[1];
                    interactionType = item[2];
                    type = item[7] ? item[7] : null;
                    typeId = item[8] ? item[8] : null;
                    if (type) {
                        queueProgressStorage = "".concat(vcc, ":").concat(processId, ":").concat(type);
                        queueProgressStorage += typeId ? ":".concat(typeId) : "";
                    }
                    logger.info('[TipificationAnalysisImplementation::TipificationAnalysisExecute] executionCode:', executionCode, 'getting queueId:', currentQueueId, 'storage :', queueProgressStorage);
                    return redis_service_1.Redis.GetTipificationInteraction(currentQueueId);
                }
                else {
                    logger.error('[TipificationAnalysisImplementation::TipificationAnalysisExecute] executionCode:', executionCode, 'getting queueId:', currentQueueId, 'QUEUE_FORMAT_ERROR');
                }
            }
            return Promise.resolve(null);
        })
            .then(function (result) {
            if (!result)
                logger.warn('[TipificationAnalysisImplementation::TipificationAnalysisExecute] executionCode:', executionCode, 'getting queueId:', currentQueueId, 'getting error: NOT_FOUND_INTERACTION_COGNITIVE');
            if (result) {
                if (cancelledBatch.find(function (b) { return b.VCC === result.vcc && b.BatchTypeId.toLowerCase() === (result.studyId ? result.studyId : result.evaluationPlanId).toLowerCase(); })) {
                    result.status = -7;
                    result.statusDescription = 'REMOVED_BY_CANCELLED';
                    result.retry = config_1.RETRY_TIPIFICATION_ANALYSIS;
                    result.errorFunction = 'TipificationAnalysisExecute';
                    result.errorMessage = 'REMOVED_BY_CANCELLED';
                    result.errorDetail = 'REMOVED_BY_CANCELLED';
                }
                interactionCognitive = result;
            }
            if ((interactionCognitive === null || interactionCognitive === void 0 ? void 0 : interactionCognitive.statusDescription) === 'pendingAnalysis') {
                logger.info('[TipificationAnalysisImplementation::TipificationAnalysisExecute] executionCode:', executionCode, 'getting queueId:', currentQueueId, 'pendingAnalysis');
                return _this.EvaluateTipification(interactionCognitive, currentQueueId, executionCode);
            }
            return Promise.resolve(null);
        })
            .then(function (result) {
            interactionCognitive = result;
            return redis_service_1.Redis.SaveCognitiveInteraction(currentQueueId, interactionCognitive);
        })
            .then(function (result) {
            var promiseArray = [];
            if (result && (interactionCognitive === null || interactionCognitive === void 0 ? void 0 : interactionCognitive.status) === 7 && (interactionCognitive === null || interactionCognitive === void 0 ? void 0 : interactionCognitive.statusDescription) === 'ready') {
                logger.info('[TipificationAnalysisImplementation::TipificationAnalysisExecute] executionCode:', executionCode, 'getting queueId:', currentQueueId, 'READY');
                promiseArray.push(redis_service_1.Redis.DeleteQueue('TipificationAnalysisExecute', currentQueueId));
            }
            else if (result && (interactionCognitive === null || interactionCognitive === void 0 ? void 0 : interactionCognitive.status) === 7 && (interactionCognitive === null || interactionCognitive === void 0 ? void 0 : interactionCognitive.statusDescription) === 'pendingNLU' && (interactionCognitive === null || interactionCognitive === void 0 ? void 0 : interactionCognitive.retry) < config_1.RETRY_TIPIFICATION_ANALYSIS) {
                promiseArray.push(redis_service_1.Redis.CreateQueueL('TipificationAnalysis', currentQueueId));
                promiseArray.push(redis_service_1.Redis.DeleteQueue('TipificationAnalysisExecute', currentQueueId));
            }
            else if (result && (interactionCognitive === null || interactionCognitive === void 0 ? void 0 : interactionCognitive.status) === -7 && (interactionCognitive === null || interactionCognitive === void 0 ? void 0 : interactionCognitive.statusDescription) === 'REMOVED_BY_CANCELLED') {
                logger.error('[TipificationAnalysisImplementation::TipificationAnalysisExecute] executionCode:', executionCode, 'getting queueId:', currentQueueId, 'REMOVED_BY_CANCELLED');
                promiseArray.push(redis_service_1.Redis.DeleteQueue('TipificationAnalysisExecute', currentQueueId));
            }
            else if (result && (interactionCognitive === null || interactionCognitive === void 0 ? void 0 : interactionCognitive.status) === -7 && (interactionCognitive === null || interactionCognitive === void 0 ? void 0 : interactionCognitive.statusDescription) === 'ERROR_FILE_JSON') {
                promiseArray.push(redis_service_1.Redis.DeleteQueue('TipificationAnalysisExecute', currentQueueId));
            }
            else if ((interactionCognitive === null || interactionCognitive === void 0 ? void 0 : interactionCognitive.retry) >= config_1.RETRY_TIPIFICATION_ANALYSIS) {
                logger.error('[TipificationAnalysisImplementation::TipificationAnalysisExecute] executionCode:', executionCode, 'getting queueId:', currentQueueId, 'REMOVED_BY_RETRIES');
                promiseArray.push(redis_service_1.Redis.DeleteQueue('TipificationAnalysisExecute', currentQueueId));
            }
            else if (currentQueueId && !interactionCognitive) {
                logger.error('[TipificationAnalysisImplementation::TipificationAnalysisExecute] executionCode:', executionCode, 'getting queueId:', currentQueueId, 'INTERACTION_DO_NOT_EXIST');
                promiseArray.push(redis_service_1.Redis.DeleteQueue('TipificationAnalysisExecute', currentQueueId));
            }
            else {
                logger.error('[TipificationAnalysisImplementation::TipificationAnalysisExecute] executionCode:', executionCode, 'getting queueId:', currentQueueId, 'QUEUE_WAS_NOT_REMOVED');
            }
            return Promise.all(promiseArray);
        })
            .then(function (result) {
            var promiseArray = [];
            if ((interactionCognitive === null || interactionCognitive === void 0 ? void 0 : interactionCognitive.status) === 7
                &&
                    (interactionCognitive === null || interactionCognitive === void 0 ? void 0 : interactionCognitive.statusDescription) === 'ready') {
                promiseArray.push(redis_service_1.Redis.UpdateProgress(queueProgressStorage));
                promiseArray.push(redis_service_1.Redis.DeleteTipificationInteraction(currentQueueId));
                if (type === 'tipification') {
                    logger.debug('[TipificationAnalysisImplementation::TipificationAnalysisExecute] executionCode:', executionCode, 'getting queueId:', currentQueueId, 'Saving cognitive result for evaluationplan:', interactionCognitive);
                    promiseArray.push(redis_service_1.Redis.SaveReadyTipificationInteraction(currentQueueId, interactionCognitive, executionCode));
                }
                else {
                    logger.error('[TipificationAnalysisImplementation::TipificationAnalysisExecute] executionCode:', executionCode, 'getting queueId:', currentQueueId, 'TYPE_IS_NOT_FOUND', type);
                }
            }
            else if ((interactionCognitive === null || interactionCognitive === void 0 ? void 0 : interactionCognitive.retry) >= config_1.RETRY_TIPIFICATION_ANALYSIS) {
                promiseArray.push(redis_service_1.Redis.UpdateProgress(queueProgressStorage));
                promiseArray.push(redis_service_1.Redis.UpdateErrorProgress(queueProgressStorage));
                promiseArray.push(redis_service_1.Redis.DeleteTipificationInteraction(currentQueueId));
                if (type === 'tipification') {
                    logger.error('[TipificationAnalysisImplementation::TipificationAnalysisExecute] executionCode:', executionCode, 'getting queueId:', currentQueueId, 'Saving cognitive result for evaluationplan:', interactionCognitive);
                    promiseArray.push(redis_service_1.Redis.SaveReadyTipificationInteraction(currentQueueId, interactionCognitive, executionCode));
                }
                else {
                    logger.error('[TipificationAnalysisImplementation::TipificationAnalysisExecute] executionCode:', executionCode, 'getting queueId:', currentQueueId, 'TYPE_IS_NOT_FOUND', type);
                }
                promiseArray.push(mssql_service_1.MSSql.SaveBatchError(vcc, CURRENT_SYSTEM_SERVICE, typeId, ['study', 'template'].includes(type) ? typeId : null, CURRENT_IMPLEMENTATION, interactionCognitive.id, interactionCognitive.nluId, _this.errorData.errorFunction, _this.errorData.errorMessage, _this.errorData.errorDetail));
            }
            return Promise.all(promiseArray);
        })
            .then(function (result) {
            return Promise.resolve(true);
        })
            .catch(function (err) {
            var errorMessage = typeof err === 'string' ? err : (err.message || err.description);
            logger.error('[TipificationAnalysisImplementation::TipificationAnalysisExecute] executionCode:', executionCode, 'getting queueId:', currentQueueId, 'getting error:', errorMessage);
            return Promise.resolve(false);
        });
    };
    TipificationAnalysisImplementation.prototype.EvaluateTipification = function (interaction, currentQueueId, executionCode) {
        var _this = this;
        var phrases = [];
        var abilities = [];
        var evaluationTipificationResults = [];
        return (0, promises_shared_1.DummyPromise)()
            .then(function (_) {
            var promiseArray = [];
            promiseArray.push(_this.GetCognitiveResultsFromFile(interaction, currentQueueId, executionCode));
            promiseArray.push(redis_service_1.Redis.GetEvaluationPlanAbilities(interaction.vcc, interaction.evaluationPlanId));
            return Promise.all(promiseArray);
        })
            .then(function (results) {
            phrases = results[0].phrases;
            abilities = results[1];
            phrases.forEach(function (p) {
                abilities.filter(function (a) { return a.abilityCalificationType === 'QUALITATIVE'; }).forEach(function (a) {
                    a.scores.filter(function (s) { return !s.value; }).forEach(function (s) {
                        s.tipifications.filter(function (t) { var _a, _b; return (_b = (_a = t.tipificationQuery) === null || _a === void 0 ? void 0 : _a.queryId) === null || _b === void 0 ? void 0 : _b.length; }).forEach(function (t) {
                            var _a;
                            var nluName = t.tipificationQuery.queryContent.split('|')[0];
                            var intentName = t.tipificationQuery.queryContent.split('|')[1];
                            var intentId = t.tipificationQuery.queryId;
                            var nluResult = p[nluName] ? p[nluName] : null;
                            if ((_a = nluResult === null || nluResult === void 0 ? void 0 : nluResult.intents) === null || _a === void 0 ? void 0 : _a.length) {
                                var intentResult = nluResult.intents.find(function (i) { return [intentId, intentName].includes(i.intent); });
                                if ((intentResult === null || intentResult === void 0 ? void 0 : intentResult.score) >= t.tipificationQuery.acceptancePercentage) {
                                    var evaluationTipificationResult = new evaluation_result_model_1.EvaluationTipificationResult();
                                    evaluationTipificationResult.abilityId = a.idAbility;
                                    evaluationTipificationResult.scoreId = t.scoreId;
                                    evaluationTipificationResult.id = t.id;
                                    evaluationTipificationResult.isChecked = true;
                                    evaluationTipificationResult.evaluationPlanId = interaction.evaluationPlanId;
                                    evaluationTipificationResult.interactionId = interaction.id;
                                    evaluationTipificationResult.auditorId = 'SPEECH';
                                    evaluationTipificationResults.push(evaluationTipificationResult);
                                    logger.info("[TipificationAnalysisImplementation::EvaluateTipification] executionCode: ".concat(executionCode, ", getting queueId: ").concat(currentQueueId, ", Interaction ").concat(interaction.id, " have got OK for tipification ").concat(t.description, "(").concat(t.id, ") from ability ").concat(a.descriptionAbility, "(").concat(a.idAbility, ") on evaluationPlan ").concat(interaction.evaluationPlanId, " with score ").concat(intentResult.score));
                                }
                            }
                        });
                    });
                });
            });
            interaction['status'] = 7;
            interaction['statusDescription'] = 'ready';
            interaction['retry'] = 0;
            interaction.succedTipifications = evaluationTipificationResults;
            return Promise.resolve(interaction);
        })
            .catch(function (err) {
            var errorMessage = typeof err === 'string' ? err : (err.message || err.description);
            logger.error("[TipificationAnalysisImplementation::EvaluateTipification] executionCode: ".concat(executionCode, ", getting queueId: ").concat(currentQueueId, ", getting error: ").concat(errorMessage));
            interaction['status'] = -7;
            interaction['retry'] = interaction['retry'] ? interaction['retry'] + 1 : 1;
            interaction['uploadedDate'] = moment().format('x');
            interaction['errorMessage'] = errorMessage;
            _this.errorData.SetErrorData('EvaluateTipification', errorMessage, err);
            return Promise.resolve(interaction);
        });
    };
    TipificationAnalysisImplementation.prototype.GetCognitiveResultsFromFile = function (interaction, currentQueueId, executionCode) {
        var _this = this;
        var path = null;
        var regex = /\.(mp3|wav|ogg|json)$/;
        var jsonFileName = interaction.fileName.replace(regex, '.json');
        path = config_1.COGNITIVE_ANALYSIS_FOLDER;
        path += '/';
        path += interaction.vcc;
        path += '/';
        path += jsonFileName;
        return (0, promises_shared_1.DummyPromise)()
            .then(function (_) {
            return (0, file_manager_1.ReadFileContent)(path, 'utf-8', true);
        })
            .then(function (result) {
            var jsonContent = null;
            try {
                jsonContent = typeof result === 'string' ? JSON.parse(result) : result;
            }
            catch (ex) {
                jsonContent = null;
                _this.errorData.SetErrorData('GetCognitiveResultsFromFile', 'ERROR_FILE_JSON', ex);
                throw 'ERROR_FILE_JSON';
            }
            return Promise.resolve(jsonContent);
        })
            .then(function (result) {
            if (!result)
                throw 'ERROR_NOT_FOUND_FILE_JSON';
            return Promise.resolve(result);
        })
            .catch(function (err) {
            var errorMessage = typeof err === 'string' ? err : (err.message || err.description);
            logger.error("[TipificationAnalysisImplementation::GetCognitiveResultsFromFile] executionCode: ".concat(executionCode, " getting queueId: ").concat(currentQueueId, " getting error: ").concat(errorMessage, " (json: ").concat(path, ")"));
            return Promise.reject(err);
        });
    };
    return TipificationAnalysisImplementation;
}());
exports.TipificationAnalysisImplementation = TipificationAnalysisImplementation;
