"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.TranscriptionsDownloadImplementation = void 0;
var log4js = require("log4js");
var mssql_service_1 = require("../db/mssql.service");
var promises_shared_1 = require("../shared/promises.shared");
var config_1 = require("../config/config");
var redis_service_1 = require("../db/redis.service");
var request_shared_1 = require("../shared/request.shared");
var file_manager_1 = require("../shared/file-manager");
var moment = require("moment");
var billing_implementation_1 = require("./billing.implementation");
var process_implementation_1 = require("./process.implementation");
var nlu_service_1 = require("../services/nlu.service");
var logger = log4js.getLogger('Task');
var CURRENT_SYSTEM_SERVICE = 'TRANSCRIPTION';
var CURRENT_IMPLEMENTATION = 'TranscriptionsDownloadImplementation';
var TranscriptionsDownloadImplementation = (function () {
    function TranscriptionsDownloadImplementation() {
    }
    TranscriptionsDownloadImplementation.prototype.Execute = function (executionCode, rateRecursive) {
        var _this = this;
        logger.info("[TranscriptionsDownloadImplementation::Execute] executionCode: ".concat(executionCode, " executing wave #").concat(rateRecursive));
        var executeIndex = 1;
        var ratio = config_1.RATIO_TRANSCRIPTION;
        var alowedRatio = 0;
        var notAlowedRatio = 0;
        var executeResult = true;
        var currentBlockPromise;
        return (0, promises_shared_1.SleepPromise)(config_1.SLEEP_TRANSCRIPTION)
            .then(function (_) {
            currentBlockPromise = 'BLOCK_01_GET_FROM_REDIS';
            return redis_service_1.Redis.LRange('TranscriptionsDownload', 0, ratio - 1);
        })
            .then(function (result) {
            currentBlockPromise = 'BLOCK_02_LAUNCH_TRANSCRIPTIONS_DOWNLOAD_IMPL';
            var promiseArray = [];
            ratio = result && result.length <= ratio ? result.length : 0;
            while (ratio >= executeIndex) {
                promiseArray.push(_this.TranscriptionDownloadExecute(executionCode));
                executeIndex++;
            }
            if (!promiseArray.length)
                promiseArray.push(Promise.resolve(true));
            return Promise.all(promiseArray);
        })
            .then(function (results) {
            if (results.filter(function (item) { return item === false; }).length > notAlowedRatio)
                executeResult = false;
            return redis_service_1.Redis.LRange('TranscriptionsDownload', 0, ratio - 1);
        })
            .then(function (result) {
            if (result && result.length && rateRecursive >= 1)
                return _this.Execute(executionCode, rateRecursive - 1);
            return Promise.resolve(executeResult);
        })
            .then(function (result) {
            if (rateRecursive === config_1.RATE_RECURSIVE_TRANSCRIPTION)
                logger.info("[TranscriptionsDownloadImplementation::Execute] executionCode: ".concat(executionCode, " Finishing execution wave #").concat(rateRecursive));
            var currentResult = executeResult && result ? true : false;
            return Promise.resolve(currentResult);
        }).catch(function (error) {
            var errorMessage = typeof error === 'string' ? error : (error.message || error.description);
            if (errorMessage.startsWith('SERVER_INFO_'))
                logger.info('[TranscriptionsDownloadImplementation::Execute] executionCode:', executionCode, ', Block:', currentBlockPromise, ', getting info:', errorMessage);
            else
                logger.error('[TranscriptionsDownloadImplementation::Execute] executionCode:', executionCode, ', Block:', currentBlockPromise, ', getting error:', errorMessage);
            return Promise.resolve(false);
        });
    };
    TranscriptionsDownloadImplementation.prototype.TranscriptionDownloadExecute = function (executionCode) {
        var _this = this;
        var storage = '';
        var currentQueueId = '';
        var processInteraction;
        var vcc = '';
        var processId = '';
        var interactionType = '';
        var interactionId = '';
        var interactionSection = 0;
        var interactionPart = 0;
        var sentimentNlu;
        var currentBlockPromise = null;
        return redis_service_1.Redis.RPopLPush('TranscriptionsDownload', 'TranscriptionsDownloadExecute')
            .then(function (queueId) {
            currentBlockPromise = 'BLOCK_01_GETTING_INTERACTION_DATA_FROM_REDIS_STORAGE';
            currentQueueId = queueId;
            var item = currentQueueId.split(':');
            if (item === null || item === void 0 ? void 0 : item.length) {
                vcc = item[0];
                processId = item[1];
                interactionType = item[2];
                interactionId = item[3];
                interactionSection = Number(item[4]);
                interactionPart = Number(item[5]);
                storage = vcc + ':' + processId;
                logger.info('[TranscriptionsDownloadImplementation::TranscriptionDownloadExecute] executionCode:', executionCode, 'getting queueId:', currentQueueId, 'storage :', storage);
                return redis_service_1.Redis.GetProcessInteraction(storage, currentQueueId);
            }
            else {
                logger.error('[TranscriptionsDownloadImplementation::TranscriptionDownloadExecute] executionCode:', executionCode, 'getting queueId:', currentQueueId, 'QUEUE_FORMAT_ERROR');
                return Promise.resolve(null);
            }
        })
            .then(function (result) {
            currentBlockPromise = 'BLOCK_02_SETTING_INTERACTION_DATA';
            if (result) {
                processInteraction = result;
                logger.info('[TranscriptionsDownloadImplementation::TranscriptionDownloadExecute] executionCode:', executionCode, 'getting queueId:', currentQueueId, 'getting GetProcessInteraction: 1, status:', processInteraction.statusDescription);
                return Promise.resolve(true);
            }
            else {
                if (currentQueueId)
                    logger.warn('[TranscriptionsDownloadImplementation::TranscriptionDownloadExecute] executionCode:', executionCode, 'getting queueId:', currentQueueId, 'getting GetProcessInteraction: 0');
                return Promise.resolve(null);
            }
        })
            .then(function (_) {
            return (0, nlu_service_1.GetNLU)(vcc, 'sentiment-' + processInteraction.language);
        })
            .then(function (result) {
            if (result)
                sentimentNlu = result;
        })
            .then(function (_) {
            currentBlockPromise = 'BLOCK_03_CREATE_TRANSCRIPTION_FOLDER';
            if (processInteraction)
                return (0, file_manager_1.CreateFolder)("".concat(config_1.TRANSCRIPTION_FOLDER, "/").concat(processInteraction.vcc));
            else
                return Promise.resolve(null);
        })
            .then(function (_) {
            currentBlockPromise = 'BLOCK_04_EXECUTE_MAIN_FUNCTION';
            if (processInteraction) {
                if (processInteraction.statusDescription === 'pendingTranscription') {
                    var validTime = Number(moment().format('x'));
                    var nextExecutionTime = Number(processInteraction.uploadedDate) + (config_1.SPEECHMATICS_CALLBACK_USAGE ? 0 : config_1.NEXT_EXECUTION_TIME_TRANSCRIPTION);
                    if (validTime > nextExecutionTime)
                        return _this.DownloadTranscription(processInteraction, null, currentQueueId, executionCode);
                    else
                        logger.info('[TranscriptionsDownloadImplementation::TranscriptionDownloadExecute] executionCode:', executionCode, 'getting queueId:', currentQueueId, 'NEXT_EXECUTION', nextExecutionTime, 'CURRENT_TIME', validTime);
                }
                else
                    logger.error("[TranscriptionsDownloadImplementation::TranscriptionDownloadExecute] executionCode: ".concat(executionCode, " getting queueId: ").concat(currentQueueId, ", Received invalid status: ").concat(processInteraction.statusDescription));
            }
            return Promise.resolve(null);
        })
            .then(function (result) {
            currentBlockPromise = 'BLOCK_05_SAVING_UPDATED_DATA_ON_REDIS';
            if (processInteraction)
                return redis_service_1.Redis.SaveProcessInteraction(storage, currentQueueId, processInteraction);
            return Promise.resolve(null);
        })
            .then(function (result) {
            currentBlockPromise = 'BLOCK_06_SENDING_TO_NEXT_QUEUE';
            var promiseArray = [];
            var billing = new billing_implementation_1.BillingImplementation();
            if (result
                &&
                    (processInteraction === null || processInteraction === void 0 ? void 0 : processInteraction.status) === 3
                &&
                    ['pendingIdentifyAgent', 'pendingInvertChannels', 'pendingSentimentAnalysis', 'pendingAnalysis'].includes(processInteraction.statusDescription)) {
                logger.info('[TranscriptionsDownloadImplementation::TranscriptionDownloadExecute] executionCode:', executionCode, 'getting queueId:', currentQueueId, 'NEXT_QUEUE');
                var process_1 = new process_implementation_1.ProcessImplementation();
                if (processInteraction.interactionType === 'CALL') {
                    billing.AddUpdateTranscriptionDetailToMainFile(processInteraction, 'ValidDurationTime', processInteraction.batchTypeName, processInteraction.batchTypeId);
                    billing.AddUpdateTranscriptionDetailToMainFile(processInteraction, 'SpeechmaticsApi', processInteraction.batchTypeName, processInteraction.batchTypeId);
                }
                switch (processInteraction.statusDescription) {
                    case 'pendingIdentifyAgent':
                    case 'pendingInvertChannels':
                        promiseArray.push(redis_service_1.Redis.CreateQueueL('DiarizationAnalysis', currentQueueId));
                        break;
                    case 'pendingSentimentAnalysis':
                        promiseArray.push(process_1.RunCurrentBatchesRasaModels(sentimentNlu.vcc, 'TRANSCRIPTION', processInteraction.processId, [sentimentNlu.name], executionCode, currentQueueId));
                        promiseArray.push(redis_service_1.Redis.CreateQueueL('SentimentAnalysis', currentQueueId));
                        break;
                    case 'pendingAnalysis':
                        promiseArray.push(redis_service_1.Redis.CreateQueueL('SemanticAnalysisUpload', currentQueueId));
                        break;
                    default:
                        break;
                }
                promiseArray.push(redis_service_1.Redis.DeleteQueue('TranscriptionsDownloadExecute', currentQueueId));
            }
            else if (result
                &&
                    (processInteraction === null || processInteraction === void 0 ? void 0 : processInteraction.status) === -3
                &&
                    (processInteraction === null || processInteraction === void 0 ? void 0 : processInteraction.statusDescription) === 'REMOVED_BY_CANCELLED') {
                logger.error('[TranscriptionsDownloadImplementation::TranscriptionDownloadExecute] executionCode:', executionCode, 'getting queueId:', currentQueueId, 'REMOVED_BY_CANCELLED');
                billing.AddCloseTranscriptionDetailToMainFile(processInteraction, processInteraction.batchTypeName, processInteraction.batchTypeId, 0, 'REMOVED_BY_CANCELLED');
                promiseArray.push(redis_service_1.Redis.DeleteQueue('TranscriptionsDownloadExecute', currentQueueId));
            }
            else if ((processInteraction === null || processInteraction === void 0 ? void 0 : processInteraction.retry) >= config_1.RETRY_TRANSCRIPTION) {
                logger.error('[TranscriptionsDownloadImplementation::TranscriptionDownloadExecute] executionCode:', executionCode, 'getting queueId:', currentQueueId, 'REMOVED_BY_RETRIES');
                billing.AddCloseTranscriptionDetailToMainFile(processInteraction, processInteraction.batchTypeName, processInteraction.batchTypeId, 0, 'REMOVED_BY_RETRIES');
                promiseArray.push(redis_service_1.Redis.DeleteQueue('TranscriptionsDownloadExecute', currentQueueId));
                processInteraction.retry = config_1.RETRY_TRANSCRIPTION;
            }
            else if (currentQueueId && !processInteraction) {
                logger.error('[TranscriptionsDownloadImplementation::TranscriptionDownloadExecute] executionCode:', executionCode, 'getting queueId:', currentQueueId, 'INTERACTION_DO_NOT_EXIST');
                var interactionData = {
                    vcc: vcc,
                    processId: processId,
                    id: interactionId,
                    section: interactionSection,
                    part: interactionPart
                };
                billing.AddCloseTranscriptionDetailToMainFile(interactionData, processInteraction.batchTypeName, processInteraction.batchTypeId, 0, 'INTERACTION_DO_NOT_EXIST');
                promiseArray.push(redis_service_1.Redis.DeleteQueue('TranscriptionsDownloadExecute', currentQueueId));
            }
            else
                logger.error('[TranscriptionsDownloadImplementation::TranscriptionDownloadExecute] executionCode:', executionCode, 'getting queueId:', currentQueueId, 'QUEUE_WAS_NOT_REMOVED');
            if (!promiseArray.length)
                promiseArray.push(Promise.resolve(true));
            return Promise.all(promiseArray);
        })
            .then(function (result) {
            currentBlockPromise = 'BLOCK_07_UPDATING_PROGRESS';
            var promiseArray = [];
            if ((processInteraction === null || processInteraction === void 0 ? void 0 : processInteraction.retry) >= config_1.RETRY_TRANSCRIPTION) {
                processInteraction.isCancel = true;
                promiseArray.push(redis_service_1.Redis.UpdateProgress(storage));
                promiseArray.push(redis_service_1.Redis.UpdateErrorProgress(storage));
                promiseArray.push(redis_service_1.Redis.DeleteProcessInteraction(storage, currentQueueId));
                promiseArray.push(redis_service_1.Redis.SaveReadyProcessInteraction(executionCode, currentQueueId, processInteraction));
                promiseArray.push(mssql_service_1.MSSql.SaveBatchError(vcc, CURRENT_SYSTEM_SERVICE, processInteraction.batchTypeId, processInteraction.batchTypeId, CURRENT_IMPLEMENTATION, processInteraction.id, null, processInteraction.errorFunction, processInteraction.errorMessage, processInteraction.errorDetail));
            }
            if (!promiseArray.length)
                promiseArray.push(Promise.resolve(true));
            return Promise.all(promiseArray);
        })
            .then(function (_) {
            currentBlockPromise = 'BLOCK_08_FINISHING';
            return Promise.resolve(true);
        })
            .catch(function (err) {
            var errorMessage = typeof err === 'string' ? err : (err.message || err.description);
            logger.error('[TranscriptionsDownloadImplementation::TranscriptionDownloadExecute] executionCode:', executionCode, 'getting queueId:', currentQueueId, ', Block:', currentBlockPromise, ', getting error:', errorMessage);
            return Promise.resolve(false);
        });
    };
    TranscriptionsDownloadImplementation.prototype.DownloadTranscription = function (interaction, content, currentQueueId, executionCode) {
        var _this = this;
        if (content === void 0) { content = null; }
        var retry = false;
        var canceled = false;
        var empty = false;
        return (0, promises_shared_1.DummyPromise)()
            .then(function (_) {
            if (content)
                return Promise.resolve(content);
            else
                return _this.DownloadSpeechMatics(interaction, currentQueueId, executionCode);
        })
            .then(function (result) {
            if ((result === null || result === void 0 ? void 0 : result.statusDescription) === 'jobInProgress') {
                retry = true;
                return Promise.resolve(null);
            }
            else if ((result === null || result === void 0 ? void 0 : result.statusDescription) === 'jobNotFound') {
                retry = false;
                canceled = true;
                return Promise.resolve(null);
            }
            else
                logger.warn("[TranscriptionsDownloadImplementation::DownloadTranscription] executionCode: ".concat(executionCode, " getting queueId: ").concat(currentQueueId, " unknown speechmatics status: ").concat((result === null || result === void 0 ? void 0 : result.statusDescription) || null));
            var regex = /\.(mp3|wav|ogg)$/;
            var jsonFileName = interaction.fileName.replace(regex, '.json');
            result.speechmaticsApi = interaction.speechmaticsApi;
            var jsonContent = JSON.stringify(result);
            var targetLocation = "".concat(config_1.TRANSCRIPTION_FOLDER, "/").concat(interaction.vcc, "/").concat(jsonFileName);
            return _this.SaveTranscriptionJSONFile(targetLocation, jsonContent, currentQueueId, executionCode);
        })
            .then(function (result) {
            var _a, _b;
            var jobStatus = null;
            if (result) {
                try {
                    jobStatus = JSON.parse(result);
                    var validChar_1 = 0;
                    if ((_a = jobStatus === null || jobStatus === void 0 ? void 0 : jobStatus.results) === null || _a === void 0 ? void 0 : _a.length) {
                        jobStatus.results.forEach(function (r) {
                            var _a;
                            if ((_a = r.alternatives) === null || _a === void 0 ? void 0 : _a.length) {
                                validChar_1 += r.alternatives[0].content ? r.alternatives[0].content.length + 1 : 0;
                            }
                        });
                    }
                    else {
                        empty = true;
                        jobStatus = null;
                    }
                    interaction['durationLinguisticTime'] = ((_b = jobStatus === null || jobStatus === void 0 ? void 0 : jobStatus.job) === null || _b === void 0 ? void 0 : _b.duration) ? jobStatus.job.duration : null;
                    interaction['validChar'] = validChar_1;
                    interaction['validWords'] = (jobStatus === null || jobStatus === void 0 ? void 0 : jobStatus.results) ? jobStatus.results.length : 0;
                }
                catch (err) {
                    jobStatus = null;
                }
            }
            return Promise.resolve(jobStatus);
        })
            .then(function (result) {
            if (result) {
                interaction['status'] = 3;
                if (interaction.channelchange && interaction.isDiariced !== true)
                    interaction['statusDescription'] = 'pendingInvertChannels';
                else
                    interaction['statusDescription'] = interaction.isDiariced === true ? 'pendingIdentifyAgent' : (config_1.USAGE_SENTIMENT_ANALYSIS ? 'pendingSentimentAnalysis' : 'pendingAnalysis');
                interaction['retry'] = 0;
                interaction['uploadedDate'] = moment().format('x');
                interaction['errorFunction'] = null;
                interaction['errorMessage'] = null;
                interaction['errorDetail'] = null;
            }
            else if (retry) {
                interaction['retry'] = interaction['retry'] ? interaction['retry'] + 1 : 1;
                interaction['uploadedDate'] = moment().format('x');
                interaction['errorFunction'] = 'DownloadTranscription';
                interaction['errorMessage'] = 'JOB_IN_PROGRESS';
                interaction['errorDetail'] = 'JOB_IN_PROGRESS';
            }
            else if (empty) {
                interaction['status'] = -3;
                interaction['retry'] = config_1.RETRY_TRANSCRIPTION;
                interaction['uploadedDate'] = moment().format('x');
                interaction['errorFunction'] = 'DownloadSpeechMatics';
                interaction['errorFunction'] = 'DownloadTranscription';
                interaction['errorMessage'] = 'JSON_TRANSCRIPTION_CONTENT_IS_EMPTY';
                interaction['errorDetail'] = 'JSON_TRANSCRIPTION_CONTENT_IS_EMPTY';
            }
            else if (canceled) {
                interaction['status'] = -3;
                interaction['retry'] = config_1.RETRY_TRANSCRIPTION;
                interaction['uploadedDate'] = moment().format('x');
                interaction['errorFunction'] = 'DownloadTranscription';
                interaction['errorMessage'] = 'JOB_EXPIRED';
                interaction['errorDetail'] = 'JOB_EXPIRED';
            }
            else {
                logger.error('[TranscriptionsDownloadImplementation::DownloadTranscription] executionCode:', executionCode, 'getting queueId:', currentQueueId, 'DOWNLOAD_ERROR');
                interaction['status'] = -3;
                interaction['retry'] = interaction['retry'] ? interaction['retry'] + 1 : 1;
                interaction['uploadedDate'] = moment().format('x');
                interaction['errorFunction'] = 'DownloadTranscription';
                interaction['errorMessage'] = 'DOWNLOAD_ERROR';
                interaction['errorDetail'] = 'DOWNLOAD_ERROR';
            }
            return Promise.resolve(interaction);
        })
            .catch(function (err) {
            var errorMessage = typeof err === 'string' ? err : (err.message || err.description);
            logger.error('[TranscriptionsDownloadImplementation::DownloadTranscription] executionCode:', executionCode, 'getting queueId:', currentQueueId, 'getting error:', errorMessage);
            interaction['status'] = -3;
            interaction['retry'] = interaction['retry'] ? interaction['retry'] + 1 : 1;
            interaction['uploadedDate'] = moment().format('x');
            interaction['errorFunction'] = 'DownloadTranscription';
            interaction['errorMessage'] = errorMessage;
            interaction['errorDetail'] = JSON.stringify(err);
            return Promise.resolve(interaction);
        });
    };
    TranscriptionsDownloadImplementation.prototype.DownloadSpeechMatics = function (interaction, currentQueueId, executionCode) {
        var options = {
            uri: "http://".concat(interaction.speechmaticsApi, "/v2/jobs/").concat(interaction.jobId, "/transcript?format=json-v2"),
            retryProtocol: false,
            validateErrorResponse: true,
            timeout: 30000
        };
        var data;
        return (0, request_shared_1.SimpleGetRequest)(options)
            .then(function (result) {
            data = result;
            return Promise.resolve(result);
        })
            .catch(function (err) {
            var errorMessage = typeof err === 'string' ? err : (err.message || err.description);
            var pending = {};
            if (typeof errorMessage === 'string'
                &&
                    (errorMessage.indexOf('Job In Progress') > -1
                        ||
                            errorMessage.indexOf('Transcription not ready') > -1)) {
                logger.warn('[TranscriptionsDownloadImplementation::DownloadSpeechMatics] executionCode:', executionCode, 'getting queueId:', currentQueueId, 'getting error:', errorMessage);
                pending['statusDescription'] = 'jobInProgress';
            }
            else if (typeof errorMessage === 'string'
                &&
                    (errorMessage.indexOf('Job not found') > -1
                        ||
                            errorMessage.includes('Job rejected'))) {
                logger.warn('[TranscriptionsDownloadImplementation::DownloadSpeechMatics] executionCode:', executionCode, 'getting queueId:', currentQueueId, 'getting error:', errorMessage);
                pending['statusDescription'] = 'jobNotFound';
            }
            else {
                logger.error('[TranscriptionsDownloadImplementation::DownloadSpeechMatics] executionCode:', executionCode, 'getting queueId:', currentQueueId, 'getting error:', errorMessage);
            }
            data = (pending === null || pending === void 0 ? void 0 : pending.statusDescription) ? pending : null;
            return Promise.resolve(data);
        })
            .catch(function (err) {
            var errorMessage = typeof err === 'string' ? err : (err.message || err.description);
            logger.error('[TranscriptionsDownloadImplementation::DownloadSpeechMatics] executionCode:', executionCode, 'getting queueId:', currentQueueId, 'getting error:', errorMessage);
            return Promise.resolve(null);
        });
    };
    TranscriptionsDownloadImplementation.prototype.SaveTranscriptionJSONFile = function (fileName, data, currentQueueId, executionCode) {
        return (0, file_manager_1.WriteFileContent)(fileName, data, 'utf-8', true)
            .then(function (result) {
            return Promise.resolve(data);
        })
            .catch(function (error) {
            var errorMessage = typeof error === 'string' ? error : (error.message || error.description);
            logger.error('[TranscriptionsDownloadImplementation::SaveTranscriptionJSONFile] executionCode:', executionCode, 'getting queueId:', currentQueueId, 'getting error:', errorMessage);
            return Promise.resolve(null);
        });
    };
    return TranscriptionsDownloadImplementation;
}());
exports.TranscriptionsDownloadImplementation = TranscriptionsDownloadImplementation;
