"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.TranscriptionsStandbyImplementation = void 0;
var config_1 = require("../config/config");
var redis_service_1 = require("../db/redis.service");
var log4js = require("log4js");
var promises_shared_1 = require("../shared/promises.shared");
var request_shared_1 = require("../shared/request.shared");
var moment = require("moment");
var logger = log4js.getLogger('Task');
var CURRENT_SYSTEM_SERVICE = 'TRANSCRIPTION';
var CURRENT_IMPLEMENTATION = 'TranscriptionsStandbyImplementation';
var TranscriptionsStandbyImplementation = (function () {
    function TranscriptionsStandbyImplementation() {
    }
    TranscriptionsStandbyImplementation.prototype.Execute = function (executionCode, rateRecursive) {
        var _this = this;
        logger.info("[TranscriptionsStandbyImplementation::Execute] executionCode: ".concat(executionCode, " executing wave #").concat(rateRecursive));
        var executeIndex = 1;
        var ratio = config_1.RATIO_TRANSCRIPTION_STANDBY;
        var executeResult = true;
        var currentBlockPromise;
        return (0, promises_shared_1.SleepPromise)(config_1.SLEEP_TRANSCRIPTION)
            .then(function (_) {
            currentBlockPromise = 'BLOCK_01_CHECK_IF_IS_EXECUTION_TIME';
            var available = false;
            var currentMoment = moment();
            if ((currentMoment.minute() % 10) === 1)
                available = true;
            if (!available)
                throw 'SERVER_INFO_WAITING_FOR_TIME_TO_EXECUTE';
            return Promise.resolve(true);
        })
            .then(function (_) {
            currentBlockPromise = 'BLOCK_02_GETTING_SPEECHMATICS_JOBS';
            return _this.GetCurrentJobs(executionCode);
        })
            .then(function (_) {
            currentBlockPromise = 'BLOCK_03_GET_FROM_REDIS';
            return redis_service_1.Redis.LRange('TranscriptionsStandby', 0, ratio - 1);
        })
            .then(function (result) {
            currentBlockPromise = 'BLOCK_04_LAUNCH_TRANSCRIPTIONS_STANDBY_IMPL';
            var promiseArray = [];
            ratio = (result === null || result === void 0 ? void 0 : result.length) <= ratio ? result.length : 0;
            while (ratio >= executeIndex) {
                promiseArray.push(_this.TranscriptionsStandbyExecute(executionCode));
                executeIndex++;
            }
            if (!promiseArray.length)
                promiseArray.push(Promise.resolve(true));
            return Promise.all(promiseArray);
        })
            .then(function (results) {
            currentBlockPromise = 'BLOCK_05_GET_FROM_REDIS_AGAIN';
            if (results.filter(function (item) { return item === false; }).length)
                executeResult = false;
            return redis_service_1.Redis.LRange('TranscriptionsStandby', 0, ratio - 1);
        })
            .then(function (result) {
            currentBlockPromise = 'BLOCK_06_EXECUTING_RECURSIVITY';
            if ((result === null || result === void 0 ? void 0 : result.length) && rateRecursive >= 1)
                return _this.Execute(executionCode, rateRecursive - 1);
            return Promise.resolve(executeResult);
        })
            .then(function (result) {
            currentBlockPromise = 'BLOCK_07_FINISHING';
            if (rateRecursive === config_1.RATE_RECURSIVE_TRANSCRIPTION_STANDBY)
                logger.info("[TranscriptionsStandbyImplementation::Execute] executionCode: ".concat(executionCode, " Finishing execution wave #").concat(rateRecursive));
            redis_service_1.Redis.Del("CurrentJobs:".concat(executionCode));
            var currentResult = executeResult && result ? true : false;
            return Promise.resolve(currentResult);
        })
            .catch(function (error) {
            var errorMessage = typeof error === 'string' ? error : (error.message || error.description);
            if (errorMessage === 'SERVER_INFO_WAITING_FOR_TIME_TO_EXECUTE')
                logger.info('[TranscriptionsStandbyImplementation::Execute] executionCode:', executionCode, 'getting info:', errorMessage);
            else
                logger.error('[TranscriptionsStandbyImplementation::Execute] executionCode:', executionCode, 'getting error:', errorMessage);
            redis_service_1.Redis.Del("CurrentJobs:".concat(executionCode));
            return Promise.resolve(false);
        });
    };
    TranscriptionsStandbyImplementation.prototype.TranscriptionsStandbyExecute = function (executionCode) {
        var _this = this;
        var storage = '';
        var currentQueueId = '';
        var processInteraction;
        var vcc = '';
        var processId = '';
        var interactionType = '';
        var interactionId = '';
        var interactionSection = 0;
        var interactionPart = 0;
        var currentBlockPromise = null;
        return redis_service_1.Redis.RPopLPush('TranscriptionsStandby', 'TranscriptionsStandbyExecute')
            .then(function (queueId) {
            currentBlockPromise = 'BLOCK_01_GETTING_INTERACTION_DATA_FROM_REDIS_STORAGE';
            currentQueueId = queueId;
            var item = currentQueueId.split(':');
            if (item === null || item === void 0 ? void 0 : item.length) {
                vcc = item[0];
                processId = item[1];
                interactionType = item[2];
                interactionId = item[3];
                interactionSection = Number(item[4]);
                interactionPart = Number(item[5]);
                storage = vcc + ':' + processId;
                logger.info('[TranscriptionsStandbyImplementation::TranscriptionsStandbyExecute] executionCode:', executionCode, 'getting queueId:', currentQueueId, 'storage :', storage);
                return redis_service_1.Redis.GetProcessInteraction(storage, currentQueueId);
            }
            else {
                logger.error('[TranscriptionsStandbyImplementation::TranscriptionsStandbyExecute] executionCode:', executionCode, 'getting queueId:', currentQueueId, 'QUEUE_FORMAT_ERROR');
                return Promise.resolve(null);
            }
        })
            .then(function (result) {
            currentBlockPromise = 'BLOCK_02_SETTING_INTERACTION_DATA';
            if (result) {
                processInteraction = result;
                logger.info('[TranscriptionsStandbyImplementation::TranscriptionsStandbyExecute] executionCode:', executionCode, 'getting queueId:', currentQueueId, 'getting GetProcessInteraction: 1, status:', processInteraction.statusDescription);
                return _this.CheckCurrentQueue(executionCode, currentQueueId, processInteraction);
            }
            else {
                if (currentQueueId)
                    logger.warn('[TranscriptionsStandbyImplementation::TranscriptionsStandbyExecute] executionCode:', executionCode, 'getting queueId:', currentQueueId, 'getting GetProcessInteraction: 0');
                return Promise.resolve(null);
            }
        })
            .then(function (result) {
            currentBlockPromise = 'BLOCK_03_REINSERTING_TO_DOWNLOAD_QUEUE';
            if (result)
                return redis_service_1.Redis.DeleteInsertOnList('TranscriptionsStandbyExecute', 'TranscriptionsDownload', currentQueueId);
            return Promise.resolve(null);
        })
            .then(function (result) {
            currentBlockPromise = 'BLOCK_04_FINISHING';
            return Promise.resolve(true);
        })
            .catch(function (err) {
            var errorMessage = typeof err === 'string' ? err : (err.message || err.description);
            logger.error('[TranscriptionsStandbyImplementation::TranscriptionDownloadExecute] executionCode:', executionCode, 'getting queueId:', currentQueueId, ', Block:', currentBlockPromise, ', getting error:', errorMessage);
            return Promise.resolve(false);
        });
    };
    TranscriptionsStandbyImplementation.prototype.GetCurrentJobs = function (executionCode) {
        var _this = this;
        var speechmaticsServers = config_1.SPEECHMATICS_SERVERS;
        return (0, promises_shared_1.DummyPromise)()
            .then(function (_) {
            var promiseArray = [];
            speechmaticsServers.forEach(function (ss) { return promiseArray.push(_this.GetCurrentJob(executionCode, ss.ip, ss.port)); });
            return Promise.all(promiseArray);
        })
            .then(function (result) {
            return Promise.resolve(true);
        })
            .catch(function (err) {
            var errorMessage = typeof err === 'string' ? err : (err.message || err.description);
            logger.error('[TranscriptionsStandbyImplementation::GetCurrentJobs] executionCode:', executionCode, 'getting error:', errorMessage);
            return Promise.resolve(false);
        });
    };
    TranscriptionsStandbyImplementation.prototype.GetCurrentJob = function (executionCode, ip, port) {
        var api = "".concat(ip, ":").concat(port);
        return (0, promises_shared_1.DummyPromise)()
            .then(function (_) {
            var options = {
                uri: "http://".concat(api, "/v2/jobs/"),
                retryProtocol: false,
                validateErrorResponse: true,
                timeout: 30000
            };
            return (0, request_shared_1.SimpleGetRequest)(options);
        })
            .then(function (result) {
            var _a;
            if (!((_a = result === null || result === void 0 ? void 0 : result.jobs) === null || _a === void 0 ? void 0 : _a.length))
                throw 'SERVER_ERROR_SPEECHMATICS_JOBS_NOT_FOUND';
            return redis_service_1.Redis.SaveCurrentJob(executionCode, api, result);
        })
            .catch(function (err) {
            var errorMessage = typeof err === 'string' ? err : (err.message || err.description);
            logger.error('[TranscriptionsStandbyImplementation::GetCurrentJob] executionCode:', executionCode, 'getting error:', errorMessage);
            return Promise.resolve(false);
        });
    };
    TranscriptionsStandbyImplementation.prototype.CheckCurrentQueue = function (executionCode, currentQueueId, processInteraction) {
        return (0, promises_shared_1.DummyPromise)()
            .then(function (_) {
            return redis_service_1.Redis.GetCurrentJob(executionCode, processInteraction.speechmaticsApi);
        })
            .then(function (result) {
            if (!result)
                throw 'SERVER_ERROR_JOBS_STORAGE_NOT_FOUND';
            var jobs = result.jobs;
            if (jobs.find(function (j) { return j.id === processInteraction.jobId; })) {
                var currentJob = jobs.find(function (j) { return j.id === processInteraction.jobId; });
                if (currentJob.status === 'done') {
                    return Promise.resolve(true);
                }
                else {
                    logger.info("[TranscriptionsStandbyImplementation::CheckCurrentQueue] executionCode: ".concat(executionCode, " queueId ").concat(currentQueueId, " status: ").concat(currentJob.job_status));
                    return Promise.resolve(false);
                }
            }
            logger.warn("[TranscriptionsStandbyImplementation::CheckCurrentQueue] executionCode: ".concat(executionCode, " interaction ").concat(currentQueueId, " not found on jobs"));
            return Promise.resolve(false);
        })
            .catch(function (err) {
            var errorMessage = typeof err === 'string' ? err : (err.message || err.description);
            logger.error('[TranscriptionsStandbyImplementation::CheckCurrentQueue] executionCode:', executionCode, 'getting error:', errorMessage);
            return Promise.resolve(false);
        });
    };
    return TranscriptionsStandbyImplementation;
}());
exports.TranscriptionsStandbyImplementation = TranscriptionsStandbyImplementation;
