"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.MailDisconect = exports.SendAttach = exports.Send = exports.PrepareEmailTemplate = exports.SmtpService = void 0;
var log4js = require("log4js");
var mail_config_1 = require("../config/mail.config");
var server_config_1 = require("../config/server.config");
var file_manager_1 = require("../shared/file-manager");
var templates_config_1 = require("../config/templates.config");
var redis_service_1 = require("../db/redis.service");
var mssql_service_1 = require("../db/mssql.service");
var logger = log4js.getLogger('ServerScripts');
var hbs = require('hbs');
var mailServer;
function SmtpService(app) {
    mailServer = app.get('mailserver');
}
exports.SmtpService = SmtpService;
function PrepareEmailTemplate(instance, userName, description, title, template) {
    var assetsUrl;
    return GetMailAssetsDomain(instance)
        .then(function (result) {
        assetsUrl = result;
        return (0, file_manager_1.ReadFileContent)((0, file_manager_1.JoinPath)(templates_config_1.TEMPLATE_MAIL_PATH, template + '.hbs'), 'utf8');
    }).then(function (content) {
        var theme = hbs.compile(content);
        var resultContent = '';
        switch (template) {
            case 'notification':
                resultContent = theme({
                    'userName': userName,
                    'description': description,
                    'title': title,
                    'assetsUrl': assetsUrl
                });
                break;
            default:
                resultContent = '';
                break;
        }
        return resultContent;
    }).catch(function (err) {
        logger.error('PrepareEmailTemplate: ' + err);
    });
}
exports.PrepareEmailTemplate = PrepareEmailTemplate;
function Send(receiver, subject, content) {
    var mailOptions = {
        from: mail_config_1.MAILSERVER_CONFIG.from,
        to: receiver,
        subject: subject,
        text: content,
        html: content
    };
    return new Promise(function (resolve, reject) {
        mailServer.sendMail(mailOptions, function (error, info) {
            if (error) {
                console.log('Error al intentar enviar por MailService: \'' + error + '\'');
                logger.error('Error al intentar enviar por MailService: \'' + error + '\'');
                reject(error);
            }
            else {
                logger.info('Mensaje enviado por MailService: \'' + info.response + '\'');
                resolve(true);
            }
        });
    });
}
exports.Send = Send;
function SendAttach(receiver, subject, content, attachments) {
    var mailOptions = {
        from: mail_config_1.MAILSERVER_CONFIG.from,
        to: receiver,
        subject: subject,
        attachments: attachments,
        text: content,
        html: content
    };
    return new Promise(function (resolve, reject) {
        mailServer.sendMail(mailOptions, function (error, info) {
            if (error) {
                console.log('Error al intentar enviar por MailService: \'' + error + '\'');
                logger.error('Error al intentar enviar por MailService: \'' + error + '\'');
                reject(error);
            }
            else {
                logger.info('Mensaje enviado por MailService: \'' + info.response + '\'');
                resolve(true);
            }
        });
    });
}
exports.SendAttach = SendAttach;
function MailDisconect() {
    return new Promise(function (resolve, reject) {
        if (mailServer) {
            logger.info('Cerrando conexión a webmail \'' + mail_config_1.MAILSERVER_CONFIG.server + '\'');
            console.log('Cerrando conexión a webmail \'' + mail_config_1.MAILSERVER_CONFIG.server + '\'');
            mailServer.close();
        }
        resolve(true);
    });
}
exports.MailDisconect = MailDisconect;
function GetMailAssetsDomain(instance) {
    var mailAssetsUrl = (server_config_1.HTTPS_ENABLED ? 'https://' : 'http://') + '127.0.0.1' +
        (server_config_1.HTTPS_ENABLED && server_config_1.HTTPS_BINDING_PORT && server_config_1.HTTPS_BINDING_PORT !== 443 ? ':' + server_config_1.HTTPS_BINDING_PORT.toString() :
            (!server_config_1.HTTPS_ENABLED && server_config_1.HTTP_BINDING_PORT && server_config_1.HTTP_BINDING_PORT !== 80 ? ':' + server_config_1.HTTP_BINDING_PORT.toString() : '')) + '/client/assets/img/';
    var cacheUpdateNeeded = false;
    return redis_service_1.Redis.GetMailingDomainByInstance(instance)
        .then(function (mailingDomain) {
        if (mailingDomain || JSON.stringify(mailingDomain) === JSON.stringify({})) {
            return Promise.resolve(mailingDomain);
        }
        else {
            logger.info('El dato de GetMailingDomainByInstance para \'' + instance + '\' no se encuentra en cache. se consultara a base de datos.');
            cacheUpdateNeeded = true;
            return mssql_service_1.MSSql.GetDomains(instance, 'domain', '');
        }
    }).then(function (resultDomains) {
        if (typeof resultDomains !== 'string') {
            resultDomains = resultDomains || [];
            var selectedDomain = resultDomains.find(function (domain) { return domain.isMainDomain; });
            if (!selectedDomain)
                selectedDomain = resultDomains.find(function (domain) { return !domain.isContentDomain; });
            if (selectedDomain) {
                var protocol = (server_config_1.HTTPS_ENABLED ? 'https://' : 'http://');
                var domain = selectedDomain.domain || '127.0.0.1';
                var port = (((!server_config_1.HTTPS_ENABLED && selectedDomain.contentPort && selectedDomain.contentPort !== '80') || (server_config_1.HTTPS_ENABLED && selectedDomain.contentPort && selectedDomain.contentPort !== '443')) ? ':' + selectedDomain.contentPort : '');
                mailAssetsUrl = protocol + domain + port + '/assets/img/';
            }
        }
        else {
            mailAssetsUrl = resultDomains;
        }
        logger.info('Devuelve la ruta para assets de mailing de la instancia  \'' + instance + ' con valor \'' + mailAssetsUrl + '\'');
        if (cacheUpdateNeeded)
            redis_service_1.Redis.SaveMailingDomainByInstance(instance, mailAssetsUrl);
        if (mailAssetsUrl || JSON.stringify(mailAssetsUrl) === JSON.stringify({})) {
            return Promise.resolve(mailAssetsUrl);
        }
        else {
            throw 'DOES_NOT_EXIST_IN_DATABASE';
        }
    }).catch(function (err) {
        var errMsg = (typeof err === 'string' ? err : err.message || err.description || 'SERVER_ERROR_UNKNOWN_MESSAGE');
        logger.error('Error al ejecutar GetMailAssetsDomain: ' + errMsg);
    });
}
