"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.CampainService = void 0;
var log4js = require("log4js");
var mssql_service_1 = require("../db/mssql.service");
var promises_shared_1 = require("../shared/promises.shared");
var integration_config_1 = require("../config/integration.config");
var mssql_config_1 = require("../config/mssql.config");
var http_request_occ_1 = require("../shared/http-request-occ");
var logger = log4js.getLogger('ServerScripts');
function CampainService(app) {
    app.get('/api/campaign/get_campaigns/:search_criteria/:search_expression?', function (req, res, next) {
        logger.info('[CampainService::get_campaigns] Received ' + req.method + ' method on \'' + req.path + '\'');
        var instance = res.locals.instance;
        var type = res.locals.type;
        var campaigns = [];
        var allegroCampaign = [];
        var search_criteria = req.params.search_criteria || '';
        var search_expression = req.params.search_expression || '';
        type = search_criteria === 'HISTORIC' ? search_criteria : type;
        (0, promises_shared_1.DummyPromise)()
            .then(function (result) {
            var promiseArray = [];
            switch (type) {
                case 'ALLEGRO':
                    var flagAllegro = integration_config_1.IP_BD === mssql_config_1.MSSQL_CONFIG.server ? 1 : 0;
                    promiseArray.push(mssql_service_1.MSSql.GetAllegroCampaigns(integration_config_1.IP_BD, instance, flagAllegro));
                    break;
                case 'I6':
                    promiseArray.push((0, http_request_occ_1.ApiI6SearchCampaigns)(instance));
                    break;
                case 'OCC':
                    var flagOCC = integration_config_1.IP_BD === mssql_config_1.MSSQL_CONFIG.server ? 1 : 0;
                    promiseArray.push(mssql_service_1.MSSql.GetAllegroCampaigns(integration_config_1.IP_BD, instance, flagOCC));
                    if (!integration_config_1.USE_OCC_WITH_ALLEGRO) {
                        promiseArray.push((0, http_request_occ_1.ApiI6SearchCampaigns)(instance));
                    }
                    break;
                case 'OTHER':
                    break;
                case 'HISTORIC':
                    var excludesworkgroupsup = search_expression == "ADD" ? true : false;
                    promiseArray.push(mssql_service_1.MSSql.GetHistoricCampaigns(instance, excludesworkgroupsup));
                    break;
            }
            return Promise.all(promiseArray);
        })
            .then(function (result) {
            if (search_criteria == 'HISTORIC') {
                if (!result[0])
                    throw 'SERVER_ERROR_GET_SQL_CAMPAIGNS_FAILED';
                campaigns = result[0];
            }
            else {
                switch (type) {
                    case 'ALLEGRO':
                        if (!result[0])
                            throw 'SERVER_ERROR_GET_SQL_CAMPAIGNS_FAILED';
                        campaigns = result[0];
                        break;
                    case 'I6':
                        if ((!result[0]) || (result[0] && result[0].status === false))
                            throw 'SERVER_ERROR_GET_API_I6_CAMPAIGNS_FAILED';
                        campaigns = result[0].body;
                        break;
                    case 'OCC':
                        if (!result[0]
                            &&
                                ((!result[1]) || (result[1] && result[1].status === false)))
                            throw 'SERVER_ERROR_GET_CAMPAIGNS_FAILED';
                        var allegroCampaigns = result[0] ? result[0] : [];
                        var campaignsApi_1;
                        if (result[1] && result[1].status === true) {
                            campaignsApi_1 = result[1] && result[1].status ? result[1].body : [];
                            campaigns = allegroCampaigns.filter(function (c) { return campaignsApi_1.find(function (ca) { return ca.id === c.id; }); });
                        }
                        else {
                            campaigns = allegroCampaigns;
                        }
                        break;
                    case 'OTHER':
                        break;
                }
            }
            if (res.headersSent)
                return;
            res.json({ status: true, description: 'SERVER_SUCCESS_MESSAGE', data: campaigns });
        }).catch(function (err) {
            var clientMsg = (typeof err === 'string' ? err : 'CANNOT_PROCESS_REQUEST');
            var errorMsg = (typeof err === 'string' ? err : err.message || err.description || 'SERVER_ERROR_UNKNOWN_MESSAGE');
            logger.error('Error al procesar \'' + req.path + '\': ' + errorMsg);
            if (res.headersSent)
                return;
            res.json({ status: false, description: clientMsg, error: errorMsg });
        });
    });
    app.post('/api/campaign/evaluation', function (req, res, next) {
        logger.info('[CampainService::evaluation] Received ' + req.method + ' method on \'' + req.path + '\'');
        var vcc = res.locals.instance;
        var type = res.locals.type;
        var userId = res.locals.userId;
        var referrer = res.locals.pageReferrer.split('/').pop();
        var searchType = null;
        var campaigns = req.body.campaigns ? req.body.campaigns : [];
        switch (referrer) {
            case 'auditorinbox':
                searchType = 'PENDING';
                break;
            case 'evaluation':
                searchType = 'HISTORY';
                break;
            case 'evaluation-agent':
                searchType = 'HISTORY';
                break;
            case 'evaluation-admin':
                searchType = 'HISTORY';
                break;
        }
        (0, promises_shared_1.DummyPromise)()
            .then(function (result) {
            return mssql_service_1.MSSql.GetCampaignsFromEvaluation(vcc, userId, searchType);
        }).then(function (result) {
            if (!result)
                throw 'SERVER_ERROR_CAMPAIGNS_SEARCH_FAILED';
            if (res.headersSent)
                return;
            res.json({ status: true, description: 'SERVER_SUCCESS_MESSAGE', data: result });
        }).catch(function (err) {
            var clientMsg = (typeof err === 'string' ? err : 'CANNOT_PROCESS_REQUEST');
            var errorMsg = (typeof err === 'string' ? err : err.message || err.description || 'SERVER_ERROR_UNKNOWN_MESSAGE');
            logger.error('[CampainService::evaluation] Error on \'' + req.path + '\': ' + errorMsg);
            if (res.headersSent)
                return;
            res.json({ status: false, description: clientMsg, error: errorMsg });
        });
    });
    app.get('/api/campaign/get_campaigns_evaluation_format/:idEvaluationFormat', function (req, res, next) {
        logger.info('Se recibió ' + req.method + ' para \'' + req.path + '\'');
        var instance = res.locals.instance;
        var idEvaluationFormat = req.params.idEvaluationFormat || '';
        (0, promises_shared_1.DummyPromise)()
            .then(function (result) {
            return mssql_service_1.MSSql.GetCampaignsEvaluationFormat(instance, idEvaluationFormat);
        }).then(function (result) {
            if (res.headersSent)
                return;
            res.json({ status: true, description: 'SERVER_SUCCESS_MESSAGE', data: result });
        }).catch(function (err) {
            var campainMsg = (typeof err === 'string' ? err : 'CANNOT_PROCESS_REQUEST');
            var errorMsg = (typeof err === 'string' ? err : err.message || err.description || 'SERVER_ERROR_UNKNOWN_MESSAGE');
            logger.error('Error al procesar \'' + req.path + '\': ' + errorMsg);
            if (res.headersSent)
                return;
            res.json({ status: false, description: campainMsg, error: errorMsg });
        });
    });
    app.get('/api/campaign/get_campaign_qualification_average/:campaignId', function (req, res, next) {
        logger.info('[CampainService::get_campaign_qualification_average] Received ' + req.method + ' method on \'' + req.path + '\'');
        var instance = res.locals.instance;
        var campaignStr = req.params.campaignId || '';
        logger.info('[CampainService::get_campaign_qualification_average] Received instance: ' + instance);
        logger.info('[CampainService::get_campaign_qualification_average] Received campaignId: ' + campaignStr);
        (0, promises_shared_1.DummyPromise)()
            .then(function (result) {
            logger.info('[CampainService::get_campaign_qualification_average] Calling GetCampaignQualificationAverage SP');
            return mssql_service_1.MSSql.GetCampaignQualificationAverage(instance, campaignStr);
        }).then(function (result) {
            logger.info('[CampainService::get_campaign_qualification_average] Received result from GetCampaignQualificationAverage SP: ' + JSON.stringify(result));
            if (!result) { }
            if (res.headersSent)
                return;
            res.json({ status: true, description: 'SERVER_SUCCESS_MESSAGE', data: result });
        }).catch(function (err) {
            var clientMsg = (typeof err === 'string' ? err : 'CANNOT_PROCESS_REQUEST');
            var errorMsg = (typeof err === 'string' ? err : err.message || err.description || 'Error requesting');
            logger.error('[CampainService::get_campaign_qualification_average] Error on \'' + req.path + '\': ' + errorMsg);
            if (res.headersSent)
                return;
            res.json({ status: false, description: clientMsg, error: errorMsg });
        });
    });
    app.get('/api/campaign/interaction/', function (req, res) {
        logger.info('[NLUService::campaigns_interaction] Receives ' + req.method + ' method on \'' + req.path + '\'');
        var vcc = res.locals.vcc;
        (0, promises_shared_1.DummyPromise)().then(function (_) {
            return mssql_service_1.MSSql.GetInteractionCampaigns(vcc);
        }).then(function (campaigns) {
            res.json({ status: true, description: 'ok', data: campaigns });
        }).catch(function (err) {
            var clientMsg = (typeof err === 'string' ? err : 'CANNOT_PROCESS_REQUEST');
            var errorMsg = (typeof err === 'string' ? err : err.message || err.description || 'SERVER_ERROR_UNKNOWN_MESSAGE');
            logger.error('[NLUService::campaigns_interaction] Error on \'' + req.path + '\': ' + errorMsg);
            res.json({ status: false, description: clientMsg, error: errorMsg });
        });
    });
}
exports.CampainService = CampainService;
