"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.DiarizationService = void 0;
var log4js = require("log4js");
var mssql_service_1 = require("../db/mssql.service");
var promises_shared_1 = require("../shared/promises.shared");
var cognitive_analysis_implementation_1 = require("../implementation/cognitive-analysis.implementation");
var logger = log4js.getLogger('ServerScripts');
function DiarizationService(app) {
    var cognitiveService;
    app.post('/api/diarization/get_model_availability', function (req, res, next) {
        logger.info('[DiarizationService::get_model_availability] Received ' + req.method + ' method on \'' + req.path + '\'');
        var vcc = res.locals.vcc;
        var diarizationModels = [];
        (0, promises_shared_1.DummyPromise)()
            .then(function (result) {
            return (0, cognitive_analysis_implementation_1.GetCognitiveService)(null, 'RASA');
        })
            .then(function (result) {
            cognitiveService = result;
            return mssql_service_1.MSSql.GetNLUs(vcc, null, null, null, true);
        })
            .then(function (result) {
            if (!result)
                throw 'SERVER_ERROR_DIARIZATION_MODELS_NOT_FOUND';
            diarizationModels = result;
            var promiseArray = [];
            var timeOut = 0;
            diarizationModels.map(function (r, i) {
                timeOut = i * 1000;
                promiseArray.push(new Promise(function (resolve, reject) {
                    setTimeout(function () {
                        return resolve(cognitiveService.GetUtterancesList(null, null, r.appId));
                    }, timeOut);
                }));
            });
            return Promise.all(promiseArray);
        })
            .catch(function (err) {
            logger.error('[DiarizationService::get_model_availability] Passing error: ', err);
            return Promise.resolve([null, null, null, null]);
        })
            .then(function (result) {
            result.map(function (r, i) {
                diarizationModels[i].utterances = r;
            });
            if (res.headersSent)
                return;
            res.json({ status: true, description: 'OK', data: diarizationModels });
        })
            .catch(function (err) {
            var clientMsg = (typeof err === 'string' ? err : 'CANNOT_PROCESS_REQUEST');
            var errorMsg = (typeof err === 'string' ? err : err.message || err.description || 'SERVER_ERROR_UNKNOWN_MESSAGE');
            logger.error('[DiarizationService::get_model_availability] Error on \'' + req.path + '\': ' + errorMsg);
            if (res.headersSent)
                return;
            res.json({ status: false, description: clientMsg, error: errorMsg });
        });
    });
}
exports.DiarizationService = DiarizationService;
