"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.DictionaryService = void 0;
var log4js = require("log4js");
var mssql_service_1 = require("../db/mssql.service");
var promises_shared_1 = require("../shared/promises.shared");
var dictionary_model_1 = require("../models/dictionary.model");
var arrays_shared_1 = require("../shared/arrays.shared");
var logger = log4js.getLogger('ServerScripts');
function DictionaryService(app) {
    app.post('/api/dictionary/get_dictionary', function (req, res, next) {
        logger.info('[DictionaryService::get_dictionary] Received ' + req.method + ' method on \'' + req.path + '\'');
        var vcc = res.locals.vcc;
        var dictionaryId = req.body.dictionaryId || '';
        (0, promises_shared_1.DummyPromise)()
            .then(function (result) {
            return mssql_service_1.MSSql.GetDictionary(vcc, dictionaryId);
        })
            .then(function (result) {
            if (res.headersSent)
                return;
            res.json({ status: true, description: 'SERVER_SUCCESS_MESSAGE', data: result });
        })
            .catch(function (err) {
            var clientMsg = (typeof err === 'string' ? err : 'CANNOT_PROCESS_REQUEST');
            var errorMsg = (typeof err === 'string' ? err : err.message || err.description || 'SERVER_ERROR_UNKNOWN_MESSAGE');
            logger.error('[DictionaryService::delete_word] Error on \'' + req.path + '\': ' + errorMsg);
            if (res.headersSent)
                return;
            res.json({ status: false, description: clientMsg, error: errorMsg });
        });
    });
    app.post('/api/dictionary/get_dictionaries', function (req, res, next) {
        logger.info('[DictionaryService::get_dictionaries] Received ' + req.method + ' method on \'' + req.path + '\'');
        var vcc = res.locals.vcc;
        (0, promises_shared_1.DummyPromise)()
            .then(function (result) {
            return mssql_service_1.MSSql.GetDictionaries(vcc, null, null);
        })
            .then(function (result) {
            if (res.headersSent)
                return;
            res.json({ status: true, description: 'SERVER_SUCCESS_MESSAGE', data: result });
        })
            .catch(function (err) {
            var environmentMsg = (typeof err === 'string' ? err : 'CANNOT_PROCESS_REQUEST');
            var errorMsg = (typeof err === 'string' ? err : err.message || err.description || 'SERVER_ERROR_UNKNOWN_MESSAGE');
            logger.error('[DictionaryService::get_dictionaries] Error al procesar \'' + req.path + '\': ' + errorMsg);
            if (res.headersSent)
                return;
            res.json({ status: false, description: environmentMsg, error: errorMsg });
        });
    });
    app.post('/api/dictionary/word/get_dictionary_word', function (req, res, next) {
        logger.info('[DictionaryService::get_dictionary_word] Received ' + req.method + ' method on \'' + req.path + '\'');
        var vcc = res.locals.vcc;
        var dictionaryWord = new dictionary_model_1.DictionaryWord();
        dictionaryWord.id = req.body.dictionaryWordId || '';
        (0, promises_shared_1.DummyPromise)()
            .then(function (result) {
            return mssql_service_1.MSSql.GetDictionaryWord(vcc, dictionaryWord.id);
        })
            .then(function (result) {
            if (!result)
                throw 'SERVER_ERROR_DICTIONARY_WORD_NOT_FOUND';
            dictionaryWord = result;
            return mssql_service_1.MSSql.GetDictionaryWordSoundLikes(vcc, dictionaryWord.id);
        })
            .then(function (result) {
            if (!result)
                throw 'SERVER_ERROR_DICTIONARY_WORD_SOUND_LIKES_NOT_FOUND';
            dictionaryWord.soundLikes = result;
            if (res.headersSent)
                return;
            res.json({ status: true, description: 'SERVER_SUCCESS_MESSAGE', data: dictionaryWord });
        })
            .catch(function (err) {
            var environmentMsg = (typeof err === 'string' ? err : 'CANNOT_PROCESS_REQUEST');
            var errorMsg = (typeof err === 'string' ? err : err.message || err.description || 'SERVER_ERROR_UNKNOWN_MESSAGE');
            logger.error('[DictionaryService::get_dictionary_word] Error al procesar \'' + req.path + '\': ' + errorMsg);
            if (res.headersSent)
                return;
            res.json({ status: false, description: environmentMsg, error: errorMsg });
        });
    });
    app.post('/api/dictionary/word/get_dictionary_words', function (req, res, next) {
        logger.info('[DictionaryService::get_dictionary_words] Received ' + req.method + ' method on \'' + req.path + '\'');
        var vcc = res.locals.vcc;
        var dictionaryId = req.body.dictionaryId || '';
        var search_criteria = req.body.searchCriteria || '';
        var search_expression = req.body.searchExpression || '';
        (0, promises_shared_1.DummyPromise)()
            .then(function (result) {
            return mssql_service_1.MSSql.GetDictionaryWords(vcc, search_criteria, search_expression, dictionaryId);
        })
            .then(function (result) {
            if (res.headersSent)
                return;
            res.json({ status: true, description: 'SERVER_SUCCESS_MESSAGE', data: result });
        })
            .catch(function (err) {
            var environmentMsg = (typeof err === 'string' ? err : 'CANNOT_PROCESS_REQUEST');
            var errorMsg = (typeof err === 'string' ? err : err.message || err.description || 'SERVER_ERROR_UNKNOWN_MESSAGE');
            logger.error('[DictionaryService::get_dictionary_words] Error al procesar \'' + req.path + '\': ' + errorMsg);
            if (res.headersSent)
                return;
            res.json({ status: false, description: environmentMsg, error: errorMsg });
        });
    });
    app.post('/api/dictionary/word/create_word', function (req, res, next) {
        logger.info('[DictionaryService::create_word] Received ' + req.method + ' method on \'' + req.path + '\'');
        var vcc = res.locals.vcc;
        var userId = res.locals.userId;
        var dictionaryWord = (req.body || {});
        dictionaryWord.word = dictionaryWord.word.trim();
        (0, promises_shared_1.DummyPromise)()
            .then(function (result) {
            return SaveDictionaryWord(vcc, dictionaryWord, userId);
        })
            .then(function (result) {
            if (res.headersSent)
                return;
            res.json({ status: true, description: (result ? 'SERVER_SUCCESS_MESSAGE' : 'SERVER_ERROR_DICTIONARY_WORD_CREATE_FAILED'), data: result });
        })
            .catch(function (err) {
            var clientMsg = (typeof err === 'string' ? err : 'CANNOT_PROCESS_REQUEST');
            var errorMsg = (typeof err === 'string' ? err : err.message || err.description || 'SERVER_ERROR_UNKNOWN_MESSAGE');
            logger.error('[DictionaryService::create_word] Error on \'' + req.path + '\': ' + errorMsg);
            if (res.headersSent)
                return;
            res.json({ status: false, description: clientMsg, error: errorMsg });
        });
    });
    app.post('/api/dictionary/word/save_word', function (req, res, next) {
        logger.info('[DictionaryService::save_word] Received ' + req.method + ' method on \'' + req.path + '\'');
        var vcc = res.locals.vcc;
        var userId = res.locals.userId;
        var dictionaryWord = (req.body || {});
        dictionaryWord.word = dictionaryWord.word.trim();
        (0, promises_shared_1.DummyPromise)()
            .then(function (_) {
            if (!dictionaryWord.soundLikes.length)
                return Promise.resolve([]);
            return mssql_service_1.MSSql.GetSoundLikesFromDictionaryWords(vcc, dictionaryWord.id, dictionaryWord.soundLikes.map(function (item) { return item.soundLike; }).join(','));
        })
            .then(function (result) {
            if (result.length)
                return Promise.resolve(result);
            return SaveDictionaryWord(vcc, dictionaryWord, userId);
        })
            .then(function (result) {
            if (res.headersSent)
                return;
            if (result.length)
                res.json({ status: true, description: 'SERVER_ERROR_DICTIONARY_SOUND_LIKE_EXISTS', data: result });
            else
                res.json({ status: true, description: (result ? 'SERVER_SUCCESS_MESSAGE' : 'SERVER_ERROR_DICTIONARY_WORD_SAVE_FAILED'), data: result });
        })
            .catch(function (err) {
            var clientMsg = (typeof err === 'string' ? err : 'CANNOT_PROCESS_REQUEST');
            var errorMsg = (typeof err === 'string' ? err : err.message || err.description || 'SERVER_ERROR_UNKNOWN_MESSAGE');
            logger.error('[DictionaryService::save_word] Error on \'' + req.path + '\': ' + errorMsg);
            if (res.headersSent)
                return;
            res.json({ status: false, description: clientMsg, error: errorMsg });
        });
    });
    app.post('/api/dictionary/get_dictionary_usage', function (req, res, next) {
        logger.info('[DictionaryService::get_dictionary] Received ' + req.method + ' method on \'' + req.path + '\'');
        var vcc = res.locals.vcc;
        var dictionaryId = req.body.dictionaryId || '';
        (0, promises_shared_1.DummyPromise)()
            .then(function (result) {
            return mssql_service_1.MSSql.GetDictionaryUsage(vcc, dictionaryId);
        })
            .then(function (result) {
            if (res.headersSent)
                return;
            res.json({ status: true, description: 'SERVER_SUCCESS_MESSAGE', data: result });
        })
            .catch(function (err) {
            var clientMsg = (typeof err === 'string' ? err : 'CANNOT_PROCESS_REQUEST');
            var errorMsg = (typeof err === 'string' ? err : err.message || err.description || 'SERVER_ERROR_UNKNOWN_MESSAGE');
            logger.error('[DictionaryService::delete_word] Error on \'' + req.path + '\': ' + errorMsg);
            if (res.headersSent)
                return;
            res.json({ status: false, description: clientMsg, error: errorMsg });
        });
    });
    app.post('/api/dictionary/word/delete_word', function (req, res, next) {
        logger.info('[DictionaryService::delete_word] Received ' + req.method + ' method on \'' + req.path + '\'');
        var vcc = res.locals.vcc;
        var dictionaryWord = (req.body || {});
        logger.info("[DictionaryService::delete_word] Deleting word ".concat(dictionaryWord, " by the user ").concat(res.locals.userId));
        (0, promises_shared_1.DummyPromise)()
            .then(function (result) {
            if (!dictionaryWord)
                throw 'SERVER_ERROR_DICTIONARY_WORD_WAS_NOT_RECEIVED';
            return mssql_service_1.MSSql.DeleteDictionaryWord(vcc, dictionaryWord.id);
        })
            .then(function (result) {
            if (res.headersSent)
                return;
            res.json({ status: result, description: (result ? 'SERVER_SUCCESS_MESSAGE' : 'SERVER_ERROR_DICTIONARY_WORD_DELETE_FAILED'), data: result });
        })
            .catch(function (err) {
            var clientMsg = (typeof err === 'string' ? err : 'CANNOT_PROCESS_REQUEST');
            var errorMsg = (typeof err === 'string' ? err : err.message || err.description || 'SERVER_ERROR_UNKNOWN_MESSAGE');
            logger.error('[DictionaryService::delete_word] Error on \'' + req.path + '\': ' + errorMsg);
            if (res.headersSent)
                return;
            res.json({ status: false, description: clientMsg, error: errorMsg });
        });
    });
    function SaveDictionaryWord(vcc, dictionaryWord, saveUser) {
        return (0, promises_shared_1.DummyPromise)()
            .then(function (result) {
            return mssql_service_1.MSSql.SaveDictionaryWord(vcc, dictionaryWord.dictionaryId, dictionaryWord.id, dictionaryWord.word, saveUser);
        })
            .then(function (result) {
            return mssql_service_1.MSSql.GetDictionaryWordSoundLikes(vcc, dictionaryWord.id);
        })
            .then(function (result) {
            if (dictionaryWord.soundLikes) {
                var classifiedItems = (0, arrays_shared_1.GetCompareItemsFromObjectArray)(result, dictionaryWord.soundLikes, 'id');
                var promiseArray_1 = [];
                classifiedItems.new.map(function (ni) {
                    promiseArray_1.push(mssql_service_1.MSSql.SaveDictionaryWordSoundLike(vcc, dictionaryWord.id, ni.id, ni.soundLike, saveUser));
                });
                classifiedItems.removed.map(function (ri) {
                    promiseArray_1.push(mssql_service_1.MSSql.DeleteDictionaryWordSoundLike(vcc, dictionaryWord.id, ri.id));
                });
                return Promise.all(promiseArray_1);
            }
            else {
                return Promise.resolve([]);
            }
        })
            .then(function (result) {
            return Promise.resolve(true);
        })
            .catch(function (err) {
            var errorMsg = (typeof err === 'string' ? err : err.message || err.description || 'Executing request error');
            logger.error('[DictionaryService::SaveDictionaryWord] Error: ' + errorMsg);
            return Promise.reject(errorMsg);
        });
    }
}
exports.DictionaryService = DictionaryService;
