"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.EvaluationFormatService = void 0;
var moment = require("moment");
var log4js = require("log4js");
var mssql_service_1 = require("../db/mssql.service");
var template_service_1 = require("./template.service");
var promises_shared_1 = require("../shared/promises.shared");
var utilities_1 = require("../shared/utilities");
var template_model_1 = require("../models/template.model");
var custom_field_service_1 = require("./custom-field.service");
var ability_service_1 = require("./ability.service");
var process_implementation_1 = require("../implementation/process.implementation");
var logger = log4js.getLogger('ServerScripts');
function EvaluationFormatService(app) {
    app.get('/api/evaluationformat/get_evaluationformats/:search_criteria/:search_expression?', function (req, res, next) {
        logger.info('[EvaluationFormatService::get_evaluationformats] Received ' + req.method + ' method on \'' + req.path + '\'');
        var instance = res.locals.instance;
        var search_criteria = req.params.search_criteria || '';
        var search_expression = req.params.search_expression || '';
        var disabled = req.query.disabled === 'true' || false;
        var historic = req.query.historic === 'true' || false;
        logger.info('[EvaluationFormatService::get_evaluationformats] Instance received: ' + instance);
        logger.info('[EvaluationFormatService::get_evaluationformats] Search_criteria received: ' + search_criteria);
        logger.info('[EvaluationFormatService::get_evaluationformats] Search_expression received: ' + search_expression);
        (0, promises_shared_1.DummyPromise)()
            .then(function (_) {
            if (historic) {
                return mssql_service_1.MSSql.GetEvaluationFormatsFromHistoricEvaluationPlans(instance);
            }
            if (search_criteria === 'idGroup' || search_criteria === 'idSubGroup' || search_criteria === 'idAbility') {
                logger.info('[EvaluationFormatService::get_evaluationformats] Calling SP GetEvaluationFormatDescriptionsBySearchCriteria');
                return mssql_service_1.MSSql.GetEvaluationFormatDescriptionsBySearchCriteria(instance, search_criteria, search_expression);
            }
            else {
                logger.info('[EvaluationFormatService::get_evaluationformats] Calling SP GetEvaluationFormats');
                return mssql_service_1.MSSql.GetEvaluationFormats(instance, search_criteria, search_expression, disabled);
            }
        }).then(function (evaluationformats) {
            if (res.headersSent)
                return;
            logger.info('[EvaluationFormatService::get_evaluationformats] Received restult from SP GetEvaluationFormats: ' + evaluationformats.length);
            res.json({ status: true, description: 'SERVER_SUCCESS_MESSAGE', data: evaluationformats });
        }).catch(function (err) {
            var clientMsg = (typeof err === 'string' ? err : 'CANNOT_PROCESS_REQUEST');
            var errorMsg = (typeof err === 'string' ? err : err.message || err.description || 'Requesting error');
            logger.error('[EvaluationFormatService::get_evaluationformats] Error on \'' + req.path + '\': ' + errorMsg);
            if (res.headersSent)
                return;
            res.json({ status: false, description: clientMsg, error: errorMsg });
        });
    });
    app.post('/api/evaluationformat/create_evaluationformat', function (req, res, next) {
        logger.info('[EvaluationFormatService::create_evaluationformat] Received ' + req.method + ' method on \'' + req.path + '\'');
        var vcc = res.locals.vcc;
        var userId = res.locals.userId;
        var nowUtc = moment().utc().toDate();
        var newEvaluationFormat = (req.body || {});
        newEvaluationFormat.createdDate = nowUtc;
        newEvaluationFormat.createdByUserId = userId;
        newEvaluationFormat.lastModifiedDate = nowUtc;
        newEvaluationFormat.lastModifiedByUserId = userId;
        logger.info('[EvaluationFormatService::create_evaluationformat] Received vcc: ' + vcc);
        (0, promises_shared_1.DummyPromise)()
            .then(function (_) {
            logger.info('[EvaluationFormatService::create_evaluationformat] Calling GetEvaluationFormatDescription SP');
            return mssql_service_1.MSSql.GetEvaluationFormatDescription(vcc, newEvaluationFormat.description);
        })
            .then(function (result) {
            logger.info('[EvaluationFormatService::create_evaluationformat] Received result from GetEvaluationFormatDescription SP: ' + JSON.stringify(result));
            if (result)
                throw 'EVALUATION_FORMAT_SERVER_ERROR_EXISTS';
            newEvaluationFormat.id = newEvaluationFormat.description;
            newEvaluationFormat.templateId = ['MANUAL_AUTOMATIC', 'MIXED'].includes(newEvaluationFormat.type) ? "qty-tmpl-".concat(newEvaluationFormat.description) : null;
            logger.info('[EvaluationFormatService::create_evaluationformat] Saving ev format ' + JSON.stringify(newEvaluationFormat));
            return mssql_service_1.MSSql.SaveEvaluationFormat(vcc, newEvaluationFormat.id, newEvaluationFormat.description, newEvaluationFormat.type, newEvaluationFormat.environmentType, newEvaluationFormat.minQualificationAcceptable, newEvaluationFormat.minAutomaticQualification, newEvaluationFormat.managementFormURL, newEvaluationFormat.createdDate, newEvaluationFormat.createdByUserId, newEvaluationFormat.lastModifiedDate, newEvaluationFormat.lastModifiedByUserId, newEvaluationFormat.enabled, newEvaluationFormat.templateId);
        })
            .then(function (result) {
            if (!result)
                throw 'EVALUATION_FORMAT_ERROR_ON_SAVE';
            return SaveEvaluationFormatContent(vcc, newEvaluationFormat, true, userId, null);
        })
            .then(function (result) {
            if (!result)
                throw 'SERVER_ERROR_EVALUATION_FORMAT_SAVE_CONTENT_FAILED';
            if (res.headersSent)
                return;
            logger.info('[EvaluationFormatService::create_evaluationformat] Sending result');
            res.json({ status: result, description: (result ? 'SERVER_SUCCESS_MESSAGE' : 'EVALUATION_FORMAT_ERROR_ON_SAVE'), data: result });
        })
            .catch(function (err) {
            var clientMsg = (typeof err === 'string' ? err : 'CANNOT_PROCESS_REQUEST');
            var errorMsg = (typeof err === 'string' ? err : err.message || err.description || 'Requesting error');
            logger.error('[EvaluationFormatService::create_evaluationformat] Error on \'' + req.path + '\': ' + errorMsg);
            if (res.headersSent)
                return;
            res.json({ status: false, description: clientMsg, error: errorMsg });
        });
    });
    app.get('/api/evaluationformat/get_evaluationformat/:evaluationformat_id', function (req, res) {
        logger.info('[EvaluationFormatService::get_evaluationformat] Received ' + req.method + ' method on \'' + req.path + '\'');
        var instance = res.locals.instance;
        var evaluationFormat = {};
        var evaluationFormatAbilities = [];
        var evaluationFormatId = req.params.evaluationformat_id;
        var findQueries = req.query.find_queries ? true : false;
        var findEntities = req.query.find_entities ? true : false;
        var evaluationPlanId = (typeof req.query.evaluation_plan_id == 'string'
            && req.query.evaluation_plan_id != 'null') ? req.query.evaluation_plan_id : null;
        logger.info('[EvaluationFormatService::get_evaluationformat] Looking for EV ' + evaluationFormatId);
        (0, promises_shared_1.DummyPromise)()
            .then(function (_) {
            if (!evaluationFormatId)
                throw 'ERROR_EVALUATION_FORMAT_ID_MISSING';
            logger.info('[EvaluationFormatService::get_evaluationformat] Calling GetEvaluationFormat SP');
            return mssql_service_1.MSSql.GetEvaluationFormat(instance, evaluationFormatId, evaluationPlanId);
        }).then(function (result) {
            evaluationFormat = result || evaluationFormat;
            logger.info('[EvaluationFormatService::get_evaluationformat] Calling GetAbilityEvaluationFormats SP');
            if (evaluationPlanId) {
                var processImplementation = new process_implementation_1.ProcessImplementation();
                return processImplementation.GetEvaluationPlanAbilities(instance, evaluationPlanId);
            }
            else
                return mssql_service_1.MSSql.GetAbilityEvaluationFormats(instance, evaluationFormatId, evaluationPlanId);
        }).then(function (result) {
            evaluationFormatAbilities = result || evaluationFormatAbilities;
            evaluationFormatAbilities.forEach(function (element) {
                if (element.idSubGroup === null) {
                    element.idSubGroup = '';
                    element.descriptionSubGroup = '';
                }
            });
            var promiseArray = [];
            if (findQueries) {
                evaluationFormatAbilities.map(function (efa) {
                    promiseArray.push((0, ability_service_1.GetAbilityQueries)(instance, efa.idAbility, findEntities, evaluationPlanId));
                });
            }
            return Promise.all(promiseArray);
        })
            .then(function (result) {
            result.map(function (r, i) {
                evaluationFormatAbilities[i].queries = r;
            });
            evaluationFormat.abilityEvaluationFormats = evaluationFormatAbilities;
            logger.info('[EvaluationFormatService::get_evaluationformat] Calling GetEvaluationFormatCustomFields SP');
            return mssql_service_1.MSSql.GetEvaluationFormatCustomFields(instance, evaluationFormatId, evaluationPlanId);
        })
            .then(function (result) {
            logger.info('[EvaluationFormatService::get_evaluationformat] Calling GetEvaluationFormatAttachedSignatures SP');
            evaluationFormat.evaluationFormatCustomFields = result;
            if (evaluationFormat.evaluationFormatCustomFields && evaluationFormat.evaluationFormatCustomFields.length) {
                var promiseArray_1 = [];
                evaluationFormat.evaluationFormatCustomFields.map(function (efcf) { return promiseArray_1.push((0, custom_field_service_1.GetCustomField)(instance, efcf.customFieldId, evaluationPlanId)); });
                return Promise.all(promiseArray_1);
            }
            else {
                return Promise.resolve([null]);
            }
        })
            .then(function (result) {
            if (evaluationFormat.evaluationFormatCustomFields && evaluationFormat.evaluationFormatCustomFields.length) {
                evaluationFormat.evaluationFormatCustomFields.map(function (efcf, index) { return efcf.customField = result[index]; });
            }
            return mssql_service_1.MSSql.GetEvaluationFormatAttachedSignatures(instance, evaluationFormatId, evaluationPlanId);
        })
            .then(function (result) {
            logger.info('[EvaluationFormatService::get_evaluationformat] Calling GetCampainEvaluationFormats SP');
            evaluationFormat.evaluationFormatAttachedSignatures = result;
            return mssql_service_1.MSSql.GetCampainEvaluationFormats(instance, evaluationFormatId, evaluationPlanId);
        }).then(function (result) {
            if (res.headersSent)
                return;
            evaluationFormat.campainEvaluationFormats = result || evaluationFormat.campainEvaluationFormats;
            logger.info('[EvaluationFormatService::get_evaluationformat] Sending response');
            res.json({ status: true, description: 'SERVER_SUCCESS_MESSAGE', data: evaluationFormat });
        }).catch(function (err) {
            var clientMsg = (typeof err === 'string' ? err : 'CANNOT_PROCESS_REQUEST');
            var errorMsg = (typeof err === 'string' ? err : err.message || err.description || 'Requesting error');
            logger.error('[EvaluationFormatService::get_evaluationformat] Error on \'' + req.path + '\': ' + errorMsg);
            if (res.headersSent)
                return;
            res.json({ status: false, description: clientMsg, error: errorMsg });
        });
    });
    app.post('/api/evaluationformat/save_evaluationformat', function (req, res, next) {
        logger.info('[EvaluationFormatService::save_evaluationformat] Received ' + req.method + ' method on \'' + req.path + '\'');
        var vcc = res.locals.vcc;
        var userId = res.locals.userId;
        var nowUtc = moment().utc().toDate();
        var newEvaluationFormat = (req.body || {});
        newEvaluationFormat.createdDate = null;
        newEvaluationFormat.createdByUserId = null;
        newEvaluationFormat.lastModifiedDate = nowUtc;
        newEvaluationFormat.lastModifiedByUserId = userId;
        var oldTemplateId = newEvaluationFormat.templateId ? newEvaluationFormat.templateId : '';
        (0, promises_shared_1.DummyPromise)()
            .then(function (_) {
            logger.info('[EvaluationFormatService::save_evaluationformat] Looking for a Evaluation format: ' + newEvaluationFormat.description);
            return mssql_service_1.MSSql.GetEvaluationFormatDescription(vcc, newEvaluationFormat.description);
        })
            .then(function (result) {
            var evaluationFormatDesc = {};
            evaluationFormatDesc = result || evaluationFormatDesc;
            if (result && evaluationFormatDesc.id !== newEvaluationFormat.id)
                throw 'EVALUATION_FORMAT_SERVER_ERROR_EXISTS';
            logger.info('[EvaluationFormatService::save_evaluationformat] Saving evaluation format');
            newEvaluationFormat.templateId = ['MANUAL_AUTOMATIC', 'MIXED'].includes(newEvaluationFormat.type) ? "qty-tmpl-".concat(newEvaluationFormat.id) : null;
            return mssql_service_1.MSSql.SaveEvaluationFormat(vcc, newEvaluationFormat.id, newEvaluationFormat.description, newEvaluationFormat.type, newEvaluationFormat.environmentType, newEvaluationFormat.minQualificationAcceptable, newEvaluationFormat.minAutomaticQualification, newEvaluationFormat.managementFormURL, newEvaluationFormat.createdDate, newEvaluationFormat.createdByUserId, newEvaluationFormat.lastModifiedDate, newEvaluationFormat.lastModifiedByUserId, newEvaluationFormat.enabled, newEvaluationFormat.templateId);
        })
            .then(function (result) {
            logger.info('[EvaluationFormatService::save_evaluationformat] Result: ', result);
            if (result === null || result === void 0 ? void 0 : result.length) {
                var status_1;
                var idEvaluationFormat = result[0].idEvaluationFormat;
                logger.info('[EvaluationFormatService::save_evaluationformat] idEvaluationFormat: ', idEvaluationFormat);
                if (idEvaluationFormat === -1) {
                    status_1 = 'EVALUATION_FORMAT_BUSY';
                    logger.info('[EvaluationFormatService::save_evaluationformat] result - status -1:', status_1);
                    return Promise.resolve(status_1);
                }
                else {
                    status_1 = idEvaluationFormat;
                    logger.info('[EvaluationFormatService::save_evaluationformat] result - status: ', status_1);
                    return SaveEvaluationFormatContent(vcc, newEvaluationFormat, false, userId, oldTemplateId);
                }
            }
            else {
                logger.error('[EvaluationFormatService::save_evaluationformat] result error: ', result);
                return Promise.reject(result);
            }
        })
            .then(function (resultSave) {
            logger.info('[EvaluationFormatService::save_evaluationformat] resultSave: ', resultSave);
            if (res.headersSent)
                return;
            res.json({ status: resultSave, description: (resultSave ? resultSave === 'EVALUATION_FORMAT_BUSY' ? 'EVALUATION_FORMAT_BUSY' : 'SERVER_SUCCESS_MESSAGE' : 'EVALUATION_FORMAT_ERROR_ON_SAVE'), data: resultSave });
        })
            .catch(function (err) {
            var clientMsg = (typeof err === 'string' ? err : 'CANNOT_PROCESS_REQUEST');
            var errorMsg = (typeof err === 'string' ? err : err.message || err.description || 'Requesting error');
            logger.error('[EvaluationFormatService::save_evaluationformat] Error on \'' + req.path + '\': ' + errorMsg);
            if (res.headersSent)
                return;
            res.json({ status: false, description: clientMsg, error: errorMsg });
        });
    });
    app.post('/api/evaluationformat/delete_evaluationformat', function (req, res, next) {
        logger.info('Se recibió ' + req.method + ' para \'' + req.path + '\'');
        var instance = res.locals.instance;
        var evaluationFormat = (req.body || {});
        var evaluationFormatId = evaluationFormat.id;
        logger.info("[EvaluationFormatService::delete_evaluationformat] Deleting evaluationFormat ".concat(evaluationFormatId, " by the user ").concat(res.locals.userId));
        (0, promises_shared_1.DummyPromise)()
            .then(function (result) {
            return mssql_service_1.MSSql.DeleteEvaluationFormat(instance, evaluationFormatId);
        }).then(function (result) {
            if (!result)
                throw 'EVALUATION_FORMAT_SERVER_ERROR_ON_DELETE';
            return Promise.resolve(true);
        }).then(function (result) {
            if (res.headersSent)
                return;
            res.json({ status: result, description: (result ? 'SERVER_SUCCESS_MESSAGE' : 'EVALUATION_FORMAT_SERVER_ERROR_ON_DELETE'), data: result });
        }).catch(function (err) {
            var clientMsg = (typeof err === 'string' ? err : 'CANNOT_PROCESS_REQUEST');
            var errorMsg = (typeof err === 'string' ? err : err.message || err.description || 'SERVER_ERROR_UNKNOWN_MESSAGE');
            logger.error('Error al procesar \'' + req.path + '\': ' + errorMsg);
            if (res.headersSent)
                return;
            res.json({ status: false, description: clientMsg, error: errorMsg });
        });
    });
    app.post('/api/evaluationformat/delete_evaluationformats', function (req, res, next) {
        logger.info('Se recibió ' + req.method + ' para \'' + req.path + '\'');
        var instance = res.locals.instance;
        var evaluationFormats = req.body.evaluationFormats || [];
        logger.info("[EvaluationFormatService::delete_evaluationformats] Deleting evaluationFormats ".concat(evaluationFormats.map(function (x) { return x.id; }).join(','), " by the user ").concat(res.locals.userId));
        (0, promises_shared_1.DummyPromise)()
            .then(function (result) {
            if (!evaluationFormats)
                throw 'EVALUATION_FORMAT_SERVER_ERROR_MISSING_IDS';
            var promiseArray = [];
            evaluationFormats.forEach(function (evaluationFormat) {
                promiseArray.push(mssql_service_1.MSSql.DeleteEvaluationFormat(instance, evaluationFormat.id));
            });
            return Promise.all(promiseArray);
        }).then(function (resultDeletes) {
            var result;
            if (resultDeletes.filter(function (result) { return (result !== true); }).length > 0) {
                result = false;
                logger.error('No se pudo eliminar todos los elementos solicitados.');
            }
            else {
                result = true;
                logger.info('Evaluation formats eliminados correctamente');
            }
            return result;
        }).then(function (resultDeletes) {
            if (res.headersSent)
                return;
            res.json({ status: resultDeletes, description: (resultDeletes ? 'SERVER_SUCCESS_MESSAGE' : 'EVALUATION_FORMAT_SERVER_ERROR_ON_DELETE'), data: resultDeletes });
        }).catch(function (err) {
            var clientMsg = (typeof err === 'string' ? err : 'CANNOT_PROCESS_REQUEST');
            var errorMsg = (typeof err === 'string' ? err : err.message || err.description || 'SERVER_ERROR_UNKNOWN_MESSAGE');
            logger.error('Error al procesar \'' + req.path + '\': ' + errorMsg);
            if (res.headersSent)
                return;
            res.json({ status: false, description: clientMsg, error: errorMsg });
        });
    });
    app.get('/api/evaluationformat/get_evaluationformat_qualification_average/:evaluationFormatId', function (req, res, next) {
        logger.info('[EvaluationFormatService::get_evaluationformat_qualification_average] Received ' + req.method + ' method on \'' + req.path + '\'');
        var instance = res.locals.instance;
        var evaluationFormatStr = req.params.evaluationFormatId || '';
        logger.info('[EvaluationFormatService::get_evaluationformat_qualification_average] Received instance: ' + instance);
        logger.info('[EvaluationFormatService::get_evaluationformat_qualification_average] Received evaluationFormatId: ' + evaluationFormatStr);
        (0, promises_shared_1.DummyPromise)()
            .then(function (result) {
            logger.info('[EvaluationFormatService::get_evaluationformat_qualification_average] Calling GetEvaluationFormatQualificationAverage SP');
            return mssql_service_1.MSSql.GetEvaluationFormatQualificationAverage(instance, evaluationFormatStr);
        }).then(function (result) {
            logger.info('[EvaluationFormatService::get_evaluationformat_qualification_average] Received result from GetEvaluationFormatQualificationAverage SP ' + JSON.stringify(result));
            if (res.headersSent)
                return;
            res.json({ status: true, description: 'SERVER_SUCCESS_MESSAGE', data: result });
        }).catch(function (err) {
            var clientMsg = (typeof err === 'string' ? err : 'CANNOT_PROCESS_REQUEST');
            var errorMsg = (typeof err === 'string' ? err : err.message || err.description || 'Requesting error');
            logger.error('[EvaluationFormatService::get_evaluationformat_qualification_average] Error on \'' + req.path + '\': ' + errorMsg);
            if (res.headersSent)
                return;
            res.json({ status: false, description: clientMsg, error: errorMsg });
        });
    });
    app.get('/api/evaluationformat/isQueryBeingUsedByQuality/:queryName', function (req, res, next) {
        logger.info('Se recibió ' + req.method + ' para \'' + req.path + '\'');
        var instance = res.locals.instance;
        var queryName = req.params.queryName;
        (0, promises_shared_1.DummyPromise)()
            .then(function (result) {
            if (!queryName)
                throw 'EVALUATION_FORMAT_SERVER_ERROR_MISSING_QUERY';
            return mssql_service_1.MSSql.IsQueryBeingUsedByQuality(instance, queryName);
        }).then(function (result) {
            if (res.headersSent)
                return;
            var data = result > 0 ? true : false;
            res.json({ status: true, description: 'SERVER_SUCCESS_MESSAGE', data: data });
        }).catch(function (err) {
            var clientMsg = (typeof err === 'string' ? err : 'CANNOT_PROCESS_REQUEST');
            var errorMsg = (typeof err === 'string' ? err : err.message || err.description || 'SERVER_ERROR_UNKNOWN_MESSAGE');
            logger.error('Error al procesar \'' + req.path + '\': ' + errorMsg);
            if (res.headersSent)
                return;
            res.json({ status: false, description: clientMsg, error: errorMsg });
        });
    });
    app.get('/api/evaluationformat/isTemplateBeingUsedByQuality/:TemplateId', function (req, res, next) {
        logger.info('Se recibió ' + req.method + ' para \'' + req.path + '\'');
        var instance = res.locals.instance;
        var templateId = req.params.TemplateId;
        (0, promises_shared_1.DummyPromise)()
            .then(function (result) {
            if (!templateId)
                throw 'EVALUATION_FORMAT_SERVER_ERROR_MISSING_QUERY';
            return mssql_service_1.MSSql.IsTemplateBeingUsedByQuality(instance, templateId);
        }).then(function (result) {
            if (res.headersSent)
                return;
            var data = result > 0 ? true : false;
            res.json({ status: true, description: 'SERVER_SUCCESS_MESSAGE', data: data });
        }).catch(function (err) {
            var clientMsg = (typeof err === 'string' ? err : 'CANNOT_PROCESS_REQUEST');
            var errorMsg = (typeof err === 'string' ? err : err.message || err.description || 'SERVER_ERROR_UNKNOWN_MESSAGE');
            logger.error('Error al procesar \'' + req.path + '\': ' + errorMsg);
            if (res.headersSent)
                return;
            res.json({ status: false, description: clientMsg, error: errorMsg });
        });
    });
    app.get('/api/evaluationformat/get_evaluationformat_attachedsignatures/:evaluationformat_id', function (req, res, next) {
        logger.info('[EvaluationFormatService::get_evaluationformat_attachedsignatures] Received ' + req.method + ' method on \'' + req.path + '\'');
        var vcc = res.locals.vcc;
        var userId = res.locals.userId;
        var evaluationFormatAttachedSignatures = [];
        var evaluationFormatId = req.params.evaluationformat_id;
        (0, promises_shared_1.DummyPromise)()
            .then(function (result) {
            if (!evaluationFormatId)
                throw 'SERVER_ERROR_EVALUATION_FORMAT_ID_MISSING';
            logger.info('[EvaluationFormatService::get_evaluationformat_attachedsignatures] Calling GetEvaluationFormatAttachedSignatues SP');
            return mssql_service_1.MSSql.GetEvaluationFormatAttachedSignatures(vcc, evaluationFormatId);
        })
            .then(function (result) {
            if (res.headersSent)
                return;
            logger.info('[EvaluationFormatService::get_evaluationformat_attachedsignatures] Received restult from SP GetEvaluationFormatAttachedSignatues: ' + result.length);
            res.json({ status: true, description: 'SERVER_SUCCESS_MESSAGE', data: result });
        })
            .catch(function (err) {
            var clientMsg = (typeof err === 'string' ? err : 'CANNOT_PROCESS_REQUEST');
            var errorMsg = (typeof err === 'string' ? err : err.message || err.description || 'Requesting error');
            logger.error('[EvaluationFormatService::get_evaluationformat_attachedsignatures] Error on \'' + req.path + '\': ' + errorMsg);
            if (res.headersSent)
                return;
            res.json({ status: false, description: clientMsg, error: errorMsg });
        });
    });
}
exports.EvaluationFormatService = EvaluationFormatService;
function SaveEvaluationFormatContent(vcc, evaluationFormat, isNewEvaluationFormat, userId, oldTemplateId) {
    if (isNewEvaluationFormat === void 0) { isNewEvaluationFormat = false; }
    if (oldTemplateId === void 0) { oldTemplateId = null; }
    var currentBlockPromise = null;
    return (0, promises_shared_1.DummyPromise)()
        .then(function (_) {
        if (isNewEvaluationFormat)
            return Promise.resolve(true);
        else {
            currentBlockPromise = 'BLOCK_TO_DELETE_EVALUATION_FORMAT_CONTENT';
            return mssql_service_1.MSSql.DeleteEvaluationFormatContent(vcc, evaluationFormat.id, oldTemplateId);
        }
    })
        .then(function (_) {
        currentBlockPromise = 'BLOCK_TO_SAVE_EVALUATION_FORMAT_CAMPAIGNS';
        return SaveAbilityAndCAmpainEF(vcc, evaluationFormat);
    })
        .then(function (_) {
        if (['MANUAL_AUTOMATIC', 'MIXED'].includes(evaluationFormat.type)) {
            currentBlockPromise = 'BLOCK_TO_SET_EVALUATION_FORMAT_ABILTY_QUERIES';
            return SetAbilityQueriesIntoAbilityEvaluationFormat(vcc, evaluationFormat);
        }
        return Promise.resolve(evaluationFormat);
    })
        .then(function (result) {
        currentBlockPromise = 'BLOCK_TO_FIND_EVALUATION_FORMAT_TEMPLATE';
        evaluationFormat = result;
        return mssql_service_1.MSSql.GetTemplate(vcc, evaluationFormat.templateId);
    })
        .then(function (result) {
        if (!result) {
            if (['MANUAL_AUTOMATIC', 'MIXED'].includes(evaluationFormat.type)) {
                currentBlockPromise = 'BLOCK_TO_GENERATE_EVALUATION_FORMAT_TEMPLATE';
                return GenerateTemplate(vcc, evaluationFormat);
            }
        }
        return Promise.resolve(true);
    })
        .then(function (_) {
        currentBlockPromise = 'BLOCK_TO_SAVE_EVALUATION_FORMAT_TIPIFICATIONS';
        return SaveEvaluationFormatAbilityScoreTipifications(vcc, evaluationFormat, userId);
    })
        .then(function (result) {
        if (['MANUAL', 'MIXED'].includes(evaluationFormat.type)) {
            currentBlockPromise = 'BLOCK_TO_SAVE_EVALUATION_FORMAT_CUSTOM_FIELDS';
            return SaveEvaluationFormatCustomFields(vcc, evaluationFormat, userId);
        }
        return Promise.resolve(true);
    })
        .then(function (result) {
        currentBlockPromise = 'BLOCK_TO_SAVE_EVALUATION_FORMAT_ATTACHED_SIGNATURES';
        return SaveEvaluationFormatAttachedSignatures(vcc, evaluationFormat, userId);
    })
        .catch(function (err) {
        var errorMsg = (typeof err === 'string' ? err : err.message || err.description || 'Executing request error');
        logger.error("[CustomFieldService::SaveEvaluationFormatContent] Block: ".concat(currentBlockPromise, ", getting error: ").concat(errorMsg));
        return Promise.reject(false);
    });
}
function SaveAbilityAndCAmpainEF(instance, evaluationFormat) {
    var promiseArray = [];
    logger.info('[EvaluationFormatService::SaveAbilityAndCAmpainEF] Saving ' + JSON.stringify(evaluationFormat.abilityEvaluationFormats));
    if (evaluationFormat.abilityEvaluationFormats) {
        evaluationFormat.abilityEvaluationFormats.forEach(function (abilityEvaluationFormat) {
            promiseArray.push(mssql_service_1.MSSql.SaveAbilityEvaluationFormat(instance, evaluationFormat.id, abilityEvaluationFormat.idAbility, abilityEvaluationFormat.idGroup, abilityEvaluationFormat.idSubGroup, abilityEvaluationFormat.penalizes, abilityEvaluationFormat.score, abilityEvaluationFormat.order, abilityEvaluationFormat.conditional));
        });
    }
    if (evaluationFormat.campainEvaluationFormats) {
        evaluationFormat.campainEvaluationFormats.forEach(function (campainEvaluationFormat) {
            promiseArray.push(mssql_service_1.MSSql.SaveCampainEvaluationFormat(instance, evaluationFormat.id, campainEvaluationFormat.idCampain));
        });
    }
    return Promise.all(promiseArray).then(function (results) {
        if (results.filter(function (result) { return (result !== true); }).length > 0) {
            return false;
        }
        else {
            return true;
        }
    });
}
function SaveEvaluationFormatAbilityScoreTipifications(vcc, evaluationFormat, userId) {
    return (0, promises_shared_1.DummyPromise)()
        .then(function (_) {
        var promiseArray = [];
        evaluationFormat.abilityEvaluationFormats.forEach(function (aef) {
            var _a;
            if ((_a = aef.abilityScores) === null || _a === void 0 ? void 0 : _a.length) {
                aef.abilityScores.forEach(function (as) {
                    var _a;
                    if ((_a = as.tipifications) === null || _a === void 0 ? void 0 : _a.length) {
                        as.tipifications.forEach(function (t) {
                            var _a, _b;
                            promiseArray.push(mssql_service_1.MSSql.SaveEvaluationFormatAbilityScoreTipification(vcc, evaluationFormat.id, as.id, t.id, t.description, t.descriptionDetail, t.order, userId));
                            if ((_b = (_a = t.tipificationQuery) === null || _a === void 0 ? void 0 : _a.queryId) === null || _b === void 0 ? void 0 : _b.length) {
                                promiseArray.push(mssql_service_1.MSSql.SaveEvaluationFormatAbilityScoreTipificationQuery(vcc, evaluationFormat.id, as.id, t.id, t.tipificationQuery.queryId, t.tipificationQuery.queryContent, t.tipificationQuery.acceptancePercentage, userId));
                            }
                            t.items.forEach(function (i) {
                                promiseArray.push(mssql_service_1.MSSql.SaveEvaluationFormatAbilityScoreTipificationItem(vcc, evaluationFormat.id, as.id, t.id, i.id, i.secondLevelItemId, i.description, i.order, userId));
                            });
                        });
                    }
                });
            }
        });
        return Promise.all(promiseArray);
    })
        .then(function (result) {
        return Promise.resolve(true);
    })
        .catch(function (err) {
        var errorMsg = (typeof err === 'string' ? err : err.message || err.description || 'Executing request error');
        logger.error('[CustomFieldService::SaveEvaluationFormatAbilityScoreTipifications] Error: ' + errorMsg);
        return Promise.reject(false);
    });
}
function SaveEvaluationFormatCustomFields(vcc, evaluationFormat, userId) {
    return (0, promises_shared_1.DummyPromise)()
        .then(function (result) {
        var promiseArray = [];
        evaluationFormat.evaluationFormatCustomFields.map(function (efcf) {
            promiseArray.push(mssql_service_1.MSSql.SaveEvaluationFormatCustomField(vcc, efcf.id, evaluationFormat.id, efcf.customField.id, efcf.order, userId, userId));
        });
        return Promise.all(promiseArray);
    })
        .then(function (result) {
        if (result.filter(function (r) { return (r !== true); }).length > 0) {
            return Promise.reject(false);
        }
        else {
            return Promise.resolve(true);
        }
    })
        .catch(function (err) {
        var errorMsg = (typeof err === 'string' ? err : err.message || err.description || 'Executing request error');
        logger.error('[CustomFieldService::SaveEvaluationFormatCustomFields] Error: ' + errorMsg);
        return Promise.reject(false);
    });
}
function SaveEvaluationFormatAttachedSignatures(vcc, evaluationFormat, userId) {
    return (0, promises_shared_1.DummyPromise)()
        .then(function (result) {
        var promiseArray = [];
        evaluationFormat.evaluationFormatAttachedSignatures.map(function (efas) {
            if (efas && efas.description && efas.description !== '')
                promiseArray.push(mssql_service_1.MSSql.SaveEvaluationFormatAttachedSignature(vcc, evaluationFormat.id, efas.description, efas.order, userId, userId));
        });
        return Promise.all(promiseArray);
    })
        .then(function (result) {
        if (result.filter(function (r) { return (r !== true); }).length > 0) {
            return Promise.reject(false);
        }
        else {
            return Promise.resolve(true);
        }
    })
        .catch(function (err) {
        var errorMsg = (typeof err === 'string' ? err : err.message || err.description || 'Executing request error');
        logger.error('[CustomFieldService::SaveEvaluationFormatAttachedSignatures] Error: ' + errorMsg);
        return Promise.reject(false);
    });
}
function SetAbilityQueriesIntoAbilityEvaluationFormat(vcc, evaluationFormat) {
    return (0, promises_shared_1.DummyPromise)()
        .then(function (result) {
        var promiseArray = [];
        evaluationFormat.abilityEvaluationFormats.map(function (aef) {
            promiseArray.push((0, ability_service_1.GetAbilityQueries)(vcc, aef.idAbility, false));
        });
        return Promise.all(promiseArray);
    })
        .then(function (result) {
        result.map(function (r, i) {
            evaluationFormat.abilityEvaluationFormats[i].queries = r;
        });
        return Promise.resolve(evaluationFormat);
    })
        .catch(function (err) {
        var errorMsg = (typeof err === 'string' ? err : err.message || err.description || 'Requesting error');
        logger.error('[EvaluationFormatService::SetAbilityQueriesIntoAbilityEvaluationFormat] ', errorMsg);
        return Promise.reject(err);
    });
}
function GenerateTemplate(instance, evaluationFormat) {
    return (0, promises_shared_1.DummyPromise)()
        .then(function (result) {
        logger.error('[EvaluationFormatService::GenerateTemplate] Creating Template');
        var template = new template_model_1.Template('custom', evaluationFormat.templateId, evaluationFormat.description, null, null, moment().utc().toDate(), evaluationFormat.createdByUserId, moment().utc().toDate(), evaluationFormat.lastModifiedByUserId);
        var i = 0;
        var category = '';
        var _loop_1 = function (aef) {
            i++;
            if (category !== aef.idGroup) {
                i = 1;
                category = aef.idGroup;
            }
            var tempTheme = new template_model_1.TemplateTheme(template.name, (0, utilities_1.guid)(), i, aef.titleAbility, aef.descriptionGroup, aef.descriptionSubGroup, moment().utc().toDate(), template.createdByUserId, moment().utc().toDate(), template.lastModifiedByUserId);
            tempTheme.isNegative = aef.isNegative;
            var tempThemeItem = null;
            if (aef.queries && aef.queries.length) {
                aef.queries.map(function (q) {
                    tempThemeItem = new template_model_1.TemplateThemeItem(template.name, tempTheme.id, q.queryId, q.queryId, moment().utc().toDate(), template.createdByUserId, moment().utc().toDate(), template.lastModifiedByUserId);
                    tempThemeItem.acceptancePercentage = q.acceptancePercentage;
                    template.templateThemeItems.push(tempThemeItem);
                });
            }
            else {
                tempThemeItem = new template_model_1.TemplateThemeItem(template.name, tempTheme.id, aef.idAbility, aef.idAbility, moment().utc().toDate(), template.createdByUserId, moment().utc().toDate(), template.lastModifiedByUserId);
                template.templateThemeItems.push(tempThemeItem);
            }
            template.templateThemes.push(tempTheme);
        };
        for (var _i = 0, _a = evaluationFormat.abilityEvaluationFormats; _i < _a.length; _i++) {
            var aef = _a[_i];
            _loop_1(aef);
        }
        return (0, template_service_1.SaveTemplate)(instance, template);
    })
        .catch(function (err) {
        var errorMsg = (typeof err === 'string' ? err : err.message || err.description || 'Requesting error');
        logger.error('[EvaluationFormatService::GenerateTemplate] ', errorMsg);
        return false;
    });
}
