"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.GenericTemplateService = void 0;
var moment = require("moment");
var log4js = require("log4js");
var mssql_service_1 = require("../db/mssql.service");
var promises_shared_1 = require("../shared/promises.shared");
var logger = log4js.getLogger('ServerScripts');
function GenericTemplateService(app) {
    app.get('/api/generic_template/get_generic_templates/:type/:search_criteria/:search_expression?', function (req, res, next) {
        logger.info('Se recibió ' + req.method + ' para \'' + req.path + '\'');
        var vcc = res.locals.vcc;
        var type = req.params.type || '';
        type = type === 'null' ? null : type;
        var search_criteria = req.params.search_criteria || '';
        var search_expression = req.params.search_expression || '';
        (0, promises_shared_1.DummyPromise)()
            .then(function (result) {
            return mssql_service_1.MSSql.GetGenericTemplates(vcc, type, search_criteria, search_expression);
        }).then(function (GenericTemplates) {
            if (res.headersSent)
                return;
            res.json({ status: true, description: 'SERVER_SUCCESS_MESSAGE', data: GenericTemplates });
        }).catch(function (err) {
            var clientMsg = (typeof err === 'string' ? err : 'CANNOT_PROCESS_REQUEST');
            var errorMsg = (typeof err === 'string' ? err : err.message || err.description || 'SERVER_ERROR_UNKNOWN_MESSAGE');
            logger.error('Error al procesar \'' + req.path + '\': ' + errorMsg);
            if (res.headersSent)
                return;
            res.json({ status: false, description: clientMsg, error: errorMsg });
        });
    });
    app.get('/api/generic_template/get_generic_template/:generic_template_id', function (req, res, next) {
        logger.info('Se recibió ' + req.method + ' para \'' + req.path + '\'');
        var vcc = res.locals.vcc;
        var genericTemplateId = req.params.generic_template_id;
        (0, promises_shared_1.DummyPromise)()
            .then(function (result) {
            if (!genericTemplateId)
                throw 'TEMPLATE_SERVER_ERROR_MISSING_ID';
            return mssql_service_1.MSSql.GetGenericTemplate(vcc, genericTemplateId);
        }).then(function (resultGenericTemplate) {
            if (res.headersSent)
                return;
            res.json({ status: true, description: 'SERVER_SUCCESS_MESSAGE', data: resultGenericTemplate });
        }).catch(function (err) {
            var clientMsg = (typeof err === 'string' ? err : 'CANNOT_PROCESS_REQUEST');
            var errorMsg = (typeof err === 'string' ? err : err.message || err.description || 'SERVER_ERROR_UNKNOWN_MESSAGE');
            logger.error('Error al procesar \'' + req.path + '\': ' + errorMsg);
            if (res.headersSent)
                return;
            res.json({ status: false, description: clientMsg, error: errorMsg });
        });
    });
    app.get('/api/generic_template/get_generic_template_queries/:generic_template_id', function (req, res, next) {
        logger.info('Se recibió ' + req.method + ' para \'' + req.path + '\'');
        var vcc = res.locals.vcc;
        var genericTemplateId = req.params.generic_template_id;
        genericTemplateId = genericTemplateId === 'null' ? '' : genericTemplateId;
        (0, promises_shared_1.DummyPromise)()
            .then(function (result) {
            return mssql_service_1.MSSql.GetGenericTemplateQueries(vcc, genericTemplateId);
        }).then(function (resultGenericTemplate) {
            if (res.headersSent)
                return;
            res.json({ status: true, description: 'SERVER_SUCCESS_MESSAGE', data: resultGenericTemplate });
        }).catch(function (err) {
            var clientMsg = (typeof err === 'string' ? err : 'CANNOT_PROCESS_REQUEST');
            var errorMsg = (typeof err === 'string' ? err : err.message || err.description || 'SERVER_ERROR_UNKNOWN_MESSAGE');
            logger.error('Error al procesar \'' + req.path + '\': ' + errorMsg);
            if (res.headersSent)
                return;
            res.json({ status: false, description: clientMsg, error: errorMsg });
        });
    });
    app.post('/api/generic_template/save_generic_template', function (req, res, next) {
        logger.info('Se recibió ' + req.method + ' para \'' + req.path + '\'');
        var vcc = res.locals.vcc;
        var userId = res.locals.userId;
        var nowUtc = moment().utc().toDate();
        var newGenericTemplate = (req.body || {});
        newGenericTemplate.createdDate = null;
        newGenericTemplate.createdByUserId = null;
        newGenericTemplate.lastModifiedDate = nowUtc;
        newGenericTemplate.lastModifiedByUserId = userId;
        (0, promises_shared_1.DummyPromise)()
            .then(function (result) {
            return mssql_service_1.MSSql.DeleteGenericTemplate(vcc, newGenericTemplate.name);
        }).then(function (resultDeleteGenericTemplate) {
            if (!resultDeleteGenericTemplate)
                throw 'TEMPLATE_SERVER_ERROR_ON_SAVE';
            return SaveGenericTemplate(vcc, newGenericTemplate);
        }).then(function (resultSaveGenericTemplate) {
            if (!resultSaveGenericTemplate)
                throw 'TEMPLATE_SERVER_ERROR_ON_SAVE';
            return Promise.resolve(true);
        }).then(function (resultSaveGenericTemplate) {
            if (res.headersSent)
                return;
            res.json({ status: resultSaveGenericTemplate, description: (resultSaveGenericTemplate ? 'SERVER_SUCCESS_MESSAGE' : 'TEMPLATE_SERVER_ERROR_ON_SAVE'), data: resultSaveGenericTemplate });
        }).catch(function (err) {
            var clientMsg = (typeof err === 'string' ? err : 'CANNOT_PROCESS_REQUEST');
            var errorMsg = (typeof err === 'string' ? err : err.message || err.description || 'SERVER_ERROR_UNKNOWN_MESSAGE');
            logger.error('Error al procesar \'' + req.path + '\': ' + errorMsg);
            if (res.headersSent)
                return;
            res.json({ status: false, description: clientMsg, error: errorMsg });
        });
    });
}
exports.GenericTemplateService = GenericTemplateService;
function SaveGenericTemplate(vcc, genericTemplate) {
    var promiseArray = [];
    promiseArray.push(mssql_service_1.MSSql.SaveGenericTemplate(vcc, genericTemplate.name, genericTemplate.type, genericTemplate.description, genericTemplate.template, genericTemplate.category, genericTemplate.createdDate, genericTemplate.createdByUserId, genericTemplate.lastModifiedDate, genericTemplate.lastModifiedByUserId));
    if (genericTemplate.genericTemplateQueries) {
        genericTemplate.genericTemplateQueries.forEach(function (genericTemplateQuery) {
            promiseArray.push(mssql_service_1.MSSql.SaveGenericTemplateQuery(vcc, genericTemplateQuery.templateId, genericTemplateQuery.id, genericTemplateQuery.query, moment().utc().toDate(), genericTemplateQuery.createdByUserId, moment().utc().toDate(), genericTemplateQuery.lastModifiedByUserId));
        });
    }
    return Promise.all(promiseArray).then(function (results) {
        if (results.filter(function (result) { return (result !== true); }).length > 0) {
            return false;
        }
        else {
            return true;
        }
    });
}
