"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.GlobalSearchService = void 0;
var log4js = require("log4js");
var mssql_service_1 = require("../db/mssql.service");
var promises_shared_1 = require("../shared/promises.shared");
var arrays_shared_1 = require("../shared/arrays.shared");
var logger = log4js.getLogger('ServerScripts');
function GlobalSearchService(app) {
    app.post('/api/global_search/get_global_search_filters', function (req, res, next) {
        logger.info('[GlobalSearchService::globalSerach] Received ' + req.method + ' method on \'' + req.path + '\'');
        var vcc = res.locals.instance;
        (0, promises_shared_1.DummyPromise)()
            .then(function (result) {
            return mssql_service_1.MSSql.GetGlobalSearchFilters(vcc);
        }).then(function (result) {
            var _a;
            if (!result)
                throw 'SERVER_ERROR_GLOBAL_SEARCH_FAILED';
            if ((_a = result === null || result === void 0 ? void 0 : result.agents) === null || _a === void 0 ? void 0 : _a.length) {
                result.agents = (0, arrays_shared_1.GetUniqueFromObjectArray)(result.agents, 'agentId');
            }
            if (res.headersSent)
                return;
            res.json({ status: true, description: 'SERVER_SUCCESS_MESSAGE', data: result });
        }).catch(function (err) {
            var clientMsg = (typeof err === 'string' ? err : 'CANNOT_PROCESS_REQUEST');
            var errorMsg = (typeof err === 'string' ? err : err.message || err.description || 'SERVER_ERROR_UNKNOWN_MESSAGE');
            logger.error('GlobalSearchService::globalSearch] Error on \'' + req.path + '\': ' + errorMsg);
            if (res.headersSent)
                return;
            res.json({ status: false, description: clientMsg, error: errorMsg });
        });
    });
    app.post('/api/global_search/get_global_search_interactions', function (req, res, next) {
        logger.info('[GlobalSearchService::globalSerachInteractions] Received ' + req.method + ' method on \'' + req.path + '\'');
        var vcc = res.locals.instance;
        var campaigns = req.body.campaigns ? req.body.campaigns : [];
        var agents = req.body.agents ? req.body.agents : [];
        var dateFrom = req.body.dateFrom ? req.body.dateFrom : [];
        var dateTo = req.body.dateTo ? req.body.dateTo : [];
        var origins = req.body.origins ? req.body.origins : [];
        var interactionTypes = req.body.interactionTypes ? req.body.interactionTypes : [];
        var interactionIds = req.body.interactionIds ? req.body.interactionIds : '';
        var contactAddress = req.body.contactAddress ? req.body.contactAddress : '';
        var dispositionCodes = req.body.dispositionCodes ? req.body.dispositionCodes : [];
        logger.debug("dataArturoVelaEnviando", vcc, agents, campaigns, dateFrom, dateTo, origins, interactionTypes, interactionIds, contactAddress, dispositionCodes);
        (0, promises_shared_1.DummyPromise)()
            .then(function (result) {
            return mssql_service_1.MSSql.GetGlobalSearchInteractions(vcc, agents, campaigns, dateFrom, dateTo, origins, interactionTypes, interactionIds, contactAddress, dispositionCodes);
        }).then(function (result) {
            if (!result)
                throw 'SERVER_ERROR_GLOBAL_SEARCH_INTERACTIONS_FAILED';
            if (res.headersSent)
                return;
            res.json({ status: true, description: 'SERVER_SUCCESS_MESSAGE', data: result });
        }).catch(function (err) {
            var clientMsg = (typeof err === 'string' ? err : 'CANNOT_PROCESS_REQUEST');
            var errorMsg = (typeof err === 'string' ? err : err.message || err.description || 'SERVER_ERROR_UNKNOWN_MESSAGE');
            logger.error('GlobalSearchService::globalSearchInteractions] Error on \'' + req.path + '\': ' + errorMsg);
            if (res.headersSent)
                return;
            res.json({ status: false, description: clientMsg, error: errorMsg });
        });
    });
}
exports.GlobalSearchService = GlobalSearchService;
