"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.InteractionTypeService = void 0;
var log4js = require("log4js");
var mssql_service_1 = require("../db/mssql.service");
var vcc_service_1 = require("./vcc.service");
var promises_shared_1 = require("../shared/promises.shared");
var http_request_occ_1 = require("../shared/http-request-occ");
var redis_service_1 = require("../db/redis.service");
var logger = log4js.getLogger('ServerScripts');
function InteractionTypeService(app) {
    app.post('/api/interaciontype/get_interactiontypes', function (req, res, next) {
        logger.info('[InteractionTypeService::get_interactiontypes] Received ' + req.method + ' method for \'' + req.path + '\'');
        var _a = res.locals, vcc = _a.vcc, type = _a.type;
        var campaigns = req.body.campaigns || [];
        var channels = [];
        var isI6OrOcc = type === 'I6' || type === 'OCC' ? true : false;
        var dataRedisVcc;
        logger.info('[InteractionTypeService::get_interactiontypes] Requesting interactionTypes for vcc: ' + vcc);
        logger.info('[InteractionTypeService::get_interactiontypes] Requesting interactionTypes for campaigns: ' + campaigns);
        (0, promises_shared_1.DummyPromise)()
            .then(function (_) {
            return redis_service_1.Redis.GetVCC(vcc, vcc);
        })
            .then(function (redisVCC) {
            var _a;
            dataRedisVcc = redisVCC;
            if ((_a = dataRedisVcc === null || dataRedisVcc === void 0 ? void 0 : dataRedisVcc.channels) === null || _a === void 0 ? void 0 : _a.length) {
                channels = redisVCC.channels;
                return Promise.resolve([]);
            }
            var promiseArray = [];
            promiseArray.push(mssql_service_1.MSSql.GetInteractionTypes(vcc, null, null));
            if (isI6OrOcc)
                promiseArray.push((0, http_request_occ_1.ApiI6ConfiguredChannels)(vcc, campaigns));
            return Promise.all(promiseArray);
        })
            .then(function (result) {
            var _a;
            if (channels.length)
                return Promise.resolve(true);
            channels = ((_a = result[0]) === null || _a === void 0 ? void 0 : _a.length) ? result[0] : [];
            if (isI6OrOcc && Array.isArray(result[1].body)) {
                var arrayInteractionTypeAdd_1 = [];
                result[1].body.forEach(function (insApi) {
                    if (!channels.find(function (insDb) { return insDb.id === insApi.id; })) {
                        arrayInteractionTypeAdd_1.push(insApi);
                        channels.push(insApi);
                    }
                });
                return (0, vcc_service_1.SaveInteractionTypes)(arrayInteractionTypeAdd_1, vcc);
            }
            return Promise.resolve(true);
        })
            .then(function (_) {
            var _a;
            if ((_a = dataRedisVcc === null || dataRedisVcc === void 0 ? void 0 : dataRedisVcc.channels) === null || _a === void 0 ? void 0 : _a.length) {
                return Promise.resolve([]);
            }
            dataRedisVcc.channels = channels;
            return redis_service_1.Redis.SaveVCC(vcc, dataRedisVcc);
        })
            .then(function (result) {
            if (res.headersSent)
                return;
            logger.info('[InteractionTypeService::get_interactiontypes] Received interactionTypes for instance: ' + vcc);
            res.json({ status: true, description: 'SERVER_SUCCESS_MESSAGE', data: channels });
        })
            .catch(function (err) {
            var msg = (typeof err === 'string' ? err : 'CANNOT_PROCESS_REQUEST');
            var errorMsg = (typeof err === 'string' ? err : err.message || err.description || 'SERVER_ERROR_UNKNOWN_MESSAGE');
            logger.error('[InteractionTypeService::get_interactiontypes] Error \'' + req.path + '\': ' + errorMsg);
            if (res.headersSent)
                return;
            res.json({ status: false, description: msg, error: errorMsg });
        });
    });
}
exports.InteractionTypeService = InteractionTypeService;
