"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.CloseI6TextInteraction = exports.CloseI6CallInteraction = exports.GetEmlDataFromI6Data = exports.GetHtmlDataFromI6Data = exports.GetJsonDataFromI6Data = exports.UpdateI6Interaction = exports.GetI6InteractionsFromRedis = exports.GetI6InteractionsFromApi = exports.GetChannelOldInteractionSummaryPromises = exports.GetChannelNewInteractionSummaryPromises = exports.GetInteractionSearchFromProcessFilter = exports.GetProcessFilterFromInteractionSearch = exports.GetInteractionSearchFromRequest = exports.CreateLocalInteractionFileContent = exports.GetLocalInteractionFileContent = exports.GetLocalInteractionFile = exports.InteractionService = void 0;
var moment = require("moment");
var log4js = require("log4js");
var fs = require("fs");
var ExternalDB = require("../db/external");
var interaction_model_1 = require("../models/interaction.model");
var mssql_service_1 = require("../db/mssql.service");
var redis_service_1 = require("../db/redis.service");
var arrays_shared_1 = require("../shared/arrays.shared");
var http_request_occ_1 = require("../shared/http-request-occ");
var promises_shared_1 = require("../shared/promises.shared");
var config_1 = require("../config/config");
var bluebird_1 = require("bluebird");
var integration_config_1 = require("../config/integration.config");
var mssql_config_1 = require("../config/mssql.config");
var process_model_1 = require("../models/process.model");
var audio_transfer_implementation_1 = require("../implementation/audio-transfer.implementation");
var file_manager_1 = require("../shared/file-manager");
var semantic_analysis_implementation_1 = require("../implementation/semantic-analysis.implementation");
var role_service_1 = require("./role.service");
var logger = log4js.getLogger('ServerScripts');
var httpRequestOccCompatible = true;
function InteractionService(app) {
    app.get('/api/interaction/get_interaction/:evaluationPlanId/:type/:id?', function (req, res, next) {
        logger.info('[InteractionService::get_interaction] Received ' + req.method + ' method on \'' + req.path + '\'');
        var vcc = res.locals.vcc;
        var evaluationPlanId = req.params.evaluationPlanId || '';
        var type = req.params.type || '';
        var id = req.params.id || '';
        var environmentId = null;
        var environmentType = null;
        var interaction = null;
        (0, promises_shared_1.DummyPromise)()
            .then(function (result) {
            if (!evaluationPlanId) {
                logger.error('[EvaluationPlanService::get_evaluationplan] Received evaluation plan ID is null');
                throw 'SERVER_ERROR_EVALUATION_PLAN_ID_MISSING';
            }
            logger.info('[EvaluationPlanService::get_evaluationplan] Sending request to SP GetEvaluationPlan for EP ID: ' + evaluationPlanId);
            return mssql_service_1.MSSql.GetEvaluationPlan(vcc, evaluationPlanId);
        })
            .then(function (result) {
            if (!result)
                throw 'SERVER_ERROR_EVALUATION_PLAN_NOT_FOUND';
            environmentId = result.environmentId;
            environmentType = result.environmentType;
            if (['ftp', 'sftp', 's3'].includes(environmentType)) {
                logger.info('[InteractionService::get_interaction] Requesting environment data');
                return mssql_service_1.MSSql.GetEnvironment(vcc, environmentId);
            }
            else {
                return Promise.resolve(true);
            }
        })
            .then(function (result) {
            if (['ftp', 'sftp', 's3'].includes(environmentType)) {
                if (!result)
                    throw 'SERVER_ERROR_ENVIRONMENT_NOT_FOUND';
                environmentId = result.name;
                environmentType = result.type;
            }
            return GetEvaluationPlanInteraction(vcc, type, environmentType, evaluationPlanId, id);
        })
            .then(function (result) {
            if (!result)
                throw 'SERVER_ERROR_INTERACTION_EVALUATION_PLAN_NOT_FOUND';
            interaction = result;
            if (type === 'CALL' && environmentType && environmentType === 'allegro') {
                fillInteractionContentURL(interaction, vcc);
            }
            return interaction;
        })
            .then(function (result) {
            if (type === 'CALL') {
                if (environmentType && environmentType === 'allegro') {
                    return ExternalDB.inconcert.getCampaignWebdesignerURL(vcc, result.idCampaign, type).then(function (url) {
                        result.managementFormURL = (url && url.length > 0) ? config_1.INCONCERT_APPS_BASE_URL + 'webdesigner/' + url + '?INTERACTIONID=' + result.id + '&AGENTID=' + result.idAgent + '&VIRTUALCC=' + vcc + '&OUTBOUNDPROCESS=&OUTBOUNDBATCH=&SOURCE=QUALITY' : '';
                        return Promise.resolve(result);
                    });
                }
                else {
                    return Promise.resolve(result);
                }
            }
            else {
                return result;
            }
        })
            .then(function (result) {
            if (type === 'CALL') {
                var i_1 = result;
                return mssql_service_1.MSSql.GetAllTagsInteraction(vcc, id).then(function (result) {
                    i_1.tags = result;
                    return Promise.resolve(i_1);
                }).catch(function (err) {
                    i_1.tags = [];
                    return Promise.resolve(i_1);
                });
            }
            else {
                return result;
            }
        })
            .then(function (result) {
            if (res.headersSent)
                return;
            res.json({ status: true, description: 'SERVER_SUCCESS_MESSAGE', data: result });
        })
            .catch(function (err) {
            if (typeof err === 'string'
                &&
                    ['SERVER_ERROR_INTERACTION_EVALUATION_PLAN_NOT_FOUND'].includes(err)) { }
            var msg = (typeof err === 'string' ? err : 'CANNOT_PROCESS_REQUEST');
            var errorMsg = (typeof err === 'string' ? err : err.message || err.description || 'Requesting error');
            logger.error('[InteractionService::get_interaction] Error on \'' + req.path + '\': ' + errorMsg);
            if (res.headersSent)
                return;
            res.json({ status: false, description: msg, error: errorMsg });
        });
    });
    app.post('/api/interaction/get_not_evaluated_interactions_auditor/:idAuditor', function (req, res, next) {
        logger.info('[InteractionService::get_not_evaluated_interactions_auditor] Received ' + req.method + ' method on \'' + req.path + '\'');
        var vcc = res.locals.instance;
        var userRole = res.locals.user.role;
        var userId = res.locals.userId;
        var params = new interaction_model_1.InteractionSearch;
        var evaluationsToUpdate = [];
        params.campaigns = req.body.campaigns || [];
        params.agents = req.body.agents || [];
        params.evaluationPlans = req.body.evaluationPlans || [];
        params.types = req.body.types || [];
        params.auditors = req.body.auditors || [];
        params.results = req.body.results ? req.body.results.map(function (r) {
            var dispositionCode = null;
            if (typeof r === 'string') {
                dispositionCode = r;
            }
            else {
                dispositionCode = r.campaignId;
                if (r.description)
                    dispositionCode += '_' + r.description;
                if (r.description2)
                    dispositionCode += '_' + r.description2;
                if (r.description3)
                    dispositionCode += '_' + r.description3;
                if (r.description4)
                    dispositionCode += '_' + r.description4;
            }
            return dispositionCode;
        }) : [];
        params.dateEvaluationPlanFin = req.body.dateLimit || '';
        params.evaluationStatus = req.body.evaluationStatus || [true];
        params.interactionIds = req.body.interactionIds || [];
        params.contactAddress = req.body.contactAddress || '';
        params.efectivityMode = req.body.efectivityMode;
        params.efectivityPlans = req.body.efectivityPlans || [];
        params.dateIni = req.body.dateIni;
        params.dateFin = req.body.dateFin;
        logger.debug('[InteractionService::get_not_evaluated_interactions_auditor] Received params: :' + JSON.stringify(params));
        (0, promises_shared_1.DummyPromise)()
            .then(function (_) {
            return (0, role_service_1.GetRoleFunctions)(vcc, userId, userRole);
        })
            .then(function (result) {
            if (result && result.data && result.data.hasAdminFunctions)
                params.evaluationStatus.push(false);
            return redis_service_1.Redis.GetEvaluationsStatus(vcc);
        })
            .then(function (result) {
            evaluationsToUpdate = result ? result : [];
            logger.info('[InteractionService::get_not_evaluated_interactions_auditor] Requesting interactions(SP GetNotEvaluatedInteractionsAuditor) for VCC|idAuditor: ' + vcc + '|' + userId);
            if (params.efectivityMode) {
                return mssql_service_1.MSSql.GetNotEvaluatedInteractionsEfectivityAuditor(vcc, userId, params.campaigns, params.agents, params.evaluationPlans, params.types, params.results, params.dateEvaluationPlanFin, params.evaluationStatus.length && params.evaluationStatus.length > 1 ? null : params.evaluationStatus, params.interactionIds.toString(), params.contactAddress.toString(), params.efectivityPlans, params.dateIni, params.dateFin);
            }
            else {
                return mssql_service_1.MSSql.GetNotEvaluatedInteractionsAuditor(vcc, userId, params.campaigns, params.agents, params.evaluationPlans, params.types, params.results, params.dateEvaluationPlanFin, params.evaluationStatus.length && params.evaluationStatus.length > 1 ? null : params.evaluationStatus, params.interactionIds.toString(), params.contactAddress.toString(), params.dateIni, params.dateFin, params.auditors);
            }
        })
            .then(function (result) {
            if (result) {
                result.map(function (r) {
                    r.isEvaluationBeingEvaluated = evaluationsToUpdate.find(function (etu) { return etu.evaluationPlanId === r.idEvaluationPlan && etu.interactionId === r.id && etu.auditorId === r.idAuditor; }) ? true : false;
                    r.isInteractionBeingEvaluated = evaluationsToUpdate.find(function (etu) { return etu.evaluationPlanId === r.idEvaluationPlan && etu.interactionId === r.id; }) ? true : false;
                });
            }
            if (res.headersSent)
                return;
            logger.debug('[InteractionService::get_not_evaluated_interactions_auditor] Received response from SP GetNotEvaluatedInteractionsAuditor: ' + JSON.stringify(result ? result.length : 0));
            res.json({ status: true, description: 'SERVER_SUCCESS_MESSAGE', data: result || [] });
        })
            .catch(function (err) {
            var msg = (typeof err === 'string' ? err : 'CANNOT_PROCESS_REQUEST');
            var errorMsg = (typeof err === 'string' ? err : err.message || err.description || 'Requesting error');
            logger.error('[InteractionService::get_not_evaluated_interactions_auditor] Error on \'' + req.path + '\': ' + errorMsg);
            if (res.headersSent)
                return;
            res.json({ status: false, description: msg, error: errorMsg });
        });
    });
    app.post('/api/interaction/get_evaluated_interactions_auditor/', function (req, res, next) {
        logger.info('[InteractionService::get_evaluated_interactions_auditor] Received ' + req.method + ' method on \'' + req.path + '\'');
        var instance = res.locals.instance;
        var idAuditor = res.locals.userId;
        logger.info('[InteractionService::get_evaluated_interactions_auditor] Filling the filter\'s object');
        var params = new interaction_model_1.InteractionSearch;
        params.dateIni = req.body.dateIni || '';
        params.dateFin = req.body.dateFin || '';
        params.campaigns = req.body.campaigns || [];
        params.evaluationPlans = req.body.evaluationPlans || [];
        params.agents = req.body.agents || [];
        params.origins = req.body.origins || [];
        params.minDuration = req.body.minDuration || null;
        params.maxDuration = req.body.maxDuration || null;
        params.evaluationTypes = req.body.evaluationTypes || [];
        params.tags = req.body.tags || [];
        params.hasVideoOptions = req.body.hasVideoOptions || [];
        params.results = req.body.results || [];
        params.hourIni = req.body.hourIni || '';
        params.hourFin = req.body.hourFin || '';
        params.multimediaFormats = ['mp3', 'wav', 'ogg'];
        if (params.hasVideoOptions.includes(true)) {
            params.multimediaFormats.push('flv');
        }
        params.origins = params.origins.map(function (o) {
            if (o == "E")
                return "IN";
            else if (o == "S")
                return "OUT";
            else
                return null;
        });
        params.interactionIds = req.body.interactionIds || [];
        params.contactAddress = req.body.contactAddress || '';
        logger.debug('[InteractionService::get_evaluated_interactions_auditor] Filter\'s object: ', JSON.stringify(params));
        (0, promises_shared_1.DummyPromise)()
            .then(function (result) {
            logger.info('[InteractionService::get_evaluated_interactions_auditor] Calling GetEvaluations SP for VCC|IdAuditor: ' + instance + '|' + idAuditor);
            return mssql_service_1.MSSql.GetEvaluatedInteractionsByAuditor(instance, idAuditor, params.agents.toString(), params.campaigns.toString(), params.evaluationPlans.toString(), params.dateIni, params.dateFin, params.hourIni, params.hourFin, params.minDuration, params.maxDuration, params.origins.toString(), params.results.toString(), params.evaluationTypes.toString(), params.tags.toString(), params.multimediaFormats.toString(), params.interactionIds.toString());
        })
            .then(function (result) {
            if (res.headersSent)
                return;
            logger.debug('[InteractionService::get_evaluated_interactions_auditor] Received result: ' + JSON.stringify(result));
            logger.info('[InteractionService::get_evaluated_interactions_auditor] Sending results: ' + result.length);
            res.json({ status: true, description: 'SERVER_SUCCESS_MESSAGE', data: result });
        })
            .catch(function (err) {
            (function (err) {
                var msg = (typeof err === 'string' ? err : 'CANNOT_PROCESS_REQUEST');
                var errorMsg = (typeof err === 'string' ? err : err.message || err.description || 'Error requesting');
                logger.error('[InteractionService::get_evaluated_interactions_auditor] Error on \'' + req.path + '\': ' + errorMsg);
                if (res.headersSent)
                    return;
                res.json({ status: false, description: msg, error: errorMsg });
            });
        });
    });
    app.get('/api/interaction/get_evaluated_interactions/:idAgent?', function (req, res, next) {
        logger.info('Se recibió ' + req.method + ' para \'' + req.path + '\'');
        var instance = res.locals.instance;
        var evaluationDateFrom = req.query.evaluationDateFrom ? req.query.evaluationDateFrom.toString() : '1980-01-01';
        var evaluationDateTo = req.query.evaluationDateTo ? req.query.evaluationDateTo.toString() : '2050-01-01';
        var idAgent = req.params.idAgent || '';
        var campaigns = req.query.campaigns ? req.query.campaigns.toString() : 'GetEvaluatedInteractions';
        (0, promises_shared_1.DummyPromise)()
            .then(function (result) {
            return mssql_service_1.MSSql.GetEvaluatedInteractions(instance, idAgent, campaigns, evaluationDateFrom, evaluationDateTo);
        }).then(function (result) {
            var interactions = result;
            var p = [];
            interactions.forEach(function (i) {
                p.push(ExternalDB.interactions.getInteractionByTypeAndId(instance, i.interactionType, i.id)
                    .then(function (r) {
                    i.address = r.address;
                    i.dispositionCode = r.dispositionCode;
                    i.origin = r.origin;
                    i.hasVideo = r.hasVideo;
                    return i;
                }).catch(function (err) {
                    var errorMsg = (typeof err === 'string' ? err : err.message || err.description || 'SERVER_ERROR_UNKNOWN_MESSAGE');
                    logger.error('Error obteniendo info de las interacciones: ' + errorMsg);
                    throw err;
                }));
            });
            return Promise.all(p).then(function (values) {
                return [].concat.apply([], values);
            });
        }).then(function (result) {
            var interactions = result || [];
            return fillInteractionTags(instance, interactions);
        }).then(function (result) {
            if (res.headersSent)
                return;
            res.json({ status: true, description: 'SERVER_SUCCESS_MESSAGE', data: result });
        }).catch(function (err) {
            var msg = (typeof err === 'string' ? err : 'CANNOT_PROCESS_REQUEST');
            var errorMsg = (typeof err === 'string' ? err : err.message || err.description || 'SERVER_ERROR_UNKNOWN_MESSAGE');
            logger.error('Error al procesar \'' + req.path + '\': ' + errorMsg);
            if (res.headersSent)
                return;
            res.json({ status: false, description: msg, error: errorMsg });
        });
    });
    app.post('/api/interaction/get_evaluated_interactions_by_campaigns_auditors_and_agents?', function (req, res, next) {
        logger.info('[InteractionService::get_evaluated_interactions_by_campaigns_auditors_and_agents] Received ' + req.method + ' method on \'' + req.path + '\'');
        var instance = res.locals.instance;
        logger.info('[InteractionService::get_evaluated_interactions_by_campaigns_auditors_and_agents] Filling the filter\'s object');
        var params = new interaction_model_1.InteractionSearch;
        params.dateIni = req.body.dateIni || '';
        params.dateFin = req.body.dateFin || '';
        params.campaigns = req.body.campaigns || [];
        params.auditors = req.body.auditors || [];
        params.agents = req.body.agents || [];
        params.origins = req.body.origins || [];
        params.minDuration = req.body.minDuration || null;
        params.maxDuration = req.body.maxDuration || null;
        params.evaluationTypes = req.body.evaluationTypes || [];
        params.tags = req.body.tags || [];
        params.hasVideoOptions = req.body.hasVideoOptions || [];
        params.results = req.body.results || [];
        params.hourIni = req.body.hourIni || '';
        params.hourFin = req.body.hourFin || '';
        params.contactAddress = req.body.contactAddress || '';
        params.multimediaFormats = ['mp3', 'wav', 'ogg'];
        if (params.hasVideoOptions.includes(true)) {
            params.multimediaFormats.push('flv');
        }
        params.origins = params.origins.map(function (o) {
            if (o == "E")
                return "IN";
            else if (o == "S")
                return "OUT";
            else
                return null;
        });
        logger.debug('[InteractionService::get_evaluated_interactions_by_campaigns_auditors_and_agents] Filter\'s object: ', JSON.stringify(params));
        (0, promises_shared_1.DummyPromise)()
            .then(function (result) {
            logger.debug('[InteractionService::get_evaluated_interactions_by_campaigns_auditors_and_agents] Calling GetEvaluatedInteractionsByCampaignsAuditorsAndAgents SP');
            return mssql_service_1.MSSql.GetEvaluatedInteractionsByCampaignsAuditorsAndAgents(instance, params.auditors.toString(), params.agents.toString(), params.campaigns.toString(), params.dateIni, params.dateFin, params.hourIni, params.hourFin, params.minDuration, params.maxDuration, params.origins.toString(), params.results.toString(), params.evaluationTypes.toString(), params.tags.toString(), params.multimediaFormats.toString(), params.contactAddress.toString());
        })
            .then(function (result) {
            if (res.headersSent)
                return;
            logger.debug('[InteractionService::get_evaluated_interactions_by_campaigns_auditors_and_agents] Results from GetEvaluatedInteractionsByCampaignsAuditorsAndAgents SP: ' + result.length);
            res.json({ status: true, description: 'SERVER_SUCCESS_MESSAGE', data: result });
        }).catch(function (err) {
            var msg = (typeof err === 'string' ? err : 'CANNOT_PROCESS_REQUEST');
            var errorMsg = (typeof err === 'string' ? err : err.message || err.description || 'SERVER_ERROR_UNKNOWN_MESSAGE');
            logger.error('[InteractionService::get_evaluated_interactions_by_campaigns_auditors_and_agents] Error on \'' + req.path + '\': ' + errorMsg);
            if (res.headersSent)
                return;
            res.json({ status: false, description: msg, error: errorMsg });
        });
    });
    app.post('/api/interaction/get_evaluated_interactions_agent?', function (req, res, next) {
        logger.info('[InteractionService::get_evaluated_interactions_agent] Received ' + req.method + ' on \'' + req.path + '\'');
        var instance = res.locals.instance;
        var idAgent = res.locals.userId;
        logger.info('[InteractionService::get_evaluated_interactions_agent] Filling the filter\'s object');
        var params = new interaction_model_1.InteractionSearch;
        params.dateIni = req.body.dateIni || '';
        params.dateFin = req.body.dateFin || '';
        params.campaigns = req.body.campaigns || [];
        params.evaluationPlans = req.body.evaluationPlans || [];
        params.origins = req.body.origins || [];
        params.minDuration = req.body.minDuration || null;
        params.maxDuration = req.body.maxDuration || null;
        params.evaluationTypes = req.body.evaluationTypes || [];
        params.tags = req.body.tags || [];
        params.hasVideoOptions = req.body.hasVideoOptions || [];
        params.results = req.body.results || [];
        params.hourIni = req.body.hourIni || '';
        params.hourFin = req.body.hourFin || '';
        params.multimediaFormats = ['mp3', 'wav', 'ogg'];
        if (params.hasVideoOptions.includes(true)) {
            params.multimediaFormats.push('flv');
        }
        params.origins = params.origins.map(function (o) {
            if (o == "E")
                return "IN";
            else if (o == "S")
                return "OUT";
            else
                return null;
        });
        params.interactionIds = req.body.interactionIds || [];
        params.contactAddress = req.body.contactAddress || '';
        logger.debug('[InteractionService::get_evaluated_interactions_agent] Filter\'s object: ', JSON.stringify(params));
        (0, promises_shared_1.DummyPromise)()
            .then(function (result) {
            logger.info('[InteractionService::get_evaluated_interactions_agent] Calling GetEvaluatedInteractions SP for VCC|idAgent: ' + instance + '|' + idAgent);
            return mssql_service_1.MSSql.GetEvaluatedInteractionsByAuditor(instance, null, idAgent, params.campaigns.toString(), params.evaluationPlans.toString(), params.dateIni, params.dateFin, params.hourIni, params.hourFin, params.minDuration, params.maxDuration, params.origins.toString(), params.results.toString(), params.evaluationTypes.toString(), params.tags.toString(), params.multimediaFormats.toString(), params.interactionIds.toString());
        }).then(function (result) {
            if (res.headersSent)
                return;
            res.json({ status: true, description: 'SERVER_SUCCESS_MESSAGE', data: result });
        }).catch(function (err) {
            var msg = (typeof err === 'string' ? err : 'CANNOT_PROCESS_REQUEST');
            var errorMsg = (typeof err === 'string' ? err : err.message || err.description || 'Requesting error');
            logger.error('[InteractionService::get_evaluated_interactions_agent] Error on \'' + req.path + '\': ' + errorMsg);
            if (res.headersSent)
                return;
            res.json({ status: false, description: msg, error: errorMsg });
        });
    });
    app.get('/api/interaction/get_interactions_evaluationplan/:idEvaluationPlan/:idAuditor?', function (req, res, next) {
        logger.info('Se recibió ' + req.method + ' para \'' + req.path + '\'');
        var instance = res.locals.instance;
        var idEvaluationPlan = req.params.idEvaluationPlan || '';
        var idAuditor = req.params.idAuditor || '';
        var interactions;
        (0, promises_shared_1.DummyPromise)()
            .then(function (result) {
            return mssql_service_1.MSSql.GetInteractionsEvaluationPlan(instance, idEvaluationPlan, idAuditor);
        }).then(function (result) {
            if (res.headersSent)
                return;
            res.json({ status: true, description: 'SERVER_SUCCESS_MESSAGE', data: result });
        }).catch(function (err) {
            var msg = (typeof err === 'string' ? err : 'CANNOT_PROCESS_REQUEST');
            var errorMsg = (typeof err === 'string' ? err : err.message || err.description || 'SERVER_ERROR_UNKNOWN_MESSAGE');
            logger.error('Error al procesar \'' + req.path + '\': ' + errorMsg);
            if (res.headersSent)
                return;
            res.json({ status: false, description: msg, error: errorMsg });
        });
    });
    app.post('/api/interaction/get_interactions', function (req, res, next) {
        logger.info('[InteractionService::get_interactions] Received ' + req.method + ' method on \'' + req.path + '\'');
        var vcc = res.locals.vcc;
        var type = res.locals.type;
        var userId = res.locals.userId;
        var limit = 10;
        var params = new interaction_model_1.InteractionSearch;
        params = GetInteractionSearchFromRequest(req.body.evaluationFormatId, req.body, false);
        params.searchType = 'TO_SEARCH';
        (0, promises_shared_1.DummyPromise)()
            .then(function (_) {
            if (params.evaluationFormatId && (params.campaigns && params.campaigns.length === 0)) {
                logger.info('[InteractionService::get_interactions] Searching evaluation format\'s campaigns');
                return mssql_service_1.MSSql.GetCampaignsEvaluationFormat(vcc, params.evaluationFormatId);
            }
            else {
                return (0, bluebird_1.resolve)(true);
            }
        })
            .then(function (result) {
            var _a;
            if (params.environmentType && ['allegro', 'i6'].includes(params.environmentType)) {
                if (params.evaluationFormatId && ((_a = params === null || params === void 0 ? void 0 : params.campaigns) === null || _a === void 0 ? void 0 : _a.length) === 0) {
                    if (!result || result.length === 0)
                        throw 'SERVER_ERROR_EVALUATION_FORMAT_CAMPAIGNS_NOT_FOUND';
                    params.campaigns = result.map(function (r) { return r.id; });
                }
            }
            return Promise.resolve(true);
        })
            .then(function (result) {
            if (params.environmentType && ['allegro', 'i6', 'i-ftp', 'i-sftp'].includes(params.environmentType)) {
                return Promise.resolve(true);
            }
            else {
                var ftpCSVRedisKey = void 0;
                if (params.evaluationFormatId)
                    ftpCSVRedisKey = 'FTP_EVALUATION_PLAN:';
                else
                    ftpCSVRedisKey = 'FTP_PROCESS:';
                return redis_service_1.Redis.GetCsvProcess(vcc, ftpCSVRedisKey + userId);
            }
        })
            .then(function (result) {
            if (['ftp', 'sftp', 's3'].includes(params.environmentType))
                if (result === null)
                    throw 'SERVER_ERROR_CSV_DATA_EXPIRED';
            var promiseArray = [];
            logger.info('[InteractionService::get_interactions] Requesting not evaluated interactions from HistoricalData');
            promiseArray = GetChannelNewInteractionSummaryPromises(vcc, type, true, 'SUMMARIZED', params, result.data);
            return Promise.all(promiseArray);
        })
            .then(function (result) {
            if (res.headersSent)
                return;
            logger.info('[InteractionService::get_interactions] Received interactions from database');
            res.json({
                status: true,
                description: 'SERVER_SUCCESS_MESSAGE',
                data: result.filter(function (r) { return r; })
            });
        })
            .catch(function (err) {
            var msg = (typeof err === 'string' ? err : 'CANNOT_PROCESS_REQUEST');
            var errorMsg = (typeof err === 'string' ? err : err.message || err.description || 'Executing request error');
            logger.error('[InteractionService::get_interactions] Error on \'' + req.path + '\': ' + errorMsg);
            if (res.headersSent)
                return;
            res.json({ status: false, description: msg, error: errorMsg });
        });
    });
    app.post('/api/interaction/get_interaction_view', function (req, res, next) {
        logger.info('[InteractionService::get_interaction_view] Received ' + req.method + ' method on \'' + req.path + '\'');
        var vcc = res.locals.vcc;
        var evaluationPlanId = null;
        var processId = null;
        var interactionType = null;
        var folder = null;
        var campaign = null;
        var interactionId = null;
        var fileType = interactionType === 'CALL' ? 'json' : 'view';
        var interactionItems = [];
        var viewResults = [];
        var transcriptionResults = [];
        var sentimentResults = [];
        var transcriptionQualification = null;
        var audioTransfer = new audio_transfer_implementation_1.AudioTransferImplementation();
        (0, promises_shared_1.DummyPromise)()
            .then(function (_) {
            if (req.body) {
                evaluationPlanId = req.body.evaluationPlanId || null;
                processId = req.body.processId || null;
                interactionType = req.body.interactionType || null;
                folder = req.body.folder || null;
                campaign = req.body.campaign || null;
                interactionId = req.body.interactionId || null;
            }
            else {
                throw 'SERVER_ERROR_MISSING_PARAMETERS';
            }
            logger.info('[InteractionService::get_interaction_view] Searching inetraction instances');
            return mssql_service_1.MSSql.GetInteractionTranscription(vcc, evaluationPlanId, processId, interactionType, interactionId);
        }).then(function (result) {
            if (!result)
                throw 'SERVER_ERROR_INTERACTION_NOT_FOUND';
            interactionItems = folder ? result.filter(function (r) { return r.folder === folder; }) : result;
            var promiseArray = [];
            interactionItems.forEach(function (i) {
                var filePath = '';
                if (i.interactionType === 'MAIL') {
                    filePath = audioTransfer.GetFileNamePathFromInteraction(i, fileType, 1);
                    promiseArray.push(GetLocalInteractionFile(i, filePath, i.folder));
                }
                else {
                    filePath = audioTransfer.GetFileNamePathFromInteraction(i, fileType);
                    promiseArray.push(GetLocalInteractionFile(i, filePath, i.folder));
                }
            });
            return Promise.all(promiseArray);
        })
            .then(function (result) {
            viewResults = result;
            var promiseArray = [];
            interactionItems.forEach(function (i) {
                var filePath = '';
                if (i.interactionType === 'MAIL') {
                    if (i.totalMessages >= 2) {
                        for (var n = 2; n <= i.totalMessages; n++) {
                            filePath = audioTransfer.GetFileNamePathFromInteraction(i, fileType, n);
                            promiseArray.push(GetLocalInteractionFile(i, filePath, i.folder));
                        }
                    }
                }
            });
            return Promise.all(promiseArray);
        }).then(function (result) {
            viewResults = viewResults.concat(result);
            var semanticAnalysis = new semantic_analysis_implementation_1.SemanticAnalysisImplementation();
            var promiseArray = [];
            interactionItems.map(function (i) {
                promiseArray.push(semanticAnalysis.SemanticAnalysisViewFileGenerate(i));
            });
            return Promise.all(promiseArray);
        }).then(function (result) {
            transcriptionResults = result;
            if (transcriptionResults[0] && transcriptionResults[0].fileName) {
                return Promise.resolve(false);
            }
            else {
                return Promise.resolve(false);
            }
        }).then(function (result) {
            if (typeof (result) !== 'boolean') {
                sentimentResults = result;
                transcriptionResults[0].transcriptionResult.sentimentResults = sentimentResults;
            }
            var transcriptionDTO = {
                vcc: vcc,
                processId: processId,
                id: interactionId,
                section: null,
                part: null
            };
            transcriptionDTO.interactionType = interactionType;
            transcriptionDTO.folder = folder;
            transcriptionDTO.campaign = campaign;
            return mssql_service_1.MSSql.GetTranscription(transcriptionDTO);
        }).then(function (result) {
            transcriptionQualification = (result && result.StatusCodeError) ? null : result;
            if (res.headersSent)
                return;
            logger.info('[InteractionService::get_interaction_view] Received results');
            res.json({
                status: true,
                description: 'SERVER_SUCCESS_MESSAGE',
                data: {
                    interactionView: viewResults,
                    transcriptionResult: transcriptionResults,
                    transcriptionQualification: transcriptionQualification && transcriptionQualification.value ? transcriptionQualification.value : null
                }
            });
        }).catch(function (err) {
            var msg = (typeof err === 'string' ? err : 'CANNOT_PROCESS_REQUEST');
            var errorMsg = (typeof err === 'string' ? err : err.message || err.description || 'Executing request error');
            logger.error('[InteractionService::get_interaction_view] Error on \'' + req.path + '\': ' + errorMsg);
            if (res.headersSent)
                return;
            res.json({ status: false, description: msg, error: errorMsg });
        });
    });
    app.post('/api/interaction/get_interaction_query_results', function (req, res, next) {
        logger.info('[InteractionService::get_interaction_query_results] Received ' + req.method + ' method on \'' + req.path + '\'');
        var vcc = res.locals.vcc;
        var interactionType = null;
        var interactionId = null;
        var processId = null;
        var folder = null;
        var campaign = null;
        var transactionType = null;
        var transactionId = null;
        (0, promises_shared_1.DummyPromise)()
            .then(function (_) {
            if (req.body) {
                interactionType = req.body.interactionType || null;
                interactionId = req.body.interactionId || null;
                folder = req.body.folder || null;
                campaign = req.body.campaign || null;
                transactionId = req.body.transactionId || null;
                transactionType = req.body.transactionType || null;
            }
            else {
                throw 'SERVER_ERROR_MISSING_PARAMETERS';
            }
            logger.info('[InteractionService::get_interaction_query_results] Searching inetraction instances');
            return mssql_service_1.MSSql.GetInteractionQueryResults(vcc, transactionType, transactionId, interactionType, interactionId, campaign, folder);
        })
            .then(function (result) {
            if (res.headersSent)
                return;
            logger.info('[InteractionService::get_interaction_query_results] Received results');
            res.json({ status: true, description: 'SERVER_SUCCESS_MESSAGE', data: result });
        })
            .catch(function (err) {
            var msg = (typeof err === 'string' ? err : 'CANNOT_PROCESS_REQUEST');
            var errorMsg = (typeof err === 'string' ? err : err.message || err.description || 'Executing request error');
            logger.error('[InteractionService::get_interaction_query_results] Error on \'' + req.path + '\': ' + errorMsg);
            if (res.headersSent)
                return;
            res.json({ status: false, description: msg, error: errorMsg });
        });
    });
    app.post('/api/interaction/discard_interaction', function (req, res, next) {
        logger.info('[InteractionService::discard_interaction] Received ' + req.method + ' method on \'' + req.path + '\'');
        var vcc = res.locals.instance;
        var userRole = res.locals.user.role;
        var userId = res.locals.userId;
        var evaluationToDiscard = (req.body.evaluation || {});
        var multiple = req.body.multiple || false;
        var saveResult = null;
        (0, promises_shared_1.DummyPromise)()
            .then(function (_) {
            return (0, role_service_1.GetRoleFunctions)(vcc, userId, userRole);
        })
            .then(function (result) {
            if (result && result.data && !result.data.hasAdminFunctions)
                throw 'SERVER_ERROR_USER_HAS_NOT_PERMISSION';
            return redis_service_1.Redis.GetEvaluationStatus(vcc, evaluationToDiscard);
        })
            .then(function (result) {
            if (result)
                throw 'SERVER_ERROR_EVALUATION_IS_LOCKED';
            return mssql_service_1.MSSql.SaveEvaluationStatus(vcc, evaluationToDiscard.evaluationPlanId, evaluationToDiscard.interactionId, evaluationToDiscard.auditorId, false, multiple, userId);
        })
            .then(function (result) {
            if (result && result.status === false)
                throw result.error;
            saveResult = result;
            return mssql_service_1.MSSql.SaveEvaluationPlanCompleted(vcc, evaluationToDiscard.evaluationPlanId);
        })
            .then(function (result) {
            if (res.headersSent)
                return;
            logger.info('[InteractionService::discard_interaction] The evaluation update has been finished.');
            res.json({ status: true, description: (saveResult ? 'SERVER_SUCCESS_MESSAGE' : 'SERVER_ERROR_UNKNOWN_MESSAGE'), data: true });
        })
            .catch(function (err) {
            var msg = (typeof err === 'string' ? err : 'CANNOT_PROCESS_REQUEST');
            var errorMsg = (typeof err === 'string' ? err : err.message || err.description || 'Executing request error');
            logger.error('[InteractionService::discard_interaction] Error on \'' + req.path + '\': ' + errorMsg);
            if (res.headersSent)
                return;
            res.json({ status: false, description: msg, error: errorMsg });
        });
    });
    app.post('/api/interaction/recover_interaction', function (req, res, next) {
        logger.info('[InteractionService::recover_interaction] Received ' + req.method + ' method on \'' + req.path + '\'');
        var vcc = res.locals.instance;
        var userRole = res.locals.user.role;
        var userId = res.locals.userId;
        var evaluationToDiscard = (req.body.evaluation || {});
        var multiple = req.body.multiple || false;
        var saveResult = null;
        (0, promises_shared_1.DummyPromise)()
            .then(function (_) {
            return (0, role_service_1.GetRoleFunctions)(vcc, userId, userRole);
        })
            .then(function (result) {
            if (result && result.data && result.data.hasAdminFunctions) { }
            else
                throw 'SERVER_ERROR_USER_HAS_NOT_PERMISSION';
            return mssql_service_1.MSSql.SaveEvaluationStatus(vcc, evaluationToDiscard.evaluationPlanId, evaluationToDiscard.interactionId, evaluationToDiscard.auditorId, true, multiple, userId);
        })
            .then(function (result) {
            if (result && result.status === false)
                throw result.error;
            saveResult = result;
            return mssql_service_1.MSSql.SaveEvaluationPlanCompleted(vcc, evaluationToDiscard.evaluationPlanId);
        })
            .then(function (result) {
            if (res.headersSent)
                return;
            logger.info('[InteractionService::recover_interaction] The evaluation update has been finished.');
            res.json({ status: true, description: (saveResult ? 'SERVER_SUCCESS_MESSAGE' : 'SERVER_ERROR_UNKNOWN_MESSAGE'), data: true });
        })
            .catch(function (err) {
            var msg = (typeof err === 'string' ? err : 'CANNOT_PROCESS_REQUEST');
            var errorMsg = (typeof err === 'string' ? err : err.message || err.description || 'Executing request error');
            logger.error('[InteractionService::recover_interaction] Error on \'' + req.path + '\': ' + errorMsg);
            if (res.headersSent)
                return;
            res.json({ status: false, description: msg, error: errorMsg });
        });
    });
}
exports.InteractionService = InteractionService;
function fillInteractionContentURL(i, vcc) {
    if (i.interactionType === 'CALL') {
        i.interactionPartGroups.forEach(function (group) {
            group.parts.forEach(function (p) {
                p.partURL = config_1.QUALITY_AUDIO_BASE_URL + i.id + '/' + p.section + '/' + p.part;
            });
        });
    }
}
function fillInteractionTags(instance, interactions) {
    var p = [];
    interactions.forEach(function (i) {
        if (i.interactionType === 'CALL')
            p.push(mssql_service_1.MSSql.GetAllTagsInteraction(instance, i.id)
                .then(function (tags) {
                i.tags = tags || [];
                return i;
            }).catch(function (err) {
                var errorMsg = (typeof err === 'string' ? err : err.message || err.description || 'SERVER_ERROR_UNKNOWN_MESSAGE');
                logger.error('Error obteniendo info de los tags: ' + errorMsg);
                throw err;
            }));
        else
            p.push(Promise.resolve(i));
    });
    return Promise.all(p).then(function (values) {
        return [].concat.apply([], values);
    });
}
function GetLocalInteractionFile(interaction, filePath, folder, searchedDate) {
    if (!searchedDate)
        searchedDate = 'CURRENT';
    var genericResult = {};
    return (0, promises_shared_1.DummyPromise)()
        .then(function (_) {
        return GetLocalInteractionFileContent(interaction, filePath);
    })
        .then(function (gr) {
        if (gr) {
            genericResult = gr;
            genericResult.result = folder;
            return Promise.resolve(genericResult);
        }
        else {
            genericResult.status = false;
            genericResult.description = 'SERVER_ERROR_COULDNOT_GET_FILE_CONTENT';
            genericResult.result = folder;
            return Promise.resolve(genericResult);
        }
    })
        .catch(function (err) {
        genericResult.status = false;
        genericResult.description = 'ERROR';
        genericResult.data = err.toString();
        return Promise.resolve(genericResult);
    });
}
exports.GetLocalInteractionFile = GetLocalInteractionFile;
function GetLocalInteractionFileContent(interaction, filePath) {
    var genericResult = {};
    return (0, promises_shared_1.DummyPromise)()
        .then(function (_) {
        return CreateLocalInteractionFileContent(interaction, filePath);
    })
        .then(function (result) {
        if (!result)
            throw 'SERVER_ERROR_CONTENT_FROM_I6_NOT_FOUND';
        genericResult.status = true;
        genericResult.description = filePath.split('/').pop();
        genericResult.data = '';
        if (interaction.interactionType === 'MAIL') {
            var EmlParser = require('eml-parser');
            return new EmlParser(fs.createReadStream(filePath))
                .parseEml()
                .then(function (data) {
                var _a;
                genericResult.data = data.html ?
                    data.html
                    :
                        data.text ?
                            data.text
                            :
                                'MESSAGE_CONTENT_NOT_FOUND';
                if ((_a = data === null || data === void 0 ? void 0 : data.attachments) === null || _a === void 0 ? void 0 : _a.length) {
                    genericResult.extraContent = [];
                    data.attachments.forEach(function (da) {
                        var cid = 'cid:';
                        var buff = new Buffer(da.content);
                        var base64 = buff.toString('base64');
                        var format = 'data:' + da.contentType.split(';')[0] + ';base64,';
                        if (da.cid) {
                            cid += da.cid.replace('<', '').replace('>', '');
                        }
                        else {
                            cid += 'vmime';
                        }
                        if (genericResult.data.includes(cid)) {
                            var attachmentSrc = format + base64;
                            if (cid === 'cid:vmime') {
                                var imgSrcPattern = /src\s*=\s*"cid:vmime(.+?)"/;
                                genericResult.data = genericResult.data.replace(imgSrcPattern, 'src="' + attachmentSrc + '"');
                            }
                            else {
                                genericResult.data = genericResult.data.replace(cid, attachmentSrc);
                            }
                        }
                        if (da.type && da.type === 'attachment' && !da.related) {
                            var aFile = {};
                            aFile.id = da.id;
                            aFile.contentType = da.contentType.split(';')[0];
                            aFile.base64 = base64;
                            if (da.filename) {
                                aFile.name = da.filename;
                            }
                            else {
                                var counter = genericResult.extraContent.filter(function (ec) { return ec.name.includes('untitled'); }).length;
                                counter = counter + 1;
                                aFile.name = 'untitled_' + counter + '.' + aFile.contentType.split('/')[1];
                            }
                            genericResult.extraContent.push(aFile);
                        }
                    });
                }
                return Promise.resolve(genericResult);
            });
        }
        else {
            return (0, file_manager_1.ReadFileContent)(filePath, 'utf-8', false)
                .then(function (res) {
                genericResult.data = res;
                return Promise.resolve(genericResult);
            });
        }
    })
        .then(function (result) {
        return Promise.resolve(genericResult);
    })
        .catch(function (err) {
        logger.error('[InteractionService::GetLocalInteractionFileContent] Getting errorMessage:', err);
        genericResult.status = false;
        genericResult.description = 'SERVER_ERROR_CANNOT_GET_FILE_CONTENT';
        return Promise.reject(err);
    });
}
exports.GetLocalInteractionFileContent = GetLocalInteractionFileContent;
function CreateLocalInteractionFileContent(interaction, filePath) {
    return (0, promises_shared_1.DummyPromise)()
        .then(function (_) {
        if (!filePath) {
            var audioTransfer = new audio_transfer_implementation_1.AudioTransferImplementation();
            if (interaction.interactionType === 'MAIL')
                filePath = audioTransfer.GetFileNamePathFromInteraction(interaction, 'view', 1);
            else
                filePath = audioTransfer.GetFileNamePathFromInteraction(interaction, 'view');
        }
        return (0, file_manager_1.ExistInFileSystem)(filePath);
    })
        .then(function (result) {
        if (!result) {
            if (interaction.interactionType === 'MAIL')
                return GetEmlDataFromI6Data(interaction);
            else
                return GetHtmlDataFromI6Data(interaction);
        }
        return Promise.resolve(true);
    })
        .catch(function (err) {
        logger.error('[InteractionService::CreateLocalInteractionFileContent] Getting errorMessage:', err);
        return Promise.reject(err);
    });
}
exports.CreateLocalInteractionFileContent = CreateLocalInteractionFileContent;
function GetEvaluationPlanInteraction(vcc, interactionType, environmentType, evaluationPlanId, interactionId) {
    var interaction = {};
    var interactionParts = [];
    var hadI6Update = false;
    return (0, promises_shared_1.DummyPromise)()
        .then(function (_) {
        return mssql_service_1.MSSql.GetEvaluationPlanInteractionByTypeAndId(vcc, evaluationPlanId, interactionType, interactionId);
    })
        .then(function (result) {
        if (!result)
            throw 'SERVER_ERROR_INTERACTION_NOT_FOUND';
        interaction = result;
        if ((interactionType !== 'CALL'
            ||
                (interactionType === 'CALL'
                    &&
                        environmentType === 'i6'))
            &&
                (interaction === null || interaction === void 0 ? void 0 : interaction.needI6Update)) {
            hadI6Update = true;
            return UpdateI6Interaction({ vcc: vcc, id: interactionId, interactionType: interactionType });
        }
        else {
            return Promise.resolve(null);
        }
    })
        .then(function (result) {
        if (hadI6Update)
            return mssql_service_1.MSSql.GetEvaluationPlanInteractionByTypeAndId(vcc, evaluationPlanId, interactionType, interactionId);
        else
            return Promise.resolve(result);
    })
        .then(function (result) {
        if (hadI6Update)
            interaction = result;
        return mssql_service_1.MSSql.GetI6InteractionAudios(vcc, interactionId);
    })
        .then(function (result) {
        interactionParts = result;
        if (interactionType === 'CALL' && environmentType === 'i6') {
            if (interactionParts && !interactionParts.length) {
                return (0, http_request_occ_1.ApiI6GetInteractionData)(vcc, interactionId);
            }
        }
        return Promise.resolve(result);
    })
        .then(function (result) {
        if (interactionType === 'CALL' && environmentType === 'i6' && interactionParts.length === 0) {
            var i6Data = null;
            var i6Audios_1 = [];
            i6Data = result;
            i6Data.body.interactionData.records = i6Data.body.interactionData.records.filter(function (r) { return r.Format !== 'mp4'; });
            i6Data.body.interactionData.records = (0, arrays_shared_1.SortObjectArrayByValue)(i6Data.body.interactionData.records, 'Part', 'A');
            i6Data.body.interactionData.records.forEach(function (r) {
                i6Audios_1.push({
                    id: interactionId,
                    section: r.Section,
                    part: r.Part,
                    agentId: r.agent.includes('@') ? r.agent.split('@')[0] : r.agent,
                    format: r.Format
                });
            });
            return Promise.resolve(i6Audios_1);
        }
        return Promise.resolve(result);
    })
        .then(function (result) {
        if (interactionParts.length === 0)
            interactionParts = result;
        interaction.interactionPartGroups = [];
        var uniqueAgentItems = (0, arrays_shared_1.GetUniqueFromObjectArray)(interactionParts, 'agentId');
        uniqueAgentItems.map(function (ua) {
            var item = {};
            item.agentId = ua.agentId;
            item.parts = interactionParts.filter(function (ip) { return ip.agentId === item.agentId; });
            interaction.interactionPartGroups.push(item);
        });
        return Promise.resolve(true);
    })
        .then(function (_) {
        return Promise.resolve(interaction);
    })
        .catch(function (err) {
        logger.error('[InteractionService::GetEvaluationPlanInteraction] Error: ', err);
        return Promise.resolve(null);
    });
}
function DeleteEvaluationPlanInteraction(vcc, interactionType, evaluationPlanId, interactionId) {
    return (0, promises_shared_1.DummyPromise)()
        .then(function (_) {
        return mssql_service_1.MSSql.DeleteEvaluationPlanInteractionByTypeAndId(vcc, evaluationPlanId, interactionType, interactionId);
    })
        .catch(function (err) {
        logger.info('[InteractionService::DeleteEvaluationPlanInteraction] Error: ', err);
        return Promise.resolve(false);
    });
}
function GetInteractionSearchFromRequest(evaluationFormatId, filtersObject, withSectionPart) {
    var params = new interaction_model_1.InteractionSearch;
    params.evaluationFormatId = evaluationFormatId || null;
    params.withSectionPart = withSectionPart;
    params.storageId = filtersObject.storageId || null;
    params.environmentId = filtersObject.environmentId || null;
    params.environmentType = filtersObject.environmentType || null;
    params.types = filtersObject.types || [];
    params.dateIni = filtersObject.dateIni ? moment(filtersObject.dateIni).toDate() : null;
    params.dateFin = filtersObject.dateFin ? moment(filtersObject.dateFin).toDate() : null;
    params.hourIni = filtersObject.hourIni || null;
    params.hourFin = filtersObject.hourFin || null;
    params.minDuration = filtersObject.minDuration === 0 ? 0 : filtersObject.minDuration || null;
    params.maxDuration = filtersObject.maxDuration || null;
    params.campaigns = filtersObject.campaigns || [];
    params.agents = filtersObject.agents || [];
    params.results = filtersObject.results || [];
    params.origins = filtersObject.origins || [];
    params.channelsWithLimit = filtersObject.channelsWithLimit || [];
    params.multimediaFormats = JSON.parse(JSON.stringify(config_1.DEFAULT_INTERACTION_FORMAT));
    params.evaluated = filtersObject.evaluated || [];
    if (params.evaluationFormatId) {
        params.evaluationTypes = filtersObject.evaluationTypes || [];
        params.tags = filtersObject.tags || [];
        params.hasVideoOptions = filtersObject.hasVideoOptions || [];
        if (params.hasVideoOptions.includes(true)) {
            params.multimediaFormats.push('flv');
        }
    }
    params.interactionIds = filtersObject.interactionIds ? filtersObject.interactionIds : [];
    params.contactAddress = filtersObject.contactAddress ? filtersObject.contactAddress : '';
    return params;
}
exports.GetInteractionSearchFromRequest = GetInteractionSearchFromRequest;
function GetProcessFilterFromInteractionSearch(processId, interactionSearch, userId) {
    var processFilterArray = [];
    var interactionSearchKeys = Object.keys(interactionSearch);
    var uniqueItemArray = ['dateIni', 'dateFin', 'hourIni', 'hourFin', 'minDuration', 'maxDuration', 'interactionType'];
    var multipleItemArray = ['campaigns', 'types', 'origins', 'results', 'interactionIds', 'agents', 'evaluated'];
    interactionSearchKeys.map(function (isk) {
        if (interactionSearch[isk]) {
            var keyName_1 = '';
            switch (isk) {
                case 'dateIni':
                    keyName_1 = 'startDate';
                    break;
                case 'dateFin':
                    keyName_1 = 'endDate';
                    break;
                case 'hourIni':
                    keyName_1 = 'startTime';
                    break;
                case 'hourFin':
                    keyName_1 = 'endTime';
                    break;
                case 'results':
                    keyName_1 = 'disposition';
                    break;
                default:
                    keyName_1 = isk;
                    break;
            }
            if (uniqueItemArray.includes(isk)) {
                var uniqueValue = '';
                if (isk === 'agents')
                    uniqueValue = interactionSearch[isk].join('-');
                else if (isk === 'dateIni')
                    uniqueValue = interactionSearch[isk].toISOString();
                else if (isk === 'dateFin')
                    uniqueValue = interactionSearch[isk].toISOString();
                else
                    uniqueValue = interactionSearch[isk];
                if (uniqueValue !== null && uniqueValue !== '') {
                    processFilterArray.push(new process_model_1.ProcessFilter(processId, keyName_1, uniqueValue, moment().utc().toDate(), userId, moment().utc().toDate(), userId));
                }
            }
            else if (multipleItemArray.includes(isk)) {
                var multipleValue = [];
                if (isk === 'origins') {
                    if (interactionSearch[isk].length === 1)
                        multipleValue = interactionSearch[isk];
                }
                else if (isk === 'results') {
                    multipleValue = interactionSearch[isk].map(function (r) { return JSON.stringify(r); });
                }
                else
                    multipleValue = interactionSearch[isk];
                multipleValue.map(function (mv) {
                    if (mv !== null && mv !== '') {
                        processFilterArray.push(new process_model_1.ProcessFilter(processId, keyName_1, mv, moment().utc().toDate(), userId, moment().utc().toDate(), userId));
                    }
                });
            }
        }
    });
    return processFilterArray;
}
exports.GetProcessFilterFromInteractionSearch = GetProcessFilterFromInteractionSearch;
function GetInteractionSearchFromProcessFilter(processFilter) {
    var interactionSearch = new interaction_model_1.InteractionSearch();
    var uniqueItemArray = ['startDate', 'startDate', 'startTime', 'endTime', 'minDuration', 'maxDuration', 'interactionType'];
    var multipleItemArray = ['campaigns', 'types', 'origins', 'disposition', 'interactionIds', 'agents', 'evaluated'];
    processFilter.map(function (pf) {
        var keyName = '';
        switch (pf.key) {
            case 'startDate':
                keyName = 'dateIni';
                break;
            case 'endDate':
                keyName = 'dateFin';
                break;
            case 'startTime':
                keyName = 'hourIni';
                break;
            case 'endTime':
                keyName = 'hourFin';
                break;
            case 'disposition':
                keyName = 'results';
                break;
            default:
                keyName = pf.key;
                break;
        }
        if (uniqueItemArray.includes(pf.key)) {
            if (['dateIni', 'dateFin'].includes(keyName)) {
                interactionSearch[keyName] = moment().toDate();
            }
            else {
                interactionSearch[keyName] = pf.value;
            }
        }
        else if (multipleItemArray.includes(pf.key)) {
            if (!interactionSearch[keyName])
                interactionSearch[keyName] = [];
            if (keyName === 'results')
                interactionSearch[keyName].push(JSON.parse(pf.value));
            else
                interactionSearch[keyName].push(pf.value);
        }
    });
    if (!interactionSearch.campaigns)
        interactionSearch.campaigns = [];
    if (!interactionSearch.agents)
        interactionSearch.agents = [];
    if (!interactionSearch.origins)
        interactionSearch.origins = [];
    if (!interactionSearch.multimediaFormats)
        interactionSearch.multimediaFormats = [];
    if (!interactionSearch.interactionIds)
        interactionSearch.interactionIds = [];
    if (!interactionSearch.contactAddress)
        interactionSearch.contactAddress = '';
    if (!interactionSearch.evaluated)
        interactionSearch.evaluated = [];
    return interactionSearch;
}
exports.GetInteractionSearchFromProcessFilter = GetInteractionSearchFromProcessFilter;
function GetChannelNewInteractionSummaryPromises(vcc, vccType, isSummary, resultType, filters, csvData) {
    if (filters.types
        &&
            ((filters.types.includes('CALL') && filters.types.length > 1)
                ||
                    (!filters.types.includes('CALL') && filters.types.length > 0))) {
        if (!integration_config_1.USER_API_I6 || integration_config_1.USER_API_I6 === '') {
            filters.types = ['CALL'];
        }
    }
    var promiseArray = [];
    var results = filters.results || [];
    var result_numbers = [];
    var result_texts = [];
    if (results.length > 0) {
        results.forEach(function (r) {
            if (typeof r === 'string') {
                result_texts.push(r);
                result_numbers.push(r);
            }
            else {
                if (r.code) {
                    var rcf = JSON.parse(JSON.stringify(r));
                    result_numbers.push(rcf.campaignId + '_' + rcf.code);
                }
                else {
                    var dc = null;
                    if (r.description)
                        dc = r.description;
                    if (r.description2)
                        dc = r.description2;
                    if (r.description3)
                        dc = r.description3;
                    if (r.description4)
                        dc = r.description4;
                    if (!result_texts.includes(dc))
                        result_texts.push(dc);
                }
            }
        });
        if (result_texts.length > 0)
            filters.results = (0, arrays_shared_1.GetUniqueFromArray)(result_texts);
        if (result_numbers.length === 0)
            result_numbers.push('NULL');
        if (result_texts.length === 0)
            result_texts.push('NULL');
    }
    if (!filters.multimediaFormats)
        filters.multimediaFormats = null;
    if (filters.environmentType
        &&
            ['ftp', 'sftp', 's3'].includes(filters.environmentType))
        vccType = 'OTHER';
    var flagAllegro = integration_config_1.IP_BD === mssql_config_1.MSSQL_CONFIG.server ? 1 : 0;
    switch (vccType) {
        case 'ALLEGRO':
            if (filters.types.includes('CALL'))
                promiseArray.push(mssql_service_1.MSSql.GetAllegroInteractions(integration_config_1.IP_BD, vcc, filters.dateIni, filters.dateFin, filters.hourIni, filters.hourFin, filters.campaigns.toString(), filters.agents.toString(), result_numbers, flagAllegro, filters.origins.toString(), filters.minDuration, filters.maxDuration, filters.multimediaFormats ? filters.multimediaFormats.toString() : null, filters.interactionIds.toString(), filters.evaluated.toString(), resultType, filters.contactAddress.toString(), filters.channelsWithLimit && filters.channelsWithLimit.length > 0 ? filters.channelsWithLimit[0].value : 0));
            break;
        case 'I6':
            if (filters.searchType === 'TO_SEARCH') {
                if (filters.types.includes('APPSTORE'))
                    promiseArray.push(GetI6InteractionsFromApi(vcc, 'APPSTORE', filters));
                if (filters.types.includes('CALL'))
                    promiseArray.push(GetI6InteractionsFromApi(vcc, 'CALL', filters));
                if (filters.types.includes('CHAT'))
                    promiseArray.push(GetI6InteractionsFromApi(vcc, 'CHAT', filters));
                if (filters.types.includes('FACEBOOK'))
                    promiseArray.push(GetI6InteractionsFromApi(vcc, 'FACEBOOK', filters));
                if (filters.types.includes('GOOGLEPLAY'))
                    promiseArray.push(GetI6InteractionsFromApi(vcc, 'GOOGLEPLAY', filters));
                if (filters.types.includes('INSTAGRAM'))
                    promiseArray.push(GetI6InteractionsFromApi(vcc, 'INSTAGRAM', filters));
                if (filters.types.includes('MAIL'))
                    promiseArray.push(GetI6InteractionsFromApi(vcc, 'MAIL', filters));
                if (filters.types.includes('TWITTER'))
                    promiseArray.push(GetI6InteractionsFromApi(vcc, 'TWITTER', filters));
                if (filters.types.includes('WHATSAPP'))
                    promiseArray.push(GetI6InteractionsFromApi(vcc, 'WHATSAPP', filters));
                if (filters.types.includes('YOUTUBE'))
                    promiseArray.push(GetI6InteractionsFromApi(vcc, 'YOUTUBE', filters));
                if (filters.types.includes('SMS'))
                    promiseArray.push(GetI6InteractionsFromApi(vcc, 'SMS', filters));
            }
            else if (filters.searchType === 'TO_OPERATE') {
                if (filters.types.includes('APPSTORE'))
                    promiseArray.push(GetI6InteractionsFromRedis(vcc, filters.storageId, 'APPSTORE'));
                if (filters.types.includes('CALL'))
                    promiseArray.push(GetI6InteractionsFromRedis(vcc, filters.storageId, 'CALL'));
                if (filters.types.includes('CHAT'))
                    promiseArray.push(GetI6InteractionsFromRedis(vcc, filters.storageId, 'CHAT'));
                if (filters.types.includes('FACEBOOK'))
                    promiseArray.push(GetI6InteractionsFromRedis(vcc, filters.storageId, 'FACEBOOK'));
                if (filters.types.includes('GOOGLEPLAY'))
                    promiseArray.push(GetI6InteractionsFromRedis(vcc, filters.storageId, 'GOOGLEPLAY'));
                if (filters.types.includes('INSTAGRAM'))
                    promiseArray.push(GetI6InteractionsFromRedis(vcc, filters.storageId, 'INSTAGRAM'));
                if (filters.types.includes('MAIL'))
                    promiseArray.push(GetI6InteractionsFromRedis(vcc, filters.storageId, 'MAIL'));
                if (filters.types.includes('TWITTER'))
                    promiseArray.push(GetI6InteractionsFromRedis(vcc, filters.storageId, 'TWITTER'));
                if (filters.types.includes('WHATSAPP'))
                    promiseArray.push(GetI6InteractionsFromRedis(vcc, filters.storageId, 'WHATSAPP'));
                if (filters.types.includes('YOUTUBE'))
                    promiseArray.push(GetI6InteractionsFromRedis(vcc, filters.storageId, 'YOUTUBE'));
                if (filters.types.includes('SMS'))
                    promiseArray.push(GetI6InteractionsFromRedis(vcc, filters.storageId, 'SMS'));
            }
            break;
        case 'OCC':
            if (filters.types.includes('CALL'))
                promiseArray.push(mssql_service_1.MSSql.GetAllegroInteractions(integration_config_1.IP_BD, vcc, filters.dateIni, filters.dateFin, filters.hourIni, filters.hourFin, filters.campaigns.toString(), filters.agents.toString(), result_numbers, flagAllegro, filters.origins.toString(), filters.minDuration, filters.maxDuration, filters.multimediaFormats ? filters.multimediaFormats.toString() : null, filters.interactionIds.toString(), filters.evaluated.toString(), resultType, filters.contactAddress, filters.channelsWithLimit && filters.channelsWithLimit.length > 0 ? filters.channelsWithLimit[0].value : 0));
            if (filters.searchType === 'TO_SEARCH') {
                if (filters.types.includes('APPSTORE'))
                    promiseArray.push(GetI6InteractionsFromApi(vcc, 'APPSTORE', filters));
                if (filters.types.includes('CHAT'))
                    promiseArray.push(GetI6InteractionsFromApi(vcc, 'CHAT', filters));
                if (filters.types.includes('FACEBOOK'))
                    promiseArray.push(GetI6InteractionsFromApi(vcc, 'FACEBOOK', filters));
                if (filters.types.includes('GOOGLEPLAY'))
                    promiseArray.push(GetI6InteractionsFromApi(vcc, 'GOOGLEPLAY', filters));
                if (filters.types.includes('INSTAGRAM'))
                    promiseArray.push(GetI6InteractionsFromApi(vcc, 'INSTAGRAM', filters));
                if (filters.types.includes('MAIL'))
                    promiseArray.push(GetI6InteractionsFromApi(vcc, 'MAIL', filters));
                if (filters.types.includes('TWITTER'))
                    promiseArray.push(GetI6InteractionsFromApi(vcc, 'TWITTER', filters));
                if (filters.types.includes('WHATSAPP'))
                    promiseArray.push(GetI6InteractionsFromApi(vcc, 'WHATSAPP', filters));
                if (filters.types.includes('YOUTUBE'))
                    promiseArray.push(GetI6InteractionsFromApi(vcc, 'YOUTUBE', filters));
                if (filters.types.includes('SMS'))
                    promiseArray.push(GetI6InteractionsFromApi(vcc, 'SMS', filters));
            }
            else if (filters.searchType === 'TO_OPERATE') {
                if (filters.types.includes('APPSTORE'))
                    promiseArray.push(GetI6InteractionsFromRedis(vcc, filters.storageId, 'APPSTORE'));
                if (filters.types.includes('CHAT'))
                    promiseArray.push(GetI6InteractionsFromRedis(vcc, filters.storageId, 'CHAT'));
                if (filters.types.includes('FACEBOOK'))
                    promiseArray.push(GetI6InteractionsFromRedis(vcc, filters.storageId, 'FACEBOOK'));
                if (filters.types.includes('GOOGLEPLAY'))
                    promiseArray.push(GetI6InteractionsFromRedis(vcc, filters.storageId, 'GOOGLEPLAY'));
                if (filters.types.includes('INSTAGRAM'))
                    promiseArray.push(GetI6InteractionsFromRedis(vcc, filters.storageId, 'INSTAGRAM'));
                if (filters.types.includes('MAIL'))
                    promiseArray.push(GetI6InteractionsFromRedis(vcc, filters.storageId, 'MAIL'));
                if (filters.types.includes('TWITTER'))
                    promiseArray.push(GetI6InteractionsFromRedis(vcc, filters.storageId, 'TWITTER'));
                if (filters.types.includes('WHATSAPP'))
                    promiseArray.push(GetI6InteractionsFromRedis(vcc, filters.storageId, 'WHATSAPP'));
                if (filters.types.includes('YOUTUBE'))
                    promiseArray.push(GetI6InteractionsFromRedis(vcc, filters.storageId, 'YOUTUBE'));
                if (filters.types.includes('SMS'))
                    promiseArray.push(GetI6InteractionsFromRedis(vcc, filters.storageId, 'SMS'));
            }
            break;
        case 'OTHER':
            promiseArray.push(mssql_service_1.MSSql.BulkCSVSummaryInteractions(csvData, vcc, filters.dateIni, filters.dateFin, filters.campaigns.toString(), filters.agents.toString(), filters.origins.toString(), filters.hourIni, filters.hourFin, filters.minDuration, filters.maxDuration, result_numbers, isSummary, filters.interactionIds.toString(), filters.channelsWithLimit && filters.channelsWithLimit.length > 0 ? filters.channelsWithLimit[0].value : 0));
            break;
    }
    return promiseArray;
}
exports.GetChannelNewInteractionSummaryPromises = GetChannelNewInteractionSummaryPromises;
function GetChannelOldInteractionSummaryPromises(vcc, vccType, modelSecondsPerRequest, modelCharsPerPhrase, filters, csvData, retranscription) {
    return (0, promises_shared_1.DummyPromise)()
        .then(function (result) {
        var promiseArray = [];
        promiseArray = GetChannelNewInteractionSummaryPromises(vcc, vccType, false, 'DETAILED', filters, csvData);
        return Promise.all(promiseArray);
    })
        .then(function (result) {
        var promiseArray = [];
        result.forEach(function (r) {
            if (r.totalInteractions)
                promiseArray.push(mssql_service_1.MSSql.BulkBillingInteractions(vcc, modelSecondsPerRequest, modelCharsPerPhrase, r.interactionType, r.interactions, retranscription));
        });
        return Promise.all(promiseArray);
    })
        .catch(function (err) {
        return Promise.resolve([]);
    });
}
exports.GetChannelOldInteractionSummaryPromises = GetChannelOldInteractionSummaryPromises;
function GetI6InteractionsFromApi(vcc, channel, filters) {
    var defaultRowsNumber = 500;
    var totalInteractions = 0;
    var apiInteractions = [];
    var bdInteractions = [];
    var formattedInteractions = [];
    var interactionsResult = {};
    var filtersAgent = filters.agents;
    var limitByChannel = filters.channelsWithLimit.length > 0 ? filters.channelsWithLimit.find(function (cl) { return cl.name === channel; }).value : 0;
    if (!httpRequestOccCompatible)
        return Promise.resolve(null);
    return (0, promises_shared_1.DummyPromise)()
        .then(function (_) {
        return (0, http_request_occ_1.ApiI6GetInteractions)(vcc, channel, filters, defaultRowsNumber);
    })
        .then(function (result) {
        if (!result.status)
            throw 'SERVER_ERROR_GET_I6_INTERACTIONS_FAILED';
        totalInteractions = result.body.total;
        apiInteractions = result.body.interactions;
        if (defaultRowsNumber >= totalInteractions) {
            return Promise.resolve([]);
        }
        else {
            var promiseArray = [];
            var totalPages = Math.ceil(totalInteractions / defaultRowsNumber);
            for (var page = 2; page <= totalPages; page++)
                promiseArray.push((0, http_request_occ_1.ApiI6GetInteractions)(vcc, channel, filters, defaultRowsNumber, page));
            return Promise.all(promiseArray);
        }
    })
        .then(function (results) {
        if (results.filter(function (r) { return !r.status; }).length)
            throw 'SERVER_ERROR_AT_LEAST_ONE_OF_GET_I6_INTERACTIONS_REQUEST_FAILED';
        results.forEach(function (r) { return apiInteractions = apiInteractions.concat(r.body.interactions); });
        formattedInteractions = apiInteractions.map(function (i) {
            var formattedInteraction = {};
            var campaign = i.campaign.includes('@') ? i.campaign.split('@')[0] : i.campaign;
            var agent = i.agent ? (i.agent.includes('@') ? i.agent.split('@')[0] : i.agent) : null;
            var durationTime = i.total_records_time_sec ?
                i.total_records_time_sec
                :
                    moment(i.state_timestamp).diff(moment(i.start_timestamp), 'seconds');
            if (channel === 'CALL')
                formattedInteraction = {
                    vcc: vcc,
                    ID: "".concat(i.id, "_0_1"),
                    InteractionId: i.id,
                    Section: 0,
                    Part: 1,
                    Channel: channel,
                    Number: null,
                    ContactAddress: null,
                    Gender: null,
                    SilenceTime: 0,
                    DurationTime: durationTime,
                    AttentionTime: durationTime,
                    TotalParts: 1,
                    Category1: null,
                    Category2: null,
                    Category3: null,
                    Description: i.disposition_code ? i.disposition_code : null,
                    DispositionCode: null,
                    Campaign: campaign,
                    Agent: agent,
                    agentName: null,
                    Type: null,
                    StartDate: i.start_timestamp,
                    EndDate: i.state_timestamp,
                    Format: null,
                    needI6Update: true
                };
            else
                formattedInteraction = {
                    vcc: vcc,
                    id: i.id,
                    InteractionId: i.id,
                    folder: moment.utc(i.state_timestamp).format('YYYY-MM-DD'),
                    exportedContent: "data/".concat(i.id, ".html"),
                    channel: channel,
                    campaign: campaign,
                    startDate: i.start_timestamp,
                    endDate: i.state_timestamp,
                    account: null,
                    initiative: null,
                    contactName: null,
                    contactAddress: null,
                    contactId: null,
                    firstTakedAgent: null,
                    lastTakedAgent: agent,
                    totalMessages: null,
                    durationTime: durationTime,
                    attentionTime: durationTime,
                    dispositionCode: i.disposition_code ? i.disposition_code : null,
                    dispositionTreePath: null,
                    rawChar: i.total_characters,
                    needI6Update: true
                };
            return formattedInteraction;
        });
        return Promise.resolve(true);
    })
        .then(function (_) {
        if (limitByChannel > 0) {
            var distinctAgents = [];
            if (filtersAgent.length >= 0) {
                if (channel === 'CALL') {
                    distinctAgents = (0, arrays_shared_1.GetUniqueFromObjectArray)(formattedInteractions, 'Agent');
                    filtersAgent = distinctAgents.map(function (a) { return a.Agent; });
                }
                else {
                    distinctAgents = (0, arrays_shared_1.GetUniqueFromObjectArray)(formattedInteractions, 'lastTakedAgent');
                    filtersAgent = distinctAgents.map(function (a) { return a.lastTakedAgent; });
                }
            }
            var formattedInteractionWithLimit_1 = [];
            filtersAgent.forEach(function (fa) {
                var agentx = [];
                if (channel === 'CALL') {
                    agentx = formattedInteractions.filter(function (fi) { return fi.Agent === fa; }).slice(0, limitByChannel);
                }
                else {
                    agentx = formattedInteractions.filter(function (fi) { return fi.lastTakedAgent === fa; }).slice(0, limitByChannel);
                }
                if (agentx.length > 0)
                    formattedInteractionWithLimit_1 = formattedInteractionWithLimit_1.concat(agentx);
            });
            formattedInteractions = [];
            formattedInteractions = formattedInteractionWithLimit_1;
        }
    })
        .then(function (_) {
        if (channel === 'CALL')
            return mssql_service_1.MSSql.GetI6AlreadyRegisteredCallInteractions(vcc, formattedInteractions);
        else
            return mssql_service_1.MSSql.GetI6AlreadyRegisteredTextInteractions(vcc, formattedInteractions);
    })
        .then(function (result) {
        if (!filters.interactionIds.length) {
            bdInteractions = result;
            formattedInteractions = formattedInteractions.filter(function (fi) { return !bdInteractions.map(function (bdi) { return bdi.InteractionId; }).includes(fi.InteractionId); });
            formattedInteractions = formattedInteractions.concat(bdInteractions);
        }
        logger.info("[InteractionService::GetI6InteractionsFromApi] Getting ".concat(bdInteractions.length, " already registered interactions for channel ").concat(channel));
        logger.info("[InteractionService::GetI6InteractionsFromApi] Getting ".concat(formattedInteractions.length, " total interactions for channel ").concat(channel));
        return Promise.resolve(true);
    })
        .then(function (_) {
        interactionsResult.interactionType = channel;
        interactionsResult.totalInteractions = (0, arrays_shared_1.GetUniqueFromObjectArray)(formattedInteractions, 'InteractionId').length;
        interactionsResult.totalInteractionDetails = formattedInteractions.length;
        interactionsResult.totalInteractionTime = formattedInteractions.reduce(function (acc, current) { return acc + (current.DurationTime ? current.DurationTime : 0); }, 0);
        interactionsResult.interactions = formattedInteractions;
        interactionsResult.filtersAgent = filtersAgent;
        return redis_service_1.Redis.SaveI6Interactions(vcc, filters.storageId, channel, interactionsResult);
    })
        .then(function (_) {
        delete interactionsResult.interactions;
        return Promise.resolve(interactionsResult);
    })
        .catch(function (err) {
        logger.info('[InteractionService::GetI6InteractionsFromApi] Getting error:', err, "for channel ".concat(channel));
        return Promise.resolve(null);
    });
}
exports.GetI6InteractionsFromApi = GetI6InteractionsFromApi;
function GetI6InteractionsFromRedis(vcc, storageId, channel) {
    return (0, promises_shared_1.DummyPromise)()
        .then(function (_) {
        return redis_service_1.Redis.GetI6Interactions(vcc, storageId, channel);
    })
        .then(function (result) {
        return Promise.resolve(result);
    })
        .catch(function (err) {
        logger.info('[InteractionService::GetI6InteractionsFromRedis] Getting error:', err, "for vcc::channel::storageId => \"".concat(vcc, "::").concat(channel, "::").concat(storageId, "\""));
        return Promise.resolve([]);
    });
}
exports.GetI6InteractionsFromRedis = GetI6InteractionsFromRedis;
function UpdateI6Interaction(interaction, processId) {
    return (0, promises_shared_1.DummyPromise)()
        .then(function (_) {
        if (interaction.interactionType === 'CALL')
            return UpdateI6CallInteraction(interaction, processId);
        else
            return UpdateI6TextInteraction(interaction, processId);
    })
        .then(function (_) {
        return Promise.resolve(interaction);
    })
        .catch(function (err) {
        return Promise.reject(err);
    });
}
exports.UpdateI6Interaction = UpdateI6Interaction;
function UpdateI6CallInteraction(interaction, processId) {
    var i6Data = null;
    var i6Parts = [];
    var i6Audios = [];
    var bdAudios = [];
    var classifiedData = null;
    return (0, promises_shared_1.DummyPromise)()
        .then(function (_) {
        var promiseArray = [];
        promiseArray.push((0, http_request_occ_1.ApiI6GetInteractionData)(interaction.vcc, interaction.id));
        promiseArray.push((0, http_request_occ_1.ApiI6GetInteractionParts)(interaction.vcc, interaction.id));
        return Promise.all(promiseArray);
    })
        .then(function (result) {
        if (!result || ((result === null || result === void 0 ? void 0 : result.length) && !result[0].status))
            throw 'SERVER_ERROR_I6_INTERACTION_DATA_NOT_FOUND';
        if (!result || ((result === null || result === void 0 ? void 0 : result.length) > 1 && !result[1].status))
            throw 'SERVER_ERROR_I6_INTERACTION_PARTS_NOT_FOUND';
        i6Data = result[0].body;
        i6Parts = result[1].body;
        return mssql_service_1.MSSql.GetI6InteractionAudios(interaction.vcc, interaction.id);
    })
        .then(function (result) {
        if (!result)
            throw 'SERVER_ERROR_BD_INTERACTION_DATA_NOT_FOUND';
        var promiseArray = [];
        bdAudios = result;
        i6Parts = i6Parts.filter(function (r) { return r.format !== 'mp4'; });
        i6Parts.forEach(function (i6p) { return i6p.section_part = "".concat(i6p.section, "_").concat(i6p.part); });
        i6Parts = (0, arrays_shared_1.SortObjectArrayByValue)(i6Parts, 'section_part', 'A');
        i6Audios = i6Parts;
        bdAudios.forEach(function (bda) { return bda.section_part = "".concat(bda.section, "_").concat(bda.part); });
        classifiedData = (0, arrays_shared_1.GetCompareItemsFromObjectArray)(bdAudios, i6Audios, 'section_part', 'section_part');
        classifiedData.old.forEach(function (o) {
            var _a, _b, _c, _d, _e, _f, _g, _h, _j, _k;
            var i6Audio = i6Audios.find(function (r) { return r.section === o.section && r.part === o.part; });
            if (!i6Audio)
                i6Audio = i6Audios.shift();
            var idAgent = ((_a = i6Audio === null || i6Audio === void 0 ? void 0 : i6Audio.agent) === null || _a === void 0 ? void 0 : _a.includes('@')) ? i6Audio.agent.split('@')[0] : (i6Audio.agent || null);
            if (!idAgent)
                idAgent = ((_c = (_b = i6Data === null || i6Data === void 0 ? void 0 : i6Data.__last_step) === null || _b === void 0 ? void 0 : _b.agent) === null || _c === void 0 ? void 0 : _c.includes('@')) ? i6Data.__last_step.agent.split('@')[0] : (i6Data.__last_step.agent || null);
            if (!idAgent)
                idAgent = ((_e = (_d = i6Data === null || i6Data === void 0 ? void 0 : i6Data.interactionData) === null || _d === void 0 ? void 0 : _d.enderAgent) === null || _e === void 0 ? void 0 : _e.includes('@')) ? i6Data.interactionData.enderAgent.split('@')[0] : (i6Data.interactionData.enderAgent || '');
            promiseArray.push(mssql_service_1.MSSql.SaveI6InteractionAudio(interaction.vcc, interaction.id, o.section, o.part, (i6Audio === null || i6Audio === void 0 ? void 0 : i6Audio.section) || 0, i6Audio === null || i6Audio === void 0 ? void 0 : i6Audio.part, i6Data.startTimestampReadable ? moment.utc(i6Data.startTimestampReadable).toDate() : null, i6Data.timestamp ? moment.utc(i6Data.timestamp).toDate() : null, moment(i6Data.timestamp).diff(moment(i6Data.startTimestampReadable), 'seconds'), idAgent, (i6Audio === null || i6Audio === void 0 ? void 0 : i6Audio.format) || null, ((_f = i6Data === null || i6Data === void 0 ? void 0 : i6Data.campaign) === null || _f === void 0 ? void 0 : _f.includes('@')) ? i6Data.campaign.split('@')[0] : i6Data.campaign, (i6Data === null || i6Data === void 0 ? void 0 : i6Data.initiative) === 'INBOUND' ? 'E' : 'S', null, null, null, (i6Data === null || i6Data === void 0 ? void 0 : i6Data.dispositionCode) ? i6Data.dispositionCode : null, "".concat(interaction.id, "_").concat((i6Audio === null || i6Audio === void 0 ? void 0 : i6Audio.section) || 0, "_").concat(i6Audio === null || i6Audio === void 0 ? void 0 : i6Audio.part, ".").concat(i6Audio.format), ((_g = i6Data === null || i6Data === void 0 ? void 0 : i6Data.contact) === null || _g === void 0 ? void 0 : _g.includes('@')) ? i6Data.contact.includes('@')[0] : i6Data.contact, ((_h = i6Data === null || i6Data === void 0 ? void 0 : i6Data.interactionData) === null || _h === void 0 ? void 0 : _h.contactName) ? i6Data.interactionData.contactName : null, (i6Data === null || i6Data === void 0 ? void 0 : i6Data.contactAddress) ? i6Data.contactAddress : null, processId || null, ((_j = i6Data === null || i6Data === void 0 ? void 0 : i6Data.interactionData) === null || _j === void 0 ? void 0 : _j.ender) || null, ((_k = i6Data === null || i6Data === void 0 ? void 0 : i6Data.interactionData) === null || _k === void 0 ? void 0 : _k.ticket) || null));
            if ((interaction === null || interaction === void 0 ? void 0 : interaction.section) === o.section
                &&
                    (interaction === null || interaction === void 0 ? void 0 : interaction.part) === o.part) {
                interaction.agent = (i6Audio === null || i6Audio === void 0 ? void 0 : i6Audio.agent.includes('@')) ? i6Audio.agent.split('@')[0] : null;
                interaction.format = i6Audio.format || null;
            }
        });
        classifiedData.new.forEach(function (n) {
            var _a, _b, _c, _d, _e, _f, _g, _h, _j;
            var currentAudio = classifiedData.removed.shift();
            var idAgent = (n === null || n === void 0 ? void 0 : n.agent.includes('@')) ? n.agent.split('@')[0] : (n.agent || null);
            if (!idAgent)
                idAgent = ((_b = (_a = i6Data === null || i6Data === void 0 ? void 0 : i6Data.__last_step) === null || _a === void 0 ? void 0 : _a.agent) === null || _b === void 0 ? void 0 : _b.includes('@')) ? i6Data.__last_step.agent.split('@')[0] : (i6Data.__last_step.agent || null);
            if (!idAgent)
                idAgent = ((_d = (_c = i6Data === null || i6Data === void 0 ? void 0 : i6Data.interactionData) === null || _c === void 0 ? void 0 : _c.enderAgent) === null || _d === void 0 ? void 0 : _d.includes('@')) ? i6Data.interactionData.enderAgent.split('@')[0] : (i6Data.interactionData.enderAgent || '');
            promiseArray.push(mssql_service_1.MSSql.SaveI6InteractionAudio(interaction.vcc, interaction.id, currentAudio ? currentAudio.section : null, currentAudio ? currentAudio.part : null, n.section, n.part, i6Data.startTimestampReadable ? moment.utc(i6Data.startTimestampReadable).toDate() : null, i6Data.timestamp ? moment.utc(i6Data.timestamp).toDate() : null, moment(i6Data.timestamp).diff(moment(i6Data.startTimestampReadable), 'seconds'), idAgent, n.format || null, ((_e = i6Data === null || i6Data === void 0 ? void 0 : i6Data.campaign) === null || _e === void 0 ? void 0 : _e.includes('@')) ? i6Data.campaign.split('@')[0] : i6Data.campaign, (i6Data === null || i6Data === void 0 ? void 0 : i6Data.initiative) === 'INBOUND' ? 'E' : 'S', null, null, null, (i6Data === null || i6Data === void 0 ? void 0 : i6Data.dispositionCode) ? i6Data.dispositionCode : null, "".concat(interaction.id, "_").concat(n.section, "_").concat(n.part, ".").concat(n.format), ((_f = i6Data === null || i6Data === void 0 ? void 0 : i6Data.contact) === null || _f === void 0 ? void 0 : _f.includes('@')) ? i6Data.contact.includes('@')[0] : i6Data.contact, ((_g = i6Data === null || i6Data === void 0 ? void 0 : i6Data.interactionData) === null || _g === void 0 ? void 0 : _g.contactName) ? i6Data.interactionData.contactName : null, (i6Data === null || i6Data === void 0 ? void 0 : i6Data.contactAddress) ? i6Data.contactAddress : null, processId || null, ((_h = i6Data === null || i6Data === void 0 ? void 0 : i6Data.interactionData) === null || _h === void 0 ? void 0 : _h.ender) || null, ((_j = i6Data === null || i6Data === void 0 ? void 0 : i6Data.interactionData) === null || _j === void 0 ? void 0 : _j.ticket) || null));
            if (currentAudio
                &&
                    (interaction === null || interaction === void 0 ? void 0 : interaction.section) === currentAudio.section
                &&
                    (interaction === null || interaction === void 0 ? void 0 : interaction.part) === currentAudio.part) {
                interaction.agent = (n === null || n === void 0 ? void 0 : n.agent.includes('@')) ? n.agent.split('@')[0] : null;
                interaction.format = n.format || null;
            }
        });
        return Promise.all(promiseArray);
    })
        .then(function (_) {
        var promiseArray = [];
        classifiedData.removed.forEach(function (r) {
            logger.info('[InteractionService::UpdateI6CallInteraction] Removing:', r);
            var bdAudio = bdAudios.find(function (a) { return a.section === r.section && a.part === r.part; });
            if (bdAudio) {
                promiseArray.push(mssql_service_1.MSSql.DeleteI6InteractionAudio(interaction.vcc, interaction.id, bdAudio.section, bdAudio.part));
            }
        });
        return Promise.all(promiseArray);
    })
        .then(function (_) {
        return Promise.resolve(interaction);
    })
        .catch(function (err) {
        logger.error('[InteractionService::UpdateI6CallInteraction] Getting error:', err);
        return Promise.reject(err);
    });
}
function UpdateI6TextInteraction(interaction, processId) {
    var i6Data = null;
    var i6Messages = [];
    var i6Texts = [];
    var bdTexts = [];
    return (0, promises_shared_1.DummyPromise)()
        .then(function (_) {
        return (0, http_request_occ_1.ApiI6GetInteractionData)(interaction.vcc, interaction.id);
    })
        .then(function (result) {
        if (!result || !result.status)
            throw 'SERVER_ERROR_I6_INTERACTION_DATA_NOT_FOUND';
        i6Data = result.body;
        i6Texts = GetI6InteractionMessages(interaction, i6Data);
        return mssql_service_1.MSSql.GetI6InteractionMessages(interaction.vcc, interaction.id);
    })
        .then(function (result) {
        var _a;
        if (!result)
            throw 'SERVER_ERROR_BD_INTERACTION_DATA_NOT_FOUND';
        var promiseArray = [];
        bdTexts = result;
        var classifiedData = (0, arrays_shared_1.GetCompareItemsFromObjectArray)(bdTexts, i6Texts, 'folder', 'folder');
        var idCampaign = ((_a = i6Data === null || i6Data === void 0 ? void 0 : i6Data.campaign) === null || _a === void 0 ? void 0 : _a.includes('@')) ? i6Data.campaign.split('@')[0] : i6Data.campaign;
        interaction.campaign = idCampaign;
        classifiedData.old.forEach(function (o) {
            var _a, _b, _c, _d, _e, _f, _g, _h, _j, _k;
            var i6Text = i6Texts.find(function (r) { return r.folder === o.folder; });
            if (i6Text) {
                if (!interaction.folder)
                    interaction.folder = i6Text.folder;
                var idAgent = ((_a = i6Text === null || i6Text === void 0 ? void 0 : i6Text.agent) === null || _a === void 0 ? void 0 : _a.includes('@')) ? i6Text.agent.split('@')[0] : (i6Text.agent || null);
                if (!idAgent)
                    idAgent = ((_c = (_b = i6Data === null || i6Data === void 0 ? void 0 : i6Data.__last_step) === null || _b === void 0 ? void 0 : _b.agent) === null || _c === void 0 ? void 0 : _c.includes('@')) ? i6Data.__last_step.agent.split('@')[0] : (i6Data.__last_step.agent || null);
                if (!idAgent)
                    idAgent = ((_e = (_d = i6Data === null || i6Data === void 0 ? void 0 : i6Data.interactionData) === null || _d === void 0 ? void 0 : _d.enderAgent) === null || _e === void 0 ? void 0 : _e.includes('@')) ? i6Data.interactionData.enderAgent.split('@')[0] : (i6Data.interactionData.enderAgent || '');
                promiseArray.push(mssql_service_1.MSSql.SaveI6InteractionAsyncInteraction(interaction.vcc, interaction.id, interaction.interactionType, o.folder, i6Text.startDate, i6Text.endDate, moment(i6Data.timestamp).diff(moment(i6Data.startTimestampReadable), 'seconds'), idAgent, interaction.campaign, ((_f = i6Data === null || i6Data === void 0 ? void 0 : i6Data.account) === null || _f === void 0 ? void 0 : _f.includes('@')) ? i6Data.account.split('@')[0] : i6Data.account, (i6Data === null || i6Data === void 0 ? void 0 : i6Data.initiative) === 'INBOUND' ? 'IN' : 'OUT', null, (i6Data === null || i6Data === void 0 ? void 0 : i6Data.dispositionCode) ? i6Data.dispositionCode : null, (i6Text === null || i6Text === void 0 ? void 0 : i6Text.exportedContent) ? i6Text.exportedContent : null, ((_g = i6Data === null || i6Data === void 0 ? void 0 : i6Data.contact) === null || _g === void 0 ? void 0 : _g.includes('@')) ? i6Data.contact.includes('@')[0] : i6Data.contact, ((_h = i6Data === null || i6Data === void 0 ? void 0 : i6Data.interactionData) === null || _h === void 0 ? void 0 : _h.contactName) ? i6Data.interactionData.contactName : null, (i6Data === null || i6Data === void 0 ? void 0 : i6Data.contactAddress) ? i6Data.contactAddress : null, i6Text.totalMessages, processId || null, ((_j = i6Data === null || i6Data === void 0 ? void 0 : i6Data.interactionData) === null || _j === void 0 ? void 0 : _j.ender) || null, ((_k = i6Data === null || i6Data === void 0 ? void 0 : i6Data.interactionData) === null || _k === void 0 ? void 0 : _k.ticket) || null));
            }
        });
        classifiedData.new.forEach(function (n) {
            var _a, _b, _c, _d, _e, _f, _g, _h, _j, _k;
            var idAgent = ((_a = n === null || n === void 0 ? void 0 : n.agent) === null || _a === void 0 ? void 0 : _a.includes('@')) ? n.agent.split('@')[0] : (n.agent || null);
            if (!idAgent)
                idAgent = ((_c = (_b = i6Data === null || i6Data === void 0 ? void 0 : i6Data.__last_step) === null || _b === void 0 ? void 0 : _b.agent) === null || _c === void 0 ? void 0 : _c.includes('@')) ? i6Data.__last_step.agent.split('@')[0] : (i6Data.__last_step.agent || null);
            if (!idAgent)
                idAgent = ((_e = (_d = i6Data === null || i6Data === void 0 ? void 0 : i6Data.interactionData) === null || _d === void 0 ? void 0 : _d.enderAgent) === null || _e === void 0 ? void 0 : _e.includes('@')) ? i6Data.interactionData.enderAgent.split('@')[0] : (i6Data.interactionData.enderAgent || '');
            promiseArray.push(mssql_service_1.MSSql.SaveI6InteractionAsyncInteraction(interaction.vcc, interaction.id, interaction.interactionType, n.folder, n.startDate, n.endDate, moment(i6Data.timestamp).diff(moment(i6Data.startTimestampReadable), 'seconds'), idAgent, interaction.campaign, ((_f = i6Data === null || i6Data === void 0 ? void 0 : i6Data.account) === null || _f === void 0 ? void 0 : _f.includes('@')) ? i6Data.account.split('@')[0] : i6Data.account, (i6Data === null || i6Data === void 0 ? void 0 : i6Data.initiative) === 'INBOUND' ? 'IN' : 'OUT', null, (i6Data === null || i6Data === void 0 ? void 0 : i6Data.dispositionCode) ? i6Data.dispositionCode : null, (n === null || n === void 0 ? void 0 : n.exportedContent) ? n.exportedContent : null, ((_g = i6Data === null || i6Data === void 0 ? void 0 : i6Data.contact) === null || _g === void 0 ? void 0 : _g.includes('@')) ? i6Data.contact.includes('@')[0] : i6Data.contact, ((_h = i6Data === null || i6Data === void 0 ? void 0 : i6Data.interactionData) === null || _h === void 0 ? void 0 : _h.contactName) ? i6Data.interactionData.contactName : null, (i6Data === null || i6Data === void 0 ? void 0 : i6Data.contactAddress) ? i6Data.contactAddress : null, n.totalMessages, processId || null, ((_j = i6Data === null || i6Data === void 0 ? void 0 : i6Data.interactionData) === null || _j === void 0 ? void 0 : _j.ender) || null, ((_k = i6Data === null || i6Data === void 0 ? void 0 : i6Data.interactionData) === null || _k === void 0 ? void 0 : _k.ticket) || null));
        });
        return Promise.all(promiseArray);
    })
        .then(function (_) {
        if (processId) {
            return GetJsonDataFromI6Data(interaction, i6Data);
        }
        return Promise.resolve(true);
    })
        .then(function (_) {
        if (processId)
            return CreateLocalInteractionFileContent(interaction);
        return Promise.resolve(true);
    })
        .then(function (_) {
        return Promise.resolve(interaction);
    })
        .catch(function (err) {
        logger.error('[InteractionService::UpdateI6TextInteraction] Getting errorMessage:', err);
        return Promise.reject(err);
    });
}
function GetI6InteractionMessages(interaction, i6Data) {
    var _a, _b, _c, _d, _e, _f;
    var messages = [];
    var texts = [];
    switch (interaction.interactionType) {
        case 'CHAT':
            messages = [{
                    id: interaction.id,
                    startDate: i6Data.startTimestampReadable ? moment.utc(i6Data.startTimestampReadable).toDate() : null,
                    endDate: i6Data.timestamp ? moment.utc(i6Data.timestamp).toDate() : null,
                    folder: moment.utc(i6Data.timestamp).format('YYYY-MM-DD'),
                    totalMessages: ((_b = (_a = i6Data === null || i6Data === void 0 ? void 0 : i6Data.interactionData) === null || _a === void 0 ? void 0 : _a.chatMessagesDetail) === null || _b === void 0 ? void 0 : _b.length) ? i6Data.interactionData.chatMessagesDetail.length : 1,
                    exportedContent: interaction.id
                }];
            texts = messages;
            break;
        default:
            messages = ((_d = (_c = i6Data.analysisTimes) === null || _c === void 0 ? void 0 : _c.openLapses) === null || _d === void 0 ? void 0 : _d.length) ? i6Data.analysisTimes.openLapses.map(function (ol) {
                var folder = moment.unix(ol.end).utc().format('YYYY-MM-DD');
                return {
                    id: interaction.id,
                    startDate: moment.unix(ol.start).toDate(),
                    endDate: moment.unix(ol.end).toDate(),
                    folder: folder,
                    totalMessages: i6Data.events.filter(function (e) { return moment.utc(e.timestamp).isSameOrBefore(moment.utc("".concat(folder, " 23:59:59"))); }).length,
                    unixStart: ol.start,
                    unixEnd: ol.end
                };
            }) : [];
            var lastMessage = messages.length > 1 ? messages.pop() : messages[0];
            texts = [{
                    id: interaction.id,
                    startDate: messages[0].startDate,
                    endDate: lastMessage.endDate,
                    folder: lastMessage.folder,
                    totalMessages: lastMessage.totalMessages,
                    exportedContent: ((_e = i6Data === null || i6Data === void 0 ? void 0 : i6Data.events) === null || _e === void 0 ? void 0 : _e.length) ? (_f = i6Data.events.pop()) === null || _f === void 0 ? void 0 : _f.id : null
                }];
            break;
    }
    return texts;
}
function GetJsonDataFromI6Data(interaction, i6Data) {
    var audioTransfer = new audio_transfer_implementation_1.AudioTransferImplementation();
    var fileNamePath = audioTransfer.GetFileNamePathFromInteraction(interaction, 'json');
    var messages = [];
    var format = '2.1';
    var results = [];
    var createdDate = moment.utc().format('YYYY-MM-DD HH:mm:ss');
    var metadata = {
        type: 'transcription',
        occ_data: {
            messages: [],
            interactionId: interaction.id
        },
        transcription_config: {
            diarization: 'channel',
            channel_diarization_labels: ['Agent', 'Caller'],
            language: interaction.language
        },
        created_at: createdDate
    };
    var job = {
        created_at: createdDate
    };
    return (0, promises_shared_1.DummyPromise)()
        .then(function (_) {
        if (!i6Data)
            return (0, http_request_occ_1.ApiI6GetInteractionData)(interaction.vcc, interaction.id);
        return Promise.resolve(i6Data);
    })
        .then(function (result) {
        i6Data !== null && i6Data !== void 0 ? i6Data : (i6Data = result.body);
        switch (interaction.interactionType) {
            case 'CHAT': return (0, http_request_occ_1.ApiI6GetAsyncInteractionContent)(interaction.vcc, interaction.interactionType, interaction.id);
            default: return Promise.resolve(i6Data);
        }
    })
        .then(function (result) {
        if (!i6Data && (!result || !result.status))
            throw 'SERVER_ERROR_I6_DATA_NOT_FOUND';
        logger.info("[InteractionService::GetJsonDataFromI6Data] Getting messages from ".concat(interaction.interactionType, " interaction"));
        messages = GetTextMessagesFromI6Data(interaction, result, i6Data);
        return Promise.resolve(true);
    })
        .then(function (_) {
        if (!messages.length)
            throw 'SERVER_ERROR_MESAGGES_NOT_FOUND';
        var firstMessageStartDate = null;
        messages.forEach(function (m, mi) {
            var startDate = moment.unix(m.msgTime);
            if (!firstMessageStartDate)
                firstMessageStartDate = startDate.set('milliseconds', 0);
            metadata.occ_data.messages.push(m);
            var tokenizedText = m.fullMsg.split(/(.+?)([\s,.!?;:)([\]]+)/);
            tokenizedText.forEach(function (tt) { return tt = tt.trim(); });
            tokenizedText = tokenizedText.filter(function (tt) { return tt.trim().length; });
            tokenizedText.forEach(function (tt) {
                var wordType = 'word';
                if (tt.length === 1 && tt.match(/[`!@#$%^&*()_+\-=\[\]{};':"\\|,.<>\/?~]/))
                    wordType = 'punctuation';
                results.push({
                    end_time: startDate.diff(firstMessageStartDate, 'milliseconds') / 1000,
                    channel: m.channel === 'Agent' ? 'Agent' : 'Caller',
                    occ_data: {
                        msgTime: m.msgTime,
                        msgDate: startDate.utc().format('YYYY-MM-DD HH:mm:ss.000'),
                        word_number: 0,
                        msgNumber: mi + 1
                    },
                    start_time: startDate.diff(firstMessageStartDate, 'milliseconds') / 1000,
                    type: wordType,
                    alternatives: [{
                            confidence: 1,
                            content: tt,
                            language: interaction.language
                        }]
                });
            });
            results.forEach(function (r, i) { return r.occ_data.word_number = (i + 1); });
        });
        return Promise.resolve(true);
    })
        .then(function (_) {
        var objectResult = {
            format: format,
            metadata: metadata,
            results: results,
            job: job
        };
        return (0, file_manager_1.WriteFileContent)(fileNamePath, JSON.stringify(objectResult), 'utf-8', false);
    })
        .then(function (_) {
        interaction.fileName = fileNamePath.split('/').pop();
        return Promise.resolve(true);
    })
        .catch(function (err) {
        logger.error('[InteractionService::GetJsonDataFromI6Data] Getting errorMessage:', err);
        return Promise.reject(err);
    });
}
exports.GetJsonDataFromI6Data = GetJsonDataFromI6Data;
function GetHtmlDataFromI6Data(interaction, i6Data) {
    var messages = null;
    var audioTransfer = new audio_transfer_implementation_1.AudioTransferImplementation();
    var jsonFileNamePath = audioTransfer.GetFileNamePathFromInteraction(interaction, 'json');
    var viewFileNamePath = audioTransfer.GetFileNamePathFromInteraction(interaction, 'view');
    return (0, promises_shared_1.DummyPromise)()
        .then(function (_) {
        if (!i6Data)
            return (0, http_request_occ_1.ApiI6GetInteractionData)(interaction.vcc, interaction.id);
        else
            return Promise.resolve(i6Data);
    })
        .then(function (result) {
        i6Data !== null && i6Data !== void 0 ? i6Data : (i6Data = result.body);
        return (0, file_manager_1.ReadFileContent)(jsonFileNamePath, 'utf-8', true);
    })
        .then(function (result) {
        var jsonResult = null;
        try {
            jsonResult = JSON.parse(result);
        }
        catch (err) {
            logger.error('[InteractionService::GetHtmlDataFromI6Data] Error: ', err);
        }
        return Promise.resolve(jsonResult);
    })
        .then(function (result) {
        var _a, _b;
        if ((_b = (_a = result === null || result === void 0 ? void 0 : result.metadata) === null || _a === void 0 ? void 0 : _a.occ_data) === null || _b === void 0 ? void 0 : _b.messages) {
            messages = result.metadata.occ_data.messages;
            return Promise.resolve(null);
        }
        else {
            switch (interaction.interactionType) {
                case 'CHAT': return (0, http_request_occ_1.ApiI6GetAsyncInteractionContent)(interaction.vcc, interaction.interactionType, interaction.id);
                default: return Promise.resolve(i6Data);
            }
        }
    })
        .then(function (result) {
        if (!messages) {
            messages = GetTextMessagesFromI6Data(interaction, result, i6Data);
        }
        return Promise.resolve(messages);
    })
        .then(function (result) {
        var htmlContent = '<!DOCTYPE html><html><head><meta charset="utf-8">';
        htmlContent += '<link rel="stylesheet" type="text/css" href="../resources/webchat.css"></link>';
        htmlContent += '</head><body><table border="0" cellspacing="0" cellpadding="0">';
        result.forEach(function (m) {
            htmlContent += "<tr class=\"".concat(m.channel === 'Agent' ? 'participantTwo' : 'participantOne', "\">");
            htmlContent += "<td>".concat(moment.unix(m.msgTime).utc().format('YYYY-MM-DD HH:mm:ss'), "</td>");
            htmlContent += "<td><span>".concat(m.channel, ":</span><span>").concat(m.fullMsg, "</span></td></tr>");
        });
        htmlContent += '</table></body></html>';
        return (0, file_manager_1.WriteFileContent)(viewFileNamePath, htmlContent, 'utf-8', false);
    })
        .then(function (_) {
        return Promise.resolve(true);
    })
        .catch(function (err) {
        logger.error('[InteractionService::GetHtmlDataFromI6Data] Getting errorMessage:', err);
        return Promise.reject(err);
    });
}
exports.GetHtmlDataFromI6Data = GetHtmlDataFromI6Data;
function GetEmlDataFromI6Data(interaction, i6Data) {
    var audioTransfer = new audio_transfer_implementation_1.AudioTransferImplementation();
    return (0, promises_shared_1.DummyPromise)()
        .then(function (_) {
        if (!i6Data)
            return (0, http_request_occ_1.ApiI6GetInteractionData)(interaction.vcc, interaction.id);
        else
            return Promise.resolve(i6Data);
    })
        .then(function (result) {
        if (!i6Data)
            i6Data = result.body;
        var promiseArray = [];
        i6Data.events.forEach(function (e) { return promiseArray.push((0, http_request_occ_1.ApiI6GetAsyncInteractionContent)(interaction.vcc, interaction.interactionType, e.id)); });
        return Promise.all(promiseArray);
    })
        .then(function (result) {
        var promiseArray = [];
        result.forEach(function (e, i) {
            var _a;
            var viewFileNamePath = audioTransfer.GetFileNamePathFromInteraction(interaction, 'view', (i + 1));
            logger.debug('[InteractionService::GetEmlDataFromI6Data] Saving:', viewFileNamePath);
            if (e.status)
                promiseArray.push((0, file_manager_1.WriteFileContent)(viewFileNamePath, e.body, 'utf-8', false));
            else if ((_a = i6Data.events[i].body) === null || _a === void 0 ? void 0 : _a.length) {
                var event_1 = i6Data.events[i];
                var toList = Object.keys(event_1.to);
                var toObject = (toList === null || toList === void 0 ? void 0 : toList.length) ?
                    { name: event_1.to[toList[0]], to: toList[0] }
                    :
                        { name: 'Unknown', to: 'Unknown' };
                var messageList = event_1.body;
                var messageObject = messageList.find(function (ml) { return (ml === null || ml === void 0 ? void 0 : ml.type) === 'text/html'; }) ? messageList.find(function (ml) { return ml.type === 'text/html'; }) : messageList[0];
                var mailContent = '';
                mailContent += 'Subject: ' + event_1.subject + '\n';
                mailContent += 'From: "' + event_1.name + '" <' + event_1.from + '>' + '\n';
                mailContent += 'To: "' + toObject.name + '" <' + toObject.to + '>' + '\n';
                mailContent += 'Date: ' + event_1.date + '\n';
                mailContent += 'Content-Type: text/html; charset=utf-8\n';
                mailContent += 'Content-Transfer-Encoding: quoted-printable\n';
                mailContent += 'Message-Id: <' + event_1.id + '>\n';
                mailContent += '\n';
                mailContent += messageObject.content + '\n';
                promiseArray.push((0, file_manager_1.WriteFileContent)(viewFileNamePath, mailContent, 'utf-8', false));
            }
            else
                promiseArray.push((0, file_manager_1.WriteFileContent)(viewFileNamePath, 'SERVER_ERROR_NO_CONTENT_FOUND', 'utf-8', false));
        });
        return Promise.all(promiseArray);
    })
        .then(function (_) {
        return Promise.resolve(true);
    })
        .catch(function (err) {
        logger.error('[InteractionService::GetEmlDataFromI6Data] Getting errorMessage:', err);
        return Promise.reject(err);
    });
}
exports.GetEmlDataFromI6Data = GetEmlDataFromI6Data;
function GetTextMessagesFromI6Data(interaction, i6Response, i6Data) {
    var _a, _b, _c, _d;
    i6Data !== null && i6Data !== void 0 ? i6Data : (i6Data = i6Response);
    var messages = [];
    switch (interaction.interactionType) {
        case 'CHAT':
            var agentHistory_1 = (0, arrays_shared_1.GetUniqueFromArray)(Object.keys(i6Data.agentHistory)).map(function (ah) { return ah.split('@')[0]; });
            i6Response.body.messages.forEach(function (m, i) {
                messages.push({
                    channel: agentHistory_1.includes((m.user_id.split('@')[0])) ? 'Agent' : m.display_name,
                    fullMsg: m.text,
                    msgTime: m.msgtime,
                    msgDate: moment.unix(m.msgtime).utc().format('YYYY-MM-DD HH:mm:ss.000'),
                    msgNumber: (i + 1)
                });
            });
            break;
        case 'MAIL':
            i6Data.events.forEach(function (e, i) {
                var _a;
                var message = e.body.find(function (b) { return b.type === 'text/plain'; });
                if ((_a = message === null || message === void 0 ? void 0 : message.content) === null || _a === void 0 ? void 0 : _a.length) {
                    messages.push({
                        channel: e.creator === 'contact' ? 'Web Client' : 'Agent',
                        fullMsg: message.content,
                        msgTime: e.timestampDouble,
                        msgDate: moment.unix(e.timestampDouble).utc().format('YYYY-MM-DD HH:mm:ss.000'),
                        msgNumber: (i + 1)
                    });
                }
            });
            messages.forEach(function (m) { return m.fullMsg = m.fullMsg.replace(/(?:\\[rn]|[\r\n]+)+/g, ' '); });
            break;
        case 'FACEBOOK':
            i6Data.events.forEach(function (e, i) {
                var _a, _b, _c, _d;
                if (((_b = (_a = e.message) === null || _a === void 0 ? void 0 : _a.message) === null || _b === void 0 ? void 0 : _b.length) || ((_d = (_c = e.message) === null || _c === void 0 ? void 0 : _c.text) === null || _d === void 0 ? void 0 : _d.length)) {
                    messages.push({
                        channel: e.creator === 'contact' ? 'Web Client' : 'Agent',
                        fullMsg: e.message.message || e.message.text,
                        msgTime: e.timestampDouble,
                        msgDate: moment.unix(e.timestampDouble).utc().format('YYYY-MM-DD HH:mm:ss.000'),
                        msgNumber: (i + 1)
                    });
                }
                ;
            });
            break;
        case 'TWITTER':
            i6Data.events.forEach(function (e, i) {
                var _a;
                if ((_a = e.text) === null || _a === void 0 ? void 0 : _a.length) {
                    messages.push({
                        channel: e.creator === 'contact' ? 'Web Client' : 'Agent',
                        fullMsg: e.text,
                        msgTime: e.timestampDouble,
                        msgDate: moment.unix(e.timestampDouble).utc().format('YYYY-MM-DD HH:mm:ss.000'),
                        msgNumber: (i + 1)
                    });
                }
                ;
            });
            break;
        case 'WEBCONTACT':
            if ((_d = (_c = (_b = (_a = i6Data.data) === null || _a === void 0 ? void 0 : _a.reportingData) === null || _b === void 0 ? void 0 : _b.contactForm) === null || _c === void 0 ? void 0 : _c.fields) === null || _d === void 0 ? void 0 : _d.length) {
                i6Data.data.reportingData.contactForm.fields.forEach(function (f, i) {
                    var _a, _b;
                    if ((_a = f.displayName) === null || _a === void 0 ? void 0 : _a.length) {
                        messages.push({
                            channel: 'Agent',
                            fullMsg: f.displayName,
                            msgTime: i6Data.timestampDouble,
                            msgDate: moment.unix(i6Data.timestampDouble).utc().format('YYYY-MM-DD HH:mm:ss.000'),
                            msgNumber: (i + 1)
                        });
                    }
                    if ((_b = f.value) === null || _b === void 0 ? void 0 : _b.length) {
                        messages.push({
                            channel: 'Web Client',
                            fullMsg: f.value,
                            msgTime: i6Data.timestampDouble,
                            msgDate: moment.unix(i6Data.timestampDouble).utc().format('YYYY-MM-DD HH:mm:ss.000'),
                            msgNumber: (i + 1)
                        });
                    }
                });
            }
            break;
        default:
            i6Data.events.forEach(function (e, i) {
                var _a, _b;
                if ((_b = (_a = e.message) === null || _a === void 0 ? void 0 : _a.text) === null || _b === void 0 ? void 0 : _b.length) {
                    messages.push({
                        channel: e.creator === 'contact' ? 'Web Client' : 'Agent',
                        fullMsg: e.message.text,
                        msgTime: e.timestampDouble,
                        msgDate: moment.unix(e.timestampDouble).utc().format('YYYY-MM-DD HH:mm:ss.000'),
                        msgNumber: (i + 1)
                    });
                }
                ;
            });
            break;
    }
    messages.forEach(function (m, i) { return m.msgNumber = (i + 1); });
    return messages;
}
function CloseI6CallInteraction(interaction) {
    return (0, promises_shared_1.DummyPromise)()
        .then(function (_) {
        return mssql_service_1.MSSql.CloseI6InteractionAudio(interaction.vcc, interaction.id, false, interaction.errorMessage);
    })
        .then(function (_) {
        return Promise.resolve(true);
    })
        .catch(function (err) {
        return Promise.reject(err);
    });
}
exports.CloseI6CallInteraction = CloseI6CallInteraction;
function CloseI6TextInteraction(interaction) {
    return (0, promises_shared_1.DummyPromise)()
        .then(function (_) {
        return mssql_service_1.MSSql.CloseI6InteractionAsyncInteraction(interaction.vcc, interaction.id, interaction.folder, interaction.interactionType, false, interaction.errorMessage);
    })
        .then(function (_) {
        return Promise.resolve(true);
    })
        .catch(function (err) {
        return Promise.reject(err);
    });
}
exports.CloseI6TextInteraction = CloseI6TextInteraction;
