"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.SaveWord = exports.SaveNeuroLinguisticProgrammings = exports.SaveNeuroLinguisticProgramming = exports.GetNeuroLinguisticProgramming = exports.NeuroLinguisticProgrammingService = void 0;
var moment = require("moment");
var log4js = require("log4js");
var mssql_service_1 = require("../db/mssql.service");
var promises_shared_1 = require("../shared/promises.shared");
var logger = log4js.getLogger('ServerScripts');
function NeuroLinguisticProgrammingService(app) {
    app.get('/api/neuro_linguistic_programming/get_neuro_linguistic_programmings/:search_criteria/:search_expression?', function (req, res, next) {
        logger.info('Se recibió ' + req.method + ' para \'' + req.path + '\'');
        var vcc = res.locals.vcc;
        var search_criteria = req.params.search_criteria || '';
        var search_expression = req.params.search_expression || '';
        (0, promises_shared_1.DummyPromise)()
            .then(function (result) {
            return mssql_service_1.MSSql.GetNeuroLinguisticProgrammings(vcc, search_criteria, search_expression);
        }).then(function (neuroLinguisticProgrammings) {
            if (res.headersSent)
                return;
            res.json({ status: true, description: 'SERVER_SUCCESS_MESSAGE', data: neuroLinguisticProgrammings });
        }).catch(function (err) {
            var clientMsg = (typeof err === 'string' ? err : 'CANNOT_PROCESS_REQUEST');
            var errorMsg = (typeof err === 'string' ? err : err.message || err.description || 'SERVER_ERROR_UNKNOWN_MESSAGE');
            logger.error('Error al procesar \'' + req.path + '\': ' + errorMsg);
            if (res.headersSent)
                return;
            res.json({ status: false, description: clientMsg, error: errorMsg });
        });
    });
    app.get('/api/neuro_linguistic_programming/get_neuro_linguistic_programming/:neuro_linguistic_programming_id/:processId/:language?', function (req, res, next) {
        logger.info('Se recibió ' + req.method + ' para \'' + req.path + '\'');
        var vcc = res.locals.vcc;
        var neuroLinguisticProgrammingId = req.params.neuro_linguistic_programming_id;
        var processId = req.params.processId;
        var language = req.params.language;
        (0, promises_shared_1.DummyPromise)()
            .then(function (result) {
            if (!neuroLinguisticProgrammingId)
                throw 'NEURO_LINGUISTIC_PROGRAMING_SERVER_ERROR_MISSING_ID';
            return mssql_service_1.MSSql.GetNeuroLinguisticProgramming(vcc, neuroLinguisticProgrammingId, processId, language);
        }).then(function (resultNeuroLinguisticProgramming) {
            if (res.headersSent)
                return;
            res.json({ status: true, description: 'SERVER_SUCCESS_MESSAGE', data: resultNeuroLinguisticProgramming });
        }).catch(function (err) {
            var clientMsg = (typeof err === 'string' ? err : 'CANNOT_PROCESS_REQUEST');
            var errorMsg = (typeof err === 'string' ? err : err.message || err.description || 'SERVER_ERROR_UNKNOWN_MESSAGE');
            logger.error('Error al procesar \'' + req.path + '\': ' + errorMsg);
            if (res.headersSent)
                return;
            res.json({ status: false, description: clientMsg, error: errorMsg });
        });
    });
    app.get('/api/neuro_linguistic_programming/get_neuro_linguistic_programming_words/:search_criteria/:search_expression/:processId/:language?', function (req, res, next) {
        logger.info('Se recibio ' + req.method + ' para \'' + req.path + '\'');
        var vcc = res.locals.vcc;
        var search_criteria = req.params.search_criteria || '';
        var search_expression = req.params.search_expression || '';
        var processId = req.params.processId;
        var language = req.params.language;
        (0, promises_shared_1.DummyPromise)()
            .then(function (result) {
            return mssql_service_1.MSSql.GetWords(vcc, search_criteria, search_expression, processId, language);
        }).then(function (resultWords) {
            if (res.headersSent)
                return;
            res.json({ status: true, description: 'SERVER_SUCCESS_MESSAGE', data: resultWords });
        }).catch(function (err) {
            var clientMsg = (typeof err === 'string' ? err : 'CANNOT_PROCESS_REQUEST');
            var errorMsg = (typeof err === 'string' ? err : err.message || err.description || 'SERVER_ERROR_UNKNOWN_MESSAGE');
            logger.error('Error al procesar \'' + req.path + '\': ' + errorMsg);
            if (res.headersSent)
                return;
            res.json({ status: false, description: clientMsg, error: errorMsg });
        });
    });
    app.get('/api/neuro_linguistic_programming/get_system_words/:search_criteria/:processId/:language?', function (req, res, next) {
        logger.info('Se recibio ' + req.method + ' para \'' + req.path + '\'');
        var vcc = res.locals.vcc;
        var search_criteria = req.params.search_criteria || '';
        var search_expression = req.params.search_expression || '';
        var processId = req.params.processId;
        var language = req.params.language;
        (0, promises_shared_1.DummyPromise)()
            .then(function (result) {
            return mssql_service_1.MSSql.GetSystemWords(vcc, search_criteria, '', processId, language);
        }).then(function (resultSystemWords) {
            if (res.headersSent)
                return;
            res.json({ status: true, description: 'SERVER_SUCCESS_MESSAGE', data: resultSystemWords });
        }).catch(function (err) {
            var clientMsg = (typeof err === 'string' ? err : 'CANNOT_PROCESS_REQUEST');
            var errorMsg = (typeof err === 'string' ? err : err.message || err.description || 'SERVER_ERROR_UNKNOWN_MESSAGE');
            logger.error('Error al procesar \'' + req.path + '\': ' + errorMsg);
            if (res.headersSent)
                return;
            res.json({ status: false, description: clientMsg, error: errorMsg });
        });
    });
    app.post('/api/neuro_linguistic_programming/save_neuro_linguistic_programming', function (req, res, next) {
        logger.info('Se recibió ' + req.method + ' para \'' + req.path + '\'');
        var vcc = res.locals.vcc;
        var userId = res.locals.userId;
        var nowUtc = moment().utc().toDate();
        var newNeuroLinguisticProgramming = (req.body || {});
        newNeuroLinguisticProgramming.id = (newNeuroLinguisticProgramming.id || '').toLowerCase();
        newNeuroLinguisticProgramming.createdDate = null;
        newNeuroLinguisticProgramming.createdByUserId = null;
        newNeuroLinguisticProgramming.lastModifiedDate = nowUtc;
        newNeuroLinguisticProgramming.lastModifiedByUserId = userId;
        (0, promises_shared_1.DummyPromise)()
            .then(function (resultDeleteNeuroLinguisticProgramming) {
            if (!resultDeleteNeuroLinguisticProgramming)
                throw 'NEURO_LINGUISTIC_PROGRAMING_SERVER_ERROR_ON_SAVE';
            return SaveNeuroLinguisticProgramming(vcc, newNeuroLinguisticProgramming);
        }).then(function (resultSaveNeuroLinguisticProgramming) {
            if (!resultSaveNeuroLinguisticProgramming)
                throw 'NEURO_LINGUISTIC_PROGRAMING_SERVER_ERROR_ON_SAVE';
            return Promise.resolve(true);
        }).then(function (resultSaveNeuroLinguisticProgramming) {
            if (res.headersSent)
                return;
            res.json({ status: resultSaveNeuroLinguisticProgramming, description: (resultSaveNeuroLinguisticProgramming ? 'SERVER_SUCCESS_MESSAGE' : 'NEURO_LINGUISTIC_PROGRAMING_SERVER_ERROR_ON_SAVE'), data: resultSaveNeuroLinguisticProgramming });
        }).catch(function (err) {
            var clientMsg = (typeof err === 'string' ? err : 'CANNOT_PROCESS_REQUEST');
            var errorMsg = (typeof err === 'string' ? err : err.message || err.description || 'SERVER_ERROR_UNKNOWN_MESSAGE');
            logger.error('Error al procesar \'' + req.path + '\': ' + errorMsg);
            if (res.headersSent)
                return;
            res.json({ status: false, description: clientMsg, error: errorMsg });
        });
    });
}
exports.NeuroLinguisticProgrammingService = NeuroLinguisticProgrammingService;
function GetNeuroLinguisticProgramming(vcc, id) {
    id = (id || '').toLowerCase();
    var cacheUpdateNeeded = false;
    return Promise.resolve()
        .then(function () {
        return mssql_service_1.MSSql.GetNeuroLinguisticProgramming(vcc, id);
    }).catch(function (err) {
        var errMsg = (typeof err === 'string' ? err : err.message || err.description || 'SERVER_ERROR_UNKNOWN_MESSAGE');
        logger.error('Error al ejecutar GetNeuroLinguisticProgramming: ' + errMsg);
    });
}
exports.GetNeuroLinguisticProgramming = GetNeuroLinguisticProgramming;
function SaveNeuroLinguisticProgramming(vcc, neuroLinguisticProgramming) {
    var promiseArray = [];
    promiseArray.push(mssql_service_1.MSSql.SaveNeuroLinguisticProgramming(vcc, neuroLinguisticProgramming.name, neuroLinguisticProgramming.description, neuroLinguisticProgramming.createdDate, neuroLinguisticProgramming.createdByUserId, neuroLinguisticProgramming.lastModifiedDate, neuroLinguisticProgramming.lastModifiedByUserId));
    if (neuroLinguisticProgramming.words) {
        neuroLinguisticProgramming.words.forEach(function (word) {
            promiseArray.push(mssql_service_1.MSSql.SaveWord(vcc, word.neuroLinguisticProgrammingId, word.name, word.processId, word.language, word.type, (word.createdDate === null ? neuroLinguisticProgramming.lastModifiedDate : null), (word.createdByUserId === null ? neuroLinguisticProgramming.lastModifiedByUserId : null), neuroLinguisticProgramming.lastModifiedDate, neuroLinguisticProgramming.lastModifiedByUserId));
        });
    }
    return Promise.all(promiseArray).then(function (results) {
        if (results.filter(function (result) { return (result !== true); }).length > 0) {
            return false;
        }
        else {
            return true;
        }
    });
}
exports.SaveNeuroLinguisticProgramming = SaveNeuroLinguisticProgramming;
function SaveNeuroLinguisticProgrammings(vcc, neuroLinguisticProgramming) {
    var promiseArray = [];
    if (neuroLinguisticProgramming) {
        neuroLinguisticProgramming.forEach(function (neuroLinguisticProgrammings) {
            promiseArray.push(mssql_service_1.MSSql.SaveNeuroLinguisticProgramming(vcc, neuroLinguisticProgrammings.name, neuroLinguisticProgrammings.description, neuroLinguisticProgrammings.createdDate, neuroLinguisticProgrammings.createdByUserId, neuroLinguisticProgrammings.lastModifiedDate, neuroLinguisticProgrammings.lastModifiedByUserId));
        });
    }
    return Promise.all(promiseArray).then(function (results) {
        if (results.filter(function (result) { return (result !== true); }).length > 0) {
            return false;
        }
        else {
            return true;
        }
    });
}
exports.SaveNeuroLinguisticProgrammings = SaveNeuroLinguisticProgrammings;
function SaveWord(vcc, word) {
    var promiseArray = [];
    if (word) {
        word.forEach(function (words) {
            promiseArray.push(mssql_service_1.MSSql.SaveWord(vcc, words.neuroLinguisticProgrammingId, words.name, words.processId, words.language, words.type, words.createdDate, words.createdByUserId, words.lastModifiedDate, words.lastModifiedByUserId));
        });
    }
    return Promise.all(promiseArray).then(function (results) {
        if (results.filter(function (result) { return (result !== true); }).length > 0) {
            return false;
        }
        else {
            return true;
        }
    });
}
exports.SaveWord = SaveWord;
