"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.GetNLUToTrainDueToRasaUpdate = exports.GetNLUToUtterancesBulk = exports.SaveNLU = exports.GetNLU = exports.FormatWithIntensDB = exports.NLUService = void 0;
var moment = require("moment");
var log4js = require("log4js");
var mssql_service_1 = require("../db/mssql.service");
var redis_service_1 = require("../db/redis.service");
var promises_shared_1 = require("../shared/promises.shared");
var config_1 = require("../config/config");
var file_manager_1 = require("../shared/file-manager");
var utilities_1 = require("../shared/utilities");
var intent_service_1 = require("./intent.service");
var cognitive_analysis_implementation_1 = require("../implementation/cognitive-analysis.implementation");
var validate_shared_1 = require("../shared/validate.shared");
var enum_1 = require("../implementation/cognitive-engine-migration/enum");
var platform_redis_1 = require("../db/redis/platform.redis");
var logger = log4js.getLogger('ServerScripts');
var useRasaApi = config_1.USE_RASA_API;
function NLUService(app) {
    var cognitiveService;
    app.post('/api/nlu/validate_azure_credentials', function (req, res, nex) {
        logger.info('[NLUService::validate_azure_credentials] Received ' + req.method + ' method on \'' + req.path + '\'');
        var vcc = res.locals.vcc;
        var azureCredentials = req.body.azureSubscription;
        var azureSubscriptionKey = req.body.azureSubscriptionKey;
        var tenantVCC = {};
        var authObject;
        (0, promises_shared_1.DummyPromise)()
            .then(function (result) {
            return (0, cognitive_analysis_implementation_1.GetCognitiveService)(null, 'LUIS');
        }).then(function (result) {
            cognitiveService = result;
            return mssql_service_1.MSSql.GetTenantVCCByVCC(vcc);
        })
            .then(function (result) {
            if (!result)
                throw 'SERVER_ERROR_TENANT_NOT_FOUND';
            tenantVCC = result;
            var info = {
                tenant_id: tenantVCC.tenantId,
                client_id: tenantVCC.clientId,
                client_secret: tenantVCC.clientSecret,
                resource: tenantVCC.resource
            };
            return cognitiveService.GetManagerToken(info);
        })
            .then(function (result) {
            if (!result)
                throw 'SERVER_ERROR_NLU_TOKEN_GENERATOR_FAILED';
            authObject = result;
            return cognitiveService.LoadAzureAccounts(azureSubscriptionKey.subscriptionRegion, azureSubscriptionKey.subscriptionKey, authObject);
        })
            .then(function (result) {
            if (!result)
                throw 'SERVER_ERROR_NLU_INVALID_SUBSCRIPTIONKEY';
            if (result && result.length === 0)
                throw 'SERVER_ERROR_NLU_ACCOUNTS_NOT_FOUND';
            var isValid = false;
            var resultDescription;
            var validResult = result.filter(function (r) { return r.AzureSubscriptionId === azureCredentials.azureSubscription; });
            if (validResult && validResult.length > 0) {
                validResult = result.filter(function (r) { return r.ResourceGroup === azureCredentials.resourceGroup; });
                if (validResult && validResult.length > 0) {
                    validResult = result.filter(function (r) { return r.AccountName === azureCredentials.accountName; });
                    if (validResult && validResult.length > 0) {
                        resultDescription = 'OK';
                    }
                    else {
                        resultDescription = 'SERVER_ERROR_NLU_ACCOUNTNAME_NOT_FOUND';
                    }
                }
                else {
                    resultDescription = 'SERVER_ERROR_NLU_RESOURCEGROUP_NOT_FOUND';
                }
            }
            else {
                resultDescription = 'SERVER_ERROR_NLU_SUBSCRIPTION_NOT_FOUND';
            }
            if (validResult && validResult.length > 0)
                isValid = true;
            if (res.headersSent)
                return;
            res.json({ status: true, description: (isValid ? 'SERVER_SUCCESS_MESSAGE' : 'VALIDATE_AZURE_ACCOUNTS_FAILED'), data: { result: isValid, description: resultDescription } });
        })
            .catch(function (err) {
            var clientMsg = (typeof err === 'string' ? err : 'ERROR_COGNITIVE_SERVICE_VALIDATE_AZURE_CREDENTIALS');
            var errorMsg = (typeof err === 'string' ? err : err.message || err.description || 'SERVER_ERROR_UNKNOWN_MESSAGE');
            logger.error('Error al procesar \'' + req.path + '\': ' + errorMsg);
            logger.error('[NLUService::validate_azure_credentials] getting errorMsg:', errorMsg);
            if (res.headersSent)
                return;
            res.json({ status: false, description: clientMsg, error: clientMsg });
        });
    });
    app.post('/api/nlu/get_nlus/', function (req, res, next) {
        logger.info('[NLUService::get_nlus] Received ' + req.method + ' method on \'' + req.path + '\'');
        logger.info('[NLUService::get_nlus] getting search_criteria:', req.params.search_criteria, 'search_expression:', req.params.search_expression);
        var vcc = res.locals.vcc;
        var azure = res.locals.azure;
        logger.info('[NLUService::get_nlus] getting vcc:', vcc, 'azure:', azure);
        var search_criteria = req.body.searchCriteria || '';
        var search_expression = req.body.searchExpression || '';
        var isSystemApp = req.body.isSystemApp || false;
        var arrayId = [];
        var stringId = '';
        (0, promises_shared_1.DummyPromise)()
            .then(function (result) {
            if (isSystemApp) {
                return (0, cognitive_analysis_implementation_1.GetCognitiveService)(null, 'RASA');
            }
            else {
                return (0, cognitive_analysis_implementation_1.GetCognitiveService)(vcc, null);
            }
        }).then(function (result) {
            if (result) {
                cognitiveService = result;
                return cognitiveService.GetAppsList(azure.subscriptionRegion, azure.subscriptionKey);
            }
            else {
                return Promise.resolve(null);
            }
        })
            .then(function (result) {
            if (result === null || result === void 0 ? void 0 : result.length) {
                arrayId = result.map(function (a) { return a.id; });
                logger.info('[NLUService::get_nlus] getting arrayId:', arrayId.length);
                stringId = arrayId.join('\',\'');
                logger.info('[NLUService::get_nlus] getting stringId:', stringId);
            }
            if (cognitiveService.GetCognitiveEngineName() === 'RASA' || cognitiveService.GetCognitiveEngineName() === 'RASAAPI')
                return mssql_service_1.MSSql.GetNLUs(vcc, search_criteria, search_expression, stringId, isSystemApp, true, useRasaApi);
            else
                return mssql_service_1.MSSql.GetNLUs(vcc, search_criteria, search_expression, stringId, isSystemApp, true);
        })
            .then(function (results) {
            logger.info('[NLUService::get_nlus] getting nlus:', results.length);
            if (res.headersSent)
                return;
            res.json({ status: true, description: 'SERVER_SUCCESS_MESSAGE', data: results });
        })
            .catch(function (err) {
            var clientMsg = (typeof err === 'string' ? err : 'ERROR_COGNITIVE_SERVICE_GET_NLUS');
            var errorMsg = (typeof err === 'string' ? err : err.message || err.description || 'SERVER_ERROR_UNKNOWN_MESSAGE');
            logger.error('Error al procesar \'' + req.path + '\': ' + errorMsg);
            logger.error('[NLUService::get_nlus] getting errorMsg:', errorMsg);
            if (res.headersSent)
                return;
            res.json({ status: false, description: clientMsg, error: clientMsg });
        });
    });
    app.get('/api/nlu/get_cultures', function (req, res, next) {
        logger.info('Se recibió ' + req.method + ' para \'' + req.path + '\'');
        var vcc = res.locals.vcc;
        logger.info('[NLUService::/api/nlu/get_cultures] getting vcc:', vcc);
        var azure = res.locals.azure;
        logger.info('[NLUService::/api/nlu/get_cultures] getting azure:', azure);
        (0, promises_shared_1.DummyPromise)()
            .then(function (result) {
            return (0, cognitive_analysis_implementation_1.GetCognitiveService)(vcc, null);
        }).then(function (result) {
            cognitiveService = result;
            if (cognitiveService) {
                return cognitiveService.GetCultures(azure.subscriptionRegion, azure.subscriptionKey);
            }
            else {
                return Promise.resolve([]);
            }
        })
            .then(function (cultures) {
            logger.info('[NLUService::/api/nlu/get_cultures] getting cultures:', cultures, 'after cognitiveService.GetCultures');
            if (res.headersSent)
                return;
            res.json({ status: true, description: 'SERVER_SUCCESS_MESSAGE', data: cultures });
        })
            .catch(function (err) {
            var clientMsg = (typeof err === 'string' ? err : 'CANNOT_PROCESS_REQUEST');
            var errorMsg = (typeof err === 'string' ? err : err.message || err.description || 'SERVER_ERROR_UNKNOWN_MESSAGE');
            logger.error('Error al procesar \'' + req.path + '\': ' + errorMsg);
            logger.error('[NLUService::/api/nlu/get_cultures] getting errorMsg:', errorMsg);
            if (res.headersSent)
                return;
            res.json({ status: false, description: clientMsg, error: errorMsg });
        });
    });
    app.get('/api/nlu/get_nlu_by_name/:name', function (req, res, next) {
        logger.info('[NLUService::get_nlu_by_name] Received ' + req.method + ' method on \'' + req.path + '\'');
        var vcc = res.locals.vcc;
        var azure = res.locals.azure;
        var nluName = req.params.name;
        var nlu = null;
        (0, promises_shared_1.DummyPromise)()
            .then(function (_) {
            return mssql_service_1.MSSql.GetNLU(vcc, nluName);
        })
            .then(function (resultNLU) {
            logger.info('[NLUService::get_nlu_by_name] getting resultNLU:', resultNLU, 'after MSSql.GetNLU');
            nlu = resultNLU;
            if (res.headersSent)
                return;
            res.json({ status: true, description: 'SERVER_SUCCESS_MESSAGE', data: nlu });
        })
            .catch(function (err) {
            var clientMsg = (typeof err === 'string' ? err : 'CANNOT_PROCESS_REQUEST');
            var errorMsg = (typeof err === 'string' ? err : err.message || err.description || 'SERVER_ERROR_UNKNOWN_MESSAGE');
            logger.error('[NLUService::get_nlu_by_name] getting errorMsg:', errorMsg, 'nluname:', nlu.name);
            if (res.headersSent)
                return;
            res.json({ status: false, description: clientMsg, error: errorMsg });
        });
    });
    app.get('/api/nlu/get_nlu/:nlu_id', function (req, res, next) {
        logger.info('[NLUService::get_nlu] Received ' + req.method + ' method on \'' + req.path + '\'');
        var vcc = res.locals.vcc;
        var azure = res.locals.azure;
        var nluId = req.params.nlu_id;
        var nlu = null;
        (0, promises_shared_1.DummyPromise)()
            .then(function (_) {
            return mssql_service_1.MSSql.GetNLUByAppId(vcc, nluId);
        })
            .then(function (resultNLU) {
            logger.info('[NLUService::get_nlu] getting resultNLU:', resultNLU, 'after MSSql.GetNLU');
            nlu = resultNLU;
            if (nlu.isSystemApp) {
                return (0, cognitive_analysis_implementation_1.GetCognitiveService)(null, 'RASA');
            }
            else {
                return (0, cognitive_analysis_implementation_1.GetCognitiveService)(vcc, null);
            }
        })
            .then(function (result) {
            cognitiveService = result;
            if (!nluId)
                throw 'MISSING_NLU_INPUT_PARAMETER';
            return cognitiveService.GetAppVersion(azure.subscriptionRegion, azure.subscriptionKey, nlu.appId, vcc);
        })
            .then(function (result) {
            logger.info('[NLUService::get_nlu] getting result:', result, 'after cognitiveService.GetAppVersion');
            nlu.version = result.version;
            nlu.createdDateTime = result.createdDateTime;
            nlu.lastModifiedDateTime = result.lastModifiedDateTime;
            nlu.lastTrainedDateTime = result.lastTrainedDateTime;
            nlu.lastPublishedDateTime = result.lastPublishedDateTime;
            nlu.endpointUrl = result.endpointUrl;
            nlu.assignedEndpointKey = result.assignedEndpointKey;
            nlu.externalApiKeys = result.externalApiKeys;
            nlu.intentsCount = result.intentsCount;
            nlu.entitiesCount = result.entitiesCount;
            nlu.endpointHitsCount = result.endpointHitsCount;
            nlu.trainingStatus = result.trainingStatus;
            return nlu;
        })
            .then(function (nluModel) {
            logger.info('[NLUService::get_nlu] getting nluModel:', nluModel);
            if (res.headersSent)
                return;
            res.json({ status: true, description: 'SERVER_SUCCESS_MESSAGE', data: nluModel });
        })
            .catch(function (err) {
            var clientMsg = (typeof err === 'string' ? err : 'CANNOT_PROCESS_REQUEST');
            var errorMsg = (typeof err === 'string' ? err : err.message || err.description || 'SERVER_ERROR_UNKNOWN_MESSAGE');
            logger.error('[NLUService::get_nlu] getting errorMsg:', errorMsg, 'nluId:', nluId);
            if (res.headersSent)
                return;
            res.json({ status: false, description: clientMsg, error: errorMsg });
        });
    });
    app.post('/api/nlu/get_unique_nlu_by_intents', function (req, res, nex) {
        var vcc = res.locals.vcc;
        var templateId = (req.body && req.body.templateId ? req.body.templateId : null);
        var intents = (req.body && req.body.intents ? req.body.intents : {});
        (0, promises_shared_1.DummyPromise)()
            .then(function (result) {
            var intentsString = '';
            if (intents && intents.length) {
                var intentNames_1 = [];
                intents.map(function (i) {
                    if (i.includes('|')) {
                        intentNames_1.push(i.split('|')[1]);
                    }
                    else {
                        intentNames_1.push(i);
                    }
                });
                intentsString = intentNames_1.join(',');
            }
            return mssql_service_1.MSSql.GetUniqueNLUByIntents(vcc, templateId, intentsString);
        }).then(function (result) {
            if (res.headersSent)
                return;
            res.json({ status: true, description: 'SERVER_SUCCESS_MESSAGE', data: result });
        }).catch(function (err) {
            var clientMsg = (typeof err === 'string' ? err : 'CANNOT_PROCESS_REQUEST');
            var errorMsg = (typeof err === 'string' ? err : err.message || err.description || 'SERVER_ERROR_UNKNOWN_MESSAGE');
            logger.error('[NLUService::/api/nlu/get_unique_nlu_by_intents] Getting errorMsg:', errorMsg, 'intents:', intents);
            if (res.headersSent)
                return;
            res.json({ status: false, description: clientMsg, error: errorMsg });
        });
    });
    app.post('/api/nlu/create_nlu', function (req, res, next) {
        req.setTimeout(300000, function () { });
        var vcc = res.locals.vcc;
        var azure = res.locals.azure;
        var userId = res.locals.userId;
        var nowUtc = moment().utc().toDate();
        var newNLU = (req.body || {});
        var saveNewNLUResult = null;
        newNLU.vcc = vcc;
        newNLU.createdDate = nowUtc;
        newNLU.createdByUserId = userId;
        newNLU.lastModifiedDate = nowUtc;
        newNLU.lastModifiedByUserId = userId;
        var cognitiveEngineName = '';
        logger.info('[NLUService::create_nlu] getting newNLU:', newNLU);
        (0, promises_shared_1.DummyPromise)()
            .then(function (result) {
            return (0, cognitive_analysis_implementation_1.GetCognitiveService)(vcc, null);
        }).then(function (result) {
            cognitiveService = result;
            cognitiveEngineName = cognitiveService.GetCognitiveEngineName();
            switch (cognitiveEngineName) {
                case 'LUIS': return mssql_service_1.MSSql.GetNLUByName(null, newNLU.name);
                case 'RASA': return mssql_service_1.MSSql.GetNLUByName(vcc, newNLU.name);
                case 'RASAAPI': return mssql_service_1.MSSql.GetNLUByName(vcc, newNLU.name);
                default: throw 'SERVER_ERROR_UNKNOWN_COGNITIVE_ENGINE';
            }
        })
            .then(function (resultNLU) {
            logger.info('[NLUService::create_nlu] getting resultNLU:', resultNLU, 'after MSSql.GetNLUByName');
            if ((resultNLU === null || resultNLU === void 0 ? void 0 : resultNLU.length) > 0)
                throw 'MODEL_NLU_ALREADY_EXIST \'' + newNLU.name + '\'';
            return cognitiveService.AddApp(azure && azure.subscriptionRegion ? azure.subscriptionRegion : null, azure && azure.subscriptionKey ? azure.subscriptionKey : null, newNLU);
        })
            .then(function (result) {
            logger.info('[NLUService::create_nlu] getting result:', result, 'after cognitiveService.AddApp');
            newNLU.appId = result;
            return SaveNLU(vcc, newNLU);
        })
            .then(function (resultSaveNLU) {
            logger.info('[NLUService::create_nlu] getting resultSaveNLU:', resultSaveNLU, 'after SaveNLU');
            if (!resultSaveNLU)
                throw 'ERROR_SAVING_MODEL_NLU';
            saveNewNLUResult = resultSaveNLU;
            if (newNLU.importNLU) {
                return ImportNLUContent(vcc, newNLU, cognitiveService);
            }
            else {
                return saveNewNLUResult;
            }
        })
            .then(function (resultSaveNLU) {
            if (res.headersSent)
                return;
            res.json({ status: true, description: (saveNewNLUResult ? 'SERVER_SUCCESS_MESSAGE' : 'MODEL_SAVE_FAILED'), data: saveNewNLUResult });
        })
            .catch(function (err) {
            var clientMsg = (typeof err === 'string' ? err : 'CANNOT_PROCESS_REQUEST');
            var errorMsg = (typeof err === 'string' ? err : err.message || err.description || 'SERVER_ERROR_UNKNOWN_MESSAGE');
            if (err.error && err.error.error && err.error.error.message) {
                var message = '';
                var message2 = message.concat('An application with the same name (', newNLU.name, ') already exists.');
                if (err.error.error.message === message2) {
                    clientMsg = 'MODEL_NLU_INVALID_NAME';
                    errorMsg = 'MODEL_NLU_INVALID_NAME';
                }
            }
            logger.error('Error al procesar \'' + req.path + '\': ' + errorMsg);
            logger.error('[NLUService::create_nlu] getting errorMsg:', errorMsg);
            if (res.headersSent)
                return;
            res.json({ status: false, description: clientMsg, error: errorMsg });
        });
    });
    app.post('/api/nlu/save_nlu', function (req, res, next) {
        logger.info('Se recibió ' + req.method + ' para \'' + req.path + '\'');
        var vcc = res.locals.vcc;
        logger.info('[NLUService::/api/nlu/save_nlu] getting vcc:', vcc);
        var userId = res.locals.userId;
        logger.info('[NLUService::/api/nlu/save_nlu] getting userId:', userId);
        var nowUtc = moment().utc().toDate();
        logger.info('[NLUService::/api/nlu/save_nlu] getting nowUtc:', nowUtc);
        var newNLU = (req.body || {});
        newNLU.createdDate = null;
        newNLU.createdByUserId = null;
        newNLU.lastModifiedDate = nowUtc;
        newNLU.lastModifiedByUserId = userId;
        logger.info('[NLUService::/api/nlu/save_nlu] getting newNLU:', newNLU);
        (0, promises_shared_1.DummyPromise)()
            .then(function (result) {
            return SaveNLU(vcc, newNLU);
        })
            .then(function (resultSaveNLU) {
            logger.info('[NLUService::/api/nlu/save_nlu] getting resultSaveNLU:', resultSaveNLU);
            if (!resultSaveNLU)
                throw 'ERROR_SAVING_MODEL_NLU';
            return Promise.resolve(true);
        })
            .then(function (resultSaveNLU) {
            if (res.headersSent)
                return;
            res.json({ status: resultSaveNLU, description: (resultSaveNLU ? 'SERVER_SUCCESS_MESSAGE' : 'MODEL_SAVE_FAILED'), data: resultSaveNLU });
        })
            .catch(function (err) {
            var clientMsg = (typeof err === 'string' ? err : 'CANNOT_PROCESS_REQUEST');
            var errorMsg = (typeof err === 'string' ? err : err.message || err.description || 'SERVER_ERROR_UNKNOWN_MESSAGE');
            logger.error('Error al procesar \'' + req.path + '\': ' + errorMsg);
            logger.error('[NLUService::/api/nlu/save_nlu] getting errorMsg:', errorMsg);
            if (res.headersSent)
                return;
            res.json({ status: false, description: clientMsg, error: errorMsg });
        });
    });
    app.post('/api/nlu/delete_nlu', function (req, res, next) {
        logger.info('[NLUService::delete_nlu] Received ' + req.method + ' method on \'' + req.path + '\'');
        var vcc = res.locals.vcc;
        logger.info('[NLUService::delete_nlu] getting vcc:', vcc);
        var azure = res.locals.azure;
        logger.info('[NLUService::delete_nlu] getting azure:', azure);
        var nlu = req.body.nlu;
        logger.info('[NLUService::delete_nlu] getting nlu:', nlu);
        var nluId = nlu.name;
        logger.info("[NluService::delete_nlu] Deleting nlu ".concat(nluId, " by the user ").concat(res.locals.userId));
        (0, promises_shared_1.DummyPromise)()
            .then(function (_) {
            return mssql_service_1.MSSql.GetNlusUsage(vcc, nluId);
        })
            .then(function (inUse) {
            if (inUse) {
                throw 'CANNOT_DELETE_NLU_IN_USE';
            }
            return (0, cognitive_analysis_implementation_1.GetCognitiveService)(vcc, null);
        }).then(function (result) {
            cognitiveService = result;
            if (!nlu)
                throw 'MISSING_NLU_INPUT_PARAMETER';
            nlu.vcc = vcc;
            return cognitiveService.DeleteApp(azure.subscriptionRegion, azure.subscriptionKey, nlu);
        }).then(function (result) {
            return mssql_service_1.MSSql.DeleteNLU(vcc, nluId);
        }).then(function (resultDeleteNLU) {
            logger.info('[NLUService::delete_nlu] getting resultDeleteNLU:', resultDeleteNLU);
            if (!resultDeleteNLU)
                throw 'ERROR_DELETING_MODEL_NLU';
            return Promise.resolve(true);
        })
            .then(function (resultDeleteNLU) {
            if (res.headersSent)
                return;
            res.json({ status: resultDeleteNLU, description: (resultDeleteNLU ? 'SERVER_SUCCESS_MESSAGE' : 'MODEL_DELETE_FAILED'), data: resultDeleteNLU });
        })
            .catch(function (err) {
            var clientMsg = (typeof err === 'string' ? err : 'CANNOT_PROCESS_REQUEST');
            var errorMsg = (typeof err === 'string' ? err : err.message || err.description || 'SERVER_ERROR_UNKNOWN_MESSAGE');
            logger.error('Error al procesar \'' + req.path + '\': ' + errorMsg);
            logger.error('[NLUService::delete_nlu] getting errorMsg:', errorMsg);
            if (res.headersSent)
                return;
            res.json({ status: false, description: clientMsg, error: errorMsg });
        });
    });
    app.post('/api/nlu/delete_nlus', function (req, res, next) {
        logger.info('Se recibió ' + req.method + ' para \'' + req.path + '\'');
        var vcc = res.locals.vcc;
        logger.info('[NLUService::/api/nlu/delete_nlus] getting vcc:', vcc);
        var nlus = req.body.nlus || [];
        logger.info('[NLUService::/api/nlu/delete_nlus] getting nlus:', nlus);
        logger.info("[NluService::delete_nlus] Deleting nlus ".concat(nlus.map(function (x) { return x.name; }).join(','), " by the user ").concat(res.locals.userId));
        (0, promises_shared_1.DummyPromise)()
            .then(function (result) {
            if (!nlus)
                throw 'MISSING_MODELS_NLU_INPUT_PARAMETER';
            var promiseArray = [];
            nlus.forEach(function (nlu) {
                promiseArray.push(mssql_service_1.MSSql.DeleteNLU(vcc, nlu.name));
            });
            return Promise.all(promiseArray);
        })
            .then(function (resultDeleteNLUs) {
            logger.info('[NLUService::/api/nlu/delete_nlus] getting resultDeleteNLUs:', resultDeleteNLUs);
            var result;
            if (resultDeleteNLUs.filter(function (result) { return (result !== true); }).length > 0) {
                result = false;
                logger.error('[NLUService::/api/nlu/delete_nlus] could not delete all requested items');
            }
            else {
                result = true;
                logger.info('[NLUService::/api/nlu/delete_nlus] models deleted successfully');
            }
            return result;
        })
            .then(function (resultDeleteNLUs) {
            if (res.headersSent)
                return;
            res.json({ status: resultDeleteNLUs, description: (resultDeleteNLUs ? 'SERVER_SUCCESS_MESSAGE' : 'MODELS_DELETE_FAILED'), data: resultDeleteNLUs });
        })
            .catch(function (err) {
            var clientMsg = (typeof err === 'string' ? err : 'CANNOT_PROCESS_REQUEST');
            var errorMsg = (typeof err === 'string' ? err : err.message || err.description || 'SERVER_ERROR_UNKNOWN_MESSAGE');
            logger.error('Error al procesar \'' + req.path + '\': ' + errorMsg);
            logger.error('[NLUService::/api/nlu/delete_nlus] getting errorMsg:', errorMsg);
            if (res.headersSent)
                return;
            res.json({ status: false, description: clientMsg, error: errorMsg });
        });
    });
    app.post('/api/nlu/get_subscriptionkey', function (req, res, next) {
        logger.info('[NLUService::get_subscriptionkey] Received ' + req.method + ' method on \'' + req.path + '\'');
        var vcc = res.locals.vcc;
        var azureSubscriptionKey = req.body;
        logger.info('[NLUService::get_subscriptionkey] getting vcc:', vcc);
        (0, promises_shared_1.DummyPromise)()
            .then(function (result) {
            if (azureSubscriptionKey.subscriptionName !== null)
                return mssql_service_1.MSSql.GetAzureSubscriptionKeyByName(vcc, azureSubscriptionKey.subscriptionName);
            else if (azureSubscriptionKey.subscriptionKey !== null)
                return mssql_service_1.MSSql.GetAzureSubscriptionKeyByKey(vcc, azureSubscriptionKey.subscriptionKey);
            else
                throw 'SERVER_ERROR_MISSING_PARAMETERS_FOR_SEARCH';
        })
            .then(function (result) {
            logger.info('[NLUService::get_subscriptionkey] getting keys:', result, 'after MSSql.GetSubscriptionKeys');
            if (res.headersSent)
                return;
            res.json({ status: true, description: 'SERVER_SUCCESS_MESSAGE', data: result });
        })
            .catch(function (err) {
            var clientMsg = (typeof err === 'string' ? err : 'CANNOT_PROCESS_REQUEST');
            var errorMsg = (typeof err === 'string' ? err : err.message || err.description || 'SERVER_ERROR_UNKNOWN_MESSAGE');
            logger.error('Error al procesar \'' + req.path + '\': ' + errorMsg);
            logger.error('[NLUService::get_subscriptionkeys] getting errorMsg:', errorMsg);
            if (res.headersSent)
                return;
            res.json({ status: false, description: clientMsg, error: errorMsg });
        });
    });
    app.post('/api/nlu/get_subscriptionkeys', function (req, res, next) {
        logger.info('[NLUService::get_subscriptionkeys] Received ' + req.method + ' method on \'' + req.path + '\'');
        var vcc = res.locals.vcc;
        logger.info('[NLUService::get_subscriptionkeys] getting vcc:', vcc);
        (0, promises_shared_1.DummyPromise)()
            .then(function (result) {
            return mssql_service_1.MSSql.GetAzureSubscriptionKeys(vcc);
        })
            .then(function (result) {
            logger.info('[NLUService::get_subscriptionkeys] getting keys:', result.length, 'after MSSql.GetSubscriptionKeys');
            if (res.headersSent)
                return;
            res.json({ status: true, description: 'SERVER_SUCCESS_MESSAGE', data: result });
        })
            .catch(function (err) {
            var clientMsg = (typeof err === 'string' ? err : 'CANNOT_PROCESS_REQUEST');
            var errorMsg = (typeof err === 'string' ? err : err.message || err.description || 'SERVER_ERROR_UNKNOWN_MESSAGE');
            logger.error('Error al procesar \'' + req.path + '\': ' + errorMsg);
            logger.error('[NLUService::get_subscriptionkeys] getting errorMsg:', errorMsg);
            if (res.headersSent)
                return;
            res.json({ status: false, description: clientMsg, error: errorMsg });
        });
    });
    app.post('/api/nlu/upload_file/:batch_type_name?/:upload_dest?', function (req, res, next) {
        logger.info('[NLUService::upload_file] Received ' + req.method + ' method on \'' + req.path + '\'');
        var vcc = res.locals.vcc;
        var userId = res.locals.userId;
        var cognitiveEngineSource = req.params.batch_type_name ? req.params.batch_type_name : null;
        var nowUtc = moment().utc().toDate();
        var uploadToFolder = req.params.upload_dest || '';
        var nluFileRedisKey = 'NLU_FILE:' + userId;
        var nluFileContent = null;
        var resultUpload = false;
        var tempFolderName = (uploadToFolder ? uploadToFolder : (config_1.NLU_TEMP_PREFIX + '-' + vcc + '-' + userId).toLowerCase() + '-' + nowUtc.valueOf());
        var tempZipFileName = tempFolderName + '.json';
        var tempUploadPath = (uploadToFolder ? uploadToFolder : config_1.NLU_FOLDER_TEMP_PATH);
        (0, promises_shared_1.DummyPromise)()
            .then(function (result) {
            return (0, cognitive_analysis_implementation_1.GetCognitiveService)(vcc, null);
        }).then(function (result) {
            cognitiveService = result;
            if (!nluFileRedisKey)
                throw 'SERVER_ERROR_NLU_TARGET_NOT_FOUND';
            logger.info('[NLUService::upload_file] Getting tempFolderName:', tempFolderName, 'with tempZipFileName:', tempZipFileName, 'with tempUploadPath:', tempUploadPath);
            return (0, file_manager_1.ExistInFileSystem)(tempUploadPath);
        }).then(function (result) {
            if (result === false) {
                return (0, file_manager_1.CreateFolder)(tempUploadPath);
            }
            else {
                return true;
            }
        }).then(function (result) {
            if (result) {
                return (0, file_manager_1.UploadFileToServer)(tempZipFileName, tempUploadPath, req, res, '(md|json)');
            }
            else {
                return false;
            }
        }).then(function (result) {
            resultUpload = result;
            logger.info('[NLUServiceService::upload_file] Getting resultUpload:', resultUpload, 'after UploadFileToServer');
            if (!resultUpload)
                throw 'SERVER_ERROR_COULD_NOT_UPLOAD_FILE';
            var fileName = "".concat(tempUploadPath, "/").concat(tempZipFileName);
            return (0, file_manager_1.ReadFileContent)(fileName, 'utf-8');
        }).then(function (result) {
            if (!result)
                throw 'SERVER_ERROR_CANNOT_READ_FILE';
            var breakLineRegex = new RegExp("(\r\n|\r)", 'gm');
            nluFileContent = result.replace(breakLineRegex, '\n');
            if (nluFileContent && nluFileContent.length > 0) {
                return redis_service_1.Redis.SaveFileNLU(vcc, nluFileRedisKey, nluFileContent);
            }
            else
                throw 'SERVER_ERROR_UPLOADED_FILE_EMPTY';
        }).then(function (result) {
            if (!result)
                throw 'SERVER_ERROR_COULDNOT_SAVE_FILE_CONTENT';
            logger.info('[NLUServiceService::upload_file] Getting result:', result, 'after ReadFileContent');
            if (result === true) {
                var tempCSVFile = "".concat(tempUploadPath, "/").concat(tempZipFileName);
                logger.info('[NLUServiceService::upload_file] Getting tempCSVFile');
                return (0, file_manager_1.DeleteFile)(tempCSVFile);
            }
            else {
                return Promise.resolve(false);
            }
        }).then(function (result) {
            if (result === true) {
                return redis_service_1.Redis.GetFileNLU(vcc, (nluFileRedisKey));
            }
            else {
                return Promise.resolve(null);
            }
        }).then(function (result) {
            var content = JSON.parse(result.data.replace('"', '\"'));
            var intents = [];
            var utterances = [];
            intents = content.intents ? content.intents : [];
            intents.map(function (i) {
                i.utterances.map(function (u) {
                    utterances.push(u);
                });
            });
            if (intents.length === 0)
                throw 'SERVER_ERROR_INTENTS_NOT_FOUND';
            intents.map(function (i) {
                if (i.name.toLowerCase() === 'none')
                    throw 'SERVER_ERROR_INTENT_ID_INVALID';
                if ((0, validate_shared_1.IsStringAndEmpty)(i.name))
                    throw 'SERVER_ERROR_INTENT_ID_MISSING';
                if (!(0, validate_shared_1.IsStringAndValiadId)(i.name))
                    throw 'SERVER_ERROR_INTENT_ID_INVALID';
            });
            if (res.headersSent)
                return;
            var data = {
                tempFolderName: tempFolderName,
                content: result.data,
                intents: intents.length,
                utterances: utterances.length
            };
            res.json({ status: resultUpload, description: (resultUpload ? 'FILE_STORED_SUCCESSFULLY' : 'FILE_UPLOAD_FAILED'), data: data });
        }).catch(function (err) {
            var processMsg = (typeof err === 'string' ? err : 'CANNOT_PROCESS_REQUEST');
            var errorMsg = (typeof err === 'string' ? err : err.message || err.description || 'SERVER_ERROR_UNKNOWN_MESSAGE');
            logger.error('Error al procesar \'' + req.path + '\': ' + errorMsg);
            logger.error('[NLUService::upload_file] Getting errorMsg:', errorMsg);
            if (res.headersSent)
                return;
            res.json({ status: false, description: processMsg, error: errorMsg });
        });
    });
    app.post('/api/nlu/delete_file_temporal/', function (req, res, next) {
        logger.info('[NLUService::delete_file_temporal] Received ' + req.method + ' method on \'' + req.path + '\'');
        var vcc = res.locals.vcc;
        var userId = res.locals.userId;
        var nluFileRedisKey = 'NLU_FILE:' + userId;
        logger.info('[NLUService::delete_file_temporal] Getting vcc:userId : ', vcc, ':', userId);
        logger.info("[NLUService::delete_file_temporal] Deleting  ".concat(nluFileRedisKey, " by the user ").concat(res.locals.userId));
        (0, promises_shared_1.DummyPromise)()
            .then(function (result) {
            return redis_service_1.Redis.GetFileNLU(vcc, nluFileRedisKey);
        }).then(function (result) {
            if (result && result.data.length > 0) {
                return redis_service_1.Redis.DeleteFileNLU(vcc, nluFileRedisKey);
            }
            else {
                return Promise.resolve(true);
            }
        }).then(function (result) {
            if (res.headersSent)
                return;
            res.json({ status: result, description: (result ? 'CLEAR SUCCESS' : 'CLEAR ERROR '), data: result });
        }).catch(function (err) {
            var processMsg = (typeof err === 'string' ? err : 'CANNOT_PROCESS_REQUEST');
            var errorMsg = (typeof err === 'string' ? err : err.message || err.description || 'SERVER_ERROR_UNKNOWN_MESSAGE');
            logger.error('Error al procesar \'' + req.path + '\': ' + errorMsg);
            logger.error('[NLUService::delete_file_temporal] Getting errorMsg:', errorMsg);
            if (res.headersSent)
                return;
            res.json({ status: false, description: processMsg, error: errorMsg });
        });
    });
    app.post('/api/nlu/export_nlu', function (req, res, next) {
        logger.info('[NLUService::export_nlu] Received ' + req.method + ' method on \'' + req.path + '\'');
        var vcc = res.locals.vcc;
        logger.info('[NLUService::export_nlu] getting vcc:', vcc);
        var nlu = req.body.nlu;
        nlu.vcc = vcc;
        logger.info('[NLUService::export_nlu] getting nlu:', nlu);
        (0, promises_shared_1.DummyPromise)()
            .then(function (result) {
            return (0, cognitive_analysis_implementation_1.GetCognitiveService)(vcc, null);
        }).then(function (result) {
            cognitiveService = result;
            return cognitiveService.ExportModel(nlu, res);
        })
            .then(function (result) {
            res.setHeader('Content-type', 'application/json; charset=utf-8');
            res.setHeader('Content-disposition', 'attachment; filename=');
            res.send(JSON.stringify(result, null, 3));
        }).catch(function (err) {
            var clientMsg = (typeof err === 'string' ? err : 'CANNOT_PROCESS_REQUEST');
            var errorMsg = (typeof err === 'string' ? err : err.message || err.description || 'SERVER_ERROR_UNKNOWN_MESSAGE');
            logger.error('[NLUService::export_nlu] Getting errorMsg:', errorMsg);
            if (res.headersSent)
                return;
            res.json({ status: false, description: clientMsg, error: errorMsg });
        });
    });
    app.get("/api/nlu/nlu-to-export/", function (req, res) {
        logger.info('[NLUService::nlu-to-export] Received ' + req.method + ' method on \'' + req.path + '\'');
        var vcc = req.query.vcc ? req.query.vcc.toString() : '';
        (0, promises_shared_1.DummyPromise)().then(function (_) {
            return mssql_service_1.MSSql.GetNluIntentUtteranceToExport(vcc, 0);
        }).then(function (result) {
            if (!result && Object.keys(result).length == 0)
                throw 'DATA_NLU_NOT_FOUND';
            res.json({ status: true, data: result, description: 'SERVER_SUCCESS_MESSAGE' });
        }).catch(function (err) {
            var clientMsg = (typeof err === 'string' ? err : 'CANNOT_PROCESS_REQUEST');
            var errorMsg = (typeof err === 'string' ? err : err.message || err.description || 'SERVER_ERROR_UNKNOWN_MESSAGE');
            logger.error('[NLUService::nlu-to-export] Getting errorMsg:', errorMsg);
            if (res.headersSent)
                return;
            res.json({ status: false, description: clientMsg, error: errorMsg });
        });
    });
    app.post('/api/nlu/retry_nlu/', function (req, res) {
        var vcc = res.locals.vcc;
        var appId = req.body.appId || null;
        (0, promises_shared_1.DummyPromise)().then(function (_) {
            return redis_service_1.Redis.GetCognitiveEngineMigrationModel("".concat(enum_1.ROOT_KEY, ":").concat(vcc, ":Nlus"), appId);
        })
            .then(function (nluStrings) {
            if (nluStrings && nluStrings.length > 0) {
                var nlu = JSON.parse(nluStrings[0]);
                var promises = [];
                promises.push(mssql_service_1.MSSql.SaveCognitiveEngineMigrateStatus(vcc, 'nlu', nlu.name, 2, 'RETRYING'));
                promises.push(redis_service_1.Redis.UpdateCognitiveEngineMigrationModel("".concat(enum_1.ROOT_KEY, ":").concat(vcc, ":Nlus"), nlu.appId, Object.assign(nlu, {
                    status: 0,
                    totalRetries: nlu.totalRetries + 1
                })));
                return Promise.all(promises);
            }
            else {
                throw 'NOT_FOUND_MODEL';
            }
        })
            .then(function (results) {
            if (!results.includes(false)) {
                res.json({ status: true, data: true, description: 'SERVER_SUCCESS_MESSAGE' });
            }
            else {
                throw 'CANNOT_UPDATE_MODEL';
            }
        })
            .catch(function (err) {
            var clientMsg = (typeof err === 'string' ? err : 'CANNOT_PROCESS_REQUEST');
            var errorMsg = (typeof err === 'string' ? err : err.message || err.description || 'SERVER_ERROR_UNKNOWN_MESSAGE');
            logger.error('[NLUService::retry_nlu] Getting errorMsg:', errorMsg);
            if (res.headersSent)
                return;
            res.json({ status: false, description: clientMsg, error: errorMsg });
        });
    });
    app.post('/api/nlu/test_nlu_phrase/', function (req, res) {
        var nlu = req.body.nlu;
        var appId = nlu.appId || null;
        var phrase = nlu.phraseToTest;
        var apiIdLabel = nlu.cognitiveEngine;
        var subscriptionRegion = null;
        var subscriptionKey = null;
        var currentSubscriptionKey = {};
        var installationId = null;
        (0, promises_shared_1.DummyPromise)().then(function (_) {
            return (0, cognitive_analysis_implementation_1.GetCognitiveService)(nlu.vcc, null);
        }).then(function (result) {
            cognitiveService = result;
            if (apiIdLabel === "RASA") {
                return (0, platform_redis_1.GetPlatformVersion)();
            }
            else if (apiIdLabel === "LUIS") {
                return mssql_service_1.MSSql.GetAzureSubscriptionKeyByKey(nlu.vcc, nlu.subscriptionKey);
            }
            else {
                return true;
            }
        })
            .then(function (result) {
            if (apiIdLabel === 'RASA') {
                installationId = result.installationId;
                return redis_service_1.Redis.GetRunningRasaModelFromTest(installationId + '::' + appId.toLowerCase());
            }
            else
                return result;
        }).then(function (result) {
            if (apiIdLabel === 'RASA') {
                if (result === null)
                    return cognitiveService.RunApp(installationId + '::' + appId.toLowerCase(), 'TEST');
                else
                    result.created = moment.utc().format('YYYY-MM-DD HH:mm:ss');
                return redis_service_1.Redis.SaveRunningRasaModelFromTest(installationId + '::' + appId.toLowerCase(), result);
            }
            else if (apiIdLabel === 'LUIS') {
                currentSubscriptionKey = result;
                subscriptionRegion = currentSubscriptionKey.subscriptionRegion;
                subscriptionKey = nlu.subscriptionKey;
                return null;
            }
            else {
                return null;
            }
        }).then(function (result) {
            if (apiIdLabel === 'RASA') {
                return cognitiveService.ModelQuery(subscriptionRegion, subscriptionKey, appId, phrase, null, null, null, 'TEST');
            }
            else
                return cognitiveService.ModelQuery(subscriptionRegion, subscriptionKey, appId, phrase, null, null, null);
        }).then(function (result) {
            if (apiIdLabel === "RASA")
                return FormatWithIntensDB(nlu, result);
            else
                return result;
        }).then(function (result) {
            res.json({ status: true, data: result, description: 'SERVER_SUCCESS_MESSAGE' });
        }).catch(function (err) {
            var clientMsg = (typeof err === 'string' ? err : 'CANNOT_PROCESS_REQUEST');
            var errorMsg = (typeof err === 'string' ? err : err.message || err.description || 'SERVER_ERROR_UNKNOWN_MESSAGE');
            logger.error('[NLUService::test_nlu_phrase] Getting err:', err);
            logger.error('[NLUService::test_nlu_phrase] Getting errmessage:', err.message);
            logger.error('[NLUService::test_nlu_phrase] Getting errorMsg:', errorMsg);
            logger.error('[NLUService::test_nlu_phrase] Getting clientMsg:', clientMsg);
            if (res.headersSent)
                return;
            res.json({ status: false, description: clientMsg, error: errorMsg });
        });
    });
    app.post('/api/nlu/stop_test_nlu_phrase/', function (req, res) {
        var nlu = req.body.nlu;
        var appId = nlu.appId || null;
        var apiIdLabel = nlu.cognitiveEngine;
        var installationId = null;
        (0, promises_shared_1.DummyPromise)().then(function (_) {
            return (0, cognitive_analysis_implementation_1.GetCognitiveService)(nlu.vcc, null);
        }).then(function (result) {
            cognitiveService = result;
            if (apiIdLabel === "RASA") {
                return (0, platform_redis_1.GetPlatformVersion)();
            }
            else
                return null;
        })
            .then(function (result) {
            if (apiIdLabel === 'RASA') {
                installationId = result.installationId;
                return redis_service_1.Redis.GetRunningRasaModelFromTest(installationId + '::' + appId.toLowerCase());
            }
        }).then(function (result) {
            if (apiIdLabel === "RASA") {
                if (result !== null)
                    return cognitiveService.StopApp(installationId + '::' + appId.toLowerCase(), result.port, 'TEST');
                return null;
            }
            else
                return null;
        }).then(function (result) {
            res.json({ status: true, data: result, description: 'SERVER_SUCCESS_MESSAGE' });
        }).catch(function (err) {
            var clientMsg = (typeof err === 'string' ? err : 'CANNOT_PROCESS_REQUEST');
            var errorMsg = (typeof err === 'string' ? err : err.message || err.description || 'SERVER_ERROR_UNKNOWN_MESSAGE');
            logger.error('[NLUService::stop_test_nlu_phrase] Getting errorMsg:', errorMsg);
            if (res.headersSent)
                return;
            res.json({ status: false, description: clientMsg, error: errorMsg });
        });
    });
}
exports.NLUService = NLUService;
function FormatWithIntensDB(nlu, resFromModelQuery) {
    return (0, promises_shared_1.DummyPromise)()
        .then(function (result) {
        return mssql_service_1.MSSql.GetIntents(nlu.vcc, nlu.appId, null, null, null);
    })
        .then(function (result) {
        var res = result ? result : [];
        var arrRes = [];
        res.map(function (r) {
            arrRes.push({
                id: r.intentId,
                name: r.name
            });
        });
        var joinIntents = [];
        arrRes.map(function (x) {
            resFromModelQuery.intents.map(function (y) {
                if (x.id === y.intent)
                    joinIntents.push({
                        intent: x.name,
                        score: y.score
                    });
            });
        });
        resFromModelQuery.intents = joinIntents;
        return Promise.resolve(resFromModelQuery);
    }).catch(function (error) {
        var errorMessage = typeof error === 'string' ? error : 'ERROR';
        logger.error('[NLUService]::GetIntentsListFromDB] Error: ', error);
        return Promise.reject(errorMessage);
    });
}
exports.FormatWithIntensDB = FormatWithIntensDB;
function GetNLU(vcc, name) {
    logger.info('[NLUService::GetNLU] getting vcc:', vcc, 'name:', name);
    var cacheUpdateNeeded = false;
    return Promise.resolve()
        .then(function () {
        return mssql_service_1.MSSql.GetNLU(vcc, name);
    })
        .catch(function (err) {
        var errorMsg = (typeof err === 'string' ? err : err.message || err.description || 'SERVER_ERROR_UNKNOWN_MESSAGE');
        logger.error('[NLUService::GetNLU] getting errorMsg:', errorMsg);
    });
}
exports.GetNLU = GetNLU;
function SaveNLU(vcc, nlu) {
    logger.info('[NLUService::SaveNLU] getting vcc:', vcc, 'nlu:', nlu);
    var promiseArray = [];
    promiseArray.push(mssql_service_1.MSSql.SaveNLU(vcc, nlu.name, nlu.culture, nlu.description, nlu.appId, nlu.subscriptionKey, nlu.isSystemApp, nlu.needTrain, nlu.needPublish, nlu.isPublished, nlu.lastPublishedDate, nlu.isTrained, nlu.lastTrainedDate, nlu.createdDate, nlu.createdByUserId, nlu.lastModifiedDate, nlu.lastModifiedByUserId, nlu.trainVersion, nlu.trainId));
    return Promise.all(promiseArray)
        .then(function (results) {
        logger.info('[NLUService::SaveNLU] getting results:', results, 'after Promise.all MSSql.SaveNLU');
        if (results.filter(function (result) { return (result !== true); }).length > 0) {
            return false;
        }
        else {
            return true;
        }
    });
}
exports.SaveNLU = SaveNLU;
function ImportNLUContent(vcc, nlu, cognitiveService) {
    logger.info('[NLUService::ImportNLUContent] getting vcc:', vcc, 'nlu:', nlu.appId);
    var nluContent = null;
    var nluFileRedisKey = 'NLU_FILE:' + nlu.createdByUserId;
    return (0, promises_shared_1.DummyPromise)().then(function (result) {
        return redis_service_1.Redis.GetFileNLU(nlu.vcc, (nluFileRedisKey));
    }).then(function (result) {
        if (!result)
            throw 'SERVER_ERROR_NLU_REDIS_DATA_NOT_FOUND';
        nluContent = JSON.parse(result.data.replace('"', '\"'));
        var intents = [];
        var promiseArray = [];
        nluContent.intents.map(function (i) {
            if (i.name.toLowerCase() === 'none')
                throw 'SERVER_ERROR_INTENT_ID_INVALID';
            var newIntent = {};
            newIntent.appId = nlu.appId;
            newIntent.name = i.name;
            newIntent.intentId = i.id;
            newIntent.createdDate = moment().utc().toDate();
            newIntent.createdByUserId = nlu.createdByUserId;
            newIntent.color = (0, utilities_1.getRandomHexColor)();
            promiseArray.push((0, intent_service_1.SaveIntent)(vcc, newIntent));
            i.utterances.map(function (u) {
                promiseArray.push(mssql_service_1.MSSql.SaveUtterance(vcc, nlu.appId, newIntent.intentId, newIntent.name, u.id, u.text, nlu.createdByUserId));
            });
        });
        nluContent.entities.map(function (e) {
            promiseArray.push(mssql_service_1.MSSql.SaveEntity(vcc, nlu.appId, e.id, e.name, e.type, nlu.createdByUserId, null));
        });
        nluContent.utteranceEntities.map(function (ue) {
            promiseArray.push(mssql_service_1.MSSql.SaveUtteranceEntity(vcc, ue.utteranceId, ue.entityName, ue.start, ue.end, ue.startTokenIndex, ue.endTokenIndex));
        });
        return Promise.all(promiseArray);
    }).then(function (result) {
        return redis_service_1.Redis.DeleteFileNLU(nlu.vcc, (nluFileRedisKey));
    }).then(function (result) {
        return Promise.resolve(true);
    }).catch(function (err) {
        var errorMsg = (typeof err === 'string' ? err : err.message || err.description || 'SERVER_ERROR_UNKNOWN_MESSAGE');
        logger.error('[NLUService::ImportNLUContent] getting errorMsg:', errorMsg);
        return Promise.reject(errorMsg);
    });
}
function GetNLUToUtterancesBulk(searchOnDB) {
    if (searchOnDB === void 0) { searchOnDB = false; }
    var searchResult = [];
    var needUpdate = false;
    return (0, promises_shared_1.DummyPromise)()
        .then(function (result) {
        return redis_service_1.Redis.GetNLUToUtterancesBulk();
    })
        .then(function (result) {
        if (!result || searchOnDB) {
            needUpdate = true;
            return mssql_service_1.MSSql.GetNLUs('ALL', 'LastModifiedByUserId', 'dev-1', null, false);
        }
        else {
            return Promise.resolve(result);
        }
    })
        .then(function (result) {
        searchResult = result;
        if (needUpdate) {
            return redis_service_1.Redis.SaveNLUToUtterancesBulk(searchResult);
        }
        else {
            return Promise.resolve(true);
        }
    })
        .then(function (result) {
        return Promise.resolve(searchResult);
    })
        .catch(function (err) {
        var errorMsg = (typeof err === 'string' ? err : err.message || err.description || 'SERVER_ERROR_UNKNOWN_MESSAGE');
        logger.error('[NLUService::GetNLUToUtterancesBulk] getting errorMsg:', errorMsg);
        return Promise.resolve([]);
    });
}
exports.GetNLUToUtterancesBulk = GetNLUToUtterancesBulk;
function GetNLUToTrainDueToRasaUpdate(searchOnDB) {
    var searchResult = [];
    var needUpdate = false;
    return (0, promises_shared_1.DummyPromise)()
        .then(function (result) {
        return redis_service_1.Redis.GetNLUToTrainDueToRasaUpdate();
    })
        .then(function (result) {
        if (!result || searchOnDB) {
            needUpdate = true;
            return mssql_service_1.MSSql.GetNLUs('ALL', 'LastModifiedByUserId', 'dev-2', null, false);
        }
        else {
            return Promise.resolve(result);
        }
    })
        .then(function (result) {
        searchResult = result;
        if (needUpdate) {
            return redis_service_1.Redis.SaveNLUToTrainDueToRasaUpdate(searchResult);
        }
        else {
            return Promise.resolve(true);
        }
    })
        .then(function (result) {
        return Promise.resolve(searchResult);
    })
        .catch(function (err) {
        var errorMsg = (typeof err === 'string' ? err : err.message || err.description || 'SERVER_ERROR_UNKNOWN_MESSAGE');
        logger.error('[NLUService::GetNLUToTrainDueToRasaUpdate] getting errorMsg:', errorMsg);
        return Promise.resolve([]);
    });
}
exports.GetNLUToTrainDueToRasaUpdate = GetNLUToTrainDueToRasaUpdate;
