"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.TeamPerformanceService = void 0;
var moment = require("moment");
var log4js = require("log4js");
var team_performance_model_1 = require("../models/team-performance.model");
var mssql_service_1 = require("../db/mssql.service");
var promises_shared_1 = require("../shared/promises.shared");
var logger = log4js.getLogger('ServerScripts');
function TeamPerformanceService(app) {
    app.get('/api/team_performance/get_filters/:batchType/:evaluationFormatId/:templateId', function (req, res, next) {
        logger.info("[TeamPerformanceService::get_filters] Received ".concat(req.method, " method on '").concat(req.path, "'"));
        var vcc = res.locals.vcc;
        var batchType = req.params.batchType || '';
        var evaluationFormatId = req.params.evaluationFormatId || '';
        var templateId = req.params.templateId || '';
        (0, promises_shared_1.DummyPromise)()
            .then(function (_) {
            return mssql_service_1.MSSql.GetTeamPerformanceFilters(vcc, batchType, evaluationFormatId, templateId);
        })
            .then(function (result) {
            if (res.headersSent)
                return;
            res.json({ status: true, description: 'SERVER_SUCCESS_MESSAGE', data: result });
        })
            .catch(function (err) {
            var clientMsg = (typeof err === 'string' ? err : 'CANNOT_PROCESS_REQUEST');
            var errorMsg = (typeof err === 'string' ? err : err.message || err.description || 'SERVER_ERROR_UNKNOWN_MESSAGE');
            logger.error("[TeamPerformanceService::get_filters] Error ".concat(req.path, ": '").concat(errorMsg, "'"));
            if (res.headersSent)
                return;
            res.json({
                status: false,
                description: clientMsg,
                error: errorMsg,
                detail: {
                    type: 'ERROR',
                    datetime: moment().utc().toDate(),
                    pid: process.pid,
                    source: __filename,
                    stack: err.stack
                }
            });
        });
    });
    app.post('/api/team_performance/get_team_performance_from_evaluation', function (req, res, next) {
        logger.info('[TeamPerformanceService::get_team_performance_from_evaluation] Received ' + req.method + ' method on \'' + req.path + '\'');
        var vcc = res.locals.vcc;
        var currentUserRole = res.locals.user.role;
        var currentUserId = res.locals.userId;
        var userFuntion = null;
        var params = new team_performance_model_1.TeamPerformanceFilters;
        params.evaluationPlans = req.body.evaluationPlans || [];
        params.evaluationFormatId = req.body.evaluationFormatId || null;
        params.agents = req.body.agents || [];
        params.abilities = req.body.abilities || [];
        params.startDate = req.body.startDate;
        params.endDate = req.body.endDate || [];
        params.drilldownTarget = req.body.drilldownTarget || null;
        params.drilldownId = req.body.drilldownId || null;
        params.supervisors = req.body.supervisors || [];
        params.resultBySupervisor = req.body.resultBySupervisor || null;
        (0, promises_shared_1.DummyPromise)()
            .then(function (_) {
            logger.info('[TeamPerformanceService::get_team_performance_from_evaluation] Calling GetTeamPerformanceFromEvaluationPlan SP for VCC ' + vcc);
            return mssql_service_1.MSSql.GetTeamPerformanceFromEvaluationPlan(vcc, params.evaluationFormatId, params.evaluationPlans, params.agents, params.abilities, params.startDate, params.endDate, params.drilldownTarget, params.drilldownId, params.supervisors, params.resultBySupervisor);
        })
            .then(function (result) {
            if (res.headersSent)
                return;
            logger.info('[TeamPerformanceService::get_team_performance_from_evaluation] Received result from SP GetTeamPerformanceFromEvaluationPlan from local DB');
            res.json({ status: true, description: 'SERVER_SUCCESS_MESSAGE', data: result });
        })
            .catch(function (err) {
            var clientMsg = (typeof err === 'string' ? err : 'CANNOT_PROCESS_REQUEST');
            var errorMsg = (typeof err === 'string' ? err : err.message || err.description || 'Requesting error');
            logger.error('[TeamPerformanceService::get_team_performance_from_evaluation] Error on \'' + req.path + '\': ' + errorMsg);
            if (res.headersSent)
                return;
            res.json({ status: false, description: clientMsg, error: errorMsg });
        });
    });
}
exports.TeamPerformanceService = TeamPerformanceService;
