"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.PublishRasaApiApp = exports.TrainApp = exports.TrainService = void 0;
var moment = require("moment");
var log4js = require("log4js");
var mssql_service_1 = require("../db/mssql.service");
var promises_shared_1 = require("../shared/promises.shared");
var nlu_service_1 = require("./nlu.service");
var cognitive_analysis_implementation_1 = require("../implementation/cognitive-analysis.implementation");
var nlu_status_const_1 = require("../const/nlu-status.const");
var nlu_mssql_1 = require("../db/mssql/nlu.mssql");
var io_model_1 = require("../models/io.model");
var io_request_1 = require("../io/io.request");
var events_1 = require("../io/events");
var utterance_service_1 = require("./utterance.service");
var utterance_service_2 = require("./utterance.service");
var logger = log4js.getLogger('ServerScripts');
function TrainService(app) {
    var cognitiveService;
    app.post('/api/train/train_app', function (req, res, next) {
        req.setTimeout(300000, function () { });
        var vcc = res.locals.vcc;
        var azure = res.locals.azure;
        var param = (req.body || {});
        var nlu = {};
        var trainResult = null;
        (0, promises_shared_1.DummyPromise)()
            .then(function (result) {
            return (0, cognitive_analysis_implementation_1.GetCognitiveService)(vcc, null);
        }).then(function (result) {
            cognitiveService = result;
            return mssql_service_1.MSSql.GetNLUByAppId(vcc, param.appId);
        })
            .then(function (result) {
            if (!result)
                throw 'SERVER_ERROR_NLU_MODEL_NOT_FOUND';
            nlu = result;
            if (nlu.cognitiveEngine === 'RASA') {
                return mssql_service_1.MSSql.GetNLUs(vcc, null, null, null, null, true);
            }
            return true;
        })
            .then(function (result) {
            if (typeof result === 'boolean') {
                return TrainApp(cognitiveService, azure, nlu);
            }
            var count = result.reduce(function (n, x) { return n + (x.statusCode === 'TRAINING'); }, 0);
            if (count < 10) {
                TrainApp(cognitiveService, azure, nlu).then(function (_) { });
                return true;
            }
            else {
                return Promise.reject('SERVER_CONCURRENT_TRAINING_MODELS_LIMIT_REACHED');
            }
        })
            .then(function (result) {
            trainResult = result;
            if (res.headersSent)
                return;
            res.json({ status: true, description: 'SERVER_SUCCESS_MESSAGE', data: trainResult });
        }).catch(function (err) {
            var clientMsg = (typeof err === 'string' ? err : 'CANNOT_PROCESS_REQUEST');
            var errorMsg = (typeof err === 'string' ? err : err.message || err.description || 'SERVER_ERROR_UNKNOWN_MESSAGE');
            logger.error('[TrainService::train_app] Getting error:', errorMsg, 'subscriptionRegion:', azure.subscriptionRegion, 'subscriptionKey:', azure.subscriptionKey, 'appId:', param.appId);
            if (res.headersSent)
                return;
            res.json({ status: false, description: clientMsg, error: errorMsg });
        });
    });
    app.get('/api/train/get_train_app/:app_id', function (req, res, next) {
        logger.info('Se recibió ' + req.method + ' para \'' + req.path + '\'');
        var vcc = res.locals.vcc;
        var azure = res.locals.azure;
        var appId = req.params.app_id || '';
        var trainData = [];
        var intents = [];
        (0, promises_shared_1.DummyPromise)()
            .then(function (result) {
            return (0, cognitive_analysis_implementation_1.GetCognitiveService)(vcc, null);
        }).then(function (result) {
            cognitiveService = result;
            return cognitiveService.GetTrainApp(azure.subscriptionRegion, azure.subscriptionKey, appId);
        }).then(function (result) {
            trainData = result;
            trainData = trainData.map(function (x) {
                if (x.details.failureReason) {
                    x.details['failureReasonMessage'] = x.details.failureReason.toUpperCase();
                }
                return x;
            });
            var arrayId = result.map(function (a) { return a.modelId; });
            var stringId = arrayId.join('\',\'');
            return mssql_service_1.MSSql.GetIntents(vcc, appId, 'name', '', stringId);
        }).then(function (result) {
            intents = result;
            logger.info('[IntentService::/api/intent/get_intents/] Getting result:', result.length, 'after MSSql.GetIntents');
            return GetVersionTrainingStatus(intents, trainData);
        }).then(function (result) {
            if (res.headersSent)
                return;
            res.json({ status: true, description: 'SERVER_SUCCESS_MESSAGE', data: result });
        })
            .catch(function (err) {
            var clientMsg = (typeof err === 'string' ? err : 'CANNOT_PROCESS_REQUEST');
            var errorMsg = (typeof err === 'string' ? err : err.message || err.description || 'SERVER_ERROR_UNKNOWN_MESSAGE');
            logger.error('[TrainService::/api/train/get_train_app/] Getting error:', errorMsg, 'subscriptionRegion:', azure.subscriptionRegion, 'subscriptionKey:', azure.subscriptionKey, 'appId:', appId);
            if (res.headersSent)
                return;
            res.json({ status: false, description: clientMsg, error: errorMsg });
        });
    });
    app.post('/api/train/publish_app', function (req, res, next) {
        logger.info('[TrainService::publish_app] Received ' + req.method + ' method on \'' + req.path + '\'');
        var vcc = res.locals.vcc;
        var azure = res.locals.azure;
        var param = (req.body || {});
        var nlu = {};
        var azureResult = null;
        var azureSubscription = {};
        var azureSubscriptionKey = {};
        var azureSubscriptionDemo = {};
        var tenantVCC = {};
        var authObject;
        (0, promises_shared_1.DummyPromise)()
            .then(function (result) {
            return (0, cognitive_analysis_implementation_1.GetCognitiveService)(vcc, null);
        }).then(function (result) {
            cognitiveService = result;
            return mssql_service_1.MSSql.GetNLUByAppId(vcc, param.appId);
        })
            .then(function (result) {
            if (!result)
                throw 'SERVER_ERROR_NLU_MODEL_NOT_FOUND';
            nlu = result;
            if (cognitiveService.GetCognitiveEngineName() === 'LUIS') {
                var promiseArray = [];
                promiseArray.push(mssql_service_1.MSSql.GetAzureSubscriptionKeyByName(vcc, 'DEMO'));
                promiseArray.push(mssql_service_1.MSSql.GetAzureSubscriptionKeyByKey(vcc, param.subscriptionKey));
                promiseArray.push(mssql_service_1.MSSql.GetTenantVCCByVCC(vcc));
                return Promise.all(promiseArray);
            }
            else {
                return null;
            }
        })
            .then(function (result) {
            if (cognitiveService.GetCognitiveEngineName() === 'LUIS') {
                if (!result[0])
                    throw 'SERVER_ERROR_NLU_SUBSCRIPTIONDEMO_NOT_FOUND';
                if (!result[1])
                    throw 'SERVER_ERROR_NLU_SUBSCRIPTIONKEY_NOT_FOUND';
                if (!result[2])
                    throw 'SERVER_ERROR_TENANT_NOT_FOUND';
                tenantVCC = result[2];
                azureSubscriptionKey = result[1];
                azureSubscriptionDemo = result[0];
                var info = {
                    tenant_id: tenantVCC.tenantId,
                    client_id: tenantVCC.clientId,
                    client_secret: tenantVCC.clientSecret,
                    resource: tenantVCC.resource
                };
                if (nlu.subscriptionKey !== param.subscriptionKey) {
                    return cognitiveService.GetManagerToken(info);
                }
                else {
                    return true;
                }
            }
            else {
                return true;
            }
        })
            .then(function (result) {
            if (cognitiveService.GetCognitiveEngineName() === 'LUIS') {
                if (!result)
                    throw 'SERVER_ERROR_NLU_TOKEN_GENERATOR_FAILED';
                if (nlu.subscriptionKey !== param.subscriptionKey) {
                    authObject = result;
                    return mssql_service_1.MSSql.GetAzureSubscription(vcc);
                }
                else {
                    return true;
                }
            }
            else {
                return true;
            }
        })
            .then(function (result) {
            if (cognitiveService.GetCognitiveEngineName() === 'LUIS') {
                if (!result)
                    throw 'SERVER_ERROR_NLU_AZURE_SUBSCRIPTION_NOT_FOUND';
                if (nlu.subscriptionKey !== param.subscriptionKey) {
                    azureSubscription = result;
                    var unAssignData = {
                        azureSubscriptionId: azureSubscription.azureSubscription,
                        resourceGroup: azureSubscription.resourceGroup,
                        accountName: azureSubscription.accountName,
                        token_type: authObject.token_type,
                        access_token: authObject.access_token
                    };
                    return cognitiveService.UnAssignSubscriptionKeyOfVCC(azureSubscriptionKey.subscriptionRegion, azureSubscriptionDemo.subscriptionKey, param.appId, unAssignData);
                }
                else {
                    return true;
                }
            }
            else {
                return true;
            }
        })
            .then(function (result) {
            if (cognitiveService.GetCognitiveEngineName() === 'LUIS') {
                if (!result)
                    throw 'SERVER_ERROR_NLU_KEY_UNASSIGNING_FAILED';
                if (nlu.subscriptionKey !== param.subscriptionKey) {
                    var assignData = {
                        azureSubscriptionId: azureSubscription.azureSubscription,
                        resourceGroup: azureSubscription.resourceGroup,
                        accountName: azureSubscription.accountName,
                        token_type: authObject.token_type,
                        access_token: authObject.access_token
                    };
                    return cognitiveService.AssignSubscriptionKeyOfVCC(azureSubscriptionKey.subscriptionRegion, azureSubscriptionDemo.subscriptionKey, param.appId, assignData);
                }
                else {
                    return true;
                }
            }
            else {
                return true;
            }
        })
            .then(function (result) {
            if (!result)
                throw 'SERVER_ERROR_NLU_KEY_ASSIGNING_FAILED';
            (0, nlu_mssql_1.UpdateNluStatus)(nlu.vcc, nlu.appId, nlu_status_const_1.NluStatus.publishing).then(function (_) { });
            TriggerUpdateNLU(nlu.vcc);
            return cognitiveService.PublishApp(azureSubscriptionDemo.subscriptionRegion, azureSubscriptionDemo.subscriptionKey, param.appId, !azureSubscriptionDemo.isDemo, nlu.trainVersion);
        })
            .then(function (result) {
            if (result) {
                nlu.isPublished = true;
                nlu.needPublish = false;
                nlu.subscriptionKey = azureSubscriptionKey.subscriptionKey;
                nlu.lastPublishedDate = moment().utc().toDate();
                azureResult = result;
                (0, nlu_mssql_1.UpdateNluStatus)(nlu.vcc, nlu.appId, nlu_status_const_1.NluStatus.available).then(function (_) { });
                TriggerUpdateNLU(nlu.vcc);
                return (0, nlu_service_1.SaveNLU)(vcc, nlu);
            }
            else {
                return Promise.resolve(result);
            }
        }).then(function (result) {
            if (res.headersSent)
                return;
            res.json({ status: true, description: 'SERVER_SUCCESS_MESSAGE', data: true });
        }).catch(function (err) {
            var clientMsg = (typeof err === 'string' ? err : 'CANNOT_PROCESS_REQUEST');
            var errorMsg = (typeof err === 'string' ? err : err.message || err.description || 'SERVER_ERROR_UNKNOWN_MESSAGE');
            logger.error('[TrainService::publish_app] Error on \'' + req.path + '\': ' + errorMsg);
            if (res.headersSent)
                return;
            res.json({ status: false, description: clientMsg, error: errorMsg });
        });
    });
}
exports.TrainService = TrainService;
function TriggerUpdateNLU(vcc) {
    var events = [];
    events.push(new io_model_1.IOEvent(io_model_1.VCC_EVENT, events_1.UPDATE_NLU_STATUS, null, vcc));
    (0, io_request_1.SendIOEvents)(events).then(function (_) { });
}
function TrainApp(cognitiveService, azure, nlu) {
    var appIdRasaApi = '';
    var rasaApiTrainId = '';
    var rasaApiTrainVersion = '';
    return (0, promises_shared_1.DummyPromise)()
        .then(function (_) {
        if (cognitiveService.GetCognitiveEngineName() === 'RASA') {
            if (nlu.isSystemApp)
                return (0, utterance_service_1.CreateSystemAppIfNotExists)(cognitiveService, nlu.vcc, nlu);
            else
                return Promise.resolve(true);
        }
        else if (cognitiveService.GetCognitiveEngineName() === 'RASAAPI') {
            if (nlu.isSystemApp)
                return (0, utterance_service_2.CreateRasaApiSystemAppIfNotExists)(cognitiveService, nlu.vcc, nlu);
            else
                return Promise.resolve(nlu.appId);
        }
        else
            return Promise.resolve(true);
    })
        .then(function (result) {
        if (!nlu.isSystemApp && !nlu.trainMode) {
            (0, nlu_mssql_1.UpdateNluStatus)(nlu.vcc, nlu.appId, nlu_status_const_1.NluStatus.training).then(function (_) { });
            TriggerUpdateNLU(nlu.vcc);
        }
        if (cognitiveService.GetCognitiveEngineName() === 'RASAAPI') {
            appIdRasaApi = result;
            if (nlu.trainId && nlu.trainVersion)
                return Promise.resolve({ id: nlu.trainId, versionId: nlu.trainVersion });
            return cognitiveService.TrainApp(azure ? azure.subscriptionRegion : null, azure ? azure.subscriptionKey : null, appIdRasaApi, nlu.vcc);
        }
        else {
            return cognitiveService.TrainApp(azure ? azure.subscriptionRegion : null, azure ? azure.subscriptionKey : null, nlu.appId);
        }
    })
        .then(function (result) {
        if (result) {
            rasaApiTrainId = result.id;
            rasaApiTrainVersion = result.versionId;
            if (nlu.cognitiveEngine === 'LUIS' || (nlu.cognitiveEngine === 'RASAAPI' && !nlu.isSystemApp && !nlu.trainMode)) {
                setTimeout(function () { return GetTrainingStatus(cognitiveService, azure, nlu, 0, rasaApiTrainId); }, 5000);
            }
            if (cognitiveService.GetCognitiveEngineName() === 'RASA') {
                ProcessSuccessfulTrainingResult(cognitiveService, nlu);
            }
        }
        return Promise.resolve(true);
    }).then(function (result) {
        if (cognitiveService.GetCognitiveEngineName() === 'RASAAPI' && nlu.isSystemApp || (nlu.trainMode === 'A' && !nlu.isSystemApp)) {
            return cognitiveService.GetTrainApp(null, null, nlu.appId, rasaApiTrainId);
        }
        else {
            return Promise.resolve(false);
        }
    })
        .then(function (result) {
        if (result) {
            nlu.trainId = result.id;
            nlu.trainVersion = result.versionId;
            if (result.status === 'trained') {
                ProcessSuccessfulTrainingResult(cognitiveService, nlu);
            }
            else {
                (0, nlu_service_1.SaveNLU)(nlu.vcc, nlu).then(function (_) { });
            }
        }
        return Promise.resolve(true);
    })
        .catch(function (err) {
        logger.error('[TrainService::TrainApp] Getting nlu: ', nlu.appId, ' error:', err);
        if (!nlu.isSystemApp) {
            (0, nlu_mssql_1.UpdateNluStatus)(nlu.vcc, nlu.appId, nlu_status_const_1.NluStatus.available).then(function (_) { });
            TriggerUpdateNLU(nlu.vcc);
        }
        return Promise.reject(err);
    });
}
exports.TrainApp = TrainApp;
function PublishRasaApiApp(cognitiveService, nlu) {
    return (0, promises_shared_1.DummyPromise)()
        .then(function (_) {
        return cognitiveService.GetTrainApp(null, null, nlu.appId, nlu.trainId);
    })
        .then(function (result) {
        if (result && result.status === 'trained') {
            return cognitiveService.PublishApp(null, null, nlu.appId, null, result.versionId);
        }
        else
            return Promise.resolve(result);
    })
        .then(function (result) {
        if (result.active) {
            nlu.isPublished = true;
            nlu.needPublish = false;
            nlu.lastPublishedDate = moment().utc().toDate();
            (0, nlu_mssql_1.UpdateNluStatus)(nlu.vcc, nlu.appId, nlu_status_const_1.NluStatus.available).then(function (_) { });
            TriggerUpdateNLU(nlu.vcc);
            return (0, nlu_service_1.SaveNLU)(nlu.vcc, nlu);
        }
        else
            return Promise.resolve(result);
    })
        .then(function (result) {
        return Promise.resolve(result);
    })
        .catch(function (err) {
        logger.error('[TrainService::PublishRasaApiApp] Getting nlu: ', nlu.appId, ' error:', err);
        if (!nlu.isSystemApp) {
            (0, nlu_mssql_1.UpdateNluStatus)(nlu.vcc, nlu.appId, nlu_status_const_1.NluStatus.available).then(function (_) { });
            TriggerUpdateNLU(nlu.vcc);
        }
        return Promise.reject(err);
    });
}
exports.PublishRasaApiApp = PublishRasaApiApp;
function ProcessSuccessfulTrainingResult(cognitiveService, nlu) {
    nlu.isTrained = true;
    nlu.needTrain = false;
    nlu.needPublish = true;
    nlu.isPublished = false;
    nlu.lastTrainedDate = moment().utc().toDate();
    nlu.createdDate = typeof nlu.createdDate === 'string' ? moment(nlu.createdDate).toDate() : nlu.createdDate;
    nlu.lastModifiedByUserId = ['dev-1', 'dev-2'].includes(nlu.lastModifiedByUserId) ? 'system' : nlu.lastModifiedByUserId;
    nlu.lastModifiedDate = moment().utc().toDate();
    if (cognitiveService.GetCognitiveEngineName() === 'RASA') {
        nlu.isPublished = true;
        nlu.needPublish = false;
        nlu.lastPublishedDate = nlu.lastTrainedDate;
    }
    else {
        nlu.needPublish = true;
    }
    (0, nlu_service_1.SaveNLU)(nlu.vcc, nlu).then(function (_) { });
    (0, nlu_mssql_1.UpdateNluStatus)(nlu.vcc, nlu.appId, nlu_status_const_1.NluStatus.available).then(function (_) { });
    TriggerUpdateNLU(nlu.vcc);
}
function GetTrainingStatus(cognitiveService, azure, nlu, retryCount, rasaApiTrainId) {
    if (retryCount === void 0) { retryCount = 0; }
    cognitiveService.GetTrainApp(azure ? azure.subscriptionRegion : null, azure ? azure.subscriptionKey : null, nlu.appId, rasaApiTrainId)
        .then(function (result) {
        if (cognitiveService.GetCognitiveEngineName() === 'LUIS') {
            if (result && result.every(function (model) { return ['UpToDate', 'Success'].includes(model.details.status); })) {
                ProcessSuccessfulTrainingResult(cognitiveService, nlu);
            }
            else {
                if (retryCount < 10) {
                    setTimeout(function () { return GetTrainingStatus(cognitiveService, azure, nlu, retryCount + 1, rasaApiTrainId); }, 5000);
                }
                else {
                    (0, nlu_mssql_1.UpdateNluStatus)(nlu.vcc, nlu.appId, nlu_status_const_1.NluStatus.trainingFailed).then(function (_) { });
                    TriggerUpdateNLU(nlu.vcc);
                }
            }
        }
        else if (cognitiveService.GetCognitiveEngineName() === 'RASAAPI') {
            if (result && result.status === 'trained') {
                nlu.trainId = result.id;
                nlu.trainVersion = result.versionId;
                ProcessSuccessfulTrainingResult(cognitiveService, nlu);
            }
            else {
                if (retryCount < 10) {
                    setTimeout(function () { return GetTrainingStatus(cognitiveService, azure, nlu, retryCount + 1, rasaApiTrainId); }, 15000);
                }
                else {
                    nlu.trainId = rasaApiTrainId;
                    (0, nlu_service_1.SaveNLU)(nlu.vcc, nlu).then(function (_) { });
                    (0, nlu_mssql_1.UpdateNluStatus)(nlu.vcc, nlu.appId, nlu_status_const_1.NluStatus.trainingFailed).then(function (_) { });
                    TriggerUpdateNLU(nlu.vcc);
                }
            }
        }
    });
}
function GetVersionTrainingStatus(intents, trainData) {
    var intent;
    function find(x) {
        return x.modelId === intent.intentId;
    }
    var result = [];
    var i, lenIntents = intents.length, found;
    for (i = 0; i < lenIntents; i++) {
        intent = intents[i];
        found = trainData.find(find);
        if (found) {
            intent.trainData = found;
        }
    }
    result = intents;
    return result;
}
