"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.GetSaveUtterances = exports.GetUtterances = exports.GetIntent = exports.CreateRasaApiSystemAppIfNotExists = exports.CreateSystemAppIfNotExists = exports.UtteranceService = void 0;
var moment = require("moment");
var log4js = require("log4js");
var mssql_service_1 = require("../db/mssql.service");
var promises_shared_1 = require("../shared/promises.shared");
var cognitive_analysis_implementation_1 = require("../implementation/cognitive-analysis.implementation");
var nlu_service_1 = require("./nlu.service");
var config_1 = require("../config/config");
var logger = log4js.getLogger('ServerScripts');
var useRasaApi = config_1.USE_RASA_API;
function UtteranceService(app) {
    var cognitiveService;
    app.get('/api/utterance/get_utterances/:app_id/:search_criteria/:search_expression?', function (req, res) {
        var vcc = res.locals.vcc;
        var azure = res.locals.azure;
        var search_criteria = req.params.search_criteria || null;
        var search_expression = req.params.search_expression || null;
        var appId = req.params.app_id || '';
        var intent = req.query.intent ? req.query.intent.toString() : null;
        var implUtterances = [];
        var dbUtterances = [];
        var filters = {
            name: 'U.[Text]'
        };
        var hasSearch = false;
        if (search_criteria && search_expression) {
            search_criteria = filters[search_criteria];
            hasSearch = true;
        }
        var nluName = '';
        (0, promises_shared_1.DummyPromise)()
            .then(function (result) {
            if (!appId)
                throw 'MISSING_NLU_INPUT_PARAMETER';
            return mssql_service_1.MSSql.GetNLUByAppId(vcc, appId);
        })
            .then(function (result) {
            if (!result)
                throw 'SERVER_ERROR_NLU_NOT_FOUND';
            nluName = result.name;
            vcc = result.vcc;
            if (result.isSystemApp) {
                return (0, cognitive_analysis_implementation_1.GetCognitiveService)(null, 'RASA');
            }
            else {
                return (0, cognitive_analysis_implementation_1.GetCognitiveService)(vcc, null);
            }
        })
            .then(function (cognitiveService) {
            return cognitiveService.GetUtterancesList(azure.subscriptionRegion, azure.subscriptionKey, appId, vcc);
        })
            .then(function (utterances) {
            implUtterances = utterances;
            return GetUtterances(vcc, appId, intent, search_criteria, search_expression, false);
        })
            .then(function (utterances) {
            dbUtterances = utterances;
            var promiseArray = [];
            if (hasSearch) {
                var filteredUtterances = [];
                for (var _i = 0, dbUtterances_1 = dbUtterances; _i < dbUtterances_1.length; _i++) {
                    var du = dbUtterances_1[_i];
                    for (var _a = 0, implUtterances_1 = implUtterances; _a < implUtterances_1.length; _a++) {
                        var iu = implUtterances_1[_a];
                        if ([du.id].includes("".concat(iu.id))) {
                            filteredUtterances.push(iu);
                        }
                    }
                }
                implUtterances = filteredUtterances;
                return Promise.all(promiseArray);
            }
            implUtterances.map(function (iu) {
                if (!dbUtterances.find(function (du) { return du.id === iu.id; }) && iu.intentLabel !== 'None') {
                    promiseArray.push(mssql_service_1.MSSql.SaveUtterance(vcc, appId, null, iu.intentLabel, iu.id, iu.text, 'system'));
                }
            });
            dbUtterances.map(function (du) {
                if (!implUtterances.find(function (iu) { return iu.id === du.id; })) {
                    promiseArray.push(mssql_service_1.MSSql.DeleteUtterance(vcc, appId, du.intentId, du.Id));
                }
            });
            return Promise.all(promiseArray);
        })
            .then(function (result) {
            if (res.headersSent)
                return;
            res.json({ status: true, description: 'SERVER_SUCCESS_MESSAGE', data: implUtterances });
        })
            .catch(function (err) {
            var clientMsg = (typeof err === 'string' ? err : 'CANNOT_PROCESS_REQUEST');
            var errorMsg = (typeof err === 'string' ? err : err.message || err.description || 'SERVER_ERROR_UNKNOWN_MESSAGE');
            logger.error('Error al procesar \'' + req.path + '\': ' + errorMsg);
            if (res.headersSent)
                return;
            res.json({ status: false, description: clientMsg, error: errorMsg });
        });
    });
    app.get('/api/utterance/get_utterance/:utterance_id', function (req, res, next) {
        logger.info('Se recibió ' + req.method + ' para \'' + req.path + '\'');
        var vcc = res.locals.vcc;
        var azure = res.locals.azure;
        var utteranceId = req.params.utterance_id;
        var utterance;
        (0, promises_shared_1.DummyPromise)()
            .then(function (result) {
            if (!utteranceId)
                throw 'UTTERANCE_SERVER_ERROR_MISSING_ID';
            return Promise.resolve(true);
        }).then(function (resultUtterance) {
            utterance = resultUtterance;
            return Promise.resolve(true);
        }).then(function (result) {
            return utterance;
        }).then(function (utterance) {
            if (res.headersSent)
                return;
            res.json({ status: true, description: 'SERVER_SUCCESS_MESSAGE', data: utterance });
        }).catch(function (err) {
            var clientMsg = (typeof err === 'string' ? err : 'CANNOT_PROCESS_REQUEST');
            var errorMsg = (typeof err === 'string' ? err : err.message || err.description || 'SERVER_ERROR_UNKNOWN_MESSAGE');
            logger.error('Error al procesar \'' + req.path + '\': ' + errorMsg);
            if (res.headersSent)
                return;
            res.json({ status: false, description: clientMsg, error: errorMsg });
        });
    });
    app.get('/api/utterance/get_utterances_by_interaction/:audio_id/:section/:part', function (req, res, next) {
        var _a, _b;
        logger.info('Se recibió ' + req.method + ' para \'' + req.path + '\'');
        var vcc = res.locals.vcc;
        var azure = res.locals.azure;
        var audioId = req.params.audio_id ? req.params.audio_id.toString() : '';
        var section = ((_a = req.params.section) === null || _a === void 0 ? void 0 : _a.length) ? Number(req.params.section) : null;
        var part = ((_b = req.params.part) === null || _b === void 0 ? void 0 : _b.length) ? Number(req.params.part) : null;
        logger.info('Request parameters: ', { audioId: audioId, section: section, part: part });
        (0, promises_shared_1.DummyPromise)()
            .then(function (_) {
            if (!audioId)
                throw 'UTTERANCE_SERVER_ERROR_MISSING_AUDIO_ID';
            if (section === null)
                throw 'UTTERANCE_SERVER_ERROR_MISSING_AUDIO_SECTION';
            if (part === null)
                throw 'UTTERANCE_SERVER_ERROR_MISSING_AUDIO_PART';
            return mssql_service_1.MSSql.GetUtterancesByInteraction(vcc, audioId, section, part);
        }).then(function (resultUtterances) {
            logger.info('resultUtterances: ', resultUtterances);
            return Promise.resolve(resultUtterances);
        }).then(function (resultUtterances) {
            if (res.headersSent)
                return;
            res.json({ status: true, description: 'SERVER_SUCCESS_MESSAGE', data: resultUtterances });
        }).catch(function (err) {
            var clientMsg = (typeof err === 'string' ? err : 'CANNOT_PROCESS_REQUEST');
            var errorMsg = (typeof err === 'string' ? err : err.message || err.description || 'SERVER_ERROR_UNKNOWN_MESSAGE');
            logger.error('Error al procesar \'' + req.path + '\': ' + errorMsg);
            if (res.headersSent)
                return;
            res.json({ status: false, description: clientMsg, error: errorMsg });
        });
    });
    app.post('/api/utterance/create_utterance', function (req, res, next) {
        logger.info('[UtteranceService::create_utterance] Received ' + req.method + ' method on \'' + req.path + '\'');
        var vcc = res.locals.vcc;
        var azure = res.locals.azure;
        var userId = res.locals.userId;
        var nowUtc = moment().utc().toDate();
        var nlu = null;
        var vcc_new = '';
        var newUtterance = (req.body || {});
        newUtterance.name = newUtterance.name.trim();
        newUtterance.name = newUtterance.name.replace(/\s+/g, ' ');
        var createUtteranceResult = null;
        (0, promises_shared_1.DummyPromise)()
            .then(function (_) {
            if (!newUtterance.appId)
                throw 'MISSING_NLU_INPUT_PARAMETER';
            return mssql_service_1.MSSql.GetNLUByAppId(vcc, newUtterance.appId);
        })
            .then(function (result) {
            if (!result)
                throw 'SERVER_ERROR_NLU_NOT_FOUND';
            nlu = result;
            if (result.vcc === 'system')
                nlu.appId = vcc + '::' + nlu.name;
            return mssql_service_1.MSSql.GetUtterance(vcc, newUtterance.appId, newUtterance.intent, newUtterance.name);
        })
            .then(function (result) {
            if (result)
                throw 'SERVER_ERROR_UTTERANCE_ALREADY_EXISTS';
            if (nlu.isSystemApp) {
                return (0, cognitive_analysis_implementation_1.GetCognitiveService)(null, 'RASA');
            }
            else {
                return (0, cognitive_analysis_implementation_1.GetCognitiveService)(vcc, null);
            }
        })
            .then(function (result) {
            cognitiveService = result;
            if (nlu.isSystemApp) {
                if (nlu.cognitiveEngine === 'RASA')
                    return CreateSystemAppIfNotExists(cognitiveService, vcc, nlu);
                return CreateRasaApiSystemAppIfNotExists(cognitiveService, vcc, nlu);
            }
            else {
                return Promise.resolve(true);
            }
        })
            .then(function (result) {
            if (typeof result === "string") {
                newUtterance.appId = result;
                nlu.appId = result;
            }
            nlu.needTrain = true;
            nlu.needPublish = false;
            nlu.isPublished = false;
            nlu.isTrained = false;
            nlu.trainId = null;
            nlu.trainVersion = null;
            nlu.lastModifiedDate = nowUtc;
            nlu.lastModifiedByUserId = userId;
            return (0, nlu_service_1.SaveNLU)(vcc, nlu);
        })
            .then(function (result) {
            return cognitiveService.AddUtterance(azure.subscriptionRegion, azure.subscriptionKey, newUtterance.appId, newUtterance, vcc);
        })
            .then(function (resultSaveUtterance) {
            if (!resultSaveUtterance)
                throw 'UTTERANCE_SERVER_ERROR_ON_SAVE';
            createUtteranceResult = resultSaveUtterance;
            if (nlu.cognitiveEngine === 'RASAAPI' && nlu.isSystemApp)
                return mssql_service_1.MSSql.SaveUtterance(vcc, newUtterance.appId, newUtterance.intent, null, resultSaveUtterance.ExampleId, resultSaveUtterance.UtteranceText, userId);
            return mssql_service_1.MSSql.SaveUtterance(vcc, newUtterance.appId, null, newUtterance.intent, resultSaveUtterance.ExampleId, resultSaveUtterance.UtteranceText, userId);
        })
            .then(function (_) {
            if (res.headersSent)
                return;
            res.json({ status: true, description: (createUtteranceResult ? 'SERVER_SUCCESS_MESSAGE' : 'UTTERANCE_SERVER_ERROR_ON_SAVE'), data: createUtteranceResult });
        })
            .catch(function (err) {
            var clientMsg = (typeof err === 'string' ? err : 'CANNOT_PROCESS_REQUEST');
            var errorMsg = (typeof err === 'string' ? err : err.message || err.description || 'SERVER_ERROR_UNKNOWN_MESSAGE');
            logger.error('[UtteranceService::create_utterance] Error on \'' + req.path + '\': ' + errorMsg);
            if (res.headersSent)
                return;
            res.json({ status: false, description: clientMsg, error: errorMsg });
        });
    });
    app.post('/api/utterance/create_utterance_underlined', function (req, res, next) {
        logger.info('[UtteranceService::create_utterance_underlined] Received ' + req.method + ' method on \'' + req.path + '\'');
        var vcc = res.locals.vcc;
        var azure = res.locals.azure;
        var userId = res.locals.userId;
        var nowUtc = moment().utc().toDate();
        var newUtteranceData = (req.body || {});
        newUtteranceData.createdDate = nowUtc;
        newUtteranceData.createdByUserId = userId;
        newUtteranceData.lastModifiedDate = nowUtc;
        newUtteranceData.lastModifiedByUserId = userId;
        logger.info('newUtteranceDTO: ', newUtteranceData);
        (0, promises_shared_1.DummyPromise)()
            .then(function (resultSaveUnderlined) {
            if (!resultSaveUnderlined)
                throw 'UTTERANCE_SERVER_ERROR_ON_SAVE';
            logger.info('[UtteranceService::create_utterance_underlined] resultSaveUnderlined: ', resultSaveUnderlined);
            return mssql_service_1.MSSql.SaveUnderlined(vcc, newUtteranceData.appId, newUtteranceData.intentId, newUtteranceData.audioId, newUtteranceData.section, newUtteranceData.part, newUtteranceData.speaker, newUtteranceData.anchorOffset, newUtteranceData.focusOffset, newUtteranceData.currentTime, newUtteranceData.endCurrentTime, newUtteranceData.utteranceId, newUtteranceData.textContent, newUtteranceData.createdDate, newUtteranceData.createdByUserId, newUtteranceData.lastModifiedDate, newUtteranceData.lastModifiedByUserId);
        }).then(function (resultSaveUtteranceUnderlined) {
            logger.info('[UtteranceService::create_utterance_underlined] resultSaveUtteranceUnderlined: ', resultSaveUtteranceUnderlined);
            if (res.headersSent)
                return;
            res.json({ status: true, description: (resultSaveUtteranceUnderlined ? 'SERVER_SUCCESS_MESSAGE' : 'UTTERANCE_SERVER_ERROR_ON_SAVE'), data: resultSaveUtteranceUnderlined });
        }).catch(function (err) {
            var clientMsg = (typeof err === 'string' ? err : 'CANNOT_PROCESS_REQUEST');
            var errorMsg = (typeof err === 'string' ? err : err.message || err.description || 'SERVER_ERROR_UNKNOWN_MESSAGE');
            logger.error('[UtteranceService::create_utterance_underlined] Error al procesar \'' + req.path + '\': ' + errorMsg);
            if (res.headersSent)
                return;
            res.json({ status: false, description: clientMsg, error: errorMsg });
        });
    });
    app.post('/api/utterance/save_utterance', function (req, res, next) {
        var vcc = res.locals.vcc;
        var azure = res.locals.azure;
        var userId = res.locals.userId;
        var nowUtc = moment().utc().toDate();
        var nlu = null;
        var utterance = (req.body || {});
        utterance.name = utterance.name.trim();
        utterance.name = utterance.name.replace(/\s+/g, ' ');
        var createUtteranceResult = null;
        (0, promises_shared_1.DummyPromise)()
            .then(function (_) {
            if (!utterance.appId)
                throw 'MISSING_NLU_INPUT_PARAMETER';
            return mssql_service_1.MSSql.GetNLUByAppId(vcc, utterance.appId);
        })
            .then(function (result) {
            if (!result)
                throw 'SERVER_ERROR_NLU_NOT_FOUND';
            nlu = result;
            return mssql_service_1.MSSql.GetUtterance(vcc, utterance.appId, utterance.intent, utterance.name);
        })
            .then(function (result) {
            if (!result)
                throw 'SERVER_ERROR_UTTERANCE_DOES_NOT_EXISTS';
            utterance.id = result.id;
            if (nlu.isSystemApp) {
                return (0, cognitive_analysis_implementation_1.GetCognitiveService)(null, 'RASA');
            }
            else {
                return (0, cognitive_analysis_implementation_1.GetCognitiveService)(vcc, null);
            }
        })
            .then(function (result) {
            cognitiveService = result;
            if (!cognitiveService.GetCognitiveEngineName().includes('RASA'))
                return cognitiveService.AddUtterance(azure.subscriptionRegion, azure.subscriptionKey, utterance.appId, utterance);
            return true;
        })
            .then(function (resultSaveUtterance) {
            if (!resultSaveUtterance)
                throw 'UTTERANCE_SERVER_ERROR_ON_SAVE';
            createUtteranceResult = resultSaveUtterance;
            nlu.needTrain = true;
            nlu.needPublish = false;
            nlu.isPublished = false;
            nlu.isTrained = false;
            nlu.trainId = null;
            nlu.trainVersion = null;
            nlu.lastModifiedDate = nowUtc;
            nlu.lastModifiedByUserId = userId;
            return (0, nlu_service_1.SaveNLU)(vcc, nlu);
        })
            .then(function (result) {
            if (utterance.entityLabels && utterance.entityLabels.length > 0) {
                if (utterance) {
                    var promises = [];
                    for (var _i = 0, _a = utterance.entityLabels; _i < _a.length; _i++) {
                        var e = _a[_i];
                        promises.push(mssql_service_1.MSSql.SaveUtteranceEntity(vcc, utterance.id, e.entityName, e.start, e.end, e.startTokenIndex, e.endTokenIndex));
                    }
                    return Promise.all(promises);
                }
            }
            return [result];
        })
            .then(function (_) {
            if (res.headersSent)
                return;
            res.json({ status: true, description: (createUtteranceResult ? 'SERVER_SUCCESS_MESSAGE' : 'UTTERANCE_SERVER_ERROR_ON_SAVE'), data: createUtteranceResult });
        })
            .catch(function (err) {
            var clientMsg = (typeof err === 'string' ? err : 'CANNOT_PROCESS_REQUEST');
            var errorMsg = (typeof err === 'string' ? err : err.message || err.description || 'SERVER_ERROR_UNKNOWN_MESSAGE');
            logger.error('[UtteranceService::save_utterance] Error on \'' + req.path + '\': ' + errorMsg);
            if (res.headersSent)
                return;
            res.json({ status: false, description: clientMsg, error: errorMsg });
        });
    });
    app.post('/api/utterance/delete_utterances', function (req, res) {
        var vcc = res.locals.vcc;
        var azure = res.locals.azure;
        var utterances = req.body.utterances || [];
        var mode = Number(req.query.mode) || 0;
        var deleteByIntent = mode == 0;
        var ids = [];
        logger.info("[UtteranceService::delete_utterances] Deleting utterances ".concat(utterances.map(function (x) { return x.id; }).join(','), " by the user ").concat(res.locals.userId));
        (0, promises_shared_1.DummyPromise)().then(function () {
            if (utterances.length == 0) {
                throw new Error("EMPTY_UTTERANCES");
            }
            return (0, cognitive_analysis_implementation_1.GetCognitiveService)(vcc, null);
        }).then(function (cognitiveService) {
            var promises = [];
            var isLuis = ['LUIS'].includes(cognitiveService.GetCognitiveEngineName());
            for (var _i = 0, utterances_1 = utterances; _i < utterances_1.length; _i++) {
                var u = utterances_1[_i];
                if (deleteByIntent) {
                    ids.push(u.intentId);
                }
                else {
                    ids.push(u.utteranceId);
                }
                if (isLuis) {
                    promises.push(cognitiveService.DeleteAppUtterance(azure.subscriptionRegion, azure.subscriptionKey, u.appId, u.intentId, u.utteranceId));
                }
            }
            if (!isLuis) {
                promises.push(Promise.resolve({
                    code: 'Success'
                }));
            }
            return Promise.all(promises);
        })
            .then(function (resultDeleteUtterance) {
            var success = resultDeleteUtterance.filter(function (r) { return ["Success"].includes(r.code); });
            if (success.length == 0)
                throw 'UTTERANCES_SERVER_ERROR_CANNOT_DELETE';
            return mssql_service_1.MSSql.DeleteMultipleUtterances(vcc, ids.toString(), deleteByIntent);
        }).then(function (result) {
            res.json({ status: result, description: (result ? 'SERVER_SUCCESS_MESSAGE' : 'UTTERANCE_SERVER_ERROR_CANNOT_DELETE'),
                data: result });
        }).catch(function (err) {
            var clientMsg = (typeof err === 'string' ? err : 'CANNOT_PROCESS_REQUEST');
            var errorMsg = (typeof err === 'string' ? err : err.message || err.description || 'SERVER_ERROR_UNKNOWN_MESSAGE');
            logger.error('[UtteranceService::delete_utterances] Error on \'' + req.path + '\': ' + errorMsg);
            if (res.headersSent)
                return;
            res.json({ status: false, description: clientMsg, error: errorMsg });
        });
    });
    app.post('/api/utterance/delete_utterance', function (req, res) {
        logger.info('[UtteranceService::delete_utterance] Received ' + req.method + ' method on \'' + req.path + '\'');
        var vcc = res.locals.vcc;
        var azure = res.locals.azure;
        var userId = res.locals.userId;
        var nowUtc = moment().utc().toDate();
        var utterance = req.body.utterance;
        var utteranceId = utterance.id;
        var intentId = utterance.intentLabel;
        var appId = utterance.appId;
        var rasaapi = false;
        var nlu = null;
        var deleteUtteranceResult = null;
        logger.info("[UtteranceService::delete_utterance] Deleting utterance ".concat(utteranceId, " by the user ").concat(res.locals.userId));
        (0, promises_shared_1.DummyPromise)()
            .then(function (_) {
            if (!appId)
                throw 'MISSING_NLU_INPUT_PARAMETER';
            return mssql_service_1.MSSql.GetNLUByAppId(vcc, appId);
        })
            .then(function (result) {
            if (!result)
                throw 'SERVER_ERROR_NLU_NOT_FOUND';
            nlu = result;
            if (nlu.isSystemApp) {
                return (0, cognitive_analysis_implementation_1.GetCognitiveService)(null, 'RASA');
            }
            else {
                return (0, cognitive_analysis_implementation_1.GetCognitiveService)(vcc, null);
            }
        })
            .then(function (cognitiveService) {
            var id = utteranceId;
            if (cognitiveService.GetCognitiveEngineName() === 'RASA' || cognitiveService.GetCognitiveEngineName() === 'RASAAPI') {
                id = utterance.text;
            }
            if (!utterance)
                throw 'UTTERANCE_SERVER_ERROR_MISSING_ID';
            return cognitiveService.DeleteAppUtterance(azure.subscriptionRegion, azure.subscriptionKey, appId, intentId, id, vcc);
        }).then(function (resultDeleteUtterance) {
            if (!resultDeleteUtterance)
                throw 'UTTERANCE_SERVER_ERROR_CANNOT_DELETE';
            var promises = [mssql_service_1.MSSql.DeleteUtterance(vcc, appId, intentId, utteranceId)];
            deleteUtteranceResult = resultDeleteUtterance;
            nlu.needTrain = true;
            nlu.needPublish = false;
            nlu.isPublished = false;
            nlu.isTrained = false;
            nlu.trainId = null;
            nlu.trainVersion = null;
            nlu.lastPublishedDate = null;
            nlu.lastTrainedDate = null;
            nlu.lastModifiedDate = nowUtc;
            nlu.lastModifiedByUserId = userId;
            promises.push((0, nlu_service_1.SaveNLU)(vcc, nlu));
            return Promise.all(promises);
        }).then(function (resultDeleteUtterance) {
            if (res.headersSent)
                return;
            var status = !resultDeleteUtterance.includes(false);
            res.json({ status: status, description: (status ? 'SERVER_SUCCESS_MESSAGE' : 'UTTERANCE_SERVER_ERROR_CANNOT_DELETE'), data: resultDeleteUtterance });
        }).catch(function (err) {
            var clientMsg = (typeof err === 'string' ? err : 'CANNOT_PROCESS_REQUEST');
            var errorMsg = (typeof err === 'string' ? err : err.message || err.description || 'SERVER_ERROR_UNKNOWN_MESSAGE');
            logger.error('[UtteranceService::delete_utterance] Error on \'' + req.path + '\': ' + errorMsg);
            if (res.headersSent)
                return;
            res.json({ status: false, description: clientMsg, error: errorMsg });
        });
    });
}
exports.UtteranceService = UtteranceService;
function CreateSystemAppIfNotExists(cognitiveService, vcc, nlu) {
    var isCreated = false;
    return (0, promises_shared_1.DummyPromise)()
        .then(function (result) {
        return cognitiveService.GetAppsList(null, null);
    })
        .then(function (result) {
        if (!result)
            throw 'SERVER_ERROR_GET_INTENTS_FAILED';
        if (!result.find(function (n) { return n.id === nlu.appId; })) {
            return cognitiveService.AddApp(null, null, nlu);
        }
        else {
            isCreated = true;
            return Promise.resolve(true);
        }
    })
        .then(function (result) {
        if (!result)
            throw 'SERVER_ERROR_MIGRATE_DATABASE_RASAAPI_TO_RASA_FAILED';
        if (!isCreated) {
            return cognitiveService.AddIntent(null, null, nlu.appId, 'Agent');
        }
        else {
            return Promise.resolve(true);
        }
    })
        .then(function (result) {
        if (result)
            throw 'SERVER_ERROR_CREATE_SYSTEM_MODEL_INTENT_FAILED';
        return Promise.resolve(true);
    })
        .catch(function (err) {
        var clientMsg = (typeof err === 'string' ? err : 'CANNOT_PROCESS_REQUEST');
        var errorMsg = (typeof err === 'string' ? err : err.message || err.description || 'SERVER_ERROR_UNKNOWN_MESSAGE');
        logger.error('[UtteranceService::CreateDiarizationAppIfNotExists] Error: ' + errorMsg);
        return Promise.resolve(false);
    });
}
exports.CreateSystemAppIfNotExists = CreateSystemAppIfNotExists;
function CreateRasaApiSystemAppIfNotExists(cognitiveService, vcc, nlu) {
    var alreadyCreated = false;
    var newAppId = '';
    return (0, promises_shared_1.DummyPromise)()
        .then(function (result) {
        return cognitiveService.GetAppById(nlu.appId, null, null);
    })
        .then(function (result) {
        if (!result)
            throw 'SERVER_ERROR_GET_APPBYID_RASAAPI_FAILED';
        logger.debug("GetAppIdResultado", result);
        if (result.id === nlu.appId) {
            alreadyCreated = true;
            return true;
        }
        else {
            return cognitiveService.AddApp(null, null, nlu);
        }
    })
        .then(function (result) {
        if (!result)
            throw 'SERVER_ERROR_UPDATENLUMIGRATION_RASAAPI_FAILED';
        newAppId = alreadyCreated ? nlu.appId : result;
        return Promise.resolve(newAppId);
    })
        .catch(function (err) {
        var clientMsg = (typeof err === 'string' ? err : 'CANNOT_PROCESS_REQUEST');
        var errorMsg = (typeof err === 'string' ? err : err.message || err.description || 'SERVER_ERROR_UNKNOWN_MESSAGE');
        logger.error('[UtteranceService::CreateRasaApiSystemAppIfNotExists] Error: ' + errorMsg);
        return Promise.resolve(false);
    });
}
exports.CreateRasaApiSystemAppIfNotExists = CreateRasaApiSystemAppIfNotExists;
function GetIntent(vcc, name) {
    var cacheUpdateNeeded = false;
    return Promise.resolve()
        .then(function () {
        return mssql_service_1.MSSql.GetIntent(vcc, name);
    }).catch(function (err) {
        var errMsg = (typeof err === 'string' ? err : err.message || err.description || 'SERVER_ERROR_UNKNOWN_MESSAGE');
        logger.error('Error al ejecutar GetIntent: ' + errMsg);
    });
}
exports.GetIntent = GetIntent;
function GetUtterances(vcc, appId, intentId, searchCriteria, searchExpression, findUtteranceEntities) {
    var utterances = [];
    return (0, promises_shared_1.DummyPromise)()
        .then(function (_) {
        return mssql_service_1.MSSql.GetUtterances(vcc, appId, intentId, searchCriteria, searchExpression);
    })
        .then(function (result) {
        utterances = result;
        var promiseArray = [];
        if (findUtteranceEntities) {
            result.map(function (r) {
                promiseArray.push(mssql_service_1.MSSql.GetUtteranceEntities(vcc, r.id));
            });
        }
        else {
            promiseArray.push(Promise.resolve([]));
        }
        return Promise.all(promiseArray);
    })
        .then(function (result) {
        var utterancesFiltered = [];
        utterances.filter(function (u) { return u && u.text; }).map(function (u, i) {
            var tokenizedText = u.text.split(/(.+?)([\s,.!?;:)([\]]+)/);
            tokenizedText = tokenizedText.filter(function (tt) { return tt.trim().length > 0; });
            u.entityLabels = result[i] ? result[i] : [];
            u.entityPredictions = [];
            u.intentLabel = u.intentName;
            u.intentPredictions = [
                {
                    name: u.intentName.trim(),
                    score: null
                }
            ],
                u.tokenizedText = tokenizedText;
        });
        utterancesFiltered = utterances.filter(function (u) { return u && u.text; });
        return Promise.resolve(utterancesFiltered);
    })
        .catch(function (err) {
        var errMsg = (typeof err === 'string' ? err : err.message || err.description || 'SERVER_ERROR_UNKNOWN_MESSAGE');
        logger.error('Error on:', errMsg);
        return Promise.resolve(null);
    });
}
exports.GetUtterances = GetUtterances;
function GetSaveUtterances(nlu) {
    var cognitiveService;
    var implUtterances = [];
    var dbUtterances = [];
    return (0, promises_shared_1.DummyPromise)()
        .then(function (result) {
        if (nlu.cognitiveEngine) {
            return (0, cognitive_analysis_implementation_1.GetCognitiveService)(null, nlu.cognitiveEngine);
        }
        else {
            return (0, cognitive_analysis_implementation_1.GetCognitiveService)(nlu.vcc, null);
        }
    })
        .then(function (result) {
        cognitiveService = result;
        if (cognitiveService.GetCognitiveEngineName() === 'LUIS') {
            return mssql_service_1.MSSql.GetAzureSubscriptionKeys(nlu.vcc);
        }
        else {
            return Promise.resolve(null);
        }
    })
        .then(function (result) {
        if (cognitiveService.GetCognitiveEngineName() === 'LUIS') {
            if (result && result.find(function (r) { return r.isDemo; })) {
                return cognitiveService.GetUtterancesList(result.find(function (r) { return r.isDemo; }) ? result.find(function (r) { return r.isDemo; }).subscriptionRegion : null, result.find(function (r) { return r.isDemo; }) ? result.find(function (r) { return r.isDemo; }).subscriptionKey : null, nlu.appId);
            }
            else {
                throw 'SERVER_ERROR_NOT_FOUND_AZURE_CREDENTIALS';
            }
        }
        else {
            return cognitiveService.GetUtterancesFromMD(nlu.appId);
        }
    })
        .then(function (utterances) {
        implUtterances = utterances;
        return GetUtterances(nlu.vcc, nlu.appId, null, null, null, true);
    })
        .then(function (utterances) {
        dbUtterances = utterances;
        var promiseArray = [];
        implUtterances.map(function (iu) {
            if (!dbUtterances.find(function (du) { return du.id === iu.id; }) && iu.intentLabel !== 'None') {
                logger.debug('[UtteranceService::GetSaveUtterances] Saving:', iu.text);
                promiseArray.push(mssql_service_1.MSSql.SaveUtterance(nlu.vcc, nlu.appId, null, iu.intentLabel, iu.id, iu.text, 'system'));
                if (iu.entityLabels) {
                    iu.entityLabels.map(function (el) {
                        if (!el.start) {
                            var utteranceText_1 = '';
                            iu.tokenizedText.map(function (tt, i) {
                                if (i > 0
                                    &&
                                        iu.text.startsWith(utteranceText_1 + ' '))
                                    utteranceText_1 += ' ';
                                if (i === el.startTokenIndex)
                                    el.start = utteranceText_1.length;
                                utteranceText_1 += tt;
                                if (i === el.endTokenIndex)
                                    el.end = utteranceText_1.length - 1;
                            });
                        }
                        promiseArray.push(mssql_service_1.MSSql.SaveUtteranceEntity(nlu.vcc, iu.id, el.entityName, el.start ? el.start : null, el.end ? el.end : null, el.startTokenIndex, el.endTokenIndex));
                    });
                }
            }
        });
        dbUtterances.map(function (du) {
            if (!implUtterances.find(function (iu) { return iu.id === du.id; })) {
                promiseArray.push(mssql_service_1.MSSql.DeleteUtterance(nlu.vcc, nlu.appId, du.intentId, du.Id));
            }
        });
        return Promise.resolve(promiseArray);
    })
        .then(function (result) {
    })
        .then(function (result) {
        nlu.lastModifiedByUserId = 'system';
        nlu.lastModifiedDate = moment().utc().toDate();
        return (0, nlu_service_1.SaveNLU)(nlu.vcc, nlu);
    })
        .then(function (result) {
        return Promise.resolve(true);
    })
        .catch(function (error) {
        logger.error('[UtteranceService::GetSaveUtterances] Error on:', error);
        if (typeof error === 'string' && error === 'SERVER_ERROR_NOT_FOUND_AZURE_CREDENTIALS') {
            nlu.lastModifiedByUserId = 'system';
            nlu.lastModifiedDate = moment().utc().toDate();
            return (0, nlu_service_1.SaveNLU)(nlu.vcc, nlu);
        }
        else {
            if (error.message) {
                if (error.message.includes("Cannot find an application with the ID ".concat(nlu.appId, "."))) {
                    return mssql_service_1.MSSql.DeleteNLU(nlu.vcc, nlu.name);
                }
                else if (error.message.includes("The user is not the owner")) {
                    return mssql_service_1.MSSql.DeleteNLU(nlu.vcc, nlu.name);
                }
                else if (error.message.includes("wrong API")) {
                    return mssql_service_1.MSSql.DeleteNLU(nlu.vcc, nlu.name);
                }
                else {
                    return Promise.resolve(false);
                }
            }
            else {
                return Promise.resolve(false);
            }
        }
    });
}
exports.GetSaveUtterances = GetSaveUtterances;
