"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.SimpleDeleteRequest = exports.SimplePutRequest = exports.SimplePostRequest = exports.SimpleGetRequest = exports.AttachedFilePostRequest = void 0;
var log4js = require("log4js");
var axios_1 = require("axios");
var http = require('http');
var https = require('https');
var logger = log4js.getLogger('ServerScripts');
function SimpleResult(rawResponse) {
    var requestResponse = null;
    if (typeof rawResponse === 'string') {
        try {
            requestResponse = JSON.parse(rawResponse);
        }
        catch (err) {
            requestResponse = null;
        }
    }
    else {
        requestResponse = rawResponse;
    }
    return requestResponse;
}
function AttachedFilePostRequest(data, formData, formDataBackup) {
    var requestConfig = {
        method: 'POST',
        timeout: 300000,
        proxy: false,
        httpAgent: new http.Agent({ keepAlive: true, keepAliveMsecs: 300000 }),
        httpsAgent: new https.Agent({ keepAlive: true, keepAliveMsecs: 300000 }),
        baseURL: data.uri,
        headers: {
            'Content-Type': "multipart/form-data; boundary=".concat(formData._boundary),
            'Connection': 'keep-alive',
            'FileName': data.fileName
        },
        maxContentLength: Infinity,
        maxBodyLength: Infinity,
        data: formData,
        onUploadProgress: function (progressEvent) {
            var percentCompleted = Math.round((progressEvent.loaded * 100) / progressEvent.total);
            var mbRate = 1024 * 1024;
            var loaded = progressEvent.loaded / mbRate;
            var total = progressEvent.total / mbRate;
            var now = new Date().getTime();
            logger.info("[request.shared::AttachedFilePostRequest] File: ".concat(data.fileName, " uploaded: ").concat(loaded.toFixed(2), "MB/").concat(total.toFixed(2), "MB (").concat(percentCompleted, "%) at ").concat(now));
        }
    };
    logger.info("[request.shared::AttachedFilePostRequest] Uploading file: ".concat(data.fileName));
    if (data.timeout)
        requestConfig.timeout = data.timeout;
    return (0, axios_1.default)(requestConfig)
        .catch(function (err) {
        var _a;
        if ((data === null || data === void 0 ? void 0 : data.validateErrorResponse) && ((_a = err === null || err === void 0 ? void 0 : err.response) === null || _a === void 0 ? void 0 : _a.data))
            err = JSON.stringify(err.response.data);
        var errorMessage = typeof err === 'string' ? err : (err.message || err.description);
        logger.error('[request.shared::AttachedFilePostRequest] Getting first try request error:', err);
        logger.error("[request.shared::AttachedFilePostRequest] Getting first try request error: ".concat(errorMessage, " from \"").concat(requestConfig.baseURL, "\""));
        if ((requestConfig.baseURL.includes('http:')
            ||
                requestConfig.baseURL.includes('https:'))
            &&
                data.retryProtocol) {
            if (requestConfig.baseURL.includes('http:'))
                requestConfig.baseURL = requestConfig.baseURL.replace('http:', 'https:');
            else if (requestConfig.baseURL.includes('https:'))
                requestConfig.baseURL = requestConfig.baseURL.replace('https', 'http');
            logger.warn("[request.shared::AttachedFilePostRequest] Failed \"".concat(data.uri, "\" retrying \"").concat(requestConfig.baseURL, "\""));
            return (0, axios_1.default)(requestConfig);
        }
        else if (data.retryUri) {
            logger.warn("[request.shared::AttachedFilePostRequest] Failed \"".concat(data.uri, "\", retrying \"").concat(data.retryUri, "\""));
            requestConfig = {
                method: 'POST',
                timeout: 300000,
                proxy: false,
                httpAgent: new http.Agent({ keepAlive: true, keepAliveMsecs: 300000 }),
                httpsAgent: new https.Agent({ keepAlive: true, keepAliveMsecs: 300000 }),
                baseURL: data.retryUri,
                headers: {
                    'Content-Type': "multipart/form-data; boundary=".concat(formDataBackup._boundary),
                    'Connection': 'keep-alive',
                    'FileName': data.fileName
                },
                maxContentLength: Infinity,
                maxBodyLength: Infinity,
                data: formDataBackup,
                onUploadProgress: function (progressEvent) {
                    var percentCompleted = Math.round((progressEvent.loaded * 100) / progressEvent.total);
                    var mbRate = 1024 * 1024;
                    var loaded = progressEvent.loaded / mbRate;
                    var total = progressEvent.total / mbRate;
                    var now = new Date().getTime();
                    logger.info("[request.shared::AttachedFilePostRequest] File: ".concat(data.fileName, " uploaded: ").concat(loaded.toFixed(2), "MB/").concat(total.toFixed(2), "MB (").concat(percentCompleted, "%) at ").concat(now));
                }
            };
            return (0, axios_1.default)(requestConfig);
        }
        else
            throw err;
    })
        .then(function (result) {
        logger.info("[request.shared::AttachedFilePostRequest] Receiving response for file: ".concat(data.fileName));
        var requestResponse = SimpleResult(result);
        if (requestResponse === null || requestResponse === void 0 ? void 0 : requestResponse.data) {
            return Promise.resolve(requestResponse.data);
        }
        else {
            throw 'SERVER_ERROR_ATTACHED_FILE_POST_REQUEST_FAILED';
        }
    })
        .catch(function (err) {
        var _a;
        if ((data === null || data === void 0 ? void 0 : data.validateErrorResponse) && ((_a = err === null || err === void 0 ? void 0 : err.response) === null || _a === void 0 ? void 0 : _a.data))
            err = JSON.stringify(err.response.data);
        var errorMessage = typeof err === 'string' ? err : (err.message || err.description);
        logger.error('[request.shared::AttachedFileRequest] Getting error:', errorMessage);
        return Promise.reject(err);
    });
}
exports.AttachedFilePostRequest = AttachedFilePostRequest;
function SimpleGetRequest(data) {
    var requestConfig = {
        method: 'GET',
        timeout: 30000,
        proxy: false,
        httpAgent: new http.Agent({ keepAlive: true, keepAliveMsecs: 300000 }),
        httpsAgent: new https.Agent({ keepAlive: true, keepAliveMsecs: 300000 }),
        baseURL: data.uri,
        headers: data.headers
    };
    if (data.responseType)
        requestConfig.responseType = data.responseType;
    if (data.timeout)
        requestConfig.timeout = data.timeout;
    return (0, axios_1.default)(requestConfig)
        .catch(function (err) {
        var _a;
        if ((data === null || data === void 0 ? void 0 : data.validateErrorResponse) && ((_a = err === null || err === void 0 ? void 0 : err.response) === null || _a === void 0 ? void 0 : _a.data))
            err = JSON.stringify(err.response.data);
        var errorMessage = typeof err === 'string' ? err : (err.message || err.description || JSON.stringify(err));
        logger.error('[request.shared::SimpleGetRequest] Getting first try request error:', err);
        logger.error("[request.shared::SimpleGetRequest] Getting first try request error: ".concat(errorMessage, " from \"").concat(requestConfig.baseURL, "\""));
        if ((requestConfig.baseURL.includes('http:')
            ||
                requestConfig.baseURL.includes('https:'))
            &&
                data.retryProtocol) {
            if (requestConfig.baseURL.includes('http:'))
                requestConfig.baseURL = requestConfig.baseURL.replace('http:', 'https:');
            else if (requestConfig.baseURL.includes('https:'))
                requestConfig.baseURL = requestConfig.baseURL.replace('https', 'http');
            logger.warn("[request.shared::SimpleGetRequest] Failed \"".concat(data.uri, "\" retrying \"").concat(requestConfig.baseURL, "\""));
            return (0, axios_1.default)(requestConfig);
        }
        else if (data.retryUri) {
            logger.warn("[request.shared::SimpleGetRequest] Failed \"".concat(data.uri, "\", retrying \"").concat(data.retryUri, "\""));
            requestConfig.baseURL = data.retryUri;
            return (0, axios_1.default)(requestConfig);
        }
        else
            throw err;
    })
        .then(function (result) {
        var requestResponse = SimpleResult(result);
        if (requestResponse && requestResponse.data) {
            return Promise.resolve(requestResponse.data);
        }
        else {
            throw 'SERVER_ERROR_SIMPLE_GET_REQUEST_FAILED';
        }
    })
        .catch(function (err) {
        var _a;
        if ((data === null || data === void 0 ? void 0 : data.validateErrorResponse) && ((_a = err === null || err === void 0 ? void 0 : err.response) === null || _a === void 0 ? void 0 : _a.data))
            err = JSON.stringify(err.response.data);
        var errorMessage = typeof err === 'string' ? err : (err.message || err.description || JSON.stringify(err));
        logger.error('[request.shared::SimpleGetRequest] Getting second error:', errorMessage, 'for:', data.uri);
        if (data === null || data === void 0 ? void 0 : data.uri)
            logger.error('[request.shared::SimpleGetRequest] Failed for:', data.uri);
        return Promise.reject(err);
    });
}
exports.SimpleGetRequest = SimpleGetRequest;
function SimplePostRequest(data) {
    var httpAgentDefault = { keepAlive: true, keepAliveMsecs: 300000 };
    var requestConfig = {
        method: 'POST',
        timeout: 30000,
        proxy: false,
        httpAgent: new http.Agent(data.httpAgent || httpAgentDefault),
        httpsAgent: new https.Agent(data.httpsAgent || httpAgentDefault),
        baseURL: data.uri,
        headers: data.headers,
        data: data.body
    };
    if (data.responseType)
        requestConfig.responseType = data.responseType;
    if (data.timeout)
        requestConfig.timeout = data.timeout;
    return (0, axios_1.default)(requestConfig)
        .catch(function (err) {
        var errorMessage = typeof err === 'string' ? err : (err.message || err.description);
        logger.error("[request.shared::SimplePostRequest] Getting first try request error: ".concat(errorMessage, " from \"").concat(requestConfig.baseURL, "\""));
        if ((requestConfig.baseURL.includes('http:')
            ||
                requestConfig.baseURL.includes('https:'))
            &&
                data.retryProtocol) {
            if (requestConfig.baseURL.includes('http:'))
                requestConfig.baseURL = requestConfig.baseURL.replace('http:', 'https:');
            else if (requestConfig.baseURL.includes('https:'))
                requestConfig.baseURL = requestConfig.baseURL.replace('https', 'http');
            logger.warn("[request.shared::SimplePostRequest] Failed \"".concat(data.uri, "\" retrying \"").concat(requestConfig.baseURL, "\""));
            return (0, axios_1.default)(requestConfig);
        }
        else if (data.retryUri) {
            logger.warn("[request.shared::SimplePostRequest] Failed \"".concat(data.uri, "\", retrying \"").concat(data.retryUri, "\""));
            requestConfig.baseURL = data.retryUri;
            return (0, axios_1.default)(requestConfig);
        }
        else
            throw err;
    })
        .then(function (result) {
        var requestResponse = SimpleResult(result);
        if (requestResponse && requestResponse.data) {
            return Promise.resolve(requestResponse.data);
        }
        else {
            throw 'SERVER_ERROR_SIMPLE_POST_REQUEST_FAILED';
        }
    })
        .catch(function (err) {
        var errorMessage = typeof err === 'string' ? err : (err.message || err.description);
        logger.error('[request.shared::SimplePostRequest] Getting second error:', errorMessage);
        if (data === null || data === void 0 ? void 0 : data.uri)
            logger.error('[request.shared::SimpleGetRequest] Failed for:', data.uri);
        return Promise.reject(err);
    });
}
exports.SimplePostRequest = SimplePostRequest;
function SimplePutRequest(data) {
    var requestConfig = {
        method: 'PUT',
        timeout: 30000,
        proxy: false,
        httpAgent: new http.Agent({ keepAlive: true, keepAliveMsecs: 300000 }),
        httpsAgent: new https.Agent({ keepAlive: true, keepAliveMsecs: 300000 }),
        baseURL: data.uri,
        headers: data.headers,
        data: data.body
    };
    if (data.timeout)
        requestConfig.timeout = data.timeout;
    return (0, axios_1.default)(requestConfig)
        .then(function (result) {
        var requestResponse = SimpleResult(result);
        if (requestResponse && requestResponse.data) {
            return Promise.resolve(requestResponse.data);
        }
        else {
            throw 'SERVER_ERROR_SIMPLE_DELETE_PUT_FAILED';
        }
    })
        .catch(function (err) {
        var errorMessage = typeof err === 'string' ? err : (err.message || err.description);
        logger.error('[request.shared::SimplePutRequest] Getting error:', errorMessage);
        return Promise.reject(err);
    });
}
exports.SimplePutRequest = SimplePutRequest;
function SimpleDeleteRequest(data) {
    var requestConfig = {
        method: 'DELETE',
        timeout: 30000,
        proxy: false,
        httpAgent: new http.Agent({ keepAlive: true, keepAliveMsecs: 300000 }),
        httpsAgent: new https.Agent({ keepAlive: true, keepAliveMsecs: 300000 }),
        baseURL: data.uri,
        headers: data.headers,
        data: data.body
    };
    if (data.timeout)
        requestConfig.timeout = data.timeout;
    return (0, axios_1.default)(requestConfig)
        .then(function (result) {
        var requestResponse = SimpleResult(result);
        if (requestResponse) {
            if (requestResponse.data)
                return Promise.resolve(requestResponse.data);
            else
                return Promise.resolve(requestResponse);
        }
        else {
            throw 'SERVER_ERROR_SIMPLE_DELETE_REQUEST_FAILED';
        }
    })
        .catch(function (err) {
        var errorMessage = typeof err === 'string' ? err : (err.message || err.description);
        logger.error('[request.shared::SimpleDeleteRequest] Getting error:', errorMessage);
        return Promise.reject(err);
    });
}
exports.SimpleDeleteRequest = SimpleDeleteRequest;
axios_1.default.interceptors.request.use(function (x) {
    if (x.headers.FileName) {
        x.data = x.data || {};
        x.data.requestStartedAt = new Date().getTime();
    }
    return x;
});
axios_1.default.interceptors.response.use(function (x) {
    if (x.config.headers.FileName) {
        var now = new Date().getTime();
        logger.info("[request.shared::interceptors.response] Total execution time for: ".concat(x.config.headers.FileName, " => ").concat(now - x.config.data.requestStartedAt, " ms at ").concat(now));
        x.headers['request-duration'] = now - x.config.data.requestStartedAt;
    }
    return x;
}, function (x) {
    if (x.config.headers.FileName)
        logger.error("[request.shared::interceptors.response] Execution time for: ".concat(x.config.baseURL, ">").concat(x.config.headers.FileName, " => ").concat(new Date().getTime() - x.config.data.requestStartedAt, " ms"));
    throw x;
});
